/************************************************
* Utilisation de GMPLIB sous licence LGPL
************************************************/

#include <gmp.h>
#include <gmpxx.h>
#include <sstream>
#include <ostream>
#include <iostream>
#include <assert.h>
#include "FieldElement.hpp"

mpz_t temp;

using namespace std;

FieldElement::FieldElement(mpz_class val, mpz_class pri){
	mpz_init(temp);
	mpz_mod(temp,val.get_mpz_t(),pri.get_mpz_t());
	value = mpz_class(temp);
	prime = pri;
}

FieldElement::FieldElement(mpz_class pri) {
	mpz_init(temp);
	prime = pri;
}

FieldElement::FieldElement() {
	mpz_init(temp);
	prime = 2;
}

mpz_class FieldElement::getValue() const{
	return value;
}

mpz_class FieldElement::getModulo() const {
	return prime;
}

void FieldElement::setValue(mpz_class val){
	mpz_mod(temp,val.get_mpz_t(),getModulo().get_mpz_t());
	value = mpz_class(temp);
}

void FieldElement::setModulo(mpz_class pri){
	prime = pri;
}


/**
* Affectation
*/
FieldElement& FieldElement::operator=(FieldElement const& a){
	setModulo(a.getModulo());
	setValue(a.getValue());
	return *this;
}

/**
* Affectation
*/
FieldElement& FieldElement::operator=(mpz_class const& a){
	setValue(a);
	return *this;
}

/**
* Plus binaire
*/
FieldElement FieldElement::operator+(FieldElement const& a){
	return FieldElement(getValue() + a.getValue(),getModulo());
}

/**
* Soustraction
*/
FieldElement FieldElement::operator-(FieldElement const& a){
	return FieldElement(getValue() - a.getValue(),getModulo());
}

/**
* Plus unaire
*/
FieldElement FieldElement::operator+() const{
	return FieldElement(+getValue(),getModulo());
}

/**
* Moins unaire
*/
FieldElement FieldElement::operator-() const{
	return FieldElement(-getValue(),getModulo());
}

/**
* Multiplication
*/
FieldElement FieldElement::operator*(FieldElement const& a){
	return FieldElement(getValue() * a.getValue(),getModulo());
}

/**
* Division
*/
FieldElement FieldElement::operator/(FieldElement const& a){
	mpz_t inv;
	mpz_init(inv);
	mpz_invert(inv,(a.getValue()).get_mpz_t(),getModulo().get_mpz_t());
	FieldElement tmp(mpz_class(inv) * getValue(),getModulo());
	mpz_clear(inv);
	return tmp;
}

/**
* Incrmentation prfixe
*/
FieldElement& FieldElement::operator++(){
	setValue(getValue() + 1);
	return *this;
}

/**
* Incrmentation postfixe
*/
FieldElement FieldElement::operator++(int x){
	FieldElement tmp(getValue(),getModulo());
	operator++();
	return tmp;
}

/**
* Dcrmentation prfixe
*/
FieldElement& FieldElement::operator--(){
	setValue(getValue() - 1);
	return *this;
}

/**
* Dcrmentation postfixe
*/
FieldElement FieldElement::operator--(int x){
	FieldElement tmp(getValue(),getModulo());
	operator--();
	return tmp;
}

/**
* gal 
*/
bool FieldElement::operator==(FieldElement const& a){
	return getValue() == a.getValue();
}

/**
* Diffrent de
*/
bool FieldElement::operator!=(FieldElement const& a){
	return getValue() != a.getValue();
}

/**
* Suprieur 
*/
bool FieldElement::operator>(FieldElement const& a){
	return getValue() > a.getValue();
}

/**
* Infrieur 
*/
bool FieldElement::operator<(FieldElement const& a){
	return getValue() < a.getValue();
}

/**
* Suprieur ou gal 
*/
bool FieldElement::operator>=(FieldElement const& a){
	return getValue() >= a.getValue();
}

/**
* Infrieur ou gal 
*/
bool FieldElement::operator<=(FieldElement const& a){
	return getValue() <= a.getValue();
}

/**
* Assignment by addition
*/
FieldElement& FieldElement::operator+=(FieldElement const& a){
	*this = *this + a;
	return *this;
}

/**
* Assignment by subtraction
*/
FieldElement& FieldElement::operator-=(FieldElement const& a){
	*this = *this - a;
	return *this;
}

/**
* Assignment by multiplication
*/
FieldElement& FieldElement::operator*=(FieldElement const& a){
	*this = *this * a;
	return *this;
}

/**
* Assignment by division
*/
FieldElement& FieldElement::operator/=(FieldElement const& a){
	*this = *this / a;
	return *this;
}