//     All Rights Reserved © Copyright 2013 - Morpho - Paris France
//
//     This module is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.


/**
 * Get image file from workstation,
 * cut this image 
 * and save datas to all databases.
 */
function splitFile() {
    
    var file   = document.getElementById('fileinput').files[0];
    var reader = new FileReader();
    var _URL   = window.URL || window.webkitURL;
   
    reader.onload = function(e) {
        
        var begin_date = Date.now();
        
        // Initialize opts
        var opts = initializeOpts();
        
        // Initialize
        var uuid     = generateUUID();
        var database = $('#restsqldatabase').attr('value');
        
        document.getElementById("drop-zone").src = reader.result;
        document.getElementById("file_name_value").value = file.name;

        // Image in base64
        resumeResult = reader.result.substring(reader.result);
        
        // Test if it's an image
        if (!isImageData(resumeResult)) {
            alert(file.name + " is not an image.");
            return;
        }
        
        // search width/height of image
        var img    = new Image();
        img.onload = function() {
            width = this.width,
            height = this.height;
            
            // split
            var details = splitImage(resumeResult, confrest.confrests.length);
            
            // Iterate and send in different database
            for (var i=0 ; i< confrest.confrests.length; i++) {
            
                // Create object JIO
                var jio = getInstanceJIO(confrest.confrests[i].url, database);
                if (!jio) {
                    return error('no jio set for cutting image');
                }
                
                // Create data in JSON
                //var docDocument   = JSON.parse('{"_id":"' + uuid + '", "titre":"' + file.name + '", "auteur":"'+ file.name +'"}');
                var docDocument   = JSON.parse('{"_id":"' + uuid + '", "titre":"' + file.name + '", "auteur":"'+ file.name + '", "width":"' + width + '", "height":"' + height +'"}');
                var docAttachment = JSON.parse('{"_id":"' + uuid + '", "_attachment":"att2", "_minetype":"image/jpeg", "_data":" '+ details[i] +'"}');
               
                // POST Document
                jio['post'](docDocument, opts, function (err, val) {
                    callback(err, val, begin_date);
                });
               
                // Post Attachment
                jio['putAttachment'](docAttachment, opts, function (err, val) {
                    callback(err, val, begin_date);
                });
            };
        };
        img.src = _URL.createObjectURL(file);
        
    };
    reader.onerror = function(evt) {
        alert("Impossible reading file : " + file.name);
    };
    reader.readAsDataURL(file);
}

/**
 * Display information image from specific id.
 */
function getDocument() {
   
    var begin_date = Date.now();
    
    // Initialize opts
    var opts = initializeOpts();
    log('opts: ' + JSON.stringify(opts));

    // Search Image ID
    var document_id = $('#indexDocument').attr('value');
    var database    = $('#restsqldatabase').attr('value');

    // Iterate in different database
    for (var i=0 ; i< confrest.confrests.length; i++) {

        // Create object JIO
        var jio = getInstanceJIO(confrest.confrests[i].url, database);
        if (!jio) {
            return error('no jio set for searching document with ID : ' + document_id);
        }
        
        // Create data in JSON
        var doc = JSON.parse('{"_id":"' +  document_id + '"}');
        log('doc: ' + JSON.stringify(doc));
       
        // Get Document
        jio['get'](doc, opts, function (err, val) {
            callback(err, val, begin_date);
        });
        
    }   
}

/**
 * Display Attachment from specific id.
 */
function getAttachment() {
    
    var begin_date = Date.now();
    
    // Search Image ID
    var document_id = $('#indexDocument').attr('value');
    var database    = $('#restsqldatabase').attr('value');
    
    // Initialize opts
    var opts = initializeOpts();
    log('opts: ' + JSON.stringify(opts));

    // Iterate in different database
    for (var i=0 ; i< confrest.confrests.length; i++) {

        // Create object JIO
        var jio = getInstanceJIO(confrest.confrests[i].url, database);
        if (!jio) {
            return error('no jio set for searching image with ID : ' + document_id);
        }
        
        // Create data in JSON
        var doc = JSON.parse('{"_id":"' +  document_id + '", "_attachment":"att2"}');
        log('doc: ' + JSON.stringify(doc));
       
        // Get Document
        jio['getAttachment'](doc, opts, function (err, val) {
            callback(err, val, begin_date);
        });
        
    }   
}

/**
 * Retrieve all datas from database contains specific id
 * and save file to workstation
 * @return {File}
 */
function grouping() {

    // Reinitialize
    lstClsImageId   = [];
    var document_id = $('#indexDocument').attr('value');;
 
    for (var j=0 ; j< confrest.confrests.length; j++) {
        var conf = confrest.confrests[j];
        var url = conf.url;
        saveImageId(j, document_id, url);
    }
}


/**
 * Save Image from database.
 * @returns
 */
function saveImageId(colonne, document_id, url) {

    var database = $('#restsqldatabase').attr('value');
    var id_jio   = getInstanceJIO(url, database);
    if (!id_jio) {
        return error('no jio set for saving Image.');
    }

    // Params request ..............................................................................
    // Initialize opts
    var opts = initializeOpts();
    
    var doc  = JSON.parse('{"_id":"' +  document_id + '", "_attachment":"att2"}');
    // .............................................................................. Params request

    // Execute request
    id_jio['getAttachment'](doc, opts, function(err, val) {
        if(val) {
            var base64Image = val.split("data:image/jpeg;base64,")[1];
            var clsImgId = getClsImageId(document_id);
            clsImgId.setPartImageBase64(colonne, base64Image == null ? val.trim() : base64Image);
            clsImgId.setDocumentId(document_id);
            clsImgId.incCountImage();
            if (isObjImageExist(document_id) == null) {
                lstClsImageId.push(clsImgId);
            }
            
            doc = JSON.parse('{"_id":"' +  document_id + '"}');
            id_jio['get'](doc, opts, function (err, val) {
                if (null != document_id) {
                    var clsImgId = isObjImageExist(document_id);
                    if (null != clsImgId) {
                        clsImgId.setFileName(val.auteur);
                        if (clsImgId.getCountImage() == confrest.confrests.length && clsImgId.getFileName() != null && !clsImgId.isMerge()) {
                            clsImgId.setMerge(true);
                            var mergeImageRefId = clsImgId.getImageBase64();
                            var binaryImg = atob(mergeImageRefId);
                            var length = binaryImg.length;
                            var ab = new ArrayBuffer(length);
                            var ua = new Uint8Array(ab);
                            for (var j = 0; j < length; j++) {
                                ua[j] = binaryImg.charCodeAt(j);
                            }
                            var blob = new Blob([ab], {
                                type: "image/jpeg"
                            });
                            saveAs(blob, clsImgId.getFileName());
                        }
                    }
               }
            });
        }
    });
}

/**
 * Retrieve all documents.
 * Use only one database for retrieve all IDs
*/
function getAllDocs() {

    // Search Image ID
    var document_id = $('#indexDocument').attr('value');
    var database    = $('#restsqldatabase').attr('value');

    // Initialize opts
    var opts = initializeOpts();
    opts.query  = '';
    log('opts: ' + JSON.stringify(opts));
 
    // Create object JIO
    var jio = getInstanceJIO(confrest.confrests[0].url, database);
    if (!jio) {
        return error('no jio set for searching all ID.');
    }
    
    // Create data in JSON
    var doc = JSON.parse('{"_id":"' +  document_id + '"}');
    log('doc: ' + JSON.stringify(doc));
   
    // Get Document
    jio['allDocs'](doc, opts, function (err, val) {
        if(val) {
            if(val.rows.length > 0) {
                log('number of images  : ' + val.rows.length); 
                log('----------------------------------------------------------------------------------');
                log('|                                       id                                        ');
                log('----------------------------------------------------------------------------------');
                for(var i=0; i < val.rows.length; i++) {
                    log('|  ' + val.rows[i].id);
                    log('----------------------------------------------------------------------------------');
                }
            } else {
                log('no row is found');
            }
        }
    });
}

/**
 * Save image to workstation (with specific image name).
 * @param {Array} arrayBuffer
 * @param {String} nameFile
 */
function saveFile(arrayBuffer, nameFile) {
    var string = arrayBuffer[0] + arrayBuffer[1];
    var binaryImg = atob(string);
    var length = binaryImg.length;
    var ab = new ArrayBuffer(length);
    var ua = new Uint8Array(ab);
    for (var i = 0; i < length; i++) {
        ua[i] = binaryImg.charCodeAt(i);
    }

    var blob = new Blob([ab], {
        type: "image/jpeg"
    });
    saveAs(blob, nameFile);
}

/**
 * Generate unique identifier.
 */
function generateUUID() {
    var date = new Date();
    var components = [
        date.getYear(),
        date.getMonth(),
        date.getDate(),
        date.getHours(),
        date.getMinutes(),
        date.getSeconds(),
        date.getMilliseconds()
    ];
    return components.join("");
}

/**
 * Split image in several parts.
 * @param buffer Data
 * @param nbrcut Number split
 * @returns list parts image
 */
function splitImage(buffer, nbrcut) {
    
    // length buffer
    var lenbuffer = buffer.length;
    
    // split
    var index = Math.ceil(lenbuffer / nbrcut);
    var details = {};
    var start = 0;
    var end   = index;
    for(var i=0;i<nbrcut;i++) {
        details[i] = buffer.substring(start, end);
        start = start + index;
        end   = (i == nbrcut-2) ? lenbuffer : end + index;
    }
    return details;
}




function httpDecomposePost(numberDatabase, threshold, fileData) {
    var xmlHttp = null;

    xmlHttp = new XMLHttpRequest();
    xmlHttp.open( "POST", "/jio-cutting/decompose?numberDatabase=" + numberPlayer + "&threshold=" + threshold + "&fileData=" + threshold, false ); //Asynchronous request as third parameter is true
    xmlHttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded"); // This is needed for any POST request made via Ajax
    xmlHttp.send(fileData);
    var matchResult = JSON.parse(xmlHttp.responseText);
    log("Resul matching between : Image matching " + fileinputMatch + " - and Image Ref : " + fileinputRef + " - with message : " + matchResult.message + " score : " + matchResult.score);
}


