/**
 * 
 */
package com.safran.morpho.utils;

import java.io.BufferedReader;
import java.io.IOException;

import javax.servlet.http.HttpServletRequest;

/**
 * @author DIN
 *
 */
public class RequestUtils {
	
    /**
     * Accept image with size 4M.
     */
    private static final int MAX_DATA_LEN = 4096 * 1024;
    
	   /**
     * Get Data from HttpRequest.
     * @param request HttpServletRequest
     * @return image in format base64
     */
    public static String getBody(HttpServletRequest request) {
    	return getBody(request, true);
    }
    
    public static String getBody(HttpServletRequest request, boolean checkLength) {
        String retVal = null; 
        BufferedReader br = null; 

        // Read data in buffer
        try {
            br = request.getReader();
        } catch (final Exception exception) {
            System.out.println("Unable to get a handle to message data : getReader() failed : " + exception.getMessage());
            return null;
        }
        
        

        // Extract data from HttpRequest
        try {
        	// stream size
            int bufLen = 10 * 1024; 
            int len = request.getContentLength();

            // Verify length data in stream
            if (len > MAX_DATA_LEN) {
                System.out.println("Message length exceeds " + MAX_DATA_LEN / 1024 + " kilo-bytes.");
                return null;
            }
            if (len <= 0) {
                System.out.println("Received message, length = " + len);
                return null;
            }

            // Extract response
            StringBuilder sb = new StringBuilder(len);
            char[] c = new char[bufLen];
            int read;

            while ((read = br.read(c, 0, bufLen)) > 0) {
                // check length another time

                if (sb.length() > MAX_DATA_LEN) {
                    System.out.println("Message from client exceeds " + MAX_DATA_LEN / 1024 + " kilo-bytes.");
                    return null;
                }
                System.out.println("char :" + c.toString());
                // Add in buffer
                sb.append(c, 0, read);
            }

            retVal = sb.toString();

            System.out.println("retVal.length() : " + retVal.length() + " len : " + len + " sb.length() : " + sb.length());
            // check length another time
            if (retVal.length() != len && checkLength) {
                System.out.println("Received message length does not match with HTTP length");
                return null;
            }
        } catch (final IOException exception) {
            System.out.println("Unable to get message data : " + exception.getMessage());
            return null;
        }

        // End process
        return retVal;
    }
    
    
    /**
     * Get parameter form request.
     * @param request HttpServletRequest
     * @param searchParam Params in HttpServletRequest
     * @return fileName to store
     */
    public static String getParameter(final HttpServletRequest request, final String param) {
        String paramValue = null;
        
        if (request != null) {
            paramValue = request.getParameter(param);
        }
        return paramValue;
    }

}
