//     All Rights Reserved © Copyright 2013 - Morpho - Paris France
//
//     This module is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.


/**
 * Display information image from specific id.
 * @param document_id Image searching
 * @param base Name database
 */
function getDocument(document_id, base) {
   
    // Set database
    database = base;
    
    // Initialize opts
    var opts = initializeOpts();

    // Iterate in different database
    for (var i=0 ; i< confrest.confrests.length; i++) {

        // Create object JIO
        var jio = getInstanceJIO(confrest.confrests[i].url, database);
        if (!jio) {
            return error('no jio set for searching document with ID : ' + document_id);
        }
        
        // Create data in JSON
        var doc = JSON.parse('{"_id":"' +  document_id + '"}');
       
        // Get Document
        jio['get'](doc, opts, function (err, val) {
            if (val) {
                setMsg("Information of Id : " + document_id + "\n" + JSON.stringify(val) , "msgS");
            }
        });
        
    }   
}

/**
 * Display Attachment from specific id.
 * @param document_id Image searching
 * @param base Name database
 */
function getAttachment(document_id, base) {
    
    // Set database
    database = base;
    
    // Initialize opts
    var opts = initializeOpts();

    // Iterate in different database
    for (var i=0 ; i< confrest.confrests.length; i++) {

        // Create object JIO
        var jio = getInstanceJIO(confrest.confrests[i].url, database);
        if (!jio) {
            return error('no jio set for searching image with ID : ' + document_id);
        }
        
        // Create data in JSON
        var doc = JSON.parse('{"_id":"' +  document_id + '", "_attachment":"att2"}');
       
        // Get Document
        jio['getAttachment'](doc, opts, function (err, val) {
            if (val) {
                setMsg("Data of Id : " + document_id + "\n" + JSON.stringify(val) , "msgS");
            }
        });
        
    }   
}


/**
 * Retrieve all datas from database contains specific id
 * and save file to workstation.
 * @param document_id Image searching
 * @param base Name database
 */
function grouping(document_id, base) {

    // Reinitialize
    lstClsImageId = [];
 
    // Set database
    database = base;
    
    for (var j=0 ; j< confrest.confrests.length; j++) {
        var conf = confrest.confrests[j];
        var url = conf.url;
        saveImageId(j, document_id, url, base);
    }
}


/**
 * Save Image from database.
 * @param colonne Rank data check in database
 * @param document_id Image searching
 * @param url URL where store data
 * @param base Name database
 */
function saveImageId(colonne, document_id, url, base) {

    var id_jio   = getInstanceJIO(url, base);
    if (!id_jio) {
        return error('no jio set for saving Image.');
    }

    // Params request ..............................................................................
    // Initialize opts
    var opts = initializeOpts();
    
    var doc  = JSON.parse('{"_id":"' +  document_id + '", "_attachment":"att2"}');
    // .............................................................................. Params request

    // Execute request
    id_jio['getAttachment'](doc, opts, function(err, val) {
        if(val) {
            var base64Image = val.split("data:image/jpeg;base64,")[1];
            var clsImgId = getClsImageId(document_id);
            clsImgId.setPartImageBase64(colonne, base64Image == null ? val.trim() : base64Image);
            clsImgId.setDocumentId(document_id);
            clsImgId.incCountImage();
            if (isObjImageExist(document_id) == null) {
                lstClsImageId.push(clsImgId);
            }
            
            doc = JSON.parse('{"_id":"' +  document_id + '"}');
            id_jio['get'](doc, opts, function (err, val) {
                if (null != document_id) {
                    var clsImgId = isObjImageExist(document_id);
                    if (null != clsImgId) {
                        clsImgId.setFileName(val.auteur);
                        //if (clsImgId.getCountImage() == confrest.confrests.length && clsImgId.getFileName() != null && !clsImgId.isMerge()) {
                        if (clsImgId.getCountImage() > shamirThreshold && clsImgId.getFileName() != null && !clsImgId.isMerge()) {
                            clsImgId.setMerge(true);
                            var mergeImageRefId = clsImgId.getImageBase64Shamir();
                            
                            // shamir merge
                            var matchResult = httpRecomposePost(confrest.confrests.length, shamirThreshold, mergeImageRefId)
                            
                            // treatement
                            var binaryImg = atob(matchResult.split("data:image/jpeg;base64,")[1]);
                            var length = binaryImg.length;
                            var ab = new ArrayBuffer(length);
                            var ua = new Uint8Array(ab);
                            for (var j = 0; j < length; j++) {
                                ua[j] = binaryImg.charCodeAt(j);
                            }
                            var blob = new Blob([ab], {
                                type: "image/jpeg"
                            });
                            saveAs(blob, clsImgId.getFileName());
                        }  
                    }
               }
            });
        }
    });
}


/**
 * view loop to retrieve image
 * @param document_id Image searching
 * @param base Name database
 * @param popupId  popup id to update specific popup 
*/
function view(document_id, base, popupId) {

    // Reinitialize
    lstClsImageId = [];
 
    // Set database
    database = base;
    
   var mask = new Ext.LoadMask(Ext.getCmp('fiche' + popupId), {msg:"Wait for retrieved data..."});
    mask.show();
    
    for (var j=0 ; j< confrest.confrests.length; j++) {
        var conf = confrest.confrests[j];
        var url = conf.url;
        viewImageId(j, document_id, url, base, popupId, mask);
    }
}

/**
 * View Image from database.
 * @param colonne Rank data check in database
 * @param document_id Image searching
 * @param url URL where store data
 * @param base Name database
 * @param popupId  popup id to update specific popup
 * @param mask  hide/display mask in popup
 */
function viewImageId(colonne, document_id, url, base, popupId, mask) {

    var id_jio   = getInstanceJIO(url, base);
    if (!id_jio) {
        return error('no jio set for saving Image.');
    }

    // Params request ..............................................................................
    // Initialize opts
    var opts = initializeOpts();
    

    
    var doc  = JSON.parse('{"_id":"' +  document_id + '", "_attachment":"att2"}');
    // .............................................................................. Params request

    // Execute request
    id_jio['getAttachment'](doc, opts, function(err, val) {
        if(val) {
            var base64Image = val.split("data:image/jpeg;base64,")[1];
            var clsImgId = getClsImageId(document_id);
            clsImgId.setPartImageBase64(colonne, base64Image == null ? val.trim() : base64Image);
            clsImgId.setDocumentId(document_id);
            clsImgId.incCountImage();
            if (isObjImageExist(document_id) == null) {
                lstClsImageId.push(clsImgId);
            }
            
            doc = JSON.parse('{"_id":"' +  document_id + '"}');
            id_jio['get'](doc, opts, function (err, val) {
                if (null != document_id) {
                    var clsImgId = isObjImageExist(document_id);
                    if (null != clsImgId) {
                        clsImgId.setFileName(val.auteur);
                        //if (clsImgId.getCountImage() == confrest.confrests.length && clsImgId.getFileName() != null && !clsImgId.isMerge()) {
                        if (clsImgId.getCountImage() > shamirThreshold && clsImgId.getFileName() != null && !clsImgId.isMerge()) {
                            clsImgId.setMerge(true);
                            var mergeImageRefId = clsImgId.getImageBase64Shamir();
                            
                            // shamir merge
                            var matchResult = httpRecomposePost(confrest.confrests.length, shamirThreshold, mergeImageRefId)
                            
                            // update popup image
                            var image = document.getElementById("imageRefPopUp" + popupId + document_id);
                            image.src = matchResult;
                            // update popup form
                            Ext.getCmp('nameField'+ popupId + document_id).setValue(val.nom);
                            Ext.getCmp('firstNameField'+ popupId + document_id).setValue(val.prenom);
                            Ext.getCmp('sexField'+ popupId + document_id).setValue(val.sex);
                            Ext.getCmp('creationDateField'+ popupId + document_id).setValue(val.creationDate);
                            mask.hide(); 
                        }  
                    }
               }
            });
        }
    });
}

function httpRecomposePost(numberDatabase, threshold, fileData) {
		
    var xmlHttp = null;
    
    var imageData = null;

    xmlHttp = new XMLHttpRequest();
    xmlHttp.open( "POST", "/WebDin/recompose?numberDatabase=" + numberDatabase + "&threshold=" + threshold, false);
    xmlHttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded"); // This is needed for any POST request made via Ajax
    xmlHttp.send(fileData);
    imageData =  xmlHttp.responseText;
    return imageData;
}

/**
 * Retrieve all documents.
 * Use only one database for retrieve all IDs
*/
function getAllDocs(document_id, base) {

	//Ext.MessageBox.alert('Success', "getAllDocs" +document_id + "ddddd " + base ); 
    // Set database
    database = base;

    // Initialize opts
    var opts = initializeOpts();
 
    // Create object JIO
    var jio = getInstanceJIO(confrest.confrests[0].url, database);
    if (!jio) {
        return error('no jio set for searching all ID.');
    }
    
    // Create data in JSON
    var doc = JSON.parse('{}');
   
    // Get Document
    jio['allDocs'](doc, opts, function (err, val) {
    	
		if(val) {
			if(val.rows.length > 0) {
								
				setMsg("number of images  : " + val.rows.length , "msgS");
				setMsg("----------------------------------------------------------------------------------" , "msgS");
				setMsg("                                        id                                        " , "msgS");
				setMsg("----------------------------------------------------------------------------------" , "msgS");
				for(i=0; i < val.rows.length; i++) {
					setMsg("  " + val.rows[i].id , "msgS");
					setMsg("----------------------------------------------------------------------------------" , "msgS");
				}
			} else {
				setMsg("no row is found" , "msgS");
			}
	}
      
    });	
}
