//     All Rights Reserved © Copyright 2013 - Morpho - Paris France
//
//     This module is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.

//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.


/* Loading ExtJS -------------------------------------------------------------------------------- */
Ext.Loader.setConfig({enabled: true});

Ext.Loader.setPath('Ext.ux', 'extjs/ux');

Ext.require([
    'Ext.tip.QuickTipManager',
    'Ext.container.Viewport',
    'Ext.layout.*',
    'Ext.form.Panel',
    'Ext.form.Label',
    'Ext.grid.*',
    'Ext.data.*',
    'Ext.tree.*',
    'Ext.selection.*',
    'Ext.tab.Panel',
    'Ext.ux.layout.Center',
    'Ext.window.MessageBox',
    'Ext.tip.*',
    'Ext.ProgressBar',
    'Ext.util.*',
    'Ext.state.*'
]);

//
// This is the main layout definition.
//
Ext.onReady(function(){
 
    Ext.tip.QuickTipManager.init();

    // Gets all layouts DIN
    var layoutExamples = [];
    Ext.Object.each(getCustomLayouts(), function(name, example){
        layoutExamples.push(example);
    });
    Ext.Object.each(getCuttingLayouts(), function(name, example){
        layoutExamples.push(example);
    });
    Ext.Object.each(getMatchingLayouts(), function(name, example){
        layoutExamples.push(example);
    });
    Ext.Object.each(getDatabaseLayouts(), function(name, example){
        layoutExamples.push(example);
    });
    
    // This is the main content center region that will contain each example layout panel.
    // It will be implemented as a CardLayout since it will contain multiple panels with
    // only one being visible at any given time.
    var contentPanel = {
         id: 'content-panel',
         region: 'center', // this is what makes this panel into a region within the containing layout
         layout: 'card',
         margins: '2 5 5 0',
         activeItem: 0,
         border: false,
         items: layoutExamples
    };
    
    var store = Ext.create('Ext.data.TreeStore', {
        root: {
            expanded: true
        },
        proxy: {
            type: 'ajax',
            url: 'src/tree-data.json'
        }
    });
    
    // Go ahead and create the TreePanel now so that we can use it below
     var treePanel = Ext.create('Ext.tree.Panel', {
        id: 'tree-panel',
        collapsible: true,
        region:'north',
        split: false,
        layout: 'fit',
        rootVisible: false,
        autoScroll: true,
        store: store
    });
    
     // Assign the changeLayout function to be called on tree node click.
     treePanel.getSelectionModel().on('select', function(selModel, record) {
         if (record.get('leaf')) {
             
             // Active panel selected
             Ext.getCmp('content-panel').layout.setActiveItem(record.getId() + '-panel');
             
             // Reload panel database
             var pnlDatabase = Ext.getCmp('database-panel').items.items[0].items.items[0].store;
             if (null != pnlDatabase && record.getId() == "database") {
                 pnlDatabase.loadData(confrest.confrests, false);
             }
             
             // Reload panel cutting
             var pnlCutting = Ext.getCmp('cutting-panel');
             if (null != pnlCutting && record.getId() == "cutting") {
                 clearCutting();
             }
           
             // Reload panel matching
             var pnlMatching = Ext.getCmp('matching-panel');
             if (null != pnlMatching && record.getId() == "matching") {
                 clearMatching();
             }
 
              // Reload panel searching
             var pnlSearching = Ext.getCmp('searching-panel');
             if (null != pnlSearching && record.getId() == "searching") {
                 clearData("msgS");
             }
 
         }
     });
    
    // Finally, build the main layout once all the pieces are ready.  This is also a good
    // example of putting together a full-screen BorderLayout within a Viewport.
    Ext.create('Ext.Viewport', {
        layout: 'border',
        title: 'D.I.N',
        items: [{
            xtype: 'box',
            id: 'header',
            region: 'north',
            html: '<h1>D&eacute;monstrateur d\'identit&eacute; num&eacute;rique</h1>',
            height: 30
        },{
            layout: 'border',
            id: 'layout-browser',
            region:'west',
            border: false,
            split:true,
            margins: '2 0 5 5',
            width: 290,
            minSize: 100,
            maxSize: 500,
            items: [treePanel,{     	
            	xtype:'image',
                id: 'safran_morpho',
                src: 'images/safran_morpho.jpg',
                region: 'south',
                width: 230,
                height: 92 
            }]
        }, 
            contentPanel
        ],
        renderTo: Ext.getBody()
    });
});
/*  ------------------------------------------------------------------------------- Loading ExtJS */
