package org.resilience;



import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.WatchService;
import java.util.ArrayList;

import static java.nio.file.StandardWatchEventKinds.ENTRY_CREATE;

import org.resilience.fileutilities.Watcher.MyWatchQueueReader;

public class RateTool {
	private final RateToolConfigParser _config;
	private static MyWatchQueueReader _fileWatcher= null;
	private static boolean stop=false;
	
	public RateTool() throws RateToolException{
		this._config=new RateToolConfigParser();
	}

	public void init() throws RateToolException{
		
		ArrayList<String> dirs = new ArrayList<String>();
		dirs.add(this._config.getHomePath());
		dirs.add(this._config.getMonitorPath());
		dirs.add(this._config.getScanPath());
		dirs.add(this._config.getResultsPath());
		boolean success = true;
		for(String dir: dirs){
			File folder=new File(dir);
			if (!folder.exists()) {
			    try{
			    	success = success && folder.mkdirs();
			    	if (!success)
			    		throw new RateToolException("Failed to create directory"+ dir);
			    } catch(SecurityException se){
			       throw new RateToolException("Failed to create directory "+dir);
			    } catch(RateToolException e){
			    	throw e;
			    }
			}
		}
	}
	
	public RateToolConfigParser getConfig(){
		return this._config;
	}

	public static void start() {
		RateTool instance=null;
		
		while(!stop){
        	try{	
        		if (instance==null){
        				instance=new RateTool();	
        			}
        		if (instance!=null){ 
	        		instance.init();
	    			Path toWatch = Paths.get(instance.getConfig().getMonitorPath());
	    			if(toWatch == null) {
	    	            throw new UnsupportedOperationException("Directory not found");
	    	        }
	    			WatchService myWatcher = toWatch.getFileSystem().newWatchService();
	    			  
	    	        // start the file watcher thread below
	    	        _fileWatcher = new MyWatchQueueReader(myWatcher,instance.getConfig());
	    	        Thread th = new Thread(_fileWatcher, "FileWatcher");
	    	        th.start();
	    	  
	    	        // register a file
	    	        toWatch.register(myWatcher, ENTRY_CREATE);
	    	        th.join();
        		}
    		} catch (InterruptedException e) {
    			System.out.println("Failure: "+e.getMessage());
    			e.printStackTrace();
    		} catch (IOException e) {
    			System.out.println("Failure: "+e.getMessage());
    			e.printStackTrace();
    		} catch (RateToolException e) {
    			System.out.println("Failure: "+e.getMessage());
    			e.printStackTrace();
    		}
		}
    }
 
    public static void stop() {
    	stop=true;
    	_fileWatcher.stop();
    }
 
    public static void main(String[] args) {
    	if (null!=args[0]){	
	        if ("start".equals(args[0])) {
	            start();
	        } else if ("stop".equals(args[0])) {
	            stop();
	        }
    	}
    }

}
