package org.resilience.fileutilities;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.nio.file.*;
import java.util.UUID;

import org.resilience.RateToolConfigParser;
import org.resilience.RateToolException;
import org.resilience.mark.GenMark;

public class Watcher{
	
	public static class MyWatchQueueReader implements Runnable {
  
        private WatchService _myWatcher;
        private RateToolConfigParser _config;
        
        public MyWatchQueueReader(WatchService myWatcher, RateToolConfigParser config) {
            this._myWatcher = myWatcher;
            this._config=config;
        }
  
        @Override
        public void run() {
            try {
            	WatchKey key = this._myWatcher.take();
                while(key != null) {
                    for (WatchEvent event : key.pollEvents()) {
                    	
                    	String zipFilename=((Path)event.context()).toString();
                        String projectName=zipFilename.substring(0, zipFilename.lastIndexOf('.'));
                        String uuid=UUID.randomUUID().toString();
                        String scan_dir_path=this._config.getScanPath()+File.separator+projectName+"_"+uuid;
                        String result_dir_path=this._config.getResultsPath()+File.separator+projectName+"_"+uuid;
                        String result_fpr_file=result_dir_path+File.separator+projectName+".fpr";
                        String build_log_file=result_dir_path+File.separator+projectName+".txt";
                    	InputStream auditFile=null;
                    	FileOperation fo = null;
                    	try{

	                        fo = new FileOperation();
	                        fo.waitAndCopy(this._config.getMonitorPath()+File.separator+zipFilename);
	                        fo.createFolders(scan_dir_path,result_dir_path);
	                        fo.move(this._config.getMonitorPath()+File.separator+zipFilename,result_dir_path+File.separator+zipFilename);
	                     
	
	                        UnzipTool.unzip(result_dir_path+File.separator+zipFilename,scan_dir_path);
	                        
	                        fo.replace_compiler(scan_dir_path+File.separator+projectName,projectName,uuid);
	                        
	                        
	                        Fortify f = new Fortify(projectName,uuid,scan_dir_path, result_fpr_file, build_log_file);
	                        
	                        f.clean();
                      	                        
	                        f.translate_make();
                             
	                        f.scan();
	                        f.clean();
	                        auditFile = UnzipTool.getAudit(result_fpr_file);
	                        GenMark instance = new GenMark(auditFile);                        
	                        instance.writeResults(zipFilename, result_dir_path, projectName);
                    	}catch (IOException|RateToolException e) {
                    		PrintWriter writer = null;
                    		try {
                    			writer = new PrintWriter(result_dir_path+File.separator+projectName+"_failed.txt", "UTF-8");
                    			writer.println("Involved Zip File: " +zipFilename);
                    			writer.println("An error has occured during the scan. "+e.getMessage());
                    			writer.println(e.getMessage());
                    		} catch (FileNotFoundException | UnsupportedEncodingException ignore) {
                    		}finally{
                    			try{
                    				if (null!=writer)
                    					writer.close();
                    			}catch(Exception ignore){}
                    		}
                    	} catch (InterruptedException e){
                    		PrintWriter writer = null;
                    		try {
                    			writer = new PrintWriter(result_dir_path+File.separator+projectName+"_failed.txt", "UTF-8");
                    			writer.println("Involved Zip File: " +zipFilename);
                    			writer.println("failed to read the file.");
                    			writer.println(e.getMessage());
                    		} catch (FileNotFoundException | UnsupportedEncodingException ignore) {
                    		}finally{
                    			try{
                    				if (null!=writer)
                    					writer.close();
                    			}catch(Exception ignore){}
                    		}
                    	
	                    }finally{
                			fo.deleteScanFolder(scan_dir_path);
	                    	try{
	                    		if (auditFile != null)
	                    			auditFile.close();
	                    	}catch(Exception ignore){}
	                    }
                    }
                    key.reset();
                    key = this._myWatcher.take();
                }
            }catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        
        public void stop() {
        	try {
				this._myWatcher.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
        }
    }
    
    
    
    
}
