package org.resilience.mark;

import java.util.ArrayList;
import java.lang.Math;

public class Fortify_category {
	private final String _name;
	private final int _tolerated_issue_number_threshold;
	private ArrayList<Fortify_issue> _list = new ArrayList<Fortify_issue>();
	
	public Fortify_category(String name, int tolerated_issue_number_threshold){
		this._name = name;
		this._tolerated_issue_number_threshold = tolerated_issue_number_threshold;
	}
	
	public boolean contains(Fortify_issue issue){
		for (Fortify_issue item : this._list) {
			if (item.getIssueID().equals(issue.getIssueID()))
					return true;
		}
		return false;
	}
	
	public boolean addFortifyIssue(Fortify_issue issue){
		if(!this.contains(issue) && issue.getCategoryName().equals(this._name))
			return this._list.add(issue);
		else
			return false;
	}
	
	public String getName() {
		return this._name;
	}
	
	public int getThreshold() {
		return this._tolerated_issue_number_threshold;
	}
	
	public int getNumber(){
		return this._list.size();
	}
	
	public double getMark(){
		if (this._tolerated_issue_number_threshold == 0 &&  this.getNumber() > 0) 
				return 0;
		if (this.getNumber() ==0)
				return 3;
		double a = this._tolerated_issue_number_threshold*Math.log10(3)/Math.log10(1.5);
		double b = this._tolerated_issue_number_threshold*Math.log10(2)/Math.log10(1.5);
		return Math.pow(2, (a - this.getNumber())/b);
	}
	
	public String toString(){
		return this._name+"  "+this.getNumber();
	}
	
	public String displayStats(){
		return "Category: "+this._name +" | Detected: "+this.getNumber()+" | Tolerated: "+this.getThreshold()+" | Mark :"+this.getMark();
	}
	

}