package org.resilience.mark;

import java.util.ArrayList;
import java.io.PrintWriter;
import java.lang.Math;

public class Owasp_practice {

	private final String _name;
	private double _weight;
	private ArrayList<Fortify_category> _list = new ArrayList<Fortify_category>();
	
	public Owasp_practice(String name,double weight){
		this._name   = name;
		this._weight = weight;
	}
	
	public boolean addFortifyCategory(String name, int tolerated_issue_number_threshold){
		Fortify_category category = new Fortify_category(name,tolerated_issue_number_threshold);
		if(!this._list.contains(category))
			return this._list.add(category);
		else
			return false;
	}
	
	public boolean contains(String categoryName){
		for (Fortify_category category : this._list) {
			 if ( category.getName().equals(categoryName) )
			 	return true;
		}	
		return false;
	}
	
	public boolean addFortifyIssue(Fortify_issue issue){
		String categoryName=issue.getCategoryName();
		if (this.contains(categoryName) ){
			for (Fortify_category category : this._list) {
				 if ( category.getName().equals(categoryName) )
					 return category.addFortifyIssue(issue);
			}	
		}
		return false;
	}
		
	public String getName() {
		return this._name;
	}
	
	public int getNumber(){
		return this._list.size();
	}
	
	public double getWeight(){
		return this._weight;
	}
	
	public int getIssuesNumber(){
		int count=0;
		for (Fortify_category category : this._list) {
			count+=category.getNumber();
		}
		return count;
	}
	
	public double getMark(){
		double sum=0;
		
		// If no category has been created, returning best mark
		if (this.getNumber()==0)
				return 3;
		for (Fortify_category category : this._list) {
			 sum+=Math.pow(this._weight, -category.getMark());
		}
		return -Math.log10(sum/this.getNumber())/(Math.log10(this._weight));
	}
	
	public void displayStats(PrintWriter writer){
		for(Fortify_category category: this._list){
			writer.println("Practice: "+this._name+" | "+category.displayStats()+"\n");
		}
	}

	
	
}
