// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof Module !== 'undefined' ? Module : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function';
var ENVIRONMENT_IS_WEB = typeof window === 'object';
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) { return Module['read'](filename, true) };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  Module['thisProgram'] = process['argv'][1];
  Module['arguments'] = process['argv'].slice(2);

  module['exports'] = Module;
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    return read(f, 'binary');
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  this['Module'] = Module;

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WEB) {
    window['Module'] = Module;
  } else {
    Module['load'] = importScripts;
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] == 'undefined' && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Auto-generated preamble library stuff ===

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  setTempRet0: function (value) {
    tempRet0 = value;
  },
  getTempRet0: function () {
    return tempRet0;
  },
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  forceAlign: function (target, quantum) {
    quantum = quantum || 4;
    if (quantum == 1) return target;
    if (isNumber(target) && isNumber(quantum)) {
      return Math.ceil(target/quantum)*quantum;
    } else if (isNumber(quantum) && isPowerOfTwo(quantum)) {
      return '(((' +target + ')+' + (quantum-1) + ')&' + -quantum + ')';
    }
    return 'Math.ceil((' + target + ')/' + quantum + ')*' + quantum;
  },
  isNumberType: function (type) {
    return type in Runtime.INT_TYPES || type in Runtime.FLOAT_TYPES;
  },
  isPointerType: function isPointerType(type) {
  return type[type.length-1] == '*';
},
  isStructType: function isStructType(type) {
  if (isPointerType(type)) return false;
  if (isArrayType(type)) return true;
  if (/<?\{ ?[^}]* ?\}>?/.test(type)) return true; // { i32, i8 } etc. - anonymous struct types
  // See comment in isStructPointerType()
  return type[0] == '%';
},
  INT_TYPES: {"i1":0,"i8":0,"i16":0,"i32":0,"i64":0},
  FLOAT_TYPES: {"float":0,"double":0},
  or64: function (x, y) {
    var l = (x | 0) | (y | 0);
    var h = (Math.round(x / 4294967296) | Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  and64: function (x, y) {
    var l = (x | 0) & (y | 0);
    var h = (Math.round(x / 4294967296) & Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  xor64: function (x, y) {
    var l = (x | 0) ^ (y | 0);
    var h = (Math.round(x / 4294967296) ^ Math.round(y / 4294967296)) * 4294967296;
    return l + h;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  dedup: function dedup(items, ident) {
  var seen = {};
  if (ident) {
    return items.filter(function(item) {
      if (seen[item[ident]]) return false;
      seen[item[ident]] = true;
      return true;
    });
  } else {
    return items.filter(function(item) {
      if (seen[item]) return false;
      seen[item] = true;
      return true;
    });
  }
},
  set: function set() {
  var args = typeof arguments[0] === 'object' ? arguments[0] : arguments;
  var ret = {};
  for (var i = 0; i < args.length; i++) {
    ret[args[i]] = 0;
  }
  return ret;
},
  STACK_ALIGN: 8,
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  calculateStructAlignment: function calculateStructAlignment(type) {
    type.flatSize = 0;
    type.alignSize = 0;
    var diffs = [];
    var prev = -1;
    var index = 0;
    type.flatIndexes = type.fields.map(function(field) {
      index++;
      var size, alignSize;
      if (Runtime.isNumberType(field) || Runtime.isPointerType(field)) {
        size = Runtime.getNativeTypeSize(field); // pack char; char; in structs, also char[X]s.
        alignSize = Runtime.getAlignSize(field, size);
      } else if (Runtime.isStructType(field)) {
        if (field[1] === '0') {
          // this is [0 x something]. When inside another structure like here, it must be at the end,
          // and it adds no size
          // XXX this happens in java-nbody for example... assert(index === type.fields.length, 'zero-length in the middle!');
          size = 0;
          if (Types.types[field]) {
            alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
          } else {
            alignSize = type.alignSize || QUANTUM_SIZE;
          }
        } else {
          size = Types.types[field].flatSize;
          alignSize = Runtime.getAlignSize(null, Types.types[field].alignSize);
        }
      } else if (field[0] == 'b') {
        // bN, large number field, like a [N x i8]
        size = field.substr(1)|0;
        alignSize = 1;
      } else if (field[0] === '<') {
        // vector type
        size = alignSize = Types.types[field].flatSize; // fully aligned
      } else if (field[0] === 'i') {
        // illegal integer field, that could not be legalized because it is an internal structure field
        // it is ok to have such fields, if we just use them as markers of field size and nothing more complex
        size = alignSize = parseInt(field.substr(1))/8;
        assert(size % 1 === 0, 'cannot handle non-byte-size field ' + field);
      } else {
        assert(false, 'invalid type for calculateStructAlignment');
      }
      if (type.packed) alignSize = 1;
      type.alignSize = Math.max(type.alignSize, alignSize);
      var curr = Runtime.alignMemory(type.flatSize, alignSize); // if necessary, place this on aligned memory
      type.flatSize = curr + size;
      if (prev >= 0) {
        diffs.push(curr-prev);
      }
      prev = curr;
      return curr;
    });
    if (type.name_ && type.name_[0] === '[') {
      // arrays have 2 elements, so we get the proper difference. then we scale here. that way we avoid
      // allocating a potentially huge array for [999999 x i8] etc.
      type.flatSize = parseInt(type.name_.substr(1))*type.flatSize/2;
    }
    type.flatSize = Runtime.alignMemory(type.flatSize, type.alignSize);
    if (diffs.length == 0) {
      type.flatFactor = type.flatSize;
    } else if (Runtime.dedup(diffs).length == 1) {
      type.flatFactor = diffs[0];
    }
    type.needsFlattening = (type.flatFactor != 1);
    return type.flatIndexes;
  },
  generateStructInfo: function (struct, typeName, offset) {
    var type, alignment;
    if (typeName) {
      offset = offset || 0;
      type = (typeof Types === 'undefined' ? Runtime.typeInfo : Types.types)[typeName];
      if (!type) return null;
      if (type.fields.length != struct.length) {
        printErr('Number of named fields must match the type for ' + typeName + ': possibly duplicate struct names. Cannot return structInfo');
        return null;
      }
      alignment = type.flatIndexes;
    } else {
      var type = { fields: struct.map(function(item) { return item[0] }) };
      alignment = Runtime.calculateStructAlignment(type);
    }
    var ret = {
      __size__: type.flatSize
    };
    if (typeName) {
      struct.forEach(function(item, i) {
        if (typeof item === 'string') {
          ret[item] = alignment[i] + offset;
        } else {
          // embedded struct
          var key;
          for (var k in item) key = k;
          ret[key] = Runtime.generateStructInfo(item[key], type.fields[i], alignment[i]);
        }
      });
    } else {
      struct.forEach(function(item, i) {
        ret[item[1]] = alignment[i];
      });
    }
    return ret;
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      assert(args.length == sig.length-1);
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      assert(sig.length == 1);
      assert(('dynCall_' + sig) in Module, 'bad function pointer type - no table for sig \'' + sig + '\'');
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  getAsmConst: function (code, numArgs) {
    // code is a constant string on the heap, so we can cache these
    if (!Runtime.asmConstCache) Runtime.asmConstCache = {};
    var func = Runtime.asmConstCache[code];
    if (func) return func;
    var args = [];
    for (var i = 0; i < numArgs; i++) {
      args.push(String.fromCharCode(36) + i); // $0, $1 etc
    }
    var source = Pointer_stringify(code);
    if (source[0] === '"') {
      // tolerate EM_ASM("..code..") even though EM_ASM(..code..) is correct
      if (source.indexOf('"', 1) === source.length-1) {
        source = source.substr(1, source.length-2);
      } else {
        // something invalid happened, e.g. EM_ASM("..code($0)..", input)
        abort('invalid EM_ASM input |' + source + '|. Please use EM_ASM(..code..) (no quotes) or EM_ASM({ ..code($0).. }, input) (to input values)');
      }
    }
    try {
      var evalled = eval('(function(' + args.join(',') + '){ ' + source + ' })'); // new Function does not allow upvars in node
    } catch(e) {
      Module.printErr('error in executing inline EM_ASM code: ' + e + ' on: \n\n' + source + '\n\nwith args |' + args + '| (make sure to use the right one out of EM_ASM, EM_ASM_ARGS, etc.)');
      throw e;
    }
    return Runtime.asmConstCache[code] = evalled;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[func]) {
      Runtime.funcWrappers[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return Runtime.funcWrappers[func];
  },
  UTF8Processor: function () {
    var buffer = [];
    var needed = 0;
    this.processCChar = function (code) {
      code = code & 0xFF;

      if (buffer.length == 0) {
        if ((code & 0x80) == 0x00) {        // 0xxxxxxx
          return String.fromCharCode(code);
        }
        buffer.push(code);
        if ((code & 0xE0) == 0xC0) {        // 110xxxxx
          needed = 1;
        } else if ((code & 0xF0) == 0xE0) { // 1110xxxx
          needed = 2;
        } else {                            // 11110xxx
          needed = 3;
        }
        return '';
      }

      if (needed) {
        buffer.push(code);
        needed--;
        if (needed > 0) return '';
      }

      var c1 = buffer[0];
      var c2 = buffer[1];
      var c3 = buffer[2];
      var c4 = buffer[3];
      var ret;
      if (buffer.length == 2) {
        ret = String.fromCharCode(((c1 & 0x1F) << 6)  | (c2 & 0x3F));
      } else if (buffer.length == 3) {
        ret = String.fromCharCode(((c1 & 0x0F) << 12) | ((c2 & 0x3F) << 6)  | (c3 & 0x3F));
      } else {
        // http://mathiasbynens.be/notes/javascript-encoding#surrogate-formulae
        var codePoint = ((c1 & 0x07) << 18) | ((c2 & 0x3F) << 12) |
                        ((c3 & 0x3F) << 6)  | (c4 & 0x3F);
        ret = String.fromCharCode(
          Math.floor((codePoint - 0x10000) / 0x400) + 0xD800,
          (codePoint - 0x10000) % 0x400 + 0xDC00);
      }
      buffer.length = 0;
      return ret;
    }
    this.processJSString = function processJSString(string) {
      /* TODO: use TextEncoder when present,
        var encoder = new TextEncoder();
        encoder['encoding'] = "utf-8";
        var utf8Array = encoder['encode'](aMsg.data);
      */
      string = unescape(encodeURIComponent(string));
      var ret = [];
      for (var i = 0; i < string.length; i++) {
        ret.push(string.charCodeAt(i));
      }
      return ret;
    }
  },
  getCompilerSetting: function (name) {
    throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+7)&-8);(assert((((STACKTOP|0) < (STACK_MAX|0))|0))|0); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + (assert(!staticSealed),size))|0;STATICTOP = (((STATICTOP)+7)&-8); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + (assert(DYNAMICTOP > 0),size))|0;DYNAMICTOP = (((DYNAMICTOP)+7)&-8); if (DYNAMICTOP >= TOTAL_MEMORY) enlargeMemory();; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 8))*(quantum ? quantum : 8); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}


Module['Runtime'] = Runtime;









//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  var func = Module['_' + ident]; // closure exported function
  if (!func) {
    try {
      func = eval('_' + ident); // explicit lookup
    } catch(e) {}
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

var cwrap, ccall;
(function(){
  var stack = 0;
  var JSfuncs = {
    'stackSave' : function() {
      stack = Runtime.stackSave();
    },
    'stackRestore' : function() {
      Runtime.stackRestore(stack);
    },
    // type conversion from js to c
    'arrayToC' : function(arr) {
      var ret = Runtime.stackAlloc(arr.length);
      writeArrayToMemory(arr, ret);
      return ret;
    },
    'stringToC' : function(str) {
      var ret = 0;
      if (str !== null && str !== undefined && str !== 0) { // null string
        ret = Runtime.stackAlloc(str.length + 1); // +1 for the trailing '\0'
        writeStringToMemory(str, ret);
      }
      return ret;
    }
  };
  // For fast lookup of conversion functions
  var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

  // C calling interface. A convenient way to call C functions (in C files, or
  // defined with extern "C").
  //
  // Note: ccall/cwrap use the C stack for temporary values. If you pass a string
  //       then it is only alive until the call is complete. If the code being
  //       called saves the pointer to be used later, it may point to invalid
  //       data. If you need a string to live forever, you can create it (and
  //       must later delete it manually!) using malloc and writeStringToMemory,
  //       for example.
  //
  // Note: LLVM optimizations can inline and remove functions, after which you will not be
  //       able to call them. Closure can also do so. To avoid that, add your function to
  //       the exports using something like
  //
  //         -s EXPORTED_FUNCTIONS='["_main", "_myfunc"]'
  //
  // @param ident      The name of the C function (note that C++ functions will be name-mangled - use extern "C")
  // @param returnType The return type of the function, one of the JS types 'number', 'string' or 'array' (use 'number' for any C pointer, and
  //                   'array' for JavaScript arrays and typed arrays; note that arrays are 8-bit).
  // @param argTypes   An array of the types of arguments for the function (if there are no arguments, this can be ommitted). Types are as in returnType,
  //                   except that 'array' is not possible (there is no way for us to know the length of the array)
  // @param args       An array of the arguments to the function, as native JS values (as in returnType)
  //                   Note that string arguments will be stored on the stack (the JS string will become a C string on the stack).
  // @return           The return value, as a native JS value (as in returnType)
  ccall = function ccallFunc(ident, returnType, argTypes, args) {
    assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');
    var func = getCFunc(ident);
    var cArgs = [];
    assert(returnType !== 'array', 'Return type should not be "array".');
    if (args) {
      for (var i = 0; i < args.length; i++) {
        var converter = toC[argTypes[i]];
        if (converter) {
          if (stack === 0) stack = Runtime.stackSave();
          cArgs[i] = converter(args[i]);
        } else {
          cArgs[i] = args[i];
        }
      }
    }
    var ret = func.apply(null, cArgs);
    if (returnType === 'string') ret = Pointer_stringify(ret);
    if (stack !== 0) JSfuncs['stackRestore']();
    return ret;
  }

  var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
  function parseJSFunc(jsfunc) {
    // Match the body and the return value of a javascript function source
    var parsed = jsfunc.toString().match(sourceRegex).slice(1);
    return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
  }
  var JSsource = {};
  for (var fun in JSfuncs) {
    if (JSfuncs.hasOwnProperty(fun)) {
      // Elements of toCsource are arrays of three items:
      // the code, and the return value
      JSsource[fun] = parseJSFunc(JSfuncs[fun]);
    }
  }
  // Returns a native JS wrapper for a C function. This is similar to ccall, but
  // returns a function you can call repeatedly in a normal way. For example:
  //
  //   var my_function = cwrap('my_c_function', 'number', ['number', 'number']);
  //   alert(my_function(5, 22));
  //   alert(my_function(99, 12));
  //
  cwrap = function cwrap(ident, returnType, argTypes) {
    var cfunc = getCFunc(ident);
    // When the function takes numbers and returns a number, we can just return
    // the original function
    var numericArgs = argTypes.every(function(type){ return type === 'number'});
    var numericRet = (returnType !== 'string');
    if ( numericRet && numericArgs) {
      return cfunc;
    }
    // Creation of the arguments list (["$1","$2",...,"$nargs"])
    var argNames = argTypes.map(function(x,i){return '$'+i});
    var funcstr = "(function(" + argNames.join(',') + ") {";
    funcstr += "assert(runtimeInitialized, 'you need to wait for the runtime to be ready (e.g. wait for main() to be called)');\n";
    var nargs = argTypes.length;
    if (!numericArgs) {
      // Generate the code needed to convert the arguments from javascript
      // values to pointers
      funcstr += JSsource['stackSave'].body + ';';
      for (var i = 0; i < nargs; i++) {
        var arg = argNames[i], type = argTypes[i];
        if (type === 'number') continue;
        var convertCode = JSsource[type + 'ToC']; // [code, return]
        funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
        funcstr += convertCode.body + ';';
        funcstr += arg + '=' + convertCode.returnValue + ';';
      }
    }

    // When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
    var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
    // Call the function
    funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
    if (!numericRet) { // Return type can only by 'string' or 'number'
      // Convert the result to a string
      var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
      funcstr += 'ret = ' + strgfy + '(ret);';
    }
    if (!numericArgs) {
      // If we had a stack, restore it
      funcstr += JSsource['stackRestore'].body + ';';
    }
    funcstr += 'return ret})';
    return eval(funcstr);
  };
})();
Module["cwrap"] = cwrap;
Module["ccall"] = ccall;

// Sets a value in memory in a dynamic way at run-time. Uses the
// type data. This is the same as makeSetValue, except that
// makeSetValue is done at compile-time and generates the needed
// code then, whereas this function picks the right code at
// run-time.
// Note that setValue and getValue only do *aligned* writes and reads!
// Note that ccall uses JS types as for defining types, while setValue and
// getValue need LLVM types ('i8', 'i32') - this is a lower-level operation
function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[((ptr)>>0)]=value; break;
      case 'i8': HEAP8[((ptr)>>0)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module['setValue'] = setValue;

// Parallel to setValue.
function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[((ptr)>>0)];
      case 'i8': return HEAP8[((ptr)>>0)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module['getValue'] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module['ALLOC_NORMAL'] = ALLOC_NORMAL;
Module['ALLOC_STACK'] = ALLOC_STACK;
Module['ALLOC_STATIC'] = ALLOC_STATIC;
Module['ALLOC_DYNAMIC'] = ALLOC_DYNAMIC;
Module['ALLOC_NONE'] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)>>0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }
    assert(type, 'Must know what type to store in allocate!');

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module['allocate'] = allocate;

function Pointer_stringify(ptr, /* optional */ length) {
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = false;
  var t;
  var i = 0;
  while (1) {
    assert(ptr + i < TOTAL_MEMORY);
    t = HEAPU8[(((ptr)+(i))>>0)];
    if (t >= 128) hasUtf = true;
    else if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (!hasUtf) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }

  var utf8 = new Runtime.UTF8Processor();
  for (i = 0; i < length; i++) {
    assert(ptr + i < TOTAL_MEMORY);
    t = HEAPU8[(((ptr)+(i))>>0)];
    ret += utf8.processCChar(t);
  }
  return ret;
}
Module['Pointer_stringify'] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module['UTF16ToString'] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16LE form. The copy will require at most (str.length*2+1)*2 bytes of space in the HEAP.
function stringToUTF16(str, outPtr) {
  for(var i = 0; i < str.length; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[(((outPtr)+(i*2))>>1)]=codeUnit;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[(((outPtr)+(str.length*2))>>1)]=0;
}
Module['stringToUTF16'] = stringToUTF16;

// Given a pointer 'ptr' to a null-terminated UTF32LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.
function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module['UTF32ToString'] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32LE form. The copy will require at most (str.length+1)*4 bytes of space in the HEAP,
// but can use less, since str.length does not return the number of characters in the string, but the number of UTF-16 code units in the string.
function stringToUTF32(str, outPtr) {
  var iChar = 0;
  for(var iCodeUnit = 0; iCodeUnit < str.length; ++iCodeUnit) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    var codeUnit = str.charCodeAt(iCodeUnit); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++iCodeUnit);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[(((outPtr)+(iChar*4))>>2)]=codeUnit;
    ++iChar;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[(((outPtr)+(iChar*4))>>2)]=0;
}
Module['stringToUTF32'] = stringToUTF32;

function demangle(func) {
  var i = 3;
  // params, etc.
  var basicTypes = {
    'v': 'void',
    'b': 'bool',
    'c': 'char',
    's': 'short',
    'i': 'int',
    'l': 'long',
    'f': 'float',
    'd': 'double',
    'w': 'wchar_t',
    'a': 'signed char',
    'h': 'unsigned char',
    't': 'unsigned short',
    'j': 'unsigned int',
    'm': 'unsigned long',
    'x': 'long long',
    'y': 'unsigned long long',
    'z': '...'
  };
  var subs = [];
  var first = true;
  function dump(x) {
    //return;
    if (x) Module.print(x);
    Module.print(func);
    var pre = '';
    for (var a = 0; a < i; a++) pre += ' ';
    Module.print (pre + '^');
  }
  function parseNested() {
    i++;
    if (func[i] === 'K') i++; // ignore const
    var parts = [];
    while (func[i] !== 'E') {
      if (func[i] === 'S') { // substitution
        i++;
        var next = func.indexOf('_', i);
        var num = func.substring(i, next) || 0;
        parts.push(subs[num] || '?');
        i = next+1;
        continue;
      }
      if (func[i] === 'C') { // constructor
        parts.push(parts[parts.length-1]);
        i += 2;
        continue;
      }
      var size = parseInt(func.substr(i));
      var pre = size.toString().length;
      if (!size || !pre) { i--; break; } // counter i++ below us
      var curr = func.substr(i + pre, size);
      parts.push(curr);
      subs.push(curr);
      i += pre + size;
    }
    i++; // skip E
    return parts;
  }
  function parse(rawList, limit, allowVoid) { // main parser
    limit = limit || Infinity;
    var ret = '', list = [];
    function flushList() {
      return '(' + list.join(', ') + ')';
    }
    var name;
    if (func[i] === 'N') {
      // namespaced N-E
      name = parseNested().join('::');
      limit--;
      if (limit === 0) return rawList ? [name] : name;
    } else {
      // not namespaced
      if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
      var size = parseInt(func.substr(i));
      if (size) {
        var pre = size.toString().length;
        name = func.substr(i + pre, size);
        i += pre + size;
      }
    }
    first = false;
    if (func[i] === 'I') {
      i++;
      var iList = parse(true);
      var iRet = parse(true, 1, true);
      ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
    } else {
      ret = name;
    }
    paramLoop: while (i < func.length && limit-- > 0) {
      //dump('paramLoop');
      var c = func[i++];
      if (c in basicTypes) {
        list.push(basicTypes[c]);
      } else {
        switch (c) {
          case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
          case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
          case 'L': { // literal
            i++; // skip basic type
            var end = func.indexOf('E', i);
            var size = end - i;
            list.push(func.substr(i, size));
            i += size + 2; // size + 'EE'
            break;
          }
          case 'A': { // array
            var size = parseInt(func.substr(i));
            i += size.toString().length;
            if (func[i] !== '_') throw '?';
            i++; // skip _
            list.push(parse(true, 1, true)[0] + ' [' + size + ']');
            break;
          }
          case 'E': break paramLoop;
          default: ret += '?' + c; break paramLoop;
        }
      }
    }
    if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
    if (rawList) {
      if (ret) {
        list.push(ret + '?');
      }
      return list;
    } else {
      return ret + flushList();
    }
  }
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    return parse();
  } catch(e) {
    return func;
  }
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function stackTrace() {
  var stack = new Error().stack;
  return stack ? demangleAll(stack) : '(no stack trace available)'; // Stack trace is not available at least on IE10 and Safari 6.
}

// Memory management

var PAGE_SIZE = 4096;
function alignMemoryPage(x) {
  return (x+4095)&-4096;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk

function enlargeMemory() {
  abort('Cannot enlarge memory arrays. Either (1) compile with -s TOTAL_MEMORY=X with X higher than the current value ' + TOTAL_MEMORY + ', (2) compile with ALLOW_MEMORY_GROWTH which adjusts the size at runtime but prevents some optimizations, or (3) set Module.TOTAL_MEMORY before the program runs.');
}

var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;
var FAST_MEMORY = Module['FAST_MEMORY'] || 2097152;

var totalMemory = 4096;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  Module.printErr('increasing TOTAL_MEMORY to ' + totalMemory + ' to be more reasonable');
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'JS engine does not provide full typed array support');

var buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);

// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;

function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
    Module.printErr('Exiting runtime. Any attempt to access the compiled C code may fail from now. If you want to keep the runtime alive, set Module["noExitRuntime"] = true or build with -s NO_EXIT_RUNTIME=1');
  }
  callRuntimeCallbacks(__ATEXIT__);
  runtimeInitialized = false;
}

function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module['addOnPreRun'] = Module.addOnPreRun = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module['addOnInit'] = Module.addOnInit = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module['addOnPreMain'] = Module.addOnPreMain = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module['addOnExit'] = Module.addOnExit = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module['addOnPostRun'] = Module.addOnPostRun = addOnPostRun;

// Tools

// This processes a JS string into a C-line array of numbers, 0-terminated.
// For LLVM-originating strings, see parser.js:parseLLVMString function
function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var ret = (new Runtime.UTF8Processor()).processJSString(stringy);
  if (length) {
    ret.length = length;
  }
  if (!dontAddNull) {
    ret.push(0);
  }
  return ret;
}
Module['intArrayFromString'] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
        assert(false, 'Character code ' + chr + ' (' + String.fromCharCode(chr) + ')  at offset ' + i + ' not in 0x00-0xFF.');
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module['intArrayToString'] = intArrayToString;

// Write a Javascript array to somewhere in the heap
function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))>>0)]=chr;
    i = i + 1;
  }
}
Module['writeStringToMemory'] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[(((buffer)+(i))>>0)]=array[i];
  }
}
Module['writeArrayToMemory'] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; i++) {
    assert(str.charCodeAt(i) === str.charCodeAt(i)&0xff);
    HEAP8[(((buffer)+(i))>>0)]=str.charCodeAt(i);
  }
  if (!dontAddNull) HEAP8[(((buffer)+(str.length))>>0)]=0;
}
Module['writeAsciiToMemory'] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}

// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled
var runDependencyTracking = {};

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(!runDependencyTracking[id]);
    runDependencyTracking[id] = 1;
    if (runDependencyWatcher === null && typeof setInterval !== 'undefined') {
      // Check for missing dependencies every few seconds
      runDependencyWatcher = setInterval(function() {
        if (ABORT) {
          clearInterval(runDependencyWatcher);
          runDependencyWatcher = null;
          return;
        }
        var shown = false;
        for (var dep in runDependencyTracking) {
          if (!shown) {
            shown = true;
            Module.printErr('still waiting on run dependencies:');
          }
          Module.printErr('dependency: ' + dep);
        }
        if (shown) {
          Module.printErr('(end of list)');
        }
      }, 10000);
    }
  } else {
    Module.printErr('warning: run dependency added without ID');
  }
}
Module['addRunDependency'] = addRunDependency;
function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (id) {
    assert(runDependencyTracking[id]);
    delete runDependencyTracking[id];
  } else {
    Module.printErr('warning: run dependency removed without ID');
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module['removeRunDependency'] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data


var memoryInitializer = null;

// === Body ===





STATIC_BASE = 8;

STATICTOP = STATIC_BASE + Runtime.alignMemory(15587);
  /* global initializers */ __ATINIT__.push();
  

/* memory initializer */ allocate([0,0,0,0,0,0,0,0,70,105,108,101,32,37,115,32,114,101,97,100,10,0,0,0,73,109,97,103,101,32,100,97,116,97,32,101,110,99,111,100,101,100,44,32,99,111,109,112,114,101,115,115,101,100,32,98,121,116,101,32,108,101,110,103,116,104,32,61,32,37,100,10,0,0,0,0,0,0,0,0,37,115,46,37,115,0,0,0,73,109,97,103,101,32,100,97,116,97,32,119,114,105,116,116,101,110,32,116,111,32,102,105,108,101,32,37,115,10,0,0,45,118,101,114,115,105,111,110,0,0,0,0,0,0,0,0,83,116,97,110,100,97,114,100,32,86,101,114,115,105,111,110,58,32,37,115,10,78,66,73,83,32,78,111,110,45,69,120,112,111,114,116,32,67,111,110,116,114,111,108,32,83,111,102,116,119,97,114,101,32,86,101,114,115,105,111,110,58,32,37,115,10,0,0,0,0,0,0,65,78,83,73,47,78,73,83,84,45,73,84,76,32,49,45,50,48,48,55,0,0,0,0,82,101,108,101,97,115,101,32,52,46,49,46,48,0,0,0,37,102,0,0,0,0,0,0,45,114,0,0,0,0,0,0,45,114,97,119,0,0,0,0,45,114,97,119,95,105,110,0,32,32,32,32,32,32,32,105,109,97,103,101,32,100,101,112,116,104,32,61,32,37,100,32,110,111,116,32,56,10,0,0,32,32,32,32,32,32,32,105,110,118,97,108,105,100,32,97,114,103,32,52,44,32,101,120,112,101,99,116,101,100,32,34,45,114,97,119,95,105,110,34,32,111,112,116,105,111,110,10,0,0,0,0,0,0,0,0,85,115,97,103,101,58,32,37,115,32,0,0,0,0,0,0,60,114,32,98,105,116,114,97,116,101,62,32,60,111,117,116,101,120,116,62,32,60,105,109,97,103,101,32,102,105,108,101,62,10,0,0,0,0,0,0,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,91,45,114,97,119,95,105,110,32,119,44,104,44,100,44,91,112,112,105,93,93,32,91,99,111,109,109,101,110,116,32,102,105,108,101,93,10,10,0,32,32,32,114,32,98,105,116,114,97,116,101,32,61,32,99,111,109,112,114,101,115,115,105,111,110,32,98,105,116,32,114,97,116,101,32,40,50,46,50,53,61,61,62,53,58,49,44,32,46,55,53,61,61,62,49,53,58,49,41,10,10,0,0,114,98,0,0,0,0,0,0,119,98,0,0,0,0,0,0,69,82,82,79,82,58,32,119,114,105,116,101,95,114,97,119,95,102,114,111,109,95,109,101,109,115,105,122,101,32,58,32,102,111,112,101,110,32,58,32,37,115,10,0,0,0,0,0,69,82,82,79,82,58,32,119,114,105,116,101,95,114,97,119,95,102,114,111,109,95,109,101,109,115,105,122,101,32,58,32,102,119,114,105,116,101,32,58,32,0,0,0,0,0,0,0,111,110,108,121,32,37,100,32,111,102,32,37,100,32,98,121,116,101,115,32,119,114,105,116,116,101,110,32,102,114,111,109,32,102,105,108,101,32,37,115,10,0,0,0,0,0,0,0,69,82,82,79,82,58,32,114,101,97,100,95,105,104,101,97,100,32,58,32,0,0,0,0,105,109,97,103,101,32,100,101,112,116,104,32,61,32,37,100,32,110,111,116,32,56,32,111,114,32,50,52,10,0,0,0,69,82,82,79,82,58,32,114,101,97,100,95,114,97,119,32,58,32,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,114,97,119,32,58,32,109,97,108,108,111,99,32,58,32,105,100,97,116,97,10,0,0,0,0,0,0,69,82,82,79,82,58,32,114,101,97,100,95,114,97,119,32,58,32,37,115,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,114,97,119,32,58,32,102,114,101,97,100,32,58,32,0,0,0,0,0,111,110,108,121,32,114,101,97,100,32,37,100,32,111,102,32,37,100,32,98,121,116,101,115,10,0,0,0,0,0,0,0,69,82,82,79,82,58,32,114,101,97,100,95,114,97,119,95,111,114,95,105,104,101,97,100,95,119,115,113,32,58,32,0,105,109,97,103,101,32,100,101,112,116,104,32,61,32,37,100,32,110,111,116,32,56,10,0,69,82,82,79,82,58,32,114,101,97,100,95,114,97,119,95,111,114,95,105,104,101,97,100,95,119,115,113,32,58,32,73,109,97,103,101,32,109,117,115,116,32,98,101,32,97,116,32,108,101,97,115,116,32,37,100,32,88,32,37,100,10,0,0,32,32,32,32,32,32,32,32,32,32,32,32,32,32,119,105,100,116,104,32,61,32,37,100,32,32,58,58,32,32,104,101,105,103,104,116,32,61,32,37,100,10,0,0,0,0,0,0,32,32,32,32,32,32,32,104,101,105,103,104,116,32,110,111,116,32,102,111,117,110,100,10,0,0,0,0,0,0,0,0,32,32,32,32,32,32,32,100,101,112,116,104,32,110,111,116,32,102,111,117,110,100,10,0,114,98,0,0,0,0,0,0,82,101,97,100,73,104,101,97,100,82,97,115,116,101,114,32,58,32,111,117,116,100,97,116,97,0,0,0,0,0,0,0,73,110,118,97,108,105,100,32,99,111,109,112,114,101,115,115,105,111,110,32,99,111,100,101,0,0,0,0,0,0,0,0,82,101,97,100,73,104,101,97,100,82,97,115,116,101,114,0,102,111,112,101,110,32,102,97,105,108,101,100,0,0,0,0,37,100,0,0,0,0,0,0,115,115,99,97,110,102,32,102,97,105,108,101,100,32,111,110,32,99,111,109,112,114,101,115,115,32,102,105,101,108,100,0,115,115,99,97,110,102,32,102,97,105,108,101,100,32,111,110,32,100,101,112,116,104,32,102,105,101,108,100,0,0,0,0,115,115,99,97,110,102,32,102,97,105,108,101,100,32,111,110,32,119,105,100,116,104,32,102,105,101,108,100,0,0,0,0,115,115,99,97,110,102,32,102,97,105,108,101,100,32,111,110,32,104,101,105,103,104,116,32,102,105,101,108,100,0,0,0,115,115,99,97,110,102,32,102,97,105,108,101,100,32,111,110,32,99,111,109,112,108,101,110,32,102,105,101,108,100,0,0,82,101,97,100,73,104,101,97,100,82,97,115,116,101,114,58,32,37,115,58,32,102,114,101,97,100,32,114,101,116,117,114,110,101,100,32,37,100,32,40,101,120,112,101,99,116,101,100,32,37,100,41,10,0,0,0,82,101,97,100,73,104,101,97,100,82,97,115,116,101,114,32,58,32,105,110,100,97,116,97,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,79,117,116,112,117,116,32,66,117,102,102,101,114,32,79,118,101,114,102,108,111,119,32,105,110,32,82,76,76,95,112,117,116,99,46,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,67,97,110,110,111,116,32,97,108,108,111,99,97,116,101,32,101,110,111,117,103,104,32,109,101,109,111,114,121,32,102,111,114,32,99,111,109,112,114,101,115,115,101,100,32,102,105,108,101,46,10,0,0,0,0,10,78,111,32,109,101,109,111,114,121,32,97,108,108,111,99,97,116,101,100,32,102,111,114,32,105,110,112,117,116,32,100,97,116,97,33,10,0,0,0,10,78,111,32,109,101,109,111,114,121,32,97,108,108,111,99,97,116,101,100,32,102,111,114,32,100,101,99,111,109,112,114,101,115,115,101,100,32,100,97,116,97,33,10,0,0,0,0,84,104,105,115,32,119,105,108,108,32,107,105,108,108,32,116,104,101,32,112,114,111,103,114,97,109,32,97,110,100,32,99,114,101,97,116,101,32,97,32,99,111,114,101,32,102,105,108,101,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,64,32,16,8,4,2,1,127,191,223,239,247,251,253,254,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,64,32,16,8,4,2,1,255,255,255,255,136,7,0,0,124,7,0,0,255,255,255,255,160,7,0,0,148,7,0,0,255,255,255,255,184,7,0,0,172,7,0,0,2,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,208,7,0,0,196,7,0,0,255,255,255,255,232,7,0,0,220,7,0,0,4,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,0,8,0,0,244,7,0,0,255,255,255,255,24,8,0,0,12,8,0,0,5,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,48,8,0,0,36,8,0,0,255,255,255,255,72,8,0,0,60,8,0,0,7,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,96,8,0,0,84,8,0,0,255,255,255,255,120,8,0,0,108,8,0,0,255,255,255,255,144,8,0,0,132,8,0,0,8,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,168,8,0,0,156,8,0,0,255,255,255,255,192,8,0,0,180,8,0,0,255,255,255,255,216,8,0,0,204,8,0,0,255,255,255,255,240,8,0,0,228,8,0,0,12,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,8,9,0,0,252,8,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,32,9,0,0,20,9,0,0,255,255,255,255,56,9,0,0,44,9,0,0,255,255,255,255,80,9,0,0,68,9,0,0,255,255,255,255,0,0,0,0,92,9,0,0,255,255,255,255,116,9,0,0,104,9,0,0,255,255,255,255,140,9,0,0,128,9,0,0,14,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,164,9,0,0,152,9,0,0,255,255,255,255,188,9,0,0,176,9,0,0,13,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,212,9,0,0,200,9,0,0,255,255,255,255,236,9,0,0,224,9,0,0,255,255,255,255,4,10,0,0,248,9,0,0,255,255,255,255,28,10,0,0,16,10,0,0,255,255,255,255,52,10,0,0,40,10,0,0,255,255,255,255,76,10,0,0,64,10,0,0,15,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,100,10,0,0,88,10,0,0,255,255,255,255,124,10,0,0,112,10,0,0,255,255,255,255,148,10,0,0,136,10,0,0,255,255,255,255,172,10,0,0,160,10,0,0,255,255,255,255,196,10,0,0,184,10,0,0,255,255,255,255,220,10,0,0,208,10,0,0,255,255,255,255,244,10,0,0,232,10,0,0,255,255,255,255,12,11,0,0,0,11,0,0,255,255,255,255,36,11,0,0,24,11,0,0,255,255,255,255,60,11,0,0,48,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,84,11,0,0,72,11,0,0,255,255,255,255,108,11,0,0,96,11,0,0,255,255,255,255,132,11,0,0,120,11,0,0,255,255,255,255,156,11,0,0,144,11,0,0,255,255,255,255,180,11,0,0,168,11,0,0,255,255,255,255,204,11,0,0,192,11,0,0,255,255,255,255,228,11,0,0,216,11,0,0,255,255,255,255,252,11,0,0,240,11,0,0,17,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,20,12,0,0,8,12,0,0,255,255,255,255,44,12,0,0,32,12,0,0,255,255,255,255,68,12,0,0,56,12,0,0,64,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,92,12,0,0,80,12,0,0,255,255,255,255,116,12,0,0,104,12,0,0,255,255,255,255,140,12,0,0,128,12,0,0,255,255,255,255,164,12,0,0,152,12,0,0,255,255,255,255,188,12,0,0,176,12,0,0,255,255,255,255,212,12,0,0,200,12,0,0,18,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,236,12,0,0,224,12,0,0,255,255,255,255,4,13,0,0,248,12,0,0,255,255,255,255,28,13,0,0,16,13,0,0,255,255,255,255,52,13,0,0,40,13,0,0,255,255,255,255,76,13,0,0,64,13,0,0,21,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,100,13,0,0,88,13,0,0,255,255,255,255,124,13,0,0,112,13,0,0,255,255,255,255,148,13,0,0,136,13,0,0,20,0,0,0,0,0,0,0,0,0,0,0,19,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,172,13,0,0,160,13,0,0,255,255,255,255,196,13,0,0,184,13,0,0,255,255,255,255,220,13,0,0,208,13,0,0,22,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,244,13,0,0,232,13,0,0,255,255,255,255,12,14,0,0,0,14,0,0,255,255,255,255,36,14,0,0,24,14,0,0,255,255,255,255,60,14,0,0,48,14,0,0,255,255,255,255,84,14,0,0,72,14,0,0,255,255,255,255,108,14,0,0,96,14,0,0,255,255,255,255,132,14,0,0,120,14,0,0,255,255,255,255,156,14,0,0,144,14,0,0,255,255,255,255,180,14,0,0,168,14,0,0,255,255,255,255,204,14,0,0,192,14,0,0,23,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,228,14,0,0,216,14,0,0,255,255,255,255,252,14,0,0,240,14,0,0,255,255,255,255,20,15,0,0,8,15,0,0,255,255,255,255,44,15,0,0,32,15,0,0,255,255,255,255,68,15,0,0,56,15,0,0,25,0,0,0,0,0,0,0,0,0,0,0,24,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,92,15,0,0,80,15,0,0,255,255,255,255,116,15,0,0,104,15,0,0,255,255,255,255,140,15,0,0,128,15,0,0,255,255,255,255,164,15,0,0,152,15,0,0,255,255,255,255,188,15,0,0,176,15,0,0,255,255,255,255,212,15,0,0,200,15,0,0,255,255,255,255,236,15,0,0,224,15,0,0,128,7,0,0,0,0,0,0,0,0,0,0,64,7,0,0,0,0,0,0,0,0,0,0,255,255,255,255,4,16,0,0,248,15,0,0,255,255,255,255,28,16,0,0,16,16,0,0,255,255,255,255,52,16,0,0,40,16,0,0,0,7,0,0,0,0,0,0,0,0,0,0,43,0,0,0,0,0,0,0,0,0,0,0,42,0,0,0,0,0,0,0,0,0,0,0,39,0,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,37,0,0,0,0,0,0,0,0,0,0,0,36,0,0,0,0,0,0,0,0,0,0,0,35,0,0,0,0,0,0,0,0,0,0,0,34,0,0,0,0,0,0,0,0,0,0,0,29,0,0,0,0,0,0,0,0,0,0,0,28,0,0,0,0,0,0,0,0,0,0,0,27,0,0,0,0,0,0,0,0,0,0,0,26,0,0,0,0,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,41,0,0,0,0,0,0,0,0,0,0,0,40,0,0,0,0,0,0,0,0,0,0,0,33,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,0,0,0,0,63,0,0,0,0,0,0,0,0,0,0,0,62,0,0,0,0,0,0,0,0,0,0,0,49,0,0,0,0,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,61,0,0,0,0,0,0,0,0,0,0,0,58,0,0,0,0,0,0,0,0,0,0,0,57,0,0,0,0,0,0,0,0,0,0,0,47,0,0,0,0,0,0,0,0,0,0,0,46,0,0,0,0,0,0,0,0,0,0,0,45,0,0,0,0,0,0,0,0,0,0,0,44,0,0,0,0,0,0,0,0,0,0,0,51,0,0,0,0,0,0,0,0,0,0,0,50,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,76,16,0,0,64,16,0,0,255,255,255,255,100,16,0,0,88,16,0,0,255,255,255,255,124,16,0,0,112,16,0,0,54,0,0,0,0,0,0,0,0,0,0,0,53,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,148,16,0,0,136,16,0,0,192,1,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,0,0,0,0,0,0,0,64,1,0,0,0,0,0,0,0,0,0,0,255,255,255,255,172,16,0,0,160,16,0,0,255,255,255,255,196,16,0,0,184,16,0,0,60,0,0,0,0,0,0,0,0,0,0,0,59,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,220,16,0,0,208,16,0,0,255,255,255,255,244,16,0,0,232,16,0,0,56,0,0,0,0,0,0,0,0,0,0,0,55,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,12,17,0,0,0,17,0,0,255,255,255,255,36,17,0,0,24,17,0,0,52,0,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,192,9,0,0,0,0,0,0,0,0,0,0,128,9,0,0,0,0,0,0,0,0,0,0,64,9,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,192,8,0,0,0,0,0,0,0,0,0,0,128,8,0,0,0,0,0,0,0,0,0,0,64,8,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,192,7,0,0,0,0,0,0,0,0,0,0,192,4,0,0,0,0,0,0,0,0,0,0,128,4,0,0,0,0,0,0,0,0,0,0,64,4,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,192,3,0,0,0,0,0,0,0,0,0,0,128,3,0,0,0,0,0,0,0,0,0,0,64,2,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,192,6,0,0,0,0,0,0,0,0,0,0,128,6,0,0,0,0,0,0,0,0,0,0,64,6,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,192,5,0,0,0,0,0,0,0,0,0,0,128,5,0,0,0,0,0,0,0,0,0,0,64,5,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,64,3,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,192,2,0,0,0,0,0,0,0,0,0,0,128,2,0,0,0,0,0,0,0,0,0,0,255,255,255,255,72,17,0,0,60,17,0,0,255,255,255,255,96,17,0,0,84,17,0,0,255,255,255,255,120,17,0,0,108,17,0,0,255,255,255,255,144,17,0,0,132,17,0,0,255,255,255,255,168,17,0,0,156,17,0,0,255,255,255,255,192,17,0,0,180,17,0,0,255,255,255,255,216,17,0,0,204,17,0,0,255,255,255,255,240,17,0,0,228,17,0,0,255,255,255,255,8,18,0,0,252,17,0,0,255,255,255,255,32,18,0,0,20,18,0,0,255,255,255,255,56,18,0,0,44,18,0,0,255,255,255,255,80,18,0,0,68,18,0,0,255,255,255,255,104,18,0,0,92,18,0,0,255,255,255,255,128,18,0,0,116,18,0,0,255,255,255,255,152,18,0,0,140,18,0,0,7,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,176,18,0,0,164,18,0,0,5,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,200,18,0,0,188,18,0,0,255,255,255,255,224,18,0,0,212,18,0,0,3,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,248,18,0,0,236,18,0,0,255,255,255,255,16,19,0,0,4,19,0,0,255,255,255,255,40,19,0,0,28,19,0,0,255,255,255,255,64,19,0,0,52,19,0,0,255,255,255,255,88,19,0,0,76,19,0,0,255,255,255,255,112,19,0,0,100,19,0,0,255,255,255,255,136,19,0,0,124,19,0,0,64,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,160,19,0,0,148,19,0,0,255,255,255,255,184,19,0,0,172,19,0,0,9,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,0,128,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,208,19,0,0,196,19,0,0,255,255,255,255,232,19,0,0,220,19,0,0,255,255,255,255,0,20,0,0,244,19,0,0,255,255,255,255,24,20,0,0,12,20,0,0,255,255,255,255,48,20,0,0,36,20,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,72,20,0,0,60,20,0,0,255,255,255,255,96,20,0,0,84,20,0,0,255,255,255,255,120,20,0,0,108,20,0,0,255,255,255,255,144,20,0,0,132,20,0,0,255,255,255,255,168,20,0,0,156,20,0,0,255,255,255,255,192,20,0,0,180,20,0,0,255,255,255,255,216,20,0,0,204,20,0,0,15,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,17,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,240,20,0,0,228,20,0,0,255,255,255,255,8,21,0,0,252,20,0,0,255,255,255,255,32,21,0,0,20,21,0,0,128,6,0,0,0,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,56,21,0,0,44,21,0,0,255,255,255,255,80,21,0,0,68,21,0,0,255,255,255,255,104,21,0,0,92,21,0,0,255,255,255,255,128,21,0,0,116,21,0,0,255,255,255,255,152,21,0,0,140,21,0,0,255,255,255,255,176,21,0,0,164,21,0,0,255,255,255,255,200,21,0,0,188,21,0,0,255,255,255,255,224,21,0,0,212,21,0,0,255,255,255,255,248,21,0,0,236,21,0,0,255,255,255,255,16,22,0,0,4,22,0,0,12,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,40,22,0,0,28,22,0,0,255,255,255,255,64,22,0,0,52,22,0,0,255,255,255,255,88,22,0,0,76,22,0,0,13,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,112,22,0,0,100,22,0,0,255,255,255,255,136,22,0,0,124,22,0,0,255,255,255,255,160,22,0,0,148,22,0,0,0,1,0,0,0,0,0,0,0,0,0,0,255,255,255,255,184,22,0,0,172,22,0,0,255,255,255,255,208,22,0,0,196,22,0,0,255,255,255,255,232,22,0,0,220,22,0,0,255,255,255,255,0,23,0,0,244,22,0,0,255,255,255,255,24,23,0,0,12,23,0,0,255,255,255,255,48,23,0,0,36,23,0,0,255,255,255,255,72,23,0,0,60,23,0,0,25,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,96,23,0,0,84,23,0,0,255,255,255,255,120,23,0,0,108,23,0,0,24,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,144,23,0,0,132,23,0,0,255,255,255,255,168,23,0,0,156,23,0,0,27,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,192,23,0,0,180,23,0,0,255,255,255,255,216,23,0,0,204,23,0,0,255,255,255,255,240,23,0,0,228,23,0,0,28,0,0,0,0,0,0,0,0,0,0,0,21,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,8,24,0,0,252,23,0,0,255,255,255,255,32,24,0,0,20,24,0,0,255,255,255,255,56,24,0,0,44,24,0,0,26,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,80,24,0,0,68,24,0,0,255,255,255,255,104,24,0,0,92,24,0,0,19,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,128,24,0,0,116,24,0,0,255,255,255,255,152,24,0,0,140,24,0,0,255,255,255,255,176,24,0,0,164,24,0,0,20,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,200,24,0,0,188,24,0,0,23,0,0,0,0,0,0,0,0,0,0,0,22,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,224,24,0,0,212,24,0,0,255,255,255,255,248,24,0,0,236,24,0,0,255,255,255,255,0,0,0,0,4,25,0,0,255,255,255,255,28,25,0,0,16,25,0,0,255,255,255,255,52,25,0,0,40,25,0,0,255,255,255,255,76,25,0,0,64,25,0,0,255,255,255,255,100,25,0,0,88,25,0,0,255,255,255,255,124,25,0,0,112,25,0,0,64,2,0,0,0,0,0,0,0,0,0,0,128,2,0,0,0,0,0,0,0,0,0,0,255,255,255,255,148,25,0,0,136,25,0,0,0,2,0,0,0,0,0,0,0,0,0,0,192,1,0,0,0,0,0,0,0,0,0,0,58,0,0,0,0,0,0,0,0,0,0,0,57,0,0,0,0,0,0,0,0,0,0,0,56,0,0,0,0,0,0,0,0,0,0,0,55,0,0,0,0,0,0,0,0,0,0,0,52,0,0,0,0,0,0,0,0,0,0,0,51,0,0,0,0,0,0,0,0,0,0,0,50,0,0,0,0,0,0,0,0,0,0,0,49,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,172,25,0,0,160,25,0,0,255,255,255,255,196,25,0,0,184,25,0,0,60,0,0,0,0,0,0,0,0,0,0,0,59,0,0,0,0,0,0,0,0,0,0,0,128,1,0,0,0,0,0,0,0,0,0,0,64,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,63,0,0,0,0,0,0,0,0,0,0,0,62,0,0,0,0,0,0,0,0,0,0,0,61,0,0,0,0,0,0,0,0,0,0,0,44,0,0,0,0,0,0,0,0,0,0,0,43,0,0,0,0,0,0,0,0,0,0,0,42,0,0,0,0,0,0,0,0,0,0,0,41,0,0,0,0,0,0,0,0,0,0,0,40,0,0,0,0,0,0,0,0,0,0,0,39,0,0,0,0,0,0,0,0,0,0,0,54,0,0,0,0,0,0,0,0,0,0,0,53,0,0,0,0,0,0,0,0,0,0,0,32,0,0,0,0,0,0,0,0,0,0,0,31,0,0,0,0,0,0,0,0,0,0,0,38,0,0,0,0,0,0,0,0,0,0,0,37,0,0,0,0,0,0,0,0,0,0,0,36,0,0,0,0,0,0,0,0,0,0,0,35,0,0,0,0,0,0,0,0,0,0,0,34,0,0,0,0,0,0,0,0,0,0,0,33,0,0,0,0,0,0,0,0,0,0,0,48,0,0,0,0,0,0,0,0,0,0,0,47,0,0,0,0,0,0,0,0,0,0,0,46,0,0,0,0,0,0,0,0,0,0,0,45,0,0,0,0,0,0,0,0,0,0,0,30,0,0,0,0,0,0,0,0,0,0,0,29,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,220,25,0,0,208,25,0,0,128,5,0,0,0,0,0,0,0,0,0,0,64,5,0,0,0,0,0,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,192,4,0,0,0,0,0,0,0,0,0,0,128,4,0,0,0,0,0,0,0,0,0,0,64,4,0,0,0,0,0,0,0,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,192,3,0,0,0,0,0,0,0,0,0,0,128,3,0,0,0,0,0,0,0,0,0,0,64,3,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,0,0,0,0,0,0,0,192,2,0,0,0,0,0,0,0,0,0,0,192,6,0,0,0,0,0,0,0,0,0,0,64,6,0,0,0,0,0,0,0,0,0,0,0,6,0,0,0,0,0,0,0,0,0,0,192,5,0,0,0,0,0,0,0,0,0,0,255,255,255,255,244,25,0,0,232,25,0,0,255,255,255,255,12,26,0,0,0,26,0,0,255,255,255,255,36,26,0,0,24,26,0,0,255,255,255,255,60,26,0,0,48,26,0,0,255,255,255,255,84,26,0,0,72,26,0,0,255,255,255,255,108,26,0,0,96,26,0,0,255,255,255,255,132,26,0,0,120,26,0,0,255,255,255,255,156,26,0,0,144,26,0,0,128,7,0,0,0,0,0,0,0,0,0,0,64,7,0,0,0,0,0,0,0,0,0,0,255,255,255,255,180,26,0,0,168,26,0,0,255,255,255,255,204,26,0,0,192,26,0,0,255,255,255,255,228,26,0,0,216,26,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,0,0,0,192,9,0,0,0,0,0,0,0,0,0,0,128,9,0,0,0,0,0,0,0,0,0,0,64,9,0,0,0,0,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,192,8,0,0,0,0,0,0,0,0,0,0,128,8,0,0,0,0,0,0,0,0,0,0,64,8,0,0,0,0,0,0,0,0,0,0,0,8,0,0,0,0,0,0,0,0,0,0,192,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,80,105,120,80,101,114,66,121,116,101,0,0,0,0,0,0,100,101,112,116,104,32,33,61,32,112,111,119,101,114,32,111,102,32,50,32,98,101,116,119,101,101,110,32,49,32,97,110,100,32,54,52,32,40,105,110,99,108,117,115,105,118,101,41,0,0,0,0,0,0,0,0,112,111,105,110,116,101,114,32,104,101,97,100,32,105,115,32,78,85,76,76,0,0,0,0,37,100,0,0,0,0,0,0,103,101,116,95,100,101,110,115,105,116,121,0,0,0,0,0,114,101,97,100,105,104,100,114,58,32,102,114,101,97,100,32,114,101,116,117,114,110,101,100,32,37,100,32,40,101,120,112,101,99,116,101,100,32,37,100,41,10,0,0,0,0,0,0,37,100,0,0,0,0,0,0,114,101,97,100,105,104,100,114,0,0,0,0,0,0,0,0,99,97,110,110,111,116,32,112,97,114,115,101,32,108,101,110,103,116,104,32,102,105,101,108,100,0,0,0,0,0,0,0,82,101,99,111,114,100,32,83,121,110,99,32,69,114,114,111,114,58,32,72,101,97,100,101,114,32,110,111,116,32,102,111,117,110,100,32,111,114,32,111,108,100,32,102,111,114,109,97,116,46,0,0,0,0,0,0,109,97,108,108,111,99,0,0,104,101,97,100,0,0,0,0,69,82,82,79,82,32,58,32,119,115,113,95,101,110,99,111,100,101,95,49,32,58,32,109,97,108,108,111,99,32,58,32,102,100,97,116,97,10,0,0,73,110,112,117,116,32,105,109,97,103,101,32,112,105,120,101,108,115,32,99,111,110,118,101,114,116,101,100,32,116,111,32,102,108,111,97,116,105,110,103,32,112,111,105,110,116,10,10,0,0,0,0,0,0,0,0,84,97,98,108,101,115,32,102,111,114,32,119,97,118,101,108,101,116,32,100,101,99,111,109,112,111,115,105,116,105,111,110,32,102,105,110,105,115,104,101,100,10,10,0,0,0,0,0,87,83,81,32,100,101,99,111,109,112,111,115,105,116,105,111,110,32,111,102,32,105,109,97,103,101,32,102,105,110,105,115,104,101,100,10,10,0,0,0,83,117,98,98,97,110,100,32,118,97,114,105,97,110,99,101,115,32,99,111,109,112,117,116,101,100,10,10,0,0,0,0,87,83,81,32,115,117,98,98,97,110,100,32,100,101,99,111,109,112,111,115,105,116,105,111,110,32,100,97,116,97,32,113,117,97,110,116,105,122,101,100,10,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,115,113,95,101,110,99,111,100,101,95,49,32,58,32,112,114,111,98,108,101,109,32,119,47,113,117,97,110,116,105,122,97,116,105,111,110,32,98,108,111,99,107,32,115,105,122,101,115,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,119,115,113,95,101,110,99,111,100,101,95,49,32,58,32,109,97,108,108,111,99,32,58,32,119,115,113,95,100,97,116,97,10,0,0,0,0,0,0,0,83,79,73,44,32,116,97,98,108,101,115,44,32,97,110,100,32,102,114,97,109,101,32,104,101,97,100,101,114,32,119,114,105,116,116,101,110,10,10,0,69,82,82,79,82,32,58,32,119,115,113,95,101,110,99,111,100,101,95,49,32,58,32,109,97,108,108,111,99,32,58,32,104,117,102,102,95,98,117,102,10,0,0,0,0,0,0,0,72,117,102,102,109,97,110,32,99,111,100,101,32,84,97,98,108,101,32,49,32,103,101,110,101,114,97,116,101,100,32,97,110,100,32,119,114,105,116,116,101,110,10,10,0,0,0,0,66,108,111,99,107,32,49,32,99,111,109,112,114,101,115,115,101,100,32,97,110,100,32,119,114,105,116,116,101,110,10,10,0,0,0,0,0,0,0,0,72,117,102,102,109,97,110,32,99,111,100,101,32,84,97,98,108,101,32,50,32,103,101,110,101,114,97,116,101,100,32,97,110,100,32,119,114,105,116,116,101,110,10,10,0,0,0,0,66,108,111,99,107,32,50,32,99,111,109,112,114,101,115,115,101,100,32,97,110,100,32,119,114,105,116,116,101,110,10,10,0,0,0,0,0,0,0,0,66,108,111,99,107,32,51,32,99,111,109,112,114,101,115,115,101,100,32,97,110,100,32,119,114,105,116,116,101,110,10,10,0,0,0,0,0,0,0,0,104,115,105,122,101,49,32,61,32,37,100,32,58,58,32,104,115,105,122,101,50,32,61,32,37,100,32,58,58,32,104,115,105,122,101,51,32,61,32,37,100,10,0,0,0,0,0,0,64,32,114,32,61,32,37,46,51,102,32,58,58,32,99,111,109,112,108,101,110,32,61,32,37,100,32,58,58,32,114,97,116,105,111,32,61,32,37,46,49,102,10,0,0,0,0,0,69,82,82,79,82,58,32,84,104,105,115,32,104,117,102,102,99,111,100,101,32,119,97,114,110,105,110,103,32,105,115,32,97,110,32,101,114,114,111,114,32,0,0,0,0,0,0,0,102,111,114,32,116,104,101,32,101,110,99,111,100,101,114,46,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,99,111,109,112,114,101,115,115,95,98,108,111,99,107,32,58,32,122,114,117,110,32,116,111,111,32,108,97,114,103,101,46,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,99,111,109,112,114,101,115,115,95,98,108,111,99,107,32,58,32,122,114,117,110,50,32,116,111,111,32,108,97,114,103,101,46,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,99,111,117,110,116,95,98,108,111,99,107,32,58,32,99,97,108,108,111,99,32,58,32,99,111,117,110,116,115,10,0,0,69,82,82,79,82,58,32,99,111,117,110,116,95,98,108,111,99,107,32,58,32,90,114,117,110,32,116,111,32,108,111,110,103,32,105,110,32,99,111,117,110,116,32,98,108,111,99,107,46,10,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,246,44,132,61,242,169,38,189,49,16,214,190,49,218,73,63,49,16,214,190,242,169,38,189,246,44,132,61,0,0,0,0,3,242,26,61,244,95,195,188,12,143,226,189,242,58,193,62,118,74,90,63,242,58,193,62,12,143,226,189,244,95,195,188,3,242,26,61,0,0,0,0,87,65,82,78,73,78,71,58,32,65,32,99,111,100,101,32,105,110,32,116,104,101,32,104,117,102,102,116,97,98,108,101,32,99,111,110,116,97,105,110,115,32,97,110,32,0,0,0,97,108,108,32,49,39,115,32,99,111,100,101,46,10,32,32,32,32,32,32,32,32,32,84,104,105,115,32,105,109,97,103,101,32,109,97,121,32,115,116,105,108,108,32,98,101,32,0,100,101,99,111,100,97,98,108,101,46,10,32,32,32,32,32,32,32,32,32,73,116,32,105,115,32,110,111,116,32,99,111,109,112,108,105,97,110,116,32,119,105,116,104,32,0,0,0,116,104,101,32,87,83,81,32,115,112,101,99,105,102,105,99,97,116,105,111,110,46,10,0,87,114,105,116,105,110,103,32,116,114,97,110,115,102,111,114,109,32,116,97,98,108,101,46,10,0,0,0,0,0,0,0,108,111,91,37,100,93,32,61,32,37,117,10,0,0,0,0,108,111,102,91,37,100,93,32,61,32,37,48,46,49,53,102,10,0,0,0,0,0,0,0,104,105,91,37,100,93,32,61,32,37,117,10,0,0,0,0,104,105,102,91,37,100,93,32,61,32,37,48,46,49,53,102,10,0,0,0,0,0,0,0,70,105,110,105,115,104,101,100,32,119,114,105,116,105,110,103,32,116,114,97,110,115,102,111,114,109,32,116,97,98,108,101,46,10,10,0,0,0,0,0,69,82,82,79,82,58,32,112,117,116,99,95,116,114,97,110,115,102,111,114,109,95,116,97,98,108,101,32,58,32,108,111,102,105,108,116,91,37,100,93,32,116,111,32,104,105,103,104,32,97,116,32,37,102,10,0,69,82,82,79,82,58,32,112,117,116,99,95,116,114,97,110,115,102,111,114,109,95,116,97,98,108,101,32,58,32,104,105,102,105,108,116,91,37,100,93,32,116,111,32,104,105,103,104,32,97,116,32,37,102,10,0,87,114,105,116,105,110,103,32,113,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,46,10,0,0,0,0,113,105,91,37,100,93,32,61,32,37,100,32,32,32,32,58,58,32,32,122,105,91,37,100,93,32,61,32,37,100,10,0,113,91,37,100,93,32,61,32,37,53,46,55,102,32,32,58,58,32,32,122,91,37,100,93,32,61,32,37,53,46,55,102,10,0,0,0,0,0,0,0,70,105,110,105,115,104,101,100,32,119,114,105,116,105,110,103,32,113,117,97,110,116,105,122,97,116,105,111,110,32,116,97,98,108,101,46,10,10,0,0,69,82,82,79,82,32,58,32,112,117,116,99,95,113,117,97,110,116,105,122,97,116,105,111,110,95,116,97,98,108,101,32,58,32,81,91,37,100,93,32,116,111,32,104,105,103,104,32,97,116,32,37,102,10,0,0,69,82,82,79,82,32,58,32,112,117,116,99,95,113,117,97,110,116,105,122,97,116,105,111,110,95,116,97,98,108,101,32,58,32,90,91,37,100,93,32,116,111,32,104,105,103,104,32,97,116,32,37,102,10,0,0,87,114,105,116,105,110,103,32,102,114,97,109,101,32,104,101,97,100,101,114,46,10,0,0,109,95,115,104,105,102,116,32,61,32,37,102,32,32,58,58,32,114,95,115,99,97,108,101,32,61,32,37,102,10,0,0,70,105,110,105,115,104,101,100,32,119,114,105,116,105,110,103,32,102,114,97,109,101,32,104,101,97,100,101,114,46,10,10,0,0,0,0,0,0,0,0,87,114,105,116,105,110,103,32,98,108,111,99,107,32,104,101,97,100,101,114,46,10,0,0,70,105,110,105,115,104,101,100,32,119,114,105,116,105,110,103,32,98,108,111,99,107,32,104,101,97,100,101,114,46,10,10,0,0,0,0,0,0,0,0,78,73,83,84,95,67,79,77,0,0,0,0,0,0,0,0,116,37,100,32,45,62,32,120,32,61,32,37,100,32,32,121,32,61,32,37,100,32,58,32,100,120,32,61,32,37,100,32,32,100,121,32,61,32,37,100,32,58,32,105,114,32,61,32,37,100,32,32,105,99,32,61,32,37,100,10,0,0,0,0,10,10,0,0,0,0,0,0,116,37,100,32,45,62,32,120,32,61,32,37,100,32,32,121,32,61,32,37,100,32,58,32,108,120,32,61,32,37,100,32,32,108,121,32,61,32,37,100,10,0,0,0,0,0,0,0,69,82,82,79,82,58,32,99,111,110,118,95,105,109,103,95,50,95,102,108,116,58,32,111,118,101,114,102,108,111,119,32,97,116,32,37,100,10,0,0,69,82,82,79,82,32,58,32,113,117,97,110,116,105,122,101,32,58,32,99,97,108,108,111,99,32,58,32,115,105,112,10,0,0,0,0,0,0,0,0,37,100,32,45,62,32,37,51,46,54,102,10,0,0,0,0,69,82,82,79,82,32,58,32,119,115,113,95,100,101,99,111,109,112,111,115,101,32,58,32,109,97,108,108,111,99,32,58,32,102,100,97,116,97,49,10,0,0,0,0,0,0,0,0,84,97,98,108,101,32,76,101,110,32,61,32,37,100,10,0,84,97,98,108,101,32,73,68,32,61,32,37,100,10,0,0,98,105,116,115,91,37,100,93,32,61,32,37,100,10,0,0,118,97,108,117,101,115,91,37,100,93,32,61,32,37,100,10,0,0,0,0,0,0,0,0,83,116,97,114,116,32,119,114,105,116,105,110,103,32,104,117,102,102,109,97,110,32,116,97,98,108,101,46,10,0,0,0,70,105,110,105,115,104,101,100,32,119,114,105,116,105,110,103,32,104,117,102,102,109,97,110,32,116,97,98,108,101,46,10,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,102,105,110,100,95,104,117,102,102,95,115,105,122,101,115,32,58,32,99,97,108,108,111,99,32,58,32,99,111,100,101,115,105,122,101,10,0,0,0,0,69,82,82,79,82,32,58,32,102,105,110,100,95,104,117,102,102,95,115,105,122,101,115,32,58,32,109,97,108,108,111,99,32,58,32,111,116,104,101,114,115,10,0,0,0,0,0,0,99,111,100,101,115,105,122,101,91,37,100,93,32,61,32,37,100,10,0,0,0,0,0,0,69,82,82,79,82,32,58,32,102,105,110,100,95,110,117,109,95,104,117,102,102,95,115,105,122,101,115,32,58,32,99,97,108,108,111,99,32,58,32,98,105,116,115,10,0,0,0,0,65,68,74,85,83,84,32,61,32,37,100,10,0,0,0,0,69,82,82,79,82,32,58,32,115,111,114,116,95,104,117,102,102,98,105,116,115,32,58,32,109,97,108,108,111,99,32,58,32,116,98,105,116,115,10,0,69,82,82,79,82,32,58,32,115,111,114,116,95,104,117,102,102,98,105,116,115,32,58,32,67,111,100,101,32,108,101,110,103,116,104,32,111,102,32,37,100,32,105,115,32,103,114,101,97,116,101,114,32,116,104,97,110,32,49,54,46,10,0,0,72,117,102,102,98,105,116,115,32,97,102,116,101,114,32,115,111,114,116,105,110,103,46,10,0,0,0,0,0,0,0,0,115,111,114,116,95,98,105,116,115,91,37,100,93,32,61,32,37,100,10,0,0,0,0,0,69,82,82,79,82,32,58,32,115,111,114,116,95,99,111,100,101,95,115,105,122,101,115,32,58,32,99,97,108,108,111,99,32,58,32,118,97,108,117,101,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,98,117,105,108,100,95,104,117,102,102,99,111,100,101,95,116,97,98,108,101,32,58,32,99,97,108,108,111,99,32,58,32,110,101,119,95,104,117,102,102,99,111,100,101,95,116,97,98,108,101,10,0,0,0,0,0,104,117,102,102,95,115,105,122,101,91,37,100,93,32,61,32,37,100,10,0,0,0,0,0,104,117,102,102,95,99,111,100,101,91,37,100,93,32,61,32,37,100,10,0,0,0,0,0,69,82,82,79,82,32,58,32,98,117,105,108,100,95,104,117,102,102,115,105,122,101,115,32,58,32,99,97,108,108,111,99,32,58,32,104,117,102,102,99,111,100,101,95,116,97,98,108,101,10,0,0,0,0,0,0,73,110,32,98,117,105,108,100,95,104,117,102,102,115,105,122,101,115,58,10,0,0,0,0,104,102,95,115,122,91,37,100,93,32,61,32,37,100,10,0,87,114,105,116,105,110,103,32,67,111,109,109,101,110,116,32,70,105,101,108,100,32,116,111,32,66,117,102,102,101,114,46,10,0,0,0,0,0,0,0,70,105,110,105,115,104,101,100,32,87,114,105,116,105,110,103,32,67,111,109,109,101,110,116,32,70,105,101,108,100,32,116,111,32,66,117,102,102,101,114,46,10,0,0,0,0,0,0,78,73,83,84,95,67,79,77,0,0,0,0,0,0,0,0,54,0,0,0,0,0,0,0,37,100,0,0,0,0,0,0,80,73,88,95,87,73,68,84,72,0,0,0,0,0,0,0,80,73,88,95,72,69,73,71,72,84,0,0,0,0,0,0,80,73,88,95,68,69,80,84,72,0,0,0,0,0,0,0,80,80,73,0,0,0,0,0,76,79,83,83,89,0,0,0,48,0,0,0,0,0,0,0,87,65,82,78,73,78,71,32,58,32,99,111,109,98,105,110,101,95,110,105,115,116,99,111,109,32,58,32,0,0,0,0,114,101,113,117,101,115,116,32,116,111,32,117,110,115,101,116,32,108,111,115,115,121,32,102,108,97,103,32,105,103,110,111,114,101,100,10,0,0,0,0,67,79,76,79,82,83,80,65,67,69,0,0,0,0,0,0,71,82,65,89,0,0,0,0,67,79,77,80,82,69,83,83,73,79,78,0,0,0,0,0,87,83,81,0,0,0,0,0,37,102,0,0,0,0,0,0,87,83,81,95,66,73,84,82,65,84,69,0,0,0,0,0,69,82,82,79,82,32,58,32,102,101,116,50,115,116,114,105,110,103,32,58,32,109,97,108,108,111,99,32,58,32,115,116,114,10,0,0,0,0,0,0,32,0,0,0,0,0,0,0,10,0,0,0,0,0,0,0,69,82,82,79,82,32,58,32,115,116,114,105,110,103,50,102,101,116,32,58,32,101,109,112,116,121,32,110,97,109,101,32,115,116,114,105,110,103,32,102,111,117,110,100,10,0,0,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,102,101,116,95,114,101,116,32,58,32,115,116,114,100,117,112,32,58,32,102,101,116,45,62,118,97,108,117,101,115,91,93,10,0,69,82,82,79,82,32,58,32,117,112,100,97,116,101,102,101,116,95,114,101,116,32,58,32,115,116,114,100,117,112,32,58,32,102,101,116,45,62,110,97,109,101,115,91,93,10,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,109,97,108,108,111,99,32,58,32,102,101,116,10,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,99,97,108,108,111,99,32,58,32,102,101,116,45,62,110,97,109,101,115,10,0,0,0,0,0,69,82,82,79,82,32,58,32,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,99,97,108,108,111,99,32,58,32,102,101,116,45,62,118,97,108,117,101,115,10,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,114,101,97,108,108,111,99,32,58,32,102,101,116,45,62,110,97,109,101,115,10,0,0,69,82,82,79,82,32,58,32,114,101,97,108,108,111,99,102,101,116,95,114,101,116,32,58,32,114,101,97,108,108,111,99,32,58,32,102,101,116,45,62,118,97,108,117,101,115,0,0,69,82,82,79,82,32,58,32,108,111,111,107,117,112,102,101,116,32,58,32,115,116,114,100,117,112,32,58,32,118,97,108,117,101,10,0,0,0,0,0,69,82,82,79,82,32,58,32,112,117,116,99,95,98,121,116,101,32,58,32,98,117,102,102,101,114,32,111,118,101,114,108,111,119,32,58,32,97,108,108,111,99,32,61,32,37,100,44,32,114,101,113,117,101,115,116,32,61,32,37,100,10,0,0,69,82,82,79,82,32,58,32,112,117,116,99,95,98,121,116,101,115,32,58,32,98,117,102,102,101,114,32,111,118,101,114,108,111,119,32,58,32,97,108,108,111,99,32,61,32,37,100,44,32,114,101,113,117,101,115,116,32,61,32,37,100,10,0,69,82,82,79,82,32,58,32,114,101,97,100,95,97,115,99,105,105,95,102,105,108,101,32,58,32,99,97,108,108,111,99,32,58,32,115,116,114,10,0,114,98,0,0,0,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,97,115,99,105,105,95,102,105,108,101,32,58,32,102,111,112,101,110,32,58,32,37,115,10,0,0,0,69,82,82,79,82,32,58,32,114,101,97,100,95,97,115,99,105,105,95,102,105,108,101,32,58,32,102,114,101,97,100,32,58,32,0,0,0,0,0,0,111,110,108,121,32,37,100,32,111,102,32,37,100,32,98,121,116,101,115,32,114,101,97,100,10,0,0,0,0,0,0,0,114,98,0,0,0,0,0,0,69,82,82,79,82,32,58,32,102,105,108,101,115,105,122,101,32,58,32,102,105,108,101,32,37,115,32,99,111,117,108,100,32,110,111,116,32,98,101,32,111,112,101,110,101,100,10,0,69,82,82,79,82,32,58,32,102,105,108,101,115,105,122,101,32,58,32,115,101,101,107,105,110,103,32,116,111,32,69,79,70,32,111,102,32,102,105,108,101,32,37,115,32,102,97,105,108,101,100,10,0,0,0,0,69,82,82,79,82,32,58,32,102,105,108,101,115,105,122,101,32,58,32,102,116,101,108,108,32,97,116,32,69,79,70,32,111,102,32,102,105,108,101,32,37,115,32,102,97,105,108,101,100,10,0,0,0,0,0,0,69,82,82,79,82,58,32,37,115,10,0,0,0,0,0,0,69,82,82,79,82,58,32,37,115,58,32,37,115,10,0,0,69,82,82,79,82,58,32,37,115,58,32,37,115,58,32,37,115,10,0,0,0,0,0,0,109,97,108,108,111,99,0,0,109,97,108,108,111,99,95,117,99,104,97,114,0,0,0,0,69,82,82,79,82,58,32,37,115,58,32,37,115,10,0,0,69,82,82,79,82,58,32,37,115,58,32,37,115,58,32,37,115,10,0,0,0,0,0,0,0,128,64,192,32,160,96,224,16,144,80,208,48,176,112,240,8,136,72,200,40,168,104,232,24,152,88,216,56,184,120,248,4,132,68,196,36,164,100,228,20,148,84,212,52,180,116,244,12,140,76,204,44,172,108,236,28,156,92,220,60,188,124,252,2,130,66,194,34,162,98,226,18,146,82,210,50,178,114,242,10,138,74,202,42,170,106,234,26,154,90,218,58,186,122,250,6,134,70,198,38,166,102,230,22,150,86,214,54,182,118,246,14,142,78,206,46,174,110,238,30,158,94,222,62,190,126,254,1,129,65,193,33,161,97,225,17,145,81,209,49,177,113,241,9,137,73,201,41,169,105,233,25,153,89,217,57,185,121,249,5,133,69,197,37,165,101,229,21,149,85,213,53,181,117,245,13,141,77,205,45,173,109,237,29,157,93,221,61,189,125,253,3,131,67,195,35,163,99,227,19,147,83,211,51,179,115,243,11,139,75,203,43,171,107,235,27,155,91,219,59,187,123,251,7,135,71,199,39,167,103,231,23,151,87,215,55,183,119,247,15,143,79,207,47,175,111,239,31,159,95,223,63,191,127,255,2,0,0,192,3,0,0,192,4,0,0,192,5,0,0,192,6,0,0,192,7,0,0,192,8,0,0,192,9,0,0,192,10,0,0,192,11,0,0,192,12,0,0,192,13,0,0,192,14,0,0,192,15,0,0,192,16,0,0,192,17,0,0,192,18,0,0,192,19,0,0,192,20,0,0,192,21,0,0,192,22,0,0,192,23,0,0,192,24,0,0,192,25,0,0,192,26,0,0,192,27,0,0,192,28,0,0,192,29,0,0,192,30,0,0,192,31,0,0,192,0,0,0,179,1,0,0,195,2,0,0,195,3,0,0,195,4,0,0,195,5,0,0,195,6,0,0,195,7,0,0,195,8,0,0,195,9,0,0,195,10,0,0,195,11,0,0,195,12,0,0,195,13,0,0,211,14,0,0,195,15,0,0,195,0,0,12,187,1,0,12,195,2,0,12,195,3,0,12,195,4,0,12,211,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,1,2,3,4,5,6,7,8,9,255,255,255,255,255,255,255,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,255,255,255,255,255,255,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,0,0,0,0,0,0,0,0,1,2,4,7,3,6,5,0,0,0,0,0,0,0,0,105,110,102,105,110,105,116,121,0,0,0,0,0,0,0,0,110,97,110,0,0,0,0,0,95,112,137,0,255,9,47,15,10,0,0,0,100,0,0,0,232,3,0,0,16,39,0,0,160,134,1,0,64,66,15,0,128,150,152,0,0,225,245,5,17,0,10,0,17,17,17,0,0,0,0,5,0,0,0,0,0,0,9,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,15,10,17,17,17,3,10,7,0,1,19,9,11,11,0,0,9,6,11,0,0,11,0,6,17,0,0,0,17,17,17,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,17,0,10,10,17,17,17,0,10,0,0,2,0,9,11,0,0,0,9,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,14,0,0,0,0,0,0,0,0,0,0,0,13,0,0,0,4,13,0,0,0,0,9,14,0,0,0,0,0,14,0,0,14,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,16,0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,0,15,0,0,0,0,9,16,0,0,0,0,0,16,0,0,16,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,18,0,0,0,18,18,18,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,11,0,0,0,0,0,0,0,0,0,0,0,10,0,0,0,0,10,0,0,0,0,9,11,0,0,0,0,0,11,0,0,11,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,0,0,0,0,0,0,0,12,0,0,0,0,12,0,0,0,0,9,12,0,0,0,0,0,12,0,0,12,0,0,45,43,32,32,32,48,88,48,120,0,0,0,0,0,0,0,40,110,117,108,108,41,0,0,45,48,88,43,48,88,32,48,88,45,48,120,43,48,120,32,48,120,0,0,0,0,0,0,105,110,102,0,0,0,0,0,73,78,70,0,0,0,0,0,110,97,110,0,0,0,0,0,78,65,78,0,0,0,0,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,46,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);




var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}


   
  Module["_i64Subtract"] = _i64Subtract;

  
  
  var ___errno_state=0;function ___setErrNo(value) {
      // For convenient setting and returning of errno.
      HEAP32[((___errno_state)>>2)]=value;
      return value;
    }
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 79:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: {
          if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
          return 1;
        }
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

  
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            continue;
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          if (stream.tty.output.length) {
            stream.tty.ops.put_char(stream.tty, 10);
          }
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              result = process['stdin']['read']();
              if (!result) {
                if (process['stdin']['_readableState'] && process['stdin']['_readableState']['ended']) {
                  return null;  // EOF
                }
                return undefined;  // no data available
              }
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](tty.output.join(''));
            tty.output = [];
          } else {
            tty.output.push(TTY.utf8.processCChar(val));
          }
        }}};
  
  var MEMFS={ops_table:null,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            },
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
          // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
          // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
          // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
          node.contents = null; 
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },getFileDataAsRegularArray:function (node) {
        if (node.contents && node.contents.subarray) {
          var arr = [];
          for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
          return arr; // Returns a copy of the original data.
        }
        return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
      },getFileDataAsTypedArray:function (node) {
        if (node.contents && node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
        return new Uint8Array(node.contents);
      },expandFileStorage:function (node, newCapacity) {
  
        // If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
        // instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
        // increase the size.
        if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
          node.contents = MEMFS.getFileDataAsRegularArray(node);
          node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
        }
  
        if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
          var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
          if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
          // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
          // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
          // avoid overshooting the allocation cap by a very large margin.
          var CAPACITY_DOUBLING_MAX = 1024 * 1024;
          newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
          if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
          var oldContents = node.contents;
          node.contents = new Uint8Array(newCapacity); // Allocate new storage.
          if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
          return;
        }
        // Not using a typed array to back the file storage. Use a standard JS array instead.
        if (!node.contents && newCapacity > 0) node.contents = [];
        while (node.contents.length < newCapacity) node.contents.push(0);
      },resizeFileStorage:function (node, newSize) {
        if (node.usedBytes == newSize) return;
        if (newSize == 0) {
          node.contents = null; // Fully decommit when requesting a resize to zero.
          node.usedBytes = 0;
          return;
        }
  
        if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
          var oldContents = node.contents;
          node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
          node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
          node.usedBytes = newSize;
          return;
        }
        // Backing with a JS array.
        if (!node.contents) node.contents = [];
        if (node.contents.length > newSize) node.contents.length = newSize;
        else while (node.contents.length < newSize) node.contents.push(0);
        node.usedBytes = newSize;
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes) return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else
          {
            for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          if (!length) return 0;
          var node = stream.node;
          node.timestamp = Date.now();
  
          if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
            if (canOwn) { // Can we just reuse the buffer we are given?
              assert(position === 0, 'canOwn must imply no weird position inside the file');
              node.contents = buffer.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
              node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
              node.contents.set(buffer.subarray(offset, offset + length), position);
              return length;
            }
          }
          // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
          MEMFS.expandFileStorage(node, position+length);
          if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
          else
            for (var i = 0; i < length; i++) {
             node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
            }
          node.usedBytes = Math.max(node.usedBytes, position+length);
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          stream.ungotten = [];
          stream.position = position;
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < stream.node.usedBytes) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        // reuse all of the core MEMFS functionality
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },getDB:function (name, callback) {
        // check the cache first
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
  
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return callback(e);
        }
        req.onupgradeneeded = function(e) {
          var db = e.target.result;
          var transaction = e.target.transaction;
  
          var fileStore;
  
          if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
            fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
          } else {
            fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
          }
  
          fileStore.createIndex('timestamp', 'timestamp', { unique: false });
        };
        req.onsuccess = function() {
          db = req.result;
  
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function() {
          callback(this.error);
        };
      },getLocalSet:function (mount, callback) {
        var entries = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat;
  
          try {
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
          }
  
          entries[path] = { timestamp: stat.mtime };
        }
  
        return callback(null, { type: 'local', entries: entries });
      },getRemoteSet:function (mount, callback) {
        var entries = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function() { callback(this.error); };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          var index = store.index('timestamp');
  
          index.openKeyCursor().onsuccess = function(event) {
            var cursor = event.target.result;
  
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, entries: entries });
            }
  
            entries[cursor.primaryKey] = { timestamp: cursor.key };
  
            cursor.continue();
          };
        });
      },loadLocalEntry:function (path, callback) {
        var stat, node;
  
        try {
          var lookup = FS.lookupPath(path);
          node = lookup.node;
          stat = FS.stat(path);
        } catch (e) {
          return callback(e);
        }
  
        if (FS.isDir(stat.mode)) {
          return callback(null, { timestamp: stat.mtime, mode: stat.mode });
        } else if (FS.isFile(stat.mode)) {
          // Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
          // Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
          node.contents = MEMFS.getFileDataAsTypedArray(node);
          return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
        } else {
          return callback(new Error('node type not supported'));
        }
      },storeLocalEntry:function (path, entry, callback) {
        try {
          if (FS.isDir(entry.mode)) {
            FS.mkdir(path, entry.mode);
          } else if (FS.isFile(entry.mode)) {
            FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
          } else {
            return callback(new Error('node type not supported'));
          }
  
          FS.utime(path, entry.timestamp, entry.timestamp);
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },removeLocalEntry:function (path, callback) {
        try {
          var lookup = FS.lookupPath(path);
          var stat = FS.stat(path);
  
          if (FS.isDir(stat.mode)) {
            FS.rmdir(path);
          } else if (FS.isFile(stat.mode)) {
            FS.unlink(path);
          }
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },loadRemoteEntry:function (store, path, callback) {
        var req = store.get(path);
        req.onsuccess = function(event) { callback(null, event.target.result); };
        req.onerror = function() { callback(this.error); };
      },storeRemoteEntry:function (store, path, entry, callback) {
        var req = store.put(entry, path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function() { callback(this.error); };
      },removeRemoteEntry:function (store, path, callback) {
        var req = store.delete(path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function() { callback(this.error); };
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = [];
        Object.keys(src.entries).forEach(function (key) {
          var e = src.entries[key];
          var e2 = dst.entries[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create.push(key);
            total++;
          }
        });
  
        var remove = [];
        Object.keys(dst.entries).forEach(function (key) {
          var e = dst.entries[key];
          var e2 = src.entries[key];
          if (!e2) {
            remove.push(key);
            total++;
          }
        });
  
        if (!total) {
          return callback(null);
        }
  
        var errored = false;
        var completed = 0;
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        transaction.onerror = function() { done(this.error); };
  
        // sort paths in ascending order so directory entries are created
        // before the files inside them
        create.sort().forEach(function (path) {
          if (dst.type === 'local') {
            IDBFS.loadRemoteEntry(store, path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeLocalEntry(path, entry, done);
            });
          } else {
            IDBFS.loadLocalEntry(path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeRemoteEntry(store, path, entry, done);
            });
          }
        });
  
        // sort paths in descending order so files are deleted before their
        // parent directories
        remove.sort().reverse().forEach(function(path) {
          if (dst.type === 'local') {
            IDBFS.removeLocalEntry(path, done);
          } else {
            IDBFS.removeRemoteEntry(store, path, done);
          }
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so 
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          return flags;
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          stream.position = position;
          return position;
        }}};
  
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);
  
  function _fflush(stream) {
      // int fflush(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fflush.html
      // we don't currently perform any user-space buffering of data
    }var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || {};
  
        var defaults = {
          follow_mount: true,
          recurse_count: 0
        };
        for (var key in defaults) {
          if (opts[key] === undefined) {
            opts[key] = defaults[key];
          }
        }
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            if (!islast || (islast && opts.follow_mount)) {
              current = current.mounted.root;
            }
          }
  
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
              
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            this.mounted = null;
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
          };
  
          FS.FSNode.prototype = {};
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); },
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); },
            },
          });
        }
  
        var node = new FS.FSNode(parent, name, mode, rdev);
  
        FS.hashAddNode(node);
  
        return node;
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return !!node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var accmode = flag & 2097155;
        var perms = ['r', 'w', 'rw'][accmode];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        return FS.nodePermissions(dir, 'x');
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        // clone it, so we can return an instance of FSStream
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },getStreamFromPtr:function (ptr) {
        return FS.streams[ptr - 1];
      },getPtrForStream:function (stream) {
        return stream ? stream.fd + 1 : 0;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },getMounts:function (mount) {
        var mounts = [];
        var check = [mount];
  
        while (check.length) {
          var m = check.pop();
  
          mounts.push(m);
  
          check.push.apply(check, m.mounts);
        }
  
        return mounts;
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= mounts.length) {
            callback(null);
          }
        };
  
        // sync all mounts
        mounts.forEach(function (mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },mount:function (type, opts, mountpoint) {
        var root = mountpoint === '/';
        var pseudo = !mountpoint;
        var node;
  
        if (root && FS.root) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
          mountpoint = lookup.path;  // use the absolute path
          node = lookup.node;
  
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
          }
  
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
          }
        }
  
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          mounts: []
        };
  
        // create a root node for the fs
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
  
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          // set as a mountpoint
          node.mounted = mount;
  
          // add the new mount to the current mount's children
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
  
        return mountRoot;
      },unmount:function (mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
  
        // destroy the nodes for this mount, and all its child mounts
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
  
        Object.keys(FS.nameTable).forEach(function (hash) {
          var current = FS.nameTable[hash];
  
          while (current) {
            var next = current.name_next;
  
            if (mounts.indexOf(current.mount) !== -1) {
              FS.destroyNode(current);
            }
  
            current = next;
          }
        });
  
        // no longer a mountpoint
        node.mounted = null;
  
        // remove this mount from the child mounts
        var idx = node.mount.mounts.indexOf(mount);
        assert(idx !== -1);
        node.mount.mounts.splice(idx, 1);
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 438 /* 0666 */;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 511 /* 0777 */;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 438 /* 0666 */;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        try {
          if (FS.trackingDelegate['willMovePath']) {
            FS.trackingDelegate['willMovePath'](old_path, new_path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
        } catch(e) {
          console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readlink:function (path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return link.node_ops.readlink(link);
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // check permissions
        var err = FS.mayOpen(node, flags);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        try {
          if (FS.trackingDelegate['onOpenFile']) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate['onOpenFile'](path, trackingFlags);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
        }
        return stream;
      },close:function (stream) {
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        return stream.stream_ops.llseek(stream, offset, whence);
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
        } catch(e) {
          console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
        }
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = '';
          var utf8 = new Runtime.UTF8Processor();
          for (var i = 0; i < length; i++) {
            ret += utf8.processCChar(buf[i]);
          }
        } else if (opts.encoding === 'binary') {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var utf8 = new Runtime.UTF8Processor();
          var buf = new Uint8Array(utf8.processJSString(data));
          FS.write(stream, buf, 0, buf.length, 0, opts.canOwn);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0, opts.canOwn);
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function() { return 0; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // setup /dev/[u]random
        var random_device;
        if (typeof crypto !== 'undefined') {
          // for modern web browsers
          var randomBuffer = new Uint8Array(1);
          random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
        } else if (ENVIRONMENT_IS_NODE) {
          // for nodejs
          random_device = function() { return require('crypto').randomBytes(1)[0]; };
        } else {
          // default for ES5 platforms
          random_device = function() { return Math.floor(Math.random()*256); };
        }
        FS.createDevice('/dev', 'random', random_device);
        FS.createDevice('/dev', 'urandom', random_device);
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        HEAP32[((_stdin)>>2)]=FS.getPtrForStream(stdin);
        assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        HEAP32[((_stdout)>>2)]=FS.getPtrForStream(stdout);
        assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        HEAP32[((_stderr)>>2)]=FS.getPtrForStream(stderr);
        assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno) {
          this.errno = errno;
          for (var key in ERRNO_CODES) {
            if (ERRNO_CODES[key] === errno) {
              this.code = key;
              break;
            }
          }
          this.message = ERRNO_MESSAGES[errno];
          if (this.stack) this.stack = demangleAll(this.stack);
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
        function LazyUint8Array() {
          this.lengthKnown = false;
          this.chunks = []; // Loaded chunks. Index is the chunk number
        }
        LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
          if (idx > this.length-1 || idx < 0) {
            return undefined;
          }
          var chunkOffset = idx % this.chunkSize;
          var chunkNum = Math.floor(idx / this.chunkSize);
          return this.getter(chunkNum)[chunkOffset];
        }
        LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
          this.getter = getter;
        }
        LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
            // Find length
            var xhr = new XMLHttpRequest();
            xhr.open('HEAD', url, false);
            xhr.send(null);
            if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
            var datalength = Number(xhr.getResponseHeader("Content-length"));
            var header;
            var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
            var chunkSize = 1024*1024; // Chunk size in bytes
  
            if (!hasByteServing) chunkSize = datalength;
  
            // Function to get a range from the remote URL.
            var doXHR = (function(from, to) {
              if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
              if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
              // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
              var xhr = new XMLHttpRequest();
              xhr.open('GET', url, false);
              if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
              // Some hints to the browser that we want binary data.
              if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
              if (xhr.overrideMimeType) {
                xhr.overrideMimeType('text/plain; charset=x-user-defined');
              }
  
              xhr.send(null);
              if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
              if (xhr.response !== undefined) {
                return new Uint8Array(xhr.response || []);
              } else {
                return intArrayFromString(xhr.responseText || '', true);
              }
            });
            var lazyArray = this;
            lazyArray.setDataGetter(function(chunkNum) {
              var start = chunkNum * chunkSize;
              var end = (chunkNum+1) * chunkSize - 1; // including this byte
              end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
              if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
                lazyArray.chunks[chunkNum] = doXHR(start, end);
              }
              if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
              return lazyArray.chunks[chunkNum];
            });
  
            this._length = datalength;
            this._chunkSize = chunkSize;
            this.lengthKnown = true;
        }
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // Add a function that defers querying the file size until it is asked the first time.
        Object.defineProperty(node, "usedBytes", {
            get: function() { return this.contents.length; }
        });
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        function processData(byteArray) {
          function finish(byteArray) {
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency('cp ' + fullname);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency('cp ' + fullname);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency('cp ' + fullname);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};
  
  
  
  
  function _mkport() { throw 'TODO' }var SOCKFS={mount:function (mount) {
        return FS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createSocket:function (family, type, protocol) {
        var streaming = type == 1;
        if (protocol) {
          assert(streaming == (protocol == 6)); // if SOCK_STREAM, must be tcp
        }
  
        // create our internal socket structure
        var sock = {
          family: family,
          type: type,
          protocol: protocol,
          server: null,
          peers: {},
          pending: [],
          recv_queue: [],
          sock_ops: SOCKFS.websocket_sock_ops
        };
  
        // create the filesystem node to store the socket structure
        var name = SOCKFS.nextname();
        var node = FS.createNode(SOCKFS.root, name, 49152, 0);
        node.sock = sock;
  
        // and the wrapping stream that enables library functions such
        // as read and write to indirectly interact with the socket
        var stream = FS.createStream({
          path: name,
          node: node,
          flags: FS.modeStringToFlags('r+'),
          seekable: false,
          stream_ops: SOCKFS.stream_ops
        });
  
        // map the new stream to the socket structure (sockets have a 1:1
        // relationship with a stream)
        sock.stream = stream;
  
        return sock;
      },getSocket:function (fd) {
        var stream = FS.getStream(fd);
        if (!stream || !FS.isSocket(stream.node.mode)) {
          return null;
        }
        return stream.node.sock;
      },stream_ops:{poll:function (stream) {
          var sock = stream.node.sock;
          return sock.sock_ops.poll(sock);
        },ioctl:function (stream, request, varargs) {
          var sock = stream.node.sock;
          return sock.sock_ops.ioctl(sock, request, varargs);
        },read:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          var msg = sock.sock_ops.recvmsg(sock, length);
          if (!msg) {
            // socket is closed
            return 0;
          }
          buffer.set(msg.buffer, offset);
          return msg.buffer.length;
        },write:function (stream, buffer, offset, length, position /* ignored */) {
          var sock = stream.node.sock;
          return sock.sock_ops.sendmsg(sock, buffer, offset, length);
        },close:function (stream) {
          var sock = stream.node.sock;
          sock.sock_ops.close(sock);
        }},nextname:function () {
        if (!SOCKFS.nextname.current) {
          SOCKFS.nextname.current = 0;
        }
        return 'socket[' + (SOCKFS.nextname.current++) + ']';
      },websocket_sock_ops:{createPeer:function (sock, addr, port) {
          var ws;
  
          if (typeof addr === 'object') {
            ws = addr;
            addr = null;
            port = null;
          }
  
          if (ws) {
            // for sockets that've already connected (e.g. we're the server)
            // we can inspect the _socket property for the address
            if (ws._socket) {
              addr = ws._socket.remoteAddress;
              port = ws._socket.remotePort;
            }
            // if we're just now initializing a connection to the remote,
            // inspect the url property
            else {
              var result = /ws[s]?:\/\/([^:]+):(\d+)/.exec(ws.url);
              if (!result) {
                throw new Error('WebSocket URL must be in the format ws(s)://address:port');
              }
              addr = result[1];
              port = parseInt(result[2], 10);
            }
          } else {
            // create the actual websocket object and connect
            try {
              // runtimeConfig gets set to true if WebSocket runtime configuration is available.
              var runtimeConfig = (Module['websocket'] && ('object' === typeof Module['websocket']));
  
              // The default value is 'ws://' the replace is needed because the compiler replaces "//" comments with '#'
              // comments without checking context, so we'd end up with ws:#, the replace swaps the "#" for "//" again.
              var url = 'ws:#'.replace('#', '//');
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['url']) {
                  url = Module['websocket']['url']; // Fetch runtime WebSocket URL config.
                }
              }
  
              if (url === 'ws://' || url === 'wss://') { // Is the supplied URL config just a prefix, if so complete it.
                url = url + addr + ':' + port;
              }
  
              // Make the WebSocket subprotocol (Sec-WebSocket-Protocol) default to binary if no configuration is set.
              var subProtocols = 'binary'; // The default value is 'binary'
  
              if (runtimeConfig) {
                if ('string' === typeof Module['websocket']['subprotocol']) {
                  subProtocols = Module['websocket']['subprotocol']; // Fetch runtime WebSocket subprotocol config.
                }
              }
  
              // The regex trims the string (removes spaces at the beginning and end, then splits the string by
              // <any space>,<any space> into an Array. Whitespace removal is important for Websockify and ws.
              subProtocols = subProtocols.replace(/^ +| +$/g,"").split(/ *, */);
  
              // The node ws library API for specifying optional subprotocol is slightly different than the browser's.
              var opts = ENVIRONMENT_IS_NODE ? {'protocol': subProtocols.toString()} : subProtocols;
  
              // If node we use the ws library.
              var WebSocket = ENVIRONMENT_IS_NODE ? require('ws') : window['WebSocket'];
              ws = new WebSocket(url, opts);
              ws.binaryType = 'arraybuffer';
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EHOSTUNREACH);
            }
          }
  
  
          var peer = {
            addr: addr,
            port: port,
            socket: ws,
            dgram_send_queue: []
          };
  
          SOCKFS.websocket_sock_ops.addPeer(sock, peer);
          SOCKFS.websocket_sock_ops.handlePeerEvents(sock, peer);
  
          // if this is a bound dgram socket, send the port number first to allow
          // us to override the ephemeral port reported to us by remotePort on the
          // remote end.
          if (sock.type === 2 && typeof sock.sport !== 'undefined') {
            peer.dgram_send_queue.push(new Uint8Array([
                255, 255, 255, 255,
                'p'.charCodeAt(0), 'o'.charCodeAt(0), 'r'.charCodeAt(0), 't'.charCodeAt(0),
                ((sock.sport & 0xff00) >> 8) , (sock.sport & 0xff)
            ]));
          }
  
          return peer;
        },getPeer:function (sock, addr, port) {
          return sock.peers[addr + ':' + port];
        },addPeer:function (sock, peer) {
          sock.peers[peer.addr + ':' + peer.port] = peer;
        },removePeer:function (sock, peer) {
          delete sock.peers[peer.addr + ':' + peer.port];
        },handlePeerEvents:function (sock, peer) {
          var first = true;
  
          var handleOpen = function () {
            try {
              var queued = peer.dgram_send_queue.shift();
              while (queued) {
                peer.socket.send(queued);
                queued = peer.dgram_send_queue.shift();
              }
            } catch (e) {
              // not much we can do here in the way of proper error handling as we've already
              // lied and said this data was sent. shut it down.
              peer.socket.close();
            }
          };
  
          function handleMessage(data) {
            assert(typeof data !== 'string' && data.byteLength !== undefined);  // must receive an ArrayBuffer
            data = new Uint8Array(data);  // make a typed array view on the array buffer
  
  
            // if this is the port message, override the peer's port with it
            var wasfirst = first;
            first = false;
            if (wasfirst &&
                data.length === 10 &&
                data[0] === 255 && data[1] === 255 && data[2] === 255 && data[3] === 255 &&
                data[4] === 'p'.charCodeAt(0) && data[5] === 'o'.charCodeAt(0) && data[6] === 'r'.charCodeAt(0) && data[7] === 't'.charCodeAt(0)) {
              // update the peer's port and it's key in the peer map
              var newport = ((data[8] << 8) | data[9]);
              SOCKFS.websocket_sock_ops.removePeer(sock, peer);
              peer.port = newport;
              SOCKFS.websocket_sock_ops.addPeer(sock, peer);
              return;
            }
  
            sock.recv_queue.push({ addr: peer.addr, port: peer.port, data: data });
          };
  
          if (ENVIRONMENT_IS_NODE) {
            peer.socket.on('open', handleOpen);
            peer.socket.on('message', function(data, flags) {
              if (!flags.binary) {
                return;
              }
              handleMessage((new Uint8Array(data)).buffer);  // copy from node Buffer -> ArrayBuffer
            });
            peer.socket.on('error', function() {
              // don't throw
            });
          } else {
            peer.socket.onopen = handleOpen;
            peer.socket.onmessage = function peer_socket_onmessage(event) {
              handleMessage(event.data);
            };
          }
        },poll:function (sock) {
          if (sock.type === 1 && sock.server) {
            // listen sockets should only say they're available for reading
            // if there are pending clients.
            return sock.pending.length ? (64 | 1) : 0;
          }
  
          var mask = 0;
          var dest = sock.type === 1 ?  // we only care about the socket state for connection-based sockets
            SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport) :
            null;
  
          if (sock.recv_queue.length ||
              !dest ||  // connection-less sockets are always ready to read
              (dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {  // let recv return 0 once closed
            mask |= (64 | 1);
          }
  
          if (!dest ||  // connection-less sockets are always ready to write
              (dest && dest.socket.readyState === dest.socket.OPEN)) {
            mask |= 4;
          }
  
          if ((dest && dest.socket.readyState === dest.socket.CLOSING) ||
              (dest && dest.socket.readyState === dest.socket.CLOSED)) {
            mask |= 16;
          }
  
          return mask;
        },ioctl:function (sock, request, arg) {
          switch (request) {
            case 21531:
              var bytes = 0;
              if (sock.recv_queue.length) {
                bytes = sock.recv_queue[0].data.length;
              }
              HEAP32[((arg)>>2)]=bytes;
              return 0;
            default:
              return ERRNO_CODES.EINVAL;
          }
        },close:function (sock) {
          // if we've spawned a listen server, close it
          if (sock.server) {
            try {
              sock.server.close();
            } catch (e) {
            }
            sock.server = null;
          }
          // close any peer connections
          var peers = Object.keys(sock.peers);
          for (var i = 0; i < peers.length; i++) {
            var peer = sock.peers[peers[i]];
            try {
              peer.socket.close();
            } catch (e) {
            }
            SOCKFS.websocket_sock_ops.removePeer(sock, peer);
          }
          return 0;
        },bind:function (sock, addr, port) {
          if (typeof sock.saddr !== 'undefined' || typeof sock.sport !== 'undefined') {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already bound
          }
          sock.saddr = addr;
          sock.sport = port || _mkport();
          // in order to emulate dgram sockets, we need to launch a listen server when
          // binding on a connection-less socket
          // note: this is only required on the server side
          if (sock.type === 2) {
            // close the existing server if it exists
            if (sock.server) {
              sock.server.close();
              sock.server = null;
            }
            // swallow error operation not supported error that occurs when binding in the
            // browser where this isn't supported
            try {
              sock.sock_ops.listen(sock, 0);
            } catch (e) {
              if (!(e instanceof FS.ErrnoError)) throw e;
              if (e.errno !== ERRNO_CODES.EOPNOTSUPP) throw e;
            }
          }
        },connect:function (sock, addr, port) {
          if (sock.server) {
            throw new FS.ErrnoError(ERRNO_CODS.EOPNOTSUPP);
          }
  
          // TODO autobind
          // if (!sock.addr && sock.type == 2) {
          // }
  
          // early out if we're already connected / in the middle of connecting
          if (typeof sock.daddr !== 'undefined' && typeof sock.dport !== 'undefined') {
            var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
            if (dest) {
              if (dest.socket.readyState === dest.socket.CONNECTING) {
                throw new FS.ErrnoError(ERRNO_CODES.EALREADY);
              } else {
                throw new FS.ErrnoError(ERRNO_CODES.EISCONN);
              }
            }
          }
  
          // add the socket to our peer list and set our
          // destination address / port to match
          var peer = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
          sock.daddr = peer.addr;
          sock.dport = peer.port;
  
          // always "fail" in non-blocking mode
          throw new FS.ErrnoError(ERRNO_CODES.EINPROGRESS);
        },listen:function (sock, backlog) {
          if (!ENVIRONMENT_IS_NODE) {
            throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
          }
          if (sock.server) {
             throw new FS.ErrnoError(ERRNO_CODES.EINVAL);  // already listening
          }
          var WebSocketServer = require('ws').Server;
          var host = sock.saddr;
          sock.server = new WebSocketServer({
            host: host,
            port: sock.sport
            // TODO support backlog
          });
  
          sock.server.on('connection', function(ws) {
            if (sock.type === 1) {
              var newsock = SOCKFS.createSocket(sock.family, sock.type, sock.protocol);
  
              // create a peer on the new socket
              var peer = SOCKFS.websocket_sock_ops.createPeer(newsock, ws);
              newsock.daddr = peer.addr;
              newsock.dport = peer.port;
  
              // push to queue for accept to pick up
              sock.pending.push(newsock);
            } else {
              // create a peer on the listen socket so calling sendto
              // with the listen socket and an address will resolve
              // to the correct client
              SOCKFS.websocket_sock_ops.createPeer(sock, ws);
            }
          });
          sock.server.on('closed', function() {
            sock.server = null;
          });
          sock.server.on('error', function() {
            // don't throw
          });
        },accept:function (listensock) {
          if (!listensock.server) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          var newsock = listensock.pending.shift();
          newsock.stream.flags = listensock.stream.flags;
          return newsock;
        },getname:function (sock, peer) {
          var addr, port;
          if (peer) {
            if (sock.daddr === undefined || sock.dport === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            }
            addr = sock.daddr;
            port = sock.dport;
          } else {
            // TODO saddr and sport will be set for bind()'d UDP sockets, but what
            // should we be returning for TCP sockets that've been connect()'d?
            addr = sock.saddr || 0;
            port = sock.sport || 0;
          }
          return { addr: addr, port: port };
        },sendmsg:function (sock, buffer, offset, length, addr, port) {
          if (sock.type === 2) {
            // connection-less sockets will honor the message address,
            // and otherwise fall back to the bound destination address
            if (addr === undefined || port === undefined) {
              addr = sock.daddr;
              port = sock.dport;
            }
            // if there was no address to fall back to, error out
            if (addr === undefined || port === undefined) {
              throw new FS.ErrnoError(ERRNO_CODES.EDESTADDRREQ);
            }
          } else {
            // connection-based sockets will only use the bound
            addr = sock.daddr;
            port = sock.dport;
          }
  
          // find the peer for the destination address
          var dest = SOCKFS.websocket_sock_ops.getPeer(sock, addr, port);
  
          // early out if not connected with a connection-based socket
          if (sock.type === 1) {
            if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
            } else if (dest.socket.readyState === dest.socket.CONNECTING) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // create a copy of the incoming data to send, as the WebSocket API
          // doesn't work entirely with an ArrayBufferView, it'll just send
          // the entire underlying buffer
          var data;
          if (buffer instanceof Array || buffer instanceof ArrayBuffer) {
            data = buffer.slice(offset, offset + length);
          } else {  // ArrayBufferView
            data = buffer.buffer.slice(buffer.byteOffset + offset, buffer.byteOffset + offset + length);
          }
  
          // if we're emulating a connection-less dgram socket and don't have
          // a cached connection, queue the buffer to send upon connect and
          // lie, saying the data was sent now.
          if (sock.type === 2) {
            if (!dest || dest.socket.readyState !== dest.socket.OPEN) {
              // if we're not connected, open a new connection
              if (!dest || dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                dest = SOCKFS.websocket_sock_ops.createPeer(sock, addr, port);
              }
              dest.dgram_send_queue.push(data);
              return length;
            }
          }
  
          try {
            // send the actual data
            dest.socket.send(data);
            return length;
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
        },recvmsg:function (sock, length) {
          // http://pubs.opengroup.org/onlinepubs/7908799/xns/recvmsg.html
          if (sock.type === 1 && sock.server) {
            // tcp servers should not be recv()'ing on the listen socket
            throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
          }
  
          var queued = sock.recv_queue.shift();
          if (!queued) {
            if (sock.type === 1) {
              var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
  
              if (!dest) {
                // if we have a destination address but are not connected, error out
                throw new FS.ErrnoError(ERRNO_CODES.ENOTCONN);
              }
              else if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
                // return null if the socket has closed
                return null;
              }
              else {
                // else, our socket is in a valid state but truly has nothing available
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
            } else {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
          }
  
          // queued.data will be an ArrayBuffer if it's unadulterated, but if it's
          // requeued TCP data it'll be an ArrayBufferView
          var queuedLength = queued.data.byteLength || queued.data.length;
          var queuedOffset = queued.data.byteOffset || 0;
          var queuedBuffer = queued.data.buffer || queued.data;
          var bytesRead = Math.min(length, queuedLength);
          var res = {
            buffer: new Uint8Array(queuedBuffer, queuedOffset, bytesRead),
            addr: queued.addr,
            port: queued.port
          };
  
  
          // push back any unread data for TCP connections
          if (sock.type === 1 && bytesRead < queuedLength) {
            var bytesRemaining = queuedLength - bytesRead;
            queued.data = new Uint8Array(queuedBuffer, queuedOffset + bytesRead, bytesRemaining);
            sock.recv_queue.unshift(queued);
          }
  
          return res;
        }}};function _recv(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _read(fd, buf, len);
    }
  
  function _pread(fildes, buf, nbyte, offset) {
      // ssize_t pread(int fildes, void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _read(fildes, buf, nbyte) {
      // ssize_t read(int fildes, void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/read.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.read(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fread(ptr, size, nitems, stream) {
      // size_t fread(void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fread.html
      var bytesToRead = nitems * size;
      if (bytesToRead == 0) {
        return 0;
      }
      var bytesRead = 0;
      var streamObj = FS.getStreamFromPtr(stream);
      if (!streamObj) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return 0;
      }
      while (streamObj.ungotten.length && bytesToRead > 0) {
        HEAP8[((ptr++)>>0)]=streamObj.ungotten.pop();
        bytesToRead--;
        bytesRead++;
      }
      var err = _read(streamObj.fd, ptr, bytesToRead);
      if (err == -1) {
        if (streamObj) streamObj.error = true;
        return 0;
      }
      bytesRead += err;
      if (bytesRead < bytesToRead) streamObj.eof = true;
      return Math.floor(bytesRead / size);
    }

  
  function _close(fildes) {
      // int close(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/close.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        FS.close(stream);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }
  
  function _fsync(fildes) {
      // int fsync(int fildes);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fsync.html
      var stream = FS.getStream(fildes);
      if (stream) {
        // We write directly to the file system, so there's nothing to do here.
        return 0;
      } else {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
    }
  
  function _fileno(stream) {
      // int fileno(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fileno.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) return -1;
      return stream.fd;
    }function _fclose(stream) {
      // int fclose(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fclose.html
      var fd = _fileno(stream);
      _fsync(fd);
      return _close(fd);
    }

  
   
  Module["_strlen"] = _strlen; 
  Module["_strcat"] = _strcat;

  
  function _strerror_r(errnum, strerrbuf, buflen) {
      if (errnum in ERRNO_MESSAGES) {
        if (ERRNO_MESSAGES[errnum].length > buflen - 1) {
          return ___setErrNo(ERRNO_CODES.ERANGE);
        } else {
          var msg = ERRNO_MESSAGES[errnum];
          writeAsciiToMemory(msg, strerrbuf);
          return 0;
        }
      } else {
        return ___setErrNo(ERRNO_CODES.EINVAL);
      }
    }
  
  function _malloc(bytes) {
      /* Over-allocate to make sure it is byte-aligned by 8.
       * This will leak memory, but this is only the dummy
       * implementation (replaced by dlmalloc normally) so
       * not an issue.
       */
      var ptr = Runtime.dynamicAlloc(bytes + 8);
      return (ptr+8) & 0xFFFFFFF8;
    }
  Module["_malloc"] = _malloc;function _strerror(errnum) {
      if (!_strerror.buffer) _strerror.buffer = _malloc(256);
      _strerror_r(errnum, _strerror.buffer, 256);
      return _strerror.buffer;
    }

   
  Module["_bitshift64Shl"] = _bitshift64Shl;

  function _abort() {
      Module['abort']();
    }

  
  
  
  function _send(fd, buf, len, flags) {
      var sock = SOCKFS.getSocket(fd);
      if (!sock) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      // TODO honor flags
      return _write(fd, buf, len);
    }
  
  function _pwrite(fildes, buf, nbyte, offset) {
      // ssize_t pwrite(int fildes, const void *buf, size_t nbyte, off_t offset);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte, offset);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _write(fildes, buf, nbyte) {
      // ssize_t write(int fildes, const void *buf, size_t nbyte);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/write.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
  
  
      try {
        var slab = HEAP8;
        return FS.write(stream, slab, buf, nbyte);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fwrite(ptr, size, nitems, stream) {
      // size_t fwrite(const void *restrict ptr, size_t size, size_t nitems, FILE *restrict stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fwrite.html
      var bytesToWrite = nitems * size;
      if (bytesToWrite == 0) return 0;
      var fd = _fileno(stream);
      var bytesWritten = _write(fd, ptr, bytesToWrite);
      if (bytesWritten == -1) {
        var streamObj = FS.getStreamFromPtr(stream);
        if (streamObj) streamObj.error = true;
        return 0;
      } else {
        return Math.floor(bytesWritten / size);
      }
    }
  
  
  function __reallyNegative(x) {
      return x < 0 || (x === 0 && (1/x) === -Infinity);
    }function __formatString(format, varargs) {
      var textIndex = format;
      var argIndex = 0;
      function getNextArg(type) {
        // NOTE: Explicitly ignoring type safety. Otherwise this fails:
        //       int x = 4; printf("%c\n", (char)x);
        var ret;
        if (type === 'double') {
          ret = (HEAP32[((tempDoublePtr)>>2)]=HEAP32[(((varargs)+(argIndex))>>2)],HEAP32[(((tempDoublePtr)+(4))>>2)]=HEAP32[(((varargs)+((argIndex)+(4)))>>2)],(+(HEAPF64[(tempDoublePtr)>>3])));
        } else if (type == 'i64') {
          ret = [HEAP32[(((varargs)+(argIndex))>>2)],
                 HEAP32[(((varargs)+(argIndex+4))>>2)]];
  
        } else {
          type = 'i32'; // varargs are always i32, i64, or double
          ret = HEAP32[(((varargs)+(argIndex))>>2)];
        }
        argIndex += Runtime.getNativeFieldSize(type);
        return ret;
      }
  
      var ret = [];
      var curr, next, currArg;
      while(1) {
        var startTextIndex = textIndex;
        curr = HEAP8[((textIndex)>>0)];
        if (curr === 0) break;
        next = HEAP8[((textIndex+1)>>0)];
        if (curr == 37) {
          // Handle flags.
          var flagAlwaysSigned = false;
          var flagLeftAlign = false;
          var flagAlternative = false;
          var flagZeroPad = false;
          var flagPadSign = false;
          flagsLoop: while (1) {
            switch (next) {
              case 43:
                flagAlwaysSigned = true;
                break;
              case 45:
                flagLeftAlign = true;
                break;
              case 35:
                flagAlternative = true;
                break;
              case 48:
                if (flagZeroPad) {
                  break flagsLoop;
                } else {
                  flagZeroPad = true;
                  break;
                }
              case 32:
                flagPadSign = true;
                break;
              default:
                break flagsLoop;
            }
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          }
  
          // Handle width.
          var width = 0;
          if (next == 42) {
            width = getNextArg('i32');
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
          } else {
            while (next >= 48 && next <= 57) {
              width = width * 10 + (next - 48);
              textIndex++;
              next = HEAP8[((textIndex+1)>>0)];
            }
          }
  
          // Handle precision.
          var precisionSet = false, precision = -1;
          if (next == 46) {
            precision = 0;
            precisionSet = true;
            textIndex++;
            next = HEAP8[((textIndex+1)>>0)];
            if (next == 42) {
              precision = getNextArg('i32');
              textIndex++;
            } else {
              while(1) {
                var precisionChr = HEAP8[((textIndex+1)>>0)];
                if (precisionChr < 48 ||
                    precisionChr > 57) break;
                precision = precision * 10 + (precisionChr - 48);
                textIndex++;
              }
            }
            next = HEAP8[((textIndex+1)>>0)];
          }
          if (precision < 0) {
            precision = 6; // Standard default.
            precisionSet = false;
          }
  
          // Handle integer sizes. WARNING: These assume a 32-bit architecture!
          var argSize;
          switch (String.fromCharCode(next)) {
            case 'h':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 104) {
                textIndex++;
                argSize = 1; // char (actually i32 in varargs)
              } else {
                argSize = 2; // short (actually i32 in varargs)
              }
              break;
            case 'l':
              var nextNext = HEAP8[((textIndex+2)>>0)];
              if (nextNext == 108) {
                textIndex++;
                argSize = 8; // long long
              } else {
                argSize = 4; // long
              }
              break;
            case 'L': // long long
            case 'q': // int64_t
            case 'j': // intmax_t
              argSize = 8;
              break;
            case 'z': // size_t
            case 't': // ptrdiff_t
            case 'I': // signed ptrdiff_t or unsigned size_t
              argSize = 4;
              break;
            default:
              argSize = null;
          }
          if (argSize) textIndex++;
          next = HEAP8[((textIndex+1)>>0)];
  
          // Handle type specifier.
          switch (String.fromCharCode(next)) {
            case 'd': case 'i': case 'u': case 'o': case 'x': case 'X': case 'p': {
              // Integer.
              var signed = next == 100 || next == 105;
              argSize = argSize || 4;
              var currArg = getNextArg('i' + (argSize * 8));
              var origArg = currArg;
              var argText;
              // Flatten i64-1 [low, high] into a (slightly rounded) double
              if (argSize == 8) {
                currArg = Runtime.makeBigInt(currArg[0], currArg[1], next == 117);
              }
              // Truncate to requested size.
              if (argSize <= 4) {
                var limit = Math.pow(256, argSize) - 1;
                currArg = (signed ? reSign : unSign)(currArg & limit, argSize * 8);
              }
              // Format the number.
              var currAbsArg = Math.abs(currArg);
              var prefix = '';
              if (next == 100 || next == 105) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], null); else
                argText = reSign(currArg, 8 * argSize, 1).toString(10);
              } else if (next == 117) {
                if (argSize == 8 && i64Math) argText = i64Math.stringify(origArg[0], origArg[1], true); else
                argText = unSign(currArg, 8 * argSize, 1).toString(10);
                currArg = Math.abs(currArg);
              } else if (next == 111) {
                argText = (flagAlternative ? '0' : '') + currAbsArg.toString(8);
              } else if (next == 120 || next == 88) {
                prefix = (flagAlternative && currArg != 0) ? '0x' : '';
                if (argSize == 8 && i64Math) {
                  if (origArg[1]) {
                    argText = (origArg[1]>>>0).toString(16);
                    var lower = (origArg[0]>>>0).toString(16);
                    while (lower.length < 8) lower = '0' + lower;
                    argText += lower;
                  } else {
                    argText = (origArg[0]>>>0).toString(16);
                  }
                } else
                if (currArg < 0) {
                  // Represent negative numbers in hex as 2's complement.
                  currArg = -currArg;
                  argText = (currAbsArg - 1).toString(16);
                  var buffer = [];
                  for (var i = 0; i < argText.length; i++) {
                    buffer.push((0xF - parseInt(argText[i], 16)).toString(16));
                  }
                  argText = buffer.join('');
                  while (argText.length < argSize * 2) argText = 'f' + argText;
                } else {
                  argText = currAbsArg.toString(16);
                }
                if (next == 88) {
                  prefix = prefix.toUpperCase();
                  argText = argText.toUpperCase();
                }
              } else if (next == 112) {
                if (currAbsArg === 0) {
                  argText = '(nil)';
                } else {
                  prefix = '0x';
                  argText = currAbsArg.toString(16);
                }
              }
              if (precisionSet) {
                while (argText.length < precision) {
                  argText = '0' + argText;
                }
              }
  
              // Add sign if needed
              if (currArg >= 0) {
                if (flagAlwaysSigned) {
                  prefix = '+' + prefix;
                } else if (flagPadSign) {
                  prefix = ' ' + prefix;
                }
              }
  
              // Move sign to prefix so we zero-pad after the sign
              if (argText.charAt(0) == '-') {
                prefix = '-' + prefix;
                argText = argText.substr(1);
              }
  
              // Add padding.
              while (prefix.length + argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad) {
                    argText = '0' + argText;
                  } else {
                    prefix = ' ' + prefix;
                  }
                }
              }
  
              // Insert the result into the buffer.
              argText = prefix + argText;
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 'f': case 'F': case 'e': case 'E': case 'g': case 'G': {
              // Float.
              var currArg = getNextArg('double');
              var argText;
              if (isNaN(currArg)) {
                argText = 'nan';
                flagZeroPad = false;
              } else if (!isFinite(currArg)) {
                argText = (currArg < 0 ? '-' : '') + 'inf';
                flagZeroPad = false;
              } else {
                var isGeneral = false;
                var effectivePrecision = Math.min(precision, 20);
  
                // Convert g/G to f/F or e/E, as per:
                // http://pubs.opengroup.org/onlinepubs/9699919799/functions/printf.html
                if (next == 103 || next == 71) {
                  isGeneral = true;
                  precision = precision || 1;
                  var exponent = parseInt(currArg.toExponential(effectivePrecision).split('e')[1], 10);
                  if (precision > exponent && exponent >= -4) {
                    next = ((next == 103) ? 'f' : 'F').charCodeAt(0);
                    precision -= exponent + 1;
                  } else {
                    next = ((next == 103) ? 'e' : 'E').charCodeAt(0);
                    precision--;
                  }
                  effectivePrecision = Math.min(precision, 20);
                }
  
                if (next == 101 || next == 69) {
                  argText = currArg.toExponential(effectivePrecision);
                  // Make sure the exponent has at least 2 digits.
                  if (/[eE][-+]\d$/.test(argText)) {
                    argText = argText.slice(0, -1) + '0' + argText.slice(-1);
                  }
                } else if (next == 102 || next == 70) {
                  argText = currArg.toFixed(effectivePrecision);
                  if (currArg === 0 && __reallyNegative(currArg)) {
                    argText = '-' + argText;
                  }
                }
  
                var parts = argText.split('e');
                if (isGeneral && !flagAlternative) {
                  // Discard trailing zeros and periods.
                  while (parts[0].length > 1 && parts[0].indexOf('.') != -1 &&
                         (parts[0].slice(-1) == '0' || parts[0].slice(-1) == '.')) {
                    parts[0] = parts[0].slice(0, -1);
                  }
                } else {
                  // Make sure we have a period in alternative mode.
                  if (flagAlternative && argText.indexOf('.') == -1) parts[0] += '.';
                  // Zero pad until required precision.
                  while (precision > effectivePrecision++) parts[0] += '0';
                }
                argText = parts[0] + (parts.length > 1 ? 'e' + parts[1] : '');
  
                // Capitalize 'E' if needed.
                if (next == 69) argText = argText.toUpperCase();
  
                // Add sign.
                if (currArg >= 0) {
                  if (flagAlwaysSigned) {
                    argText = '+' + argText;
                  } else if (flagPadSign) {
                    argText = ' ' + argText;
                  }
                }
              }
  
              // Add padding.
              while (argText.length < width) {
                if (flagLeftAlign) {
                  argText += ' ';
                } else {
                  if (flagZeroPad && (argText[0] == '-' || argText[0] == '+')) {
                    argText = argText[0] + '0' + argText.slice(1);
                  } else {
                    argText = (flagZeroPad ? '0' : ' ') + argText;
                  }
                }
              }
  
              // Adjust case.
              if (next < 97) argText = argText.toUpperCase();
  
              // Insert the result into the buffer.
              argText.split('').forEach(function(chr) {
                ret.push(chr.charCodeAt(0));
              });
              break;
            }
            case 's': {
              // String.
              var arg = getNextArg('i8*');
              var argLength = arg ? _strlen(arg) : '(null)'.length;
              if (precisionSet) argLength = Math.min(argLength, precision);
              if (!flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              if (arg) {
                for (var i = 0; i < argLength; i++) {
                  ret.push(HEAPU8[((arg++)>>0)]);
                }
              } else {
                ret = ret.concat(intArrayFromString('(null)'.substr(0, argLength), true));
              }
              if (flagLeftAlign) {
                while (argLength < width--) {
                  ret.push(32);
                }
              }
              break;
            }
            case 'c': {
              // Character.
              if (flagLeftAlign) ret.push(getNextArg('i8'));
              while (--width > 0) {
                ret.push(32);
              }
              if (!flagLeftAlign) ret.push(getNextArg('i8'));
              break;
            }
            case 'n': {
              // Write the length written so far to the next parameter.
              var ptr = getNextArg('i32*');
              HEAP32[((ptr)>>2)]=ret.length;
              break;
            }
            case '%': {
              // Literal percent sign.
              ret.push(curr);
              break;
            }
            default: {
              // Unknown specifiers remain untouched.
              for (var i = startTextIndex; i < textIndex + 2; i++) {
                ret.push(HEAP8[((i)>>0)]);
              }
            }
          }
          textIndex += 2;
          // TODO: Support a/A (hex float) and m (last error) specifiers.
          // TODO: Support %1${specifier} for arg selection.
        } else {
          ret.push(curr);
          textIndex += 1;
        }
      }
      return ret;
    }function _fprintf(stream, format, varargs) {
      // int fprintf(FILE *restrict stream, const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var result = __formatString(format, varargs);
      var stack = Runtime.stackSave();
      var ret = _fwrite(allocate(result, 'i8', ALLOC_STACK), 1, result.length, stream);
      Runtime.stackRestore(stack);
      return ret;
    }

  function _printf(format, varargs) {
      // int printf(const char *restrict format, ...);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/printf.html
      var stdout = HEAP32[((_stdout)>>2)];
      return _fprintf(stdout, format, varargs);
    }


  
  function _open(path, oflag, varargs) {
      // int open(const char *path, int oflag, ...);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/open.html
      var mode = HEAP32[((varargs)>>2)];
      path = Pointer_stringify(path);
      try {
        var stream = FS.open(path, oflag, mode);
        return stream.fd;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fopen(filename, mode) {
      // FILE *fopen(const char *restrict filename, const char *restrict mode);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fopen.html
      var flags;
      mode = Pointer_stringify(mode);
      if (mode[0] == 'r') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 0;
        }
      } else if (mode[0] == 'w') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 512;
      } else if (mode[0] == 'a') {
        if (mode.indexOf('+') != -1) {
          flags = 2;
        } else {
          flags = 1;
        }
        flags |= 64;
        flags |= 1024;
      } else {
        ___setErrNo(ERRNO_CODES.EINVAL);
        return 0;
      }
      var fd = _open(filename, flags, allocate([0x1FF, 0, 0, 0], 'i32', ALLOC_STACK));  // All creation permissions.
      return fd === -1 ? 0 : FS.getPtrForStream(FS.getStream(fd));
    }


   
  Module["_i64Add"] = _i64Add;

  function _fabs() {
  return Math_abs.apply(null, arguments)
  }


  
  function _lseek(fildes, offset, whence) {
      // off_t lseek(int fildes, off_t offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/lseek.html
      var stream = FS.getStream(fildes);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      try {
        return FS.llseek(stream, offset, whence);
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }function _fseek(stream, offset, whence) {
      // int fseek(FILE *stream, long offset, int whence);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/fseek.html
      var fd = _fileno(stream);
      var ret = _lseek(fd, offset, whence);
      if (ret == -1) {
        return -1;
      }
      stream = FS.getStreamFromPtr(stream);
      stream.eof = false;
      return 0;
    }

  function _sqrt() {
  return Math_sqrt.apply(null, arguments)
  }

  function _ftell(stream) {
      // long ftell(FILE *stream);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/ftell.html
      stream = FS.getStreamFromPtr(stream);
      if (!stream) {
        ___setErrNo(ERRNO_CODES.EBADF);
        return -1;
      }
      if (FS.isChrdev(stream.node.mode)) {
        ___setErrNo(ERRNO_CODES.ESPIPE);
        return -1;
      } else {
        return stream.position;
      }
    }

  var Browser={mainLoop:{scheduler:null,method:"",shouldPause:false,paused:false,queue:[],pause:function () {
          Browser.mainLoop.shouldPause = true;
        },resume:function () {
          if (Browser.mainLoop.paused) {
            Browser.mainLoop.paused = false;
            Browser.mainLoop.scheduler();
          }
          Browser.mainLoop.shouldPause = false;
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        },runIter:function (func) {
          if (ABORT) return;
          if (Module['preMainLoop']) {
            var preRet = Module['preMainLoop']();
            if (preRet === false) {
              return; // |return false| skips a frame
            }
          }
          try {
            func();
          } catch (e) {
            if (e instanceof ExitStatus) {
              return;
            } else {
              if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
              throw e;
            }
          }
          if (Module['postMainLoop']) Module['postMainLoop']();
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          assert(typeof url == 'string', 'createObjectURL must return a url as a string');
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            assert(typeof url == 'string', 'createObjectURL must return a url as a string');
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas ||
                                document['msPointerLockElement'] === canvas;
        }
        if (canvas) {
          // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
          // Module['forcedAspectRatio'] = 4 / 3;
          
          canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                      canvas['mozRequestPointerLock'] ||
                                      canvas['webkitRequestPointerLock'] ||
                                      canvas['msRequestPointerLock'] ||
                                      function(){};
          canvas.exitPointerLock = document['exitPointerLock'] ||
                                   document['mozExitPointerLock'] ||
                                   document['webkitExitPointerLock'] ||
                                   document['msExitPointerLock'] ||
                                   function(){}; // no-op if function does not exist
          canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
  
          document.addEventListener('pointerlockchange', pointerLockChange, false);
          document.addEventListener('mozpointerlockchange', pointerLockChange, false);
          document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
          document.addEventListener('mspointerlockchange', pointerLockChange, false);
  
          if (Module['elementPointerLock']) {
            canvas.addEventListener("click", function(ev) {
              if (!Browser.pointerLock && canvas.requestPointerLock) {
                canvas.requestPointerLock();
                ev.preventDefault();
              }
            }, false);
          }
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        if (useWebGL && Module.ctx) return Module.ctx; // no need to recreate singleton GL context
  
        var ctx;
        var errorInfo = '?';
        function onContextCreationError(event) {
          errorInfo = event.statusMessage || errorInfo;
        }
        try {
          if (useWebGL) {
            var contextAttributes = {
              antialias: false,
              alpha: false
            };
  
            if (webGLContextAttributes) {
              for (var attribute in webGLContextAttributes) {
                contextAttributes[attribute] = webGLContextAttributes[attribute];
              }
            }
  
  
            canvas.addEventListener('webglcontextcreationerror', onContextCreationError, false);
            try {
              ['experimental-webgl', 'webgl'].some(function(webglId) {
                return ctx = canvas.getContext(webglId, contextAttributes);
              });
            } finally {
              canvas.removeEventListener('webglcontextcreationerror', onContextCreationError, false);
            }
          } else {
            ctx = canvas.getContext('2d');
          }
          if (!ctx) throw ':(';
        } catch (e) {
          Module.print('Could not create canvas: ' + [errorInfo, e]);
          return null;
        }
        if (useWebGL) {
          // possible GL_DEBUG entry point: ctx = wrapDebugGL(ctx);
  
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
        }
        if (setInModule) {
          if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
          Module.ctx = ctx;
          if (useWebGL) GLctx = ctx;
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          var canvasContainer = canvas.parentNode;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement'] ||
               document['msFullScreenElement'] || document['msFullscreenElement'] ||
               document['webkitCurrentFullScreenElement']) === canvasContainer) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'] ||
                                      document['msExitFullscreen'] ||
                                      document['exitFullscreen'] ||
                                      function() {};
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else {
            
            // remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
            canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
            canvasContainer.parentNode.removeChild(canvasContainer);
            
            if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
          Browser.updateCanvasDimensions(canvas);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
          document.addEventListener('MSFullscreenChange', fullScreenChange, false);
        }
  
        // create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
        var canvasContainer = document.createElement("div");
        canvas.parentNode.insertBefore(canvasContainer, canvas);
        canvasContainer.appendChild(canvas);
        
        // use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
        canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
                                            canvasContainer['mozRequestFullScreen'] ||
                                            canvasContainer['msRequestFullscreen'] ||
                                           (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
        canvasContainer.requestFullScreen();
      },nextRAF:0,fakeRequestAnimationFrame:function (func) {
        // try to keep 60fps between calls to here
        var now = Date.now();
        if (Browser.nextRAF === 0) {
          Browser.nextRAF = now + 1000/60;
        } else {
          while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
            Browser.nextRAF += 1000/60;
          }
        }
        var delay = Math.max(Browser.nextRAF - now, 0);
        setTimeout(func, delay);
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          Browser.fakeRequestAnimationFrame(func);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           Browser.fakeRequestAnimationFrame;
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (!ABORT) func();
        });
      },safeSetTimeout:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setTimeout(function() {
          if (!ABORT) func();
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setInterval(function() {
          if (!ABORT) func();
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },getMouseWheelDelta:function (event) {
        var delta = 0;
        switch (event.type) {
          case 'DOMMouseScroll': 
            delta = event.detail;
            break;
          case 'mousewheel': 
            delta = -event.wheelDelta;
            break;
          case 'wheel': 
            delta = event.deltaY;
            break;
          default:
            throw 'unrecognized mouse wheel event: ' + event.type;
        }
        return Math.max(-1, Math.min(1, delta));
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
  
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
          // If this assert lands, it's likely because the browser doesn't support scrollX or pageXOffset
          // and we have no viable fallback.
          assert((typeof scrollX !== 'undefined') && (typeof scrollY !== 'undefined'), 'Unable to retrieve scroll position, mouse positions likely broken.');
  
          if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
            var touch = event.touch;
            if (touch === undefined) {
              return; // the "touch" property is only defined in SDL
  
            }
            var adjustedX = touch.pageX - (scrollX + rect.left);
            var adjustedY = touch.pageY - (scrollY + rect.top);
  
            adjustedX = adjustedX * (cw / rect.width);
            adjustedY = adjustedY * (ch / rect.height);
  
            var coords = { x: adjustedX, y: adjustedY };
            
            if (event.type === 'touchstart') {
              Browser.lastTouches[touch.identifier] = coords;
              Browser.touches[touch.identifier] = coords;
            } else if (event.type === 'touchend' || event.type === 'touchmove') {
              Browser.lastTouches[touch.identifier] = Browser.touches[touch.identifier];
              Browser.touches[touch.identifier] = { x: adjustedX, y: adjustedY };
            } 
            return;
          }
  
          var x = event.pageX - (scrollX + rect.left);
          var y = event.pageY - (scrollY + rect.top);
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        Browser.updateCanvasDimensions(canvas, width, height);
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },updateCanvasDimensions:function (canvas, wNative, hNative) {
        if (wNative && hNative) {
          canvas.widthNative = wNative;
          canvas.heightNative = hNative;
        } else {
          wNative = canvas.widthNative;
          hNative = canvas.heightNative;
        }
        var w = wNative;
        var h = hNative;
        if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
          if (w/h < Module['forcedAspectRatio']) {
            w = Math.round(h * Module['forcedAspectRatio']);
          } else {
            h = Math.round(w / Module['forcedAspectRatio']);
          }
        }
        if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
             document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
             document['fullScreenElement'] || document['fullscreenElement'] ||
             document['msFullScreenElement'] || document['msFullscreenElement'] ||
             document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
           var factor = Math.min(screen.width / w, screen.height / h);
           w = Math.round(w * factor);
           h = Math.round(h * factor);
        }
        if (Browser.resizeCanvas) {
          if (canvas.width  != w) canvas.width  = w;
          if (canvas.height != h) canvas.height = h;
          if (typeof canvas.style != 'undefined') {
            canvas.style.removeProperty( "width");
            canvas.style.removeProperty("height");
          }
        } else {
          if (canvas.width  != wNative) canvas.width  = wNative;
          if (canvas.height != hNative) canvas.height = hNative;
          if (typeof canvas.style != 'undefined') {
            if (w != wNative || h != hNative) {
              canvas.style.setProperty( "width", w + "px", "important");
              canvas.style.setProperty("height", h + "px", "important");
            } else {
              canvas.style.removeProperty( "width");
              canvas.style.removeProperty("height");
            }
          }
        }
      }};

  function _time(ptr) {
      var ret = Math.floor(Date.now()/1000);
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }

  
  function __exit(status) {
      // void _exit(int status);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/exit.html
      Module['exit'](status);
    }function _exit(status) {
      __exit(status);
    }

  
  function _fmod(x, y) {
      return x % y;
    }function _fmodl() {
  return _fmod.apply(null, arguments)
  }

   
  Module["_bitshift64Lshr"] = _bitshift64Lshr;

   
  Module["_memset"] = _memset;

  var _BDtoILow=true;

  var _BDtoIHigh=true;

  function _ceil() {
  return Math_ceil.apply(null, arguments)
  }

  
  function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
      return dest;
    } 
  Module["_memcpy"] = _memcpy;

  function _log() {
  return Math_log.apply(null, arguments)
  }

  function _llvm_pow_f64() {
  return Math_pow.apply(null, arguments)
  }

  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) self.alloc(bytes);
      return ret;  // Previous break location.
    }

  function ___errno_location() {
      return ___errno_state;
    }

  var _BItoD=true;

  
  function _copysign(a, b) {
      return __reallyNegative(a) === __reallyNegative(b) ? a : -a;
    }function _copysignl() {
  return _copysign.apply(null, arguments)
  }


___errno_state = Runtime.staticAlloc(4); HEAP32[((___errno_state)>>2)]=0;
FS.staticInit();__ATINIT__.unshift({ func: function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() } });__ATMAIN__.push({ func: function() { FS.ignorePermissions = false } });__ATEXIT__.push({ func: function() { FS.quit() } });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;
__ATINIT__.unshift({ func: function() { TTY.init() } });__ATEXIT__.push({ func: function() { TTY.shutdown() } });TTY.utf8 = new Runtime.UTF8Processor();
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); NODEFS.staticInit(); }
__ATINIT__.push({ func: function() { SOCKFS.root = FS.mount(SOCKFS, {}, null); } });
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas) { Browser.requestFullScreen(lockPointer, resizeCanvas) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + 5242880;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

 var ctlz_i8 = allocate([8,7,6,6,5,5,5,5,4,4,4,4,4,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0], "i8", ALLOC_DYNAMIC);
 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);

  var Math_min = Math.min;
function nullFunc_iiii(x) { Module["printErr"]("Invalid function pointer called with signature 'iiii'. Perhaps this is an invalid value (e.g. caused by calling a virtual method on a NULL pointer)? Or calling a function with an incorrect type, which will fail? (it is worth building your source files with -Werror (warnings are errors), as warnings can indicate undefined behavior which can cause this)");  Module["printErr"]("Build with ASSERTIONS=2 for more info."); abort(x) }

function invoke_iiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_iiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

  function asmPrintInt(x, y) {
    Module.print('int ' + x + ',' + y);// + ' ' + new Error().stack);
  }
  function asmPrintFloat(x, y) {
    Module.print('float ' + x + ',' + y);// + ' ' + new Error().stack);
  }
  // EMSCRIPTEN_START_ASM
  var asm = (function(global, env, buffer) {
    'almost asm';
    var HEAP8 = new global.Int8Array(buffer);
    var HEAP16 = new global.Int16Array(buffer);
    var HEAP32 = new global.Int32Array(buffer);
    var HEAPU8 = new global.Uint8Array(buffer);
    var HEAPU16 = new global.Uint16Array(buffer);
    var HEAPU32 = new global.Uint32Array(buffer);
    var HEAPF32 = new global.Float32Array(buffer);
    var HEAPF64 = new global.Float64Array(buffer);
  
  var STACKTOP=env.STACKTOP|0;
  var STACK_MAX=env.STACK_MAX|0;
  var tempDoublePtr=env.tempDoublePtr|0;
  var ABORT=env.ABORT|0;
  var cttz_i8=env.cttz_i8|0;
  var ctlz_i8=env.ctlz_i8|0;
  var _stderr=env._stderr|0;
  var _stdout=env._stdout|0;

    var __THREW__ = 0;
    var threwValue = 0;
    var setjmpId = 0;
    var undef = 0;
    var nan = +env.NaN, inf = +env.Infinity;
    var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;
  
    var tempRet0 = 0;
    var tempRet1 = 0;
    var tempRet2 = 0;
    var tempRet3 = 0;
    var tempRet4 = 0;
    var tempRet5 = 0;
    var tempRet6 = 0;
    var tempRet7 = 0;
    var tempRet8 = 0;
    var tempRet9 = 0;
  var Math_floor=global.Math.floor;
  var Math_abs=global.Math.abs;
  var Math_sqrt=global.Math.sqrt;
  var Math_pow=global.Math.pow;
  var Math_cos=global.Math.cos;
  var Math_sin=global.Math.sin;
  var Math_tan=global.Math.tan;
  var Math_acos=global.Math.acos;
  var Math_asin=global.Math.asin;
  var Math_atan=global.Math.atan;
  var Math_atan2=global.Math.atan2;
  var Math_exp=global.Math.exp;
  var Math_log=global.Math.log;
  var Math_ceil=global.Math.ceil;
  var Math_imul=global.Math.imul;
  var abort=env.abort;
  var assert=env.assert;
  var asmPrintInt=env.asmPrintInt;
  var asmPrintFloat=env.asmPrintFloat;
  var Math_min=env.min;
  var nullFunc_iiii=env.nullFunc_iiii;
  var invoke_iiii=env.invoke_iiii;
  var _fabs=env._fabs;
  var _llvm_pow_f64=env._llvm_pow_f64;
  var _send=env._send;
  var _fread=env._fread;
  var _fmod=env._fmod;
  var _lseek=env._lseek;
  var __reallyNegative=env.__reallyNegative;
  var _fsync=env._fsync;
  var _fflush=env._fflush;
  var _pwrite=env._pwrite;
  var _strerror_r=env._strerror_r;
  var _open=env._open;
  var _sbrk=env._sbrk;
  var _emscripten_memcpy_big=env._emscripten_memcpy_big;
  var _fileno=env._fileno;
  var _sysconf=env._sysconf;
  var _close=env._close;
  var _fseek=env._fseek;
  var _pread=env._pread;
  var _printf=env._printf;
  var _fclose=env._fclose;
  var _log=env._log;
  var _write=env._write;
  var _ftell=env._ftell;
  var ___errno_location=env.___errno_location;
  var _recv=env._recv;
  var _copysign=env._copysign;
  var _mkport=env._mkport;
  var __exit=env.__exit;
  var _read=env._read;
  var _copysignl=env._copysignl;
  var _abort=env._abort;
  var _fwrite=env._fwrite;
  var _time=env._time;
  var _fprintf=env._fprintf;
  var _strerror=env._strerror;
  var _ceil=env._ceil;
  var __formatString=env.__formatString;
  var _sqrt=env._sqrt;
  var _fopen=env._fopen;
  var _exit=env._exit;
  var ___setErrNo=env.___setErrNo;
  var _fmodl=env._fmodl;
  var tempFloat = 0.0;

  // EMSCRIPTEN_START_FUNCS
  function stackAlloc(size) {
    size = size|0;
    var ret = 0;
    ret = STACKTOP;
    STACKTOP = (STACKTOP + size)|0;
  STACKTOP = (STACKTOP + 7)&-8;
    return ret|0;
  }
  function stackSave() {
    return STACKTOP|0;
  }
  function stackRestore(top) {
    top = top|0;
    STACKTOP = top;
  }
  function setThrew(threw, value) {
    threw = threw|0;
    value = value|0;
    if ((__THREW__|0) == 0) {
      __THREW__ = threw;
      threwValue = value;
    }
  }
  function copyTempFloat(ptr) {
    ptr = ptr|0;
    HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
    HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
    HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
    HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
  }
  function copyTempDouble(ptr) {
    ptr = ptr|0;
    HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
    HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
    HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
    HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
    HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
    HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
    HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
    HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
  }
  function setTempRet0(value) {
    value = value|0;
    tempRet0 = value;
  }
  function getTempRet0() {
    return tempRet0|0;
  }
  
function _main($argc,$argv) {
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $cfile = 0, $comment_text = 0, $depth = 0, $height = 0, $idata = 0, $ifile = 0, $ihead = 0, $odata = 0, $ofile = 0, $olen = 0, $outext = 0, $ppi = 0, $r_bitrate = 0, $rawflag = 0, $ret = 0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_buffer8 = 0, $vararg_ptr7 = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 4208|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $outext = sp + 80|0;
 $rawflag = sp + 76|0;
 $r_bitrate = sp + 72|0;
 $ifile = sp + 68|0;
 $cfile = sp + 64|0;
 $ofile = sp + 104|0;
 $ihead = sp + 60|0;
 $idata = sp + 56|0;
 $width = sp + 52|0;
 $height = sp + 48|0;
 $depth = sp + 44|0;
 $ppi = sp + 40|0;
 $odata = sp + 36|0;
 $olen = sp + 32|0;
 $comment_text = sp + 28|0;
 $0 = 0;
 $1 = $argc;
 $2 = $argv;
 $3 = $1;
 $4 = $2;
 _procargs($3,$4,$r_bitrate,$outext,$ifile,$rawflag,$width,$height,$depth,$ppi,$cfile);
 $5 = HEAP32[$rawflag>>2]|0;
 $6 = ($5|0)!=(0);
 $7 = $6 ^ 1;
 $8 = $7&1;
 $9 = HEAP32[$ifile>>2]|0;
 $10 = (_read_raw_or_ihead_wsq($8,$9,$ihead,$idata,$width,$height,$depth)|0);
 $ret = $10;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = $ret;
  _exit(($12|0));
  // unreachable;
 }
 $13 = HEAP32[8>>2]|0;
 $14 = ($13|0)>(0);
 if ($14) {
  $15 = HEAP32[_stdout>>2]|0;
  $16 = HEAP32[$ifile>>2]|0;
  HEAP32[$vararg_buffer>>2] = $16;
  (_fprintf(($15|0),(16|0),($vararg_buffer|0))|0);
 }
 $17 = HEAP32[$rawflag>>2]|0;
 $18 = ($17|0)!=(0);
 if (!($18)) {
  $19 = HEAP32[$ihead>>2]|0;
  $20 = (_get_density($19)|0);
  HEAP32[$ppi>>2] = $20;
  $21 = HEAP32[$ihead>>2]|0;
  _free($21);
 }
 $22 = HEAP32[$cfile>>2]|0;
 $23 = ($22|0)==(0|0);
 do {
  if ($23) {
   HEAP32[$comment_text>>2] = 0;
  } else {
   $24 = HEAP32[$cfile>>2]|0;
   $25 = (_read_ascii_file($24,$comment_text)|0);
   $ret = $25;
   $26 = ($25|0)!=(0);
   if ($26) {
    $27 = HEAP32[$idata>>2]|0;
    _free($27);
    $28 = $ret;
    _exit(($28|0));
    // unreachable;
   } else {
    break;
   }
  }
 } while(0);
 $29 = +HEAPF32[$r_bitrate>>2];
 $30 = HEAP32[$idata>>2]|0;
 $31 = HEAP32[$width>>2]|0;
 $32 = HEAP32[$height>>2]|0;
 $33 = HEAP32[$depth>>2]|0;
 $34 = HEAP32[$ppi>>2]|0;
 $35 = HEAP32[$comment_text>>2]|0;
 $36 = (_wsq_encode_mem($odata,$olen,$29,$30,$31,$32,$33,$34,$35)|0);
 $ret = $36;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$idata>>2]|0;
  _free($38);
  $39 = HEAP32[$comment_text>>2]|0;
  $40 = ($39|0)!=(0|0);
  if (!($40)) {
   $42 = $ret;
   _exit(($42|0));
   // unreachable;
  }
  $41 = HEAP32[$comment_text>>2]|0;
  _free($41);
  $42 = $ret;
  _exit(($42|0));
  // unreachable;
 }
 $43 = HEAP32[$idata>>2]|0;
 _free($43);
 $44 = HEAP32[$comment_text>>2]|0;
 $45 = ($44|0)!=(0|0);
 if ($45) {
  $46 = HEAP32[$comment_text>>2]|0;
  _free($46);
 }
 $47 = HEAP32[8>>2]|0;
 $48 = ($47|0)>(0);
 if ($48) {
  $49 = HEAP32[_stdout>>2]|0;
  $50 = HEAP32[$olen>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $50;
  (_fprintf(($49|0),(32|0),($vararg_buffer1|0))|0);
 }
 $51 = HEAP32[$ifile>>2]|0;
 _fileroot($51);
 $52 = HEAP32[$ifile>>2]|0;
 $53 = HEAP32[$outext>>2]|0;
 HEAP32[$vararg_buffer4>>2] = $52;
 $vararg_ptr7 = (($vararg_buffer4) + 4|0);
 HEAP32[$vararg_ptr7>>2] = $53;
 (_sprintf($ofile,88,$vararg_buffer4)|0);
 $54 = HEAP32[$odata>>2]|0;
 $55 = HEAP32[$olen>>2]|0;
 $56 = (_write_raw_from_memsize($ofile,$54,$55)|0);
 $ret = $56;
 $57 = ($56|0)!=(0);
 if ($57) {
  $58 = HEAP32[$odata>>2]|0;
  _free($58);
  $59 = $ret;
  _exit(($59|0));
  // unreachable;
 }
 $60 = HEAP32[8>>2]|0;
 $61 = ($60|0)>(0);
 if (!($61)) {
  $63 = HEAP32[$odata>>2]|0;
  _free($63);
  _exit(0);
  // unreachable;
 }
 $62 = HEAP32[_stdout>>2]|0;
 HEAP32[$vararg_buffer8>>2] = $ofile;
 (_fprintf(($62|0),(96|0),($vararg_buffer8|0))|0);
 $63 = HEAP32[$odata>>2]|0;
 _free($63);
 _exit(0);
 // unreachable;
 return 0|0;
}
function _procargs($argc,$argv,$r_bitrate,$outext,$ifile,$rawflag,$width,$height,$depth,$ppi,$cfile) {
 $argc = $argc|0;
 $argv = $argv|0;
 $r_bitrate = $r_bitrate|0;
 $outext = $outext|0;
 $ifile = $ifile|0;
 $rawflag = $rawflag|0;
 $width = $width|0;
 $height = $height|0;
 $depth = $depth|0;
 $ppi = $ppi|0;
 $cfile = $cfile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $argc;
 $1 = $argv;
 $2 = $r_bitrate;
 $3 = $outext;
 $4 = $ifile;
 $5 = $rawflag;
 $6 = $width;
 $7 = $height;
 $8 = $depth;
 $9 = $ppi;
 $10 = $cfile;
 $11 = $0;
 $12 = ($11|0)==(2);
 if ($12) {
  $13 = $1;
  $14 = (($13) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (_strcmp($15,128)|0);
  $17 = ($16|0)==(0);
  if ($17) {
   HEAP32[$vararg_buffer>>2] = 216;
   $vararg_ptr1 = (($vararg_buffer) + 4|0);
   HEAP32[$vararg_ptr1>>2] = 240;
   (_printf((144|0),($vararg_buffer|0))|0);
   _exit(0);
   // unreachable;
  }
 }
 $18 = $0;
 $19 = ($18|0)<(4);
 if ($19) {
  $22 = $1;
  $23 = HEAP32[$22>>2]|0;
  _print_usage($23);
  _exit(-1);
  // unreachable;
 }
 $20 = $0;
 $21 = ($20|0)>(7);
 if ($21) {
  $22 = $1;
  $23 = HEAP32[$22>>2]|0;
  _print_usage($23);
  _exit(-1);
  // unreachable;
 }
 $24 = $1;
 $25 = (($24) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $2;
 HEAP32[$vararg_buffer2>>2] = $27;
 (_sscanf($26,256,$vararg_buffer2)|0);
 $28 = $1;
 $29 = (($28) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $3;
 HEAP32[$31>>2] = $30;
 $32 = $1;
 $33 = (($32) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = $4;
 HEAP32[$35>>2] = $34;
 $36 = $5;
 HEAP32[$36>>2] = 0;
 $37 = $6;
 HEAP32[$37>>2] = -1;
 $38 = $7;
 HEAP32[$38>>2] = -1;
 $39 = $8;
 HEAP32[$39>>2] = -1;
 $40 = $9;
 HEAP32[$40>>2] = -1;
 $41 = $10;
 HEAP32[$41>>2] = 0;
 $42 = $0;
 $43 = ($42|0)==(5);
 if ($43) {
  $44 = $1;
  $45 = (($44) + 16|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = $10;
  HEAP32[$47>>2] = $46;
  STACKTOP = sp;return;
 }
 $48 = $0;
 $49 = ($48|0)==(6);
 if ($49) {
  label = 11;
 } else {
  $50 = $0;
  $51 = ($50|0)==(7);
  if ($51) {
   label = 11;
  }
 }
 if ((label|0) == 11) {
  $52 = $1;
  $53 = (($52) + 16|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_strncmp($54,264,2)|0);
  $56 = ($55|0)==(0);
  if (!($56)) {
   $57 = $1;
   $58 = (($57) + 16|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_strncmp($59,272,4)|0);
   $61 = ($60|0)==(0);
   if (!($61)) {
    $62 = $1;
    $63 = (($62) + 16|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = (_strncmp($64,280,7)|0);
    $66 = ($65|0)==(0);
    if (!($66)) {
     $85 = $1;
     $86 = HEAP32[$85>>2]|0;
     _print_usage($86);
     $87 = HEAP32[_stderr>>2]|0;
     (_fprintf(($87|0),(320|0),($vararg_buffer8|0))|0);
     _exit(-1);
     // unreachable;
    }
   }
  }
  $67 = $5;
  HEAP32[$67>>2] = 1;
  $68 = $1;
  $69 = (($68) + 20|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $1;
  $72 = HEAP32[$71>>2]|0;
  $73 = $6;
  $74 = $7;
  $75 = $8;
  $76 = $9;
  _parse_w_h_d_ppi($70,$72,$73,$74,$75,$76);
  $77 = $8;
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)!=(8);
  if ($79) {
   $80 = $1;
   $81 = HEAP32[$80>>2]|0;
   _print_usage($81);
   $82 = HEAP32[_stderr>>2]|0;
   $83 = $8;
   $84 = HEAP32[$83>>2]|0;
   HEAP32[$vararg_buffer5>>2] = $84;
   (_fprintf(($82|0),(288|0),($vararg_buffer5|0))|0);
   _exit(-1);
   // unreachable;
  }
  $88 = $0;
  $89 = ($88|0)==(7);
  if ($89) {
   $90 = $1;
   $91 = (($90) + 24|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = $10;
   HEAP32[$93>>2] = $92;
  }
 }
 STACKTOP = sp;return;
}
function _print_usage($arg0) {
 $arg0 = $arg0|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $arg0;
 $1 = HEAP32[_stderr>>2]|0;
 $2 = $0;
 HEAP32[$vararg_buffer>>2] = $2;
 (_fprintf(($1|0),(376|0),($vararg_buffer|0))|0);
 $3 = HEAP32[_stderr>>2]|0;
 (_fprintf(($3|0),(392|0),($vararg_buffer1|0))|0);
 $4 = HEAP32[_stderr>>2]|0;
 (_fprintf(($4|0),(432|0),($vararg_buffer3|0))|0);
 $5 = HEAP32[_stderr>>2]|0;
 (_fprintf(($5|0),(488|0),($vararg_buffer5|0))|0);
 STACKTOP = sp;return;
}
function _write_raw_from_memsize($ofile,$odata,$olen) {
 $ofile = $ofile|0;
 $odata = $odata|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $n = 0, $outfp = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ofile;
 $2 = $odata;
 $3 = $olen;
 $4 = $1;
 $5 = (_fopen(($4|0),(560|0))|0);
 $outfp = $5;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  $8 = $1;
  HEAP32[$vararg_buffer>>2] = $8;
  (_fprintf(($7|0),(568|0),($vararg_buffer|0))|0);
  $0 = -2;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 $9 = $2;
 $10 = $3;
 $11 = $outfp;
 $12 = (_fwrite(($9|0),1,($10|0),($11|0))|0);
 $n = $12;
 $13 = $3;
 $14 = ($12|0)!=($13|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(616|0),($vararg_buffer1|0))|0);
  $16 = HEAP32[_stderr>>2]|0;
  $17 = $n;
  $18 = $3;
  $19 = $1;
  HEAP32[$vararg_buffer3>>2] = $17;
  $vararg_ptr6 = (($vararg_buffer3) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $18;
  $vararg_ptr7 = (($vararg_buffer3) + 8|0);
  HEAP32[$vararg_ptr7>>2] = $19;
  (_fprintf(($16|0),(664|0),($vararg_buffer3|0))|0);
  $0 = -3;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 } else {
  $20 = $outfp;
  (_fclose(($20|0))|0);
  $0 = 0;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _read_ihead($ifile,$ohead,$odata,$owidth,$oheight,$odepth) {
 $ifile = $ifile|0;
 $ohead = $ohead|0;
 $odata = $odata|0;
 $owidth = $owidth|0;
 $oheight = $oheight|0;
 $odepth = $odepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $depth = 0, $height = 0, $idata = 0, $ihead = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $ihead = sp + 28|0;
 $idata = sp + 24|0;
 $width = sp + 20|0;
 $height = sp + 16|0;
 $depth = sp + 12|0;
 $1 = $ifile;
 $2 = $ohead;
 $3 = $odata;
 $4 = $owidth;
 $5 = $oheight;
 $6 = $odepth;
 $7 = $1;
 _ReadIheadRaster($7,$ihead,$idata,$width,$height,$depth);
 $8 = HEAP32[$depth>>2]|0;
 $9 = ($8|0)!=(8);
 if ($9) {
  $10 = HEAP32[$depth>>2]|0;
  $11 = ($10|0)!=(24);
  if ($11) {
   $12 = HEAP32[$ihead>>2]|0;
   _free($12);
   $13 = HEAP32[$idata>>2]|0;
   _free($13);
   $14 = HEAP32[_stderr>>2]|0;
   (_fprintf(($14|0),(712|0),($vararg_buffer|0))|0);
   $15 = HEAP32[_stderr>>2]|0;
   $16 = HEAP32[$depth>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $16;
   (_fprintf(($15|0),(736|0),($vararg_buffer1|0))|0);
   $0 = -2;
   $27 = $0;
   STACKTOP = sp;return ($27|0);
  }
 }
 $17 = HEAP32[$ihead>>2]|0;
 $18 = $2;
 HEAP32[$18>>2] = $17;
 $19 = HEAP32[$idata>>2]|0;
 $20 = $3;
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$width>>2]|0;
 $22 = $4;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$height>>2]|0;
 $24 = $5;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$depth>>2]|0;
 $26 = $6;
 HEAP32[$26>>2] = $25;
 $0 = 0;
 $27 = $0;
 STACKTOP = sp;return ($27|0);
}
function _read_raw($ifile,$odata,$owidth,$oheight,$odepth) {
 $ifile = $ifile|0;
 $odata = $odata|0;
 $owidth = $owidth|0;
 $oheight = $oheight|0;
 $odepth = $odepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $depth = 0, $height = 0;
 var $idata = 0, $img_siz = 0, $infp = 0, $num_pix = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr13 = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ifile;
 $2 = $odata;
 $3 = $owidth;
 $4 = $oheight;
 $5 = $odepth;
 $6 = $3;
 $7 = HEAP32[$6>>2]|0;
 $width = $7;
 $8 = $4;
 $9 = HEAP32[$8>>2]|0;
 $height = $9;
 $10 = $5;
 $11 = HEAP32[$10>>2]|0;
 $depth = $11;
 $12 = $depth;
 $13 = ($12|0)!=(8);
 if ($13) {
  $14 = $depth;
  $15 = ($14|0)!=(24);
  if ($15) {
   $16 = HEAP32[_stderr>>2]|0;
   (_fprintf(($16|0),(768|0),($vararg_buffer|0))|0);
   $17 = HEAP32[_stderr>>2]|0;
   $18 = $depth;
   HEAP32[$vararg_buffer1>>2] = $18;
   (_fprintf(($17|0),(736|0),($vararg_buffer1|0))|0);
   $0 = -3;
   $57 = $0;
   STACKTOP = sp;return ($57|0);
  }
 }
 $19 = $width;
 $20 = $height;
 $21 = Math_imul($19, $20)|0;
 $22 = $depth;
 $23 = $22 >> 3;
 $24 = Math_imul($21, $23)|0;
 $num_pix = $24;
 $25 = $num_pix;
 $26 = $25;
 $27 = (_malloc($26)|0);
 $idata = $27;
 $28 = $idata;
 $29 = ($28|0)==(0|0);
 if ($29) {
  $30 = HEAP32[_stderr>>2]|0;
  (_fprintf(($30|0),(792|0),($vararg_buffer3|0))|0);
  $0 = -4;
  $57 = $0;
  STACKTOP = sp;return ($57|0);
 }
 $31 = $1;
 $32 = (_fopen(($31|0),(552|0))|0);
 $infp = $32;
 $33 = ($32|0)==(0|0);
 if ($33) {
  $34 = HEAP32[_stderr>>2]|0;
  $35 = $1;
  HEAP32[$vararg_buffer5>>2] = $35;
  (_fprintf(($34|0),(832|0),($vararg_buffer5|0))|0);
  $0 = -5;
  $57 = $0;
  STACKTOP = sp;return ($57|0);
 }
 $36 = $idata;
 $37 = $num_pix;
 $38 = $infp;
 $39 = (_fread(($36|0),1,($37|0),($38|0))|0);
 $img_siz = $39;
 $40 = $img_siz;
 $41 = $num_pix;
 $42 = ($40|0)!=($41|0);
 if ($42) {
  $43 = $idata;
  _free($43);
  $44 = HEAP32[_stderr>>2]|0;
  (_fprintf(($44|0),(856|0),($vararg_buffer8|0))|0);
  $45 = HEAP32[_stderr>>2]|0;
  $46 = $img_siz;
  $47 = $num_pix;
  HEAP32[$vararg_buffer10>>2] = $46;
  $vararg_ptr13 = (($vararg_buffer10) + 4|0);
  HEAP32[$vararg_ptr13>>2] = $47;
  (_fprintf(($45|0),(888|0),($vararg_buffer10|0))|0);
  $0 = -6;
  $57 = $0;
  STACKTOP = sp;return ($57|0);
 } else {
  $48 = $infp;
  (_fclose(($48|0))|0);
  $49 = $idata;
  $50 = $2;
  HEAP32[$50>>2] = $49;
  $51 = $width;
  $52 = $3;
  HEAP32[$52>>2] = $51;
  $53 = $height;
  $54 = $4;
  HEAP32[$54>>2] = $53;
  $55 = $depth;
  $56 = $5;
  HEAP32[$56>>2] = $55;
  $0 = 0;
  $57 = $0;
  STACKTOP = sp;return ($57|0);
 }
 return 0|0;
}
function _read_raw_or_ihead_wsq($iheadflag,$ifile,$ohead,$odata,$owidth,$oheight,$odepth) {
 $iheadflag = $iheadflag|0;
 $ifile = $ifile|0;
 $ohead = $ohead|0;
 $odata = $odata|0;
 $owidth = $owidth|0;
 $oheight = $oheight|0;
 $odepth = $odepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $height = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0;
 var $vararg_ptr6 = 0, $width = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $iheadflag;
 $2 = $ifile;
 $3 = $ohead;
 $4 = $odata;
 $5 = $owidth;
 $6 = $oheight;
 $7 = $odepth;
 $8 = $1;
 $9 = ($8|0)!=(0);
 do {
  if ($9) {
   $10 = $2;
   $11 = $3;
   $12 = $4;
   $13 = $5;
   $14 = $6;
   $15 = $7;
   $16 = (_read_ihead($10,$11,$12,$13,$14,$15)|0);
   $ret = $16;
   $17 = ($16|0)!=(0);
   if (!($17)) {
    break;
   }
   $18 = $ret;
   $0 = $18;
   $52 = $0;
   STACKTOP = sp;return ($52|0);
  } else {
   $19 = $2;
   $20 = $4;
   $21 = $5;
   $22 = $6;
   $23 = $7;
   $24 = (_read_raw($19,$20,$21,$22,$23)|0);
   $ret = $24;
   $25 = ($24|0)!=(0);
   if (!($25)) {
    break;
   }
   $26 = $ret;
   $0 = $26;
   $52 = $0;
   STACKTOP = sp;return ($52|0);
  }
 } while(0);
 $27 = $7;
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(8);
 if ($29) {
  $30 = HEAP32[_stderr>>2]|0;
  (_fprintf(($30|0),(920|0),($vararg_buffer|0))|0);
  $31 = HEAP32[_stderr>>2]|0;
  $32 = $7;
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $33;
  (_fprintf(($31|0),(952|0),($vararg_buffer1|0))|0);
  $0 = -2;
  $52 = $0;
  STACKTOP = sp;return ($52|0);
 }
 $34 = $5;
 $35 = HEAP32[$34>>2]|0;
 $width = $35;
 $36 = $6;
 $37 = HEAP32[$36>>2]|0;
 $height = $37;
 $38 = $width;
 $39 = ($38|0)<(256);
 if (!($39)) {
  $40 = $height;
  $41 = ($40|0)<(256);
  if (!($41)) {
   $0 = 0;
   $52 = $0;
   STACKTOP = sp;return ($52|0);
  }
 }
 $42 = $1;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = $3;
  $45 = HEAP32[$44>>2]|0;
  _free($45);
 }
 $46 = $4;
 $47 = HEAP32[$46>>2]|0;
 _free($47);
 $48 = HEAP32[_stderr>>2]|0;
 HEAP32[$vararg_buffer3>>2] = 256;
 $vararg_ptr6 = (($vararg_buffer3) + 4|0);
 HEAP32[$vararg_ptr6>>2] = 256;
 (_fprintf(($48|0),(976|0),($vararg_buffer3|0))|0);
 $49 = HEAP32[_stderr>>2]|0;
 $50 = $width;
 $51 = $height;
 HEAP32[$vararg_buffer7>>2] = $50;
 $vararg_ptr10 = (($vararg_buffer7) + 4|0);
 HEAP32[$vararg_ptr10>>2] = $51;
 (_fprintf(($49|0),(1040|0),($vararg_buffer7|0))|0);
 $0 = -3;
 $52 = $0;
 STACKTOP = sp;return ($52|0);
}
function _parse_w_h_d_ppi($argstr,$arg0,$width,$height,$depth,$ppi) {
 $argstr = $argstr|0;
 $arg0 = $arg0|0;
 $width = $width|0;
 $height = $height|0;
 $depth = $depth|0;
 $ppi = $ppi|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $aptr = 0, $cbuff = 0, $cptr = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $cbuff = sp + 44|0;
 $0 = $argstr;
 $1 = $arg0;
 $2 = $width;
 $3 = $height;
 $4 = $depth;
 $5 = $ppi;
 $6 = $0;
 $aptr = $6;
 $cptr = $cbuff;
 while(1) {
  $7 = $aptr;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = $aptr;
   $12 = HEAP8[$11>>0]|0;
   $13 = $12 << 24 >> 24;
   $14 = ($13|0)!=(44);
   $90 = $14;
  } else {
   $90 = 0;
  }
  if (!($90)) {
   break;
  }
  $15 = $aptr;
  $16 = (($15) + 1|0);
  $aptr = $16;
  $17 = HEAP8[$15>>0]|0;
  $18 = $cptr;
  $19 = (($18) + 1|0);
  $cptr = $19;
  HEAP8[$18>>0] = $17;
 }
 $20 = $aptr;
 $21 = HEAP8[$20>>0]|0;
 $22 = $21 << 24 >> 24;
 $23 = ($22|0)==(0);
 if ($23) {
  $24 = $1;
  _print_usage($24);
  $25 = HEAP32[_stderr>>2]|0;
  (_fprintf(($25|0),(1088|0),($vararg_buffer|0))|0);
  _exit(-1);
  // unreachable;
 }
 $26 = $cptr;
 HEAP8[$26>>0] = 0;
 $27 = (_atoi($cbuff)|0);
 $28 = $2;
 HEAP32[$28>>2] = $27;
 $cptr = $cbuff;
 $29 = $aptr;
 $30 = (($29) + 1|0);
 $aptr = $30;
 while(1) {
  $31 = $aptr;
  $32 = HEAP8[$31>>0]|0;
  $33 = $32 << 24 >> 24;
  $34 = ($33|0)!=(0);
  if ($34) {
   $35 = $aptr;
   $36 = HEAP8[$35>>0]|0;
   $37 = $36 << 24 >> 24;
   $38 = ($37|0)!=(44);
   $91 = $38;
  } else {
   $91 = 0;
  }
  if (!($91)) {
   break;
  }
  $39 = $aptr;
  $40 = (($39) + 1|0);
  $aptr = $40;
  $41 = HEAP8[$39>>0]|0;
  $42 = $cptr;
  $43 = (($42) + 1|0);
  $cptr = $43;
  HEAP8[$42>>0] = $41;
 }
 $44 = $aptr;
 $45 = HEAP8[$44>>0]|0;
 $46 = $45 << 24 >> 24;
 $47 = ($46|0)==(0);
 if ($47) {
  $48 = $1;
  _print_usage($48);
  $49 = HEAP32[_stderr>>2]|0;
  (_fprintf(($49|0),(1120|0),($vararg_buffer1|0))|0);
  _exit(-1);
  // unreachable;
 }
 $50 = $cptr;
 HEAP8[$50>>0] = 0;
 $51 = (_atoi($cbuff)|0);
 $52 = $3;
 HEAP32[$52>>2] = $51;
 $cptr = $cbuff;
 $53 = $aptr;
 $54 = (($53) + 1|0);
 $aptr = $54;
 while(1) {
  $55 = $aptr;
  $56 = HEAP8[$55>>0]|0;
  $57 = $56 << 24 >> 24;
  $58 = ($57|0)!=(0);
  if ($58) {
   $59 = $aptr;
   $60 = HEAP8[$59>>0]|0;
   $61 = $60 << 24 >> 24;
   $62 = ($61|0)!=(44);
   $92 = $62;
  } else {
   $92 = 0;
  }
  if (!($92)) {
   break;
  }
  $63 = $aptr;
  $64 = (($63) + 1|0);
  $aptr = $64;
  $65 = HEAP8[$63>>0]|0;
  $66 = $cptr;
  $67 = (($66) + 1|0);
  $cptr = $67;
  HEAP8[$66>>0] = $65;
 }
 $68 = $cptr;
 HEAP8[$68>>0] = 0;
 $69 = (_atoi($cbuff)|0);
 $70 = $4;
 HEAP32[$70>>2] = $69;
 $71 = $aptr;
 $72 = HEAP8[$71>>0]|0;
 $73 = $72 << 24 >> 24;
 $74 = ($73|0)!=(0);
 if (!($74)) {
  $89 = $5;
  HEAP32[$89>>2] = -1;
  STACKTOP = sp;return;
 }
 $cptr = $cbuff;
 $75 = $aptr;
 $76 = (($75) + 1|0);
 $aptr = $76;
 while(1) {
  $77 = $aptr;
  $78 = HEAP8[$77>>0]|0;
  $79 = $78 << 24 >> 24;
  $80 = ($79|0)!=(0);
  if (!($80)) {
   break;
  }
  $81 = $aptr;
  $82 = (($81) + 1|0);
  $aptr = $82;
  $83 = HEAP8[$81>>0]|0;
  $84 = $cptr;
  $85 = (($84) + 1|0);
  $cptr = $85;
  HEAP8[$84>>0] = $83;
 }
 $86 = $cptr;
 HEAP8[$86>>0] = 0;
 $87 = (_atoi($cbuff)|0);
 $88 = $5;
 HEAP32[$88>>2] = $87;
 STACKTOP = sp;return;
}
function _ReadIheadRaster($file,$head,$data,$width,$height,$depth) {
 $file = $file|0;
 $head = $head|0;
 $data = $data|0;
 $width = $width|0;
 $height = $height|0;
 $depth = $depth|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $comp = 0, $complen = 0, $filesize = 0, $fp = 0, $ihead = 0, $indata = 0, $n = 0, $outbytes = 0, $outdata = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0;
 var $vararg_buffer18 = 0, $vararg_buffer23 = 0, $vararg_buffer26 = 0, $vararg_buffer29 = 0, $vararg_buffer32 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer32 = sp + 96|0;
 $vararg_buffer29 = sp + 88|0;
 $vararg_buffer26 = sp + 80|0;
 $vararg_buffer23 = sp + 72|0;
 $vararg_buffer18 = sp + 56|0;
 $vararg_buffer13 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $outbytes = sp + 124|0;
 $comp = sp + 120|0;
 $complen = sp + 112|0;
 $indata = sp + 104|0;
 $outdata = sp + 100|0;
 $0 = $file;
 $1 = $head;
 $2 = $data;
 $3 = $width;
 $4 = $height;
 $5 = $depth;
 $6 = $0;
 $7 = (_fopen(($6|0),(1144|0))|0);
 $fp = $7;
 $8 = $fp;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = $0;
  _syserr(1216,$10,1232);
 }
 $11 = $fp;
 $12 = (_readihdr($11)|0);
 $13 = $1;
 HEAP32[$13>>2] = $12;
 $14 = $1;
 $15 = HEAP32[$14>>2]|0;
 $ihead = $15;
 $16 = $1;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + 138|0);
 HEAP32[$vararg_buffer>>2] = $comp;
 $19 = (_sscanf($18,1248,$vararg_buffer)|0);
 $n = $19;
 $20 = $n;
 $21 = ($20|0)!=(1);
 if ($21) {
  $22 = $0;
  _fatalerr(1216,$22,1256);
 }
 $23 = $1;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + 122|0);
 $26 = $5;
 HEAP32[$vararg_buffer1>>2] = $26;
 $27 = (_sscanf($25,1248,$vararg_buffer1)|0);
 $n = $27;
 $28 = $n;
 $29 = ($28|0)!=(1);
 if ($29) {
  $30 = $0;
  _fatalerr(1216,$30,1288);
 }
 $31 = $1;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + 106|0);
 $34 = $3;
 HEAP32[$vararg_buffer4>>2] = $34;
 $35 = (_sscanf($33,1248,$vararg_buffer4)|0);
 $n = $35;
 $36 = $n;
 $37 = ($36|0)!=(1);
 if ($37) {
  $38 = $0;
  _fatalerr(1216,$38,1320);
 }
 $39 = $1;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + 114|0);
 $42 = $4;
 HEAP32[$vararg_buffer7>>2] = $42;
 $43 = (_sscanf($41,1248,$vararg_buffer7)|0);
 $n = $43;
 $44 = $n;
 $45 = ($44|0)!=(1);
 if ($45) {
  $46 = $0;
  _fatalerr(1216,$46,1352);
 }
 $47 = $1;
 $48 = HEAP32[$47>>2]|0;
 $49 = (($48) + 146|0);
 HEAP32[$vararg_buffer10>>2] = $complen;
 $50 = (_sscanf($49,1248,$vararg_buffer10)|0);
 $n = $50;
 $51 = $n;
 $52 = ($51|0)!=(1);
 if ($52) {
  $53 = $0;
  _fatalerr(1216,$53,1384);
 }
 $54 = $3;
 $55 = HEAP32[$54>>2]|0;
 $56 = $4;
 $57 = HEAP32[$56>>2]|0;
 $58 = $5;
 $59 = HEAP32[$58>>2]|0;
 $60 = (_SizeFromDepth($55,$57,$59)|0);
 $filesize = $60;
 $61 = $filesize;
 _malloc_uchar($outdata,$61,1152);
 $62 = HEAP32[$comp>>2]|0;
 $63 = ($62|0)==(0);
 do {
  if ($63) {
   $64 = HEAP32[$outdata>>2]|0;
   $65 = $filesize;
   $66 = $fp;
   $67 = (_fread(($64|0),1,($65|0),($66|0))|0);
   $n = $67;
   $68 = $n;
   $69 = $filesize;
   $70 = ($68|0)!=($69|0);
   if ($70) {
    $71 = HEAP32[_stderr>>2]|0;
    $72 = $0;
    $73 = $n;
    $74 = $filesize;
    HEAP32[$vararg_buffer13>>2] = $72;
    $vararg_ptr16 = (($vararg_buffer13) + 4|0);
    HEAP32[$vararg_ptr16>>2] = $73;
    $vararg_ptr17 = (($vararg_buffer13) + 8|0);
    HEAP32[$vararg_ptr17>>2] = $74;
    (_fprintf(($71|0),(1416|0),($vararg_buffer13|0))|0);
    _exit(1);
    // unreachable;
   } else {
    break;
   }
  } else {
   $75 = HEAP32[$complen>>2]|0;
   _malloc_uchar($indata,$75,1472);
   $76 = HEAP32[$indata>>2]|0;
   $77 = HEAP32[$complen>>2]|0;
   $78 = $fp;
   $79 = (_fread(($76|0),1,($77|0),($78|0))|0);
   $n = $79;
   $80 = $n;
   $81 = HEAP32[$complen>>2]|0;
   $82 = ($80|0)!=($81|0);
   if ($82) {
    $83 = HEAP32[_stderr>>2]|0;
    $84 = $0;
    $85 = $n;
    $86 = HEAP32[$complen>>2]|0;
    HEAP32[$vararg_buffer18>>2] = $84;
    $vararg_ptr21 = (($vararg_buffer18) + 4|0);
    HEAP32[$vararg_ptr21>>2] = $85;
    $vararg_ptr22 = (($vararg_buffer18) + 8|0);
    HEAP32[$vararg_ptr22>>2] = $86;
    (_fprintf(($83|0),(1416|0),($vararg_buffer18|0))|0);
    _exit(1);
    // unreachable;
   } else {
    break;
   }
  }
 } while(0);
 $87 = HEAP32[$comp>>2]|0;
 if ((($87|0) == 5)) {
  $88 = HEAP32[$indata>>2]|0;
  $89 = HEAP32[$complen>>2]|0;
  $90 = HEAP32[$outdata>>2]|0;
  $91 = $filesize;
  _rldecomp($88,$89,$90,$outbytes,$91);
  $92 = $1;
  $93 = HEAP32[$92>>2]|0;
  $94 = (($93) + 146|0);
  ;HEAP8[$94+0>>0]=0|0;HEAP8[$94+1>>0]=0|0;HEAP8[$94+2>>0]=0|0;HEAP8[$94+3>>0]=0|0;HEAP8[$94+4>>0]=0|0;HEAP8[$94+5>>0]=0|0;HEAP8[$94+6>>0]=0|0;HEAP8[$94+7>>0]=0|0;
  $95 = $1;
  $96 = HEAP32[$95>>2]|0;
  $97 = (($96) + 138|0);
  ;HEAP8[$97+0>>0]=0|0;HEAP8[$97+1>>0]=0|0;HEAP8[$97+2>>0]=0|0;HEAP8[$97+3>>0]=0|0;HEAP8[$97+4>>0]=0|0;HEAP8[$97+5>>0]=0|0;HEAP8[$97+6>>0]=0|0;HEAP8[$97+7>>0]=0|0;
  $98 = $1;
  $99 = HEAP32[$98>>2]|0;
  $100 = (($99) + 146|0);
  HEAP32[$vararg_buffer23>>2] = 0;
  (_sprintf($100,1248,$vararg_buffer23)|0);
  $101 = $1;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($102) + 138|0);
  HEAP32[$vararg_buffer26>>2] = 0;
  (_sprintf($103,1248,$vararg_buffer26)|0);
  $104 = HEAP32[$outdata>>2]|0;
  $105 = $2;
  HEAP32[$105>>2] = $104;
  $106 = HEAP32[$indata>>2]|0;
  _free($106);
  $146 = $fp;
  (_fclose(($146|0))|0);
  STACKTOP = sp;return;
 } else if ((($87|0) == 2)) {
  $107 = $1;
  $108 = HEAP32[$107>>2]|0;
  $109 = (($108) + 170|0);
  $110 = HEAP8[$109>>0]|0;
  $111 = $110 << 24 >> 24;
  $112 = ($111|0)==(49);
  if ($112) {
   $113 = HEAP32[$indata>>2]|0;
   $114 = HEAP32[$complen>>2]|0;
   _inv_bytes($113,$114);
   $115 = $1;
   $116 = HEAP32[$115>>2]|0;
   $117 = (($116) + 170|0);
   HEAP8[$117>>0] = 48;
   $118 = $1;
   $119 = HEAP32[$118>>2]|0;
   $120 = (($119) + 171|0);
   HEAP8[$120>>0] = 48;
  }
  $121 = HEAP32[$indata>>2]|0;
  $122 = HEAP32[$complen>>2]|0;
  $123 = $3;
  $124 = HEAP32[$123>>2]|0;
  $125 = $4;
  $126 = HEAP32[$125>>2]|0;
  $127 = HEAP32[$outdata>>2]|0;
  _grp4decomp($121,$122,$124,$126,$127,$outbytes);
  $128 = $1;
  $129 = HEAP32[$128>>2]|0;
  $130 = (($129) + 146|0);
  ;HEAP8[$130+0>>0]=0|0;HEAP8[$130+1>>0]=0|0;HEAP8[$130+2>>0]=0|0;HEAP8[$130+3>>0]=0|0;HEAP8[$130+4>>0]=0|0;HEAP8[$130+5>>0]=0|0;HEAP8[$130+6>>0]=0|0;HEAP8[$130+7>>0]=0|0;
  $131 = $1;
  $132 = HEAP32[$131>>2]|0;
  $133 = (($132) + 138|0);
  ;HEAP8[$133+0>>0]=0|0;HEAP8[$133+1>>0]=0|0;HEAP8[$133+2>>0]=0|0;HEAP8[$133+3>>0]=0|0;HEAP8[$133+4>>0]=0|0;HEAP8[$133+5>>0]=0|0;HEAP8[$133+6>>0]=0|0;HEAP8[$133+7>>0]=0|0;
  $134 = $1;
  $135 = HEAP32[$134>>2]|0;
  $136 = (($135) + 146|0);
  HEAP32[$vararg_buffer29>>2] = 0;
  (_sprintf($136,1248,$vararg_buffer29)|0);
  $137 = $1;
  $138 = HEAP32[$137>>2]|0;
  $139 = (($138) + 138|0);
  HEAP32[$vararg_buffer32>>2] = 0;
  (_sprintf($139,1248,$vararg_buffer32)|0);
  $140 = HEAP32[$outdata>>2]|0;
  $141 = $2;
  HEAP32[$141>>2] = $140;
  $142 = HEAP32[$indata>>2]|0;
  _free($142);
  $146 = $fp;
  (_fclose(($146|0))|0);
  STACKTOP = sp;return;
 } else if ((($87|0) == 0)) {
  $143 = HEAP32[$outdata>>2]|0;
  $144 = $2;
  HEAP32[$144>>2] = $143;
  $146 = $fp;
  (_fclose(($146|0))|0);
  STACKTOP = sp;return;
 } else {
  $145 = $0;
  _fatalerr(1216,$145,1184);
  $146 = $fp;
  (_fclose(($146|0))|0);
  STACKTOP = sp;return;
 }
}
function _rldecomp($indata,$inbytes,$outdata,$outbytes,$outsize) {
 $indata = $indata|0;
 $inbytes = $inbytes|0;
 $outdata = $outdata|0;
 $outbytes = $outbytes|0;
 $outsize = $outsize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ch = 0, $incount = 0, $outptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $outptr = sp + 4|0;
 $0 = $indata;
 $1 = $inbytes;
 $2 = $outdata;
 $3 = $outbytes;
 $4 = $outsize;
 $5 = $2;
 HEAP32[$outptr>>2] = $5;
 $6 = $3;
 HEAP32[$6>>2] = 0;
 $7 = $0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $ch = $9;
 $incount = 1;
 $10 = $ch;
 $11 = $10&255;
 $12 = $4;
 $13 = $3;
 _RLL_putc($outptr,$11,$12,$13);
 while(1) {
  $14 = $incount;
  $15 = $1;
  $16 = ($14|0)<($15|0);
  if (!($16)) {
   break;
  }
  $17 = $0;
  $18 = (($17) + 1|0);
  $0 = $18;
  $19 = $0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $ch = $21;
  $22 = $incount;
  $23 = (($22) + 1)|0;
  $incount = $23;
  $24 = $ch;
  $25 = $24&255;
  $26 = $4;
  $27 = $3;
  _RLL_putc($outptr,$25,$26,$27);
 }
 STACKTOP = sp;return;
}
function _RLL_putc($outptr,$code,$outsize,$out_count) {
 $outptr = $outptr|0;
 $code = $code|0;
 $outsize = $outsize|0;
 $out_count = $out_count|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $tmpbytes = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $outptr;
 $1 = $code;
 $2 = $outsize;
 $3 = $out_count;
 $4 = HEAP32[1504>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  $41 = $1;
  $42 = $41&255;
  $43 = ($42|0)==(144);
  if ($43) {
   HEAP32[1504>>2] = 1;
   STACKTOP = sp;return;
  }
  $44 = $3;
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 1)|0;
  $tmpbytes = $46;
  $47 = $2;
  $48 = ($46|0)>($47|0);
  if ($48) {
   $49 = HEAP32[_stderr>>2]|0;
   (_fprintf(($49|0),(1512|0),($vararg_buffer3|0))|0);
   _exit(-1);
   // unreachable;
  }
  $50 = $1;
  $51 = $0;
  $52 = HEAP32[$51>>2]|0;
  HEAP8[$52>>0] = $50;
  $53 = $0;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 1|0);
  HEAP32[$53>>2] = $55;
  $56 = $3;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + 1)|0;
  HEAP32[$56>>2] = $58;
  $59 = $1;
  $60 = $59&255;
  HEAP32[1552>>2] = $60;
  STACKTOP = sp;return;
 }
 HEAP32[1504>>2] = 0;
 $6 = $1;
 $7 = ($6<<24>>24)!=(0);
 do {
  if ($7) {
   while(1) {
    $8 = $1;
    $9 = (($8) + -1)<<24>>24;
    $1 = $9;
    $10 = ($9<<24>>24)!=(0);
    if (!($10)) {
     label = 8;
     break;
    }
    $11 = $3;
    $12 = HEAP32[$11>>2]|0;
    $13 = (($12) + 1)|0;
    $tmpbytes = $13;
    $14 = $2;
    $15 = ($13|0)>($14|0);
    if ($15) {
     label = 6;
     break;
    }
    $17 = HEAP32[1552>>2]|0;
    $18 = $17&255;
    $19 = $0;
    $20 = HEAP32[$19>>2]|0;
    HEAP8[$20>>0] = $18;
    $21 = $0;
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + 1|0);
    HEAP32[$21>>2] = $23;
    $24 = $3;
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + 1)|0;
    HEAP32[$24>>2] = $26;
   }
   if ((label|0) == 6) {
    $16 = HEAP32[_stderr>>2]|0;
    (_fprintf(($16|0),(1512|0),($vararg_buffer|0))|0);
    _exit(-1);
    // unreachable;
   }
   else if ((label|0) == 8) {
    break;
   }
  } else {
   $27 = $3;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($28) + 1)|0;
   $tmpbytes = $29;
   $30 = $2;
   $31 = ($29|0)>($30|0);
   if ($31) {
    $32 = HEAP32[_stderr>>2]|0;
    (_fprintf(($32|0),(1512|0),($vararg_buffer1|0))|0);
    _exit(-1);
    // unreachable;
   } else {
    $33 = $0;
    $34 = HEAP32[$33>>2]|0;
    HEAP8[$34>>0] = -112;
    $35 = $0;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($36) + 1|0);
    HEAP32[$35>>2] = $37;
    $38 = $3;
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + 1)|0;
    HEAP32[$38>>2] = $40;
    break;
   }
  }
 } while(0);
 STACKTOP = sp;return;
}
function _grp4decomp($indata,$inbytes,$width,$height,$outdata,$outbytes) {
 $indata = $indata|0;
 $inbytes = $inbytes|0;
 $width = $width|0;
 $height = $height|0;
 $outdata = $outdata|0;
 $outbytes = $outbytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $compressed = 0, $decompressed = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $compressed = sp + 16|0;
 $decompressed = sp;
 $0 = $indata;
 $1 = $inbytes;
 $2 = $width;
 $3 = $height;
 $4 = $outdata;
 $5 = $outbytes;
 $6 = $2;
 $7 = (($compressed) + 4|0);
 HEAP32[$7>>2] = $6;
 $8 = $3;
 $9 = (($compressed) + 8|0);
 HEAP32[$9>>2] = $8;
 $10 = $1;
 $11 = (($compressed) + 12|0);
 HEAP32[$11>>2] = $10;
 $12 = $0;
 HEAP32[$compressed>>2] = $12;
 HEAP32[1560>>2] = 0;
 HEAP32[1568>>2] = 1;
 HEAP32[1576>>2] = 1;
 _read_compressed_file_into_memory($compressed);
 $13 = $4;
 HEAP32[$decompressed>>2] = $13;
 _control_decompression($compressed,$decompressed);
 $14 = (($decompressed) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 >> 3;
 $17 = (($decompressed) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = Math_imul($16, $18)|0;
 $20 = $5;
 HEAP32[$20>>2] = $19;
 STACKTOP = sp;return;
}
function _read_compressed_file_into_memory($compressed) {
 $compressed = $compressed|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $compressed;
 $1 = HEAP32[1560>>2]|0;
 $2 = ($1|0)!=(0);
 if ($2) {
  $3 = $0;
  $4 = (($3) + 12|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = (_calloc($5,1)|0);
  $7 = $0;
  HEAP32[$7>>2] = $6;
  $8 = ($6|0)==(0|0);
  if ($8) {
   (_printf((1600|0),($vararg_buffer|0))|0);
   _exit(1);
   // unreachable;
  }
  STACKTOP = sp;return;
 } else {
  $9 = $0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)==(0|0);
  if ($11) {
   (_printf((1656|0),($vararg_buffer1|0))|0);
   _exit(1);
   // unreachable;
  }
  STACKTOP = sp;return;
 }
}
function _control_decompression($compressed,$decompressed) {
 $compressed = $compressed|0;
 $decompressed = $decompressed|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $params = 0, $sole_parameters = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $sole_parameters = sp + 8|0;
 $0 = $compressed;
 $1 = $decompressed;
 $params = $sole_parameters;
 $2 = $0;
 $3 = $1;
 $4 = $params;
 _prepare_to_decompress($2,$3,$4);
 while(1) {
  $5 = $params;
  $6 = (_decompress_line($5)|0);
  $7 = ($6|0)!=(6);
  if (!($7)) {
   break;
  }
  $8 = $params;
  _prepare_to_decompress_next_line($8);
 }
 $9 = $params;
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 _free($11);
 $12 = $params;
 $13 = (($12) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 _free($14);
 $15 = HEAP32[1584>>2]|0;
 _free($15);
 $16 = HEAP32[1592>>2]|0;
 _free($16);
 STACKTOP = sp;return;
}
function _prepare_to_decompress($compressed,$decompressed,$params) {
 $compressed = $compressed|0;
 $decompressed = $decompressed|0;
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = $compressed;
 $1 = $decompressed;
 $2 = $params;
 $3 = $0;
 $4 = (($3) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $2;
 $7 = (($6) + 4|0);
 HEAP32[$7>>2] = $5;
 $8 = $0;
 $9 = (($8) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $1;
 $12 = (($11) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = $0;
 $14 = (($13) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $1;
 $17 = (($16) + 8|0);
 HEAP32[$17>>2] = $15;
 $18 = $2;
 _set_up_first_line_d($18);
 $19 = $0;
 $20 = HEAP32[$19>>2]|0;
 _prepare_to_read_bits($20);
 $21 = HEAP32[1560>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = $0;
  $24 = (($23) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $0;
  $27 = (($26) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = Math_imul($25, $28)|0;
  $30 = (($29|0) / 8)&-1;
  $31 = (_calloc($30,1)|0);
  $32 = $1;
  HEAP32[$32>>2] = $31;
  $36 = $1;
  $37 = HEAP32[$36>>2]|0;
  $38 = $0;
  $39 = (($38) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40|0) / 8)&-1;
  _prepare_to_write_bits_d($37,$41);
  STACKTOP = sp;return;
 }
 $33 = $1;
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)==(0|0);
 if ($35) {
  (_printf((1696|0),($vararg_buffer|0))|0);
  _exit(1);
  // unreachable;
 }
 $36 = $1;
 $37 = HEAP32[$36>>2]|0;
 $38 = $0;
 $39 = (($38) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40|0) / 8)&-1;
 _prepare_to_write_bits_d($37,$41);
 STACKTOP = sp;return;
}
function _decompress_line($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = $params;
 HEAP32[1800>>2] = 1;
 HEAP32[((1808 + 4|0))>>2] = 0;
 HEAP32[1808>>2] = 0;
 L1: while(1) {
  $2 = (_get_mode()|0);
  $mode = $2;
  $3 = $mode;
  switch ($3|0) {
  case 6:  {
   label = 6;
   break L1;
   break;
  }
  case -3: case 3: case -2: case 2: case -1: case 1: case 0:  {
   $4 = $1;
   $5 = $mode;
   _vertical_mode_d($4,$5);
   label = 8;
   break;
  }
  case 5:  {
   $6 = $1;
   _horizontal_mode_d($6);
   label = 8;
   break;
  }
  case 4:  {
   $7 = $1;
   _pass_mode_d($7);
   break;
  }
  default: {
   _crash_d();
   label = 8;
  }
  }
  if ((label|0) == 8) {
   label = 0;
  }
  $8 = HEAP32[((1808 + 4|0))>>2]|0;
  $9 = $1;
  $10 = (($9) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   label = 10;
   break;
  }
 }
 if ((label|0) == 6) {
  $0 = 6;
  $13 = $0;
  STACKTOP = sp;return ($13|0);
 }
 else if ((label|0) == 10) {
  $0 = 0;
  $13 = $0;
  STACKTOP = sp;return ($13|0);
 }
 return 0|0;
}
function _prepare_to_decompress_next_line($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $0;
 _set_up_first_and_last_changing_elements_d($1);
 $2 = $0;
 _swap_the_reference_and_coding_lines($2);
 $3 = $0;
 HEAP32[$3>>2] = 0;
 STACKTOP = sp;return;
}
function _set_up_first_line_d($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $0;
 $2 = (($1) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 25)|0;
 $5 = $4<<2;
 $6 = (_malloc($5)|0);
 $7 = $0;
 $8 = (($7) + 8|0);
 HEAP32[$8>>2] = $6;
 $9 = $0;
 $10 = (($9) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 25)|0;
 $13 = $12<<2;
 $14 = (_malloc($13)|0);
 $15 = $0;
 $16 = (($15) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = $0;
 $18 = (($17) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$19>>2] = -1;
 $20 = $0;
 $21 = (($20) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = (($23) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 4|0);
 HEAP32[$26>>2] = $22;
 $27 = $0;
 $28 = (($27) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $0;
 $31 = (($30) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + 8|0);
 HEAP32[$33>>2] = $29;
 $34 = $0;
 $35 = (($34) + 4|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = $0;
 $38 = (($37) + 8|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = (($39) + 12|0);
 HEAP32[$40>>2] = $36;
 $41 = $0;
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$43>>2] = -1;
 $44 = $0;
 HEAP32[$44>>2] = 0;
 STACKTOP = sp;return;
}
function _prepare_to_read_bits($input_pointer) {
 $input_pointer = $input_pointer|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $input_pointer;
 $1 = $0;
 HEAP32[1872>>2] = $1;
 STACKTOP = sp;return;
}
function _prepare_to_write_bits_d($output_pointer,$bytes_per_line) {
 $output_pointer = $output_pointer|0;
 $bytes_per_line = $bytes_per_line|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $output_pointer;
 $1 = $bytes_per_line;
 $2 = $0;
 HEAP32[1832>>2] = $2;
 $3 = $1;
 $4 = (_calloc($3,1)|0);
 HEAP32[1584>>2] = $4;
 $5 = $1;
 $6 = (_calloc($5,1)|0);
 HEAP32[1592>>2] = $6;
 $7 = HEAP32[1592>>2]|0;
 $8 = $1;
 _memset(($7|0),-1,($8|0))|0;
 STACKTOP = sp;return;
}
function _set_up_first_and_last_changing_elements_d($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $0;
 $2 = (($1) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$3>>2] = -1;
 $4 = $0;
 $5 = (($4) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 1)|0;
 HEAP32[$10>>2] = $12;
 $13 = (($9) + ($12<<2)|0);
 HEAP32[$13>>2] = $6;
 $14 = $0;
 $15 = (($14) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = $0;
 $18 = (($17) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $0;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 1)|0;
 HEAP32[$20>>2] = $22;
 $23 = (($19) + ($22<<2)|0);
 HEAP32[$23>>2] = $16;
 $24 = $0;
 $25 = (($24) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $0;
 $28 = (($27) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $0;
 $31 = HEAP32[$30>>2]|0;
 $32 = (($31) + 1)|0;
 HEAP32[$30>>2] = $32;
 $33 = (($29) + ($32<<2)|0);
 HEAP32[$33>>2] = $26;
 STACKTOP = sp;return;
}
function _swap_the_reference_and_coding_lines($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $temp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $0;
 $2 = (($1) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $temp = $3;
 $4 = $0;
 $5 = (($4) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 8|0);
 HEAP32[$8>>2] = $6;
 $9 = $temp;
 $10 = $0;
 $11 = (($10) + 12|0);
 HEAP32[$11>>2] = $9;
 STACKTOP = sp;return;
}
function _crash_d() {
 var $0 = 0, $crash_program = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $crash_program = 0;
 $0 = $crash_program;
 (_fprintf(($0|0),(1744|0),($vararg_buffer|0))|0);
 STACKTOP = sp;return;
}
function _get_mode() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $1 = (_read_bit()|0);
 $2 = ($1|0)==(1);
 do {
  if ($2) {
   $0 = 0;
  } else {
   $3 = (_read_bit()|0);
   $4 = ($3|0)==(1);
   if ($4) {
    $5 = (_read_bit()|0);
    $6 = ($5|0)==(1);
    if ($6) {
     $0 = 1;
     break;
    } else {
     $0 = -1;
     break;
    }
   }
   $7 = (_read_bit()|0);
   $8 = ($7|0)==(1);
   if ($8) {
    $0 = 5;
    break;
   }
   $9 = (_read_bit()|0);
   $10 = ($9|0)==(1);
   if ($10) {
    $0 = 4;
    break;
   }
   $11 = (_read_bit()|0);
   $12 = ($11|0)==(1);
   if ($12) {
    $13 = (_read_bit()|0);
    $14 = ($13|0)==(1);
    if ($14) {
     $0 = 2;
     break;
    } else {
     $0 = -2;
     break;
    }
   }
   $15 = (_read_bit()|0);
   $16 = ($15|0)==(1);
   if ($16) {
    $17 = (_read_bit()|0);
    $18 = ($17|0)==(1);
    if ($18) {
     $0 = 3;
     break;
    } else {
     $0 = -3;
     break;
    }
   }
   $i = 0;
   while(1) {
    $19 = $i;
    $20 = ($19|0)<(5);
    if (!($20)) {
     break;
    }
    $21 = (_read_bit()|0);
    $22 = ($21|0)!=(0);
    if ($22) {
     _crash_d();
    }
    $23 = $i;
    $24 = (($23) + 1)|0;
    $i = $24;
   }
   $25 = (_read_bit()|0);
   $26 = ($25|0)!=(1);
   if ($26) {
    _crash_d();
   }
   $i = 0;
   while(1) {
    $27 = $i;
    $28 = ($27|0)<(11);
    if (!($28)) {
     break;
    }
    $29 = (_read_bit()|0);
    $30 = ($29|0)!=(0);
    if ($30) {
     _crash_d();
    }
    $31 = $i;
    $32 = (($31) + 1)|0;
    $i = $32;
   }
   $33 = (_read_bit()|0);
   $34 = ($33|0)!=(1);
   if ($34) {
    _crash_d();
   }
   $0 = 6;
  }
 } while(0);
 $35 = $0;
 STACKTOP = sp;return ($35|0);
}
function _vertical_mode_d($params,$offset) {
 $params = $params|0;
 $offset = $offset|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $run_length = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = $offset;
 $2 = $0;
 $3 = (($2) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[1800>>2]|0;
 $6 = (($4) + ($5<<2)|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $1;
 $9 = (($7) + ($8))|0;
 HEAP32[((1824 + 4|0))>>2] = $9;
 $10 = HEAP32[((1824 + 4|0))>>2]|0;
 $11 = HEAP32[((1808 + 4|0))>>2]|0;
 $12 = (($10) - ($11))|0;
 $run_length = $12;
 $13 = $run_length;
 $14 = HEAP32[1808>>2]|0;
 _write_bits_d($13,$14);
 $15 = HEAP32[((1824 + 4|0))>>2]|0;
 HEAP32[((1808 + 4|0))>>2] = $15;
 $16 = HEAP32[1808>>2]|0;
 $17 = ($16|0)!=(0);
 $18 = $17 ^ 1;
 $19 = $18&1;
 HEAP32[1808>>2] = $19;
 $20 = HEAP32[((1808 + 4|0))>>2]|0;
 $21 = $0;
 $22 = (($21) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $0;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$24>>2] = $26;
 $27 = (($23) + ($26<<2)|0);
 HEAP32[$27>>2] = $20;
 $28 = $1;
 $29 = ($28|0)==(-1);
 if (!($29)) {
  $30 = $1;
  $31 = ($30|0)==(0);
  if (!($31)) {
   $44 = $1;
   $45 = ($44|0)==(1);
   if (!($45)) {
    $46 = $1;
    $47 = ($46|0)==(2);
    if (!($47)) {
     $70 = $1;
     $71 = ($70|0)==(-2);
     if ($71) {
      label = 14;
     } else {
      $72 = $1;
      $73 = ($72|0)==(-3);
      if ($73) {
       label = 14;
      }
     }
     do {
      if ((label|0) == 14) {
       $74 = $0;
       $75 = (($74) + 8|0);
       $76 = HEAP32[$75>>2]|0;
       $77 = HEAP32[1800>>2]|0;
       $78 = (($76) + ($77<<2)|0);
       $79 = (($78) + -4|0);
       $80 = HEAP32[$79>>2]|0;
       $81 = HEAP32[((1808 + 4|0))>>2]|0;
       $82 = ($80|0)>($81|0);
       if ($82) {
        $83 = HEAP32[1800>>2]|0;
        $84 = (($83) + -1)|0;
        HEAP32[1800>>2] = $84;
        break;
       }
       $85 = $0;
       $86 = (($85) + 8|0);
       $87 = HEAP32[$86>>2]|0;
       $88 = HEAP32[1800>>2]|0;
       $89 = (($87) + ($88<<2)|0);
       $90 = HEAP32[$89>>2]|0;
       $91 = $0;
       $92 = (($91) + 4|0);
       $93 = HEAP32[$92>>2]|0;
       $94 = ($90|0)!=($93|0);
       if ($94) {
        $95 = HEAP32[1800>>2]|0;
        $96 = (($95) + 1)|0;
        HEAP32[1800>>2] = $96;
       }
       STACKTOP = sp;return;
      }
     } while(0);
     $97 = $1;
     $98 = ($97|0)==(3);
     if (!($98)) {
      STACKTOP = sp;return;
     }
     $99 = HEAP32[1800>>2]|0;
     $100 = (($99) + 1)|0;
     HEAP32[1800>>2] = $100;
     while(1) {
      $101 = $0;
      $102 = (($101) + 8|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = HEAP32[1800>>2]|0;
      $105 = (($103) + ($104<<2)|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = HEAP32[((1808 + 4|0))>>2]|0;
      $108 = ($106|0)<=($107|0);
      if ($108) {
       $109 = $0;
       $110 = (($109) + 8|0);
       $111 = HEAP32[$110>>2]|0;
       $112 = HEAP32[1800>>2]|0;
       $113 = (($111) + ($112<<2)|0);
       $114 = HEAP32[$113>>2]|0;
       $115 = $0;
       $116 = (($115) + 4|0);
       $117 = HEAP32[$116>>2]|0;
       $118 = ($114|0)!=($117|0);
       $121 = $118;
      } else {
       $121 = 0;
      }
      if (!($121)) {
       break;
      }
      $119 = HEAP32[1800>>2]|0;
      $120 = (($119) + 2)|0;
      HEAP32[1800>>2] = $120;
     }
     STACKTOP = sp;return;
    }
   }
   $48 = HEAP32[1800>>2]|0;
   $49 = (($48) + 1)|0;
   HEAP32[1800>>2] = $49;
   $50 = $0;
   $51 = (($50) + 8|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = HEAP32[1800>>2]|0;
   $54 = (($52) + ($53<<2)|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = HEAP32[((1808 + 4|0))>>2]|0;
   $57 = ($55|0)<=($56|0);
   if ($57) {
    $58 = $0;
    $59 = (($58) + 8|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[1800>>2]|0;
    $62 = (($60) + ($61<<2)|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = $0;
    $65 = (($64) + 4|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = ($63|0)!=($66|0);
    if ($67) {
     $68 = HEAP32[1800>>2]|0;
     $69 = (($68) + 2)|0;
     HEAP32[1800>>2] = $69;
    }
   }
   STACKTOP = sp;return;
  }
 }
 $32 = $0;
 $33 = (($32) + 8|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[1800>>2]|0;
 $36 = (($34) + ($35<<2)|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $0;
 $39 = (($38) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = ($37|0)!=($40|0);
 if ($41) {
  $42 = HEAP32[1800>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[1800>>2] = $43;
 }
 STACKTOP = sp;return;
}
function _horizontal_mode_d($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $length = 0, $total_length = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $total_length = 0;
 while(1) {
  $1 = HEAP32[1808>>2]|0;
  $2 = (_find_run_length_code($1)|0);
  $length = $2;
  $3 = $length;
  $4 = $total_length;
  $5 = (($4) + ($3))|0;
  $total_length = $5;
  $6 = $length;
  $7 = ($6|0)>(63);
  if (!($7)) {
   break;
  }
 }
 $8 = $total_length;
 $9 = HEAP32[1808>>2]|0;
 _write_bits_d($8,$9);
 $10 = $total_length;
 $11 = HEAP32[((1808 + 4|0))>>2]|0;
 $12 = (($11) + ($10))|0;
 HEAP32[((1808 + 4|0))>>2] = $12;
 $13 = HEAP32[1808>>2]|0;
 $14 = ($13|0)!=(0);
 $15 = $14 ^ 1;
 $16 = $15&1;
 HEAP32[1808>>2] = $16;
 $17 = HEAP32[((1808 + 4|0))>>2]|0;
 $18 = $0;
 $19 = (($18) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $0;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + 1)|0;
 HEAP32[$21>>2] = $23;
 $24 = (($20) + ($23<<2)|0);
 HEAP32[$24>>2] = $17;
 $total_length = 0;
 while(1) {
  $25 = HEAP32[1808>>2]|0;
  $26 = (_find_run_length_code($25)|0);
  $length = $26;
  $27 = $length;
  $28 = $total_length;
  $29 = (($28) + ($27))|0;
  $total_length = $29;
  $30 = $length;
  $31 = ($30|0)>(63);
  if (!($31)) {
   break;
  }
 }
 $32 = $total_length;
 $33 = HEAP32[1808>>2]|0;
 _write_bits_d($32,$33);
 $34 = $total_length;
 $35 = HEAP32[((1808 + 4|0))>>2]|0;
 $36 = (($35) + ($34))|0;
 HEAP32[((1808 + 4|0))>>2] = $36;
 $37 = HEAP32[1808>>2]|0;
 $38 = ($37|0)!=(0);
 $39 = $38 ^ 1;
 $40 = $39&1;
 HEAP32[1808>>2] = $40;
 $41 = HEAP32[((1808 + 4|0))>>2]|0;
 $42 = $0;
 $43 = (($42) + 12|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = $0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + 1)|0;
 HEAP32[$45>>2] = $47;
 $48 = (($44) + ($47<<2)|0);
 HEAP32[$48>>2] = $41;
 while(1) {
  $49 = $0;
  $50 = (($49) + 8|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[1800>>2]|0;
  $53 = (($51) + ($52<<2)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[((1808 + 4|0))>>2]|0;
  $56 = ($54|0)<=($55|0);
  if ($56) {
   $57 = $0;
   $58 = (($57) + 8|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[1800>>2]|0;
   $61 = (($59) + ($60<<2)|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = $0;
   $64 = (($63) + 4|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = ($62|0)<($65|0);
   $69 = $66;
  } else {
   $69 = 0;
  }
  if (!($69)) {
   break;
  }
  $67 = HEAP32[1800>>2]|0;
  $68 = (($67) + 2)|0;
  HEAP32[1800>>2] = $68;
 }
 STACKTOP = sp;return;
}
function _pass_mode_d($params) {
 $params = $params|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $run_length = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $params;
 $1 = HEAP32[1800>>2]|0;
 $2 = (($1) + 1)|0;
 HEAP32[1816>>2] = $2;
 $3 = $0;
 $4 = (($3) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[1816>>2]|0;
 $7 = (($5) + ($6<<2)|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[((1808 + 4|0))>>2]|0;
 $10 = (($8) - ($9))|0;
 $run_length = $10;
 $11 = $run_length;
 $12 = HEAP32[1808>>2]|0;
 _write_bits_d($11,$12);
 $13 = $run_length;
 $14 = HEAP32[((1808 + 4|0))>>2]|0;
 $15 = (($14) + ($13))|0;
 HEAP32[((1808 + 4|0))>>2] = $15;
 $16 = HEAP32[1800>>2]|0;
 $17 = (($16) + 2)|0;
 HEAP32[1800>>2] = $17;
 STACKTOP = sp;return;
}
function _read_bit() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = HEAP32[1576>>2]|0;
 $1 = ($0|0)!=(0);
 if ($1) {
  HEAP32[1880>>2] = 0;
  HEAP32[1888>>2] = 0;
  HEAP32[1576>>2] = 0;
 }
 $2 = HEAP32[1872>>2]|0;
 $3 = HEAP32[1888>>2]|0;
 $4 = (($2) + ($3)|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = HEAP32[1880>>2]|0;
 $8 = (1896 + ($7)|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 $11 = $6 & $10;
 $bit = $11;
 $12 = HEAP32[1880>>2]|0;
 $13 = ($12|0)==(7);
 if ($13) {
  HEAP32[1880>>2] = 0;
  $14 = HEAP32[1888>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[1888>>2] = $15;
  $18 = $bit;
  $19 = ($18|0)==(0);
  $20 = $19 ^ 1;
  $21 = $20&1;
  STACKTOP = sp;return ($21|0);
 } else {
  $16 = HEAP32[1880>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[1880>>2] = $17;
  $18 = $bit;
  $19 = ($18|0)==(0);
  $20 = $19 ^ 1;
  $21 = $20&1;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _write_bits_d($length,$color) {
 $length = $length|0;
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bytes = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $length;
 $1 = $color;
 $2 = HEAP32[1568>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[1840>>2] = 0;
  HEAP32[1848>>2] = 0;
  HEAP32[1568>>2] = 0;
 }
 $4 = $1;
 $5 = ($4|0)==(1);
 if ($5) {
  while(1) {
   $6 = $0;
   $7 = ($6>>>0)>(0);
   if ($7) {
    $8 = HEAP32[1840>>2]|0;
    $9 = ($8|0)!=(0);
    $116 = $9;
   } else {
    $116 = 0;
   }
   if (!($116)) {
    break;
   }
   $10 = HEAP32[1840>>2]|0;
   $11 = (1856 + ($10)|0);
   $12 = HEAP8[$11>>0]|0;
   $13 = $12 << 24 >> 24;
   $14 = HEAP32[1832>>2]|0;
   $15 = HEAP32[1848>>2]|0;
   $16 = (($14) + ($15)|0);
   $17 = HEAP8[$16>>0]|0;
   $18 = $17 << 24 >> 24;
   $19 = $18 | $13;
   $20 = $19&255;
   HEAP8[$16>>0] = $20;
   $21 = $0;
   $22 = (($21) + -1)|0;
   $0 = $22;
   $23 = HEAP32[1840>>2]|0;
   $24 = ($23|0)==(7);
   if ($24) {
    HEAP32[1840>>2] = 0;
    $25 = HEAP32[1848>>2]|0;
    $26 = (($25) + 1)|0;
    HEAP32[1848>>2] = $26;
   } else {
    $27 = HEAP32[1840>>2]|0;
    $28 = (($27) + 1)|0;
    HEAP32[1840>>2] = $28;
   }
  }
  $29 = HEAP32[1832>>2]|0;
  $30 = HEAP32[1848>>2]|0;
  $31 = (($29) + ($30)|0);
  $32 = HEAP32[1592>>2]|0;
  $33 = $0;
  $34 = (($33>>>0) / 8)&-1;
  $bytes = $34;
  _memcpy(($31|0),($32|0),($34|0))|0;
  $35 = $bytes;
  $36 = HEAP32[1848>>2]|0;
  $37 = (($36) + ($35))|0;
  HEAP32[1848>>2] = $37;
  $38 = $0;
  $39 = (($38>>>0) % 8)&-1;
  $0 = $39;
  while(1) {
   $40 = $0;
   $41 = ($40>>>0)>(0);
   if (!($41)) {
    break;
   }
   $42 = HEAP32[1840>>2]|0;
   $43 = (1856 + ($42)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44 << 24 >> 24;
   $46 = HEAP32[1832>>2]|0;
   $47 = HEAP32[1848>>2]|0;
   $48 = (($46) + ($47)|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49 << 24 >> 24;
   $51 = $50 | $45;
   $52 = $51&255;
   HEAP8[$48>>0] = $52;
   $53 = $0;
   $54 = (($53) + -1)|0;
   $0 = $54;
   $55 = HEAP32[1840>>2]|0;
   $56 = ($55|0)==(7);
   if ($56) {
    HEAP32[1840>>2] = 0;
    $57 = HEAP32[1848>>2]|0;
    $58 = (($57) + 1)|0;
    HEAP32[1848>>2] = $58;
   } else {
    $59 = HEAP32[1840>>2]|0;
    $60 = (($59) + 1)|0;
    HEAP32[1840>>2] = $60;
   }
  }
  STACKTOP = sp;return;
 } else {
  while(1) {
   $61 = $0;
   $62 = ($61>>>0)>(0);
   if ($62) {
    $63 = HEAP32[1840>>2]|0;
    $64 = ($63|0)!=(0);
    $117 = $64;
   } else {
    $117 = 0;
   }
   if (!($117)) {
    break;
   }
   $65 = HEAP32[1840>>2]|0;
   $66 = (1864 + ($65)|0);
   $67 = HEAP8[$66>>0]|0;
   $68 = $67 << 24 >> 24;
   $69 = HEAP32[1832>>2]|0;
   $70 = HEAP32[1848>>2]|0;
   $71 = (($69) + ($70)|0);
   $72 = HEAP8[$71>>0]|0;
   $73 = $72 << 24 >> 24;
   $74 = $73 & $68;
   $75 = $74&255;
   HEAP8[$71>>0] = $75;
   $76 = $0;
   $77 = (($76) + -1)|0;
   $0 = $77;
   $78 = HEAP32[1840>>2]|0;
   $79 = ($78|0)==(7);
   if ($79) {
    HEAP32[1840>>2] = 0;
    $80 = HEAP32[1848>>2]|0;
    $81 = (($80) + 1)|0;
    HEAP32[1848>>2] = $81;
   } else {
    $82 = HEAP32[1840>>2]|0;
    $83 = (($82) + 1)|0;
    HEAP32[1840>>2] = $83;
   }
  }
  $84 = HEAP32[1832>>2]|0;
  $85 = HEAP32[1848>>2]|0;
  $86 = (($84) + ($85)|0);
  $87 = HEAP32[1584>>2]|0;
  $88 = $0;
  $89 = (($88>>>0) / 8)&-1;
  $bytes = $89;
  _memcpy(($86|0),($87|0),($89|0))|0;
  $90 = $bytes;
  $91 = HEAP32[1848>>2]|0;
  $92 = (($91) + ($90))|0;
  HEAP32[1848>>2] = $92;
  $93 = $0;
  $94 = (($93>>>0) % 8)&-1;
  $0 = $94;
  while(1) {
   $95 = $0;
   $96 = ($95>>>0)>(0);
   if (!($96)) {
    break;
   }
   $97 = HEAP32[1840>>2]|0;
   $98 = (1864 + ($97)|0);
   $99 = HEAP8[$98>>0]|0;
   $100 = $99 << 24 >> 24;
   $101 = HEAP32[1832>>2]|0;
   $102 = HEAP32[1848>>2]|0;
   $103 = (($101) + ($102)|0);
   $104 = HEAP8[$103>>0]|0;
   $105 = $104 << 24 >> 24;
   $106 = $105 & $100;
   $107 = $106&255;
   HEAP8[$103>>0] = $107;
   $108 = $0;
   $109 = (($108) + -1)|0;
   $0 = $109;
   $110 = HEAP32[1840>>2]|0;
   $111 = ($110|0)==(7);
   if ($111) {
    HEAP32[1840>>2] = 0;
    $112 = HEAP32[1848>>2]|0;
    $113 = (($112) + 1)|0;
    HEAP32[1848>>2] = $113;
   } else {
    $114 = HEAP32[1840>>2]|0;
    $115 = (($114) + 1)|0;
    HEAP32[1840>>2] = $115;
   }
  }
  STACKTOP = sp;return;
 }
}
function _find_run_length_code($color) {
 $color = $color|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $color;
 $1 = $0;
 $2 = ($1|0)==(0);
 if ($2) {
  HEAP32[6896>>2] = 4400;
 } else {
  HEAP32[6896>>2] = 1904;
 }
 while(1) {
  $3 = HEAP32[6896>>2]|0;
  $4 = HEAP32[$3>>2]|0;
  $5 = ($4|0)==(-1);
  if (!($5)) {
   break;
  }
  $6 = (_read_bit()|0);
  $7 = ($6|0)==(0);
  if ($7) {
   $8 = HEAP32[6896>>2]|0;
   $9 = (($8) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   HEAP32[6896>>2] = $10;
  } else {
   $11 = HEAP32[6896>>2]|0;
   $12 = (($11) + 8|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[6896>>2] = $13;
  }
 }
 $14 = HEAP32[6896>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _SizeFromDepth($pixwidth,$pixheight,$depth) {
 $pixwidth = $pixwidth|0;
 $pixheight = $pixheight|0;
 $depth = $depth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0.0, $13 = 0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $filesize = 0, $pixperbyte = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $1 = $pixwidth;
 $2 = $pixheight;
 $3 = $depth;
 $4 = $3;
 $5 = ($4|0)==(24);
 if ($5) {
  $6 = $1;
  $7 = $2;
  $8 = Math_imul($6, $7)|0;
  $9 = ($8*3)|0;
  $filesize = $9;
  $10 = $filesize;
  $0 = $10;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 } else {
  $11 = $3;
  $12 = (+_PixPerByte($11));
  $pixperbyte = $12;
  $13 = $1;
  $14 = (+($13|0));
  $15 = $pixperbyte;
  $16 = $14 / $15;
  $17 = $16;
  $18 = (+Math_ceil((+$17)));
  $19 = (~~(($18)));
  $20 = $2;
  $21 = Math_imul($19, $20)|0;
  $filesize = $21;
  $22 = $filesize;
  $0 = $22;
  $23 = $0;
  STACKTOP = sp;return ($23|0);
 }
 return 0|0;
}
function _PixPerByte($depth) {
 $depth = $depth|0;
 var $0 = 0, $1 = 0, $2 = 0.0, $pixperbyte = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $depth;
 $pixperbyte = 0.0;
 $1 = $0;
 switch ($1|0) {
 case 16:  {
  $pixperbyte = 0.5;
  break;
 }
 case 8:  {
  $pixperbyte = 1.0;
  break;
 }
 case 1:  {
  $pixperbyte = 8.0;
  break;
 }
 case 2:  {
  $pixperbyte = 4.0;
  break;
 }
 case 64:  {
  $pixperbyte = 0.125;
  break;
 }
 case 4:  {
  $pixperbyte = 2.0;
  break;
 }
 case 32:  {
  $pixperbyte = 0.25;
  break;
 }
 default: {
  _fatalerr(6904,6920,0);
 }
 }
 $2 = $pixperbyte;
 STACKTOP = sp;return (+$2);
}
function _get_density($head) {
 $head = $head|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $density = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $density = sp + 4|0;
 $0 = $head;
 $1 = $0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  _fatalerr(7008,6976,0);
 }
 $3 = $0;
 $4 = (($3) + 130|0);
 HEAP32[$vararg_buffer>>2] = $density;
 $5 = (_sscanf($4,7000,$vararg_buffer)|0);
 $6 = ($5|0)!=(1);
 if (!($6)) {
  $7 = HEAP32[$density>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 HEAP32[$density>>2] = -1;
 $7 = HEAP32[$density>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _readihdr($fp) {
 $fp = $fp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $head = 0, $len = 0, $lenstr = 0, $n = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_ptr1 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $lenstr = sp + 40|0;
 $len = sp + 24|0;
 $0 = $fp;
 $1 = $0;
 $2 = (_fread(($lenstr|0),1,8,($1|0))|0);
 $n = $2;
 $3 = $n;
 $4 = ($3|0)!=(8);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  $6 = $n;
  HEAP32[$vararg_buffer>>2] = $6;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 8;
  (_fprintf(($5|0),(7024|0),($vararg_buffer|0))|0);
  _exit(1);
  // unreachable;
 }
 HEAP32[$vararg_buffer2>>2] = $len;
 $7 = (_sscanf($lenstr,7072,$vararg_buffer2)|0);
 $8 = ($7|0)!=(1);
 if ($8) {
  _fatalerr(7080,7096,0);
 }
 $9 = HEAP32[$len>>2]|0;
 $10 = ($9|0)!=(288);
 if ($10) {
  _fatalerr(7080,7128,0);
 }
 $11 = (_malloc(288)|0);
 $head = $11;
 $12 = $head;
 $13 = ($12|0)==(0|0);
 if ($13) {
  _syserr(7080,7184,7192);
 }
 $14 = $head;
 $15 = HEAP32[$len>>2]|0;
 $16 = $0;
 $17 = (_fread(($14|0),1,($15|0),($16|0))|0);
 $n = $17;
 $18 = $n;
 $19 = HEAP32[$len>>2]|0;
 $20 = ($18|0)!=($19|0);
 if ($20) {
  $21 = HEAP32[_stderr>>2]|0;
  $22 = $n;
  $23 = HEAP32[$len>>2]|0;
  HEAP32[$vararg_buffer5>>2] = $22;
  $vararg_ptr8 = (($vararg_buffer5) + 4|0);
  HEAP32[$vararg_ptr8>>2] = $23;
  (_fprintf(($21|0),(7024|0),($vararg_buffer5|0))|0);
  _exit(1);
  // unreachable;
 } else {
  $24 = $head;
  STACKTOP = sp;return ($24|0);
 }
 return 0|0;
}
function _wsq_encode_mem($odata,$olen,$r_bitrate,$idata,$w,$h,$d,$ppi,$comment_text) {
 $odata = $odata|0;
 $olen = $olen|0;
 $r_bitrate = +$r_bitrate;
 $idata = $idata|0;
 $w = $w|0;
 $h = $h|0;
 $d = $d|0;
 $ppi = $ppi|0;
 $comment_text = $comment_text|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0.0, $113 = 0.0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0.0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0.0, $313 = 0.0;
 var $314 = 0, $315 = 0, $316 = 0.0, $317 = 0, $318 = 0.0, $319 = 0.0, $32 = 0, $320 = 0.0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0.0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0.0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $block_sizes = 0, $fdata = 0, $hsize = 0, $hsize1 = 0, $hsize2 = 0, $hsize3 = 0, $huff_buf = 0, $huffbits = 0, $hufftable = 0, $huffvalues = 0, $m_shift = 0, $num_pix = 0, $qdata = 0, $qsize = 0;
 var $qsize1 = 0, $qsize2 = 0, $qsize3 = 0, $r_scale = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer13 = 0, $vararg_buffer15 = 0, $vararg_buffer17 = 0, $vararg_buffer19 = 0, $vararg_buffer21 = 0, $vararg_buffer23 = 0, $vararg_buffer25 = 0, $vararg_buffer27 = 0, $vararg_buffer29 = 0, $vararg_buffer3 = 0, $vararg_buffer33 = 0, $vararg_buffer5 = 0;
 var $vararg_buffer7 = 0, $vararg_buffer9 = 0, $vararg_ptr31 = 0, $vararg_ptr32 = 0, $vararg_ptr36 = 0, $vararg_ptr37 = 0, $wsq_alloc = 0, $wsq_data = 0, $wsq_len = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0;
 $vararg_buffer33 = sp + 136|0;
 $vararg_buffer29 = sp + 120|0;
 $vararg_buffer27 = sp + 112|0;
 $vararg_buffer25 = sp + 104|0;
 $vararg_buffer23 = sp + 96|0;
 $vararg_buffer21 = sp + 88|0;
 $vararg_buffer19 = sp + 80|0;
 $vararg_buffer17 = sp + 72|0;
 $vararg_buffer15 = sp + 64|0;
 $vararg_buffer13 = sp + 56|0;
 $vararg_buffer11 = sp + 48|0;
 $vararg_buffer9 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $m_shift = sp + 236|0;
 $r_scale = sp + 232|0;
 $qdata = sp + 228|0;
 $qsize = sp + 224|0;
 $qsize1 = sp + 220|0;
 $qsize2 = sp + 216|0;
 $qsize3 = sp + 212|0;
 $huffbits = sp + 208|0;
 $huffvalues = sp + 204|0;
 $hufftable = sp + 200|0;
 $hsize1 = sp + 188|0;
 $hsize2 = sp + 184|0;
 $hsize3 = sp + 180|0;
 $wsq_len = sp + 168|0;
 $block_sizes = sp + 160|0;
 $1 = $odata;
 $2 = $olen;
 $3 = $r_bitrate;
 $4 = $idata;
 $5 = $w;
 $6 = $h;
 $7 = $d;
 $8 = $ppi;
 $9 = $comment_text;
 $10 = $5;
 $11 = $6;
 $12 = Math_imul($10, $11)|0;
 $num_pix = $12;
 $13 = $num_pix;
 $14 = $13<<2;
 $15 = (_malloc($14)|0);
 $fdata = $15;
 $16 = ($15|0)==(0|0);
 if ($16) {
  $17 = HEAP32[_stderr>>2]|0;
  (_fprintf(($17|0),(7200|0),($vararg_buffer|0))|0);
  $0 = -10;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $18 = $fdata;
 $19 = $4;
 $20 = $num_pix;
 $21 = (_conv_img_2_flt_ret($18,$m_shift,$r_scale,$19,$20)|0);
 $ret = $21;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = $fdata;
  _free($23);
  $24 = $ret;
  $0 = $24;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $25 = HEAP32[8>>2]|0;
 $26 = ($25|0)>(0);
 if ($26) {
  $27 = HEAP32[_stderr>>2]|0;
  (_fprintf(($27|0),(7240|0),($vararg_buffer1|0))|0);
 }
 $28 = $5;
 $29 = $6;
 _build_wsq_trees(9280,20,9760,64,$28,$29);
 $30 = HEAP32[8>>2]|0;
 $31 = ($30|0)>(0);
 if ($31) {
  $32 = HEAP32[_stderr>>2]|0;
  (_fprintf(($32|0),(7296|0),($vararg_buffer3|0))|0);
 }
 $33 = $fdata;
 $34 = $5;
 $35 = $6;
 $36 = (_wsq_decompose($33,$34,$35,9280,20,10272,7,10304,9)|0);
 $ret = $36;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = $fdata;
  _free($38);
  $39 = $ret;
  $0 = $39;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $40 = HEAP32[8>>2]|0;
 $41 = ($40|0)>(0);
 if ($41) {
  $42 = HEAP32[_stderr>>2]|0;
  (_fprintf(($42|0),(7344|0),($vararg_buffer5|0))|0);
 }
 HEAPF32[((8240 + 4|0))>>2] = 0.0;
 HEAPF32[8240>>2] = 0.0;
 $43 = $3;
 HEAPF32[((8240 + 8|0))>>2] = $43;
 $44 = $fdata;
 $45 = $5;
 $46 = $6;
 _variance(8240,9760,64,$44,$45,$46);
 $47 = HEAP32[8>>2]|0;
 $48 = ($47|0)>(0);
 if ($48) {
  $49 = HEAP32[_stderr>>2]|0;
  (_fprintf(($49|0),(7384|0),($vararg_buffer7|0))|0);
 }
 $50 = $fdata;
 $51 = $5;
 $52 = $6;
 $53 = (_quantize($qdata,$qsize,8240,9760,64,$50,$51,$52)|0);
 $ret = $53;
 $54 = ($53|0)!=(0);
 if ($54) {
  $55 = $fdata;
  _free($55);
  $56 = $ret;
  $0 = $56;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $57 = $fdata;
 _free($57);
 $58 = HEAP32[8>>2]|0;
 $59 = ($58|0)>(0);
 if ($59) {
  $60 = HEAP32[_stderr>>2]|0;
  (_fprintf(($60|0),(7416|0),($vararg_buffer9|0))|0);
 }
 _quant_block_sizes($qsize1,$qsize2,$qsize3,8240,9280,20,9760,64);
 $61 = HEAP32[$qsize>>2]|0;
 $62 = HEAP32[$qsize1>>2]|0;
 $63 = HEAP32[$qsize2>>2]|0;
 $64 = (($62) + ($63))|0;
 $65 = HEAP32[$qsize3>>2]|0;
 $66 = (($64) + ($65))|0;
 $67 = ($61|0)!=($66|0);
 if ($67) {
  $68 = HEAP32[_stderr>>2]|0;
  (_fprintf(($68|0),(7464|0),($vararg_buffer11|0))|0);
  $0 = -11;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $69 = $num_pix;
 $70 = (_malloc($69)|0);
 $wsq_data = $70;
 $71 = $wsq_data;
 $72 = ($71|0)==(0|0);
 if ($72) {
  $73 = HEAP32[$qdata>>2]|0;
  _free($73);
  $74 = HEAP32[_stderr>>2]|0;
  (_fprintf(($74|0),(7528|0),($vararg_buffer13|0))|0);
  $0 = -12;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $75 = $num_pix;
 $wsq_alloc = $75;
 HEAP32[$wsq_len>>2] = 0;
 $76 = $wsq_data;
 $77 = $wsq_alloc;
 $78 = (_putc_ushort(-96,$76,$77,$wsq_len)|0);
 $ret = $78;
 $79 = ($78|0)!=(0);
 if ($79) {
  $80 = HEAP32[$qdata>>2]|0;
  _free($80);
  $81 = $wsq_data;
  _free($81);
  $82 = $ret;
  $0 = $82;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $83 = $9;
 $84 = $5;
 $85 = $6;
 $86 = $7;
 $87 = $8;
 $88 = $3;
 $89 = $wsq_data;
 $90 = $wsq_alloc;
 $91 = (_putc_nistcom_wsq($83,$84,$85,$86,$87,1,$88,$89,$90,$wsq_len)|0);
 $ret = $91;
 $92 = ($91|0)!=(0);
 if ($92) {
  $93 = HEAP32[$qdata>>2]|0;
  _free($93);
  $94 = $wsq_data;
  _free($94);
  $95 = $ret;
  $0 = $95;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $96 = $wsq_data;
 $97 = $wsq_alloc;
 $98 = (_putc_transform_table(10304,9,10272,7,$96,$97,$wsq_len)|0);
 $ret = $98;
 $99 = ($98|0)!=(0);
 if ($99) {
  $100 = HEAP32[$qdata>>2]|0;
  _free($100);
  $101 = $wsq_data;
  _free($101);
  $102 = $ret;
  $0 = $102;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $103 = $wsq_data;
 $104 = $wsq_alloc;
 $105 = (_putc_quantization_table(8240,$103,$104,$wsq_len)|0);
 $ret = $105;
 $106 = ($105|0)!=(0);
 if ($106) {
  $107 = HEAP32[$qdata>>2]|0;
  _free($107);
  $108 = $wsq_data;
  _free($108);
  $109 = $ret;
  $0 = $109;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $110 = $5;
 $111 = $6;
 $112 = +HEAPF32[$m_shift>>2];
 $113 = +HEAPF32[$r_scale>>2];
 $114 = $wsq_data;
 $115 = $wsq_alloc;
 $116 = (_putc_frame_header_wsq($110,$111,$112,$113,$114,$115,$wsq_len)|0);
 $ret = $116;
 $117 = ($116|0)!=(0);
 if ($117) {
  $118 = HEAP32[$qdata>>2]|0;
  _free($118);
  $119 = $wsq_data;
  _free($119);
  $120 = $ret;
  $0 = $120;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $121 = HEAP32[8>>2]|0;
 $122 = ($121|0)>(0);
 if ($122) {
  $123 = HEAP32[_stderr>>2]|0;
  (_fprintf(($123|0),(7576|0),($vararg_buffer15|0))|0);
 }
 $124 = $num_pix;
 $125 = (_malloc($124)|0);
 $huff_buf = $125;
 $126 = $huff_buf;
 $127 = ($126|0)==(0|0);
 if ($127) {
  $128 = HEAP32[$qdata>>2]|0;
  _free($128);
  $129 = $wsq_data;
  _free($129);
  $130 = HEAP32[_stderr>>2]|0;
  (_fprintf(($130|0),(7616|0),($vararg_buffer17|0))|0);
  $0 = -13;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $131 = HEAP32[$qdata>>2]|0;
 $132 = (_gen_hufftable_wsq($hufftable,$huffbits,$huffvalues,$131,$qsize1,1)|0);
 $ret = $132;
 $133 = ($132|0)!=(0);
 if ($133) {
  $134 = HEAP32[$qdata>>2]|0;
  _free($134);
  $135 = $wsq_data;
  _free($135);
  $136 = $huff_buf;
  _free($136);
  $137 = $ret;
  $0 = $137;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $138 = HEAP32[$huffbits>>2]|0;
 $139 = HEAP32[$huffvalues>>2]|0;
 $140 = $wsq_data;
 $141 = $wsq_alloc;
 $142 = (_putc_huffman_table(-90,0,$138,$139,$140,$141,$wsq_len)|0);
 $ret = $142;
 $143 = ($142|0)!=(0);
 if ($143) {
  $144 = HEAP32[$qdata>>2]|0;
  _free($144);
  $145 = $wsq_data;
  _free($145);
  $146 = $huff_buf;
  _free($146);
  $147 = HEAP32[$huffbits>>2]|0;
  _free($147);
  $148 = HEAP32[$huffvalues>>2]|0;
  _free($148);
  $149 = HEAP32[$hufftable>>2]|0;
  _free($149);
  $150 = $ret;
  $0 = $150;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $151 = HEAP32[$huffbits>>2]|0;
 _free($151);
 $152 = HEAP32[$huffvalues>>2]|0;
 _free($152);
 $153 = HEAP32[8>>2]|0;
 $154 = ($153|0)>(0);
 if ($154) {
  $155 = HEAP32[_stderr>>2]|0;
  (_fprintf(($155|0),(7664|0),($vararg_buffer19|0))|0);
 }
 $156 = $huff_buf;
 $157 = HEAP32[$qdata>>2]|0;
 $158 = HEAP32[$qsize1>>2]|0;
 $159 = HEAP32[$hufftable>>2]|0;
 $160 = (_compress_block($156,$hsize1,$157,$158,74,100,$159)|0);
 $ret = $160;
 $161 = ($160|0)!=(0);
 if ($161) {
  $162 = HEAP32[$qdata>>2]|0;
  _free($162);
  $163 = $wsq_data;
  _free($163);
  $164 = $huff_buf;
  _free($164);
  $165 = HEAP32[$hufftable>>2]|0;
  _free($165);
  $166 = $ret;
  $0 = $166;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $167 = HEAP32[$hufftable>>2]|0;
 _free($167);
 $168 = HEAP32[$hsize1>>2]|0;
 $hsize = $168;
 $169 = $wsq_data;
 $170 = $wsq_alloc;
 $171 = (_putc_block_header(0,$169,$170,$wsq_len)|0);
 $ret = $171;
 $172 = ($171|0)!=(0);
 if ($172) {
  $173 = HEAP32[$qdata>>2]|0;
  _free($173);
  $174 = $wsq_data;
  _free($174);
  $175 = $huff_buf;
  _free($175);
  $176 = $ret;
  $0 = $176;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $177 = $huff_buf;
 $178 = HEAP32[$hsize1>>2]|0;
 $179 = $wsq_data;
 $180 = $wsq_alloc;
 $181 = (_putc_bytes($177,$178,$179,$180,$wsq_len)|0);
 $ret = $181;
 $182 = ($181|0)!=(0);
 if ($182) {
  $183 = HEAP32[$qdata>>2]|0;
  _free($183);
  $184 = $wsq_data;
  _free($184);
  $185 = $huff_buf;
  _free($185);
  $186 = $ret;
  $0 = $186;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $187 = HEAP32[8>>2]|0;
 $188 = ($187|0)>(0);
 if ($188) {
  $189 = HEAP32[_stderr>>2]|0;
  (_fprintf(($189|0),(7712|0),($vararg_buffer21|0))|0);
 }
 $190 = HEAP32[$qsize2>>2]|0;
 HEAP32[$block_sizes>>2] = $190;
 $191 = HEAP32[$qsize3>>2]|0;
 $192 = (($block_sizes) + 4|0);
 HEAP32[$192>>2] = $191;
 $193 = HEAP32[$qdata>>2]|0;
 $194 = HEAP32[$qsize1>>2]|0;
 $195 = (($193) + ($194<<1)|0);
 $196 = (_gen_hufftable_wsq($hufftable,$huffbits,$huffvalues,$195,$block_sizes,2)|0);
 $ret = $196;
 $197 = ($196|0)!=(0);
 if ($197) {
  $198 = HEAP32[$qdata>>2]|0;
  _free($198);
  $199 = $wsq_data;
  _free($199);
  $200 = $huff_buf;
  _free($200);
  $201 = $ret;
  $0 = $201;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $202 = HEAP32[$huffbits>>2]|0;
 $203 = HEAP32[$huffvalues>>2]|0;
 $204 = $wsq_data;
 $205 = $wsq_alloc;
 $206 = (_putc_huffman_table(-90,1,$202,$203,$204,$205,$wsq_len)|0);
 $ret = $206;
 $207 = ($206|0)!=(0);
 if ($207) {
  $208 = HEAP32[$qdata>>2]|0;
  _free($208);
  $209 = $wsq_data;
  _free($209);
  $210 = $huff_buf;
  _free($210);
  $211 = HEAP32[$huffbits>>2]|0;
  _free($211);
  $212 = HEAP32[$huffvalues>>2]|0;
  _free($212);
  $213 = HEAP32[$hufftable>>2]|0;
  _free($213);
  $214 = $ret;
  $0 = $214;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $215 = HEAP32[$huffbits>>2]|0;
 _free($215);
 $216 = HEAP32[$huffvalues>>2]|0;
 _free($216);
 $217 = HEAP32[8>>2]|0;
 $218 = ($217|0)>(0);
 if ($218) {
  $219 = HEAP32[_stderr>>2]|0;
  (_fprintf(($219|0),(7752|0),($vararg_buffer23|0))|0);
 }
 $220 = $huff_buf;
 $221 = HEAP32[$qdata>>2]|0;
 $222 = HEAP32[$qsize1>>2]|0;
 $223 = (($221) + ($222<<1)|0);
 $224 = HEAP32[$qsize2>>2]|0;
 $225 = HEAP32[$hufftable>>2]|0;
 $226 = (_compress_block($220,$hsize2,$223,$224,74,100,$225)|0);
 $ret = $226;
 $227 = ($226|0)!=(0);
 if ($227) {
  $228 = HEAP32[$qdata>>2]|0;
  _free($228);
  $229 = $wsq_data;
  _free($229);
  $230 = $huff_buf;
  _free($230);
  $231 = HEAP32[$hufftable>>2]|0;
  _free($231);
  $232 = $ret;
  $0 = $232;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $233 = HEAP32[$hsize2>>2]|0;
 $234 = $hsize;
 $235 = (($234) + ($233))|0;
 $hsize = $235;
 $236 = $wsq_data;
 $237 = $wsq_alloc;
 $238 = (_putc_block_header(1,$236,$237,$wsq_len)|0);
 $ret = $238;
 $239 = ($238|0)!=(0);
 if ($239) {
  $240 = HEAP32[$qdata>>2]|0;
  _free($240);
  $241 = $wsq_data;
  _free($241);
  $242 = $huff_buf;
  _free($242);
  $243 = HEAP32[$hufftable>>2]|0;
  _free($243);
  $244 = $ret;
  $0 = $244;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $245 = $huff_buf;
 $246 = HEAP32[$hsize2>>2]|0;
 $247 = $wsq_data;
 $248 = $wsq_alloc;
 $249 = (_putc_bytes($245,$246,$247,$248,$wsq_len)|0);
 $ret = $249;
 $250 = ($249|0)!=(0);
 if ($250) {
  $251 = HEAP32[$qdata>>2]|0;
  _free($251);
  $252 = $wsq_data;
  _free($252);
  $253 = $huff_buf;
  _free($253);
  $254 = HEAP32[$hufftable>>2]|0;
  _free($254);
  $255 = $ret;
  $0 = $255;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $256 = HEAP32[8>>2]|0;
 $257 = ($256|0)>(0);
 if ($257) {
  $258 = HEAP32[_stderr>>2]|0;
  (_fprintf(($258|0),(7800|0),($vararg_buffer25|0))|0);
 }
 $259 = $huff_buf;
 $260 = HEAP32[$qdata>>2]|0;
 $261 = HEAP32[$qsize1>>2]|0;
 $262 = (($260) + ($261<<1)|0);
 $263 = HEAP32[$qsize2>>2]|0;
 $264 = (($262) + ($263<<1)|0);
 $265 = HEAP32[$qsize3>>2]|0;
 $266 = HEAP32[$hufftable>>2]|0;
 $267 = (_compress_block($259,$hsize3,$264,$265,74,100,$266)|0);
 $ret = $267;
 $268 = ($267|0)!=(0);
 if ($268) {
  $269 = HEAP32[$qdata>>2]|0;
  _free($269);
  $270 = $wsq_data;
  _free($270);
  $271 = $huff_buf;
  _free($271);
  $272 = HEAP32[$hufftable>>2]|0;
  _free($272);
  $273 = $ret;
  $0 = $273;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $274 = HEAP32[$hufftable>>2]|0;
 _free($274);
 $275 = HEAP32[$qdata>>2]|0;
 _free($275);
 $276 = HEAP32[$hsize3>>2]|0;
 $277 = $hsize;
 $278 = (($277) + ($276))|0;
 $hsize = $278;
 $279 = $wsq_data;
 $280 = $wsq_alloc;
 $281 = (_putc_block_header(1,$279,$280,$wsq_len)|0);
 $ret = $281;
 $282 = ($281|0)!=(0);
 if ($282) {
  $283 = $wsq_data;
  _free($283);
  $284 = $huff_buf;
  _free($284);
  $285 = $ret;
  $0 = $285;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $286 = $huff_buf;
 $287 = HEAP32[$hsize3>>2]|0;
 $288 = $wsq_data;
 $289 = $wsq_alloc;
 $290 = (_putc_bytes($286,$287,$288,$289,$wsq_len)|0);
 $ret = $290;
 $291 = ($290|0)!=(0);
 if ($291) {
  $292 = $wsq_data;
  _free($292);
  $293 = $huff_buf;
  _free($293);
  $294 = $ret;
  $0 = $294;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $295 = HEAP32[8>>2]|0;
 $296 = ($295|0)>(0);
 if ($296) {
  $297 = HEAP32[_stderr>>2]|0;
  (_fprintf(($297|0),(7840|0),($vararg_buffer27|0))|0);
 }
 $298 = $huff_buf;
 _free($298);
 $299 = $wsq_data;
 $300 = $wsq_alloc;
 $301 = (_putc_ushort(-95,$299,$300,$wsq_len)|0);
 $ret = $301;
 $302 = ($301|0)!=(0);
 if ($302) {
  $303 = $wsq_data;
  _free($303);
  $304 = $ret;
  $0 = $304;
  $325 = $0;
  STACKTOP = sp;return ($325|0);
 }
 $305 = HEAP32[8>>2]|0;
 $306 = ($305|0)>=(1);
 if ($306) {
  $307 = HEAP32[_stderr>>2]|0;
  $308 = HEAP32[$hsize1>>2]|0;
  $309 = HEAP32[$hsize2>>2]|0;
  $310 = HEAP32[$hsize3>>2]|0;
  HEAP32[$vararg_buffer29>>2] = $308;
  $vararg_ptr31 = (($vararg_buffer29) + 4|0);
  HEAP32[$vararg_ptr31>>2] = $309;
  $vararg_ptr32 = (($vararg_buffer29) + 8|0);
  HEAP32[$vararg_ptr32>>2] = $310;
  (_fprintf(($307|0),(7880|0),($vararg_buffer29|0))|0);
  $311 = HEAP32[_stderr>>2]|0;
  $312 = $3;
  $313 = $312;
  $314 = $hsize;
  $315 = $num_pix;
  $316 = (+($315|0));
  $317 = $hsize;
  $318 = (+($317|0));
  $319 = $316 / $318;
  $320 = $319;
  HEAPF64[tempDoublePtr>>3]=$313;HEAP32[$vararg_buffer33>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer33+4>>2]=HEAP32[tempDoublePtr+4>>2];
  $vararg_ptr36 = (($vararg_buffer33) + 8|0);
  HEAP32[$vararg_ptr36>>2] = $314;
  $vararg_ptr37 = (($vararg_buffer33) + 12|0);
  HEAPF64[tempDoublePtr>>3]=$320;HEAP32[$vararg_ptr37>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr37+4>>2]=HEAP32[tempDoublePtr+4>>2];
  (_fprintf(($311|0),(7928|0),($vararg_buffer33|0))|0);
 }
 $321 = $wsq_data;
 $322 = $1;
 HEAP32[$322>>2] = $321;
 $323 = HEAP32[$wsq_len>>2]|0;
 $324 = $2;
 HEAP32[$324>>2] = $323;
 $0 = 0;
 $325 = $0;
 STACKTOP = sp;return ($325|0);
}
function _gen_hufftable_wsq($ohufftable,$ohuffbits,$ohuffvalues,$sip,$block_sizes,$num_sizes) {
 $ohufftable = $ohufftable|0;
 $ohuffbits = $ohuffbits|0;
 $ohuffvalues = $ohuffvalues|0;
 $sip = $sip|0;
 $block_sizes = $block_sizes|0;
 $num_sizes = $num_sizes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $adjust = 0, $codesize = 0, $huffbits = 0, $huffcounts = 0, $huffcounts2 = 0, $hufftable1 = 0, $hufftable2 = 0, $huffvalues = 0, $i = 0, $j = 0, $last_size = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $adjust = sp + 44|0;
 $codesize = sp + 40|0;
 $last_size = sp + 36|0;
 $huffbits = sp + 32|0;
 $huffvalues = sp + 28|0;
 $huffcounts = sp + 24|0;
 $huffcounts2 = sp + 20|0;
 $hufftable1 = sp + 16|0;
 $hufftable2 = sp + 12|0;
 $1 = $ohufftable;
 $2 = $ohuffbits;
 $3 = $ohuffvalues;
 $4 = $sip;
 $5 = $block_sizes;
 $6 = $num_sizes;
 $7 = $4;
 $8 = $5;
 $9 = HEAP32[$8>>2]|0;
 $10 = (_count_block($huffcounts,256,$7,$9,74,100)|0);
 $ret = $10;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = $ret;
  $0 = $12;
  $105 = $0;
  STACKTOP = sp;return ($105|0);
 }
 $i = 1;
 while(1) {
  $13 = $i;
  $14 = $6;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = $4;
  $17 = $i;
  $18 = (($17) - 1)|0;
  $19 = $5;
  $20 = (($19) + ($18<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($16) + ($21<<1)|0);
  $23 = $i;
  $24 = $5;
  $25 = (($24) + ($23<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (_count_block($huffcounts2,256,$22,$26,74,100)|0);
  $ret = $27;
  $28 = ($27|0)!=(0);
  if ($28) {
   label = 6;
   break;
  }
  $j = 0;
  while(1) {
   $30 = $j;
   $31 = ($30|0)<(256);
   if (!($31)) {
    break;
   }
   $32 = $j;
   $33 = HEAP32[$huffcounts2>>2]|0;
   $34 = (($33) + ($32<<2)|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = $j;
   $37 = HEAP32[$huffcounts>>2]|0;
   $38 = (($37) + ($36<<2)|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($35))|0;
   HEAP32[$38>>2] = $40;
   $41 = $j;
   $42 = (($41) + 1)|0;
   $j = $42;
  }
  $43 = HEAP32[$huffcounts2>>2]|0;
  _free($43);
  $44 = $i;
  $45 = (($44) + 1)|0;
  $i = $45;
 }
 if ((label|0) == 6) {
  $29 = $ret;
  $0 = $29;
  $105 = $0;
  STACKTOP = sp;return ($105|0);
 }
 $46 = HEAP32[$huffcounts>>2]|0;
 $47 = (_find_huff_sizes($codesize,$46,256)|0);
 $ret = $47;
 $48 = ($47|0)!=(0);
 if ($48) {
  $49 = HEAP32[$huffcounts>>2]|0;
  _free($49);
  $50 = $ret;
  $0 = $50;
  $105 = $0;
  STACKTOP = sp;return ($105|0);
 }
 $51 = HEAP32[$huffcounts>>2]|0;
 _free($51);
 $52 = HEAP32[$codesize>>2]|0;
 $53 = (_find_num_huff_sizes($huffbits,$adjust,$52,256)|0);
 $ret = $53;
 $54 = ($53|0)!=(0);
 if ($54) {
  $55 = HEAP32[$codesize>>2]|0;
  _free($55);
  $56 = $ret;
  $0 = $56;
  $105 = $0;
  STACKTOP = sp;return ($105|0);
 }
 $57 = HEAP32[$adjust>>2]|0;
 $58 = ($57|0)!=(0);
 do {
  if ($58) {
   $59 = HEAP32[$huffbits>>2]|0;
   $60 = (_sort_huffbits($59)|0);
   $ret = $60;
   $61 = ($60|0)!=(0);
   if (!($61)) {
    break;
   }
   $62 = HEAP32[$codesize>>2]|0;
   _free($62);
   $63 = HEAP32[$huffbits>>2]|0;
   _free($63);
   $64 = $ret;
   $0 = $64;
   $105 = $0;
   STACKTOP = sp;return ($105|0);
  }
 } while(0);
 $65 = HEAP32[$codesize>>2]|0;
 $66 = (_sort_code_sizes($huffvalues,$65,256)|0);
 $ret = $66;
 $67 = ($66|0)!=(0);
 if ($67) {
  $68 = HEAP32[$codesize>>2]|0;
  _free($68);
  $69 = HEAP32[$huffbits>>2]|0;
  _free($69);
  $70 = $ret;
  $0 = $70;
  $105 = $0;
  STACKTOP = sp;return ($105|0);
 }
 $71 = HEAP32[$codesize>>2]|0;
 _free($71);
 $72 = HEAP32[$huffbits>>2]|0;
 $73 = (_build_huffsizes($hufftable1,$last_size,$72,256)|0);
 $ret = $73;
 $74 = ($73|0)!=(0);
 if ($74) {
  $75 = HEAP32[$huffbits>>2]|0;
  _free($75);
  $76 = HEAP32[$huffvalues>>2]|0;
  _free($76);
  $77 = $ret;
  $0 = $77;
  $105 = $0;
  STACKTOP = sp;return ($105|0);
 }
 $78 = HEAP32[$hufftable1>>2]|0;
 _build_huffcodes($78);
 $79 = HEAP32[$hufftable1>>2]|0;
 $80 = HEAP32[$last_size>>2]|0;
 $81 = (_check_huffcodes_wsq($79,$80)|0);
 $ret = $81;
 $82 = ($81|0)!=(0);
 if ($82) {
  $83 = HEAP32[_stderr>>2]|0;
  (_fprintf(($83|0),(7976|0),($vararg_buffer|0))|0);
  $84 = HEAP32[_stderr>>2]|0;
  (_fprintf(($84|0),(8024|0),($vararg_buffer1|0))|0);
  $85 = HEAP32[$huffbits>>2]|0;
  _free($85);
  $86 = HEAP32[$huffvalues>>2]|0;
  _free($86);
  $87 = HEAP32[$hufftable1>>2]|0;
  _free($87);
  $88 = $ret;
  $0 = $88;
  $105 = $0;
  STACKTOP = sp;return ($105|0);
 }
 $89 = HEAP32[$hufftable1>>2]|0;
 $90 = HEAP32[$last_size>>2]|0;
 $91 = HEAP32[$huffvalues>>2]|0;
 $92 = (_build_huffcode_table($hufftable2,$89,$90,$91,256)|0);
 $ret = $92;
 $93 = ($92|0)!=(0);
 if ($93) {
  $94 = HEAP32[$huffbits>>2]|0;
  _free($94);
  $95 = HEAP32[$huffvalues>>2]|0;
  _free($95);
  $96 = HEAP32[$hufftable1>>2]|0;
  _free($96);
  $97 = $ret;
  $0 = $97;
  $105 = $0;
  STACKTOP = sp;return ($105|0);
 } else {
  $98 = HEAP32[$hufftable1>>2]|0;
  _free($98);
  $99 = HEAP32[$huffbits>>2]|0;
  $100 = $2;
  HEAP32[$100>>2] = $99;
  $101 = HEAP32[$huffvalues>>2]|0;
  $102 = $3;
  HEAP32[$102>>2] = $101;
  $103 = HEAP32[$hufftable2>>2]|0;
  $104 = $1;
  HEAP32[$104>>2] = $103;
  $0 = 0;
  $105 = $0;
  STACKTOP = sp;return ($105|0);
 }
 return 0|0;
}
function _compress_block($outbuf,$obytes,$sip,$sip_siz,$MaxCoeff,$MaxZRun,$codes) {
 $outbuf = $outbuf|0;
 $obytes = $obytes|0;
 $sip = $sip|0;
 $sip_siz = $sip_siz|0;
 $MaxCoeff = $MaxCoeff|0;
 $MaxZRun = $MaxZRun|0;
 $codes = $codes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $LoMaxCoeff = 0, $bits = 0, $bytes = 0, $cnt = 0, $optr = 0;
 var $outbit = 0, $pix = 0, $rcnt = 0, $state = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $optr = sp + 36|0;
 $outbit = sp + 16|0;
 $bytes = sp + 12|0;
 $bits = sp + 74|0;
 $1 = $outbuf;
 $2 = $obytes;
 $3 = $sip;
 $4 = $sip_siz;
 $5 = $MaxCoeff;
 $6 = $MaxZRun;
 $7 = $codes;
 $rcnt = 0;
 $8 = $5;
 $9 = (1 - ($8))|0;
 $LoMaxCoeff = $9;
 $10 = $1;
 HEAP32[$optr>>2] = $10;
 HEAP32[$outbit>>2] = 7;
 HEAP32[$bytes>>2] = 0;
 HEAP8[$bits>>0] = 0;
 $state = 0;
 $cnt = 0;
 L1: while(1) {
  $11 = $cnt;
  $12 = $4;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = $3;
  $15 = $cnt;
  $16 = (($14) + ($15<<1)|0);
  $17 = HEAP16[$16>>1]|0;
  $pix = $17;
  $18 = $state;
  do {
   if ((($18|0) == 1)) {
    $92 = $pix;
    $93 = $92 << 16 >> 16;
    $94 = ($93|0)==(0);
    if ($94) {
     $95 = $rcnt;
     $96 = ($95>>>0)<(65535);
     if ($96) {
      $97 = $rcnt;
      $98 = (($97) + 1)|0;
      $rcnt = $98;
      break;
     }
    }
    $99 = $rcnt;
    $100 = $6;
    $101 = ($99>>>0)<=($100>>>0);
    if ($101) {
     $102 = $rcnt;
     $103 = $7;
     $104 = (($103) + ($102<<3)|0);
     $105 = (($104) + 4|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = $106&65535;
     $108 = $rcnt;
     $109 = $7;
     $110 = (($109) + ($108<<3)|0);
     $111 = HEAP16[$110>>1]|0;
     _write_bits($optr,$107,$111,$outbit,$bits,$bytes);
    } else {
     $112 = $rcnt;
     $113 = ($112>>>0)<=(255);
     if ($113) {
      $114 = $7;
      $115 = (($114) + 840|0);
      $116 = (($115) + 4|0);
      $117 = HEAP32[$116>>2]|0;
      $118 = $117&65535;
      $119 = $7;
      $120 = (($119) + 840|0);
      $121 = HEAP16[$120>>1]|0;
      _write_bits($optr,$118,$121,$outbit,$bits,$bytes);
      $122 = $rcnt;
      $123 = $122&65535;
      _write_bits($optr,$123,8,$outbit,$bits,$bytes);
     } else {
      $124 = $rcnt;
      $125 = ($124>>>0)<=(65535);
      if (!($125)) {
       label = 28;
       break L1;
      }
      $126 = $7;
      $127 = (($126) + 848|0);
      $128 = (($127) + 4|0);
      $129 = HEAP32[$128>>2]|0;
      $130 = $129&65535;
      $131 = $7;
      $132 = (($131) + 848|0);
      $133 = HEAP16[$132>>1]|0;
      _write_bits($optr,$130,$133,$outbit,$bits,$bytes);
      $134 = $rcnt;
      $135 = $134&65535;
      _write_bits($optr,$135,16,$outbit,$bits,$bytes);
     }
    }
    $137 = $pix;
    $138 = $137 << 16 >> 16;
    $139 = ($138|0)!=(0);
    if ($139) {
     $140 = $pix;
     $141 = $140 << 16 >> 16;
     $142 = $5;
     $143 = ($141|0)>($142|0);
     if ($143) {
      $144 = $pix;
      $145 = $144 << 16 >> 16;
      $146 = ($145|0)>(255);
      if ($146) {
       $147 = $7;
       $148 = (($147) + 824|0);
       $149 = (($148) + 4|0);
       $150 = HEAP32[$149>>2]|0;
       $151 = $150&65535;
       $152 = $7;
       $153 = (($152) + 824|0);
       $154 = HEAP16[$153>>1]|0;
       _write_bits($optr,$151,$154,$outbit,$bits,$bytes);
       $155 = $pix;
       _write_bits($optr,$155,16,$outbit,$bits,$bytes);
      } else {
       $156 = $7;
       $157 = (($156) + 808|0);
       $158 = (($157) + 4|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = $159&65535;
       $161 = $7;
       $162 = (($161) + 808|0);
       $163 = HEAP16[$162>>1]|0;
       _write_bits($optr,$160,$163,$outbit,$bits,$bytes);
       $164 = $pix;
       _write_bits($optr,$164,8,$outbit,$bits,$bytes);
      }
     } else {
      $165 = $pix;
      $166 = $165 << 16 >> 16;
      $167 = $LoMaxCoeff;
      $168 = ($166|0)<($167|0);
      if ($168) {
       $169 = $pix;
       $170 = $169 << 16 >> 16;
       $171 = ($170|0)<(-255);
       if ($171) {
        $172 = $7;
        $173 = (($172) + 832|0);
        $174 = (($173) + 4|0);
        $175 = HEAP32[$174>>2]|0;
        $176 = $175&65535;
        $177 = $7;
        $178 = (($177) + 832|0);
        $179 = HEAP16[$178>>1]|0;
        _write_bits($optr,$176,$179,$outbit,$bits,$bytes);
        $180 = $pix;
        $181 = $180 << 16 >> 16;
        $182 = (0 - ($181))|0;
        $183 = $182&65535;
        _write_bits($optr,$183,16,$outbit,$bits,$bytes);
       } else {
        $184 = $7;
        $185 = (($184) + 816|0);
        $186 = (($185) + 4|0);
        $187 = HEAP32[$186>>2]|0;
        $188 = $187&65535;
        $189 = $7;
        $190 = (($189) + 816|0);
        $191 = HEAP16[$190>>1]|0;
        _write_bits($optr,$188,$191,$outbit,$bits,$bytes);
        $192 = $pix;
        $193 = $192 << 16 >> 16;
        $194 = (0 - ($193))|0;
        $195 = $194&65535;
        _write_bits($optr,$195,8,$outbit,$bits,$bytes);
       }
      } else {
       $196 = $pix;
       $197 = $196 << 16 >> 16;
       $198 = (($197) + 180)|0;
       $199 = $7;
       $200 = (($199) + ($198<<3)|0);
       $201 = (($200) + 4|0);
       $202 = HEAP32[$201>>2]|0;
       $203 = $202&65535;
       $204 = $pix;
       $205 = $204 << 16 >> 16;
       $206 = (($205) + 180)|0;
       $207 = $7;
       $208 = (($207) + ($206<<3)|0);
       $209 = HEAP16[$208>>1]|0;
       _write_bits($optr,$203,$209,$outbit,$bits,$bytes);
      }
     }
     $state = 0;
    } else {
     $rcnt = 1;
     $state = 1;
    }
   } else if ((($18|0) == 0)) {
    $19 = $pix;
    $20 = $19 << 16 >> 16;
    $21 = ($20|0)==(0);
    if ($21) {
     $state = 1;
     $rcnt = 1;
     break;
    }
    $22 = $pix;
    $23 = $22 << 16 >> 16;
    $24 = $5;
    $25 = ($23|0)>($24|0);
    if ($25) {
     $26 = $pix;
     $27 = $26 << 16 >> 16;
     $28 = ($27|0)>(255);
     if ($28) {
      $29 = $7;
      $30 = (($29) + 824|0);
      $31 = (($30) + 4|0);
      $32 = HEAP32[$31>>2]|0;
      $33 = $32&65535;
      $34 = $7;
      $35 = (($34) + 824|0);
      $36 = HEAP16[$35>>1]|0;
      _write_bits($optr,$33,$36,$outbit,$bits,$bytes);
      $37 = $pix;
      _write_bits($optr,$37,16,$outbit,$bits,$bytes);
     } else {
      $38 = $7;
      $39 = (($38) + 808|0);
      $40 = (($39) + 4|0);
      $41 = HEAP32[$40>>2]|0;
      $42 = $41&65535;
      $43 = $7;
      $44 = (($43) + 808|0);
      $45 = HEAP16[$44>>1]|0;
      _write_bits($optr,$42,$45,$outbit,$bits,$bytes);
      $46 = $pix;
      _write_bits($optr,$46,8,$outbit,$bits,$bytes);
     }
    } else {
     $47 = $pix;
     $48 = $47 << 16 >> 16;
     $49 = $LoMaxCoeff;
     $50 = ($48|0)<($49|0);
     if ($50) {
      $51 = $pix;
      $52 = $51 << 16 >> 16;
      $53 = ($52|0)<(-255);
      if ($53) {
       $54 = $7;
       $55 = (($54) + 832|0);
       $56 = (($55) + 4|0);
       $57 = HEAP32[$56>>2]|0;
       $58 = $57&65535;
       $59 = $7;
       $60 = (($59) + 832|0);
       $61 = HEAP16[$60>>1]|0;
       _write_bits($optr,$58,$61,$outbit,$bits,$bytes);
       $62 = $pix;
       $63 = $62 << 16 >> 16;
       $64 = (0 - ($63))|0;
       $65 = $64&65535;
       _write_bits($optr,$65,16,$outbit,$bits,$bytes);
      } else {
       $66 = $7;
       $67 = (($66) + 816|0);
       $68 = (($67) + 4|0);
       $69 = HEAP32[$68>>2]|0;
       $70 = $69&65535;
       $71 = $7;
       $72 = (($71) + 816|0);
       $73 = HEAP16[$72>>1]|0;
       _write_bits($optr,$70,$73,$outbit,$bits,$bytes);
       $74 = $pix;
       $75 = $74 << 16 >> 16;
       $76 = (0 - ($75))|0;
       $77 = $76&65535;
       _write_bits($optr,$77,8,$outbit,$bits,$bytes);
      }
     } else {
      $78 = $pix;
      $79 = $78 << 16 >> 16;
      $80 = (($79) + 180)|0;
      $81 = $7;
      $82 = (($81) + ($80<<3)|0);
      $83 = (($82) + 4|0);
      $84 = HEAP32[$83>>2]|0;
      $85 = $84&65535;
      $86 = $pix;
      $87 = $86 << 16 >> 16;
      $88 = (($87) + 180)|0;
      $89 = $7;
      $90 = (($89) + ($88<<3)|0);
      $91 = HEAP16[$90>>1]|0;
      _write_bits($optr,$85,$91,$outbit,$bits,$bytes);
     }
    }
   }
  } while(0);
  $210 = $cnt;
  $211 = (($210) + 1)|0;
  $cnt = $211;
 }
 if ((label|0) == 28) {
  $136 = HEAP32[_stderr>>2]|0;
  (_fprintf(($136|0),(8048|0),($vararg_buffer|0))|0);
  $0 = -47;
  $254 = $0;
  STACKTOP = sp;return ($254|0);
 }
 $212 = $state;
 $213 = ($212|0)==(1);
 if ($213) {
  $214 = $rcnt;
  $215 = $6;
  $216 = ($214>>>0)<=($215>>>0);
  if ($216) {
   $217 = $rcnt;
   $218 = $7;
   $219 = (($218) + ($217<<3)|0);
   $220 = (($219) + 4|0);
   $221 = HEAP32[$220>>2]|0;
   $222 = $221&65535;
   $223 = $rcnt;
   $224 = $7;
   $225 = (($224) + ($223<<3)|0);
   $226 = HEAP16[$225>>1]|0;
   _write_bits($optr,$222,$226,$outbit,$bits,$bytes);
  } else {
   $227 = $rcnt;
   $228 = ($227>>>0)<=(255);
   do {
    if ($228) {
     $229 = $7;
     $230 = (($229) + 840|0);
     $231 = (($230) + 4|0);
     $232 = HEAP32[$231>>2]|0;
     $233 = $232&65535;
     $234 = $7;
     $235 = (($234) + 840|0);
     $236 = HEAP16[$235>>1]|0;
     _write_bits($optr,$233,$236,$outbit,$bits,$bytes);
     $237 = $rcnt;
     $238 = $237&65535;
     _write_bits($optr,$238,8,$outbit,$bits,$bytes);
    } else {
     $239 = $rcnt;
     $240 = ($239>>>0)<=(65535);
     if ($240) {
      $241 = $7;
      $242 = (($241) + 848|0);
      $243 = (($242) + 4|0);
      $244 = HEAP32[$243>>2]|0;
      $245 = $244&65535;
      $246 = $7;
      $247 = (($246) + 848|0);
      $248 = HEAP16[$247>>1]|0;
      _write_bits($optr,$245,$248,$outbit,$bits,$bytes);
      $249 = $rcnt;
      $250 = $249&65535;
      _write_bits($optr,$250,16,$outbit,$bits,$bytes);
      break;
     } else {
      $251 = HEAP32[_stderr>>2]|0;
      (_fprintf(($251|0),(8096|0),($vararg_buffer1|0))|0);
      $0 = -48;
      $254 = $0;
      STACKTOP = sp;return ($254|0);
     }
    }
   } while(0);
  }
 }
 _flush_bits($optr,$outbit,$bits,$bytes);
 $252 = HEAP32[$bytes>>2]|0;
 $253 = $2;
 HEAP32[$253>>2] = $252;
 $0 = 0;
 $254 = $0;
 STACKTOP = sp;return ($254|0);
}
function _count_block($ocounts,$max_huffcounts,$sip,$sip_siz,$MaxCoeff,$MaxZRun) {
 $ocounts = $ocounts|0;
 $max_huffcounts = $max_huffcounts|0;
 $sip = $sip|0;
 $sip_siz = $sip_siz|0;
 $MaxCoeff = $MaxCoeff|0;
 $MaxZRun = $MaxZRun|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $LoMaxCoeff = 0, $cnt = 0, $counts = 0, $pix = 0, $rcnt = 0, $state = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ocounts;
 $2 = $max_huffcounts;
 $3 = $sip;
 $4 = $sip_siz;
 $5 = $MaxCoeff;
 $6 = $MaxZRun;
 $rcnt = 0;
 $7 = $2;
 $8 = (($7) + 1)|0;
 $9 = (_calloc($8,4)|0);
 $counts = $9;
 $10 = $counts;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = HEAP32[_stderr>>2]|0;
  (_fprintf(($12|0),(8144|0),($vararg_buffer|0))|0);
  $0 = -48;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $13 = $2;
 $14 = $counts;
 $15 = (($14) + ($13<<2)|0);
 HEAP32[$15>>2] = 1;
 $16 = $5;
 $17 = (1 - ($16))|0;
 $LoMaxCoeff = $17;
 $state = 0;
 $cnt = 0;
 L5: while(1) {
  $18 = $cnt;
  $19 = $4;
  $20 = ($18|0)<($19|0);
  if (!($20)) {
   break;
  }
  $21 = $3;
  $22 = $cnt;
  $23 = (($21) + ($22<<1)|0);
  $24 = HEAP16[$23>>1]|0;
  $pix = $24;
  $25 = $state;
  do {
   if ((($25|0) == 0)) {
    $26 = $pix;
    $27 = $26 << 16 >> 16;
    $28 = ($27|0)==(0);
    if ($28) {
     $state = 1;
     $rcnt = 1;
     break;
    }
    $29 = $pix;
    $30 = $29 << 16 >> 16;
    $31 = $5;
    $32 = ($30|0)>($31|0);
    if ($32) {
     $33 = $pix;
     $34 = $33 << 16 >> 16;
     $35 = ($34|0)>(255);
     if ($35) {
      $36 = $counts;
      $37 = (($36) + 412|0);
      $38 = HEAP32[$37>>2]|0;
      $39 = (($38) + 1)|0;
      HEAP32[$37>>2] = $39;
     } else {
      $40 = $counts;
      $41 = (($40) + 404|0);
      $42 = HEAP32[$41>>2]|0;
      $43 = (($42) + 1)|0;
      HEAP32[$41>>2] = $43;
     }
    } else {
     $44 = $pix;
     $45 = $44 << 16 >> 16;
     $46 = $LoMaxCoeff;
     $47 = ($45|0)<($46|0);
     if ($47) {
      $48 = $pix;
      $49 = $48 << 16 >> 16;
      $50 = ($49|0)<(-255);
      if ($50) {
       $51 = $counts;
       $52 = (($51) + 416|0);
       $53 = HEAP32[$52>>2]|0;
       $54 = (($53) + 1)|0;
       HEAP32[$52>>2] = $54;
      } else {
       $55 = $counts;
       $56 = (($55) + 408|0);
       $57 = HEAP32[$56>>2]|0;
       $58 = (($57) + 1)|0;
       HEAP32[$56>>2] = $58;
      }
     } else {
      $59 = $pix;
      $60 = $59 << 16 >> 16;
      $61 = (($60) + 180)|0;
      $62 = $counts;
      $63 = (($62) + ($61<<2)|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (($64) + 1)|0;
      HEAP32[$63>>2] = $65;
     }
    }
   } else if ((($25|0) == 1)) {
    $66 = $pix;
    $67 = $66 << 16 >> 16;
    $68 = ($67|0)==(0);
    if ($68) {
     $69 = $rcnt;
     $70 = ($69>>>0)<(65535);
     if ($70) {
      $71 = $rcnt;
      $72 = (($71) + 1)|0;
      $rcnt = $72;
      break;
     }
    }
    $73 = $rcnt;
    $74 = $6;
    $75 = ($73>>>0)<=($74>>>0);
    if ($75) {
     $76 = $rcnt;
     $77 = $counts;
     $78 = (($77) + ($76<<2)|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = (($79) + 1)|0;
     HEAP32[$78>>2] = $80;
    } else {
     $81 = $rcnt;
     $82 = ($81>>>0)<=(255);
     if ($82) {
      $83 = $counts;
      $84 = (($83) + 420|0);
      $85 = HEAP32[$84>>2]|0;
      $86 = (($85) + 1)|0;
      HEAP32[$84>>2] = $86;
     } else {
      $87 = $rcnt;
      $88 = ($87>>>0)<=(65535);
      if (!($88)) {
       label = 30;
       break L5;
      }
      $89 = $counts;
      $90 = (($89) + 424|0);
      $91 = HEAP32[$90>>2]|0;
      $92 = (($91) + 1)|0;
      HEAP32[$90>>2] = $92;
     }
    }
    $94 = $pix;
    $95 = $94 << 16 >> 16;
    $96 = ($95|0)!=(0);
    if ($96) {
     $97 = $pix;
     $98 = $97 << 16 >> 16;
     $99 = $5;
     $100 = ($98|0)>($99|0);
     if ($100) {
      $101 = $pix;
      $102 = $101 << 16 >> 16;
      $103 = ($102|0)>(255);
      if ($103) {
       $104 = $counts;
       $105 = (($104) + 412|0);
       $106 = HEAP32[$105>>2]|0;
       $107 = (($106) + 1)|0;
       HEAP32[$105>>2] = $107;
      } else {
       $108 = $counts;
       $109 = (($108) + 404|0);
       $110 = HEAP32[$109>>2]|0;
       $111 = (($110) + 1)|0;
       HEAP32[$109>>2] = $111;
      }
     } else {
      $112 = $pix;
      $113 = $112 << 16 >> 16;
      $114 = $LoMaxCoeff;
      $115 = ($113|0)<($114|0);
      if ($115) {
       $116 = $pix;
       $117 = $116 << 16 >> 16;
       $118 = ($117|0)<(-255);
       if ($118) {
        $119 = $counts;
        $120 = (($119) + 416|0);
        $121 = HEAP32[$120>>2]|0;
        $122 = (($121) + 1)|0;
        HEAP32[$120>>2] = $122;
       } else {
        $123 = $counts;
        $124 = (($123) + 408|0);
        $125 = HEAP32[$124>>2]|0;
        $126 = (($125) + 1)|0;
        HEAP32[$124>>2] = $126;
       }
      } else {
       $127 = $pix;
       $128 = $127 << 16 >> 16;
       $129 = (($128) + 180)|0;
       $130 = $counts;
       $131 = (($130) + ($129<<2)|0);
       $132 = HEAP32[$131>>2]|0;
       $133 = (($132) + 1)|0;
       HEAP32[$131>>2] = $133;
      }
     }
     $state = 0;
    } else {
     $rcnt = 1;
     $state = 1;
    }
   }
  } while(0);
  $134 = $cnt;
  $135 = (($134) + 1)|0;
  $cnt = $135;
 }
 if ((label|0) == 30) {
  $93 = HEAP32[_stderr>>2]|0;
  (_fprintf(($93|0),(8184|0),($vararg_buffer1|0))|0);
  $0 = -49;
  $161 = $0;
  STACKTOP = sp;return ($161|0);
 }
 $136 = $state;
 $137 = ($136|0)==(1);
 if ($137) {
  $138 = $rcnt;
  $139 = $6;
  $140 = ($138>>>0)<=($139>>>0);
  if ($140) {
   $141 = $rcnt;
   $142 = $counts;
   $143 = (($142) + ($141<<2)|0);
   $144 = HEAP32[$143>>2]|0;
   $145 = (($144) + 1)|0;
   HEAP32[$143>>2] = $145;
  } else {
   $146 = $rcnt;
   $147 = ($146>>>0)<=(255);
   do {
    if ($147) {
     $148 = $counts;
     $149 = (($148) + 420|0);
     $150 = HEAP32[$149>>2]|0;
     $151 = (($150) + 1)|0;
     HEAP32[$149>>2] = $151;
    } else {
     $152 = $rcnt;
     $153 = ($152>>>0)<=(65535);
     if ($153) {
      $154 = $counts;
      $155 = (($154) + 424|0);
      $156 = HEAP32[$155>>2]|0;
      $157 = (($156) + 1)|0;
      HEAP32[$155>>2] = $157;
      break;
     } else {
      $158 = HEAP32[_stderr>>2]|0;
      (_fprintf(($158|0),(8184|0),($vararg_buffer3|0))|0);
      $0 = -50;
      $161 = $0;
      STACKTOP = sp;return ($161|0);
     }
    }
   } while(0);
  }
 }
 $159 = $counts;
 $160 = $1;
 HEAP32[$160>>2] = $159;
 $0 = 0;
 $161 = $0;
 STACKTOP = sp;return ($161|0);
}
function _check_huffcodes_wsq($hufftable,$last_size) {
 $hufftable = $hufftable|0;
 $last_size = $last_size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $all_ones = 0, $i = 0, $k = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $hufftable;
 $2 = $last_size;
 $i = 0;
 while(1) {
  $3 = $i;
  $4 = $2;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   label = 15;
   break;
  }
  $all_ones = 1;
  $k = 0;
  while(1) {
   $6 = $k;
   $7 = $1;
   $8 = $i;
   $9 = (($7) + ($8<<3)|0);
   $10 = HEAP16[$9>>1]|0;
   $11 = $10 << 16 >> 16;
   $12 = ($6|0)<($11|0);
   if ($12) {
    $13 = $all_ones;
    $14 = ($13|0)!=(0);
    $39 = $14;
   } else {
    $39 = 0;
   }
   if (!($39)) {
    break;
   }
   $15 = $all_ones;
   $16 = ($15|0)!=(0);
   if ($16) {
    $17 = $1;
    $18 = $i;
    $19 = (($17) + ($18<<3)|0);
    $20 = (($19) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = $k;
    $23 = $21 >>> $22;
    $24 = $23 & 1;
    $25 = ($24|0)!=(0);
    $27 = $25;
   } else {
    $27 = 0;
   }
   $26 = $27&1;
   $all_ones = $26;
   $28 = $k;
   $29 = (($28) + 1)|0;
   $k = $29;
  }
  $30 = $all_ones;
  $31 = ($30|0)!=(0);
  if ($31) {
   label = 12;
   break;
  }
  $36 = $i;
  $37 = (($36) + 1)|0;
  $i = $37;
 }
 if ((label|0) == 12) {
  $32 = HEAP32[_stderr>>2]|0;
  (_fprintf(($32|0),(10344|0),($vararg_buffer|0))|0);
  $33 = HEAP32[_stderr>>2]|0;
  (_fprintf(($33|0),(10392|0),($vararg_buffer1|0))|0);
  $34 = HEAP32[_stderr>>2]|0;
  (_fprintf(($34|0),(10440|0),($vararg_buffer3|0))|0);
  $35 = HEAP32[_stderr>>2]|0;
  (_fprintf(($35|0),(10488|0),($vararg_buffer5|0))|0);
  $0 = -1;
  $38 = $0;
  STACKTOP = sp;return ($38|0);
 }
 else if ((label|0) == 15) {
  $0 = 0;
  $38 = $0;
  STACKTOP = sp;return ($38|0);
 }
 return 0|0;
}
function _putc_transform_table($lofilt,$losz,$hifilt,$hisz,$odata,$oalloc,$olen) {
 $lofilt = $lofilt|0;
 $losz = $losz|0;
 $hifilt = $hifilt|0;
 $hisz = $hisz|0;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0.0, $142 = 0.0, $143 = 0.0, $144 = 0, $145 = 0.0, $146 = 0.0, $147 = 0.0, $148 = 0.0, $149 = 0.0, $15 = 0, $150 = 0.0, $151 = 0;
 var $152 = 0.0, $153 = 0.0, $154 = 0, $155 = 0.0, $156 = 0.0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0.0, $163 = 0.0, $164 = 0.0, $165 = 0.0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0.0, $171 = 0.0, $172 = 0.0, $173 = 0, $174 = 0.0, $175 = 0.0, $176 = 0.0, $177 = 0.0, $178 = 0.0, $179 = 0.0, $18 = 0, $180 = 0.0, $181 = 0.0, $182 = 0, $183 = 0.0, $184 = 0, $185 = 0, $186 = 0, $187 = 0.0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0.0, $203 = 0.0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0.0;
 var $53 = 0.0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0.0, $62 = 0.0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0;
 var $71 = 0.0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0.0, $77 = 0.0, $78 = 0.0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0.0, $87 = 0.0, $88 = 0, $89 = 0.0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0.0, $94 = 0, $95 = 0, $96 = 0.0, $97 = 0.0, $98 = 0, $99 = 0, $coef = 0, $dbl_tmp = 0.0, $int_dat = 0, $ret = 0, $scale_ex = 0, $sign = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0;
 var $vararg_buffer16 = 0, $vararg_buffer20 = 0, $vararg_buffer24 = 0, $vararg_buffer4 = 0, $vararg_buffer8 = 0, $vararg_ptr11 = 0, $vararg_ptr15 = 0, $vararg_ptr19 = 0, $vararg_ptr23 = 0, $vararg_ptr3 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer24 = sp + 88|0;
 $vararg_buffer20 = sp + 72|0;
 $vararg_buffer16 = sp + 64|0;
 $vararg_buffer12 = sp + 48|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer4 = sp + 24|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $lofilt;
 $2 = $losz;
 $3 = $hifilt;
 $4 = $hisz;
 $5 = $odata;
 $6 = $oalloc;
 $7 = $olen;
 $8 = HEAP32[8>>2]|0;
 $9 = ($8|0)>(0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(10512|0),($vararg_buffer|0))|0);
 }
 $11 = $5;
 $12 = $6;
 $13 = $7;
 $14 = (_putc_ushort(-92,$11,$12,$13)|0);
 $ret = $14;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $ret;
  $0 = $16;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 $17 = $5;
 $18 = $6;
 $19 = $7;
 $20 = (_putc_ushort(58,$17,$18,$19)|0);
 $ret = $20;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = $ret;
  $0 = $22;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 $23 = $2;
 $24 = $23&255;
 $25 = $5;
 $26 = $6;
 $27 = $7;
 $28 = (_putc_byte($24,$25,$26,$27)|0);
 $ret = $28;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = $ret;
  $0 = $30;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 $31 = $4;
 $32 = $31&255;
 $33 = $5;
 $34 = $6;
 $35 = $7;
 $36 = (_putc_byte($32,$33,$34,$35)|0);
 $ret = $36;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = $ret;
  $0 = $38;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 $39 = $2;
 $40 = $39 >> 1;
 $coef = $40;
 while(1) {
  $41 = $coef;
  $42 = $2;
  $43 = ($41>>>0)<($42>>>0);
  if (!($43)) {
   label = 38;
   break;
  }
  $44 = $coef;
  $45 = $1;
  $46 = (($45) + ($44<<2)|0);
  $47 = +HEAPF32[$46>>2];
  $dbl_tmp = $47;
  $48 = $dbl_tmp;
  $49 = $48;
  $50 = $49 >= 0.0;
  if ($50) {
   $sign = 0;
  } else {
   $sign = 1;
   $51 = $dbl_tmp;
   $52 = $51;
   $53 = $52 * -1.0;
   $54 = $53;
   $dbl_tmp = $54;
  }
  $scale_ex = 0;
  $55 = $dbl_tmp;
  $56 = $55;
  $57 = $56 == 0.0;
  if ($57) {
   $int_dat = 0;
  } else {
   $58 = $dbl_tmp;
   $59 = $58;
   $60 = $59 < 4294967295.0;
   if (!($60)) {
    label = 26;
    break;
   }
   while(1) {
    $61 = $dbl_tmp;
    $62 = $61;
    $63 = $62 < 4294967295.0;
    if (!($63)) {
     break;
    }
    $64 = $scale_ex;
    $65 = $64 << 24 >> 24;
    $66 = (($65) + 1)|0;
    $67 = $66&255;
    $scale_ex = $67;
    $68 = $dbl_tmp;
    $69 = $68;
    $70 = $69 * 10.0;
    $71 = $70;
    $dbl_tmp = $71;
   }
   $72 = $scale_ex;
   $73 = $72 << 24 >> 24;
   $74 = (($73) - 1)|0;
   $75 = $74&255;
   $scale_ex = $75;
   $76 = $dbl_tmp;
   $77 = $76;
   $78 = $77 / 10.0;
   $79 = $78 < 0.0;
   if ($79) {
    $80 = $dbl_tmp;
    $81 = $80;
    $82 = $81 / 10.0;
    $83 = $82 - 0.5;
    $89 = $83;
   } else {
    $84 = $dbl_tmp;
    $85 = $84;
    $86 = $85 / 10.0;
    $87 = $86 + 0.5;
    $89 = $87;
   }
   $88 = (~~(($89))>>>0);
   $int_dat = $88;
  }
  $98 = HEAP32[8>>2]|0;
  $99 = ($98|0)>(2);
  if ($99) {
   $100 = HEAP32[_stderr>>2]|0;
   $101 = $coef;
   $102 = $int_dat;
   HEAP32[$vararg_buffer4>>2] = $101;
   $vararg_ptr7 = (($vararg_buffer4) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $102;
   (_fprintf(($100|0),(10544|0),($vararg_buffer4|0))|0);
   $103 = HEAP32[_stderr>>2]|0;
   $104 = $coef;
   $105 = $coef;
   $106 = $1;
   $107 = (($106) + ($105<<2)|0);
   $108 = +HEAPF32[$107>>2];
   $109 = $108;
   HEAP32[$vararg_buffer8>>2] = $104;
   $vararg_ptr11 = (($vararg_buffer8) + 4|0);
   HEAPF64[tempDoublePtr>>3]=$109;HEAP32[$vararg_ptr11>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr11+4>>2]=HEAP32[tempDoublePtr+4>>2];
   (_fprintf(($103|0),(10560|0),($vararg_buffer8|0))|0);
  }
  $110 = $sign;
  $111 = $5;
  $112 = $6;
  $113 = $7;
  $114 = (_putc_byte($110,$111,$112,$113)|0);
  $ret = $114;
  $115 = ($114|0)!=(0);
  if ($115) {
   label = 31;
   break;
  }
  $117 = $scale_ex;
  $118 = $5;
  $119 = $6;
  $120 = $7;
  $121 = (_putc_byte($117,$118,$119,$120)|0);
  $ret = $121;
  $122 = ($121|0)!=(0);
  if ($122) {
   label = 33;
   break;
  }
  $124 = $int_dat;
  $125 = $5;
  $126 = $6;
  $127 = $7;
  $128 = (_putc_uint($124,$125,$126,$127)|0);
  $ret = $128;
  $129 = ($128|0)!=(0);
  if ($129) {
   label = 35;
   break;
  }
  $131 = $coef;
  $132 = (($131) + 1)|0;
  $coef = $132;
 }
 if ((label|0) == 26) {
  $90 = $coef;
  $91 = $1;
  $92 = (($91) + ($90<<2)|0);
  $93 = +HEAPF32[$92>>2];
  $dbl_tmp = $93;
  $94 = HEAP32[_stderr>>2]|0;
  $95 = $coef;
  $96 = $dbl_tmp;
  $97 = $96;
  HEAP32[$vararg_buffer1>>2] = $95;
  $vararg_ptr3 = (($vararg_buffer1) + 4|0);
  HEAPF64[tempDoublePtr>>3]=$97;HEAP32[$vararg_ptr3>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr3+4>>2]=HEAP32[tempDoublePtr+4>>2];
  (_fprintf(($94|0),(10664|0),($vararg_buffer1|0))|0);
  $0 = -82;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 else if ((label|0) == 31) {
  $116 = $ret;
  $0 = $116;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 else if ((label|0) == 33) {
  $123 = $ret;
  $0 = $123;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 else if ((label|0) == 35) {
  $130 = $ret;
  $0 = $130;
  $230 = $0;
  STACKTOP = sp;return ($230|0);
 }
 else if ((label|0) == 38) {
  $133 = $4;
  $134 = $133 >> 1;
  $coef = $134;
  while(1) {
   $135 = $coef;
   $136 = $4;
   $137 = ($135>>>0)<($136>>>0);
   if (!($137)) {
    label = 65;
    break;
   }
   $138 = $coef;
   $139 = $3;
   $140 = (($139) + ($138<<2)|0);
   $141 = +HEAPF32[$140>>2];
   $dbl_tmp = $141;
   $142 = $dbl_tmp;
   $143 = $142;
   $144 = $143 >= 0.0;
   if ($144) {
    $sign = 0;
   } else {
    $sign = 1;
    $145 = $dbl_tmp;
    $146 = $145;
    $147 = $146 * -1.0;
    $148 = $147;
    $dbl_tmp = $148;
   }
   $scale_ex = 0;
   $149 = $dbl_tmp;
   $150 = $149;
   $151 = $150 == 0.0;
   if ($151) {
    $int_dat = 0;
   } else {
    $152 = $dbl_tmp;
    $153 = $152;
    $154 = $153 < 4294967295.0;
    if (!($154)) {
     label = 53;
     break;
    }
    while(1) {
     $155 = $dbl_tmp;
     $156 = $155;
     $157 = $156 < 4294967295.0;
     if (!($157)) {
      break;
     }
     $158 = $scale_ex;
     $159 = $158 << 24 >> 24;
     $160 = (($159) + 1)|0;
     $161 = $160&255;
     $scale_ex = $161;
     $162 = $dbl_tmp;
     $163 = $162;
     $164 = $163 * 10.0;
     $165 = $164;
     $dbl_tmp = $165;
    }
    $166 = $scale_ex;
    $167 = $166 << 24 >> 24;
    $168 = (($167) - 1)|0;
    $169 = $168&255;
    $scale_ex = $169;
    $170 = $dbl_tmp;
    $171 = $170;
    $172 = $171 / 10.0;
    $173 = $172 < 0.0;
    if ($173) {
     $174 = $dbl_tmp;
     $175 = $174;
     $176 = $175 / 10.0;
     $177 = $176 - 0.5;
     $183 = $177;
    } else {
     $178 = $dbl_tmp;
     $179 = $178;
     $180 = $179 / 10.0;
     $181 = $180 + 0.5;
     $183 = $181;
    }
    $182 = (~~(($183))>>>0);
    $int_dat = $182;
   }
   $192 = HEAP32[8>>2]|0;
   $193 = ($192|0)>(2);
   if ($193) {
    $194 = HEAP32[_stderr>>2]|0;
    $195 = $coef;
    $196 = $int_dat;
    HEAP32[$vararg_buffer16>>2] = $195;
    $vararg_ptr19 = (($vararg_buffer16) + 4|0);
    HEAP32[$vararg_ptr19>>2] = $196;
    (_fprintf(($194|0),(10584|0),($vararg_buffer16|0))|0);
    $197 = HEAP32[_stderr>>2]|0;
    $198 = $coef;
    $199 = $coef;
    $200 = $3;
    $201 = (($200) + ($199<<2)|0);
    $202 = +HEAPF32[$201>>2];
    $203 = $202;
    HEAP32[$vararg_buffer20>>2] = $198;
    $vararg_ptr23 = (($vararg_buffer20) + 4|0);
    HEAPF64[tempDoublePtr>>3]=$203;HEAP32[$vararg_ptr23>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr23+4>>2]=HEAP32[tempDoublePtr+4>>2];
    (_fprintf(($197|0),(10600|0),($vararg_buffer20|0))|0);
   }
   $204 = $sign;
   $205 = $5;
   $206 = $6;
   $207 = $7;
   $208 = (_putc_byte($204,$205,$206,$207)|0);
   $ret = $208;
   $209 = ($208|0)!=(0);
   if ($209) {
    label = 58;
    break;
   }
   $211 = $scale_ex;
   $212 = $5;
   $213 = $6;
   $214 = $7;
   $215 = (_putc_byte($211,$212,$213,$214)|0);
   $ret = $215;
   $216 = ($215|0)!=(0);
   if ($216) {
    label = 60;
    break;
   }
   $218 = $int_dat;
   $219 = $5;
   $220 = $6;
   $221 = $7;
   $222 = (_putc_uint($218,$219,$220,$221)|0);
   $ret = $222;
   $223 = ($222|0)!=(0);
   if ($223) {
    label = 62;
    break;
   }
   $225 = $coef;
   $226 = (($225) + 1)|0;
   $coef = $226;
  }
  if ((label|0) == 53) {
   $184 = $coef;
   $185 = $3;
   $186 = (($185) + ($184<<2)|0);
   $187 = +HEAPF32[$186>>2];
   $dbl_tmp = $187;
   $188 = HEAP32[_stderr>>2]|0;
   $189 = $coef;
   $190 = $dbl_tmp;
   $191 = $190;
   HEAP32[$vararg_buffer12>>2] = $189;
   $vararg_ptr15 = (($vararg_buffer12) + 4|0);
   HEAPF64[tempDoublePtr>>3]=$191;HEAP32[$vararg_ptr15>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr15+4>>2]=HEAP32[tempDoublePtr+4>>2];
   (_fprintf(($188|0),(10720|0),($vararg_buffer12|0))|0);
   $0 = -83;
   $230 = $0;
   STACKTOP = sp;return ($230|0);
  }
  else if ((label|0) == 58) {
   $210 = $ret;
   $0 = $210;
   $230 = $0;
   STACKTOP = sp;return ($230|0);
  }
  else if ((label|0) == 60) {
   $217 = $ret;
   $0 = $217;
   $230 = $0;
   STACKTOP = sp;return ($230|0);
  }
  else if ((label|0) == 62) {
   $224 = $ret;
   $0 = $224;
   $230 = $0;
   STACKTOP = sp;return ($230|0);
  }
  else if ((label|0) == 65) {
   $227 = HEAP32[8>>2]|0;
   $228 = ($227|0)>(0);
   if ($228) {
    $229 = HEAP32[_stderr>>2]|0;
    (_fprintf(($229|0),(10624|0),($vararg_buffer24|0))|0);
   }
   $0 = 0;
   $230 = $0;
   STACKTOP = sp;return ($230|0);
  }
 }
 return 0|0;
}
function _putc_quantization_table($quant_vals,$odata,$oalloc,$olen) {
 $quant_vals = $quant_vals|0;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0.0, $102 = 0.0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0.0, $109 = 0.0, $11 = 0, $110 = 0, $111 = 0.0, $112 = 0.0, $113 = 0.0, $114 = 0.0, $115 = 0.0;
 var $116 = 0.0, $117 = 0.0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0.0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0.0, $127 = 0, $128 = 0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0.0, $147 = 0.0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0.0, $154 = 0.0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0.0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0.0;
 var $73 = 0.0, $74 = 0.0, $75 = 0.0, $76 = 0, $77 = 0.0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0.0, $87 = 0.0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0.0, $93 = 0.0, $94 = 0, $95 = 0.0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $flt_tmp = 0.0, $ret = 0, $scale_ex = 0, $scale_ex2 = 0, $shrt_dat = 0, $shrt_dat2 = 0, $sub = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer14 = 0, $vararg_buffer20 = 0;
 var $vararg_buffer4 = 0, $vararg_buffer8 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $vararg_ptr3 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer20 = sp + 80|0;
 $vararg_buffer14 = sp + 56|0;
 $vararg_buffer8 = sp + 40|0;
 $vararg_buffer4 = sp + 24|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $quant_vals;
 $2 = $odata;
 $3 = $oalloc;
 $4 = $olen;
 $5 = HEAP32[8>>2]|0;
 $6 = ($5|0)>(0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  (_fprintf(($7|0),(10776|0),($vararg_buffer|0))|0);
 }
 $8 = $2;
 $9 = $3;
 $10 = $4;
 $11 = (_putc_ushort(-91,$8,$9,$10)|0);
 $ret = $11;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = $ret;
  $0 = $13;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 $14 = $2;
 $15 = $3;
 $16 = $4;
 $17 = (_putc_ushort(389,$14,$15,$16)|0);
 $ret = $17;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = $ret;
  $0 = $19;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 $20 = $2;
 $21 = $3;
 $22 = $4;
 $23 = (_putc_byte(2,$20,$21,$22)|0);
 $ret = $23;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = $ret;
  $0 = $25;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 $26 = $2;
 $27 = $3;
 $28 = $4;
 $29 = (_putc_ushort(44,$26,$27,$28)|0);
 $ret = $29;
 $30 = ($29|0)!=(0);
 if ($30) {
  $31 = $ret;
  $0 = $31;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 $sub = 0;
 while(1) {
  $32 = $sub;
  $33 = ($32|0)<(64);
  if (!($33)) {
   label = 50;
   break;
  }
  $34 = $sub;
  $35 = ($34|0)>=(0);
  if ($35) {
   $36 = $sub;
   $37 = ($36|0)<(60);
   if ($37) {
    $38 = $sub;
    $39 = $1;
    $40 = (($39) + 268|0);
    $41 = (($40) + ($38<<2)|0);
    $42 = +HEAPF32[$41>>2];
    $43 = $42;
    $44 = $43 != 0.0;
    if ($44) {
     $45 = $sub;
     $46 = $1;
     $47 = (($46) + 268|0);
     $48 = (($47) + ($45<<2)|0);
     $49 = +HEAPF32[$48>>2];
     $flt_tmp = $49;
     $scale_ex = 0;
     $50 = $flt_tmp;
     $51 = $50 < 65535.0;
     if (!($51)) {
      label = 24;
      break;
     }
     while(1) {
      $52 = $flt_tmp;
      $53 = $52 < 65535.0;
      if (!($53)) {
       break;
      }
      $54 = $scale_ex;
      $55 = $54 << 24 >> 24;
      $56 = (($55) + 1)|0;
      $57 = $56&255;
      $scale_ex = $57;
      $58 = $flt_tmp;
      $59 = $58 * 10.0;
      $flt_tmp = $59;
     }
     $60 = $scale_ex;
     $61 = $60 << 24 >> 24;
     $62 = (($61) - 1)|0;
     $63 = $62&255;
     $scale_ex = $63;
     $64 = $flt_tmp;
     $65 = $64;
     $66 = $65 / 10.0;
     $67 = $66 < 0.0;
     if ($67) {
      $68 = $flt_tmp;
      $69 = $68;
      $70 = $69 / 10.0;
      $71 = $70 - 0.5;
      $77 = $71;
     } else {
      $72 = $flt_tmp;
      $73 = $72;
      $74 = $73 / 10.0;
      $75 = $74 + 0.5;
      $77 = $75;
     }
     $76 = (~~(($77)));
     $78 = $76&65535;
     $shrt_dat = $78;
     $88 = $sub;
     $89 = $1;
     $90 = (($89) + 524|0);
     $91 = (($90) + ($88<<2)|0);
     $92 = +HEAPF32[$91>>2];
     $flt_tmp = $92;
     $scale_ex2 = 0;
     $93 = $flt_tmp;
     $94 = $93 < 65535.0;
     if (!($94)) {
      label = 33;
      break;
     }
     while(1) {
      $95 = $flt_tmp;
      $96 = $95 < 65535.0;
      if (!($96)) {
       break;
      }
      $97 = $scale_ex2;
      $98 = $97 << 24 >> 24;
      $99 = (($98) + 1)|0;
      $100 = $99&255;
      $scale_ex2 = $100;
      $101 = $flt_tmp;
      $102 = $101 * 10.0;
      $flt_tmp = $102;
     }
     $103 = $scale_ex2;
     $104 = $103 << 24 >> 24;
     $105 = (($104) - 1)|0;
     $106 = $105&255;
     $scale_ex2 = $106;
     $107 = $flt_tmp;
     $108 = $107;
     $109 = $108 / 10.0;
     $110 = $109 < 0.0;
     if ($110) {
      $111 = $flt_tmp;
      $112 = $111;
      $113 = $112 / 10.0;
      $114 = $113 - 0.5;
      $120 = $114;
     } else {
      $115 = $flt_tmp;
      $116 = $115;
      $117 = $116 / 10.0;
      $118 = $117 + 0.5;
      $120 = $118;
     }
     $119 = (~~(($120)));
     $121 = $119&65535;
     $shrt_dat2 = $121;
    } else {
     $scale_ex = 0;
     $scale_ex2 = 0;
     $shrt_dat = 0;
     $shrt_dat2 = 0;
    }
   } else {
    label = 37;
   }
  } else {
   label = 37;
  }
  if ((label|0) == 37) {
   label = 0;
   $scale_ex = 0;
   $scale_ex2 = 0;
   $shrt_dat = 0;
   $shrt_dat2 = 0;
  }
  $131 = HEAP32[8>>2]|0;
  $132 = ($131|0)>(2);
  if ($132) {
   $133 = HEAP32[_stderr>>2]|0;
   $134 = $sub;
   $135 = $shrt_dat;
   $136 = $135&65535;
   $137 = $sub;
   $138 = $shrt_dat2;
   $139 = $138&65535;
   HEAP32[$vararg_buffer8>>2] = $134;
   $vararg_ptr11 = (($vararg_buffer8) + 4|0);
   HEAP32[$vararg_ptr11>>2] = $136;
   $vararg_ptr12 = (($vararg_buffer8) + 8|0);
   HEAP32[$vararg_ptr12>>2] = $137;
   $vararg_ptr13 = (($vararg_buffer8) + 12|0);
   HEAP32[$vararg_ptr13>>2] = $139;
   (_fprintf(($133|0),(10808|0),($vararg_buffer8|0))|0);
   $140 = HEAP32[_stderr>>2]|0;
   $141 = $sub;
   $142 = $sub;
   $143 = $1;
   $144 = (($143) + 268|0);
   $145 = (($144) + ($142<<2)|0);
   $146 = +HEAPF32[$145>>2];
   $147 = $146;
   $148 = $sub;
   $149 = $sub;
   $150 = $1;
   $151 = (($150) + 524|0);
   $152 = (($151) + ($149<<2)|0);
   $153 = +HEAPF32[$152>>2];
   $154 = $153;
   HEAP32[$vararg_buffer14>>2] = $141;
   $vararg_ptr17 = (($vararg_buffer14) + 4|0);
   HEAPF64[tempDoublePtr>>3]=$147;HEAP32[$vararg_ptr17>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr17+4>>2]=HEAP32[tempDoublePtr+4>>2];
   $vararg_ptr18 = (($vararg_buffer14) + 12|0);
   HEAP32[$vararg_ptr18>>2] = $148;
   $vararg_ptr19 = (($vararg_buffer14) + 16|0);
   HEAPF64[tempDoublePtr>>3]=$154;HEAP32[$vararg_ptr19>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr19+4>>2]=HEAP32[tempDoublePtr+4>>2];
   (_fprintf(($140|0),(10840|0),($vararg_buffer14|0))|0);
  }
  $155 = $scale_ex;
  $156 = $2;
  $157 = $3;
  $158 = $4;
  $159 = (_putc_byte($155,$156,$157,$158)|0);
  $ret = $159;
  $160 = ($159|0)!=(0);
  if ($160) {
   label = 41;
   break;
  }
  $162 = $shrt_dat;
  $163 = $2;
  $164 = $3;
  $165 = $4;
  $166 = (_putc_ushort($162,$163,$164,$165)|0);
  $ret = $166;
  $167 = ($166|0)!=(0);
  if ($167) {
   label = 43;
   break;
  }
  $169 = $scale_ex2;
  $170 = $2;
  $171 = $3;
  $172 = $4;
  $173 = (_putc_byte($169,$170,$171,$172)|0);
  $ret = $173;
  $174 = ($173|0)!=(0);
  if ($174) {
   label = 45;
   break;
  }
  $176 = $shrt_dat2;
  $177 = $2;
  $178 = $3;
  $179 = $4;
  $180 = (_putc_ushort($176,$177,$178,$179)|0);
  $ret = $180;
  $181 = ($180|0)!=(0);
  if ($181) {
   label = 47;
   break;
  }
  $183 = $sub;
  $184 = (($183) + 1)|0;
  $sub = $184;
 }
 if ((label|0) == 24) {
  $79 = $sub;
  $80 = $1;
  $81 = (($80) + 268|0);
  $82 = (($81) + ($79<<2)|0);
  $83 = +HEAPF32[$82>>2];
  $flt_tmp = $83;
  $84 = HEAP32[_stderr>>2]|0;
  $85 = $sub;
  $86 = $flt_tmp;
  $87 = $86;
  HEAP32[$vararg_buffer1>>2] = $85;
  $vararg_ptr3 = (($vararg_buffer1) + 4|0);
  HEAPF64[tempDoublePtr>>3]=$87;HEAP32[$vararg_ptr3>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr3+4>>2]=HEAP32[tempDoublePtr+4>>2];
  (_fprintf(($84|0),(10920|0),($vararg_buffer1|0))|0);
  $0 = -86;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 else if ((label|0) == 33) {
  $122 = $sub;
  $123 = $1;
  $124 = (($123) + 524|0);
  $125 = (($124) + ($122<<2)|0);
  $126 = +HEAPF32[$125>>2];
  $flt_tmp = $126;
  $127 = HEAP32[_stderr>>2]|0;
  $128 = $sub;
  $129 = $flt_tmp;
  $130 = $129;
  HEAP32[$vararg_buffer4>>2] = $128;
  $vararg_ptr7 = (($vararg_buffer4) + 4|0);
  HEAPF64[tempDoublePtr>>3]=$130;HEAP32[$vararg_ptr7>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr7+4>>2]=HEAP32[tempDoublePtr+4>>2];
  (_fprintf(($127|0),(10976|0),($vararg_buffer4|0))|0);
  $0 = -87;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 else if ((label|0) == 41) {
  $161 = $ret;
  $0 = $161;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 else if ((label|0) == 43) {
  $168 = $ret;
  $0 = $168;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 else if ((label|0) == 45) {
  $175 = $ret;
  $0 = $175;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 else if ((label|0) == 47) {
  $182 = $ret;
  $0 = $182;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 else if ((label|0) == 50) {
  $185 = HEAP32[8>>2]|0;
  $186 = ($185|0)>(0);
  if ($186) {
   $187 = HEAP32[_stderr>>2]|0;
   (_fprintf(($187|0),(10880|0),($vararg_buffer20|0))|0);
  }
  $0 = 0;
  $188 = $0;
  STACKTOP = sp;return ($188|0);
 }
 return 0|0;
}
function _putc_frame_header_wsq($width,$height,$m_shift,$r_scale,$odata,$oalloc,$olen) {
 $width = $width|0;
 $height = $height|0;
 $m_shift = +$m_shift;
 $r_scale = +$r_scale;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0.0, $105 = 0.0, $106 = 0, $107 = 0.0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0.0, $114 = 0.0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0.0, $12 = 0, $120 = 0.0, $121 = 0.0, $122 = 0, $123 = 0.0, $124 = 0.0, $125 = 0.0, $126 = 0.0, $127 = 0.0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0.0, $131 = 0, $132 = 0.0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0.0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0.0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0.0, $59 = 0.0;
 var $6 = 0, $60 = 0.0, $61 = 0, $62 = 0.0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0.0, $75 = 0.0, $76 = 0.0, $77 = 0;
 var $78 = 0.0, $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0.0, $86 = 0, $87 = 0.0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $flt_tmp = 0.0, $ret = 0, $scale_ex = 0, $shrt_dat = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp + 24|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $width;
 $2 = $height;
 $3 = $m_shift;
 $4 = $r_scale;
 $5 = $odata;
 $6 = $oalloc;
 $7 = $olen;
 $8 = HEAP32[8>>2]|0;
 $9 = ($8|0)>(0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(11032|0),($vararg_buffer|0))|0);
 }
 $11 = $5;
 $12 = $6;
 $13 = $7;
 $14 = (_putc_ushort(-94,$11,$12,$13)|0);
 $ret = $14;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $ret;
  $0 = $16;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $17 = $5;
 $18 = $6;
 $19 = $7;
 $20 = (_putc_ushort(17,$17,$18,$19)|0);
 $ret = $20;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = $ret;
  $0 = $22;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $23 = $5;
 $24 = $6;
 $25 = $7;
 $26 = (_putc_byte(0,$23,$24,$25)|0);
 $ret = $26;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = $ret;
  $0 = $28;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $29 = $5;
 $30 = $6;
 $31 = $7;
 $32 = (_putc_byte(-1,$29,$30,$31)|0);
 $ret = $32;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = $ret;
  $0 = $34;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $35 = $2;
 $36 = $35&65535;
 $37 = $5;
 $38 = $6;
 $39 = $7;
 $40 = (_putc_ushort($36,$37,$38,$39)|0);
 $ret = $40;
 $41 = ($40|0)!=(0);
 if ($41) {
  $42 = $ret;
  $0 = $42;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $43 = $1;
 $44 = $43&65535;
 $45 = $5;
 $46 = $6;
 $47 = $7;
 $48 = (_putc_ushort($44,$45,$46,$47)|0);
 $ret = $48;
 $49 = ($48|0)!=(0);
 if ($49) {
  $50 = $ret;
  $0 = $50;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $51 = HEAP32[8>>2]|0;
 $52 = ($51|0)>(2);
 if ($52) {
  $53 = HEAP32[_stderr>>2]|0;
  $54 = $3;
  $55 = $54;
  $56 = $4;
  $57 = $56;
  HEAPF64[tempDoublePtr>>3]=$55;HEAP32[$vararg_buffer1>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer1+4>>2]=HEAP32[tempDoublePtr+4>>2];
  $vararg_ptr3 = (($vararg_buffer1) + 8|0);
  HEAPF64[tempDoublePtr>>3]=$57;HEAP32[$vararg_ptr3>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr3+4>>2]=HEAP32[tempDoublePtr+4>>2];
  (_fprintf(($53|0),(11056|0),($vararg_buffer1|0))|0);
 }
 $58 = $3;
 $flt_tmp = $58;
 $scale_ex = 0;
 $59 = $flt_tmp;
 $60 = $59;
 $61 = $60 != 0.0;
 if ($61) {
  while(1) {
   $62 = $flt_tmp;
   $63 = $62 < 65535.0;
   if (!($63)) {
    break;
   }
   $64 = $scale_ex;
   $65 = $64 << 24 >> 24;
   $66 = (($65) + 1)|0;
   $67 = $66&255;
   $scale_ex = $67;
   $68 = $flt_tmp;
   $69 = $68 * 10.0;
   $flt_tmp = $69;
  }
  $70 = $scale_ex;
  $71 = $70 << 24 >> 24;
  $72 = (($71) - 1)|0;
  $73 = $72&255;
  $scale_ex = $73;
  $74 = $flt_tmp;
  $75 = $74;
  $76 = $75 / 10.0;
  $77 = $76 < 0.0;
  if ($77) {
   $78 = $flt_tmp;
   $79 = $78;
   $80 = $79 / 10.0;
   $81 = $80 - 0.5;
   $87 = $81;
  } else {
   $82 = $flt_tmp;
   $83 = $82;
   $84 = $83 / 10.0;
   $85 = $84 + 0.5;
   $87 = $85;
  }
  $86 = (~~(($87)));
  $88 = $86&65535;
  $shrt_dat = $88;
 } else {
  $shrt_dat = 0;
 }
 $89 = $scale_ex;
 $90 = $5;
 $91 = $6;
 $92 = $7;
 $93 = (_putc_byte($89,$90,$91,$92)|0);
 $ret = $93;
 $94 = ($93|0)!=(0);
 if ($94) {
  $95 = $ret;
  $0 = $95;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $96 = $shrt_dat;
 $97 = $5;
 $98 = $6;
 $99 = $7;
 $100 = (_putc_ushort($96,$97,$98,$99)|0);
 $ret = $100;
 $101 = ($100|0)!=(0);
 if ($101) {
  $102 = $ret;
  $0 = $102;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $103 = $4;
 $flt_tmp = $103;
 $scale_ex = 0;
 $104 = $flt_tmp;
 $105 = $104;
 $106 = $105 != 0.0;
 if ($106) {
  while(1) {
   $107 = $flt_tmp;
   $108 = $107 < 65535.0;
   if (!($108)) {
    break;
   }
   $109 = $scale_ex;
   $110 = $109 << 24 >> 24;
   $111 = (($110) + 1)|0;
   $112 = $111&255;
   $scale_ex = $112;
   $113 = $flt_tmp;
   $114 = $113 * 10.0;
   $flt_tmp = $114;
  }
  $115 = $scale_ex;
  $116 = $115 << 24 >> 24;
  $117 = (($116) - 1)|0;
  $118 = $117&255;
  $scale_ex = $118;
  $119 = $flt_tmp;
  $120 = $119;
  $121 = $120 / 10.0;
  $122 = $121 < 0.0;
  if ($122) {
   $123 = $flt_tmp;
   $124 = $123;
   $125 = $124 / 10.0;
   $126 = $125 - 0.5;
   $132 = $126;
  } else {
   $127 = $flt_tmp;
   $128 = $127;
   $129 = $128 / 10.0;
   $130 = $129 + 0.5;
   $132 = $130;
  }
  $131 = (~~(($132)));
  $133 = $131&65535;
  $shrt_dat = $133;
 } else {
  $shrt_dat = 0;
 }
 $134 = $scale_ex;
 $135 = $5;
 $136 = $6;
 $137 = $7;
 $138 = (_putc_byte($134,$135,$136,$137)|0);
 $ret = $138;
 $139 = ($138|0)!=(0);
 if ($139) {
  $140 = $ret;
  $0 = $140;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $141 = $shrt_dat;
 $142 = $5;
 $143 = $6;
 $144 = $7;
 $145 = (_putc_ushort($141,$142,$143,$144)|0);
 $ret = $145;
 $146 = ($145|0)!=(0);
 if ($146) {
  $147 = $ret;
  $0 = $147;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $148 = $5;
 $149 = $6;
 $150 = $7;
 $151 = (_putc_byte(0,$148,$149,$150)|0);
 $ret = $151;
 $152 = ($151|0)!=(0);
 if ($152) {
  $153 = $ret;
  $0 = $153;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $154 = $5;
 $155 = $6;
 $156 = $7;
 $157 = (_putc_ushort(0,$154,$155,$156)|0);
 $ret = $157;
 $158 = ($157|0)!=(0);
 if ($158) {
  $159 = $ret;
  $0 = $159;
  $163 = $0;
  STACKTOP = sp;return ($163|0);
 }
 $160 = HEAP32[8>>2]|0;
 $161 = ($160|0)>(0);
 if ($161) {
  $162 = HEAP32[_stderr>>2]|0;
  (_fprintf(($162|0),(11088|0),($vararg_buffer4|0))|0);
 }
 $0 = 0;
 $163 = $0;
 STACKTOP = sp;return ($163|0);
}
function _putc_block_header($table,$odata,$oalloc,$olen) {
 $table = $table|0;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $table;
 $2 = $odata;
 $3 = $oalloc;
 $4 = $olen;
 $5 = HEAP32[8>>2]|0;
 $6 = ($5|0)>(0);
 if ($6) {
  $7 = HEAP32[_stderr>>2]|0;
  (_fprintf(($7|0),(11128|0),($vararg_buffer|0))|0);
 }
 $8 = $2;
 $9 = $3;
 $10 = $4;
 $11 = (_putc_ushort(-93,$8,$9,$10)|0);
 $ret = $11;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = $ret;
  $0 = $13;
  $31 = $0;
  STACKTOP = sp;return ($31|0);
 }
 $14 = $2;
 $15 = $3;
 $16 = $4;
 $17 = (_putc_ushort(3,$14,$15,$16)|0);
 $ret = $17;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = $ret;
  $0 = $19;
  $31 = $0;
  STACKTOP = sp;return ($31|0);
 }
 $20 = $1;
 $21 = $20&255;
 $22 = $2;
 $23 = $3;
 $24 = $4;
 $25 = (_putc_byte($21,$22,$23,$24)|0);
 $ret = $25;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = $ret;
  $0 = $27;
  $31 = $0;
  STACKTOP = sp;return ($31|0);
 }
 $28 = HEAP32[8>>2]|0;
 $29 = ($28|0)>(0);
 if ($29) {
  $30 = HEAP32[_stderr>>2]|0;
  (_fprintf(($30|0),(11152|0),($vararg_buffer1|0))|0);
 }
 $0 = 0;
 $31 = $0;
 STACKTOP = sp;return ($31|0);
}
function _putc_nistcom_wsq($comment_text,$w,$h,$d,$ppi,$lossyflag,$r_bitrate,$odata,$oalloc,$olen) {
 $comment_text = $comment_text|0;
 $w = $w|0;
 $h = $h|0;
 $d = $d|0;
 $ppi = $ppi|0;
 $lossyflag = $lossyflag|0;
 $r_bitrate = +$r_bitrate;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0.0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0.0;
 var $8 = 0, $9 = 0, $comstr = 0, $gencomflag = 0, $nistcom = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $nistcom = sp + 4|0;
 $comstr = sp;
 $1 = $comment_text;
 $2 = $w;
 $3 = $h;
 $4 = $d;
 $5 = $ppi;
 $6 = $lossyflag;
 $7 = $r_bitrate;
 $8 = $odata;
 $9 = $oalloc;
 $10 = $olen;
 HEAP32[$nistcom>>2] = 0;
 $gencomflag = 0;
 $11 = $1;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = $1;
  $14 = (_strncmp($13,11192,8)|0);
  $15 = ($14|0)==(0);
  do {
   if ($15) {
    $16 = $1;
    $17 = (_string2fet($nistcom,$16)|0);
    $ret = $17;
    $18 = ($17|0)!=(0);
    if (!($18)) {
     break;
    }
    $19 = $ret;
    $0 = $19;
    $61 = $0;
    STACKTOP = sp;return ($61|0);
   } else {
    $gencomflag = 1;
   }
  } while(0);
 }
 $20 = $2;
 $21 = $3;
 $22 = $4;
 $23 = $5;
 $24 = $6;
 $25 = $7;
 $26 = (_combine_wsq_nistcom($nistcom,$20,$21,$22,$23,$24,$25)|0);
 $ret = $26;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$nistcom>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$nistcom>>2]|0;
   _freefet($30);
  }
  $31 = $ret;
  $0 = $31;
  $61 = $0;
  STACKTOP = sp;return ($61|0);
 }
 $32 = HEAP32[$nistcom>>2]|0;
 $33 = (_fet2string($comstr,$32)|0);
 $ret = $33;
 $34 = ($33|0)!=(0);
 if ($34) {
  $35 = HEAP32[$nistcom>>2]|0;
  _freefet($35);
  $36 = $ret;
  $0 = $36;
  $61 = $0;
  STACKTOP = sp;return ($61|0);
 }
 $37 = HEAP32[$comstr>>2]|0;
 $38 = HEAP32[$comstr>>2]|0;
 $39 = (_strlen(($38|0))|0);
 $40 = $8;
 $41 = $9;
 $42 = $10;
 $43 = (_putc_comment(-88,$37,$39,$40,$41,$42)|0);
 $ret = $43;
 $44 = ($43|0)!=(0);
 if ($44) {
  $45 = HEAP32[$nistcom>>2]|0;
  _freefet($45);
  $46 = HEAP32[$comstr>>2]|0;
  _free($46);
  $47 = $ret;
  $0 = $47;
  $61 = $0;
  STACKTOP = sp;return ($61|0);
 }
 $48 = HEAP32[$nistcom>>2]|0;
 _freefet($48);
 $49 = HEAP32[$comstr>>2]|0;
 _free($49);
 $50 = $gencomflag;
 $51 = ($50|0)!=(0);
 do {
  if ($51) {
   $52 = $1;
   $53 = $1;
   $54 = (_strlen(($53|0))|0);
   $55 = $8;
   $56 = $9;
   $57 = $10;
   $58 = (_putc_comment(-88,$52,$54,$55,$56,$57)|0);
   $ret = $58;
   $59 = ($58|0)!=(0);
   if (!($59)) {
    break;
   }
   $60 = $ret;
   $0 = $60;
   $61 = $0;
   STACKTOP = sp;return ($61|0);
  }
 } while(0);
 $0 = 0;
 $61 = $0;
 STACKTOP = sp;return ($61|0);
}
function _build_wsq_trees($w_tree,$w_treelen,$q_tree,$q_treelen,$width,$height) {
 $w_tree = $w_tree|0;
 $w_treelen = $w_treelen|0;
 $q_tree = $q_tree|0;
 $q_treelen = $q_treelen|0;
 $width = $width|0;
 $height = $height|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $w_tree;
 $1 = $w_treelen;
 $2 = $q_tree;
 $3 = $q_treelen;
 $4 = $width;
 $5 = $height;
 $6 = $0;
 $7 = $4;
 $8 = $5;
 _build_w_tree($6,$7,$8);
 $9 = $0;
 $10 = $2;
 _build_q_tree($9,$10);
 STACKTOP = sp;return;
}
function _build_w_tree($w_tree,$width,$height) {
 $w_tree = $w_tree|0;
 $width = $width|0;
 $height = $height|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $lenx = 0, $lenx2 = 0, $leny = 0, $leny2 = 0, $node = 0, $vararg_buffer = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer = sp;
 $0 = $w_tree;
 $1 = $width;
 $2 = $height;
 $node = 0;
 while(1) {
  $3 = $node;
  $4 = ($3|0)<(20);
  if (!($4)) {
   break;
  }
  $5 = $node;
  $6 = $0;
  $7 = (($6) + (($5*24)|0)|0);
  $8 = (($7) + 16|0);
  HEAP32[$8>>2] = 0;
  $9 = $node;
  $10 = $0;
  $11 = (($10) + (($9*24)|0)|0);
  $12 = (($11) + 20|0);
  HEAP32[$12>>2] = 0;
  $13 = $node;
  $14 = (($13) + 1)|0;
  $node = $14;
 }
 $15 = $0;
 $16 = (($15) + 48|0);
 $17 = (($16) + 16|0);
 HEAP32[$17>>2] = 1;
 $18 = $0;
 $19 = (($18) + 96|0);
 $20 = (($19) + 16|0);
 HEAP32[$20>>2] = 1;
 $21 = $0;
 $22 = (($21) + 168|0);
 $23 = (($22) + 16|0);
 HEAP32[$23>>2] = 1;
 $24 = $0;
 $25 = (($24) + 216|0);
 $26 = (($25) + 16|0);
 HEAP32[$26>>2] = 1;
 $27 = $0;
 $28 = (($27) + 264|0);
 $29 = (($28) + 16|0);
 HEAP32[$29>>2] = 1;
 $30 = $0;
 $31 = (($30) + 312|0);
 $32 = (($31) + 16|0);
 HEAP32[$32>>2] = 1;
 $33 = $0;
 $34 = (($33) + 384|0);
 $35 = (($34) + 16|0);
 HEAP32[$35>>2] = 1;
 $36 = $0;
 $37 = (($36) + 432|0);
 $38 = (($37) + 16|0);
 HEAP32[$38>>2] = 1;
 $39 = $0;
 $40 = (($39) + 72|0);
 $41 = (($40) + 20|0);
 HEAP32[$41>>2] = 1;
 $42 = $0;
 $43 = (($42) + 120|0);
 $44 = (($43) + 20|0);
 HEAP32[$44>>2] = 1;
 $45 = $0;
 $46 = (($45) + 192|0);
 $47 = (($46) + 20|0);
 HEAP32[$47>>2] = 1;
 $48 = $0;
 $49 = (($48) + 216|0);
 $50 = (($49) + 20|0);
 HEAP32[$50>>2] = 1;
 $51 = $0;
 $52 = (($51) + 288|0);
 $53 = (($52) + 20|0);
 HEAP32[$53>>2] = 1;
 $54 = $0;
 $55 = (($54) + 312|0);
 $56 = (($55) + 20|0);
 HEAP32[$56>>2] = 1;
 $57 = $0;
 $58 = (($57) + 408|0);
 $59 = (($58) + 20|0);
 HEAP32[$59>>2] = 1;
 $60 = $0;
 $61 = (($60) + 432|0);
 $62 = (($61) + 20|0);
 HEAP32[$62>>2] = 1;
 $63 = $0;
 $64 = $1;
 $65 = $2;
 _w_tree4($63,0,1,$64,$65,0,0,1);
 $66 = $0;
 $67 = (($66) + 24|0);
 $68 = (($67) + 8|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = (($69|0) % 2)&-1;
 $71 = ($70|0)==(0);
 if ($71) {
  $72 = $0;
  $73 = (($72) + 24|0);
  $74 = (($73) + 8|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75|0) / 2)&-1;
  $lenx = $76;
  $77 = $lenx;
  $lenx2 = $77;
 } else {
  $78 = $0;
  $79 = (($78) + 24|0);
  $80 = (($79) + 8|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 1)|0;
  $83 = (($82|0) / 2)&-1;
  $lenx = $83;
  $84 = $lenx;
  $85 = (($84) - 1)|0;
  $lenx2 = $85;
 }
 $86 = $0;
 $87 = (($86) + 24|0);
 $88 = (($87) + 12|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89|0) % 2)&-1;
 $91 = ($90|0)==(0);
 if ($91) {
  $92 = $0;
  $93 = (($92) + 24|0);
  $94 = (($93) + 12|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = (($95|0) / 2)&-1;
  $leny = $96;
  $97 = $leny;
  $leny2 = $97;
 } else {
  $98 = $0;
  $99 = (($98) + 24|0);
  $100 = (($99) + 12|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = (($101) + 1)|0;
  $103 = (($102|0) / 2)&-1;
  $leny = $103;
  $104 = $leny;
  $105 = (($104) - 1)|0;
  $leny2 = $105;
 }
 $106 = $0;
 $107 = $lenx2;
 $108 = $leny;
 $109 = $lenx;
 _w_tree4($106,4,6,$107,$108,$109,0,0);
 $110 = $0;
 $111 = $lenx;
 $112 = $leny2;
 $113 = $leny;
 _w_tree4($110,5,10,$111,$112,0,$113,0);
 $114 = $0;
 $115 = $lenx;
 $116 = $leny;
 _w_tree4($114,14,15,$115,$116,0,0,0);
 $117 = $0;
 $118 = (($117) + 456|0);
 HEAP32[$118>>2] = 0;
 $119 = $0;
 $120 = (($119) + 456|0);
 $121 = (($120) + 4|0);
 HEAP32[$121>>2] = 0;
 $122 = $0;
 $123 = (($122) + 360|0);
 $124 = (($123) + 8|0);
 $125 = HEAP32[$124>>2]|0;
 $126 = (($125|0) % 2)&-1;
 $127 = ($126|0)==(0);
 if ($127) {
  $128 = $0;
  $129 = (($128) + 360|0);
  $130 = (($129) + 8|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = (($131|0) / 2)&-1;
  $133 = $0;
  $134 = (($133) + 456|0);
  $135 = (($134) + 8|0);
  HEAP32[$135>>2] = $132;
 } else {
  $136 = $0;
  $137 = (($136) + 360|0);
  $138 = (($137) + 8|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = (($139) + 1)|0;
  $141 = (($140|0) / 2)&-1;
  $142 = $0;
  $143 = (($142) + 456|0);
  $144 = (($143) + 8|0);
  HEAP32[$144>>2] = $141;
 }
 $145 = $0;
 $146 = (($145) + 360|0);
 $147 = (($146) + 12|0);
 $148 = HEAP32[$147>>2]|0;
 $149 = (($148|0) % 2)&-1;
 $150 = ($149|0)==(0);
 if ($150) {
  $151 = $0;
  $152 = (($151) + 360|0);
  $153 = (($152) + 12|0);
  $154 = HEAP32[$153>>2]|0;
  $155 = (($154|0) / 2)&-1;
  $156 = $0;
  $157 = (($156) + 456|0);
  $158 = (($157) + 12|0);
  HEAP32[$158>>2] = $155;
 } else {
  $159 = $0;
  $160 = (($159) + 360|0);
  $161 = (($160) + 12|0);
  $162 = HEAP32[$161>>2]|0;
  $163 = (($162) + 1)|0;
  $164 = (($163|0) / 2)&-1;
  $165 = $0;
  $166 = (($165) + 456|0);
  $167 = (($166) + 12|0);
  HEAP32[$167>>2] = $164;
 }
 $168 = HEAP32[8>>2]|0;
 $169 = ($168|0)>(1);
 if (!($169)) {
  STACKTOP = sp;return;
 }
 $node = 0;
 while(1) {
  $170 = $node;
  $171 = ($170|0)<(20);
  if (!($171)) {
   break;
  }
  $172 = HEAP32[_stderr>>2]|0;
  $173 = $node;
  $174 = $node;
  $175 = $0;
  $176 = (($175) + (($174*24)|0)|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = $node;
  $179 = $0;
  $180 = (($179) + (($178*24)|0)|0);
  $181 = (($180) + 4|0);
  $182 = HEAP32[$181>>2]|0;
  $183 = $node;
  $184 = $0;
  $185 = (($184) + (($183*24)|0)|0);
  $186 = (($185) + 8|0);
  $187 = HEAP32[$186>>2]|0;
  $188 = $node;
  $189 = $0;
  $190 = (($189) + (($188*24)|0)|0);
  $191 = (($190) + 12|0);
  $192 = HEAP32[$191>>2]|0;
  $193 = $node;
  $194 = $0;
  $195 = (($194) + (($193*24)|0)|0);
  $196 = (($195) + 16|0);
  $197 = HEAP32[$196>>2]|0;
  $198 = $node;
  $199 = $0;
  $200 = (($199) + (($198*24)|0)|0);
  $201 = (($200) + 20|0);
  $202 = HEAP32[$201>>2]|0;
  HEAP32[$vararg_buffer>>2] = $173;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $177;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $182;
  $vararg_ptr3 = (($vararg_buffer) + 12|0);
  HEAP32[$vararg_ptr3>>2] = $187;
  $vararg_ptr4 = (($vararg_buffer) + 16|0);
  HEAP32[$vararg_ptr4>>2] = $192;
  $vararg_ptr5 = (($vararg_buffer) + 20|0);
  HEAP32[$vararg_ptr5>>2] = $197;
  $vararg_ptr6 = (($vararg_buffer) + 24|0);
  HEAP32[$vararg_ptr6>>2] = $202;
  (_fprintf(($172|0),(11208|0),($vararg_buffer|0))|0);
  $203 = $node;
  $204 = (($203) + 1)|0;
  $node = $204;
 }
 $205 = HEAP32[_stderr>>2]|0;
 (_fprintf(($205|0),(11272|0),($vararg_buffer7|0))|0);
 STACKTOP = sp;return;
}
function _build_q_tree($w_tree,$q_tree) {
 $w_tree = $w_tree|0;
 $q_tree = $q_tree|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $node = 0, $vararg_buffer = 0, $vararg_buffer5 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer = sp;
 $0 = $w_tree;
 $1 = $q_tree;
 $2 = $1;
 $3 = $0;
 $4 = (($3) + 336|0);
 $5 = (($4) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $0;
 $8 = (($7) + 336|0);
 $9 = (($8) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $0;
 $12 = (($11) + 336|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $0;
 $15 = (($14) + 336|0);
 $16 = (($15) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 _q_tree16($2,3,$6,$10,$13,$17,0,0);
 $18 = $1;
 $19 = $0;
 $20 = (($19) + 96|0);
 $21 = (($20) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = $0;
 $24 = (($23) + 96|0);
 $25 = (($24) + 12|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $0;
 $28 = (($27) + 96|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $0;
 $31 = (($30) + 96|0);
 $32 = (($31) + 4|0);
 $33 = HEAP32[$32>>2]|0;
 _q_tree16($18,19,$22,$26,$29,$33,0,1);
 $34 = $1;
 $35 = $0;
 $36 = (($35) + 8|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $0;
 $39 = (($38) + 12|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = $0;
 $42 = HEAP32[$41>>2]|0;
 $43 = $0;
 $44 = (($43) + 4|0);
 $45 = HEAP32[$44>>2]|0;
 _q_tree16($34,48,$37,$40,$42,$45,0,0);
 $46 = $1;
 $47 = $0;
 $48 = (($47) + 120|0);
 $49 = (($48) + 8|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = $0;
 $52 = (($51) + 120|0);
 $53 = (($52) + 12|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = $0;
 $56 = (($55) + 120|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = $0;
 $59 = (($58) + 120|0);
 $60 = (($59) + 4|0);
 $61 = HEAP32[$60>>2]|0;
 _q_tree16($46,35,$50,$54,$57,$61,1,0);
 $62 = $1;
 $63 = $0;
 $64 = (($63) + 456|0);
 $65 = (($64) + 8|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = $0;
 $68 = (($67) + 456|0);
 $69 = (($68) + 12|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = $0;
 $72 = (($71) + 456|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = $0;
 $75 = (($74) + 456|0);
 $76 = (($75) + 4|0);
 $77 = HEAP32[$76>>2]|0;
 _q_tree4($62,0,$66,$70,$73,$77);
 $78 = HEAP32[8>>2]|0;
 $79 = ($78|0)>(1);
 if (!($79)) {
  STACKTOP = sp;return;
 }
 $node = 0;
 while(1) {
  $80 = $node;
  $81 = ($80|0)<(60);
  if (!($81)) {
   break;
  }
  $82 = HEAP32[_stderr>>2]|0;
  $83 = $node;
  $84 = $node;
  $85 = $1;
  $86 = (($85) + ($84<<3)|0);
  $87 = HEAP16[$86>>1]|0;
  $88 = $87 << 16 >> 16;
  $89 = $node;
  $90 = $1;
  $91 = (($90) + ($89<<3)|0);
  $92 = (($91) + 2|0);
  $93 = HEAP16[$92>>1]|0;
  $94 = $93 << 16 >> 16;
  $95 = $node;
  $96 = $1;
  $97 = (($96) + ($95<<3)|0);
  $98 = (($97) + 4|0);
  $99 = HEAP16[$98>>1]|0;
  $100 = $99 << 16 >> 16;
  $101 = $node;
  $102 = $1;
  $103 = (($102) + ($101<<3)|0);
  $104 = (($103) + 6|0);
  $105 = HEAP16[$104>>1]|0;
  $106 = $105 << 16 >> 16;
  HEAP32[$vararg_buffer>>2] = $83;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $88;
  $vararg_ptr2 = (($vararg_buffer) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $94;
  $vararg_ptr3 = (($vararg_buffer) + 12|0);
  HEAP32[$vararg_ptr3>>2] = $100;
  $vararg_ptr4 = (($vararg_buffer) + 16|0);
  HEAP32[$vararg_ptr4>>2] = $106;
  (_fprintf(($82|0),(11280|0),($vararg_buffer|0))|0);
  $107 = $node;
  $108 = (($107) + 1)|0;
  $node = $108;
 }
 $109 = HEAP32[_stderr>>2]|0;
 (_fprintf(($109|0),(11272|0),($vararg_buffer5|0))|0);
 STACKTOP = sp;return;
}
function _w_tree4($w_tree,$start1,$start2,$lenx,$leny,$x,$y,$stop1) {
 $w_tree = $w_tree|0;
 $start1 = $start1|0;
 $start2 = $start2|0;
 $lenx = $lenx|0;
 $leny = $leny|0;
 $x = $x|0;
 $y = $y|0;
 $stop1 = $stop1|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $evenx = 0, $eveny = 0, $p1 = 0, $p2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $w_tree;
 $1 = $start1;
 $2 = $start2;
 $3 = $lenx;
 $4 = $leny;
 $5 = $x;
 $6 = $y;
 $7 = $stop1;
 $8 = $1;
 $p1 = $8;
 $9 = $2;
 $p2 = $9;
 $10 = $3;
 $11 = (($10|0) % 2)&-1;
 $evenx = $11;
 $12 = $4;
 $13 = (($12|0) % 2)&-1;
 $eveny = $13;
 $14 = $5;
 $15 = $p1;
 $16 = $0;
 $17 = (($16) + (($15*24)|0)|0);
 HEAP32[$17>>2] = $14;
 $18 = $6;
 $19 = $p1;
 $20 = $0;
 $21 = (($20) + (($19*24)|0)|0);
 $22 = (($21) + 4|0);
 HEAP32[$22>>2] = $18;
 $23 = $3;
 $24 = $p1;
 $25 = $0;
 $26 = (($25) + (($24*24)|0)|0);
 $27 = (($26) + 8|0);
 HEAP32[$27>>2] = $23;
 $28 = $4;
 $29 = $p1;
 $30 = $0;
 $31 = (($30) + (($29*24)|0)|0);
 $32 = (($31) + 12|0);
 HEAP32[$32>>2] = $28;
 $33 = $5;
 $34 = $p2;
 $35 = $0;
 $36 = (($35) + (($34*24)|0)|0);
 HEAP32[$36>>2] = $33;
 $37 = $5;
 $38 = $p2;
 $39 = (($38) + 2)|0;
 $40 = $0;
 $41 = (($40) + (($39*24)|0)|0);
 HEAP32[$41>>2] = $37;
 $42 = $6;
 $43 = $p2;
 $44 = $0;
 $45 = (($44) + (($43*24)|0)|0);
 $46 = (($45) + 4|0);
 HEAP32[$46>>2] = $42;
 $47 = $6;
 $48 = $p2;
 $49 = (($48) + 1)|0;
 $50 = $0;
 $51 = (($50) + (($49*24)|0)|0);
 $52 = (($51) + 4|0);
 HEAP32[$52>>2] = $47;
 $53 = $evenx;
 $54 = ($53|0)==(0);
 if ($54) {
  $55 = $3;
  $56 = (($55|0) / 2)&-1;
  $57 = $p2;
  $58 = $0;
  $59 = (($58) + (($57*24)|0)|0);
  $60 = (($59) + 8|0);
  HEAP32[$60>>2] = $56;
  $61 = $p2;
  $62 = $0;
  $63 = (($62) + (($61*24)|0)|0);
  $64 = (($63) + 8|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = $p2;
  $67 = (($66) + 1)|0;
  $68 = $0;
  $69 = (($68) + (($67*24)|0)|0);
  $70 = (($69) + 8|0);
  HEAP32[$70>>2] = $65;
 } else {
  $71 = $p1;
  $72 = ($71|0)==(4);
  if ($72) {
   $73 = $3;
   $74 = (($73) - 1)|0;
   $75 = (($74|0) / 2)&-1;
   $76 = $p2;
   $77 = $0;
   $78 = (($77) + (($76*24)|0)|0);
   $79 = (($78) + 8|0);
   HEAP32[$79>>2] = $75;
   $80 = $p2;
   $81 = $0;
   $82 = (($81) + (($80*24)|0)|0);
   $83 = (($82) + 8|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = (($84) + 1)|0;
   $86 = $p2;
   $87 = (($86) + 1)|0;
   $88 = $0;
   $89 = (($88) + (($87*24)|0)|0);
   $90 = (($89) + 8|0);
   HEAP32[$90>>2] = $85;
  } else {
   $91 = $3;
   $92 = (($91) + 1)|0;
   $93 = (($92|0) / 2)&-1;
   $94 = $p2;
   $95 = $0;
   $96 = (($95) + (($94*24)|0)|0);
   $97 = (($96) + 8|0);
   HEAP32[$97>>2] = $93;
   $98 = $p2;
   $99 = $0;
   $100 = (($99) + (($98*24)|0)|0);
   $101 = (($100) + 8|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = (($102) - 1)|0;
   $104 = $p2;
   $105 = (($104) + 1)|0;
   $106 = $0;
   $107 = (($106) + (($105*24)|0)|0);
   $108 = (($107) + 8|0);
   HEAP32[$108>>2] = $103;
  }
 }
 $109 = $p2;
 $110 = $0;
 $111 = (($110) + (($109*24)|0)|0);
 $112 = (($111) + 8|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = $5;
 $115 = (($113) + ($114))|0;
 $116 = $p2;
 $117 = (($116) + 1)|0;
 $118 = $0;
 $119 = (($118) + (($117*24)|0)|0);
 HEAP32[$119>>2] = $115;
 $120 = $7;
 $121 = ($120|0)==(0);
 if ($121) {
  $122 = $p2;
  $123 = (($122) + 1)|0;
  $124 = $0;
  $125 = (($124) + (($123*24)|0)|0);
  $126 = (($125) + 8|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = $p2;
  $129 = (($128) + 3)|0;
  $130 = $0;
  $131 = (($130) + (($129*24)|0)|0);
  $132 = (($131) + 8|0);
  HEAP32[$132>>2] = $127;
  $133 = $p2;
  $134 = (($133) + 1)|0;
  $135 = $0;
  $136 = (($135) + (($134*24)|0)|0);
  $137 = HEAP32[$136>>2]|0;
  $138 = $p2;
  $139 = (($138) + 3)|0;
  $140 = $0;
  $141 = (($140) + (($139*24)|0)|0);
  HEAP32[$141>>2] = $137;
 }
 $142 = $p2;
 $143 = $0;
 $144 = (($143) + (($142*24)|0)|0);
 $145 = (($144) + 8|0);
 $146 = HEAP32[$145>>2]|0;
 $147 = $p2;
 $148 = (($147) + 2)|0;
 $149 = $0;
 $150 = (($149) + (($148*24)|0)|0);
 $151 = (($150) + 8|0);
 HEAP32[$151>>2] = $146;
 $152 = $eveny;
 $153 = ($152|0)==(0);
 if ($153) {
  $154 = $4;
  $155 = (($154|0) / 2)&-1;
  $156 = $p2;
  $157 = $0;
  $158 = (($157) + (($156*24)|0)|0);
  $159 = (($158) + 12|0);
  HEAP32[$159>>2] = $155;
  $160 = $p2;
  $161 = $0;
  $162 = (($161) + (($160*24)|0)|0);
  $163 = (($162) + 12|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = $p2;
  $166 = (($165) + 2)|0;
  $167 = $0;
  $168 = (($167) + (($166*24)|0)|0);
  $169 = (($168) + 12|0);
  HEAP32[$169>>2] = $164;
 } else {
  $170 = $p1;
  $171 = ($170|0)==(5);
  if ($171) {
   $172 = $4;
   $173 = (($172) - 1)|0;
   $174 = (($173|0) / 2)&-1;
   $175 = $p2;
   $176 = $0;
   $177 = (($176) + (($175*24)|0)|0);
   $178 = (($177) + 12|0);
   HEAP32[$178>>2] = $174;
   $179 = $p2;
   $180 = $0;
   $181 = (($180) + (($179*24)|0)|0);
   $182 = (($181) + 12|0);
   $183 = HEAP32[$182>>2]|0;
   $184 = (($183) + 1)|0;
   $185 = $p2;
   $186 = (($185) + 2)|0;
   $187 = $0;
   $188 = (($187) + (($186*24)|0)|0);
   $189 = (($188) + 12|0);
   HEAP32[$189>>2] = $184;
  } else {
   $190 = $4;
   $191 = (($190) + 1)|0;
   $192 = (($191|0) / 2)&-1;
   $193 = $p2;
   $194 = $0;
   $195 = (($194) + (($193*24)|0)|0);
   $196 = (($195) + 12|0);
   HEAP32[$196>>2] = $192;
   $197 = $p2;
   $198 = $0;
   $199 = (($198) + (($197*24)|0)|0);
   $200 = (($199) + 12|0);
   $201 = HEAP32[$200>>2]|0;
   $202 = (($201) - 1)|0;
   $203 = $p2;
   $204 = (($203) + 2)|0;
   $205 = $0;
   $206 = (($205) + (($204*24)|0)|0);
   $207 = (($206) + 12|0);
   HEAP32[$207>>2] = $202;
  }
 }
 $208 = $p2;
 $209 = $0;
 $210 = (($209) + (($208*24)|0)|0);
 $211 = (($210) + 12|0);
 $212 = HEAP32[$211>>2]|0;
 $213 = $6;
 $214 = (($212) + ($213))|0;
 $215 = $p2;
 $216 = (($215) + 2)|0;
 $217 = $0;
 $218 = (($217) + (($216*24)|0)|0);
 $219 = (($218) + 4|0);
 HEAP32[$219>>2] = $214;
 $220 = $7;
 $221 = ($220|0)==(0);
 if (!($221)) {
  $244 = $p2;
  $245 = $0;
  $246 = (($245) + (($244*24)|0)|0);
  $247 = (($246) + 12|0);
  $248 = HEAP32[$247>>2]|0;
  $249 = $p2;
  $250 = (($249) + 1)|0;
  $251 = $0;
  $252 = (($251) + (($250*24)|0)|0);
  $253 = (($252) + 12|0);
  HEAP32[$253>>2] = $248;
  STACKTOP = sp;return;
 }
 $222 = $p2;
 $223 = (($222) + 2)|0;
 $224 = $0;
 $225 = (($224) + (($223*24)|0)|0);
 $226 = (($225) + 12|0);
 $227 = HEAP32[$226>>2]|0;
 $228 = $p2;
 $229 = (($228) + 3)|0;
 $230 = $0;
 $231 = (($230) + (($229*24)|0)|0);
 $232 = (($231) + 12|0);
 HEAP32[$232>>2] = $227;
 $233 = $p2;
 $234 = (($233) + 2)|0;
 $235 = $0;
 $236 = (($235) + (($234*24)|0)|0);
 $237 = (($236) + 4|0);
 $238 = HEAP32[$237>>2]|0;
 $239 = $p2;
 $240 = (($239) + 3)|0;
 $241 = $0;
 $242 = (($241) + (($240*24)|0)|0);
 $243 = (($242) + 4|0);
 HEAP32[$243>>2] = $238;
 $244 = $p2;
 $245 = $0;
 $246 = (($245) + (($244*24)|0)|0);
 $247 = (($246) + 12|0);
 $248 = HEAP32[$247>>2]|0;
 $249 = $p2;
 $250 = (($249) + 1)|0;
 $251 = $0;
 $252 = (($251) + (($250*24)|0)|0);
 $253 = (($252) + 12|0);
 HEAP32[$253>>2] = $248;
 STACKTOP = sp;return;
}
function _q_tree16($q_tree,$start,$lenx,$leny,$x,$y,$rw,$cl) {
 $q_tree = $q_tree|0;
 $start = $start|0;
 $lenx = $lenx|0;
 $leny = $leny|0;
 $x = $x|0;
 $y = $y|0;
 $rw = $rw|0;
 $cl = $cl|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $evenx = 0;
 var $eveny = 0, $p = 0, $temp2x = 0, $temp2y = 0, $tempx = 0, $tempy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = $q_tree;
 $1 = $start;
 $2 = $lenx;
 $3 = $leny;
 $4 = $x;
 $5 = $y;
 $6 = $rw;
 $7 = $cl;
 $8 = $1;
 $p = $8;
 $9 = $2;
 $10 = (($9|0) % 2)&-1;
 $evenx = $10;
 $11 = $3;
 $12 = (($11|0) % 2)&-1;
 $eveny = $12;
 $13 = $evenx;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = $2;
  $16 = (($15|0) / 2)&-1;
  $tempx = $16;
  $17 = $tempx;
  $temp2x = $17;
 } else {
  $18 = $7;
  $19 = ($18|0)!=(0);
  if ($19) {
   $20 = $2;
   $21 = (($20) + 1)|0;
   $22 = (($21|0) / 2)&-1;
   $temp2x = $22;
   $23 = $temp2x;
   $24 = (($23) - 1)|0;
   $tempx = $24;
  } else {
   $25 = $2;
   $26 = (($25) + 1)|0;
   $27 = (($26|0) / 2)&-1;
   $tempx = $27;
   $28 = $tempx;
   $29 = (($28) - 1)|0;
   $temp2x = $29;
  }
 }
 $30 = $eveny;
 $31 = ($30|0)==(0);
 if ($31) {
  $32 = $3;
  $33 = (($32|0) / 2)&-1;
  $tempy = $33;
  $34 = $tempy;
  $temp2y = $34;
 } else {
  $35 = $6;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = $3;
   $38 = (($37) + 1)|0;
   $39 = (($38|0) / 2)&-1;
   $temp2y = $39;
   $40 = $temp2y;
   $41 = (($40) - 1)|0;
   $tempy = $41;
  } else {
   $42 = $3;
   $43 = (($42) + 1)|0;
   $44 = (($43|0) / 2)&-1;
   $tempy = $44;
   $45 = $tempy;
   $46 = (($45) - 1)|0;
   $temp2y = $46;
  }
 }
 $47 = $tempx;
 $48 = (($47|0) % 2)&-1;
 $evenx = $48;
 $49 = $tempy;
 $50 = (($49|0) % 2)&-1;
 $eveny = $50;
 $51 = $4;
 $52 = $51&65535;
 $53 = $p;
 $54 = $0;
 $55 = (($54) + ($53<<3)|0);
 HEAP16[$55>>1] = $52;
 $56 = $4;
 $57 = $56&65535;
 $58 = $p;
 $59 = (($58) + 2)|0;
 $60 = $0;
 $61 = (($60) + ($59<<3)|0);
 HEAP16[$61>>1] = $57;
 $62 = $5;
 $63 = $62&65535;
 $64 = $p;
 $65 = $0;
 $66 = (($65) + ($64<<3)|0);
 $67 = (($66) + 2|0);
 HEAP16[$67>>1] = $63;
 $68 = $5;
 $69 = $68&65535;
 $70 = $p;
 $71 = (($70) + 1)|0;
 $72 = $0;
 $73 = (($72) + ($71<<3)|0);
 $74 = (($73) + 2|0);
 HEAP16[$74>>1] = $69;
 $75 = $evenx;
 $76 = ($75|0)==(0);
 if ($76) {
  $77 = $tempx;
  $78 = (($77|0) / 2)&-1;
  $79 = $78&65535;
  $80 = $p;
  $81 = $0;
  $82 = (($81) + ($80<<3)|0);
  $83 = (($82) + 4|0);
  HEAP16[$83>>1] = $79;
  $84 = $p;
  $85 = $0;
  $86 = (($85) + ($84<<3)|0);
  $87 = (($86) + 4|0);
  $88 = HEAP16[$87>>1]|0;
  $89 = $p;
  $90 = (($89) + 1)|0;
  $91 = $0;
  $92 = (($91) + ($90<<3)|0);
  $93 = (($92) + 4|0);
  HEAP16[$93>>1] = $88;
  $94 = $p;
  $95 = $0;
  $96 = (($95) + ($94<<3)|0);
  $97 = (($96) + 4|0);
  $98 = HEAP16[$97>>1]|0;
  $99 = $p;
  $100 = (($99) + 2)|0;
  $101 = $0;
  $102 = (($101) + ($100<<3)|0);
  $103 = (($102) + 4|0);
  HEAP16[$103>>1] = $98;
  $104 = $p;
  $105 = $0;
  $106 = (($105) + ($104<<3)|0);
  $107 = (($106) + 4|0);
  $108 = HEAP16[$107>>1]|0;
  $109 = $p;
  $110 = (($109) + 3)|0;
  $111 = $0;
  $112 = (($111) + ($110<<3)|0);
  $113 = (($112) + 4|0);
  HEAP16[$113>>1] = $108;
 } else {
  $114 = $tempx;
  $115 = (($114) + 1)|0;
  $116 = (($115|0) / 2)&-1;
  $117 = $116&65535;
  $118 = $p;
  $119 = $0;
  $120 = (($119) + ($118<<3)|0);
  $121 = (($120) + 4|0);
  HEAP16[$121>>1] = $117;
  $122 = $p;
  $123 = $0;
  $124 = (($123) + ($122<<3)|0);
  $125 = (($124) + 4|0);
  $126 = HEAP16[$125>>1]|0;
  $127 = $126 << 16 >> 16;
  $128 = (($127) - 1)|0;
  $129 = $128&65535;
  $130 = $p;
  $131 = (($130) + 1)|0;
  $132 = $0;
  $133 = (($132) + ($131<<3)|0);
  $134 = (($133) + 4|0);
  HEAP16[$134>>1] = $129;
  $135 = $p;
  $136 = $0;
  $137 = (($136) + ($135<<3)|0);
  $138 = (($137) + 4|0);
  $139 = HEAP16[$138>>1]|0;
  $140 = $p;
  $141 = (($140) + 2)|0;
  $142 = $0;
  $143 = (($142) + ($141<<3)|0);
  $144 = (($143) + 4|0);
  HEAP16[$144>>1] = $139;
  $145 = $p;
  $146 = (($145) + 1)|0;
  $147 = $0;
  $148 = (($147) + ($146<<3)|0);
  $149 = (($148) + 4|0);
  $150 = HEAP16[$149>>1]|0;
  $151 = $p;
  $152 = (($151) + 3)|0;
  $153 = $0;
  $154 = (($153) + ($152<<3)|0);
  $155 = (($154) + 4|0);
  HEAP16[$155>>1] = $150;
 }
 $156 = $4;
 $157 = $p;
 $158 = $0;
 $159 = (($158) + ($157<<3)|0);
 $160 = (($159) + 4|0);
 $161 = HEAP16[$160>>1]|0;
 $162 = $161 << 16 >> 16;
 $163 = (($156) + ($162))|0;
 $164 = $163&65535;
 $165 = $p;
 $166 = (($165) + 1)|0;
 $167 = $0;
 $168 = (($167) + ($166<<3)|0);
 HEAP16[$168>>1] = $164;
 $169 = $p;
 $170 = (($169) + 1)|0;
 $171 = $0;
 $172 = (($171) + ($170<<3)|0);
 $173 = HEAP16[$172>>1]|0;
 $174 = $p;
 $175 = (($174) + 3)|0;
 $176 = $0;
 $177 = (($176) + ($175<<3)|0);
 HEAP16[$177>>1] = $173;
 $178 = $eveny;
 $179 = ($178|0)==(0);
 if ($179) {
  $180 = $tempy;
  $181 = (($180|0) / 2)&-1;
  $182 = $181&65535;
  $183 = $p;
  $184 = $0;
  $185 = (($184) + ($183<<3)|0);
  $186 = (($185) + 6|0);
  HEAP16[$186>>1] = $182;
  $187 = $p;
  $188 = $0;
  $189 = (($188) + ($187<<3)|0);
  $190 = (($189) + 6|0);
  $191 = HEAP16[$190>>1]|0;
  $192 = $p;
  $193 = (($192) + 1)|0;
  $194 = $0;
  $195 = (($194) + ($193<<3)|0);
  $196 = (($195) + 6|0);
  HEAP16[$196>>1] = $191;
  $197 = $p;
  $198 = $0;
  $199 = (($198) + ($197<<3)|0);
  $200 = (($199) + 6|0);
  $201 = HEAP16[$200>>1]|0;
  $202 = $p;
  $203 = (($202) + 2)|0;
  $204 = $0;
  $205 = (($204) + ($203<<3)|0);
  $206 = (($205) + 6|0);
  HEAP16[$206>>1] = $201;
  $207 = $p;
  $208 = $0;
  $209 = (($208) + ($207<<3)|0);
  $210 = (($209) + 6|0);
  $211 = HEAP16[$210>>1]|0;
  $212 = $p;
  $213 = (($212) + 3)|0;
  $214 = $0;
  $215 = (($214) + ($213<<3)|0);
  $216 = (($215) + 6|0);
  HEAP16[$216>>1] = $211;
 } else {
  $217 = $tempy;
  $218 = (($217) + 1)|0;
  $219 = (($218|0) / 2)&-1;
  $220 = $219&65535;
  $221 = $p;
  $222 = $0;
  $223 = (($222) + ($221<<3)|0);
  $224 = (($223) + 6|0);
  HEAP16[$224>>1] = $220;
  $225 = $p;
  $226 = $0;
  $227 = (($226) + ($225<<3)|0);
  $228 = (($227) + 6|0);
  $229 = HEAP16[$228>>1]|0;
  $230 = $p;
  $231 = (($230) + 1)|0;
  $232 = $0;
  $233 = (($232) + ($231<<3)|0);
  $234 = (($233) + 6|0);
  HEAP16[$234>>1] = $229;
  $235 = $p;
  $236 = $0;
  $237 = (($236) + ($235<<3)|0);
  $238 = (($237) + 6|0);
  $239 = HEAP16[$238>>1]|0;
  $240 = $239 << 16 >> 16;
  $241 = (($240) - 1)|0;
  $242 = $241&65535;
  $243 = $p;
  $244 = (($243) + 2)|0;
  $245 = $0;
  $246 = (($245) + ($244<<3)|0);
  $247 = (($246) + 6|0);
  HEAP16[$247>>1] = $242;
  $248 = $p;
  $249 = (($248) + 2)|0;
  $250 = $0;
  $251 = (($250) + ($249<<3)|0);
  $252 = (($251) + 6|0);
  $253 = HEAP16[$252>>1]|0;
  $254 = $p;
  $255 = (($254) + 3)|0;
  $256 = $0;
  $257 = (($256) + ($255<<3)|0);
  $258 = (($257) + 6|0);
  HEAP16[$258>>1] = $253;
 }
 $259 = $5;
 $260 = $p;
 $261 = $0;
 $262 = (($261) + ($260<<3)|0);
 $263 = (($262) + 6|0);
 $264 = HEAP16[$263>>1]|0;
 $265 = $264 << 16 >> 16;
 $266 = (($259) + ($265))|0;
 $267 = $266&65535;
 $268 = $p;
 $269 = (($268) + 2)|0;
 $270 = $0;
 $271 = (($270) + ($269<<3)|0);
 $272 = (($271) + 2|0);
 HEAP16[$272>>1] = $267;
 $273 = $p;
 $274 = (($273) + 2)|0;
 $275 = $0;
 $276 = (($275) + ($274<<3)|0);
 $277 = (($276) + 2|0);
 $278 = HEAP16[$277>>1]|0;
 $279 = $p;
 $280 = (($279) + 3)|0;
 $281 = $0;
 $282 = (($281) + ($280<<3)|0);
 $283 = (($282) + 2|0);
 HEAP16[$283>>1] = $278;
 $284 = $temp2x;
 $285 = (($284|0) % 2)&-1;
 $evenx = $285;
 $286 = $4;
 $287 = $tempx;
 $288 = (($286) + ($287))|0;
 $289 = $288&65535;
 $290 = $p;
 $291 = (($290) + 4)|0;
 $292 = $0;
 $293 = (($292) + ($291<<3)|0);
 HEAP16[$293>>1] = $289;
 $294 = $p;
 $295 = (($294) + 4)|0;
 $296 = $0;
 $297 = (($296) + ($295<<3)|0);
 $298 = HEAP16[$297>>1]|0;
 $299 = $p;
 $300 = (($299) + 6)|0;
 $301 = $0;
 $302 = (($301) + ($300<<3)|0);
 HEAP16[$302>>1] = $298;
 $303 = $5;
 $304 = $303&65535;
 $305 = $p;
 $306 = (($305) + 4)|0;
 $307 = $0;
 $308 = (($307) + ($306<<3)|0);
 $309 = (($308) + 2|0);
 HEAP16[$309>>1] = $304;
 $310 = $5;
 $311 = $310&65535;
 $312 = $p;
 $313 = (($312) + 5)|0;
 $314 = $0;
 $315 = (($314) + ($313<<3)|0);
 $316 = (($315) + 2|0);
 HEAP16[$316>>1] = $311;
 $317 = $p;
 $318 = (($317) + 2)|0;
 $319 = $0;
 $320 = (($319) + ($318<<3)|0);
 $321 = (($320) + 2|0);
 $322 = HEAP16[$321>>1]|0;
 $323 = $p;
 $324 = (($323) + 6)|0;
 $325 = $0;
 $326 = (($325) + ($324<<3)|0);
 $327 = (($326) + 2|0);
 HEAP16[$327>>1] = $322;
 $328 = $p;
 $329 = (($328) + 2)|0;
 $330 = $0;
 $331 = (($330) + ($329<<3)|0);
 $332 = (($331) + 2|0);
 $333 = HEAP16[$332>>1]|0;
 $334 = $p;
 $335 = (($334) + 7)|0;
 $336 = $0;
 $337 = (($336) + ($335<<3)|0);
 $338 = (($337) + 2|0);
 HEAP16[$338>>1] = $333;
 $339 = $p;
 $340 = $0;
 $341 = (($340) + ($339<<3)|0);
 $342 = (($341) + 6|0);
 $343 = HEAP16[$342>>1]|0;
 $344 = $p;
 $345 = (($344) + 4)|0;
 $346 = $0;
 $347 = (($346) + ($345<<3)|0);
 $348 = (($347) + 6|0);
 HEAP16[$348>>1] = $343;
 $349 = $p;
 $350 = $0;
 $351 = (($350) + ($349<<3)|0);
 $352 = (($351) + 6|0);
 $353 = HEAP16[$352>>1]|0;
 $354 = $p;
 $355 = (($354) + 5)|0;
 $356 = $0;
 $357 = (($356) + ($355<<3)|0);
 $358 = (($357) + 6|0);
 HEAP16[$358>>1] = $353;
 $359 = $p;
 $360 = (($359) + 2)|0;
 $361 = $0;
 $362 = (($361) + ($360<<3)|0);
 $363 = (($362) + 6|0);
 $364 = HEAP16[$363>>1]|0;
 $365 = $p;
 $366 = (($365) + 6)|0;
 $367 = $0;
 $368 = (($367) + ($366<<3)|0);
 $369 = (($368) + 6|0);
 HEAP16[$369>>1] = $364;
 $370 = $p;
 $371 = (($370) + 2)|0;
 $372 = $0;
 $373 = (($372) + ($371<<3)|0);
 $374 = (($373) + 6|0);
 $375 = HEAP16[$374>>1]|0;
 $376 = $p;
 $377 = (($376) + 7)|0;
 $378 = $0;
 $379 = (($378) + ($377<<3)|0);
 $380 = (($379) + 6|0);
 HEAP16[$380>>1] = $375;
 $381 = $evenx;
 $382 = ($381|0)==(0);
 if ($382) {
  $383 = $temp2x;
  $384 = (($383|0) / 2)&-1;
  $385 = $384&65535;
  $386 = $p;
  $387 = (($386) + 4)|0;
  $388 = $0;
  $389 = (($388) + ($387<<3)|0);
  $390 = (($389) + 4|0);
  HEAP16[$390>>1] = $385;
  $391 = $p;
  $392 = (($391) + 4)|0;
  $393 = $0;
  $394 = (($393) + ($392<<3)|0);
  $395 = (($394) + 4|0);
  $396 = HEAP16[$395>>1]|0;
  $397 = $p;
  $398 = (($397) + 5)|0;
  $399 = $0;
  $400 = (($399) + ($398<<3)|0);
  $401 = (($400) + 4|0);
  HEAP16[$401>>1] = $396;
  $402 = $p;
  $403 = (($402) + 4)|0;
  $404 = $0;
  $405 = (($404) + ($403<<3)|0);
  $406 = (($405) + 4|0);
  $407 = HEAP16[$406>>1]|0;
  $408 = $p;
  $409 = (($408) + 6)|0;
  $410 = $0;
  $411 = (($410) + ($409<<3)|0);
  $412 = (($411) + 4|0);
  HEAP16[$412>>1] = $407;
  $413 = $p;
  $414 = (($413) + 4)|0;
  $415 = $0;
  $416 = (($415) + ($414<<3)|0);
  $417 = (($416) + 4|0);
  $418 = HEAP16[$417>>1]|0;
  $419 = $p;
  $420 = (($419) + 7)|0;
  $421 = $0;
  $422 = (($421) + ($420<<3)|0);
  $423 = (($422) + 4|0);
  HEAP16[$423>>1] = $418;
 } else {
  $424 = $temp2x;
  $425 = (($424) + 1)|0;
  $426 = (($425|0) / 2)&-1;
  $427 = $426&65535;
  $428 = $p;
  $429 = (($428) + 5)|0;
  $430 = $0;
  $431 = (($430) + ($429<<3)|0);
  $432 = (($431) + 4|0);
  HEAP16[$432>>1] = $427;
  $433 = $p;
  $434 = (($433) + 5)|0;
  $435 = $0;
  $436 = (($435) + ($434<<3)|0);
  $437 = (($436) + 4|0);
  $438 = HEAP16[$437>>1]|0;
  $439 = $438 << 16 >> 16;
  $440 = (($439) - 1)|0;
  $441 = $440&65535;
  $442 = $p;
  $443 = (($442) + 4)|0;
  $444 = $0;
  $445 = (($444) + ($443<<3)|0);
  $446 = (($445) + 4|0);
  HEAP16[$446>>1] = $441;
  $447 = $p;
  $448 = (($447) + 4)|0;
  $449 = $0;
  $450 = (($449) + ($448<<3)|0);
  $451 = (($450) + 4|0);
  $452 = HEAP16[$451>>1]|0;
  $453 = $p;
  $454 = (($453) + 6)|0;
  $455 = $0;
  $456 = (($455) + ($454<<3)|0);
  $457 = (($456) + 4|0);
  HEAP16[$457>>1] = $452;
  $458 = $p;
  $459 = (($458) + 5)|0;
  $460 = $0;
  $461 = (($460) + ($459<<3)|0);
  $462 = (($461) + 4|0);
  $463 = HEAP16[$462>>1]|0;
  $464 = $p;
  $465 = (($464) + 7)|0;
  $466 = $0;
  $467 = (($466) + ($465<<3)|0);
  $468 = (($467) + 4|0);
  HEAP16[$468>>1] = $463;
 }
 $469 = $p;
 $470 = (($469) + 4)|0;
 $471 = $0;
 $472 = (($471) + ($470<<3)|0);
 $473 = HEAP16[$472>>1]|0;
 $474 = $473 << 16 >> 16;
 $475 = $p;
 $476 = (($475) + 4)|0;
 $477 = $0;
 $478 = (($477) + ($476<<3)|0);
 $479 = (($478) + 4|0);
 $480 = HEAP16[$479>>1]|0;
 $481 = $480 << 16 >> 16;
 $482 = (($474) + ($481))|0;
 $483 = $482&65535;
 $484 = $p;
 $485 = (($484) + 5)|0;
 $486 = $0;
 $487 = (($486) + ($485<<3)|0);
 HEAP16[$487>>1] = $483;
 $488 = $p;
 $489 = (($488) + 5)|0;
 $490 = $0;
 $491 = (($490) + ($489<<3)|0);
 $492 = HEAP16[$491>>1]|0;
 $493 = $p;
 $494 = (($493) + 7)|0;
 $495 = $0;
 $496 = (($495) + ($494<<3)|0);
 HEAP16[$496>>1] = $492;
 $497 = $temp2y;
 $498 = (($497|0) % 2)&-1;
 $eveny = $498;
 $499 = $4;
 $500 = $499&65535;
 $501 = $p;
 $502 = (($501) + 8)|0;
 $503 = $0;
 $504 = (($503) + ($502<<3)|0);
 HEAP16[$504>>1] = $500;
 $505 = $p;
 $506 = (($505) + 1)|0;
 $507 = $0;
 $508 = (($507) + ($506<<3)|0);
 $509 = HEAP16[$508>>1]|0;
 $510 = $p;
 $511 = (($510) + 9)|0;
 $512 = $0;
 $513 = (($512) + ($511<<3)|0);
 HEAP16[$513>>1] = $509;
 $514 = $4;
 $515 = $514&65535;
 $516 = $p;
 $517 = (($516) + 10)|0;
 $518 = $0;
 $519 = (($518) + ($517<<3)|0);
 HEAP16[$519>>1] = $515;
 $520 = $p;
 $521 = (($520) + 1)|0;
 $522 = $0;
 $523 = (($522) + ($521<<3)|0);
 $524 = HEAP16[$523>>1]|0;
 $525 = $p;
 $526 = (($525) + 11)|0;
 $527 = $0;
 $528 = (($527) + ($526<<3)|0);
 HEAP16[$528>>1] = $524;
 $529 = $5;
 $530 = $tempy;
 $531 = (($529) + ($530))|0;
 $532 = $531&65535;
 $533 = $p;
 $534 = (($533) + 8)|0;
 $535 = $0;
 $536 = (($535) + ($534<<3)|0);
 $537 = (($536) + 2|0);
 HEAP16[$537>>1] = $532;
 $538 = $p;
 $539 = (($538) + 8)|0;
 $540 = $0;
 $541 = (($540) + ($539<<3)|0);
 $542 = (($541) + 2|0);
 $543 = HEAP16[$542>>1]|0;
 $544 = $p;
 $545 = (($544) + 9)|0;
 $546 = $0;
 $547 = (($546) + ($545<<3)|0);
 $548 = (($547) + 2|0);
 HEAP16[$548>>1] = $543;
 $549 = $p;
 $550 = $0;
 $551 = (($550) + ($549<<3)|0);
 $552 = (($551) + 4|0);
 $553 = HEAP16[$552>>1]|0;
 $554 = $p;
 $555 = (($554) + 8)|0;
 $556 = $0;
 $557 = (($556) + ($555<<3)|0);
 $558 = (($557) + 4|0);
 HEAP16[$558>>1] = $553;
 $559 = $p;
 $560 = (($559) + 1)|0;
 $561 = $0;
 $562 = (($561) + ($560<<3)|0);
 $563 = (($562) + 4|0);
 $564 = HEAP16[$563>>1]|0;
 $565 = $p;
 $566 = (($565) + 9)|0;
 $567 = $0;
 $568 = (($567) + ($566<<3)|0);
 $569 = (($568) + 4|0);
 HEAP16[$569>>1] = $564;
 $570 = $p;
 $571 = $0;
 $572 = (($571) + ($570<<3)|0);
 $573 = (($572) + 4|0);
 $574 = HEAP16[$573>>1]|0;
 $575 = $p;
 $576 = (($575) + 10)|0;
 $577 = $0;
 $578 = (($577) + ($576<<3)|0);
 $579 = (($578) + 4|0);
 HEAP16[$579>>1] = $574;
 $580 = $p;
 $581 = (($580) + 1)|0;
 $582 = $0;
 $583 = (($582) + ($581<<3)|0);
 $584 = (($583) + 4|0);
 $585 = HEAP16[$584>>1]|0;
 $586 = $p;
 $587 = (($586) + 11)|0;
 $588 = $0;
 $589 = (($588) + ($587<<3)|0);
 $590 = (($589) + 4|0);
 HEAP16[$590>>1] = $585;
 $591 = $eveny;
 $592 = ($591|0)==(0);
 if ($592) {
  $593 = $temp2y;
  $594 = (($593|0) / 2)&-1;
  $595 = $594&65535;
  $596 = $p;
  $597 = (($596) + 8)|0;
  $598 = $0;
  $599 = (($598) + ($597<<3)|0);
  $600 = (($599) + 6|0);
  HEAP16[$600>>1] = $595;
  $601 = $p;
  $602 = (($601) + 8)|0;
  $603 = $0;
  $604 = (($603) + ($602<<3)|0);
  $605 = (($604) + 6|0);
  $606 = HEAP16[$605>>1]|0;
  $607 = $p;
  $608 = (($607) + 9)|0;
  $609 = $0;
  $610 = (($609) + ($608<<3)|0);
  $611 = (($610) + 6|0);
  HEAP16[$611>>1] = $606;
  $612 = $p;
  $613 = (($612) + 8)|0;
  $614 = $0;
  $615 = (($614) + ($613<<3)|0);
  $616 = (($615) + 6|0);
  $617 = HEAP16[$616>>1]|0;
  $618 = $p;
  $619 = (($618) + 10)|0;
  $620 = $0;
  $621 = (($620) + ($619<<3)|0);
  $622 = (($621) + 6|0);
  HEAP16[$622>>1] = $617;
  $623 = $p;
  $624 = (($623) + 8)|0;
  $625 = $0;
  $626 = (($625) + ($624<<3)|0);
  $627 = (($626) + 6|0);
  $628 = HEAP16[$627>>1]|0;
  $629 = $p;
  $630 = (($629) + 11)|0;
  $631 = $0;
  $632 = (($631) + ($630<<3)|0);
  $633 = (($632) + 6|0);
  HEAP16[$633>>1] = $628;
 } else {
  $634 = $temp2y;
  $635 = (($634) + 1)|0;
  $636 = (($635|0) / 2)&-1;
  $637 = $636&65535;
  $638 = $p;
  $639 = (($638) + 10)|0;
  $640 = $0;
  $641 = (($640) + ($639<<3)|0);
  $642 = (($641) + 6|0);
  HEAP16[$642>>1] = $637;
  $643 = $p;
  $644 = (($643) + 10)|0;
  $645 = $0;
  $646 = (($645) + ($644<<3)|0);
  $647 = (($646) + 6|0);
  $648 = HEAP16[$647>>1]|0;
  $649 = $p;
  $650 = (($649) + 11)|0;
  $651 = $0;
  $652 = (($651) + ($650<<3)|0);
  $653 = (($652) + 6|0);
  HEAP16[$653>>1] = $648;
  $654 = $p;
  $655 = (($654) + 10)|0;
  $656 = $0;
  $657 = (($656) + ($655<<3)|0);
  $658 = (($657) + 6|0);
  $659 = HEAP16[$658>>1]|0;
  $660 = $659 << 16 >> 16;
  $661 = (($660) - 1)|0;
  $662 = $661&65535;
  $663 = $p;
  $664 = (($663) + 8)|0;
  $665 = $0;
  $666 = (($665) + ($664<<3)|0);
  $667 = (($666) + 6|0);
  HEAP16[$667>>1] = $662;
  $668 = $p;
  $669 = (($668) + 8)|0;
  $670 = $0;
  $671 = (($670) + ($669<<3)|0);
  $672 = (($671) + 6|0);
  $673 = HEAP16[$672>>1]|0;
  $674 = $p;
  $675 = (($674) + 9)|0;
  $676 = $0;
  $677 = (($676) + ($675<<3)|0);
  $678 = (($677) + 6|0);
  HEAP16[$678>>1] = $673;
 }
 $679 = $p;
 $680 = (($679) + 8)|0;
 $681 = $0;
 $682 = (($681) + ($680<<3)|0);
 $683 = (($682) + 2|0);
 $684 = HEAP16[$683>>1]|0;
 $685 = $684 << 16 >> 16;
 $686 = $p;
 $687 = (($686) + 8)|0;
 $688 = $0;
 $689 = (($688) + ($687<<3)|0);
 $690 = (($689) + 6|0);
 $691 = HEAP16[$690>>1]|0;
 $692 = $691 << 16 >> 16;
 $693 = (($685) + ($692))|0;
 $694 = $693&65535;
 $695 = $p;
 $696 = (($695) + 10)|0;
 $697 = $0;
 $698 = (($697) + ($696<<3)|0);
 $699 = (($698) + 2|0);
 HEAP16[$699>>1] = $694;
 $700 = $p;
 $701 = (($700) + 10)|0;
 $702 = $0;
 $703 = (($702) + ($701<<3)|0);
 $704 = (($703) + 2|0);
 $705 = HEAP16[$704>>1]|0;
 $706 = $p;
 $707 = (($706) + 11)|0;
 $708 = $0;
 $709 = (($708) + ($707<<3)|0);
 $710 = (($709) + 2|0);
 HEAP16[$710>>1] = $705;
 $711 = $p;
 $712 = (($711) + 4)|0;
 $713 = $0;
 $714 = (($713) + ($712<<3)|0);
 $715 = HEAP16[$714>>1]|0;
 $716 = $p;
 $717 = (($716) + 12)|0;
 $718 = $0;
 $719 = (($718) + ($717<<3)|0);
 HEAP16[$719>>1] = $715;
 $720 = $p;
 $721 = (($720) + 5)|0;
 $722 = $0;
 $723 = (($722) + ($721<<3)|0);
 $724 = HEAP16[$723>>1]|0;
 $725 = $p;
 $726 = (($725) + 13)|0;
 $727 = $0;
 $728 = (($727) + ($726<<3)|0);
 HEAP16[$728>>1] = $724;
 $729 = $p;
 $730 = (($729) + 4)|0;
 $731 = $0;
 $732 = (($731) + ($730<<3)|0);
 $733 = HEAP16[$732>>1]|0;
 $734 = $p;
 $735 = (($734) + 14)|0;
 $736 = $0;
 $737 = (($736) + ($735<<3)|0);
 HEAP16[$737>>1] = $733;
 $738 = $p;
 $739 = (($738) + 5)|0;
 $740 = $0;
 $741 = (($740) + ($739<<3)|0);
 $742 = HEAP16[$741>>1]|0;
 $743 = $p;
 $744 = (($743) + 15)|0;
 $745 = $0;
 $746 = (($745) + ($744<<3)|0);
 HEAP16[$746>>1] = $742;
 $747 = $p;
 $748 = (($747) + 8)|0;
 $749 = $0;
 $750 = (($749) + ($748<<3)|0);
 $751 = (($750) + 2|0);
 $752 = HEAP16[$751>>1]|0;
 $753 = $p;
 $754 = (($753) + 12)|0;
 $755 = $0;
 $756 = (($755) + ($754<<3)|0);
 $757 = (($756) + 2|0);
 HEAP16[$757>>1] = $752;
 $758 = $p;
 $759 = (($758) + 8)|0;
 $760 = $0;
 $761 = (($760) + ($759<<3)|0);
 $762 = (($761) + 2|0);
 $763 = HEAP16[$762>>1]|0;
 $764 = $p;
 $765 = (($764) + 13)|0;
 $766 = $0;
 $767 = (($766) + ($765<<3)|0);
 $768 = (($767) + 2|0);
 HEAP16[$768>>1] = $763;
 $769 = $p;
 $770 = (($769) + 10)|0;
 $771 = $0;
 $772 = (($771) + ($770<<3)|0);
 $773 = (($772) + 2|0);
 $774 = HEAP16[$773>>1]|0;
 $775 = $p;
 $776 = (($775) + 14)|0;
 $777 = $0;
 $778 = (($777) + ($776<<3)|0);
 $779 = (($778) + 2|0);
 HEAP16[$779>>1] = $774;
 $780 = $p;
 $781 = (($780) + 10)|0;
 $782 = $0;
 $783 = (($782) + ($781<<3)|0);
 $784 = (($783) + 2|0);
 $785 = HEAP16[$784>>1]|0;
 $786 = $p;
 $787 = (($786) + 15)|0;
 $788 = $0;
 $789 = (($788) + ($787<<3)|0);
 $790 = (($789) + 2|0);
 HEAP16[$790>>1] = $785;
 $791 = $p;
 $792 = (($791) + 4)|0;
 $793 = $0;
 $794 = (($793) + ($792<<3)|0);
 $795 = (($794) + 4|0);
 $796 = HEAP16[$795>>1]|0;
 $797 = $p;
 $798 = (($797) + 12)|0;
 $799 = $0;
 $800 = (($799) + ($798<<3)|0);
 $801 = (($800) + 4|0);
 HEAP16[$801>>1] = $796;
 $802 = $p;
 $803 = (($802) + 5)|0;
 $804 = $0;
 $805 = (($804) + ($803<<3)|0);
 $806 = (($805) + 4|0);
 $807 = HEAP16[$806>>1]|0;
 $808 = $p;
 $809 = (($808) + 13)|0;
 $810 = $0;
 $811 = (($810) + ($809<<3)|0);
 $812 = (($811) + 4|0);
 HEAP16[$812>>1] = $807;
 $813 = $p;
 $814 = (($813) + 4)|0;
 $815 = $0;
 $816 = (($815) + ($814<<3)|0);
 $817 = (($816) + 4|0);
 $818 = HEAP16[$817>>1]|0;
 $819 = $p;
 $820 = (($819) + 14)|0;
 $821 = $0;
 $822 = (($821) + ($820<<3)|0);
 $823 = (($822) + 4|0);
 HEAP16[$823>>1] = $818;
 $824 = $p;
 $825 = (($824) + 5)|0;
 $826 = $0;
 $827 = (($826) + ($825<<3)|0);
 $828 = (($827) + 4|0);
 $829 = HEAP16[$828>>1]|0;
 $830 = $p;
 $831 = (($830) + 15)|0;
 $832 = $0;
 $833 = (($832) + ($831<<3)|0);
 $834 = (($833) + 4|0);
 HEAP16[$834>>1] = $829;
 $835 = $p;
 $836 = (($835) + 8)|0;
 $837 = $0;
 $838 = (($837) + ($836<<3)|0);
 $839 = (($838) + 6|0);
 $840 = HEAP16[$839>>1]|0;
 $841 = $p;
 $842 = (($841) + 12)|0;
 $843 = $0;
 $844 = (($843) + ($842<<3)|0);
 $845 = (($844) + 6|0);
 HEAP16[$845>>1] = $840;
 $846 = $p;
 $847 = (($846) + 8)|0;
 $848 = $0;
 $849 = (($848) + ($847<<3)|0);
 $850 = (($849) + 6|0);
 $851 = HEAP16[$850>>1]|0;
 $852 = $p;
 $853 = (($852) + 13)|0;
 $854 = $0;
 $855 = (($854) + ($853<<3)|0);
 $856 = (($855) + 6|0);
 HEAP16[$856>>1] = $851;
 $857 = $p;
 $858 = (($857) + 10)|0;
 $859 = $0;
 $860 = (($859) + ($858<<3)|0);
 $861 = (($860) + 6|0);
 $862 = HEAP16[$861>>1]|0;
 $863 = $p;
 $864 = (($863) + 14)|0;
 $865 = $0;
 $866 = (($865) + ($864<<3)|0);
 $867 = (($866) + 6|0);
 HEAP16[$867>>1] = $862;
 $868 = $p;
 $869 = (($868) + 10)|0;
 $870 = $0;
 $871 = (($870) + ($869<<3)|0);
 $872 = (($871) + 6|0);
 $873 = HEAP16[$872>>1]|0;
 $874 = $p;
 $875 = (($874) + 15)|0;
 $876 = $0;
 $877 = (($876) + ($875<<3)|0);
 $878 = (($877) + 6|0);
 HEAP16[$878>>1] = $873;
 STACKTOP = sp;return;
}
function _q_tree4($q_tree,$start,$lenx,$leny,$x,$y) {
 $q_tree = $q_tree|0;
 $start = $start|0;
 $lenx = $lenx|0;
 $leny = $leny|0;
 $x = $x|0;
 $y = $y|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $evenx = 0, $eveny = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $q_tree;
 $1 = $start;
 $2 = $lenx;
 $3 = $leny;
 $4 = $x;
 $5 = $y;
 $6 = $1;
 $p = $6;
 $7 = $2;
 $8 = (($7|0) % 2)&-1;
 $evenx = $8;
 $9 = $3;
 $10 = (($9|0) % 2)&-1;
 $eveny = $10;
 $11 = $4;
 $12 = $11&65535;
 $13 = $p;
 $14 = $0;
 $15 = (($14) + ($13<<3)|0);
 HEAP16[$15>>1] = $12;
 $16 = $4;
 $17 = $16&65535;
 $18 = $p;
 $19 = (($18) + 2)|0;
 $20 = $0;
 $21 = (($20) + ($19<<3)|0);
 HEAP16[$21>>1] = $17;
 $22 = $5;
 $23 = $22&65535;
 $24 = $p;
 $25 = $0;
 $26 = (($25) + ($24<<3)|0);
 $27 = (($26) + 2|0);
 HEAP16[$27>>1] = $23;
 $28 = $5;
 $29 = $28&65535;
 $30 = $p;
 $31 = (($30) + 1)|0;
 $32 = $0;
 $33 = (($32) + ($31<<3)|0);
 $34 = (($33) + 2|0);
 HEAP16[$34>>1] = $29;
 $35 = $evenx;
 $36 = ($35|0)==(0);
 if ($36) {
  $37 = $2;
  $38 = (($37|0) / 2)&-1;
  $39 = $38&65535;
  $40 = $p;
  $41 = $0;
  $42 = (($41) + ($40<<3)|0);
  $43 = (($42) + 4|0);
  HEAP16[$43>>1] = $39;
  $44 = $p;
  $45 = $0;
  $46 = (($45) + ($44<<3)|0);
  $47 = (($46) + 4|0);
  $48 = HEAP16[$47>>1]|0;
  $49 = $p;
  $50 = (($49) + 1)|0;
  $51 = $0;
  $52 = (($51) + ($50<<3)|0);
  $53 = (($52) + 4|0);
  HEAP16[$53>>1] = $48;
  $54 = $p;
  $55 = $0;
  $56 = (($55) + ($54<<3)|0);
  $57 = (($56) + 4|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $p;
  $60 = (($59) + 2)|0;
  $61 = $0;
  $62 = (($61) + ($60<<3)|0);
  $63 = (($62) + 4|0);
  HEAP16[$63>>1] = $58;
  $64 = $p;
  $65 = $0;
  $66 = (($65) + ($64<<3)|0);
  $67 = (($66) + 4|0);
  $68 = HEAP16[$67>>1]|0;
  $69 = $p;
  $70 = (($69) + 3)|0;
  $71 = $0;
  $72 = (($71) + ($70<<3)|0);
  $73 = (($72) + 4|0);
  HEAP16[$73>>1] = $68;
 } else {
  $74 = $2;
  $75 = (($74) + 1)|0;
  $76 = (($75|0) / 2)&-1;
  $77 = $76&65535;
  $78 = $p;
  $79 = $0;
  $80 = (($79) + ($78<<3)|0);
  $81 = (($80) + 4|0);
  HEAP16[$81>>1] = $77;
  $82 = $p;
  $83 = $0;
  $84 = (($83) + ($82<<3)|0);
  $85 = (($84) + 4|0);
  $86 = HEAP16[$85>>1]|0;
  $87 = $86 << 16 >> 16;
  $88 = (($87) - 1)|0;
  $89 = $88&65535;
  $90 = $p;
  $91 = (($90) + 1)|0;
  $92 = $0;
  $93 = (($92) + ($91<<3)|0);
  $94 = (($93) + 4|0);
  HEAP16[$94>>1] = $89;
  $95 = $p;
  $96 = $0;
  $97 = (($96) + ($95<<3)|0);
  $98 = (($97) + 4|0);
  $99 = HEAP16[$98>>1]|0;
  $100 = $p;
  $101 = (($100) + 2)|0;
  $102 = $0;
  $103 = (($102) + ($101<<3)|0);
  $104 = (($103) + 4|0);
  HEAP16[$104>>1] = $99;
  $105 = $p;
  $106 = (($105) + 1)|0;
  $107 = $0;
  $108 = (($107) + ($106<<3)|0);
  $109 = (($108) + 4|0);
  $110 = HEAP16[$109>>1]|0;
  $111 = $p;
  $112 = (($111) + 3)|0;
  $113 = $0;
  $114 = (($113) + ($112<<3)|0);
  $115 = (($114) + 4|0);
  HEAP16[$115>>1] = $110;
 }
 $116 = $4;
 $117 = $p;
 $118 = $0;
 $119 = (($118) + ($117<<3)|0);
 $120 = (($119) + 4|0);
 $121 = HEAP16[$120>>1]|0;
 $122 = $121 << 16 >> 16;
 $123 = (($116) + ($122))|0;
 $124 = $123&65535;
 $125 = $p;
 $126 = (($125) + 1)|0;
 $127 = $0;
 $128 = (($127) + ($126<<3)|0);
 HEAP16[$128>>1] = $124;
 $129 = $p;
 $130 = (($129) + 1)|0;
 $131 = $0;
 $132 = (($131) + ($130<<3)|0);
 $133 = HEAP16[$132>>1]|0;
 $134 = $p;
 $135 = (($134) + 3)|0;
 $136 = $0;
 $137 = (($136) + ($135<<3)|0);
 HEAP16[$137>>1] = $133;
 $138 = $eveny;
 $139 = ($138|0)==(0);
 if ($139) {
  $140 = $3;
  $141 = (($140|0) / 2)&-1;
  $142 = $141&65535;
  $143 = $p;
  $144 = $0;
  $145 = (($144) + ($143<<3)|0);
  $146 = (($145) + 6|0);
  HEAP16[$146>>1] = $142;
  $147 = $p;
  $148 = $0;
  $149 = (($148) + ($147<<3)|0);
  $150 = (($149) + 6|0);
  $151 = HEAP16[$150>>1]|0;
  $152 = $p;
  $153 = (($152) + 1)|0;
  $154 = $0;
  $155 = (($154) + ($153<<3)|0);
  $156 = (($155) + 6|0);
  HEAP16[$156>>1] = $151;
  $157 = $p;
  $158 = $0;
  $159 = (($158) + ($157<<3)|0);
  $160 = (($159) + 6|0);
  $161 = HEAP16[$160>>1]|0;
  $162 = $p;
  $163 = (($162) + 2)|0;
  $164 = $0;
  $165 = (($164) + ($163<<3)|0);
  $166 = (($165) + 6|0);
  HEAP16[$166>>1] = $161;
  $167 = $p;
  $168 = $0;
  $169 = (($168) + ($167<<3)|0);
  $170 = (($169) + 6|0);
  $171 = HEAP16[$170>>1]|0;
  $172 = $p;
  $173 = (($172) + 3)|0;
  $174 = $0;
  $175 = (($174) + ($173<<3)|0);
  $176 = (($175) + 6|0);
  HEAP16[$176>>1] = $171;
  $219 = $5;
  $220 = $p;
  $221 = $0;
  $222 = (($221) + ($220<<3)|0);
  $223 = (($222) + 6|0);
  $224 = HEAP16[$223>>1]|0;
  $225 = $224 << 16 >> 16;
  $226 = (($219) + ($225))|0;
  $227 = $226&65535;
  $228 = $p;
  $229 = (($228) + 2)|0;
  $230 = $0;
  $231 = (($230) + ($229<<3)|0);
  $232 = (($231) + 2|0);
  HEAP16[$232>>1] = $227;
  $233 = $p;
  $234 = (($233) + 2)|0;
  $235 = $0;
  $236 = (($235) + ($234<<3)|0);
  $237 = (($236) + 2|0);
  $238 = HEAP16[$237>>1]|0;
  $239 = $p;
  $240 = (($239) + 3)|0;
  $241 = $0;
  $242 = (($241) + ($240<<3)|0);
  $243 = (($242) + 2|0);
  HEAP16[$243>>1] = $238;
  STACKTOP = sp;return;
 } else {
  $177 = $3;
  $178 = (($177) + 1)|0;
  $179 = (($178|0) / 2)&-1;
  $180 = $179&65535;
  $181 = $p;
  $182 = $0;
  $183 = (($182) + ($181<<3)|0);
  $184 = (($183) + 6|0);
  HEAP16[$184>>1] = $180;
  $185 = $p;
  $186 = $0;
  $187 = (($186) + ($185<<3)|0);
  $188 = (($187) + 6|0);
  $189 = HEAP16[$188>>1]|0;
  $190 = $p;
  $191 = (($190) + 1)|0;
  $192 = $0;
  $193 = (($192) + ($191<<3)|0);
  $194 = (($193) + 6|0);
  HEAP16[$194>>1] = $189;
  $195 = $p;
  $196 = $0;
  $197 = (($196) + ($195<<3)|0);
  $198 = (($197) + 6|0);
  $199 = HEAP16[$198>>1]|0;
  $200 = $199 << 16 >> 16;
  $201 = (($200) - 1)|0;
  $202 = $201&65535;
  $203 = $p;
  $204 = (($203) + 2)|0;
  $205 = $0;
  $206 = (($205) + ($204<<3)|0);
  $207 = (($206) + 6|0);
  HEAP16[$207>>1] = $202;
  $208 = $p;
  $209 = (($208) + 2)|0;
  $210 = $0;
  $211 = (($210) + ($209<<3)|0);
  $212 = (($211) + 6|0);
  $213 = HEAP16[$212>>1]|0;
  $214 = $p;
  $215 = (($214) + 3)|0;
  $216 = $0;
  $217 = (($216) + ($215<<3)|0);
  $218 = (($217) + 6|0);
  HEAP16[$218>>1] = $213;
  $219 = $5;
  $220 = $p;
  $221 = $0;
  $222 = (($221) + ($220<<3)|0);
  $223 = (($222) + 6|0);
  $224 = HEAP16[$223>>1]|0;
  $225 = $224 << 16 >> 16;
  $226 = (($219) + ($225))|0;
  $227 = $226&65535;
  $228 = $p;
  $229 = (($228) + 2)|0;
  $230 = $0;
  $231 = (($230) + ($229<<3)|0);
  $232 = (($231) + 2|0);
  HEAP16[$232>>1] = $227;
  $233 = $p;
  $234 = (($233) + 2)|0;
  $235 = $0;
  $236 = (($235) + ($234<<3)|0);
  $237 = (($236) + 2|0);
  $238 = HEAP16[$237>>1]|0;
  $239 = $p;
  $240 = (($239) + 3)|0;
  $241 = $0;
  $242 = (($241) + ($240<<3)|0);
  $243 = (($242) + 2|0);
  HEAP16[$243>>1] = $238;
  STACKTOP = sp;return;
 }
}
function _conv_img_2_flt_ret($fip,$m_shift,$r_scale,$data,$num_pix) {
 $fip = $fip|0;
 $m_shift = $m_shift|0;
 $r_scale = $r_scale|0;
 $data = $data|0;
 $num_pix = $num_pix|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0, $55 = 0, $56 = 0.0, $57 = 0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0.0, $62 = 0;
 var $63 = 0.0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0, $73 = 0.0, $74 = 0.0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0.0, $83 = 0, $84 = 0.0, $85 = 0.0, $86 = 0, $87 = 0.0, $88 = 0.0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $cnt = 0, $high = 0, $high_diff = 0.0, $low = 0, $low_diff = 0.0;
 var $mean = 0.0, $overflow = 0, $sum = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $1 = $fip;
 $2 = $m_shift;
 $3 = $r_scale;
 $4 = $data;
 $5 = $num_pix;
 $sum = 0;
 $overflow = 0;
 $low = 255;
 $high = 0;
 $cnt = 0;
 while(1) {
  $6 = $cnt;
  $7 = $5;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = $cnt;
  $10 = $4;
  $11 = (($10) + ($9)|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = $high;
  $15 = ($13|0)>($14|0);
  if ($15) {
   $16 = $cnt;
   $17 = $4;
   $18 = (($17) + ($16)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $high = $20;
  }
  $21 = $cnt;
  $22 = $4;
  $23 = (($22) + ($21)|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = $low;
  $27 = ($25|0)<($26|0);
  if ($27) {
   $28 = $cnt;
   $29 = $4;
   $30 = (($29) + ($28)|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31&255;
   $low = $32;
  }
  $33 = $cnt;
  $34 = $4;
  $35 = (($34) + ($33)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = $sum;
  $39 = (($38) + ($37))|0;
  $sum = $39;
  $40 = $sum;
  $41 = $overflow;
  $42 = ($40>>>0)<($41>>>0);
  if ($42) {
   label = 8;
   break;
  }
  $45 = $sum;
  $overflow = $45;
  $46 = $cnt;
  $47 = (($46) + 1)|0;
  $cnt = $47;
 }
 if ((label|0) == 8) {
  $43 = HEAP32[_stderr>>2]|0;
  $44 = $cnt;
  HEAP32[$vararg_buffer>>2] = $44;
  (_fprintf(($43|0),(11328|0),($vararg_buffer|0))|0);
  $0 = -91;
  $94 = $0;
  STACKTOP = sp;return ($94|0);
 }
 $48 = $sum;
 $49 = (+($48>>>0));
 $50 = $5;
 $51 = (+($50|0));
 $52 = $49 / $51;
 $mean = $52;
 $53 = $mean;
 $54 = $2;
 HEAPF32[$54>>2] = $53;
 $55 = $2;
 $56 = +HEAPF32[$55>>2];
 $57 = $low;
 $58 = (+($57|0));
 $59 = $56 - $58;
 $low_diff = $59;
 $60 = $high;
 $61 = (+($60|0));
 $62 = $2;
 $63 = +HEAPF32[$62>>2];
 $64 = $61 - $63;
 $high_diff = $64;
 $65 = $low_diff;
 $66 = $high_diff;
 $67 = $65 >= $66;
 if ($67) {
  $68 = $low_diff;
  $69 = $3;
  HEAPF32[$69>>2] = $68;
 } else {
  $70 = $high_diff;
  $71 = $3;
  HEAPF32[$71>>2] = $70;
 }
 $72 = $3;
 $73 = +HEAPF32[$72>>2];
 $74 = $73 / 128.0;
 HEAPF32[$72>>2] = $74;
 $cnt = 0;
 while(1) {
  $75 = $cnt;
  $76 = $5;
  $77 = ($75|0)<($76|0);
  if (!($77)) {
   break;
  }
  $78 = $cnt;
  $79 = $4;
  $80 = (($79) + ($78)|0);
  $81 = HEAP8[$80>>0]|0;
  $82 = (+($81&255));
  $83 = $2;
  $84 = +HEAPF32[$83>>2];
  $85 = $82 - $84;
  $86 = $3;
  $87 = +HEAPF32[$86>>2];
  $88 = $85 / $87;
  $89 = $cnt;
  $90 = $1;
  $91 = (($90) + ($89<<2)|0);
  HEAPF32[$91>>2] = $88;
  $92 = $cnt;
  $93 = (($92) + 1)|0;
  $cnt = $93;
 }
 $0 = 0;
 $94 = $0;
 STACKTOP = sp;return ($94|0);
}
function _variance($quant_vals,$q_tree,$q_treelen,$fip,$width,$height) {
 $quant_vals = $quant_vals|0;
 $q_tree = $q_tree|0;
 $q_treelen = $q_treelen|0;
 $fip = $fip|0;
 $width = $width|0;
 $height = $height|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0.0, $102 = 0, $103 = 0, $104 = 0, $105 = 0.0, $106 = 0.0, $107 = 0.0, $108 = 0.0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0.0, $118 = 0.0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0, $122 = 0.0, $123 = 0.0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0.0, $163 = 0.0, $164 = 0.0, $165 = 0, $166 = 0.0, $167 = 0, $168 = 0.0, $169 = 0.0, $17 = 0;
 var $170 = 0.0, $171 = 0.0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0.0, $184 = 0.0, $185 = 0.0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0.0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0.0, $193 = 0.0, $194 = 0.0, $195 = 0, $196 = 0, $197 = 0, $198 = 0.0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0.0, $201 = 0.0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0.0, $272 = 0.0, $273 = 0.0, $274 = 0, $275 = 0.0, $276 = 0, $277 = 0.0, $278 = 0.0;
 var $279 = 0.0, $28 = 0, $280 = 0.0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0.0, $293 = 0.0, $294 = 0.0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0.0, $299 = 0.0, $3 = 0, $30 = 0, $300 = 0.0, $301 = 0.0, $302 = 0.0, $303 = 0.0, $304 = 0, $305 = 0, $306 = 0, $307 = 0.0, $308 = 0.0, $309 = 0.0, $31 = 0, $310 = 0.0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0, $73 = 0.0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0.0, $78 = 0.0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0.0, $91 = 0.0, $92 = 0.0, $93 = 0, $94 = 0, $95 = 0, $96 = 0.0, $97 = 0.0, $98 = 0.0, $99 = 0.0, $col = 0, $cvr = 0, $fp = 0;
 var $lenx = 0, $leny = 0, $row = 0, $skipx = 0, $skipy = 0, $ssq = 0.0, $sum2 = 0.0, $sum_pix = 0.0, $vsum = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = $quant_vals;
 $1 = $q_tree;
 $2 = $q_treelen;
 $3 = $fip;
 $4 = $width;
 $5 = $height;
 $lenx = 0;
 $leny = 0;
 $vsum = 0.0;
 $cvr = 0;
 while(1) {
  $6 = $cvr;
  $7 = ($6|0)<(4);
  if (!($7)) {
   break;
  }
  $8 = $3;
  $9 = $cvr;
  $10 = $1;
  $11 = (($10) + ($9<<3)|0);
  $12 = (($11) + 2|0);
  $13 = HEAP16[$12>>1]|0;
  $14 = $13 << 16 >> 16;
  $15 = $4;
  $16 = Math_imul($14, $15)|0;
  $17 = (($8) + ($16<<2)|0);
  $18 = $cvr;
  $19 = $1;
  $20 = (($19) + ($18<<3)|0);
  $21 = HEAP16[$20>>1]|0;
  $22 = $21 << 16 >> 16;
  $23 = (($17) + ($22<<2)|0);
  $fp = $23;
  $ssq = 0.0;
  $sum_pix = 0.0;
  $24 = $cvr;
  $25 = $1;
  $26 = (($25) + ($24<<3)|0);
  $27 = (($26) + 4|0);
  $28 = HEAP16[$27>>1]|0;
  $29 = $28 << 16 >> 16;
  $30 = (($29|0) / 8)&-1;
  $skipx = $30;
  $31 = $cvr;
  $32 = $1;
  $33 = (($32) + ($31<<3)|0);
  $34 = (($33) + 6|0);
  $35 = HEAP16[$34>>1]|0;
  $36 = $35 << 16 >> 16;
  $37 = ($36*9)|0;
  $38 = (($37|0) / 32)&-1;
  $skipy = $38;
  $39 = $cvr;
  $40 = $1;
  $41 = (($40) + ($39<<3)|0);
  $42 = (($41) + 4|0);
  $43 = HEAP16[$42>>1]|0;
  $44 = $43 << 16 >> 16;
  $45 = ($44*3)|0;
  $46 = (($45|0) / 4)&-1;
  $lenx = $46;
  $47 = $cvr;
  $48 = $1;
  $49 = (($48) + ($47<<3)|0);
  $50 = (($49) + 6|0);
  $51 = HEAP16[$50>>1]|0;
  $52 = $51 << 16 >> 16;
  $53 = ($52*7)|0;
  $54 = (($53|0) / 16)&-1;
  $leny = $54;
  $55 = $skipy;
  $56 = $4;
  $57 = Math_imul($55, $56)|0;
  $58 = $skipx;
  $59 = (($57) + ($58))|0;
  $60 = $fp;
  $61 = (($60) + ($59<<2)|0);
  $fp = $61;
  $row = 0;
  while(1) {
   $62 = $row;
   $63 = $leny;
   $64 = ($62|0)<($63|0);
   if (!($64)) {
    break;
   }
   $col = 0;
   while(1) {
    $65 = $col;
    $66 = $lenx;
    $67 = ($65|0)<($66|0);
    if (!($67)) {
     break;
    }
    $68 = $fp;
    $69 = +HEAPF32[$68>>2];
    $70 = $sum_pix;
    $71 = $70 + $69;
    $sum_pix = $71;
    $72 = $fp;
    $73 = +HEAPF32[$72>>2];
    $74 = $fp;
    $75 = +HEAPF32[$74>>2];
    $76 = $73 * $75;
    $77 = $ssq;
    $78 = $77 + $76;
    $ssq = $78;
    $79 = $fp;
    $80 = (($79) + 4|0);
    $fp = $80;
    $81 = $col;
    $82 = (($81) + 1)|0;
    $col = $82;
   }
   $83 = $row;
   $84 = (($83) + 1)|0;
   $row = $84;
   $85 = $4;
   $86 = $lenx;
   $87 = (($85) - ($86))|0;
   $88 = $fp;
   $89 = (($88) + ($87<<2)|0);
   $fp = $89;
  }
  $90 = $sum_pix;
  $91 = $sum_pix;
  $92 = $90 * $91;
  $93 = $lenx;
  $94 = $leny;
  $95 = Math_imul($93, $94)|0;
  $96 = (+($95|0));
  $97 = $92 / $96;
  $sum2 = $97;
  $98 = $ssq;
  $99 = $sum2;
  $100 = $98 - $99;
  $101 = $100;
  $102 = $lenx;
  $103 = $leny;
  $104 = Math_imul($102, $103)|0;
  $105 = (+($104|0));
  $106 = $105 - 1.0;
  $107 = $101 / $106;
  $108 = $107;
  $109 = $cvr;
  $110 = $0;
  $111 = (($110) + 780|0);
  $112 = (($111) + ($109<<2)|0);
  HEAPF32[$112>>2] = $108;
  $113 = $cvr;
  $114 = $0;
  $115 = (($114) + 780|0);
  $116 = (($115) + ($113<<2)|0);
  $117 = +HEAPF32[$116>>2];
  $118 = $vsum;
  $119 = $118 + $117;
  $vsum = $119;
  $120 = $cvr;
  $121 = (($120) + 1)|0;
  $cvr = $121;
 }
 $122 = $vsum;
 $123 = $122;
 $124 = $123 < 2.0E+4;
 if ($124) {
  $cvr = 0;
  while(1) {
   $125 = $cvr;
   $126 = ($125|0)<(60);
   if (!($126)) {
    break;
   }
   $127 = $3;
   $128 = $cvr;
   $129 = $1;
   $130 = (($129) + ($128<<3)|0);
   $131 = (($130) + 2|0);
   $132 = HEAP16[$131>>1]|0;
   $133 = $132 << 16 >> 16;
   $134 = $4;
   $135 = Math_imul($133, $134)|0;
   $136 = (($127) + ($135<<2)|0);
   $137 = $cvr;
   $138 = $1;
   $139 = (($138) + ($137<<3)|0);
   $140 = HEAP16[$139>>1]|0;
   $141 = $140 << 16 >> 16;
   $142 = (($136) + ($141<<2)|0);
   $fp = $142;
   $ssq = 0.0;
   $sum_pix = 0.0;
   $143 = $cvr;
   $144 = $1;
   $145 = (($144) + ($143<<3)|0);
   $146 = (($145) + 4|0);
   $147 = HEAP16[$146>>1]|0;
   $148 = $147 << 16 >> 16;
   $lenx = $148;
   $149 = $cvr;
   $150 = $1;
   $151 = (($150) + ($149<<3)|0);
   $152 = (($151) + 6|0);
   $153 = HEAP16[$152>>1]|0;
   $154 = $153 << 16 >> 16;
   $leny = $154;
   $row = 0;
   while(1) {
    $155 = $row;
    $156 = $leny;
    $157 = ($155|0)<($156|0);
    if (!($157)) {
     break;
    }
    $col = 0;
    while(1) {
     $158 = $col;
     $159 = $lenx;
     $160 = ($158|0)<($159|0);
     if (!($160)) {
      break;
     }
     $161 = $fp;
     $162 = +HEAPF32[$161>>2];
     $163 = $sum_pix;
     $164 = $163 + $162;
     $sum_pix = $164;
     $165 = $fp;
     $166 = +HEAPF32[$165>>2];
     $167 = $fp;
     $168 = +HEAPF32[$167>>2];
     $169 = $166 * $168;
     $170 = $ssq;
     $171 = $170 + $169;
     $ssq = $171;
     $172 = $fp;
     $173 = (($172) + 4|0);
     $fp = $173;
     $174 = $col;
     $175 = (($174) + 1)|0;
     $col = $175;
    }
    $176 = $row;
    $177 = (($176) + 1)|0;
    $row = $177;
    $178 = $4;
    $179 = $lenx;
    $180 = (($178) - ($179))|0;
    $181 = $fp;
    $182 = (($181) + ($180<<2)|0);
    $fp = $182;
   }
   $183 = $sum_pix;
   $184 = $sum_pix;
   $185 = $183 * $184;
   $186 = $lenx;
   $187 = $leny;
   $188 = Math_imul($186, $187)|0;
   $189 = (+($188|0));
   $190 = $185 / $189;
   $sum2 = $190;
   $191 = $ssq;
   $192 = $sum2;
   $193 = $191 - $192;
   $194 = $193;
   $195 = $lenx;
   $196 = $leny;
   $197 = Math_imul($195, $196)|0;
   $198 = (+($197|0));
   $199 = $198 - 1.0;
   $200 = $194 / $199;
   $201 = $200;
   $202 = $cvr;
   $203 = $0;
   $204 = (($203) + 780|0);
   $205 = (($204) + ($202<<2)|0);
   HEAPF32[$205>>2] = $201;
   $206 = $cvr;
   $207 = (($206) + 1)|0;
   $cvr = $207;
  }
  STACKTOP = sp;return;
 } else {
  $cvr = 4;
  while(1) {
   $208 = $cvr;
   $209 = ($208|0)<(60);
   if (!($209)) {
    break;
   }
   $210 = $3;
   $211 = $cvr;
   $212 = $1;
   $213 = (($212) + ($211<<3)|0);
   $214 = (($213) + 2|0);
   $215 = HEAP16[$214>>1]|0;
   $216 = $215 << 16 >> 16;
   $217 = $4;
   $218 = Math_imul($216, $217)|0;
   $219 = (($210) + ($218<<2)|0);
   $220 = $cvr;
   $221 = $1;
   $222 = (($221) + ($220<<3)|0);
   $223 = HEAP16[$222>>1]|0;
   $224 = $223 << 16 >> 16;
   $225 = (($219) + ($224<<2)|0);
   $fp = $225;
   $ssq = 0.0;
   $sum_pix = 0.0;
   $226 = $cvr;
   $227 = $1;
   $228 = (($227) + ($226<<3)|0);
   $229 = (($228) + 4|0);
   $230 = HEAP16[$229>>1]|0;
   $231 = $230 << 16 >> 16;
   $232 = (($231|0) / 8)&-1;
   $skipx = $232;
   $233 = $cvr;
   $234 = $1;
   $235 = (($234) + ($233<<3)|0);
   $236 = (($235) + 6|0);
   $237 = HEAP16[$236>>1]|0;
   $238 = $237 << 16 >> 16;
   $239 = ($238*9)|0;
   $240 = (($239|0) / 32)&-1;
   $skipy = $240;
   $241 = $cvr;
   $242 = $1;
   $243 = (($242) + ($241<<3)|0);
   $244 = (($243) + 4|0);
   $245 = HEAP16[$244>>1]|0;
   $246 = $245 << 16 >> 16;
   $247 = ($246*3)|0;
   $248 = (($247|0) / 4)&-1;
   $lenx = $248;
   $249 = $cvr;
   $250 = $1;
   $251 = (($250) + ($249<<3)|0);
   $252 = (($251) + 6|0);
   $253 = HEAP16[$252>>1]|0;
   $254 = $253 << 16 >> 16;
   $255 = ($254*7)|0;
   $256 = (($255|0) / 16)&-1;
   $leny = $256;
   $257 = $skipy;
   $258 = $4;
   $259 = Math_imul($257, $258)|0;
   $260 = $skipx;
   $261 = (($259) + ($260))|0;
   $262 = $fp;
   $263 = (($262) + ($261<<2)|0);
   $fp = $263;
   $row = 0;
   while(1) {
    $264 = $row;
    $265 = $leny;
    $266 = ($264|0)<($265|0);
    if (!($266)) {
     break;
    }
    $col = 0;
    while(1) {
     $267 = $col;
     $268 = $lenx;
     $269 = ($267|0)<($268|0);
     if (!($269)) {
      break;
     }
     $270 = $fp;
     $271 = +HEAPF32[$270>>2];
     $272 = $sum_pix;
     $273 = $272 + $271;
     $sum_pix = $273;
     $274 = $fp;
     $275 = +HEAPF32[$274>>2];
     $276 = $fp;
     $277 = +HEAPF32[$276>>2];
     $278 = $275 * $277;
     $279 = $ssq;
     $280 = $279 + $278;
     $ssq = $280;
     $281 = $fp;
     $282 = (($281) + 4|0);
     $fp = $282;
     $283 = $col;
     $284 = (($283) + 1)|0;
     $col = $284;
    }
    $285 = $row;
    $286 = (($285) + 1)|0;
    $row = $286;
    $287 = $4;
    $288 = $lenx;
    $289 = (($287) - ($288))|0;
    $290 = $fp;
    $291 = (($290) + ($289<<2)|0);
    $fp = $291;
   }
   $292 = $sum_pix;
   $293 = $sum_pix;
   $294 = $292 * $293;
   $295 = $lenx;
   $296 = $leny;
   $297 = Math_imul($295, $296)|0;
   $298 = (+($297|0));
   $299 = $294 / $298;
   $sum2 = $299;
   $300 = $ssq;
   $301 = $sum2;
   $302 = $300 - $301;
   $303 = $302;
   $304 = $lenx;
   $305 = $leny;
   $306 = Math_imul($304, $305)|0;
   $307 = (+($306|0));
   $308 = $307 - 1.0;
   $309 = $303 / $308;
   $310 = $309;
   $311 = $cvr;
   $312 = $0;
   $313 = (($312) + 780|0);
   $314 = (($313) + ($311<<2)|0);
   HEAPF32[$314>>2] = $310;
   $315 = $cvr;
   $316 = (($315) + 1)|0;
   $cvr = $316;
  }
  STACKTOP = sp;return;
 }
}
function _quantize($osip,$ocmp_siz,$quant_vals,$q_tree,$q_treelen,$fip,$width,$height) {
 $osip = $osip|0;
 $ocmp_siz = $ocmp_siz|0;
 $quant_vals = $quant_vals|0;
 $q_tree = $q_tree|0;
 $q_treelen = $q_treelen|0;
 $fip = $fip|0;
 $width = $width|0;
 $height = $height|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0.0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0.0, $126 = 0.0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0.0, $14 = 0, $140 = 0.0, $141 = 0.0, $142 = 0.0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0.0, $157 = 0.0, $158 = 0.0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0.0, $178 = 0.0, $179 = 0.0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0.0, $186 = 0.0, $187 = 0.0, $188 = 0.0;
 var $189 = 0.0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0.0, $197 = 0.0, $198 = 0.0, $199 = 0.0, $2 = 0, $20 = 0, $200 = 0.0, $201 = 0.0, $202 = 0.0, $203 = 0.0, $204 = 0.0, $205 = 0.0;
 var $206 = 0.0, $207 = 0.0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0.0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0.0, $222 = 0.0, $223 = 0.0;
 var $224 = 0.0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0.0, $231 = 0.0, $232 = 0.0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0.0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0.0, $293 = 0.0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0.0, $303 = 0.0, $304 = 0.0, $305 = 0.0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0.0, $335 = 0.0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0.0, $342 = 0.0, $343 = 0.0, $344 = 0.0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0.0, $362 = 0.0, $363 = 0, $364 = 0.0, $365 = 0, $366 = 0, $367 = 0.0, $368 = 0.0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0.0, $373 = 0.0, $374 = 0, $375 = 0, $376 = 0.0, $377 = 0.0, $378 = 0.0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0.0, $384 = 0.0, $385 = 0.0, $386 = 0.0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0.0, $391 = 0.0, $392 = 0.0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0.0, $398 = 0.0, $399 = 0.0, $4 = 0, $40 = 0, $400 = 0.0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0.0, $43 = 0, $430 = 0.0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0.0, $58 = 0.0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0.0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0.0, $78 = 0.0;
 var $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0.0, $83 = 0.0, $84 = 0.0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $A = 0, $K = 0, $K0 = 0, $K0len = 0, $K1 = 0, $Klen = 0, $NP = 0, $NPlen = 0, $P = 0.0, $S = 0.0, $cnt = 0, $col = 0, $fptr = 0, $i = 0, $j = 0, $m = 0, $m1 = 0.0;
 var $m2 = 0.0, $m3 = 0.0, $nK = 0, $nKlen = 0, $q = 0.0, $row = 0, $sigma = 0, $sip = 0, $sptr = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr3 = 0, $zbin = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1584|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $A = sp + 1272|0;
 $m = sp + 1032|0;
 $sigma = sp + 776|0;
 $K0 = sp + 536|0;
 $K1 = sp + 296|0;
 $NP = sp + 48|0;
 $1 = $osip;
 $2 = $ocmp_siz;
 $3 = $quant_vals;
 $4 = $q_tree;
 $5 = $q_treelen;
 $6 = $fip;
 $7 = $width;
 $8 = $height;
 $cnt = 0;
 while(1) {
  $9 = $cnt;
  $10 = ($9|0)<(52);
  if (!($10)) {
   break;
  }
  $11 = $cnt;
  $12 = (($A) + ($11<<2)|0);
  HEAPF32[$12>>2] = 1.0;
  $13 = $cnt;
  $14 = (($13) + 1)|0;
  $cnt = $14;
 }
 $15 = $cnt;
 $16 = (($15) + 1)|0;
 $cnt = $16;
 $17 = (($A) + ($15<<2)|0);
 HEAPF32[$17>>2] = 1.32000005245208740234;
 $18 = $cnt;
 $19 = (($18) + 1)|0;
 $cnt = $19;
 $20 = (($A) + ($18<<2)|0);
 HEAPF32[$20>>2] = 1.08000004291534423828;
 $21 = $cnt;
 $22 = (($21) + 1)|0;
 $cnt = $22;
 $23 = (($A) + ($21<<2)|0);
 HEAPF32[$23>>2] = 1.41999995708465576172;
 $24 = $cnt;
 $25 = (($24) + 1)|0;
 $cnt = $25;
 $26 = (($A) + ($24<<2)|0);
 HEAPF32[$26>>2] = 1.08000004291534423828;
 $27 = $cnt;
 $28 = (($27) + 1)|0;
 $cnt = $28;
 $29 = (($A) + ($27<<2)|0);
 HEAPF32[$29>>2] = 1.32000005245208740234;
 $30 = $cnt;
 $31 = (($30) + 1)|0;
 $cnt = $31;
 $32 = (($A) + ($30<<2)|0);
 HEAPF32[$32>>2] = 1.41999995708465576172;
 $33 = $cnt;
 $34 = (($33) + 1)|0;
 $cnt = $34;
 $35 = (($A) + ($33<<2)|0);
 HEAPF32[$35>>2] = 1.08000004291534423828;
 $36 = $cnt;
 $37 = (($36) + 1)|0;
 $cnt = $37;
 $38 = (($A) + ($36<<2)|0);
 HEAPF32[$38>>2] = 1.08000004291534423828;
 $cnt = 0;
 while(1) {
  $39 = $cnt;
  $40 = ($39|0)<(64);
  if (!($40)) {
   break;
  }
  $41 = $cnt;
  $42 = $3;
  $43 = (($42) + 268|0);
  $44 = (($43) + ($41<<2)|0);
  HEAPF32[$44>>2] = 0.0;
  $45 = $cnt;
  $46 = $3;
  $47 = (($46) + 524|0);
  $48 = (($47) + ($45<<2)|0);
  HEAPF32[$48>>2] = 0.0;
  $49 = $cnt;
  $50 = (($49) + 1)|0;
  $cnt = $50;
 }
 $cnt = 0;
 while(1) {
  $51 = $cnt;
  $52 = ($51|0)<(60);
  if (!($52)) {
   break;
  }
  $53 = $cnt;
  $54 = $3;
  $55 = (($54) + 780|0);
  $56 = (($55) + ($53<<2)|0);
  $57 = +HEAPF32[$56>>2];
  $58 = $57;
  $59 = $58 < 1.01000000000000000888;
  if ($59) {
   $60 = $cnt;
   $61 = $3;
   $62 = (($61) + 268|0);
   $63 = (($62) + ($60<<2)|0);
   HEAPF32[$63>>2] = 0.0;
  } else {
   $64 = $cnt;
   $65 = ($64|0)<(4);
   if ($65) {
    $66 = $cnt;
    $67 = $3;
    $68 = (($67) + 268|0);
    $69 = (($68) + ($66<<2)|0);
    HEAPF32[$69>>2] = 1.0;
   } else {
    $70 = $cnt;
    $71 = (($A) + ($70<<2)|0);
    $72 = +HEAPF32[$71>>2];
    $73 = $cnt;
    $74 = $3;
    $75 = (($74) + 780|0);
    $76 = (($75) + ($73<<2)|0);
    $77 = +HEAPF32[$76>>2];
    $78 = $77;
    $79 = (+Math_log((+$78)));
    $80 = $79;
    $81 = $72 * $80;
    $82 = $81;
    $83 = 10.0 / $82;
    $84 = $83;
    $85 = $cnt;
    $86 = $3;
    $87 = (($86) + 268|0);
    $88 = (($87) + ($85<<2)|0);
    HEAPF32[$88>>2] = $84;
   }
  }
  $89 = $cnt;
  $90 = (($89) + 1)|0;
  $cnt = $90;
 }
 $91 = $7;
 $92 = $8;
 $93 = Math_imul($91, $92)|0;
 $94 = (_calloc($93,2)|0);
 $sip = $94;
 $95 = ($94|0)==(0|0);
 if ($95) {
  $96 = HEAP32[_stderr>>2]|0;
  (_fprintf(($96|0),(11368|0),($vararg_buffer|0))|0);
  $0 = -90;
  $442 = $0;
  STACKTOP = sp;return ($442|0);
 }
 $97 = $sip;
 $sptr = $97;
 $m1 = 9.765625E-4;
 $m2 = 0.00390625;
 $m3 = 0.0625;
 $cnt = 0;
 while(1) {
  $98 = $cnt;
  $99 = ($98|0)<(4);
  if (!($99)) {
   break;
  }
  $100 = $m1;
  $101 = $cnt;
  $102 = (($m) + ($101<<2)|0);
  HEAPF32[$102>>2] = $100;
  $103 = $cnt;
  $104 = (($103) + 1)|0;
  $cnt = $104;
 }
 $cnt = 4;
 while(1) {
  $105 = $cnt;
  $106 = ($105|0)<(51);
  if (!($106)) {
   break;
  }
  $107 = $m2;
  $108 = $cnt;
  $109 = (($m) + ($108<<2)|0);
  HEAPF32[$109>>2] = $107;
  $110 = $cnt;
  $111 = (($110) + 1)|0;
  $cnt = $111;
 }
 $cnt = 51;
 while(1) {
  $112 = $cnt;
  $113 = ($112|0)<(60);
  if (!($113)) {
   break;
  }
  $114 = $m3;
  $115 = $cnt;
  $116 = (($m) + ($115<<2)|0);
  HEAPF32[$116>>2] = $114;
  $117 = $cnt;
  $118 = (($117) + 1)|0;
  $cnt = $118;
 }
 $j = 0;
 $K0len = 0;
 $cnt = 0;
 while(1) {
  $119 = $cnt;
  $120 = ($119|0)<(60);
  if (!($120)) {
   break;
  }
  $121 = $cnt;
  $122 = $3;
  $123 = (($122) + 780|0);
  $124 = (($123) + ($121<<2)|0);
  $125 = +HEAPF32[$124>>2];
  $126 = $125;
  $127 = $126 >= 1.01000000000000000888;
  if ($127) {
   $128 = $cnt;
   $129 = $K0len;
   $130 = (($K0) + ($129<<2)|0);
   HEAP32[$130>>2] = $128;
   $131 = $cnt;
   $132 = $K0len;
   $133 = (($132) + 1)|0;
   $K0len = $133;
   $134 = (($K1) + ($132<<2)|0);
   HEAP32[$134>>2] = $131;
   $135 = $cnt;
   $136 = $3;
   $137 = (($136) + 780|0);
   $138 = (($137) + ($135<<2)|0);
   $139 = +HEAPF32[$138>>2];
   $140 = $139;
   $141 = (+Math_sqrt((+$140)));
   $142 = $141;
   $143 = $cnt;
   $144 = (($sigma) + ($143<<2)|0);
   HEAPF32[$144>>2] = $142;
  }
  $145 = $cnt;
  $146 = (($145) + 1)|0;
  $cnt = $146;
 }
 $K = $K1;
 $147 = $K0len;
 $Klen = $147;
 while(1) {
  $S = 0.0;
  $i = 0;
  while(1) {
   $148 = $i;
   $149 = $Klen;
   $150 = ($148|0)<($149|0);
   if (!($150)) {
    break;
   }
   $151 = $i;
   $152 = $K;
   $153 = (($152) + ($151<<2)|0);
   $154 = HEAP32[$153>>2]|0;
   $155 = (($m) + ($154<<2)|0);
   $156 = +HEAPF32[$155>>2];
   $157 = $S;
   $158 = $157 + $156;
   $S = $158;
   $159 = $i;
   $160 = (($159) + 1)|0;
   $i = $160;
  }
  $P = 1.0;
  $i = 0;
  while(1) {
   $161 = $i;
   $162 = $Klen;
   $163 = ($161|0)<($162|0);
   if (!($163)) {
    break;
   }
   $164 = $i;
   $165 = $K;
   $166 = (($165) + ($164<<2)|0);
   $167 = HEAP32[$166>>2]|0;
   $168 = (($sigma) + ($167<<2)|0);
   $169 = +HEAPF32[$168>>2];
   $170 = $i;
   $171 = $K;
   $172 = (($171) + ($170<<2)|0);
   $173 = HEAP32[$172>>2]|0;
   $174 = $3;
   $175 = (($174) + 268|0);
   $176 = (($175) + ($173<<2)|0);
   $177 = +HEAPF32[$176>>2];
   $178 = $169 / $177;
   $179 = $178;
   $180 = $i;
   $181 = $K;
   $182 = (($181) + ($180<<2)|0);
   $183 = HEAP32[$182>>2]|0;
   $184 = (($m) + ($183<<2)|0);
   $185 = +HEAPF32[$184>>2];
   $186 = $185;
   $187 = (+Math_pow((+$179),(+$186)));
   $188 = $P;
   $189 = $188;
   $190 = $189 * $187;
   $191 = $190;
   $P = $191;
   $192 = $i;
   $193 = (($192) + 1)|0;
   $i = $193;
  }
  $194 = $3;
  $195 = (($194) + 8|0);
  $196 = +HEAPF32[$195>>2];
  $197 = $S;
  $198 = $196 / $197;
  $199 = $198;
  $200 = $199 - 1.0;
  $201 = (+Math_pow(2.0,(+$200)));
  $202 = $201 / 2.5;
  $203 = $P;
  $204 = $203;
  $205 = $S;
  $206 = $205;
  $207 = 1.0 / $206;
  $208 = (+Math_pow((+$204),(+$207)));
  $209 = $202 / $208;
  $210 = $209;
  $q = $210;
  _memset(($NP|0),0,240)|0;
  $NPlen = 0;
  $i = 0;
  while(1) {
   $211 = $i;
   $212 = $Klen;
   $213 = ($211|0)<($212|0);
   if (!($213)) {
    break;
   }
   $214 = $i;
   $215 = $K;
   $216 = (($215) + ($214<<2)|0);
   $217 = HEAP32[$216>>2]|0;
   $218 = $3;
   $219 = (($218) + 268|0);
   $220 = (($219) + ($217<<2)|0);
   $221 = +HEAPF32[$220>>2];
   $222 = $q;
   $223 = $221 / $222;
   $224 = $223;
   $225 = $i;
   $226 = $K;
   $227 = (($226) + ($225<<2)|0);
   $228 = HEAP32[$227>>2]|0;
   $229 = (($sigma) + ($228<<2)|0);
   $230 = +HEAPF32[$229>>2];
   $231 = $230;
   $232 = 5.0 * $231;
   $233 = $224 >= $232;
   if ($233) {
    $234 = $i;
    $235 = $K;
    $236 = (($235) + ($234<<2)|0);
    $237 = HEAP32[$236>>2]|0;
    $238 = (($NP) + ($237<<2)|0);
    HEAP32[$238>>2] = 1;
    $239 = $NPlen;
    $240 = (($239) + 1)|0;
    $NPlen = $240;
   }
   $241 = $i;
   $242 = (($241) + 1)|0;
   $i = $242;
  }
  $243 = $NPlen;
  $244 = ($243|0)==(0);
  if ($244) {
   break;
  }
  $nK = $K1;
  $nKlen = 0;
  $i = 0;
  while(1) {
   $245 = $i;
   $246 = $Klen;
   $247 = ($245|0)<($246|0);
   if (!($247)) {
    break;
   }
   $248 = $i;
   $249 = $K;
   $250 = (($249) + ($248<<2)|0);
   $251 = HEAP32[$250>>2]|0;
   $252 = (($NP) + ($251<<2)|0);
   $253 = HEAP32[$252>>2]|0;
   $254 = ($253|0)!=(0);
   if (!($254)) {
    $255 = $i;
    $256 = $K;
    $257 = (($256) + ($255<<2)|0);
    $258 = HEAP32[$257>>2]|0;
    $259 = $nKlen;
    $260 = (($259) + 1)|0;
    $nKlen = $260;
    $261 = $nK;
    $262 = (($261) + ($259<<2)|0);
    HEAP32[$262>>2] = $258;
   }
   $263 = $i;
   $264 = (($263) + 1)|0;
   $i = $264;
  }
  $265 = $nK;
  $K = $265;
  $266 = $nKlen;
  $Klen = $266;
  $267 = $j;
  $268 = (($267) + 1)|0;
  $j = $268;
 }
 $nK = $K1;
 $269 = $nK;
 _memset(($269|0),0,240)|0;
 $i = 0;
 while(1) {
  $270 = $i;
  $271 = $K0len;
  $272 = ($270|0)<($271|0);
  if (!($272)) {
   break;
  }
  $273 = $i;
  $274 = (($K0) + ($273<<2)|0);
  $275 = HEAP32[$274>>2]|0;
  $276 = $nK;
  $277 = (($276) + ($275<<2)|0);
  HEAP32[$277>>2] = 1;
  $278 = $i;
  $279 = (($278) + 1)|0;
  $i = $279;
 }
 $cnt = 0;
 while(1) {
  $280 = $cnt;
  $281 = ($280|0)<(60);
  if (!($281)) {
   break;
  }
  $282 = $cnt;
  $283 = $nK;
  $284 = (($283) + ($282<<2)|0);
  $285 = HEAP32[$284>>2]|0;
  $286 = ($285|0)!=(0);
  if ($286) {
   $287 = $q;
   $288 = $cnt;
   $289 = $3;
   $290 = (($289) + 268|0);
   $291 = (($290) + ($288<<2)|0);
   $292 = +HEAPF32[$291>>2];
   $293 = $292 / $287;
   HEAPF32[$291>>2] = $293;
  } else {
   $294 = $cnt;
   $295 = $3;
   $296 = (($295) + 268|0);
   $297 = (($296) + ($294<<2)|0);
   HEAPF32[$297>>2] = 0.0;
  }
  $298 = $cnt;
  $299 = $3;
  $300 = (($299) + 268|0);
  $301 = (($300) + ($298<<2)|0);
  $302 = +HEAPF32[$301>>2];
  $303 = $302;
  $304 = 1.19999999999999995559 * $303;
  $305 = $304;
  $306 = $cnt;
  $307 = $3;
  $308 = (($307) + 524|0);
  $309 = (($308) + ($306<<2)|0);
  HEAPF32[$309>>2] = $305;
  $310 = $cnt;
  $311 = (($310) + 1)|0;
  $cnt = $311;
 }
 $cnt = 0;
 while(1) {
  $312 = $cnt;
  $313 = ($312|0)<(60);
  if (!($313)) {
   break;
  }
  $314 = $6;
  $315 = $cnt;
  $316 = $4;
  $317 = (($316) + ($315<<3)|0);
  $318 = (($317) + 2|0);
  $319 = HEAP16[$318>>1]|0;
  $320 = $319 << 16 >> 16;
  $321 = $7;
  $322 = Math_imul($320, $321)|0;
  $323 = (($314) + ($322<<2)|0);
  $324 = $cnt;
  $325 = $4;
  $326 = (($325) + ($324<<3)|0);
  $327 = HEAP16[$326>>1]|0;
  $328 = $327 << 16 >> 16;
  $329 = (($323) + ($328<<2)|0);
  $fptr = $329;
  $330 = $cnt;
  $331 = $3;
  $332 = (($331) + 268|0);
  $333 = (($332) + ($330<<2)|0);
  $334 = +HEAPF32[$333>>2];
  $335 = $334;
  $336 = $335 != 0.0;
  if ($336) {
   $337 = $cnt;
   $338 = $3;
   $339 = (($338) + 524|0);
   $340 = (($339) + ($337<<2)|0);
   $341 = +HEAPF32[$340>>2];
   $342 = $341;
   $343 = $342 / 2.0;
   $344 = $343;
   $zbin = $344;
   $row = 0;
   while(1) {
    $345 = $row;
    $346 = $cnt;
    $347 = $4;
    $348 = (($347) + ($346<<3)|0);
    $349 = (($348) + 6|0);
    $350 = HEAP16[$349>>1]|0;
    $351 = $350 << 16 >> 16;
    $352 = ($345|0)<($351|0);
    if (!($352)) {
     break;
    }
    $col = 0;
    while(1) {
     $353 = $col;
     $354 = $cnt;
     $355 = $4;
     $356 = (($355) + ($354<<3)|0);
     $357 = (($356) + 4|0);
     $358 = HEAP16[$357>>1]|0;
     $359 = $358 << 16 >> 16;
     $360 = ($353|0)<($359|0);
     if (!($360)) {
      break;
     }
     $361 = $zbin;
     $362 = -$361;
     $363 = $fptr;
     $364 = +HEAPF32[$363>>2];
     $365 = $362 <= $364;
     if ($365) {
      $366 = $fptr;
      $367 = +HEAPF32[$366>>2];
      $368 = $zbin;
      $369 = $367 <= $368;
      if ($369) {
       $370 = $sptr;
       HEAP16[$370>>1] = 0;
      } else {
       label = 84;
      }
     } else {
      label = 84;
     }
     if ((label|0) == 84) {
      label = 0;
      $371 = $fptr;
      $372 = +HEAPF32[$371>>2];
      $373 = $372;
      $374 = $373 > 0.0;
      if ($374) {
       $375 = $fptr;
       $376 = +HEAPF32[$375>>2];
       $377 = $zbin;
       $378 = $376 - $377;
       $379 = $cnt;
       $380 = $3;
       $381 = (($380) + 268|0);
       $382 = (($381) + ($379<<2)|0);
       $383 = +HEAPF32[$382>>2];
       $384 = $378 / $383;
       $385 = $384;
       $386 = $385 + 1.0;
       $387 = (~~(($386)));
       $388 = $sptr;
       HEAP16[$388>>1] = $387;
      } else {
       $389 = $fptr;
       $390 = +HEAPF32[$389>>2];
       $391 = $zbin;
       $392 = $390 + $391;
       $393 = $cnt;
       $394 = $3;
       $395 = (($394) + 268|0);
       $396 = (($395) + ($393<<2)|0);
       $397 = +HEAPF32[$396>>2];
       $398 = $392 / $397;
       $399 = $398;
       $400 = $399 - 1.0;
       $401 = (~~(($400)));
       $402 = $sptr;
       HEAP16[$402>>1] = $401;
      }
     }
     $403 = $sptr;
     $404 = (($403) + 2|0);
     $sptr = $404;
     $405 = $fptr;
     $406 = (($405) + 4|0);
     $fptr = $406;
     $407 = $col;
     $408 = (($407) + 1)|0;
     $col = $408;
    }
    $409 = $row;
    $410 = (($409) + 1)|0;
    $row = $410;
    $411 = $7;
    $412 = $cnt;
    $413 = $4;
    $414 = (($413) + ($412<<3)|0);
    $415 = (($414) + 4|0);
    $416 = HEAP16[$415>>1]|0;
    $417 = $416 << 16 >> 16;
    $418 = (($411) - ($417))|0;
    $419 = $fptr;
    $420 = (($419) + ($418<<2)|0);
    $fptr = $420;
   }
  } else {
   $421 = HEAP32[8>>2]|0;
   $422 = ($421|0)>(0);
   if ($422) {
    $423 = HEAP32[_stderr>>2]|0;
    $424 = $cnt;
    $425 = $cnt;
    $426 = $3;
    $427 = (($426) + 268|0);
    $428 = (($427) + ($425<<2)|0);
    $429 = +HEAPF32[$428>>2];
    $430 = $429;
    HEAP32[$vararg_buffer1>>2] = $424;
    $vararg_ptr3 = (($vararg_buffer1) + 4|0);
    HEAPF64[tempDoublePtr>>3]=$430;HEAP32[$vararg_ptr3>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_ptr3+4>>2]=HEAP32[tempDoublePtr+4>>2];
    (_fprintf(($423|0),(11408|0),($vararg_buffer1|0))|0);
   }
  }
  $431 = $cnt;
  $432 = (($431) + 1)|0;
  $cnt = $432;
 }
 $433 = $sip;
 $434 = $1;
 HEAP32[$434>>2] = $433;
 $435 = $sptr;
 $436 = $sip;
 $437 = $435;
 $438 = $436;
 $439 = (($437) - ($438))|0;
 $440 = (($439|0) / 2)&-1;
 $441 = $2;
 HEAP32[$441>>2] = $440;
 $0 = 0;
 $442 = $0;
 STACKTOP = sp;return ($442|0);
}
function _quant_block_sizes($oqsize1,$oqsize2,$oqsize3,$quant_vals,$w_tree,$w_treelen,$q_tree,$q_treelen) {
 $oqsize1 = $oqsize1|0;
 $oqsize2 = $oqsize2|0;
 $oqsize3 = $oqsize3|0;
 $quant_vals = $quant_vals|0;
 $w_tree = $w_tree|0;
 $w_treelen = $w_treelen|0;
 $q_tree = $q_tree|0;
 $q_treelen = $q_treelen|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0.0, $114 = 0.0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0.0, $62 = 0.0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0.0, $88 = 0.0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $node = 0;
 var $qsize1 = 0, $qsize2 = 0, $qsize3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $oqsize1;
 $1 = $oqsize2;
 $2 = $oqsize3;
 $3 = $quant_vals;
 $4 = $w_tree;
 $5 = $w_treelen;
 $6 = $q_tree;
 $7 = $q_treelen;
 $8 = $4;
 $9 = (($8) + 336|0);
 $10 = (($9) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $4;
 $13 = (($12) + 336|0);
 $14 = (($13) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = Math_imul($11, $15)|0;
 $qsize1 = $16;
 $17 = $4;
 $18 = (($17) + 120|0);
 $19 = (($18) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $4;
 $22 = (($21) + 24|0);
 $23 = (($22) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = Math_imul($20, $24)|0;
 $26 = $4;
 $27 = (($26) + 96|0);
 $28 = (($27) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = $4;
 $31 = (($30) + 96|0);
 $32 = (($31) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = Math_imul($29, $33)|0;
 $35 = (($25) + ($34))|0;
 $qsize2 = $35;
 $36 = $4;
 $37 = (($36) + 48|0);
 $38 = (($37) + 8|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = $4;
 $41 = (($40) + 48|0);
 $42 = (($41) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = Math_imul($39, $43)|0;
 $45 = $4;
 $46 = (($45) + 72|0);
 $47 = (($46) + 8|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = $4;
 $50 = (($49) + 72|0);
 $51 = (($50) + 12|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = Math_imul($48, $52)|0;
 $54 = (($44) + ($53))|0;
 $qsize3 = $54;
 $node = 0;
 while(1) {
  $55 = $node;
  $56 = ($55|0)<(19);
  if (!($56)) {
   break;
  }
  $57 = $node;
  $58 = $3;
  $59 = (($58) + 268|0);
  $60 = (($59) + ($57<<2)|0);
  $61 = +HEAPF32[$60>>2];
  $62 = $61;
  $63 = $62 == 0.0;
  if ($63) {
   $64 = $node;
   $65 = $6;
   $66 = (($65) + ($64<<3)|0);
   $67 = (($66) + 4|0);
   $68 = HEAP16[$67>>1]|0;
   $69 = $68 << 16 >> 16;
   $70 = $node;
   $71 = $6;
   $72 = (($71) + ($70<<3)|0);
   $73 = (($72) + 6|0);
   $74 = HEAP16[$73>>1]|0;
   $75 = $74 << 16 >> 16;
   $76 = Math_imul($69, $75)|0;
   $77 = $qsize1;
   $78 = (($77) - ($76))|0;
   $qsize1 = $78;
  }
  $79 = $node;
  $80 = (($79) + 1)|0;
  $node = $80;
 }
 $node = 19;
 while(1) {
  $81 = $node;
  $82 = ($81|0)<(52);
  if (!($82)) {
   break;
  }
  $83 = $node;
  $84 = $3;
  $85 = (($84) + 268|0);
  $86 = (($85) + ($83<<2)|0);
  $87 = +HEAPF32[$86>>2];
  $88 = $87;
  $89 = $88 == 0.0;
  if ($89) {
   $90 = $node;
   $91 = $6;
   $92 = (($91) + ($90<<3)|0);
   $93 = (($92) + 4|0);
   $94 = HEAP16[$93>>1]|0;
   $95 = $94 << 16 >> 16;
   $96 = $node;
   $97 = $6;
   $98 = (($97) + ($96<<3)|0);
   $99 = (($98) + 6|0);
   $100 = HEAP16[$99>>1]|0;
   $101 = $100 << 16 >> 16;
   $102 = Math_imul($95, $101)|0;
   $103 = $qsize2;
   $104 = (($103) - ($102))|0;
   $qsize2 = $104;
  }
  $105 = $node;
  $106 = (($105) + 1)|0;
  $node = $106;
 }
 $node = 52;
 while(1) {
  $107 = $node;
  $108 = ($107|0)<(60);
  if (!($108)) {
   break;
  }
  $109 = $node;
  $110 = $3;
  $111 = (($110) + 268|0);
  $112 = (($111) + ($109<<2)|0);
  $113 = +HEAPF32[$112>>2];
  $114 = $113;
  $115 = $114 == 0.0;
  if ($115) {
   $116 = $node;
   $117 = $6;
   $118 = (($117) + ($116<<3)|0);
   $119 = (($118) + 4|0);
   $120 = HEAP16[$119>>1]|0;
   $121 = $120 << 16 >> 16;
   $122 = $node;
   $123 = $6;
   $124 = (($123) + ($122<<3)|0);
   $125 = (($124) + 6|0);
   $126 = HEAP16[$125>>1]|0;
   $127 = $126 << 16 >> 16;
   $128 = Math_imul($121, $127)|0;
   $129 = $qsize3;
   $130 = (($129) - ($128))|0;
   $qsize3 = $130;
  }
  $131 = $node;
  $132 = (($131) + 1)|0;
  $node = $132;
 }
 $133 = $qsize1;
 $134 = $0;
 HEAP32[$134>>2] = $133;
 $135 = $qsize2;
 $136 = $1;
 HEAP32[$136>>2] = $135;
 $137 = $qsize3;
 $138 = $2;
 HEAP32[$138>>2] = $137;
 STACKTOP = sp;return;
}
function _wsq_decompose($fdata,$width,$height,$w_tree,$w_treelen,$hifilt,$hisz,$lofilt,$losz) {
 $fdata = $fdata|0;
 $width = $width|0;
 $height = $height|0;
 $w_tree = $w_tree|0;
 $w_treelen = $w_treelen|0;
 $hifilt = $hifilt|0;
 $hisz = $hisz|0;
 $lofilt = $lofilt|0;
 $losz = $losz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $fdata1 = 0, $fdata_bse = 0, $node = 0, $num_pix = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $1 = $fdata;
 $2 = $width;
 $3 = $height;
 $4 = $w_tree;
 $5 = $w_treelen;
 $6 = $hifilt;
 $7 = $hisz;
 $8 = $lofilt;
 $9 = $losz;
 $10 = $2;
 $11 = $3;
 $12 = Math_imul($10, $11)|0;
 $num_pix = $12;
 $13 = $num_pix;
 $14 = $13<<2;
 $15 = (_malloc($14)|0);
 $fdata1 = $15;
 $16 = ($15|0)==(0|0);
 if ($16) {
  $17 = HEAP32[_stderr>>2]|0;
  (_fprintf(($17|0),(11424|0),($vararg_buffer|0))|0);
  $0 = -94;
  $82 = $0;
  STACKTOP = sp;return ($82|0);
 }
 $node = 0;
 while(1) {
  $18 = $node;
  $19 = $5;
  $20 = ($18|0)<($19|0);
  if (!($20)) {
   break;
  }
  $21 = $1;
  $22 = $node;
  $23 = $4;
  $24 = (($23) + (($22*24)|0)|0);
  $25 = (($24) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = $2;
  $28 = Math_imul($26, $27)|0;
  $29 = (($21) + ($28<<2)|0);
  $30 = $node;
  $31 = $4;
  $32 = (($31) + (($30*24)|0)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($29) + ($33<<2)|0);
  $fdata_bse = $34;
  $35 = $fdata1;
  $36 = $fdata_bse;
  $37 = $node;
  $38 = $4;
  $39 = (($38) + (($37*24)|0)|0);
  $40 = (($39) + 12|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = $node;
  $43 = $4;
  $44 = (($43) + (($42*24)|0)|0);
  $45 = (($44) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = $2;
  $48 = $6;
  $49 = $7;
  $50 = $8;
  $51 = $9;
  $52 = $node;
  $53 = $4;
  $54 = (($53) + (($52*24)|0)|0);
  $55 = (($54) + 16|0);
  $56 = HEAP32[$55>>2]|0;
  _get_lets($35,$36,$41,$46,$47,1,$48,$49,$50,$51,$56);
  $57 = $fdata_bse;
  $58 = $fdata1;
  $59 = $node;
  $60 = $4;
  $61 = (($60) + (($59*24)|0)|0);
  $62 = (($61) + 8|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = $node;
  $65 = $4;
  $66 = (($65) + (($64*24)|0)|0);
  $67 = (($66) + 12|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = $2;
  $70 = $6;
  $71 = $7;
  $72 = $8;
  $73 = $9;
  $74 = $node;
  $75 = $4;
  $76 = (($75) + (($74*24)|0)|0);
  $77 = (($76) + 20|0);
  $78 = HEAP32[$77>>2]|0;
  _get_lets($57,$58,$63,$68,1,$69,$70,$71,$72,$73,$78);
  $79 = $node;
  $80 = (($79) + 1)|0;
  $node = $80;
 }
 $81 = $fdata1;
 _free($81);
 $0 = 0;
 $82 = $0;
 STACKTOP = sp;return ($82|0);
}
function _get_lets($new,$old,$len1,$len2,$pitch,$stride,$hi,$hsz,$lo,$lsz,$inv) {
 $new = $new|0;
 $old = $old|0;
 $len1 = $len1|0;
 $len2 = $len2|0;
 $pitch = $pitch|0;
 $stride = $stride|0;
 $hi = $hi|0;
 $hsz = $hsz|0;
 $lo = $lo|0;
 $lsz = $lsz|0;
 $inv = $inv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0.0, $12 = 0, $120 = 0, $121 = 0.0, $122 = 0.0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0.0, $144 = 0, $145 = 0, $146 = 0, $147 = 0.0, $148 = 0.0, $149 = 0, $15 = 0, $150 = 0.0, $151 = 0.0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0.0, $163 = 0, $164 = 0.0, $165 = 0.0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0.0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0, $193 = 0.0, $194 = 0.0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0.0, $232 = 0, $233 = 0.0, $234 = 0.0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0.0, $256 = 0, $257 = 0, $258 = 0, $259 = 0.0, $26 = 0;
 var $260 = 0.0, $261 = 0, $262 = 0.0, $263 = 0.0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0.0, $28 = 0, $280 = 0.0, $281 = 0.0, $282 = 0.0, $283 = 0, $284 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0.0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $da_ev = 0, $fi_ev = 0, $hipass = 0, $hle = 0, $hle2 = 0, $hlen = 0, $hoc = 0, $hpx = 0, $hpxstr = 0, $hre = 0, $hre2 = 0, $hspx = 0, $hspxstr = 0, $i = 0, $lle = 0;
 var $lle2 = 0, $llen = 0, $loc = 0, $lopass = 0, $lpx = 0, $lpxstr = 0, $lre = 0, $lre2 = 0, $lspx = 0, $lspxstr = 0, $nstr = 0, $ohle = 0, $ohre = 0, $olle = 0, $olre = 0, $p0 = 0, $p1 = 0, $pix = 0, $pstr = 0, $rw_cl = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = $new;
 $1 = $old;
 $2 = $len1;
 $3 = $len2;
 $4 = $pitch;
 $5 = $stride;
 $6 = $hi;
 $7 = $hsz;
 $8 = $lo;
 $9 = $lsz;
 $10 = $inv;
 $11 = $3;
 $12 = (($11|0) % 2)&-1;
 $da_ev = $12;
 $13 = $9;
 $14 = (($13|0) % 2)&-1;
 $fi_ev = $14;
 $15 = $fi_ev;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = $9;
  $18 = (($17) - 1)|0;
  $19 = (($18|0) / 2)&-1;
  $loc = $19;
  $20 = $7;
  $21 = (($20) - 1)|0;
  $22 = (($21|0) / 2)&-1;
  $23 = (($22) - 1)|0;
  $hoc = $23;
  $olle = 0;
  $ohle = 0;
  $olre = 0;
  $ohre = 0;
 } else {
  $24 = $9;
  $25 = (($24|0) / 2)&-1;
  $26 = (($25) - 2)|0;
  $loc = $26;
  $27 = $7;
  $28 = (($27|0) / 2)&-1;
  $29 = (($28) - 2)|0;
  $hoc = $29;
  $olle = 1;
  $ohle = 1;
  $olre = 1;
  $ohre = 1;
  $30 = $loc;
  $31 = ($30|0)==(-1);
  if ($31) {
   $loc = 0;
   $olle = 0;
  }
  $32 = $hoc;
  $33 = ($32|0)==(-1);
  if ($33) {
   $hoc = 0;
   $ohle = 0;
  }
  $i = 0;
  while(1) {
   $34 = $i;
   $35 = $7;
   $36 = ($34|0)<($35|0);
   if (!($36)) {
    break;
   }
   $37 = $i;
   $38 = $6;
   $39 = (($38) + ($37<<2)|0);
   $40 = +HEAPF32[$39>>2];
   $41 = $40;
   $42 = $41 * -1.0;
   $43 = $42;
   HEAPF32[$39>>2] = $43;
   $44 = $i;
   $45 = (($44) + 1)|0;
   $i = $45;
  }
 }
 $46 = $5;
 $pstr = $46;
 $47 = $pstr;
 $48 = (0 - ($47))|0;
 $nstr = $48;
 $49 = $da_ev;
 $50 = ($49|0)!=(0);
 if ($50) {
  $51 = $3;
  $52 = (($51) + 1)|0;
  $53 = (($52|0) / 2)&-1;
  $llen = $53;
  $54 = $llen;
  $55 = (($54) - 1)|0;
  $hlen = $55;
 } else {
  $56 = $3;
  $57 = (($56|0) / 2)&-1;
  $llen = $57;
  $58 = $llen;
  $hlen = $58;
 }
 $rw_cl = 0;
 while(1) {
  $59 = $rw_cl;
  $60 = $2;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   break;
  }
  $62 = $10;
  $63 = ($62|0)!=(0);
  if ($63) {
   $64 = $0;
   $65 = $rw_cl;
   $66 = $4;
   $67 = Math_imul($65, $66)|0;
   $68 = (($64) + ($67<<2)|0);
   $hipass = $68;
   $69 = $hipass;
   $70 = $hlen;
   $71 = $5;
   $72 = Math_imul($70, $71)|0;
   $73 = (($69) + ($72<<2)|0);
   $lopass = $73;
  } else {
   $74 = $0;
   $75 = $rw_cl;
   $76 = $4;
   $77 = Math_imul($75, $76)|0;
   $78 = (($74) + ($77<<2)|0);
   $lopass = $78;
   $79 = $lopass;
   $80 = $llen;
   $81 = $5;
   $82 = Math_imul($80, $81)|0;
   $83 = (($79) + ($82<<2)|0);
   $hipass = $83;
  }
  $84 = $1;
  $85 = $rw_cl;
  $86 = $4;
  $87 = Math_imul($85, $86)|0;
  $88 = (($84) + ($87<<2)|0);
  $p0 = $88;
  $89 = $p0;
  $90 = $3;
  $91 = (($90) - 1)|0;
  $92 = $5;
  $93 = Math_imul($91, $92)|0;
  $94 = (($89) + ($93<<2)|0);
  $p1 = $94;
  $95 = $p0;
  $96 = $loc;
  $97 = $5;
  $98 = Math_imul($96, $97)|0;
  $99 = (($95) + ($98<<2)|0);
  $lspx = $99;
  $100 = $nstr;
  $lspxstr = $100;
  $101 = $olle;
  $lle2 = $101;
  $102 = $olre;
  $lre2 = $102;
  $103 = $p0;
  $104 = $hoc;
  $105 = $5;
  $106 = Math_imul($104, $105)|0;
  $107 = (($103) + ($106<<2)|0);
  $hspx = $107;
  $108 = $nstr;
  $hspxstr = $108;
  $109 = $ohle;
  $hle2 = $109;
  $110 = $ohre;
  $hre2 = $110;
  $pix = 0;
  while(1) {
   $111 = $pix;
   $112 = $hlen;
   $113 = ($111|0)<($112|0);
   if (!($113)) {
    break;
   }
   $114 = $lspxstr;
   $lpxstr = $114;
   $115 = $lspx;
   $lpx = $115;
   $116 = $lle2;
   $lle = $116;
   $117 = $lre2;
   $lre = $117;
   $118 = $lpx;
   $119 = +HEAPF32[$118>>2];
   $120 = $8;
   $121 = +HEAPF32[$120>>2];
   $122 = $119 * $121;
   $123 = $lopass;
   HEAPF32[$123>>2] = $122;
   $i = 1;
   while(1) {
    $124 = $i;
    $125 = $9;
    $126 = ($124|0)<($125|0);
    if (!($126)) {
     break;
    }
    $127 = $lpx;
    $128 = $p0;
    $129 = ($127|0)==($128|0);
    if ($129) {
     $130 = $lle;
     $131 = ($130|0)!=(0);
     if ($131) {
      $lpxstr = 0;
      $lle = 0;
     } else {
      $132 = $pstr;
      $lpxstr = $132;
     }
    }
    $133 = $lpx;
    $134 = $p1;
    $135 = ($133|0)==($134|0);
    if ($135) {
     $136 = $lre;
     $137 = ($136|0)!=(0);
     if ($137) {
      $lpxstr = 0;
      $lre = 0;
     } else {
      $138 = $nstr;
      $lpxstr = $138;
     }
    }
    $139 = $lpxstr;
    $140 = $lpx;
    $141 = (($140) + ($139<<2)|0);
    $lpx = $141;
    $142 = $lpx;
    $143 = +HEAPF32[$142>>2];
    $144 = $i;
    $145 = $8;
    $146 = (($145) + ($144<<2)|0);
    $147 = +HEAPF32[$146>>2];
    $148 = $143 * $147;
    $149 = $lopass;
    $150 = +HEAPF32[$149>>2];
    $151 = $150 + $148;
    HEAPF32[$149>>2] = $151;
    $152 = $i;
    $153 = (($152) + 1)|0;
    $i = $153;
   }
   $154 = $5;
   $155 = $lopass;
   $156 = (($155) + ($154<<2)|0);
   $lopass = $156;
   $157 = $hspxstr;
   $hpxstr = $157;
   $158 = $hspx;
   $hpx = $158;
   $159 = $hle2;
   $hle = $159;
   $160 = $hre2;
   $hre = $160;
   $161 = $hpx;
   $162 = +HEAPF32[$161>>2];
   $163 = $6;
   $164 = +HEAPF32[$163>>2];
   $165 = $162 * $164;
   $166 = $hipass;
   HEAPF32[$166>>2] = $165;
   $i = 1;
   while(1) {
    $167 = $i;
    $168 = $7;
    $169 = ($167|0)<($168|0);
    if (!($169)) {
     break;
    }
    $170 = $hpx;
    $171 = $p0;
    $172 = ($170|0)==($171|0);
    if ($172) {
     $173 = $hle;
     $174 = ($173|0)!=(0);
     if ($174) {
      $hpxstr = 0;
      $hle = 0;
     } else {
      $175 = $pstr;
      $hpxstr = $175;
     }
    }
    $176 = $hpx;
    $177 = $p1;
    $178 = ($176|0)==($177|0);
    if ($178) {
     $179 = $hre;
     $180 = ($179|0)!=(0);
     if ($180) {
      $hpxstr = 0;
      $hre = 0;
     } else {
      $181 = $nstr;
      $hpxstr = $181;
     }
    }
    $182 = $hpxstr;
    $183 = $hpx;
    $184 = (($183) + ($182<<2)|0);
    $hpx = $184;
    $185 = $hpx;
    $186 = +HEAPF32[$185>>2];
    $187 = $i;
    $188 = $6;
    $189 = (($188) + ($187<<2)|0);
    $190 = +HEAPF32[$189>>2];
    $191 = $186 * $190;
    $192 = $hipass;
    $193 = +HEAPF32[$192>>2];
    $194 = $193 + $191;
    HEAPF32[$192>>2] = $194;
    $195 = $i;
    $196 = (($195) + 1)|0;
    $i = $196;
   }
   $197 = $5;
   $198 = $hipass;
   $199 = (($198) + ($197<<2)|0);
   $hipass = $199;
   $i = 0;
   while(1) {
    $200 = $i;
    $201 = ($200|0)<(2);
    if (!($201)) {
     break;
    }
    $202 = $lspx;
    $203 = $p0;
    $204 = ($202|0)==($203|0);
    if ($204) {
     $205 = $lle2;
     $206 = ($205|0)!=(0);
     if ($206) {
      $lspxstr = 0;
      $lle2 = 0;
     } else {
      $207 = $pstr;
      $lspxstr = $207;
     }
    }
    $208 = $lspxstr;
    $209 = $lspx;
    $210 = (($209) + ($208<<2)|0);
    $lspx = $210;
    $211 = $hspx;
    $212 = $p0;
    $213 = ($211|0)==($212|0);
    if ($213) {
     $214 = $hle2;
     $215 = ($214|0)!=(0);
     if ($215) {
      $hspxstr = 0;
      $hle2 = 0;
     } else {
      $216 = $pstr;
      $hspxstr = $216;
     }
    }
    $217 = $hspxstr;
    $218 = $hspx;
    $219 = (($218) + ($217<<2)|0);
    $hspx = $219;
    $220 = $i;
    $221 = (($220) + 1)|0;
    $i = $221;
   }
   $222 = $pix;
   $223 = (($222) + 1)|0;
   $pix = $223;
  }
  $224 = $da_ev;
  $225 = ($224|0)!=(0);
  if ($225) {
   $226 = $lspxstr;
   $lpxstr = $226;
   $227 = $lspx;
   $lpx = $227;
   $228 = $lle2;
   $lle = $228;
   $229 = $lre2;
   $lre = $229;
   $230 = $lpx;
   $231 = +HEAPF32[$230>>2];
   $232 = $8;
   $233 = +HEAPF32[$232>>2];
   $234 = $231 * $233;
   $235 = $lopass;
   HEAPF32[$235>>2] = $234;
   $i = 1;
   while(1) {
    $236 = $i;
    $237 = $9;
    $238 = ($236|0)<($237|0);
    if (!($238)) {
     break;
    }
    $239 = $lpx;
    $240 = $p0;
    $241 = ($239|0)==($240|0);
    if ($241) {
     $242 = $lle;
     $243 = ($242|0)!=(0);
     if ($243) {
      $lpxstr = 0;
      $lle = 0;
     } else {
      $244 = $pstr;
      $lpxstr = $244;
     }
    }
    $245 = $lpx;
    $246 = $p1;
    $247 = ($245|0)==($246|0);
    if ($247) {
     $248 = $lre;
     $249 = ($248|0)!=(0);
     if ($249) {
      $lpxstr = 0;
      $lre = 0;
     } else {
      $250 = $nstr;
      $lpxstr = $250;
     }
    }
    $251 = $lpxstr;
    $252 = $lpx;
    $253 = (($252) + ($251<<2)|0);
    $lpx = $253;
    $254 = $lpx;
    $255 = +HEAPF32[$254>>2];
    $256 = $i;
    $257 = $8;
    $258 = (($257) + ($256<<2)|0);
    $259 = +HEAPF32[$258>>2];
    $260 = $255 * $259;
    $261 = $lopass;
    $262 = +HEAPF32[$261>>2];
    $263 = $262 + $260;
    HEAPF32[$261>>2] = $263;
    $264 = $i;
    $265 = (($264) + 1)|0;
    $i = $265;
   }
   $266 = $5;
   $267 = $lopass;
   $268 = (($267) + ($266<<2)|0);
   $lopass = $268;
  }
  $269 = $rw_cl;
  $270 = (($269) + 1)|0;
  $rw_cl = $270;
 }
 $271 = $fi_ev;
 $272 = ($271|0)!=(0);
 if ($272) {
  STACKTOP = sp;return;
 }
 $i = 0;
 while(1) {
  $273 = $i;
  $274 = $7;
  $275 = ($273|0)<($274|0);
  if (!($275)) {
   break;
  }
  $276 = $i;
  $277 = $6;
  $278 = (($277) + ($276<<2)|0);
  $279 = +HEAPF32[$278>>2];
  $280 = $279;
  $281 = $280 * -1.0;
  $282 = $281;
  HEAPF32[$278>>2] = $282;
  $283 = $i;
  $284 = (($283) + 1)|0;
  $i = $284;
 }
 STACKTOP = sp;return;
}
function _putc_huffman_table($marker,$table_id,$huffbits,$huffvalues,$outbuf,$outalloc,$outlen) {
 $marker = $marker|0;
 $table_id = $table_id|0;
 $huffbits = $huffbits|0;
 $huffvalues = $huffvalues|0;
 $outbuf = $outbuf|0;
 $outalloc = $outalloc|0;
 $outlen = $outlen|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $ret = 0, $table_len = 0;
 var $values_offset = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer14 = 0, $vararg_buffer3 = 0, $vararg_buffer6 = 0, $vararg_ptr13 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer14 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $marker;
 $2 = $table_id;
 $3 = $huffbits;
 $4 = $huffvalues;
 $5 = $outbuf;
 $6 = $outalloc;
 $7 = $outlen;
 $8 = HEAP32[8>>2]|0;
 $9 = ($8|0)>(0);
 if ($9) {
  $10 = HEAP32[_stdout>>2]|0;
  (_fprintf(($10|0),(11544|0),($vararg_buffer|0))|0);
 }
 $11 = $1;
 $12 = $5;
 $13 = $6;
 $14 = $7;
 $15 = (_putc_ushort($11,$12,$13,$14)|0);
 $ret = $15;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = $ret;
  $0 = $17;
  $116 = $0;
  STACKTOP = sp;return ($116|0);
 }
 $values_offset = 19;
 $table_len = 19;
 $i = 0;
 while(1) {
  $18 = $i;
  $19 = ($18|0)<(16);
  if (!($19)) {
   break;
  }
  $20 = $i;
  $21 = $3;
  $22 = (($21) + ($20)|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  $25 = $table_len;
  $26 = $25&65535;
  $27 = (($26) + ($24))|0;
  $28 = $27&65535;
  $table_len = $28;
  $29 = $i;
  $30 = (($29) + 1)|0;
  $i = $30;
 }
 $31 = HEAP32[8>>2]|0;
 $32 = ($31|0)>(1);
 if ($32) {
  $33 = HEAP32[_stdout>>2]|0;
  $34 = $table_len;
  $35 = $34&65535;
  HEAP32[$vararg_buffer1>>2] = $35;
  (_fprintf(($33|0),(11472|0),($vararg_buffer1|0))|0);
  $36 = HEAP32[_stdout>>2]|0;
  $37 = $2;
  $38 = $37&255;
  HEAP32[$vararg_buffer3>>2] = $38;
  (_fprintf(($36|0),(11488|0),($vararg_buffer3|0))|0);
  $i = 0;
  while(1) {
   $39 = $i;
   $40 = ($39|0)<(16);
   if (!($40)) {
    break;
   }
   $41 = HEAP32[_stdout>>2]|0;
   $42 = $i;
   $43 = $i;
   $44 = $3;
   $45 = (($44) + ($43)|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = $46&255;
   HEAP32[$vararg_buffer6>>2] = $42;
   $vararg_ptr9 = (($vararg_buffer6) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $47;
   (_fprintf(($41|0),(11504|0),($vararg_buffer6|0))|0);
   $48 = $i;
   $49 = (($48) + 1)|0;
   $i = $49;
  }
  $i = 0;
  while(1) {
   $50 = $i;
   $51 = $table_len;
   $52 = $51&65535;
   $53 = $values_offset;
   $54 = $53&65535;
   $55 = (($52) - ($54))|0;
   $56 = ($50|0)<($55|0);
   if (!($56)) {
    break;
   }
   $57 = HEAP32[_stdout>>2]|0;
   $58 = $i;
   $59 = $i;
   $60 = $4;
   $61 = (($60) + ($59)|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   HEAP32[$vararg_buffer10>>2] = $58;
   $vararg_ptr13 = (($vararg_buffer10) + 4|0);
   HEAP32[$vararg_ptr13>>2] = $63;
   (_fprintf(($57|0),(11520|0),($vararg_buffer10|0))|0);
   $64 = $i;
   $65 = (($64) + 1)|0;
   $i = $65;
  }
 }
 $66 = $table_len;
 $67 = $5;
 $68 = $6;
 $69 = $7;
 $70 = (_putc_ushort($66,$67,$68,$69)|0);
 $ret = $70;
 $71 = ($70|0)!=(0);
 if ($71) {
  $72 = $ret;
  $0 = $72;
  $116 = $0;
  STACKTOP = sp;return ($116|0);
 }
 $73 = $2;
 $74 = $5;
 $75 = $6;
 $76 = $7;
 $77 = (_putc_byte($73,$74,$75,$76)|0);
 $ret = $77;
 $78 = ($77|0)!=(0);
 if ($78) {
  $79 = $ret;
  $0 = $79;
  $116 = $0;
  STACKTOP = sp;return ($116|0);
 }
 $i = 0;
 while(1) {
  $80 = $i;
  $81 = ($80|0)<(16);
  if (!($81)) {
   break;
  }
  $82 = $i;
  $83 = $3;
  $84 = (($83) + ($82)|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = $5;
  $87 = $6;
  $88 = $7;
  $89 = (_putc_byte($85,$86,$87,$88)|0);
  $ret = $89;
  $90 = ($89|0)!=(0);
  if ($90) {
   label = 26;
   break;
  }
  $92 = $i;
  $93 = (($92) + 1)|0;
  $i = $93;
 }
 if ((label|0) == 26) {
  $91 = $ret;
  $0 = $91;
  $116 = $0;
  STACKTOP = sp;return ($116|0);
 }
 $i = 0;
 while(1) {
  $94 = $i;
  $95 = $table_len;
  $96 = $95&65535;
  $97 = $values_offset;
  $98 = $97&65535;
  $99 = (($96) - ($98))|0;
  $100 = ($94|0)<($99|0);
  if (!($100)) {
   break;
  }
  $101 = $i;
  $102 = $4;
  $103 = (($102) + ($101)|0);
  $104 = HEAP8[$103>>0]|0;
  $105 = $5;
  $106 = $6;
  $107 = $7;
  $108 = (_putc_byte($104,$105,$106,$107)|0);
  $ret = $108;
  $109 = ($108|0)!=(0);
  if ($109) {
   label = 32;
   break;
  }
  $111 = $i;
  $112 = (($111) + 1)|0;
  $i = $112;
 }
 if ((label|0) == 32) {
  $110 = $ret;
  $0 = $110;
  $116 = $0;
  STACKTOP = sp;return ($116|0);
 }
 $113 = HEAP32[8>>2]|0;
 $114 = ($113|0)>(0);
 if ($114) {
  $115 = HEAP32[_stdout>>2]|0;
  (_fprintf(($115|0),(11576|0),($vararg_buffer14|0))|0);
 }
 $0 = 0;
 $116 = $0;
 STACKTOP = sp;return ($116|0);
}
function _find_huff_sizes($ocodesize,$freq,$max_huffcounts) {
 $ocodesize = $ocodesize|0;
 $freq = $freq|0;
 $max_huffcounts = $max_huffcounts|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $codesize = 0, $i = 0, $others = 0, $value1 = 0, $value2 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $value1 = sp + 32|0;
 $value2 = sp + 28|0;
 $1 = $ocodesize;
 $2 = $freq;
 $3 = $max_huffcounts;
 $4 = $3;
 $5 = (($4) + 1)|0;
 $6 = (_calloc($5,4)|0);
 $codesize = $6;
 $7 = $codesize;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = HEAP32[_stderr>>2]|0;
  (_fprintf(($9|0),(11616|0),($vararg_buffer|0))|0);
  $0 = -2;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 $10 = $3;
 $11 = (($10) + 1)|0;
 $12 = $11<<2;
 $13 = (_malloc($12)|0);
 $others = $13;
 $14 = $others;
 $15 = ($14|0)==(0|0);
 if ($15) {
  $16 = HEAP32[_stderr>>2]|0;
  (_fprintf(($16|0),(11664|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 $i = 0;
 while(1) {
  $17 = $i;
  $18 = $3;
  $19 = ($17|0)<=($18|0);
  if (!($19)) {
   break;
  }
  $20 = $i;
  $21 = $others;
  $22 = (($21) + ($20<<2)|0);
  HEAP32[$22>>2] = -1;
  $23 = $i;
  $24 = (($23) + 1)|0;
  $i = $24;
 }
 while(1) {
  $25 = $2;
  $26 = $3;
  _find_least_freq($value1,$value2,$25,$26);
  $27 = HEAP32[$value2>>2]|0;
  $28 = ($27|0)==(-1);
  if ($28) {
   break;
  }
  $43 = HEAP32[$value2>>2]|0;
  $44 = $2;
  $45 = (($44) + ($43<<2)|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$value1>>2]|0;
  $48 = $2;
  $49 = (($48) + ($47<<2)|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + ($46))|0;
  HEAP32[$49>>2] = $51;
  $52 = HEAP32[$value2>>2]|0;
  $53 = $2;
  $54 = (($53) + ($52<<2)|0);
  HEAP32[$54>>2] = 0;
  $55 = HEAP32[$value1>>2]|0;
  $56 = $codesize;
  $57 = (($56) + ($55<<2)|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$57>>2] = $59;
  while(1) {
   $60 = HEAP32[$value1>>2]|0;
   $61 = $others;
   $62 = (($61) + ($60<<2)|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($63|0)!=(-1);
   if (!($64)) {
    break;
   }
   $65 = HEAP32[$value1>>2]|0;
   $66 = $others;
   $67 = (($66) + ($65<<2)|0);
   $68 = HEAP32[$67>>2]|0;
   HEAP32[$value1>>2] = $68;
   $69 = HEAP32[$value1>>2]|0;
   $70 = $codesize;
   $71 = (($70) + ($69<<2)|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = (($72) + 1)|0;
   HEAP32[$71>>2] = $73;
  }
  $74 = HEAP32[$value2>>2]|0;
  $75 = HEAP32[$value1>>2]|0;
  $76 = $others;
  $77 = (($76) + ($75<<2)|0);
  HEAP32[$77>>2] = $74;
  $78 = HEAP32[$value2>>2]|0;
  $79 = $codesize;
  $80 = (($79) + ($78<<2)|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 1)|0;
  HEAP32[$80>>2] = $82;
  while(1) {
   $83 = HEAP32[$value2>>2]|0;
   $84 = $others;
   $85 = (($84) + ($83<<2)|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = ($86|0)!=(-1);
   if (!($87)) {
    break;
   }
   $88 = HEAP32[$value2>>2]|0;
   $89 = $others;
   $90 = (($89) + ($88<<2)|0);
   $91 = HEAP32[$90>>2]|0;
   HEAP32[$value2>>2] = $91;
   $92 = HEAP32[$value2>>2]|0;
   $93 = $codesize;
   $94 = (($93) + ($92<<2)|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = (($95) + 1)|0;
   HEAP32[$94>>2] = $96;
  }
 }
 $29 = $others;
 _free($29);
 $30 = HEAP32[8>>2]|0;
 $31 = ($30|0)>(2);
 if ($31) {
  $i = 0;
  while(1) {
   $32 = $i;
   $33 = $3;
   $34 = ($32|0)<=($33|0);
   if (!($34)) {
    break;
   }
   $35 = HEAP32[_stdout>>2]|0;
   $36 = $i;
   $37 = $i;
   $38 = $codesize;
   $39 = (($38) + ($37<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$vararg_buffer3>>2] = $36;
   $vararg_ptr5 = (($vararg_buffer3) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $40;
   (_fprintf(($35|0),(11712|0),($vararg_buffer3|0))|0);
   $41 = $i;
   $42 = (($41) + 1)|0;
   $i = $42;
  }
 }
 $97 = $codesize;
 $98 = $1;
 HEAP32[$98>>2] = $97;
 $0 = 0;
 $99 = $0;
 STACKTOP = sp;return ($99|0);
}
function _find_least_freq($value1,$value2,$freq,$max_huffcounts) {
 $value1 = $value1|0;
 $value2 = $value2|0;
 $freq = $freq|0;
 $max_huffcounts = $max_huffcounts|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $code1 = 0, $code2 = 0, $code_temp = 0;
 var $i = 0, $set = 0, $value_temp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = $value1;
 $1 = $value2;
 $2 = $freq;
 $3 = $max_huffcounts;
 $code2 = 0;
 $code1 = 0;
 $set = 1;
 $4 = $0;
 HEAP32[$4>>2] = -1;
 $5 = $1;
 HEAP32[$5>>2] = -1;
 $i = 0;
 while(1) {
  $6 = $i;
  $7 = $3;
  $8 = ($6|0)<=($7|0);
  if (!($8)) {
   break;
  }
  $9 = $i;
  $10 = $2;
  $11 = (($10) + ($9<<2)|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)==(0);
  L4: do {
   if ($13) {
   } else {
    $14 = $set;
    $15 = ($14|0)==(1);
    if ($15) {
     $16 = $i;
     $17 = $2;
     $18 = (($17) + ($16<<2)|0);
     $19 = HEAP32[$18>>2]|0;
     $code1 = $19;
     $20 = $i;
     $21 = $0;
     HEAP32[$21>>2] = $20;
     $22 = $set;
     $23 = (($22) + 1)|0;
     $set = $23;
     break;
    }
    $24 = $set;
    $25 = ($24|0)==(2);
    if ($25) {
     $26 = $i;
     $27 = $2;
     $28 = (($27) + ($26<<2)|0);
     $29 = HEAP32[$28>>2]|0;
     $code2 = $29;
     $30 = $i;
     $31 = $1;
     HEAP32[$31>>2] = $30;
     $32 = $set;
     $33 = (($32) + 1)|0;
     $set = $33;
    }
    $34 = $i;
    $35 = $2;
    $36 = (($35) + ($34<<2)|0);
    $37 = HEAP32[$36>>2]|0;
    $code_temp = $37;
    $38 = $i;
    $value_temp = $38;
    $39 = $code1;
    $40 = $code_temp;
    $41 = ($39|0)<($40|0);
    if ($41) {
     $42 = $code2;
     $43 = $code_temp;
     $44 = ($42|0)<($43|0);
     if ($44) {
      break;
     }
    }
    $45 = $code_temp;
    $46 = $code1;
    $47 = ($45|0)<($46|0);
    do {
     if (!($47)) {
      $48 = $code_temp;
      $49 = $code1;
      $50 = ($48|0)==($49|0);
      if ($50) {
       $51 = $value_temp;
       $52 = $0;
       $53 = HEAP32[$52>>2]|0;
       $54 = ($51|0)>($53|0);
       if ($54) {
        break;
       }
      }
      $62 = $code_temp;
      $63 = $code2;
      $64 = ($62|0)<($63|0);
      if ($64) {
       label = 19;
      } else {
       $65 = $code_temp;
       $66 = $code2;
       $67 = ($65|0)==($66|0);
       if ($67) {
        $68 = $value_temp;
        $69 = $1;
        $70 = HEAP32[$69>>2]|0;
        $71 = ($68|0)>($70|0);
        if ($71) {
         label = 19;
        }
       }
      }
      if ((label|0) == 19) {
       label = 0;
       $72 = $code_temp;
       $code2 = $72;
       $73 = $value_temp;
       $74 = $1;
       HEAP32[$74>>2] = $73;
      }
      break L4;
     }
    } while(0);
    $55 = $code1;
    $code2 = $55;
    $56 = $0;
    $57 = HEAP32[$56>>2]|0;
    $58 = $1;
    HEAP32[$58>>2] = $57;
    $59 = $code_temp;
    $code1 = $59;
    $60 = $value_temp;
    $61 = $0;
    HEAP32[$61>>2] = $60;
   }
  } while(0);
  $75 = $i;
  $76 = (($75) + 1)|0;
  $i = $76;
 }
 STACKTOP = sp;return;
}
function _find_num_huff_sizes($obits,$adjust,$codesize,$max_huffcounts) {
 $obits = $obits|0;
 $adjust = $adjust|0;
 $codesize = $codesize|0;
 $max_huffcounts = $max_huffcounts|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bits = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_ptr3 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $obits;
 $2 = $adjust;
 $3 = $codesize;
 $4 = $max_huffcounts;
 $5 = $2;
 HEAP32[$5>>2] = 0;
 $6 = (_calloc(32,1)|0);
 $bits = $6;
 $7 = $bits;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = HEAP32[_stderr>>2]|0;
  (_fprintf(($9|0),(11736|0),($vararg_buffer|0))|0);
  $0 = -2;
  $53 = $0;
  STACKTOP = sp;return ($53|0);
 }
 $i = 0;
 while(1) {
  $10 = $i;
  $11 = $4;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = $i;
  $14 = $3;
  $15 = (($14) + ($13<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = $i;
   $19 = $3;
   $20 = (($19) + ($18<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) - 1)|0;
   $23 = $bits;
   $24 = (($23) + ($22)|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = (($25) + 1)<<24>>24;
   HEAP8[$24>>0] = $26;
  }
  $27 = $i;
  $28 = $3;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)>(16);
  if ($31) {
   $32 = $2;
   HEAP32[$32>>2] = 1;
  }
  $33 = $i;
  $34 = (($33) + 1)|0;
  $i = $34;
 }
 $35 = HEAP32[8>>2]|0;
 $36 = ($35|0)>(2);
 if ($36) {
  $i = 0;
  while(1) {
   $37 = $i;
   $38 = ($37|0)<(32);
   if (!($38)) {
    break;
   }
   $39 = HEAP32[_stdout>>2]|0;
   $40 = $i;
   $41 = $i;
   $42 = $bits;
   $43 = (($42) + ($41)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   HEAP32[$vararg_buffer1>>2] = $40;
   $vararg_ptr3 = (($vararg_buffer1) + 4|0);
   HEAP32[$vararg_ptr3>>2] = $45;
   (_fprintf(($39|0),(11504|0),($vararg_buffer1|0))|0);
   $46 = $i;
   $47 = (($46) + 1)|0;
   $i = $47;
  }
  $48 = HEAP32[_stdout>>2]|0;
  $49 = $2;
  $50 = HEAP32[$49>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $50;
  (_fprintf(($48|0),(11784|0),($vararg_buffer4|0))|0);
 }
 $51 = $bits;
 $52 = $1;
 HEAP32[$52>>2] = $51;
 $0 = 0;
 $53 = $0;
 STACKTOP = sp;return ($53|0);
}
function _sort_huffbits($bits) {
 $bits = $bits|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $j = 0, $l1 = 0, $l2 = 0, $l3 = 0, $tbits = 0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $bits;
 $l3 = 32;
 $2 = $l3;
 $3 = (($2) - 1)|0;
 $l1 = $3;
 $l2 = 15;
 $4 = $l3;
 $5 = $4<<1;
 $6 = (_malloc($5)|0);
 $tbits = $6;
 $7 = $tbits;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = HEAP32[_stderr>>2]|0;
  (_fprintf(($9|0),(11800|0),($vararg_buffer|0))|0);
  $0 = -2;
  $132 = $0;
  STACKTOP = sp;return ($132|0);
 }
 $i = 0;
 while(1) {
  $10 = $i;
  $11 = ($10|0)<(32);
  if (!($11)) {
   break;
  }
  $12 = $i;
  $13 = $1;
  $14 = (($13) + ($12)|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = $i;
  $18 = $tbits;
  $19 = (($18) + ($17<<1)|0);
  HEAP16[$19>>1] = $16;
  $20 = $i;
  $21 = (($20) + 1)|0;
  $i = $21;
 }
 $22 = $l1;
 $i = $22;
 while(1) {
  $23 = $i;
  $24 = $l2;
  $25 = ($23|0)>($24|0);
  if (!($25)) {
   break;
  }
  while(1) {
   $26 = $i;
   $27 = $tbits;
   $28 = (($27) + ($26<<1)|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29 << 16 >> 16;
   $31 = ($30|0)>(0);
   if (!($31)) {
    break;
   }
   $32 = $i;
   $33 = (($32) - 2)|0;
   $j = $33;
   while(1) {
    $34 = $j;
    $35 = $tbits;
    $36 = (($35) + ($34<<1)|0);
    $37 = HEAP16[$36>>1]|0;
    $38 = $37 << 16 >> 16;
    $39 = ($38|0)==(0);
    if (!($39)) {
     break;
    }
    $40 = $j;
    $41 = (($40) + -1)|0;
    $j = $41;
   }
   $42 = $i;
   $43 = $tbits;
   $44 = (($43) + ($42<<1)|0);
   $45 = HEAP16[$44>>1]|0;
   $46 = $45 << 16 >> 16;
   $47 = (($46) - 2)|0;
   $48 = $47&65535;
   HEAP16[$44>>1] = $48;
   $49 = $i;
   $50 = (($49) - 1)|0;
   $51 = $tbits;
   $52 = (($51) + ($50<<1)|0);
   $53 = HEAP16[$52>>1]|0;
   $54 = $53 << 16 >> 16;
   $55 = (($54) + 1)|0;
   $56 = $55&65535;
   HEAP16[$52>>1] = $56;
   $57 = $j;
   $58 = (($57) + 1)|0;
   $59 = $tbits;
   $60 = (($59) + ($58<<1)|0);
   $61 = HEAP16[$60>>1]|0;
   $62 = $61 << 16 >> 16;
   $63 = (($62) + 2)|0;
   $64 = $63&65535;
   HEAP16[$60>>1] = $64;
   $65 = $j;
   $66 = $tbits;
   $67 = (($66) + ($65<<1)|0);
   $68 = HEAP16[$67>>1]|0;
   $69 = $68 << 16 >> 16;
   $70 = (($69) - 1)|0;
   $71 = $70&65535;
   HEAP16[$67>>1] = $71;
  }
  $72 = $i;
  $73 = $tbits;
  $74 = (($73) + ($72<<1)|0);
  HEAP16[$74>>1] = 0;
  $75 = $i;
  $76 = (($75) + -1)|0;
  $i = $76;
 }
 while(1) {
  $77 = $i;
  $78 = $tbits;
  $79 = (($78) + ($77<<1)|0);
  $80 = HEAP16[$79>>1]|0;
  $81 = $80 << 16 >> 16;
  $82 = ($81|0)==(0);
  if (!($82)) {
   break;
  }
  $83 = $i;
  $84 = (($83) + -1)|0;
  $i = $84;
 }
 $85 = $i;
 $86 = $tbits;
 $87 = (($86) + ($85<<1)|0);
 $88 = HEAP16[$87>>1]|0;
 $89 = $88 << 16 >> 16;
 $90 = (($89) - 1)|0;
 $91 = $90&65535;
 HEAP16[$87>>1] = $91;
 $i = 0;
 while(1) {
  $92 = $i;
  $93 = ($92|0)<(32);
  if (!($93)) {
   break;
  }
  $94 = $i;
  $95 = $tbits;
  $96 = (($95) + ($94<<1)|0);
  $97 = HEAP16[$96>>1]|0;
  $98 = $97&255;
  $99 = $i;
  $100 = $1;
  $101 = (($100) + ($99)|0);
  HEAP8[$101>>0] = $98;
  $102 = $i;
  $103 = (($102) + 1)|0;
  $i = $103;
 }
 $104 = $tbits;
 _free($104);
 $i = 16;
 while(1) {
  $105 = $i;
  $106 = $l3;
  $107 = ($105|0)<($106|0);
  if (!($107)) {
   break;
  }
  $108 = $i;
  $109 = $1;
  $110 = (($109) + ($108)|0);
  $111 = HEAP8[$110>>0]|0;
  $112 = $111&255;
  $113 = ($112|0)>(0);
  if ($113) {
   label = 27;
   break;
  }
  $116 = $i;
  $117 = (($116) + 1)|0;
  $i = $117;
 }
 if ((label|0) == 27) {
  $114 = HEAP32[_stderr>>2]|0;
  $115 = $i;
  HEAP32[$vararg_buffer1>>2] = $115;
  (_fprintf(($114|0),(11840|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $132 = $0;
  STACKTOP = sp;return ($132|0);
 }
 $118 = HEAP32[8>>2]|0;
 $119 = ($118|0)>(1);
 if ($119) {
  $120 = HEAP32[_stdout>>2]|0;
  (_fprintf(($120|0),(11904|0),($vararg_buffer3|0))|0);
  $i = 0;
  while(1) {
   $121 = $i;
   $122 = ($121|0)<(32);
   if (!($122)) {
    break;
   }
   $123 = HEAP32[_stdout>>2]|0;
   $124 = $i;
   $125 = $i;
   $126 = $1;
   $127 = (($126) + ($125)|0);
   $128 = HEAP8[$127>>0]|0;
   $129 = $128&255;
   HEAP32[$vararg_buffer5>>2] = $124;
   $vararg_ptr8 = (($vararg_buffer5) + 4|0);
   HEAP32[$vararg_ptr8>>2] = $129;
   (_fprintf(($123|0),(11936|0),($vararg_buffer5|0))|0);
   $130 = $i;
   $131 = (($130) + 1)|0;
   $i = $131;
  }
 }
 $0 = 0;
 $132 = $0;
 STACKTOP = sp;return ($132|0);
}
function _sort_code_sizes($ovalues,$codesize,$max_huffcounts) {
 $ovalues = $ovalues|0;
 $codesize = $codesize|0;
 $max_huffcounts = $max_huffcounts|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $i2 = 0, $i3 = 0, $values = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ovalues;
 $2 = $codesize;
 $3 = $max_huffcounts;
 $i2 = 0;
 $4 = $3;
 $5 = (($4) + 1)|0;
 $6 = (_calloc($5,1)|0);
 $values = $6;
 $7 = $values;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = HEAP32[_stderr>>2]|0;
  (_fprintf(($9|0),(11960|0),($vararg_buffer|0))|0);
  $0 = -2;
  $48 = $0;
  STACKTOP = sp;return ($48|0);
 }
 $i = 1;
 while(1) {
  $10 = $i;
  $11 = ($10|0)<=(32);
  if (!($11)) {
   break;
  }
  $i3 = 0;
  while(1) {
   $12 = $i3;
   $13 = $3;
   $14 = ($12|0)<($13|0);
   if (!($14)) {
    break;
   }
   $15 = $i3;
   $16 = $2;
   $17 = (($16) + ($15<<2)|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = $i;
   $20 = ($18|0)==($19|0);
   if ($20) {
    $21 = $i3;
    $22 = $21&255;
    $23 = $i2;
    $24 = $values;
    $25 = (($24) + ($23)|0);
    HEAP8[$25>>0] = $22;
    $26 = $i2;
    $27 = (($26) + 1)|0;
    $i2 = $27;
   }
   $28 = $i3;
   $29 = (($28) + 1)|0;
   $i3 = $29;
  }
  $30 = $i;
  $31 = (($30) + 1)|0;
  $i = $31;
 }
 $32 = HEAP32[8>>2]|0;
 $33 = ($32|0)>(2);
 if ($33) {
  $i = 0;
  while(1) {
   $34 = $i;
   $35 = $3;
   $36 = ($34|0)<=($35|0);
   if (!($36)) {
    break;
   }
   $37 = HEAP32[_stdout>>2]|0;
   $38 = $i;
   $39 = $i;
   $40 = $values;
   $41 = (($40) + ($39)|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = $42&255;
   HEAP32[$vararg_buffer1>>2] = $38;
   $vararg_ptr3 = (($vararg_buffer1) + 4|0);
   HEAP32[$vararg_ptr3>>2] = $43;
   (_fprintf(($37|0),(11520|0),($vararg_buffer1|0))|0);
   $44 = $i;
   $45 = (($44) + 1)|0;
   $i = $45;
  }
 }
 $46 = $values;
 $47 = $1;
 HEAP32[$47>>2] = $46;
 $0 = 0;
 $48 = $0;
 STACKTOP = sp;return ($48|0);
}
function _build_huffcode_table($ohuffcode_table,$in_huffcode_table,$last_size,$values,$max_huffcounts) {
 $ohuffcode_table = $ohuffcode_table|0;
 $in_huffcode_table = $in_huffcode_table|0;
 $last_size = $last_size|0;
 $values = $values|0;
 $max_huffcounts = $max_huffcounts|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $new_huffcode_table = 0, $size = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_ptr3 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ohuffcode_table;
 $2 = $in_huffcode_table;
 $3 = $last_size;
 $4 = $values;
 $5 = $max_huffcounts;
 $6 = $5;
 $7 = (($6) + 1)|0;
 $8 = (_calloc($7,8)|0);
 $new_huffcode_table = $8;
 $9 = $new_huffcode_table;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[_stderr>>2]|0;
  (_fprintf(($11|0),(12008|0),($vararg_buffer|0))|0);
  $0 = -2;
  $64 = $0;
  STACKTOP = sp;return ($64|0);
 }
 $size = 0;
 while(1) {
  $12 = $size;
  $13 = $3;
  $14 = ($12|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = $2;
  $16 = $size;
  $17 = (($15) + ($16<<3)|0);
  $18 = (($17) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $new_huffcode_table;
  $21 = $size;
  $22 = $4;
  $23 = (($22) + ($21)|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = (($20) + ($25<<3)|0);
  $27 = (($26) + 4|0);
  HEAP32[$27>>2] = $19;
  $28 = $2;
  $29 = $size;
  $30 = (($28) + ($29<<3)|0);
  $31 = HEAP16[$30>>1]|0;
  $32 = $new_huffcode_table;
  $33 = $size;
  $34 = $4;
  $35 = (($34) + ($33)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = (($32) + ($37<<3)|0);
  HEAP16[$38>>1] = $31;
  $39 = $size;
  $40 = (($39) + 1)|0;
  $size = $40;
 }
 $41 = HEAP32[8>>2]|0;
 $42 = ($41|0)>(3);
 if ($42) {
  $size = 0;
  while(1) {
   $43 = $size;
   $44 = $5;
   $45 = ($43|0)<=($44|0);
   if (!($45)) {
    break;
   }
   $46 = HEAP32[_stdout>>2]|0;
   $47 = $size;
   $48 = $size;
   $49 = $new_huffcode_table;
   $50 = (($49) + ($48<<3)|0);
   $51 = HEAP16[$50>>1]|0;
   $52 = $51 << 16 >> 16;
   HEAP32[$vararg_buffer1>>2] = $47;
   $vararg_ptr3 = (($vararg_buffer1) + 4|0);
   HEAP32[$vararg_ptr3>>2] = $52;
   (_fprintf(($46|0),(12072|0),($vararg_buffer1|0))|0);
   $53 = HEAP32[_stdout>>2]|0;
   $54 = $size;
   $55 = $size;
   $56 = $new_huffcode_table;
   $57 = (($56) + ($55<<3)|0);
   $58 = (($57) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   HEAP32[$vararg_buffer4>>2] = $54;
   $vararg_ptr7 = (($vararg_buffer4) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $59;
   (_fprintf(($53|0),(12096|0),($vararg_buffer4|0))|0);
   $60 = $size;
   $61 = (($60) + 1)|0;
   $size = $61;
  }
 }
 $62 = $new_huffcode_table;
 $63 = $1;
 HEAP32[$63>>2] = $62;
 $0 = 0;
 $64 = $0;
 STACKTOP = sp;return ($64|0);
}
function _build_huffsizes($ohuffcode_table,$temp_size,$huffbits,$max_huffcounts) {
 $ohuffcode_table = $ohuffcode_table|0;
 $temp_size = $temp_size|0;
 $huffbits = $huffbits|0;
 $max_huffcounts = $max_huffcounts|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $code_size = 0;
 var $huffcode_table = 0, $ii = 0, $number_of_codes = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ohuffcode_table;
 $2 = $temp_size;
 $3 = $huffbits;
 $4 = $max_huffcounts;
 $number_of_codes = 1;
 $5 = $4;
 $6 = (($5) + 1)|0;
 $7 = (_calloc($6,8)|0);
 $huffcode_table = $7;
 $8 = $huffcode_table;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  (_fprintf(($10|0),(12120|0),($vararg_buffer|0))|0);
  $0 = -2;
  $58 = $0;
  STACKTOP = sp;return ($58|0);
 }
 $11 = $2;
 HEAP32[$11>>2] = 0;
 $code_size = 1;
 while(1) {
  $12 = $code_size;
  $13 = ($12|0)<=(16);
  if (!($13)) {
   break;
  }
  while(1) {
   $14 = $number_of_codes;
   $15 = $code_size;
   $16 = (($15) - 1)|0;
   $17 = $3;
   $18 = (($17) + ($16)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = ($14|0)<=($20|0);
   if (!($21)) {
    break;
   }
   $22 = $code_size;
   $23 = $22&65535;
   $24 = $huffcode_table;
   $25 = $2;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($24) + ($26<<3)|0);
   HEAP16[$27>>1] = $23;
   $28 = $2;
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + 1)|0;
   HEAP32[$28>>2] = $30;
   $31 = $number_of_codes;
   $32 = (($31) + 1)|0;
   $number_of_codes = $32;
  }
  $number_of_codes = 1;
  $33 = $code_size;
  $34 = (($33) + 1)|0;
  $code_size = $34;
 }
 $35 = $huffcode_table;
 $36 = $2;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + ($37<<3)|0);
 HEAP16[$38>>1] = 0;
 $39 = HEAP32[8>>2]|0;
 $40 = ($39|0)>(2);
 if ($40) {
  $41 = HEAP32[_stderr>>2]|0;
  (_fprintf(($41|0),(12176|0),($vararg_buffer1|0))|0);
  $ii = 0;
  while(1) {
   $42 = $ii;
   $43 = $4;
   $44 = (($43) + 1)|0;
   $45 = ($42|0)<($44|0);
   if (!($45)) {
    break;
   }
   $46 = HEAP32[_stderr>>2]|0;
   $47 = $ii;
   $48 = $ii;
   $49 = $huffcode_table;
   $50 = (($49) + ($48<<3)|0);
   $51 = HEAP16[$50>>1]|0;
   $52 = $51 << 16 >> 16;
   HEAP32[$vararg_buffer3>>2] = $47;
   $vararg_ptr5 = (($vararg_buffer3) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $52;
   (_fprintf(($46|0),(12200|0),($vararg_buffer3|0))|0);
   $53 = $ii;
   $54 = (($53) + 1)|0;
   $ii = $54;
  }
  $55 = HEAP32[_stderr>>2]|0;
  (_fflush(($55|0))|0);
 }
 $56 = $huffcode_table;
 $57 = $1;
 HEAP32[$57>>2] = $56;
 $0 = 0;
 $58 = $0;
 STACKTOP = sp;return ($58|0);
}
function _build_huffcodes($huffcode_table) {
 $huffcode_table = $huffcode_table|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pointer = 0, $temp_code = 0, $temp_size = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $huffcode_table;
 $pointer = 0;
 $temp_code = 0;
 $1 = $0;
 $2 = HEAP16[$1>>1]|0;
 $temp_size = $2;
 $3 = $0;
 $4 = $pointer;
 $5 = (($3) + ($4<<3)|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6 << 16 >> 16;
 $8 = ($7|0)==(0);
 if ($8) {
  STACKTOP = sp;return;
 }
 while(1) {
  while(1) {
   $9 = $temp_code;
   $10 = $9&65535;
   $11 = $0;
   $12 = $pointer;
   $13 = (($11) + ($12<<3)|0);
   $14 = (($13) + 4|0);
   HEAP32[$14>>2] = $10;
   $15 = $temp_code;
   $16 = (($15) + 1)<<16>>16;
   $temp_code = $16;
   $17 = $pointer;
   $18 = (($17) + 1)|0;
   $pointer = $18;
   $19 = $0;
   $20 = $pointer;
   $21 = (($19) + ($20<<3)|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = $temp_size;
   $25 = $24 << 16 >> 16;
   $26 = ($23|0)==($25|0);
   if (!($26)) {
    break;
   }
  }
  $27 = $0;
  $28 = $pointer;
  $29 = (($27) + ($28<<3)|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30 << 16 >> 16;
  $32 = ($31|0)==(0);
  if ($32) {
   break;
  }
  while(1) {
   $33 = $temp_code;
   $34 = $33&65535;
   $35 = $34 << 1;
   $36 = $35&65535;
   $temp_code = $36;
   $37 = $temp_size;
   $38 = (($37) + 1)<<16>>16;
   $temp_size = $38;
   $39 = $0;
   $40 = $pointer;
   $41 = (($39) + ($40<<3)|0);
   $42 = HEAP16[$41>>1]|0;
   $43 = $42 << 16 >> 16;
   $44 = $temp_size;
   $45 = $44 << 16 >> 16;
   $46 = ($43|0)!=($45|0);
   if (!($46)) {
    break;
   }
  }
  $47 = $0;
  $48 = $pointer;
  $49 = (($47) + ($48<<3)|0);
  $50 = HEAP16[$49>>1]|0;
  $51 = $50 << 16 >> 16;
  $52 = $temp_size;
  $53 = $52 << 16 >> 16;
  $54 = ($51|0)==($53|0);
  if (!($54)) {
   label = 14;
   break;
  }
 }
 if ((label|0) == 14) {
  STACKTOP = sp;return;
 }
 STACKTOP = sp;return;
}
function _putc_comment($marker,$comment,$cs,$odata,$oalloc,$olen) {
 $marker = $marker|0;
 $comment = $comment|0;
 $cs = $cs|0;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $hdr_size = 0, $i = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $marker;
 $2 = $comment;
 $3 = $cs;
 $4 = $odata;
 $5 = $oalloc;
 $6 = $olen;
 $7 = HEAP32[8>>2]|0;
 $8 = ($7|0)>(0);
 if ($8) {
  $9 = HEAP32[_stderr>>2]|0;
  (_fprintf(($9|0),(12216|0),($vararg_buffer|0))|0);
 }
 $10 = $1;
 $11 = $4;
 $12 = $5;
 $13 = $6;
 $14 = (_putc_ushort($10,$11,$12,$13)|0);
 $ret = $14;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = $ret;
  $0 = $16;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 $17 = $3;
 $18 = (2 + ($17))|0;
 $19 = $18&65535;
 $hdr_size = $19;
 $20 = $hdr_size;
 $21 = $4;
 $22 = $5;
 $23 = $6;
 $24 = (_putc_ushort($20,$21,$22,$23)|0);
 $ret = $24;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = $ret;
  $0 = $26;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 $i = 0;
 while(1) {
  $27 = $i;
  $28 = $3;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = $i;
  $31 = $2;
  $32 = (($31) + ($30)|0);
  $33 = HEAP8[$32>>0]|0;
  $34 = $4;
  $35 = $5;
  $36 = $6;
  $37 = (_putc_byte($33,$34,$35,$36)|0);
  $ret = $37;
  $38 = ($37|0)!=(0);
  if ($38) {
   label = 10;
   break;
  }
  $40 = $i;
  $41 = (($40) + 1)|0;
  $i = $41;
 }
 if ((label|0) == 10) {
  $39 = $ret;
  $0 = $39;
  $45 = $0;
  STACKTOP = sp;return ($45|0);
 }
 $42 = HEAP32[8>>2]|0;
 $43 = ($42|0)>(0);
 if ($43) {
  $44 = HEAP32[_stderr>>2]|0;
  (_fprintf(($44|0),(12256|0),($vararg_buffer1|0))|0);
 }
 $0 = 0;
 $45 = $0;
 STACKTOP = sp;return ($45|0);
}
function _freefet($fet) {
 $fet = $fet|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $item = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $fet;
 $item = 0;
 while(1) {
  $1 = $item;
  $2 = $0;
  $3 = (($2) + 4|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = $item;
  $7 = $0;
  $8 = (($7) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _free($11);
  $12 = $item;
  $13 = $0;
  $14 = (($13) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  _free($17);
  $18 = $item;
  $19 = (($18) + 1)|0;
  $item = $19;
 }
 $20 = $0;
 $21 = (($20) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 _free($22);
 $23 = $0;
 $24 = (($23) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 _free($25);
 $26 = $0;
 _free($26);
 STACKTOP = sp;return;
}
function _combine_nistcom($onistcom,$w,$h,$d,$ppi,$lossyflag) {
 $onistcom = $onistcom|0;
 $w = $w|0;
 $h = $h|0;
 $d = $d|0;
 $ppi = $ppi|0;
 $lossyflag = $lossyflag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $allocflag = 0, $cbuff = 0, $lossyval = 0, $n = 0, $nistcom = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer14 = 0, $vararg_buffer17 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer17 = sp + 56|0;
 $vararg_buffer14 = sp + 48|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $lossyval = sp + 64|0;
 $cbuff = sp + 108|0;
 $nistcom = sp + 60|0;
 $1 = $onistcom;
 $2 = $w;
 $3 = $h;
 $4 = $d;
 $5 = $ppi;
 $6 = $lossyflag;
 $7 = $1;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0|0);
 do {
  if ($9) {
   $10 = (_allocfet_ret($nistcom,6)|0);
   $ret = $10;
   $11 = ($10|0)!=(0);
   if ($11) {
    $12 = $ret;
    $0 = $12;
    $99 = $0;
    STACKTOP = sp;return ($99|0);
   }
   $allocflag = 1;
   $13 = HEAP32[$nistcom>>2]|0;
   $14 = (_updatefet_ret(12304,12320,$13)|0);
   $ret = $14;
   $15 = ($14|0)!=(0);
   if (!($15)) {
    break;
   }
   $16 = $allocflag;
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = HEAP32[$nistcom>>2]|0;
    _freefet($18);
    $19 = $1;
    HEAP32[$19>>2] = 0;
   }
   $20 = $ret;
   $0 = $20;
   $99 = $0;
   STACKTOP = sp;return ($99|0);
  } else {
   $21 = $1;
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$nistcom>>2] = $22;
   $allocflag = 0;
   $n = 6;
  }
 } while(0);
 $23 = $2;
 HEAP32[$vararg_buffer>>2] = $23;
 (_sprintf($cbuff,12328,$vararg_buffer)|0);
 $24 = HEAP32[$nistcom>>2]|0;
 $25 = (_updatefet_ret(12336,$cbuff,$24)|0);
 $ret = $25;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = $allocflag;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$nistcom>>2]|0;
   _freefet($29);
   $30 = $1;
   HEAP32[$30>>2] = 0;
  }
  $31 = $ret;
  $0 = $31;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 $32 = $3;
 HEAP32[$vararg_buffer1>>2] = $32;
 (_sprintf($cbuff,12328,$vararg_buffer1)|0);
 $33 = HEAP32[$nistcom>>2]|0;
 $34 = (_updatefet_ret(12352,$cbuff,$33)|0);
 $ret = $34;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = $allocflag;
  $37 = ($36|0)!=(0);
  if ($37) {
   $38 = HEAP32[$nistcom>>2]|0;
   _freefet($38);
   $39 = $1;
   HEAP32[$39>>2] = 0;
  }
  $40 = $ret;
  $0 = $40;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 $41 = $4;
 HEAP32[$vararg_buffer4>>2] = $41;
 (_sprintf($cbuff,12328,$vararg_buffer4)|0);
 $42 = HEAP32[$nistcom>>2]|0;
 $43 = (_updatefet_ret(12368,$cbuff,$42)|0);
 $ret = $43;
 $44 = ($43|0)!=(0);
 if ($44) {
  $45 = $allocflag;
  $46 = ($45|0)!=(0);
  if ($46) {
   $47 = HEAP32[$nistcom>>2]|0;
   _freefet($47);
   $48 = $1;
   HEAP32[$48>>2] = 0;
  }
  $49 = $ret;
  $0 = $49;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 $50 = $5;
 HEAP32[$vararg_buffer7>>2] = $50;
 (_sprintf($cbuff,12328,$vararg_buffer7)|0);
 $51 = HEAP32[$nistcom>>2]|0;
 $52 = (_updatefet_ret(12384,$cbuff,$51)|0);
 $ret = $52;
 $53 = ($52|0)!=(0);
 if ($53) {
  $54 = $allocflag;
  $55 = ($54|0)!=(0);
  if ($55) {
   $56 = HEAP32[$nistcom>>2]|0;
   _freefet($56);
   $57 = $1;
   HEAP32[$57>>2] = 0;
  }
  $58 = $ret;
  $0 = $58;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 $59 = HEAP32[$nistcom>>2]|0;
 $60 = (_lookupfet($lossyval,12392,$59)|0);
 $ret = $60;
 $61 = $ret;
 $62 = ($61|0)<(0);
 if ($62) {
  $63 = $allocflag;
  $64 = ($63|0)!=(0);
  if ($64) {
   $65 = HEAP32[$nistcom>>2]|0;
   _freefet($65);
   $66 = $1;
   HEAP32[$66>>2] = 0;
  }
  $67 = $ret;
  $0 = $67;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 $68 = $ret;
 $69 = ($68|0)!=(0);
 if ($69) {
  $70 = HEAP32[$lossyval>>2]|0;
  $71 = (_strcmp($70,12400)|0);
  $72 = ($71|0)!=(0);
  if ($72) {
   $73 = $6;
   $74 = ($73|0)==(0);
   if ($74) {
    $75 = HEAP32[_stderr>>2]|0;
    (_fprintf(($75|0),(12408|0),($vararg_buffer10|0))|0);
    $76 = HEAP32[_stderr>>2]|0;
    (_fprintf(($76|0),(12440|0),($vararg_buffer12|0))|0);
   } else {
    label = 34;
   }
  } else {
   label = 34;
  }
 } else {
  label = 34;
 }
 do {
  if ((label|0) == 34) {
   $77 = $6;
   HEAP32[$vararg_buffer14>>2] = $77;
   (_sprintf($cbuff,12328,$vararg_buffer14)|0);
   $78 = HEAP32[$nistcom>>2]|0;
   $79 = (_updatefet_ret(12392,$cbuff,$78)|0);
   $ret = $79;
   $80 = ($79|0)!=(0);
   if (!($80)) {
    break;
   }
   $81 = $allocflag;
   $82 = ($81|0)!=(0);
   if ($82) {
    $83 = HEAP32[$nistcom>>2]|0;
    _freefet($83);
    $84 = $1;
    HEAP32[$84>>2] = 0;
   }
   $85 = $ret;
   $0 = $85;
   $99 = $0;
   STACKTOP = sp;return ($99|0);
  }
 } while(0);
 $86 = HEAP32[$nistcom>>2]|0;
 $87 = (($86) + 4|0);
 $88 = HEAP32[$87>>2]|0;
 HEAP32[$vararg_buffer17>>2] = $88;
 (_sprintf($cbuff,12328,$vararg_buffer17)|0);
 $89 = HEAP32[$nistcom>>2]|0;
 $90 = (_updatefet_ret(12304,$cbuff,$89)|0);
 $ret = $90;
 $91 = ($90|0)!=(0);
 if (!($91)) {
  $97 = HEAP32[$nistcom>>2]|0;
  $98 = $1;
  HEAP32[$98>>2] = $97;
  $0 = 0;
  $99 = $0;
  STACKTOP = sp;return ($99|0);
 }
 $92 = $allocflag;
 $93 = ($92|0)!=(0);
 if ($93) {
  $94 = HEAP32[$nistcom>>2]|0;
  _freefet($94);
  $95 = $1;
  HEAP32[$95>>2] = 0;
 }
 $96 = $ret;
 $0 = $96;
 $99 = $0;
 STACKTOP = sp;return ($99|0);
}
function _combine_wsq_nistcom($onistcom,$w,$h,$d,$ppi,$lossyflag,$r_bitrate) {
 $onistcom = $onistcom|0;
 $w = $w|0;
 $h = $h|0;
 $d = $d|0;
 $ppi = $ppi|0;
 $lossyflag = $lossyflag|0;
 $r_bitrate = +$r_bitrate;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0.0, $37 = 0.0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0.0, $8 = 0, $9 = 0, $allocflag = 0, $cbuff = 0, $nistcom = 0, $ret = 0;
 var $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 576|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $cbuff = sp + 56|0;
 $1 = $onistcom;
 $2 = $w;
 $3 = $h;
 $4 = $d;
 $5 = $ppi;
 $6 = $lossyflag;
 $7 = $r_bitrate;
 $8 = $1;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $allocflag = 1;
 } else {
  $allocflag = 0;
 }
 $11 = $1;
 $12 = $2;
 $13 = $3;
 $14 = $4;
 $15 = $5;
 $16 = $6;
 $17 = (_combine_nistcom($11,$12,$13,$14,$15,$16)|0);
 $ret = $17;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = $ret;
  $0 = $19;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $20 = $1;
 $21 = HEAP32[$20>>2]|0;
 $nistcom = $21;
 $22 = $nistcom;
 $23 = (_updatefet_ret(12480,12496,$22)|0);
 $ret = $23;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = $allocflag;
  $26 = ($25|0)!=(0);
  if ($26) {
   $27 = $nistcom;
   _freefet($27);
  }
  $28 = $ret;
  $0 = $28;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $29 = $nistcom;
 $30 = (_updatefet_ret(12504,12520,$29)|0);
 $ret = $30;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = $allocflag;
  $33 = ($32|0)!=(0);
  if ($33) {
   $34 = $nistcom;
   _freefet($34);
  }
  $35 = $ret;
  $0 = $35;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $36 = $7;
 $37 = $36;
 HEAPF64[tempDoublePtr>>3]=$37;HEAP32[$vararg_buffer>>2]=HEAP32[tempDoublePtr>>2];HEAP32[$vararg_buffer+4>>2]=HEAP32[tempDoublePtr+4>>2];
 (_sprintf($cbuff,12528,$vararg_buffer)|0);
 $38 = $nistcom;
 $39 = (_updatefet_ret(12536,$cbuff,$38)|0);
 $ret = $39;
 $40 = ($39|0)!=(0);
 if ($40) {
  $41 = $allocflag;
  $42 = ($41|0)!=(0);
  if ($42) {
   $43 = $nistcom;
   _freefet($43);
  }
  $44 = $ret;
  $0 = $44;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $45 = $nistcom;
 $46 = (($45) + 4|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$vararg_buffer1>>2] = $47;
 (_sprintf($cbuff,12328,$vararg_buffer1)|0);
 $48 = $nistcom;
 $49 = (_updatefet_ret(12304,$cbuff,$48)|0);
 $ret = $49;
 $50 = ($49|0)!=(0);
 if (!($50)) {
  $0 = 0;
  $55 = $0;
  STACKTOP = sp;return ($55|0);
 }
 $51 = $allocflag;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = $nistcom;
  _freefet($53);
 }
 $54 = $ret;
 $0 = $54;
 $55 = $0;
 STACKTOP = sp;return ($55|0);
}
function _fet2string($ostr,$fet) {
 $ostr = $ostr|0;
 $fet = $fet|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $size = 0, $str = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $ostr;
 $2 = $fet;
 $size = 0;
 $i = 0;
 while(1) {
  $3 = $i;
  $4 = $2;
  $5 = (($4) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = $i;
  $9 = $2;
  $10 = (($9) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_strlen(($13|0))|0);
  $15 = $size;
  $16 = (($15) + ($14))|0;
  $size = $16;
  $17 = $i;
  $18 = $2;
  $19 = (($18) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (($20) + ($17<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (_strlen(($22|0))|0);
  $24 = $size;
  $25 = (($24) + ($23))|0;
  $size = $25;
  $26 = $size;
  $27 = (($26) + 2)|0;
  $size = $27;
  $28 = $i;
  $29 = (($28) + 1)|0;
  $i = $29;
 }
 $30 = $size;
 $31 = (($30) + 1)|0;
 $size = $31;
 $32 = $size;
 $33 = (_calloc($32,1)|0);
 $str = $33;
 $34 = ($33|0)==(0|0);
 if ($34) {
  $35 = HEAP32[_stderr>>2]|0;
  (_fprintf(($35|0),(12552|0),($vararg_buffer|0))|0);
  $0 = -2;
  $66 = $0;
  STACKTOP = sp;return ($66|0);
 }
 $i = 0;
 while(1) {
  $36 = $i;
  $37 = $2;
  $38 = (($37) + 4|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ($36|0)<($39|0);
  if (!($40)) {
   break;
  }
  $41 = $str;
  $42 = $i;
  $43 = $2;
  $44 = (($43) + 8|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<2)|0);
  $47 = HEAP32[$46>>2]|0;
  (_strcat(($41|0),($47|0))|0);
  $48 = $str;
  (_strcat(($48|0),(12592|0))|0);
  $49 = $str;
  $50 = $i;
  $51 = $2;
  $52 = (($51) + 12|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($50<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  (_strcat(($49|0),($55|0))|0);
  $56 = $str;
  (_strcat(($56|0),(12600|0))|0);
  $57 = $i;
  $58 = (($57) + 1)|0;
  $i = $58;
 }
 $59 = $str;
 $60 = (_strlen(($59|0))|0);
 $61 = (($60) - 1)|0;
 $62 = $str;
 $63 = (($62) + ($61)|0);
 HEAP8[$63>>0] = 0;
 $64 = $str;
 $65 = $1;
 HEAP32[$65>>2] = $64;
 $0 = 0;
 $66 = $0;
 STACKTOP = sp;return ($66|0);
}
function _string2fet($ofet,$istr) {
 $ofet = $ofet|0;
 $istr = $istr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $fet = 0, $iptr = 0, $name = 0, $optr = 0, $ret = 0;
 var $value = 0, $vararg_buffer = 0, $vptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1072|0;
 $vararg_buffer = sp;
 $name = sp + 552|0;
 $value = sp + 40|0;
 $fet = sp + 4|0;
 $1 = $ofet;
 $2 = $istr;
 $3 = (_allocfet_ret($fet,100)|0);
 $ret = $3;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = $ret;
  $0 = $5;
  $88 = $0;
  STACKTOP = sp;return ($88|0);
 }
 $6 = $2;
 $iptr = $6;
 while(1) {
  $7 = $iptr;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   label = 39;
   break;
  }
  $optr = $name;
  while(1) {
   $11 = $iptr;
   $12 = HEAP8[$11>>0]|0;
   $13 = $12 << 24 >> 24;
   $14 = ($13|0)!=(0);
   if ($14) {
    $15 = $iptr;
    $16 = HEAP8[$15>>0]|0;
    $17 = $16 << 24 >> 24;
    $18 = ($17|0)!=(32);
    if ($18) {
     $19 = $iptr;
     $20 = HEAP8[$19>>0]|0;
     $21 = $20 << 24 >> 24;
     $22 = ($21|0)!=(9);
     $89 = $22;
    } else {
     $89 = 0;
    }
   } else {
    $89 = 0;
   }
   if (!($89)) {
    break;
   }
   $23 = $iptr;
   $24 = (($23) + 1|0);
   $iptr = $24;
   $25 = HEAP8[$23>>0]|0;
   $26 = $optr;
   $27 = (($26) + 1|0);
   $optr = $27;
   HEAP8[$26>>0] = $25;
  }
  $28 = $optr;
  HEAP8[$28>>0] = 0;
  while(1) {
   $29 = $iptr;
   $30 = HEAP8[$29>>0]|0;
   $31 = $30 << 24 >> 24;
   $32 = ($31|0)!=(0);
   if ($32) {
    $33 = $iptr;
    $34 = HEAP8[$33>>0]|0;
    $35 = $34 << 24 >> 24;
    $36 = ($35|0)==(32);
    if ($36) {
     $91 = 1;
    } else {
     $37 = $iptr;
     $38 = HEAP8[$37>>0]|0;
     $39 = $38 << 24 >> 24;
     $40 = ($39|0)==(9);
     $91 = $40;
    }
    $90 = $91;
   } else {
    $90 = 0;
   }
   if (!($90)) {
    break;
   }
   $41 = $iptr;
   $42 = (($41) + 1|0);
   $iptr = $42;
  }
  $optr = $value;
  while(1) {
   $43 = $iptr;
   $44 = HEAP8[$43>>0]|0;
   $45 = $44 << 24 >> 24;
   $46 = ($45|0)!=(0);
   if ($46) {
    $47 = $iptr;
    $48 = HEAP8[$47>>0]|0;
    $49 = $48 << 24 >> 24;
    $50 = ($49|0)!=(10);
    $92 = $50;
   } else {
    $92 = 0;
   }
   if (!($92)) {
    break;
   }
   $51 = $iptr;
   $52 = (($51) + 1|0);
   $iptr = $52;
   $53 = HEAP8[$51>>0]|0;
   $54 = $optr;
   $55 = (($54) + 1|0);
   $optr = $55;
   HEAP8[$54>>0] = $53;
  }
  $56 = $optr;
  HEAP8[$56>>0] = 0;
  while(1) {
   $57 = $iptr;
   $58 = HEAP8[$57>>0]|0;
   $59 = $58 << 24 >> 24;
   $60 = ($59|0)!=(0);
   if ($60) {
    $61 = $iptr;
    $62 = HEAP8[$61>>0]|0;
    $63 = $62 << 24 >> 24;
    $64 = ($63|0)==(32);
    if ($64) {
     $94 = 1;
    } else {
     $65 = $iptr;
     $66 = HEAP8[$65>>0]|0;
     $67 = $66 << 24 >> 24;
     $68 = ($67|0)==(9);
     if ($68) {
      $94 = 1;
     } else {
      $69 = $iptr;
      $70 = HEAP8[$69>>0]|0;
      $71 = $70 << 24 >> 24;
      $72 = ($71|0)==(10);
      $94 = $72;
     }
    }
    $93 = $94;
   } else {
    $93 = 0;
   }
   if (!($93)) {
    break;
   }
   $73 = $iptr;
   $74 = (($73) + 1|0);
   $iptr = $74;
  }
  $75 = (_strlen(($name|0))|0);
  $76 = ($75|0)==(0);
  if ($76) {
   label = 32;
   break;
  }
  $78 = (_strlen(($value|0))|0);
  $79 = ($78|0)==(0);
  if ($79) {
   $vptr = 0;
  } else {
   $vptr = $value;
  }
  $80 = $vptr;
  $81 = HEAP32[$fet>>2]|0;
  $82 = (_updatefet_ret($name,$80,$81)|0);
  $ret = $82;
  $83 = ($82|0)!=(0);
  if ($83) {
   label = 37;
   break;
  }
 }
 if ((label|0) == 32) {
  $77 = HEAP32[_stderr>>2]|0;
  (_fprintf(($77|0),(12608|0),($vararg_buffer|0))|0);
  $0 = -2;
  $88 = $0;
  STACKTOP = sp;return ($88|0);
 }
 else if ((label|0) == 37) {
  $84 = HEAP32[$fet>>2]|0;
  _freefet($84);
  $85 = $ret;
  $0 = $85;
  $88 = $0;
  STACKTOP = sp;return ($88|0);
 }
 else if ((label|0) == 39) {
  $86 = HEAP32[$fet>>2]|0;
  $87 = $1;
  HEAP32[$87>>2] = $86;
  $0 = 0;
  $88 = $0;
  STACKTOP = sp;return ($88|0);
 }
 return 0|0;
}
function _updatefet_ret($feature,$value,$fet) {
 $feature = $feature|0;
 $value = $value|0;
 $fet = $fet|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $incr = 0, $increased = 0, $item = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $3 = sp + 36|0;
 $1 = $feature;
 $2 = $value;
 HEAP32[$3>>2] = $fet;
 $item = 0;
 while(1) {
  $4 = $item;
  $5 = HEAP32[$3>>2]|0;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if ($8) {
   $9 = $item;
   $10 = HEAP32[$3>>2]|0;
   $11 = (($10) + 8|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + ($9<<2)|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $1;
   $16 = (_strcmp($14,$15)|0);
   $17 = ($16|0)!=(0);
   $125 = $17;
  } else {
   $125 = 0;
  }
  if (!($125)) {
   break;
  }
  $18 = $item;
  $19 = (($18) + 1)|0;
  $item = $19;
 }
 $20 = $item;
 $21 = HEAP32[$3>>2]|0;
 $22 = (($21) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($20|0)<($23|0);
 if ($24) {
  $25 = $item;
  $26 = HEAP32[$3>>2]|0;
  $27 = (($26) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($25<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)!=(0|0);
  if ($31) {
   $32 = $item;
   $33 = HEAP32[$3>>2]|0;
   $34 = (($33) + 12|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32<<2)|0);
   $37 = HEAP32[$36>>2]|0;
   _free($37);
   $38 = $item;
   $39 = HEAP32[$3>>2]|0;
   $40 = (($39) + 12|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + ($38<<2)|0);
   HEAP32[$42>>2] = 0;
  }
  $43 = $2;
  $44 = ($43|0)!=(0|0);
  do {
   if ($44) {
    $45 = $2;
    $46 = (___strdup($45)|0);
    $47 = $item;
    $48 = HEAP32[$3>>2]|0;
    $49 = (($48) + 12|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + ($47<<2)|0);
    HEAP32[$51>>2] = $46;
    $52 = $item;
    $53 = HEAP32[$3>>2]|0;
    $54 = (($53) + 12|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + ($52<<2)|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)==(0|0);
    if (!($58)) {
     break;
    }
    $59 = HEAP32[_stderr>>2]|0;
    (_fprintf(($59|0),(12656|0),($vararg_buffer|0))|0);
    $0 = -2;
    $124 = $0;
    STACKTOP = sp;return ($124|0);
   }
  } while(0);
 } else {
  $60 = HEAP32[$3>>2]|0;
  $61 = (($60) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  $64 = HEAP32[$63>>2]|0;
  $65 = ($62|0)>=($64|0);
  do {
   if ($65) {
    $66 = HEAP32[$3>>2]|0;
    $67 = HEAP32[$66>>2]|0;
    $68 = (($67|0) / 10)&-1;
    $incr = $68;
    $69 = HEAP32[$3>>2]|0;
    $70 = HEAP32[$69>>2]|0;
    $71 = $incr;
    $72 = (10)>($71|0);
    if ($72) {
     $75 = 10;
    } else {
     $73 = $incr;
     $75 = $73;
    }
    $74 = (($70) + ($75))|0;
    $increased = $74;
    $76 = $increased;
    $77 = (_reallocfet_ret($3,$76)|0);
    $ret = $77;
    $78 = ($77|0)!=(0);
    if (!($78)) {
     break;
    }
    $79 = $ret;
    $0 = $79;
    $124 = $0;
    STACKTOP = sp;return ($124|0);
   }
  } while(0);
  $80 = $1;
  $81 = (___strdup($80)|0);
  $82 = HEAP32[$3>>2]|0;
  $83 = (($82) + 4|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = HEAP32[$3>>2]|0;
  $86 = (($85) + 8|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (($87) + ($84<<2)|0);
  HEAP32[$88>>2] = $81;
  $89 = HEAP32[$3>>2]|0;
  $90 = (($89) + 4|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = HEAP32[$3>>2]|0;
  $93 = (($92) + 8|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (($94) + ($91<<2)|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)==(0|0);
  if ($97) {
   $98 = HEAP32[_stderr>>2]|0;
   (_fprintf(($98|0),(12704|0),($vararg_buffer1|0))|0);
   $0 = -3;
   $124 = $0;
   STACKTOP = sp;return ($124|0);
  }
  $99 = $2;
  $100 = ($99|0)!=(0|0);
  do {
   if ($100) {
    $101 = $2;
    $102 = (___strdup($101)|0);
    $103 = HEAP32[$3>>2]|0;
    $104 = (($103) + 4|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = HEAP32[$3>>2]|0;
    $107 = (($106) + 12|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = (($108) + ($105<<2)|0);
    HEAP32[$109>>2] = $102;
    $110 = HEAP32[$3>>2]|0;
    $111 = (($110) + 4|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = HEAP32[$3>>2]|0;
    $114 = (($113) + 12|0);
    $115 = HEAP32[$114>>2]|0;
    $116 = (($115) + ($112<<2)|0);
    $117 = HEAP32[$116>>2]|0;
    $118 = ($117|0)==(0|0);
    if (!($118)) {
     break;
    }
    $119 = HEAP32[_stderr>>2]|0;
    (_fprintf(($119|0),(12656|0),($vararg_buffer3|0))|0);
    $0 = -4;
    $124 = $0;
    STACKTOP = sp;return ($124|0);
   }
  } while(0);
  $120 = HEAP32[$3>>2]|0;
  $121 = (($120) + 4|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = (($122) + 1)|0;
  HEAP32[$121>>2] = $123;
 }
 $0 = 0;
 $124 = $0;
 STACKTOP = sp;return ($124|0);
}
function _allocfet_ret($ofet,$numfeatures) {
 $ofet = $ofet|0;
 $numfeatures = $numfeatures|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fet = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ofet;
 $2 = $numfeatures;
 $3 = (_malloc(16)|0);
 $fet = $3;
 $4 = $fet;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  (_fprintf(($6|0),(12752|0),($vararg_buffer|0))|0);
  $0 = -2;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 $7 = $2;
 $8 = (_calloc($7,4)|0);
 $9 = $fet;
 $10 = (($9) + 8|0);
 HEAP32[$10>>2] = $8;
 $11 = $fet;
 $12 = (($11) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  (_fprintf(($15|0),(12792|0),($vararg_buffer1|0))|0);
  $16 = $fet;
  _free($16);
  $0 = -3;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 $17 = $2;
 $18 = (_calloc($17,4)|0);
 $19 = $fet;
 $20 = (($19) + 12|0);
 HEAP32[$20>>2] = $18;
 $21 = $fet;
 $22 = (($21) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)==(0|0);
 if ($24) {
  $25 = HEAP32[_stderr>>2]|0;
  (_fprintf(($25|0),(12840|0),($vararg_buffer3|0))|0);
  $26 = $fet;
  $27 = (($26) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  _free($28);
  $29 = $fet;
  _free($29);
  $0 = -4;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 } else {
  $30 = $2;
  $31 = $fet;
  HEAP32[$31>>2] = $30;
  $32 = $fet;
  $33 = (($32) + 4|0);
  HEAP32[$33>>2] = 0;
  $34 = $fet;
  $35 = $1;
  HEAP32[$35>>2] = $34;
  $0 = 0;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 return 0|0;
}
function _reallocfet_ret($ofet,$newlen) {
 $ofet = $ofet|0;
 $newlen = $newlen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fet = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ofet;
 $2 = $newlen;
 $3 = $1;
 $4 = HEAP32[$3>>2]|0;
 $fet = $4;
 $5 = $fet;
 $6 = ($5|0)==(0|0);
 if (!($6)) {
  $7 = $fet;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0);
  if (!($9)) {
   $15 = $fet;
   $16 = (($15) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $2;
   $19 = $18<<2;
   $20 = (_realloc($17,$19)|0);
   $21 = $fet;
   $22 = (($21) + 8|0);
   HEAP32[$22>>2] = $20;
   $23 = $fet;
   $24 = (($23) + 8|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)==(0|0);
   if ($26) {
    $27 = HEAP32[_stderr>>2]|0;
    (_fprintf(($27|0),(12888|0),($vararg_buffer|0))|0);
    $0 = -2;
    $43 = $0;
    STACKTOP = sp;return ($43|0);
   }
   $28 = $fet;
   $29 = (($28) + 12|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = $2;
   $32 = $31<<2;
   $33 = (_realloc($30,$32)|0);
   $34 = $fet;
   $35 = (($34) + 12|0);
   HEAP32[$35>>2] = $33;
   $36 = $fet;
   $37 = (($36) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ($38|0)==(0|0);
   if ($39) {
    $40 = HEAP32[_stderr>>2]|0;
    (_fprintf(($40|0),(12936|0),($vararg_buffer1|0))|0);
    $0 = -3;
    $43 = $0;
    STACKTOP = sp;return ($43|0);
   } else {
    $41 = $2;
    $42 = $fet;
    HEAP32[$42>>2] = $41;
    $0 = 0;
    $43 = $0;
    STACKTOP = sp;return ($43|0);
   }
  }
 }
 $10 = $1;
 $11 = $2;
 $12 = (_allocfet_ret($10,$11)|0);
 $ret = $12;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = $ret;
  $0 = $14;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
 } else {
  $0 = 0;
  $43 = $0;
  STACKTOP = sp;return ($43|0);
 }
 return 0|0;
}
function _lookupfet($ovalue,$feature,$fet) {
 $ovalue = $ovalue|0;
 $feature = $feature|0;
 $fet = $fet|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $item = 0, $value = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $ovalue;
 $2 = $feature;
 $3 = $fet;
 $item = 0;
 while(1) {
  $4 = $item;
  $5 = $3;
  $6 = (($5) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if ($8) {
   $9 = $item;
   $10 = $3;
   $11 = (($10) + 8|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + ($9<<2)|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = $2;
   $16 = (_strcmp($14,$15)|0);
   $17 = ($16|0)!=(0);
   $45 = $17;
  } else {
   $45 = 0;
  }
  if (!($45)) {
   break;
  }
  $18 = $item;
  $19 = (($18) + 1)|0;
  $item = $19;
 }
 $20 = $item;
 $21 = $3;
 $22 = (($21) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($20|0)>=($23|0);
 if ($24) {
  $0 = 0;
  $44 = $0;
  STACKTOP = sp;return ($44|0);
 }
 $25 = $item;
 $26 = $3;
 $27 = (($26) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($25<<2)|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)!=(0|0);
 do {
  if ($31) {
   $32 = $item;
   $33 = $3;
   $34 = (($33) + 12|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32<<2)|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (___strdup($37)|0);
   $value = $38;
   $39 = $value;
   $40 = ($39|0)==(0|0);
   if (!($40)) {
    break;
   }
   $41 = HEAP32[_stderr>>2]|0;
   (_fprintf(($41|0),(12984|0),($vararg_buffer|0))|0);
   $0 = -2;
   $44 = $0;
   STACKTOP = sp;return ($44|0);
  } else {
   $value = 0;
  }
 } while(0);
 $42 = $value;
 $43 = $1;
 HEAP32[$43>>2] = $42;
 $0 = 1;
 $44 = $0;
 STACKTOP = sp;return ($44|0);
}
function _putc_byte($idata,$odata,$oalloc,$olen) {
 $idata = $idata|0;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $idata;
 $2 = $odata;
 $3 = $oalloc;
 $4 = $olen;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = $3;
 $8 = ($6|0)>=($7|0);
 if ($8) {
  $9 = HEAP32[_stderr>>2]|0;
  $10 = $3;
  $11 = $4;
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$vararg_buffer>>2] = $10;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $12;
  (_fprintf(($9|0),(13024|0),($vararg_buffer|0))|0);
  $0 = -32;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 } else {
  $13 = $1;
  $14 = $2;
  $15 = $4;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + ($16)|0);
  HEAP8[$17>>0] = $13;
  $18 = $4;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$18>>2] = $20;
  $0 = 0;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _putc_bytes($idata,$ilen,$odata,$oalloc,$olen) {
 $idata = $idata|0;
 $ilen = $ilen|0;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $1 = $idata;
 $2 = $ilen;
 $3 = $odata;
 $4 = $oalloc;
 $5 = $olen;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = $2;
 $9 = (($7) + ($8))|0;
 $10 = $4;
 $11 = ($9|0)>($10|0);
 if ($11) {
  $12 = HEAP32[_stderr>>2]|0;
  $13 = $4;
  $14 = $5;
  $15 = HEAP32[$14>>2]|0;
  $16 = $2;
  $17 = (($15) + ($16))|0;
  HEAP32[$vararg_buffer>>2] = $13;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $17;
  (_fprintf(($12|0),(13088|0),($vararg_buffer|0))|0);
  $0 = -33;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 } else {
  $18 = $3;
  $19 = $5;
  $20 = HEAP32[$19>>2]|0;
  $21 = (($18) + ($20)|0);
  $22 = $1;
  $23 = $2;
  _memcpy(($21|0),($22|0),($23|0))|0;
  $24 = $2;
  $25 = $5;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + ($24))|0;
  HEAP32[$25>>2] = $27;
  $0 = 0;
  $28 = $0;
  STACKTOP = sp;return ($28|0);
 }
 return 0|0;
}
function _putc_ushort($ishort,$odata,$oalloc,$olen) {
 $ishort = $ishort|0;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $_b_ = 0, $_f_ = 0, $_t_ = 0, $cptr = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = sp + 34|0;
 $_b_ = sp + 32|0;
 HEAP16[$1>>1] = $ishort;
 $2 = $odata;
 $3 = $oalloc;
 $4 = $olen;
 $5 = HEAP16[$1>>1]|0;
 HEAP16[$_b_>>1] = $5;
 $_f_ = $_b_;
 $_t_ = $1;
 $6 = $_f_;
 $7 = HEAP8[$6>>0]|0;
 $8 = $_t_;
 $9 = (($8) + 1|0);
 HEAP8[$9>>0] = $7;
 $10 = $_f_;
 $11 = (($10) + 1|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $_t_;
 HEAP8[$13>>0] = $12;
 $cptr = $1;
 $14 = $cptr;
 $15 = $2;
 $16 = $3;
 $17 = $4;
 $18 = (_putc_bytes($14,2,$15,$16,$17)|0);
 $ret = $18;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = $ret;
  $0 = $20;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 } else {
  $0 = 0;
  $21 = $0;
  STACKTOP = sp;return ($21|0);
 }
 return 0|0;
}
function _putc_uint($iint,$odata,$oalloc,$olen) {
 $iint = $iint|0;
 $odata = $odata|0;
 $oalloc = $oalloc|0;
 $olen = $olen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $_b_ = 0, $_f_ = 0, $_t_ = 0, $cptr = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $1 = sp + 32|0;
 $_b_ = sp + 8|0;
 HEAP32[$1>>2] = $iint;
 $2 = $odata;
 $3 = $oalloc;
 $4 = $olen;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$_b_>>2] = $5;
 $_f_ = $_b_;
 $_t_ = $1;
 $6 = $_f_;
 $7 = HEAP8[$6>>0]|0;
 $8 = $_t_;
 $9 = (($8) + 3|0);
 HEAP8[$9>>0] = $7;
 $10 = $_f_;
 $11 = (($10) + 1|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $_t_;
 $14 = (($13) + 2|0);
 HEAP8[$14>>0] = $12;
 $15 = $_f_;
 $16 = (($15) + 2|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $_t_;
 $19 = (($18) + 1|0);
 HEAP8[$19>>0] = $17;
 $20 = $_f_;
 $21 = (($20) + 3|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $_t_;
 HEAP8[$23>>0] = $22;
 $cptr = $1;
 $24 = $cptr;
 $25 = $2;
 $26 = $3;
 $27 = $4;
 $28 = (_putc_bytes($24,4,$25,$26,$27)|0);
 $ret = $28;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = $ret;
  $0 = $30;
  $31 = $0;
  STACKTOP = sp;return ($31|0);
 } else {
  $0 = 0;
  $31 = $0;
  STACKTOP = sp;return ($31|0);
 }
 return 0|0;
}
function _write_bits($outbuf,$code,$size,$outbit,$bits,$bytes) {
 $outbuf = $outbuf|0;
 $code = $code|0;
 $size = $size|0;
 $outbit = $outbit|0;
 $bits = $bits|0;
 $bytes = $bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $num = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $outbuf;
 $1 = $code;
 $2 = $size;
 $3 = $outbit;
 $4 = $bits;
 $5 = $bytes;
 $6 = $2;
 $num = $6;
 $7 = $num;
 $8 = (($7) + -1)<<16>>16;
 $num = $8;
 while(1) {
  $9 = $num;
  $10 = $9 << 16 >> 16;
  $11 = ($10|0)>=(0);
  if (!($11)) {
   break;
  }
  $12 = $4;
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = $14 << 1;
  $16 = $15&255;
  HEAP8[$12>>0] = $16;
  $17 = $1;
  $18 = $17&65535;
  $19 = $num;
  $20 = $19 << 16 >> 16;
  $21 = $18 >> $20;
  $22 = $21 & 1;
  $23 = $22&255;
  $24 = $23&255;
  $25 = $4;
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = $27 | $24;
  $29 = $28&255;
  HEAP8[$25>>0] = $29;
  $30 = $3;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + -1)|0;
  HEAP32[$30>>2] = $32;
  $33 = ($32|0)<(0);
  if ($33) {
   $34 = $4;
   $35 = HEAP8[$34>>0]|0;
   $36 = $0;
   $37 = HEAP32[$36>>2]|0;
   HEAP8[$37>>0] = $35;
   $38 = $0;
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + 1|0);
   HEAP32[$38>>2] = $40;
   $41 = $4;
   $42 = HEAP8[$41>>0]|0;
   $43 = $42&255;
   $44 = ($43|0)==(255);
   if ($44) {
    $45 = $0;
    $46 = HEAP32[$45>>2]|0;
    HEAP8[$46>>0] = 0;
    $47 = $0;
    $48 = HEAP32[$47>>2]|0;
    $49 = (($48) + 1|0);
    HEAP32[$47>>2] = $49;
    $50 = $5;
    $51 = HEAP32[$50>>2]|0;
    $52 = (($51) + 1)|0;
    HEAP32[$50>>2] = $52;
   }
   $53 = $5;
   $54 = HEAP32[$53>>2]|0;
   $55 = (($54) + 1)|0;
   HEAP32[$53>>2] = $55;
   $56 = $3;
   HEAP32[$56>>2] = 7;
   $57 = $4;
   HEAP8[$57>>0] = 0;
  }
  $58 = $num;
  $59 = (($58) + -1)<<16>>16;
  $num = $59;
 }
 STACKTOP = sp;return;
}
function _flush_bits($outbuf,$outbit,$bits,$bytes) {
 $outbuf = $outbuf|0;
 $outbit = $outbit|0;
 $bits = $bits|0;
 $bytes = $bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = $outbuf;
 $1 = $outbit;
 $2 = $bits;
 $3 = $bytes;
 $4 = $1;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(7);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = $1;
 $8 = HEAP32[$7>>2]|0;
 $cnt = $8;
 while(1) {
  $9 = $cnt;
  $10 = ($9|0)>=(0);
  if (!($10)) {
   break;
  }
  $11 = $2;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = $13 << 1;
  $15 = $14&255;
  HEAP8[$11>>0] = $15;
  $16 = $2;
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = $18 | 1;
  $20 = $19&255;
  HEAP8[$16>>0] = $20;
  $21 = $cnt;
  $22 = (($21) + -1)|0;
  $cnt = $22;
 }
 $23 = $2;
 $24 = HEAP8[$23>>0]|0;
 $25 = $0;
 $26 = HEAP32[$25>>2]|0;
 HEAP8[$26>>0] = $24;
 $27 = $0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + 1|0);
 HEAP32[$27>>2] = $29;
 $30 = $2;
 $31 = HEAP8[$30>>0]|0;
 $32 = $31&255;
 $33 = ($32|0)==(255);
 if ($33) {
  $34 = $2;
  HEAP8[$34>>0] = 0;
  $35 = $0;
  $36 = HEAP32[$35>>2]|0;
  HEAP8[$36>>0] = 0;
  $37 = $0;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + 1|0);
  HEAP32[$37>>2] = $39;
  $40 = $3;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$40>>2] = $42;
 }
 $43 = $3;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + 1)|0;
 HEAP32[$43>>2] = $45;
 $46 = $1;
 HEAP32[$46>>2] = 7;
 $47 = $2;
 HEAP8[$47>>0] = 0;
 STACKTOP = sp;return;
}
function _read_ascii_file($filename,$ostr) {
 $filename = $filename|0;
 $ostr = $ostr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fp = 0, $fsize = 0, $ret = 0;
 var $str = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $vararg_buffer5 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $filename;
 $2 = $ostr;
 $3 = $1;
 $4 = (_filesize($3)|0);
 $ret = $4;
 $5 = $ret;
 $6 = ($5|0)<(0);
 if ($6) {
  $7 = $ret;
  $0 = $7;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 $8 = $ret;
 $fsize = $8;
 $9 = $fsize;
 $10 = (($9) + 1)|0;
 $11 = (_calloc($10,1)|0);
 $str = $11;
 $12 = $str;
 $13 = ($12|0)==(0|0);
 if ($13) {
  $14 = HEAP32[_stderr>>2]|0;
  (_fprintf(($14|0),(13152|0),($vararg_buffer|0))|0);
  $0 = -2;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 $15 = $1;
 $16 = (_fopen(($15|0),(13192|0))|0);
 $fp = $16;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[_stderr>>2]|0;
  $19 = $1;
  HEAP32[$vararg_buffer1>>2] = $19;
  (_fprintf(($18|0),(13200|0),($vararg_buffer1|0))|0);
  $20 = $str;
  _free($20);
  $0 = -3;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 $21 = $str;
 $22 = $fsize;
 $23 = $fp;
 $24 = (_fread(($21|0),1,($22|0),($23|0))|0);
 $ret = $24;
 $25 = $ret;
 $26 = $fsize;
 $27 = ($25|0)!=($26|0);
 if ($27) {
  $28 = HEAP32[_stderr>>2]|0;
  (_fprintf(($28|0),(13240|0),($vararg_buffer3|0))|0);
  $29 = HEAP32[_stderr>>2]|0;
  $30 = $ret;
  $31 = $fsize;
  HEAP32[$vararg_buffer5>>2] = $30;
  $vararg_ptr8 = (($vararg_buffer5) + 4|0);
  HEAP32[$vararg_ptr8>>2] = $31;
  (_fprintf(($29|0),(13280|0),($vararg_buffer5|0))|0);
  $32 = $str;
  _free($32);
  $0 = -4;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 } else {
  $33 = $fp;
  (_fclose(($33|0))|0);
  $34 = $str;
  $35 = $2;
  HEAP32[$35>>2] = $34;
  $0 = 0;
  $36 = $0;
  STACKTOP = sp;return ($36|0);
 }
 return 0|0;
}
function _fileroot($file) {
 $file = $file|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $cptr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $file;
 $1 = $0;
 $2 = $0;
 $3 = (_strlen(($2|0))|0);
 $4 = (($1) + ($3)|0);
 $cptr = $4;
 while(1) {
  $5 = $cptr;
  $6 = $0;
  $7 = ($5|0)!=($6|0);
  if ($7) {
   $8 = $cptr;
   $9 = HEAP8[$8>>0]|0;
   $10 = $9 << 24 >> 24;
   $11 = ($10|0)!=(46);
   $19 = $11;
  } else {
   $19 = 0;
  }
  if (!($19)) {
   break;
  }
  $12 = $cptr;
  $13 = (($12) + -1|0);
  $cptr = $13;
 }
 $14 = $cptr;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)==(46);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = $cptr;
 HEAP8[$18>>0] = 0;
 STACKTOP = sp;return;
}
function _filesize($ifile) {
 $ifile = $ifile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $fp = 0, $ret = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $1 = $ifile;
 $2 = $1;
 $3 = (_fopen(($2|0),(13312|0))|0);
 $fp = $3;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = HEAP32[_stderr>>2]|0;
  $6 = $1;
  HEAP32[$vararg_buffer>>2] = $6;
  (_fprintf(($5|0),(13320|0),($vararg_buffer|0))|0);
  $0 = -2;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 $7 = $fp;
 $8 = (_fseek(($7|0),0,2)|0);
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  $11 = $1;
  HEAP32[$vararg_buffer1>>2] = $11;
  (_fprintf(($10|0),(13368|0),($vararg_buffer1|0))|0);
  $0 = -3;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 $12 = $fp;
 $13 = (_ftell(($12|0))|0);
 $ret = $13;
 $14 = ($13|0)<(0);
 if ($14) {
  $15 = HEAP32[_stderr>>2]|0;
  $16 = $1;
  HEAP32[$vararg_buffer4>>2] = $16;
  (_fprintf(($15|0),(13424|0),($vararg_buffer4|0))|0);
  $0 = -4;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 } else {
  $17 = $fp;
  (_fclose(($17|0))|0);
  $18 = $ret;
  $0 = $18;
  $19 = $0;
  STACKTOP = sp;return ($19|0);
 }
 return 0|0;
}
function _fatalerr($s1,$s2,$s3) {
 $s1 = $s1|0;
 $s2 = $s2|0;
 $s3 = $s3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $vararg_buffer5 = 0, $vararg_ptr4 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $s1;
 $1 = $s2;
 $2 = $s3;
 $3 = HEAP32[_stdout>>2]|0;
 (_fflush(($3|0))|0);
 $4 = $1;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  $7 = $0;
  HEAP32[$vararg_buffer>>2] = $7;
  (_fprintf(($6|0),(13480|0),($vararg_buffer|0))|0);
  $17 = HEAP32[_stderr>>2]|0;
  (_fflush(($17|0))|0);
  _exit(1);
  // unreachable;
 }
 $8 = $2;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[_stderr>>2]|0;
  $11 = $0;
  $12 = $1;
  HEAP32[$vararg_buffer1>>2] = $11;
  $vararg_ptr4 = (($vararg_buffer1) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $12;
  (_fprintf(($10|0),(13496|0),($vararg_buffer1|0))|0);
 } else {
  $13 = HEAP32[_stderr>>2]|0;
  $14 = $0;
  $15 = $1;
  $16 = $2;
  HEAP32[$vararg_buffer5>>2] = $14;
  $vararg_ptr8 = (($vararg_buffer5) + 4|0);
  HEAP32[$vararg_ptr8>>2] = $15;
  $vararg_ptr9 = (($vararg_buffer5) + 8|0);
  HEAP32[$vararg_ptr9>>2] = $16;
  (_fprintf(($13|0),(13512|0),($vararg_buffer5|0))|0);
 }
 $17 = HEAP32[_stderr>>2]|0;
 (_fflush(($17|0))|0);
 _exit(1);
 // unreachable;
}
function _inv_bytes($data,$nbytes) {
 $data = $data|0;
 $nbytes = $nbytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $data;
 $1 = $nbytes;
 while(1) {
  $2 = $1;
  $3 = ($2|0)!=(0);
  if (!($3)) {
   break;
  }
  $4 = $0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5&255;
  $7 = (13600 + ($6)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $0;
  HEAP8[$9>>0] = $8;
  $10 = $1;
  $11 = (($10) + -1)|0;
  $1 = $11;
  $12 = $0;
  $13 = (($12) + 1|0);
  $0 = $13;
 }
 STACKTOP = sp;return;
}
function _malloc_uchar($ptr,$n,$s) {
 $ptr = $ptr|0;
 $n = $n|0;
 $s = $s|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = $ptr;
 $1 = $n;
 $2 = $s;
 $3 = $1;
 $4 = $3;
 $5 = (_malloc($4)|0);
 $6 = $0;
 HEAP32[$6>>2] = $5;
 $7 = ($5|0)==(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = $2;
 _syserr(13544,13536,$8);
 STACKTOP = sp;return;
}
function _syserr($funcname,$syscall,$msg) {
 $funcname = $funcname|0;
 $syscall = $syscall|0;
 $msg = $msg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = $funcname;
 $1 = $syscall;
 $2 = $msg;
 $3 = HEAP32[_stdout>>2]|0;
 (_fflush(($3|0))|0);
 $4 = $2;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[_stderr>>2]|0;
  $7 = $0;
  $8 = $1;
  HEAP32[$vararg_buffer>>2] = $7;
  $vararg_ptr1 = (($vararg_buffer) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $8;
  (_fprintf(($6|0),(13560|0),($vararg_buffer|0))|0);
  $13 = HEAP32[_stderr>>2]|0;
  (_fflush(($13|0))|0);
  _exit(-1);
  // unreachable;
 } else {
  $9 = HEAP32[_stderr>>2]|0;
  $10 = $0;
  $11 = $1;
  $12 = $2;
  HEAP32[$vararg_buffer2>>2] = $10;
  $vararg_ptr5 = (($vararg_buffer2) + 4|0);
  HEAP32[$vararg_ptr5>>2] = $11;
  $vararg_ptr6 = (($vararg_buffer2) + 8|0);
  HEAP32[$vararg_ptr6>>2] = $12;
  (_fprintf(($9|0),(13576|0),($vararg_buffer2|0))|0);
  $13 = HEAP32[_stderr>>2]|0;
  (_fflush(($13|0))|0);
  _exit(-1);
  // unreachable;
 }
}
function _mbrtowc($wc,$src,$n,$st) {
 $wc = $wc|0;
 $src = $src|0;
 $n = $n|0;
 $st = $st|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $$1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c$0 = 0, $c$1 = 0, $c$2 = 0, $s$0 = 0, $s$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $wc;
 $1 = ($st|0)==(0|0);
 $$01 = $1 ? 14064 : $st;
 $2 = HEAP32[$$01>>2]|0;
 $3 = ($src|0)==(0|0);
 L1: do {
  if ($3) {
   $4 = ($2|0)==(0);
   if ($4) {
    $$0 = 0;
    STACKTOP = sp;return ($$0|0);
   }
  } else {
   $5 = ($wc|0)==(0|0);
   if ($5) {
    HEAP32[$0>>2] = $0;
    $11 = $0;
   } else {
    $11 = $wc;
   }
   $6 = ($n|0)==(0);
   if ($6) {
    $$0 = -2;
    STACKTOP = sp;return ($$0|0);
   }
   $7 = ($2|0)==(0);
   do {
    if ($7) {
     $8 = HEAP8[$src>>0]|0;
     $9 = $8&255;
     $10 = ($8<<24>>24)>(-1);
     if ($10) {
      HEAP32[$11>>2] = $9;
      $12 = ($8<<24>>24)!=(0);
      $13 = $12&1;
      $$0 = $13;
      STACKTOP = sp;return ($$0|0);
     } else {
      $14 = (($9) + -194)|0;
      $15 = ($14>>>0)>(50);
      if ($15) {
       break L1;
      }
      $16 = (($src) + 1|0);
      $17 = (13856 + ($14<<2)|0);
      $18 = HEAP32[$17>>2]|0;
      $19 = (($n) + -1)|0;
      $$02 = $19;$c$0 = $18;$s$0 = $16;
      break;
     }
    } else {
     $$02 = $n;$c$0 = $2;$s$0 = $src;
    }
   } while(0);
   $20 = ($$02|0)==(0);
   L17: do {
    if ($20) {
     $c$2 = $c$0;
    } else {
     $21 = HEAP8[$s$0>>0]|0;
     $22 = $21&255;
     $23 = $22 >>> 3;
     $24 = (($23) + -16)|0;
     $25 = $c$0 >> 26;
     $26 = (($23) + ($25))|0;
     $27 = $24 | $26;
     $28 = ($27>>>0)>(7);
     if ($28) {
      break L1;
     } else {
      $$1 = $$02;$32 = $21;$c$1 = $c$0;$s$1 = $s$0;
     }
     while(1) {
      $29 = $c$1 << 6;
      $30 = (($s$1) + 1|0);
      $31 = $32&255;
      $33 = (($31) + -128)|0;
      $34 = $33 | $29;
      $35 = (($$1) + -1)|0;
      $36 = ($34|0)<(0);
      if (!($36)) {
       break;
      }
      $38 = ($35|0)==(0);
      if ($38) {
       $c$2 = $34;
       break L17;
      }
      $39 = HEAP8[$30>>0]|0;
      $40 = $39&255;
      $41 = (($40) + -128)|0;
      $42 = ($41>>>0)>(63);
      if ($42) {
       break L1;
      } else {
       $$1 = $35;$32 = $39;$c$1 = $34;$s$1 = $30;
      }
     }
     HEAP32[$$01>>2] = 0;
     HEAP32[$11>>2] = $34;
     $37 = (($n) - ($35))|0;
     $$0 = $37;
     STACKTOP = sp;return ($$0|0);
    }
   } while(0);
   HEAP32[$$01>>2] = $c$2;
   $$0 = -2;
   STACKTOP = sp;return ($$0|0);
  }
 } while(0);
 HEAP32[$$01>>2] = 0;
 $43 = (___errno_location()|0);
 HEAP32[$43>>2] = 84;
 $$0 = -1;
 STACKTOP = sp;return ($$0|0);
}
function _mbsinit($st) {
 $st = $st|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($st|0)==(0|0);
 if ($0) {
  $4 = 1;
 } else {
  $1 = HEAP32[$st>>2]|0;
  $2 = ($1|0)==(0);
  $4 = $2;
 }
 $3 = $4&1;
 STACKTOP = sp;return ($3|0);
}
function ___string_read($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $k$0 = 0, $k$0$len = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 84|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = (($len) + 256)|0;
 $3 = (_memchr($1,0,$2)|0);
 $4 = ($3|0)==(0|0);
 if ($4) {
  $k$0 = $2;
 } else {
  $5 = $3;
  $6 = $1;
  $7 = (($5) - ($6))|0;
  $k$0 = $7;
 }
 $8 = ($k$0>>>0)<($len>>>0);
 $k$0$len = $8 ? $k$0 : $len;
 _memcpy(($buf|0),($1|0),($k$0$len|0))|0;
 $9 = (($1) + ($k$0$len)|0);
 $10 = (($f) + 4|0);
 HEAP32[$10>>2] = $9;
 $11 = (($1) + ($k$0)|0);
 $12 = (($f) + 8|0);
 HEAP32[$12>>2] = $11;
 HEAP32[$0>>2] = $11;
 STACKTOP = sp;return ($k$0$len|0);
}
function _sscanf($s,$fmt,$varargs) {
 $s = $s|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsscanf($s,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _MUSL_vfscanf($f,$fmt,$ap) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$ = 0, $$10 = 0, $$4 = 0, $$7 = 0, $$8 = 0, $$9 = 0, $$lcssa18 = 0, $$matches$0 = 0, $$not = 0, $$pre = 0, $$pre$phi213Z2D = 0, $$pre193 = 0, $$pre206 = 0, $$pre207 = 0, $$pre208 = 0, $$pre209 = 0, $$pre210 = 0, $$pre211 = 0, $$pre212 = 0, $$size$0 = 0;
 var $$width$0 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0.0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0.0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $alloc$0 = 0, $alloc$1 = 0, $alloc$2 = 0, $ap2$i = 0, $arglist_current = 0, $arglist_current2 = 0, $arglist_next = 0, $arglist_next3 = 0, $base$0 = 0, $c$027 = 0, $dest$0 = 0, $factor = 0;
 var $factor14 = 0, $i$0$i = 0, $i$0$ph = 0, $i$0$ph46 = 0, $i$0$ph46$lcssa = 0, $i$0$ph46$us53 = 0, $i$1$lcssa = 0, $i$2 = 0, $i$2$ph = 0, $i$2$ph$phi = 0, $i$3 = 0, $i$4 = 0, $invert$0 = 0, $isdigit = 0, $isdigit3 = 0, $isdigit320 = 0, $isdigittmp = 0, $isdigittmp2 = 0, $isdigittmp219 = 0, $k$0$ph = 0;
 var $k$1$ph = 0, $matches$0$ = 0, $matches$0116 = 0, $matches$1 = 0, $matches$2 = 0, $matches$3 = 0, $not$ = 0, $or$cond$us = 0, $or$cond5 = 0, $or$cond6 = 0, $p$0122 = 0, $p$1 = 0, $p$10 = 0, $p$11 = 0, $p$2 = 0, $p$3$lcssa = 0, $p$321 = 0, $p$4 = 0, $p$5 = 0, $p$6 = 0;
 var $p$7 = 0, $p$7$ph = 0, $p$8 = 0, $p$9 = 0, $pos$0121 = 0, $pos$1 = 0, $pos$2 = 0, $s$0114 = 0, $s$1$ph = 0, $s$2 = 0, $s$3 = 0, $s$4 = 0, $s$5 = 0, $s$6 = 0, $scanset = 0, $size$0 = 0, $st = 0, $vacopy_currentptr = 0, $wc = 0, $wcs$0115 = 0;
 var $wcs$1 = 0, $wcs$2$ph = 0, $wcs$2$ph146 = 0, $wcs$3 = 0, $wcs$4 = 0, $wcs$5 = 0, $wcs$6 = 0, $wcs$7 = 0, $width$0$lcssa = 0, $width$022 = 0, $width$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0;
 $ap2$i = sp + 16|0;
 $st = sp + 8|0;
 $scanset = sp + 33|0;
 $wc = sp;
 $0 = sp + 32|0;
 $1 = HEAP8[$fmt>>0]|0;
 $2 = ($1<<24>>24)==(0);
 if ($2) {
  $matches$3 = 0;
  STACKTOP = sp;return ($matches$3|0);
 }
 $3 = (($f) + 4|0);
 $4 = (($f) + 100|0);
 $5 = (($f) + 108|0);
 $6 = (($f) + 8|0);
 $7 = (($scanset) + 10|0);
 $8 = (($scanset) + 33|0);
 $9 = (($st) + 4|0);
 $10 = (($scanset) + 46|0);
 $11 = (($scanset) + 94|0);
 $13 = $1;$matches$0116 = 0;$p$0122 = $fmt;$pos$0121 = 0;$s$0114 = 0;$wcs$0115 = 0;
 L4: while(1) {
  $12 = $13&255;
  $14 = (_isspace($12)|0);
  $15 = ($14|0)==(0);
  L6: do {
   if ($15) {
    $42 = HEAP8[$p$0122>>0]|0;
    $43 = ($42<<24>>24)==(37);
    L21: do {
     if ($43) {
      $44 = (($p$0122) + 1|0);
      $45 = HEAP8[$44>>0]|0;
      do {
       if ((($45<<24>>24) == 37)) {
        break L21;
       } else if ((($45<<24>>24) == 42)) {
        $65 = (($p$0122) + 2|0);
        $dest$0 = 0;$p$2 = $65;
       } else {
        $66 = $45&255;
        $isdigittmp = (($66) + -48)|0;
        $isdigit = ($isdigittmp>>>0)<(10);
        if ($isdigit) {
         $67 = (($p$0122) + 2|0);
         $68 = HEAP8[$67>>0]|0;
         $69 = ($68<<24>>24)==(36);
         if ($69) {
          $vacopy_currentptr = HEAP32[$ap>>2]|0;
          HEAP32[$ap2$i>>2] = $vacopy_currentptr;
          $i$0$i = $isdigittmp;
          while(1) {
           $70 = ($i$0$i>>>0)>(1);
           $arglist_current = HEAP32[$ap2$i>>2]|0;
           $71 = HEAP32[$arglist_current>>2]|0;
           $arglist_next = (($arglist_current) + 4|0);
           HEAP32[$ap2$i>>2] = $arglist_next;
           $72 = (($i$0$i) + -1)|0;
           if ($70) {
            $i$0$i = $72;
           } else {
            break;
           }
          }
          $73 = (($p$0122) + 3|0);
          $dest$0 = $71;$p$2 = $73;
          break;
         }
        }
        $arglist_current2 = HEAP32[$ap>>2]|0;
        $74 = HEAP32[$arglist_current2>>2]|0;
        $arglist_next3 = (($arglist_current2) + 4|0);
        HEAP32[$ap>>2] = $arglist_next3;
        $dest$0 = $74;$p$2 = $44;
       }
      } while(0);
      $75 = HEAP8[$p$2>>0]|0;
      $76 = $75&255;
      $isdigittmp219 = (($76) + -48)|0;
      $isdigit320 = ($isdigittmp219>>>0)<(10);
      if ($isdigit320) {
       $80 = $76;$p$321 = $p$2;$width$022 = 0;
       while(1) {
        $77 = ($width$022*10)|0;
        $78 = (($77) + -48)|0;
        $79 = (($78) + ($80))|0;
        $81 = (($p$321) + 1|0);
        $82 = HEAP8[$81>>0]|0;
        $83 = $82&255;
        $isdigittmp2 = (($83) + -48)|0;
        $isdigit3 = ($isdigittmp2>>>0)<(10);
        if ($isdigit3) {
         $80 = $83;$p$321 = $81;$width$022 = $79;
        } else {
         $$lcssa18 = $82;$p$3$lcssa = $81;$width$0$lcssa = $79;
         break;
        }
       }
      } else {
       $$lcssa18 = $75;$p$3$lcssa = $p$2;$width$0$lcssa = 0;
      }
      $84 = ($$lcssa18<<24>>24)==(109);
      if ($84) {
       $85 = ($dest$0|0)!=(0|0);
       $86 = $85&1;
       $87 = (($p$3$lcssa) + 1|0);
       $$pre = HEAP8[$87>>0]|0;
       $90 = $$pre;$alloc$0 = $86;$p$4 = $87;
      } else {
       $90 = $$lcssa18;$alloc$0 = 0;$p$4 = $p$3$lcssa;
      }
      $88 = (($p$4) + 1|0);
      $89 = $90&255;
      switch ($89|0) {
      case 108:  {
       $94 = HEAP8[$88>>0]|0;
       $95 = ($94<<24>>24)==(108);
       $96 = (($p$4) + 2|0);
       $$9 = $95 ? $96 : $88;
       $$10 = $95 ? 3 : 1;
       $p$5 = $$9;$size$0 = $$10;
       break;
      }
      case 116: case 122:  {
       $p$5 = $88;$size$0 = 1;
       break;
      }
      case 104:  {
       $91 = HEAP8[$88>>0]|0;
       $92 = ($91<<24>>24)==(104);
       $93 = (($p$4) + 2|0);
       $$7 = $92 ? $93 : $88;
       $$8 = $92 ? -2 : -1;
       $p$5 = $$7;$size$0 = $$8;
       break;
      }
      case 76:  {
       $p$5 = $88;$size$0 = 2;
       break;
      }
      case 110: case 112: case 67: case 83: case 91: case 99: case 115: case 88: case 71: case 70: case 69: case 65: case 103: case 102: case 101: case 97: case 120: case 117: case 111: case 105: case 100:  {
       $p$5 = $p$4;$size$0 = 0;
       break;
      }
      case 106:  {
       $p$5 = $88;$size$0 = 3;
       break;
      }
      default: {
       $alloc$1 = $alloc$0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
       label = 162;
       break L4;
      }
      }
      $97 = HEAP8[$p$5>>0]|0;
      $98 = $97&255;
      $99 = $98 & 47;
      $100 = ($99|0)==(3);
      $101 = $98 | 32;
      $$ = $100 ? $101 : $98;
      $$size$0 = $100 ? 1 : $size$0;
      if ((($$|0) == 110)) {
       $103 = ($pos$0121|0)<(0);
       $104 = $103 << 31 >> 31;
       $105 = ($dest$0|0)==(0|0);
       if ($105) {
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
       }
       switch ($$size$0|0) {
       case -2:  {
        $106 = $pos$0121&255;
        HEAP8[$dest$0>>0] = $106;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case 3:  {
        $108 = $dest$0;
        $109 = $108;
        HEAP32[$109>>2] = $pos$0121;
        $110 = (($108) + 4)|0;
        $111 = $110;
        HEAP32[$111>>2] = $104;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case -1:  {
        $107 = $pos$0121&65535;
        HEAP16[$dest$0>>1] = $107;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case 0:  {
        HEAP32[$dest$0>>2] = $pos$0121;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       case 1:  {
        HEAP32[$dest$0>>2] = $pos$0121;
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
        break;
       }
       default: {
        $matches$1 = $matches$0116;$p$11 = $p$5;$pos$2 = $pos$0121;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
        break L6;
       }
       }
      } else if ((($$|0) == 91)) {
       $pos$1 = $pos$0121;$width$1 = $width$0$lcssa;
      } else if ((($$|0) == 99)) {
       $102 = ($width$0$lcssa|0)<(1);
       $$width$0 = $102 ? 1 : $width$0$lcssa;
       $pos$1 = $pos$0121;$width$1 = $$width$0;
      } else {
       ___shlim($f,0);
       while(1) {
        $112 = HEAP32[$3>>2]|0;
        $113 = HEAP32[$4>>2]|0;
        $114 = ($112>>>0)<($113>>>0);
        if ($114) {
         $115 = (($112) + 1|0);
         HEAP32[$3>>2] = $115;
         $116 = HEAP8[$112>>0]|0;
         $117 = $116&255;
         $119 = $117;
        } else {
         $118 = (___shgetc($f)|0);
         $119 = $118;
        }
        $120 = (_isspace($119)|0);
        $121 = ($120|0)==(0);
        if ($121) {
         break;
        }
       }
       $122 = HEAP32[$4>>2]|0;
       $123 = ($122|0)==(0|0);
       $$pre206 = HEAP32[$3>>2]|0;
       if ($123) {
        $128 = $$pre206;
       } else {
        $124 = (($$pre206) + -1|0);
        HEAP32[$3>>2] = $124;
        $128 = $124;
       }
       $125 = HEAP32[$5>>2]|0;
       $126 = HEAP32[$6>>2]|0;
       $127 = $128;
       $129 = $126;
       $130 = (($125) + ($pos$0121))|0;
       $131 = (($130) + ($127))|0;
       $132 = (($131) - ($129))|0;
       $pos$1 = $132;$width$1 = $width$0$lcssa;
      }
      ___shlim($f,$width$1);
      $133 = HEAP32[$3>>2]|0;
      $134 = HEAP32[$4>>2]|0;
      $135 = ($133>>>0)<($134>>>0);
      if ($135) {
       $136 = (($133) + 1|0);
       HEAP32[$3>>2] = $136;
       $140 = $134;
      } else {
       $137 = (___shgetc($f)|0);
       $138 = ($137|0)<(0);
       if ($138) {
        $alloc$1 = $alloc$0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
        label = 162;
        break L4;
       }
       $$pre207 = HEAP32[$4>>2]|0;
       $140 = $$pre207;
      }
      $139 = ($140|0)==(0|0);
      if (!($139)) {
       $141 = HEAP32[$3>>2]|0;
       $142 = (($141) + -1|0);
       HEAP32[$3>>2] = $142;
      }
      L78: do {
       switch ($$|0) {
       case 91: case 99: case 115:  {
        $143 = ($$|0)==(99);
        $144 = $$ & 239;
        $145 = ($144|0)==(99);
        L80: do {
         if ($145) {
          $146 = ($$|0)==(115);
          _memset(($scanset|0),-1,257)|0;
          HEAP8[$scanset>>0] = 0;
          if ($146) {
           HEAP8[$8>>0] = 0;
           ;HEAP8[$7+0>>0]=0|0;HEAP8[$7+1>>0]=0|0;HEAP8[$7+2>>0]=0|0;HEAP8[$7+3>>0]=0|0;HEAP8[$7+4>>0]=0|0;
           $p$9 = $p$5;
          } else {
           $p$9 = $p$5;
          }
         } else {
          $147 = (($p$5) + 1|0);
          $148 = HEAP8[$147>>0]|0;
          $149 = ($148<<24>>24)==(94);
          $150 = (($p$5) + 2|0);
          $invert$0 = $149&1;
          $p$6 = $149 ? $150 : $147;
          $151 = $149&1;
          _memset(($scanset|0),($151|0),257)|0;
          HEAP8[$scanset>>0] = 0;
          $152 = HEAP8[$p$6>>0]|0;
          if ((($152<<24>>24) == 45)) {
           $153 = (($p$6) + 1|0);
           $154 = $invert$0 ^ 1;
           $155 = $154&255;
           HEAP8[$10>>0] = $155;
           $$pre$phi213Z2D = $155;$p$7$ph = $153;
          } else if ((($152<<24>>24) == 93)) {
           $156 = (($p$6) + 1|0);
           $157 = $invert$0 ^ 1;
           $158 = $157&255;
           HEAP8[$11>>0] = $158;
           $$pre$phi213Z2D = $158;$p$7$ph = $156;
          } else {
           $$pre211 = $invert$0 ^ 1;
           $$pre212 = $$pre211&255;
           $$pre$phi213Z2D = $$pre212;$p$7$ph = $p$6;
          }
          $p$7 = $p$7$ph;
          while(1) {
           $159 = HEAP8[$p$7>>0]|0;
           if ((($159<<24>>24) == 93)) {
            $p$9 = $p$7;
            break L80;
           } else if ((($159<<24>>24) == 45)) {
            $160 = (($p$7) + 1|0);
            $161 = HEAP8[$160>>0]|0;
            if ((($161<<24>>24) == 93) | (($161<<24>>24) == 0)) {
             $172 = 45;$p$8 = $p$7;
            } else {
             $162 = (($p$7) + -1|0);
             $163 = HEAP8[$162>>0]|0;
             $164 = ($163&255)<($161&255);
             if ($164) {
              $165 = $163&255;
              $c$027 = $165;
              while(1) {
               $166 = (($c$027) + 1)|0;
               $167 = (($scanset) + ($166)|0);
               HEAP8[$167>>0] = $$pre$phi213Z2D;
               $168 = HEAP8[$160>>0]|0;
               $169 = $168&255;
               $170 = ($166|0)<($169|0);
               if ($170) {
                $c$027 = $166;
               } else {
                $172 = $168;$p$8 = $160;
                break;
               }
              }
             } else {
              $172 = $161;$p$8 = $160;
             }
            }
           } else if ((($159<<24>>24) == 0)) {
            $alloc$1 = $alloc$0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
            label = 162;
            break L4;
           } else {
            $172 = $159;$p$8 = $p$7;
           }
           $171 = $172&255;
           $173 = (($171) + 1)|0;
           $174 = (($scanset) + ($173)|0);
           HEAP8[$174>>0] = $$pre$phi213Z2D;
           $175 = (($p$8) + 1|0);
           $p$7 = $175;
          }
         }
        } while(0);
        $176 = (($width$1) + 1)|0;
        $$4 = $143 ? $176 : 31;
        $177 = ($$size$0|0)==(1);
        $178 = ($alloc$0|0)!=(0);
        L99: do {
         if ($177) {
          if ($178) {
           $179 = $$4 << 2;
           $180 = (_malloc($179)|0);
           $181 = ($180|0)==(0|0);
           if ($181) {
            $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $180;
            label = 162;
            break L4;
           } else {
            $wcs$1 = $180;
           }
          } else {
           $wcs$1 = $dest$0;
          }
          HEAP32[$st>>2] = 0;
          HEAP32[$9>>2] = 0;
          $i$0$ph = 0;$k$0$ph = $$4;$wcs$2$ph = $wcs$1;
          L105: while(1) {
           $182 = ($wcs$2$ph|0)==(0|0);
           if ($182) {
            while(1) {
             $185 = HEAP32[$3>>2]|0;
             $196 = HEAP32[$4>>2]|0;
             $197 = ($185>>>0)<($196>>>0);
             if ($197) {
              $184 = (($185) + 1|0);
              HEAP32[$3>>2] = $184;
              $186 = HEAP8[$185>>0]|0;
              $187 = $186&255;
              $189 = $187;
             } else {
              $183 = (___shgetc($f)|0);
              $189 = $183;
             }
             $188 = (($189) + 1)|0;
             $190 = (($scanset) + ($188)|0);
             $191 = HEAP8[$190>>0]|0;
             $192 = ($191<<24>>24)==(0);
             if ($192) {
              $i$0$ph46$lcssa = $i$0$ph;$wcs$2$ph146 = 0;
              break L105;
             }
             $193 = $189&255;
             HEAP8[$0>>0] = $193;
             $194 = (_mbrtowc($wc,$0,1,$st)|0);
             if ((($194|0) == -2)) {
              continue;
             } else if ((($194|0) == -1)) {
              $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = 0;
              label = 162;
              break L4;
             }
             $195 = ($i$0$ph|0)==($k$0$ph|0);
             $or$cond$us = $178 & $195;
             if ($or$cond$us) {
              $i$1$lcssa = $i$0$ph;
              break;
             }
            }
           } else {
            if ($178) {
             $i$0$ph46$us53 = $i$0$ph;
            } else {
             $i$0$ph46 = $i$0$ph;
             label = 98;
             break;
            }
            while(1) {
             while(1) {
              $200 = HEAP32[$3>>2]|0;
              $214 = HEAP32[$4>>2]|0;
              $215 = ($200>>>0)<($214>>>0);
              if ($215) {
               $199 = (($200) + 1|0);
               HEAP32[$3>>2] = $199;
               $201 = HEAP8[$200>>0]|0;
               $202 = $201&255;
               $204 = $202;
              } else {
               $198 = (___shgetc($f)|0);
               $204 = $198;
              }
              $203 = (($204) + 1)|0;
              $205 = (($scanset) + ($203)|0);
              $206 = HEAP8[$205>>0]|0;
              $207 = ($206<<24>>24)==(0);
              if ($207) {
               $i$0$ph46$lcssa = $i$0$ph46$us53;$wcs$2$ph146 = $wcs$2$ph;
               break L105;
              }
              $208 = $204&255;
              HEAP8[$0>>0] = $208;
              $209 = (_mbrtowc($wc,$0,1,$st)|0);
              if ((($209|0) == -1)) {
               $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $wcs$2$ph;
               label = 162;
               break L4;
              } else if (!((($209|0) == -2))) {
               break;
              }
             }
             $210 = HEAP32[$wc>>2]|0;
             $211 = (($i$0$ph46$us53) + 1)|0;
             $212 = (($wcs$2$ph) + ($i$0$ph46$us53<<2)|0);
             HEAP32[$212>>2] = $210;
             $213 = ($211|0)==($k$0$ph|0);
             if ($213) {
              $i$1$lcssa = $k$0$ph;
              break;
             } else {
              $i$0$ph46$us53 = $211;
             }
            }
           }
           $factor = $k$0$ph << 1;
           $233 = $factor | 1;
           $234 = $233 << 2;
           $235 = (_realloc($wcs$2$ph,$234)|0);
           $236 = ($235|0)==(0|0);
           if ($236) {
            $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $wcs$2$ph;
            label = 162;
            break L4;
           }
           $i$0$ph = $i$1$lcssa;$k$0$ph = $233;$wcs$2$ph = $235;
          }
          L129: do {
           if ((label|0) == 98) {
            while(1) {
             label = 0;
             while(1) {
              $216 = HEAP32[$3>>2]|0;
              $217 = HEAP32[$4>>2]|0;
              $218 = ($216>>>0)<($217>>>0);
              if ($218) {
               $219 = (($216) + 1|0);
               HEAP32[$3>>2] = $219;
               $220 = HEAP8[$216>>0]|0;
               $221 = $220&255;
               $224 = $221;
              } else {
               $222 = (___shgetc($f)|0);
               $224 = $222;
              }
              $223 = (($224) + 1)|0;
              $225 = (($scanset) + ($223)|0);
              $226 = HEAP8[$225>>0]|0;
              $227 = ($226<<24>>24)==(0);
              if ($227) {
               $i$0$ph46$lcssa = $i$0$ph46;$wcs$2$ph146 = $wcs$2$ph;
               break L129;
              }
              $228 = $224&255;
              HEAP8[$0>>0] = $228;
              $229 = (_mbrtowc($wc,$0,1,$st)|0);
              if ((($229|0) == -1)) {
               $alloc$1 = 0;$s$5 = 0;$wcs$6 = $wcs$2$ph;
               label = 162;
               break L4;
              } else if (!((($229|0) == -2))) {
               break;
              }
             }
             $230 = HEAP32[$wc>>2]|0;
             $231 = (($i$0$ph46) + 1)|0;
             $232 = (($wcs$2$ph) + ($i$0$ph46<<2)|0);
             HEAP32[$232>>2] = $230;
             $i$0$ph46 = $231;
             label = 98;
            }
           }
          } while(0);
          $237 = (_mbsinit($st)|0);
          $238 = ($237|0)==(0);
          if ($238) {
           $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = $wcs$2$ph146;
           label = 162;
           break L4;
          } else {
           $i$4 = $i$0$ph46$lcssa;$s$2 = 0;$wcs$3 = $wcs$2$ph146;
          }
         } else {
          if ($178) {
           $239 = (_malloc($$4)|0);
           $240 = ($239|0)==(0|0);
           if ($240) {
            $alloc$1 = $alloc$0;$s$5 = 0;$wcs$6 = 0;
            label = 162;
            break L4;
           } else {
            $i$2$ph = 0;$k$1$ph = $$4;$s$1$ph = $239;
           }
           while(1) {
            $i$2 = $i$2$ph;
            while(1) {
             $241 = HEAP32[$3>>2]|0;
             $242 = HEAP32[$4>>2]|0;
             $243 = ($241>>>0)<($242>>>0);
             if ($243) {
              $244 = (($241) + 1|0);
              HEAP32[$3>>2] = $244;
              $245 = HEAP8[$241>>0]|0;
              $246 = $245&255;
              $249 = $246;
             } else {
              $247 = (___shgetc($f)|0);
              $249 = $247;
             }
             $248 = (($249) + 1)|0;
             $250 = (($scanset) + ($248)|0);
             $251 = HEAP8[$250>>0]|0;
             $252 = ($251<<24>>24)==(0);
             if ($252) {
              $i$4 = $i$2;$s$2 = $s$1$ph;$wcs$3 = 0;
              break L99;
             }
             $253 = $249&255;
             $254 = (($i$2) + 1)|0;
             $255 = (($s$1$ph) + ($i$2)|0);
             HEAP8[$255>>0] = $253;
             $256 = ($254|0)==($k$1$ph|0);
             if ($256) {
              break;
             } else {
              $i$2 = $254;
             }
            }
            $factor14 = $k$1$ph << 1;
            $257 = $factor14 | 1;
            $258 = (_realloc($s$1$ph,$257)|0);
            $259 = ($258|0)==(0|0);
            if ($259) {
             $alloc$1 = $alloc$0;$s$5 = $s$1$ph;$wcs$6 = 0;
             label = 162;
             break L4;
            } else {
             $i$2$ph$phi = $k$1$ph;$k$1$ph = $257;$s$1$ph = $258;$i$2$ph = $i$2$ph$phi;
            }
           }
          }
          $260 = ($dest$0|0)==(0|0);
          if ($260) {
           $278 = $140;
           while(1) {
            $276 = HEAP32[$3>>2]|0;
            $277 = ($276>>>0)<($278>>>0);
            if ($277) {
             $279 = (($276) + 1|0);
             HEAP32[$3>>2] = $279;
             $280 = HEAP8[$276>>0]|0;
             $281 = $280&255;
             $284 = $281;
            } else {
             $282 = (___shgetc($f)|0);
             $284 = $282;
            }
            $283 = (($284) + 1)|0;
            $285 = (($scanset) + ($283)|0);
            $286 = HEAP8[$285>>0]|0;
            $287 = ($286<<24>>24)==(0);
            if ($287) {
             $i$4 = 0;$s$2 = 0;$wcs$3 = 0;
             break L99;
            }
            $$pre209 = HEAP32[$4>>2]|0;
            $278 = $$pre209;
           }
          } else {
           $263 = $140;$i$3 = 0;
           while(1) {
            $261 = HEAP32[$3>>2]|0;
            $262 = ($261>>>0)<($263>>>0);
            if ($262) {
             $264 = (($261) + 1|0);
             HEAP32[$3>>2] = $264;
             $265 = HEAP8[$261>>0]|0;
             $266 = $265&255;
             $269 = $266;
            } else {
             $267 = (___shgetc($f)|0);
             $269 = $267;
            }
            $268 = (($269) + 1)|0;
            $270 = (($scanset) + ($268)|0);
            $271 = HEAP8[$270>>0]|0;
            $272 = ($271<<24>>24)==(0);
            if ($272) {
             $i$4 = $i$3;$s$2 = $dest$0;$wcs$3 = 0;
             break L99;
            }
            $273 = $269&255;
            $274 = (($i$3) + 1)|0;
            $275 = (($dest$0) + ($i$3)|0);
            HEAP8[$275>>0] = $273;
            $$pre193 = HEAP32[$4>>2]|0;
            $263 = $$pre193;$i$3 = $274;
           }
          }
         }
        } while(0);
        $288 = HEAP32[$4>>2]|0;
        $289 = ($288|0)==(0|0);
        $$pre208 = HEAP32[$3>>2]|0;
        if ($289) {
         $294 = $$pre208;
        } else {
         $290 = (($$pre208) + -1|0);
         HEAP32[$3>>2] = $290;
         $294 = $290;
        }
        $291 = HEAP32[$5>>2]|0;
        $292 = HEAP32[$6>>2]|0;
        $293 = $294;
        $295 = $292;
        $296 = (($293) - ($295))|0;
        $297 = (($296) + ($291))|0;
        $298 = ($297|0)==(0);
        if ($298) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$2;$wcs$7 = $wcs$3;
         break L4;
        }
        $$not = $143 ^ 1;
        $299 = ($297|0)==($width$1|0);
        $or$cond5 = $299 | $$not;
        if (!($or$cond5)) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$2;$wcs$7 = $wcs$3;
         break L4;
        }
        do {
         if ($178) {
          if ($177) {
           HEAP32[$dest$0>>2] = $wcs$3;
           break;
          } else {
           HEAP32[$dest$0>>2] = $s$2;
           break;
          }
         }
        } while(0);
        if ($143) {
         $p$10 = $p$9;$s$3 = $s$2;$wcs$4 = $wcs$3;
        } else {
         $300 = ($wcs$3|0)==(0|0);
         if (!($300)) {
          $301 = (($wcs$3) + ($i$4<<2)|0);
          HEAP32[$301>>2] = 0;
         }
         $302 = ($s$2|0)==(0|0);
         if ($302) {
          $p$10 = $p$9;$s$3 = 0;$wcs$4 = $wcs$3;
         } else {
          $303 = (($s$2) + ($i$4)|0);
          HEAP8[$303>>0] = 0;
          $p$10 = $p$9;$s$3 = $s$2;$wcs$4 = $wcs$3;
         }
        }
        break;
       }
       case 71: case 103: case 70: case 102: case 69: case 101: case 65: case 97:  {
        $322 = (+___floatscan($f,$$size$0,0));
        $323 = HEAP32[$5>>2]|0;
        $324 = HEAP32[$3>>2]|0;
        $325 = HEAP32[$6>>2]|0;
        $326 = $324;
        $327 = $325;
        $328 = (($327) - ($326))|0;
        $329 = ($323|0)==($328|0);
        if ($329) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$0114;$wcs$7 = $wcs$0115;
         break L4;
        }
        $330 = ($dest$0|0)==(0|0);
        if ($330) {
         $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
        } else {
         if ((($$size$0|0) == 0)) {
          $331 = $322;
          HEAPF32[$dest$0>>2] = $331;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         } else if ((($$size$0|0) == 1)) {
          HEAPF64[$dest$0>>3] = $322;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         } else if ((($$size$0|0) == 2)) {
          HEAPF64[$dest$0>>3] = $322;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         } else {
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L78;
         }
        }
        break;
       }
       case 111:  {
        $base$0 = 8;
        label = 144;
        break;
       }
       case 117: case 100:  {
        $base$0 = 10;
        label = 144;
        break;
       }
       case 105:  {
        $base$0 = 0;
        label = 144;
        break;
       }
       case 120: case 88: case 112:  {
        $base$0 = 16;
        label = 144;
        break;
       }
       default: {
        $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
       }
       }
      } while(0);
      L197: do {
       if ((label|0) == 144) {
        label = 0;
        $304 = (___intscan($f,$base$0,0,-1,-1)|0);
        $305 = tempRet0;
        $306 = HEAP32[$5>>2]|0;
        $307 = HEAP32[$3>>2]|0;
        $308 = HEAP32[$6>>2]|0;
        $309 = $307;
        $310 = $308;
        $311 = (($310) - ($309))|0;
        $312 = ($306|0)==($311|0);
        if ($312) {
         $alloc$2 = $alloc$0;$matches$2 = $matches$0116;$s$6 = $s$0114;$wcs$7 = $wcs$0115;
         break L4;
        }
        $313 = ($$|0)!=(112);
        $314 = ($dest$0|0)==(0|0);
        $or$cond6 = $313 | $314;
        if (!($or$cond6)) {
         $315 = $304;
         HEAP32[$dest$0>>2] = $315;
         $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
         break;
        }
        if ($314) {
         $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
        } else {
         switch ($$size$0|0) {
         case -1:  {
          $317 = $304&65535;
          HEAP16[$dest$0>>1] = $317;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case 0:  {
          HEAP32[$dest$0>>2] = $304;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case 1:  {
          HEAP32[$dest$0>>2] = $304;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case 3:  {
          $318 = $dest$0;
          $319 = $318;
          HEAP32[$319>>2] = $304;
          $320 = (($318) + 4)|0;
          $321 = $320;
          HEAP32[$321>>2] = $305;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         case -2:  {
          $316 = $304&255;
          HEAP8[$dest$0>>0] = $316;
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
          break;
         }
         default: {
          $p$10 = $p$5;$s$3 = $s$0114;$wcs$4 = $wcs$0115;
          break L197;
         }
         }
        }
       }
      } while(0);
      $332 = HEAP32[$5>>2]|0;
      $333 = HEAP32[$3>>2]|0;
      $334 = HEAP32[$6>>2]|0;
      $335 = $333;
      $336 = $334;
      $337 = (($332) + ($pos$1))|0;
      $338 = (($337) + ($335))|0;
      $339 = (($338) - ($336))|0;
      $not$ = ($dest$0|0)!=(0|0);
      $340 = $not$&1;
      $matches$0$ = (($340) + ($matches$0116))|0;
      $matches$1 = $matches$0$;$p$11 = $p$10;$pos$2 = $339;$s$4 = $s$3;$wcs$5 = $wcs$4;
      break L6;
     }
    } while(0);
    $46 = $43&1;
    $47 = (($p$0122) + ($46)|0);
    ___shlim($f,0);
    $48 = HEAP32[$3>>2]|0;
    $49 = HEAP32[$4>>2]|0;
    $50 = ($48>>>0)<($49>>>0);
    if ($50) {
     $51 = (($48) + 1|0);
     HEAP32[$3>>2] = $51;
     $52 = HEAP8[$48>>0]|0;
     $53 = $52&255;
     $58 = $53;
    } else {
     $54 = (___shgetc($f)|0);
     $58 = $54;
    }
    $55 = HEAP8[$47>>0]|0;
    $56 = $55&255;
    $57 = ($58|0)==($56|0);
    if (!($57)) {
     label = 19;
     break L4;
    }
    $64 = (($pos$0121) + 1)|0;
    $matches$1 = $matches$0116;$p$11 = $47;$pos$2 = $64;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
   } else {
    $p$1 = $p$0122;
    while(1) {
     $16 = (($p$1) + 1|0);
     $17 = HEAP8[$16>>0]|0;
     $18 = $17&255;
     $19 = (_isspace($18)|0);
     $20 = ($19|0)==(0);
     if ($20) {
      break;
     } else {
      $p$1 = $16;
     }
    }
    ___shlim($f,0);
    while(1) {
     $21 = HEAP32[$3>>2]|0;
     $22 = HEAP32[$4>>2]|0;
     $23 = ($21>>>0)<($22>>>0);
     if ($23) {
      $24 = (($21) + 1|0);
      HEAP32[$3>>2] = $24;
      $25 = HEAP8[$21>>0]|0;
      $26 = $25&255;
      $28 = $26;
     } else {
      $27 = (___shgetc($f)|0);
      $28 = $27;
     }
     $29 = (_isspace($28)|0);
     $30 = ($29|0)==(0);
     if ($30) {
      break;
     }
    }
    $31 = HEAP32[$4>>2]|0;
    $32 = ($31|0)==(0|0);
    $$pre210 = HEAP32[$3>>2]|0;
    if ($32) {
     $37 = $$pre210;
    } else {
     $33 = (($$pre210) + -1|0);
     HEAP32[$3>>2] = $33;
     $37 = $33;
    }
    $34 = HEAP32[$5>>2]|0;
    $35 = HEAP32[$6>>2]|0;
    $36 = $37;
    $38 = $35;
    $39 = (($34) + ($pos$0121))|0;
    $40 = (($39) + ($36))|0;
    $41 = (($40) - ($38))|0;
    $matches$1 = $matches$0116;$p$11 = $p$1;$pos$2 = $41;$s$4 = $s$0114;$wcs$5 = $wcs$0115;
   }
  } while(0);
  $341 = (($p$11) + 1|0);
  $342 = HEAP8[$341>>0]|0;
  $343 = ($342<<24>>24)==(0);
  if ($343) {
   $matches$3 = $matches$1;
   label = 165;
   break;
  } else {
   $13 = $342;$matches$0116 = $matches$1;$p$0122 = $341;$pos$0121 = $pos$2;$s$0114 = $s$4;$wcs$0115 = $wcs$5;
  }
 }
 if ((label|0) == 165) {
  STACKTOP = sp;return ($matches$3|0);
 }
 if ((label|0) == 19) {
  $59 = HEAP32[$4>>2]|0;
  $60 = ($59|0)==(0|0);
  if (!($60)) {
   $61 = HEAP32[$3>>2]|0;
   $62 = (($61) + -1|0);
   HEAP32[$3>>2] = $62;
  }
  $63 = ($58|0)<(0);
  if ($63) {
   $alloc$1 = 0;$s$5 = $s$0114;$wcs$6 = $wcs$0115;
   label = 162;
  } else {
   $matches$3 = $matches$0116;
   STACKTOP = sp;return ($matches$3|0);
  }
 }
 if ((label|0) == 162) {
  $344 = ($matches$0116|0)==(0);
  $345 = $344 << 31 >> 31;
  $$matches$0 = (($345) + ($matches$0116))|0;
  $alloc$2 = $alloc$1;$matches$2 = $$matches$0;$s$6 = $s$5;$wcs$7 = $wcs$6;
 }
 $346 = ($alloc$2|0)==(0);
 if ($346) {
  $matches$3 = $matches$2;
  STACKTOP = sp;return ($matches$3|0);
 }
 _free($s$6);
 _free($wcs$7);
 $matches$3 = $matches$2;
 STACKTOP = sp;return ($matches$3|0);
}
function _vsscanf($s,$fmt,$ap) {
 $s = $s|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $f = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $f = sp;
 dest=$f+0|0; stop=dest+112|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $0 = (($f) + 32|0);
 HEAP32[$0>>2] = 2;
 $1 = (($f) + 44|0);
 HEAP32[$1>>2] = $s;
 $2 = (($f) + 76|0);
 HEAP32[$2>>2] = -1;
 $3 = (($f) + 84|0);
 HEAP32[$3>>2] = $s;
 $4 = (_MUSL_vfscanf($f,$fmt,$ap)|0);
 STACKTOP = sp;return ($4|0);
}
function _do_read($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (___string_read($f,$buf,$len)|0);
 STACKTOP = sp;return ($0|0);
}
function ___strdup($s) {
 $s = $s|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_strlen(($s|0))|0);
 $1 = (($0) + 1)|0;
 $2 = (_malloc($1)|0);
 $3 = ($2|0)==(0|0);
 if ($3) {
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 _memcpy(($2|0),($s|0),($1|0))|0;
 $$0 = $2;
 STACKTOP = sp;return ($$0|0);
}
function _malloc($bytes) {
 $bytes = $bytes|0;
 var $$$i = 0, $$3$i = 0, $$4$i = 0, $$pre = 0, $$pre$i = 0, $$pre$i$i = 0, $$pre$i25 = 0, $$pre$i25$i = 0, $$pre$phi$i$iZ2D = 0, $$pre$phi$i26$iZ2D = 0, $$pre$phi$i26Z2D = 0, $$pre$phi$iZ2D = 0, $$pre$phi58$i$iZ2D = 0, $$pre$phiZ2D = 0, $$pre57$i$i = 0, $$rsize$0$i = 0, $$rsize$3$i = 0, $$sum = 0, $$sum$i$i = 0, $$sum$i$i$i = 0;
 var $$sum$i14$i = 0, $$sum$i15$i = 0, $$sum$i18$i = 0, $$sum$i21$i = 0, $$sum$i2334 = 0, $$sum$i32 = 0, $$sum$i35 = 0, $$sum1 = 0, $$sum1$i = 0, $$sum1$i$i = 0, $$sum1$i16$i = 0, $$sum1$i22$i = 0, $$sum1$i24 = 0, $$sum10 = 0, $$sum10$i = 0, $$sum10$i$i = 0, $$sum10$pre$i$i = 0, $$sum107$i = 0, $$sum108$i = 0, $$sum109$i = 0;
 var $$sum11$i = 0, $$sum11$i$i = 0, $$sum11$i24$i = 0, $$sum110$i = 0, $$sum111$i = 0, $$sum1112 = 0, $$sum112$i = 0, $$sum113$i = 0, $$sum114$i = 0, $$sum115$i = 0, $$sum116$i = 0, $$sum117$i = 0, $$sum118$i = 0, $$sum119$i = 0, $$sum12$i = 0, $$sum12$i$i = 0, $$sum120$i = 0, $$sum13$i = 0, $$sum13$i$i = 0, $$sum14$i$i = 0;
 var $$sum14$pre$i = 0, $$sum15$i = 0, $$sum15$i$i = 0, $$sum16$i = 0, $$sum16$i$i = 0, $$sum17$i = 0, $$sum17$i$i = 0, $$sum18$i = 0, $$sum1819$i$i = 0, $$sum2 = 0, $$sum2$i = 0, $$sum2$i$i = 0, $$sum2$i$i$i = 0, $$sum2$i17$i = 0, $$sum2$i19$i = 0, $$sum2$i23$i = 0, $$sum2$pre$i = 0, $$sum20$i$i = 0, $$sum21$i$i = 0, $$sum22$i$i = 0;
 var $$sum23$i$i = 0, $$sum24$i$i = 0, $$sum25$i$i = 0, $$sum26$pre$i$i = 0, $$sum27$i$i = 0, $$sum28$i$i = 0, $$sum29$i$i = 0, $$sum3$i = 0, $$sum3$i$i = 0, $$sum3$i27 = 0, $$sum30$i$i = 0, $$sum3132$i$i = 0, $$sum34$i$i = 0, $$sum3536$i$i = 0, $$sum3738$i$i = 0, $$sum39$i$i = 0, $$sum4 = 0, $$sum4$i = 0, $$sum4$i28 = 0, $$sum40$i$i = 0;
 var $$sum41$i$i = 0, $$sum42$i$i = 0, $$sum5$i = 0, $$sum5$i$i = 0, $$sum56 = 0, $$sum6$i = 0, $$sum67$i$i = 0, $$sum7$i = 0, $$sum8$i = 0, $$sum8$pre = 0, $$sum9 = 0, $$sum9$i = 0, $$sum9$i$i = 0, $$tsize$1$i = 0, $$v$0$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0;
 var $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0;
 var $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0;
 var $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0;
 var $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0;
 var $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0;
 var $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0;
 var $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0;
 var $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0;
 var $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0;
 var $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0;
 var $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0;
 var $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0;
 var $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0;
 var $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0;
 var $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0;
 var $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0;
 var $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0;
 var $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0;
 var $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0;
 var $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0;
 var $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0;
 var $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0;
 var $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0;
 var $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0;
 var $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0;
 var $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0;
 var $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0;
 var $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0;
 var $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0;
 var $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0;
 var $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0;
 var $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0;
 var $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0;
 var $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0;
 var $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0;
 var $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0;
 var $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0;
 var $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0;
 var $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0;
 var $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0;
 var $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0;
 var $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0;
 var $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0;
 var $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0;
 var $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0;
 var $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0;
 var $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0;
 var $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0;
 var $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0;
 var $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0;
 var $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0;
 var $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0;
 var $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0;
 var $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $F$0$i$i = 0, $F1$0$i = 0, $F4$0 = 0, $F4$0$i$i = 0, $F5$0$i = 0;
 var $I1$0$c$i$i = 0, $I1$0$i$i = 0, $I7$0$i = 0, $I7$0$i$i = 0, $K12$025$i = 0, $K2$014$i$i = 0, $K8$052$i$i = 0, $R$0$i = 0, $R$0$i$i = 0, $R$0$i18 = 0, $R$1$i = 0, $R$1$i$i = 0, $R$1$i20 = 0, $RP$0$i = 0, $RP$0$i$i = 0, $RP$0$i17 = 0, $T$0$lcssa$i = 0, $T$0$lcssa$i$i = 0, $T$0$lcssa$i28$i = 0, $T$013$i$i = 0;
 var $T$024$i = 0, $T$051$i$i = 0, $br$0$i = 0, $cond$i = 0, $cond$i$i = 0, $cond$i21 = 0, $exitcond$i$i = 0, $i$02$i$i = 0, $idx$0$i = 0, $mem$0 = 0, $nb$0 = 0, $notlhs$i = 0, $notrhs$i = 0, $oldfirst$0$i$i = 0, $or$cond$i = 0, $or$cond$i29 = 0, $or$cond1$i = 0, $or$cond10$i = 0, $or$cond19$i = 0, $or$cond2$i = 0;
 var $or$cond49$i = 0, $or$cond5$i = 0, $or$cond6$i = 0, $or$cond8$not$i = 0, $or$cond9$i = 0, $qsize$0$i$i = 0, $rsize$0$i = 0, $rsize$0$i15 = 0, $rsize$1$i = 0, $rsize$2$i = 0, $rsize$3$lcssa$i = 0, $rsize$329$i = 0, $rst$0$i = 0, $rst$1$i = 0, $sizebits$0$i = 0, $sp$0$i$i = 0, $sp$0$i$i$i = 0, $sp$075$i = 0, $sp$168$i = 0, $ssize$0$$i = 0;
 var $ssize$0$i = 0, $ssize$1$i = 0, $ssize$2$i = 0, $t$0$i = 0, $t$0$i14 = 0, $t$1$i = 0, $t$2$ph$i = 0, $t$2$v$3$i = 0, $t$228$i = 0, $tbase$0$i = 0, $tbase$247$i = 0, $tsize$0$i = 0, $tsize$0323841$i = 0, $tsize$1$i = 0, $tsize$246$i = 0, $v$0$i = 0, $v$0$i16 = 0, $v$1$i = 0, $v$2$i = 0, $v$3$lcssa$i = 0;
 var $v$330$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($bytes>>>0)<(245);
 do {
  if ($0) {
   $1 = ($bytes>>>0)<(11);
   if ($1) {
    $5 = 16;
   } else {
    $2 = (($bytes) + 11)|0;
    $3 = $2 & -8;
    $5 = $3;
   }
   $4 = $5 >>> 3;
   $6 = HEAP32[14072>>2]|0;
   $7 = $6 >>> $4;
   $8 = $7 & 3;
   $9 = ($8|0)==(0);
   if (!($9)) {
    $10 = $7 & 1;
    $11 = $10 ^ 1;
    $12 = (($11) + ($4))|0;
    $13 = $12 << 1;
    $14 = ((14072 + ($13<<2)|0) + 40|0);
    $$sum10 = (($13) + 2)|0;
    $15 = ((14072 + ($$sum10<<2)|0) + 40|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = (($16) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($14|0)==($18|0);
    do {
     if ($19) {
      $20 = 1 << $12;
      $21 = $20 ^ -1;
      $22 = $6 & $21;
      HEAP32[14072>>2] = $22;
     } else {
      $23 = HEAP32[((14072 + 16|0))>>2]|0;
      $24 = ($18>>>0)<($23>>>0);
      if ($24) {
       _abort();
       // unreachable;
      }
      $25 = (($18) + 12|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==($16|0);
      if ($27) {
       HEAP32[$25>>2] = $14;
       HEAP32[$15>>2] = $18;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $28 = $12 << 3;
    $29 = $28 | 3;
    $30 = (($16) + 4|0);
    HEAP32[$30>>2] = $29;
    $$sum1112 = $28 | 4;
    $31 = (($16) + ($$sum1112)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 1;
    HEAP32[$31>>2] = $33;
    $mem$0 = $17;
    STACKTOP = sp;return ($mem$0|0);
   }
   $34 = HEAP32[((14072 + 8|0))>>2]|0;
   $35 = ($5>>>0)>($34>>>0);
   if ($35) {
    $36 = ($7|0)==(0);
    if (!($36)) {
     $37 = $7 << $4;
     $38 = 2 << $4;
     $39 = (0 - ($38))|0;
     $40 = $38 | $39;
     $41 = $37 & $40;
     $42 = (0 - ($41))|0;
     $43 = $41 & $42;
     $44 = (($43) + -1)|0;
     $45 = $44 >>> 12;
     $46 = $45 & 16;
     $47 = $44 >>> $46;
     $48 = $47 >>> 5;
     $49 = $48 & 8;
     $50 = $49 | $46;
     $51 = $47 >>> $49;
     $52 = $51 >>> 2;
     $53 = $52 & 4;
     $54 = $50 | $53;
     $55 = $51 >>> $53;
     $56 = $55 >>> 1;
     $57 = $56 & 2;
     $58 = $54 | $57;
     $59 = $55 >>> $57;
     $60 = $59 >>> 1;
     $61 = $60 & 1;
     $62 = $58 | $61;
     $63 = $59 >>> $61;
     $64 = (($62) + ($63))|0;
     $65 = $64 << 1;
     $66 = ((14072 + ($65<<2)|0) + 40|0);
     $$sum4 = (($65) + 2)|0;
     $67 = ((14072 + ($$sum4<<2)|0) + 40|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + 8|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($66|0)==($70|0);
     do {
      if ($71) {
       $72 = 1 << $64;
       $73 = $72 ^ -1;
       $74 = $6 & $73;
       HEAP32[14072>>2] = $74;
      } else {
       $75 = HEAP32[((14072 + 16|0))>>2]|0;
       $76 = ($70>>>0)<($75>>>0);
       if ($76) {
        _abort();
        // unreachable;
       }
       $77 = (($70) + 12|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = ($78|0)==($68|0);
       if ($79) {
        HEAP32[$77>>2] = $66;
        HEAP32[$67>>2] = $70;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $80 = $64 << 3;
     $81 = (($80) - ($5))|0;
     $82 = $5 | 3;
     $83 = (($68) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = (($68) + ($5)|0);
     $85 = $81 | 1;
     $$sum56 = $5 | 4;
     $86 = (($68) + ($$sum56)|0);
     HEAP32[$86>>2] = $85;
     $87 = (($68) + ($80)|0);
     HEAP32[$87>>2] = $81;
     $88 = HEAP32[((14072 + 8|0))>>2]|0;
     $89 = ($88|0)==(0);
     if (!($89)) {
      $90 = HEAP32[((14072 + 20|0))>>2]|0;
      $91 = $88 >>> 3;
      $92 = $91 << 1;
      $93 = ((14072 + ($92<<2)|0) + 40|0);
      $94 = HEAP32[14072>>2]|0;
      $95 = 1 << $91;
      $96 = $94 & $95;
      $97 = ($96|0)==(0);
      if ($97) {
       $98 = $94 | $95;
       HEAP32[14072>>2] = $98;
       $$sum8$pre = (($92) + 2)|0;
       $$pre = ((14072 + ($$sum8$pre<<2)|0) + 40|0);
       $$pre$phiZ2D = $$pre;$F4$0 = $93;
      } else {
       $$sum9 = (($92) + 2)|0;
       $99 = ((14072 + ($$sum9<<2)|0) + 40|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = HEAP32[((14072 + 16|0))>>2]|0;
       $102 = ($100>>>0)<($101>>>0);
       if ($102) {
        _abort();
        // unreachable;
       } else {
        $$pre$phiZ2D = $99;$F4$0 = $100;
       }
      }
      HEAP32[$$pre$phiZ2D>>2] = $90;
      $103 = (($F4$0) + 12|0);
      HEAP32[$103>>2] = $90;
      $104 = (($90) + 8|0);
      HEAP32[$104>>2] = $F4$0;
      $105 = (($90) + 12|0);
      HEAP32[$105>>2] = $93;
     }
     HEAP32[((14072 + 8|0))>>2] = $81;
     HEAP32[((14072 + 20|0))>>2] = $84;
     $mem$0 = $69;
     STACKTOP = sp;return ($mem$0|0);
    }
    $106 = HEAP32[((14072 + 4|0))>>2]|0;
    $107 = ($106|0)==(0);
    if ($107) {
     $nb$0 = $5;
    } else {
     $108 = (0 - ($106))|0;
     $109 = $106 & $108;
     $110 = (($109) + -1)|0;
     $111 = $110 >>> 12;
     $112 = $111 & 16;
     $113 = $110 >>> $112;
     $114 = $113 >>> 5;
     $115 = $114 & 8;
     $116 = $115 | $112;
     $117 = $113 >>> $115;
     $118 = $117 >>> 2;
     $119 = $118 & 4;
     $120 = $116 | $119;
     $121 = $117 >>> $119;
     $122 = $121 >>> 1;
     $123 = $122 & 2;
     $124 = $120 | $123;
     $125 = $121 >>> $123;
     $126 = $125 >>> 1;
     $127 = $126 & 1;
     $128 = $124 | $127;
     $129 = $125 >>> $127;
     $130 = (($128) + ($129))|0;
     $131 = ((14072 + ($130<<2)|0) + 304|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = (($132) + 4|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = $134 & -8;
     $136 = (($135) - ($5))|0;
     $rsize$0$i = $136;$t$0$i = $132;$v$0$i = $132;
     while(1) {
      $137 = (($t$0$i) + 16|0);
      $138 = HEAP32[$137>>2]|0;
      $139 = ($138|0)==(0|0);
      if ($139) {
       $140 = (($t$0$i) + 20|0);
       $141 = HEAP32[$140>>2]|0;
       $142 = ($141|0)==(0|0);
       if ($142) {
        break;
       } else {
        $144 = $141;
       }
      } else {
       $144 = $138;
      }
      $143 = (($144) + 4|0);
      $145 = HEAP32[$143>>2]|0;
      $146 = $145 & -8;
      $147 = (($146) - ($5))|0;
      $148 = ($147>>>0)<($rsize$0$i>>>0);
      $$rsize$0$i = $148 ? $147 : $rsize$0$i;
      $$v$0$i = $148 ? $144 : $v$0$i;
      $rsize$0$i = $$rsize$0$i;$t$0$i = $144;$v$0$i = $$v$0$i;
     }
     $149 = HEAP32[((14072 + 16|0))>>2]|0;
     $150 = ($v$0$i>>>0)<($149>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($v$0$i) + ($5)|0);
     $152 = ($v$0$i>>>0)<($151>>>0);
     if (!($152)) {
      _abort();
      // unreachable;
     }
     $153 = (($v$0$i) + 24|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = (($v$0$i) + 12|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($156|0)==($v$0$i|0);
     do {
      if ($157) {
       $167 = (($v$0$i) + 20|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = ($168|0)==(0|0);
       if ($169) {
        $170 = (($v$0$i) + 16|0);
        $171 = HEAP32[$170>>2]|0;
        $172 = ($171|0)==(0|0);
        if ($172) {
         $R$1$i = 0;
         break;
        } else {
         $R$0$i = $171;$RP$0$i = $170;
        }
       } else {
        $R$0$i = $168;$RP$0$i = $167;
       }
       while(1) {
        $173 = (($R$0$i) + 20|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = ($174|0)==(0|0);
        if (!($175)) {
         $R$0$i = $174;$RP$0$i = $173;
         continue;
        }
        $176 = (($R$0$i) + 16|0);
        $177 = HEAP32[$176>>2]|0;
        $178 = ($177|0)==(0|0);
        if ($178) {
         break;
        } else {
         $R$0$i = $177;$RP$0$i = $176;
        }
       }
       $179 = ($RP$0$i>>>0)<($149>>>0);
       if ($179) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$RP$0$i>>2] = 0;
        $R$1$i = $R$0$i;
        break;
       }
      } else {
       $158 = (($v$0$i) + 8|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = ($159>>>0)<($149>>>0);
       if ($160) {
        _abort();
        // unreachable;
       }
       $161 = (($159) + 12|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = ($162|0)==($v$0$i|0);
       if (!($163)) {
        _abort();
        // unreachable;
       }
       $164 = (($156) + 8|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = ($165|0)==($v$0$i|0);
       if ($166) {
        HEAP32[$161>>2] = $156;
        HEAP32[$164>>2] = $159;
        $R$1$i = $156;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $180 = ($154|0)==(0|0);
     do {
      if (!($180)) {
       $181 = (($v$0$i) + 28|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = ((14072 + ($182<<2)|0) + 304|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($v$0$i|0)==($184|0);
       if ($185) {
        HEAP32[$183>>2] = $R$1$i;
        $cond$i = ($R$1$i|0)==(0|0);
        if ($cond$i) {
         $186 = 1 << $182;
         $187 = $186 ^ -1;
         $188 = HEAP32[((14072 + 4|0))>>2]|0;
         $189 = $188 & $187;
         HEAP32[((14072 + 4|0))>>2] = $189;
         break;
        }
       } else {
        $190 = HEAP32[((14072 + 16|0))>>2]|0;
        $191 = ($154>>>0)<($190>>>0);
        if ($191) {
         _abort();
         // unreachable;
        }
        $192 = (($154) + 16|0);
        $193 = HEAP32[$192>>2]|0;
        $194 = ($193|0)==($v$0$i|0);
        if ($194) {
         HEAP32[$192>>2] = $R$1$i;
        } else {
         $195 = (($154) + 20|0);
         HEAP32[$195>>2] = $R$1$i;
        }
        $196 = ($R$1$i|0)==(0|0);
        if ($196) {
         break;
        }
       }
       $197 = HEAP32[((14072 + 16|0))>>2]|0;
       $198 = ($R$1$i>>>0)<($197>>>0);
       if ($198) {
        _abort();
        // unreachable;
       }
       $199 = (($R$1$i) + 24|0);
       HEAP32[$199>>2] = $154;
       $200 = (($v$0$i) + 16|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)==(0|0);
       do {
        if (!($202)) {
         $203 = HEAP32[((14072 + 16|0))>>2]|0;
         $204 = ($201>>>0)<($203>>>0);
         if ($204) {
          _abort();
          // unreachable;
         } else {
          $205 = (($R$1$i) + 16|0);
          HEAP32[$205>>2] = $201;
          $206 = (($201) + 24|0);
          HEAP32[$206>>2] = $R$1$i;
          break;
         }
        }
       } while(0);
       $207 = (($v$0$i) + 20|0);
       $208 = HEAP32[$207>>2]|0;
       $209 = ($208|0)==(0|0);
       if (!($209)) {
        $210 = HEAP32[((14072 + 16|0))>>2]|0;
        $211 = ($208>>>0)<($210>>>0);
        if ($211) {
         _abort();
         // unreachable;
        } else {
         $212 = (($R$1$i) + 20|0);
         HEAP32[$212>>2] = $208;
         $213 = (($208) + 24|0);
         HEAP32[$213>>2] = $R$1$i;
         break;
        }
       }
      }
     } while(0);
     $214 = ($rsize$0$i>>>0)<(16);
     if ($214) {
      $215 = (($rsize$0$i) + ($5))|0;
      $216 = $215 | 3;
      $217 = (($v$0$i) + 4|0);
      HEAP32[$217>>2] = $216;
      $$sum4$i = (($215) + 4)|0;
      $218 = (($v$0$i) + ($$sum4$i)|0);
      $219 = HEAP32[$218>>2]|0;
      $220 = $219 | 1;
      HEAP32[$218>>2] = $220;
     } else {
      $221 = $5 | 3;
      $222 = (($v$0$i) + 4|0);
      HEAP32[$222>>2] = $221;
      $223 = $rsize$0$i | 1;
      $$sum$i35 = $5 | 4;
      $224 = (($v$0$i) + ($$sum$i35)|0);
      HEAP32[$224>>2] = $223;
      $$sum1$i = (($rsize$0$i) + ($5))|0;
      $225 = (($v$0$i) + ($$sum1$i)|0);
      HEAP32[$225>>2] = $rsize$0$i;
      $226 = HEAP32[((14072 + 8|0))>>2]|0;
      $227 = ($226|0)==(0);
      if (!($227)) {
       $228 = HEAP32[((14072 + 20|0))>>2]|0;
       $229 = $226 >>> 3;
       $230 = $229 << 1;
       $231 = ((14072 + ($230<<2)|0) + 40|0);
       $232 = HEAP32[14072>>2]|0;
       $233 = 1 << $229;
       $234 = $232 & $233;
       $235 = ($234|0)==(0);
       if ($235) {
        $236 = $232 | $233;
        HEAP32[14072>>2] = $236;
        $$sum2$pre$i = (($230) + 2)|0;
        $$pre$i = ((14072 + ($$sum2$pre$i<<2)|0) + 40|0);
        $$pre$phi$iZ2D = $$pre$i;$F1$0$i = $231;
       } else {
        $$sum3$i = (($230) + 2)|0;
        $237 = ((14072 + ($$sum3$i<<2)|0) + 40|0);
        $238 = HEAP32[$237>>2]|0;
        $239 = HEAP32[((14072 + 16|0))>>2]|0;
        $240 = ($238>>>0)<($239>>>0);
        if ($240) {
         _abort();
         // unreachable;
        } else {
         $$pre$phi$iZ2D = $237;$F1$0$i = $238;
        }
       }
       HEAP32[$$pre$phi$iZ2D>>2] = $228;
       $241 = (($F1$0$i) + 12|0);
       HEAP32[$241>>2] = $228;
       $242 = (($228) + 8|0);
       HEAP32[$242>>2] = $F1$0$i;
       $243 = (($228) + 12|0);
       HEAP32[$243>>2] = $231;
      }
      HEAP32[((14072 + 8|0))>>2] = $rsize$0$i;
      HEAP32[((14072 + 20|0))>>2] = $151;
     }
     $244 = (($v$0$i) + 8|0);
     $mem$0 = $244;
     STACKTOP = sp;return ($mem$0|0);
    }
   } else {
    $nb$0 = $5;
   }
  } else {
   $245 = ($bytes>>>0)>(4294967231);
   if ($245) {
    $nb$0 = -1;
   } else {
    $246 = (($bytes) + 11)|0;
    $247 = $246 & -8;
    $248 = HEAP32[((14072 + 4|0))>>2]|0;
    $249 = ($248|0)==(0);
    if ($249) {
     $nb$0 = $247;
    } else {
     $250 = (0 - ($247))|0;
     $251 = $246 >>> 8;
     $252 = ($251|0)==(0);
     if ($252) {
      $idx$0$i = 0;
     } else {
      $253 = ($247>>>0)>(16777215);
      if ($253) {
       $idx$0$i = 31;
      } else {
       $254 = (($251) + 1048320)|0;
       $255 = $254 >>> 16;
       $256 = $255 & 8;
       $257 = $251 << $256;
       $258 = (($257) + 520192)|0;
       $259 = $258 >>> 16;
       $260 = $259 & 4;
       $261 = $260 | $256;
       $262 = $257 << $260;
       $263 = (($262) + 245760)|0;
       $264 = $263 >>> 16;
       $265 = $264 & 2;
       $266 = $261 | $265;
       $267 = (14 - ($266))|0;
       $268 = $262 << $265;
       $269 = $268 >>> 15;
       $270 = (($267) + ($269))|0;
       $271 = $270 << 1;
       $272 = (($270) + 7)|0;
       $273 = $247 >>> $272;
       $274 = $273 & 1;
       $275 = $274 | $271;
       $idx$0$i = $275;
      }
     }
     $276 = ((14072 + ($idx$0$i<<2)|0) + 304|0);
     $277 = HEAP32[$276>>2]|0;
     $278 = ($277|0)==(0|0);
     L126: do {
      if ($278) {
       $rsize$2$i = $250;$t$1$i = 0;$v$2$i = 0;
      } else {
       $279 = ($idx$0$i|0)==(31);
       if ($279) {
        $283 = 0;
       } else {
        $280 = $idx$0$i >>> 1;
        $281 = (25 - ($280))|0;
        $283 = $281;
       }
       $282 = $247 << $283;
       $rsize$0$i15 = $250;$rst$0$i = 0;$sizebits$0$i = $282;$t$0$i14 = $277;$v$0$i16 = 0;
       while(1) {
        $284 = (($t$0$i14) + 4|0);
        $285 = HEAP32[$284>>2]|0;
        $286 = $285 & -8;
        $287 = (($286) - ($247))|0;
        $288 = ($287>>>0)<($rsize$0$i15>>>0);
        if ($288) {
         $289 = ($286|0)==($247|0);
         if ($289) {
          $rsize$2$i = $287;$t$1$i = $t$0$i14;$v$2$i = $t$0$i14;
          break L126;
         } else {
          $rsize$1$i = $287;$v$1$i = $t$0$i14;
         }
        } else {
         $rsize$1$i = $rsize$0$i15;$v$1$i = $v$0$i16;
        }
        $290 = (($t$0$i14) + 20|0);
        $291 = HEAP32[$290>>2]|0;
        $292 = $sizebits$0$i >>> 31;
        $293 = ((($t$0$i14) + ($292<<2)|0) + 16|0);
        $294 = HEAP32[$293>>2]|0;
        $295 = ($291|0)==(0|0);
        $296 = ($291|0)==($294|0);
        $or$cond$i = $295 | $296;
        $rst$1$i = $or$cond$i ? $rst$0$i : $291;
        $297 = ($294|0)==(0|0);
        $298 = $sizebits$0$i << 1;
        if ($297) {
         $rsize$2$i = $rsize$1$i;$t$1$i = $rst$1$i;$v$2$i = $v$1$i;
         break;
        } else {
         $rsize$0$i15 = $rsize$1$i;$rst$0$i = $rst$1$i;$sizebits$0$i = $298;$t$0$i14 = $294;$v$0$i16 = $v$1$i;
        }
       }
      }
     } while(0);
     $299 = ($t$1$i|0)==(0|0);
     $300 = ($v$2$i|0)==(0|0);
     $or$cond19$i = $299 & $300;
     if ($or$cond19$i) {
      $301 = 2 << $idx$0$i;
      $302 = (0 - ($301))|0;
      $303 = $301 | $302;
      $304 = $248 & $303;
      $305 = ($304|0)==(0);
      if ($305) {
       $nb$0 = $247;
       break;
      }
      $306 = (0 - ($304))|0;
      $307 = $304 & $306;
      $308 = (($307) + -1)|0;
      $309 = $308 >>> 12;
      $310 = $309 & 16;
      $311 = $308 >>> $310;
      $312 = $311 >>> 5;
      $313 = $312 & 8;
      $314 = $313 | $310;
      $315 = $311 >>> $313;
      $316 = $315 >>> 2;
      $317 = $316 & 4;
      $318 = $314 | $317;
      $319 = $315 >>> $317;
      $320 = $319 >>> 1;
      $321 = $320 & 2;
      $322 = $318 | $321;
      $323 = $319 >>> $321;
      $324 = $323 >>> 1;
      $325 = $324 & 1;
      $326 = $322 | $325;
      $327 = $323 >>> $325;
      $328 = (($326) + ($327))|0;
      $329 = ((14072 + ($328<<2)|0) + 304|0);
      $330 = HEAP32[$329>>2]|0;
      $t$2$ph$i = $330;
     } else {
      $t$2$ph$i = $t$1$i;
     }
     $331 = ($t$2$ph$i|0)==(0|0);
     if ($331) {
      $rsize$3$lcssa$i = $rsize$2$i;$v$3$lcssa$i = $v$2$i;
     } else {
      $rsize$329$i = $rsize$2$i;$t$228$i = $t$2$ph$i;$v$330$i = $v$2$i;
      while(1) {
       $332 = (($t$228$i) + 4|0);
       $333 = HEAP32[$332>>2]|0;
       $334 = $333 & -8;
       $335 = (($334) - ($247))|0;
       $336 = ($335>>>0)<($rsize$329$i>>>0);
       $$rsize$3$i = $336 ? $335 : $rsize$329$i;
       $t$2$v$3$i = $336 ? $t$228$i : $v$330$i;
       $337 = (($t$228$i) + 16|0);
       $338 = HEAP32[$337>>2]|0;
       $339 = ($338|0)==(0|0);
       if (!($339)) {
        $rsize$329$i = $$rsize$3$i;$t$228$i = $338;$v$330$i = $t$2$v$3$i;
        continue;
       }
       $340 = (($t$228$i) + 20|0);
       $341 = HEAP32[$340>>2]|0;
       $342 = ($341|0)==(0|0);
       if ($342) {
        $rsize$3$lcssa$i = $$rsize$3$i;$v$3$lcssa$i = $t$2$v$3$i;
        break;
       } else {
        $rsize$329$i = $$rsize$3$i;$t$228$i = $341;$v$330$i = $t$2$v$3$i;
       }
      }
     }
     $343 = ($v$3$lcssa$i|0)==(0|0);
     if ($343) {
      $nb$0 = $247;
     } else {
      $344 = HEAP32[((14072 + 8|0))>>2]|0;
      $345 = (($344) - ($247))|0;
      $346 = ($rsize$3$lcssa$i>>>0)<($345>>>0);
      if ($346) {
       $347 = HEAP32[((14072 + 16|0))>>2]|0;
       $348 = ($v$3$lcssa$i>>>0)<($347>>>0);
       if ($348) {
        _abort();
        // unreachable;
       }
       $349 = (($v$3$lcssa$i) + ($247)|0);
       $350 = ($v$3$lcssa$i>>>0)<($349>>>0);
       if (!($350)) {
        _abort();
        // unreachable;
       }
       $351 = (($v$3$lcssa$i) + 24|0);
       $352 = HEAP32[$351>>2]|0;
       $353 = (($v$3$lcssa$i) + 12|0);
       $354 = HEAP32[$353>>2]|0;
       $355 = ($354|0)==($v$3$lcssa$i|0);
       do {
        if ($355) {
         $365 = (($v$3$lcssa$i) + 20|0);
         $366 = HEAP32[$365>>2]|0;
         $367 = ($366|0)==(0|0);
         if ($367) {
          $368 = (($v$3$lcssa$i) + 16|0);
          $369 = HEAP32[$368>>2]|0;
          $370 = ($369|0)==(0|0);
          if ($370) {
           $R$1$i20 = 0;
           break;
          } else {
           $R$0$i18 = $369;$RP$0$i17 = $368;
          }
         } else {
          $R$0$i18 = $366;$RP$0$i17 = $365;
         }
         while(1) {
          $371 = (($R$0$i18) + 20|0);
          $372 = HEAP32[$371>>2]|0;
          $373 = ($372|0)==(0|0);
          if (!($373)) {
           $R$0$i18 = $372;$RP$0$i17 = $371;
           continue;
          }
          $374 = (($R$0$i18) + 16|0);
          $375 = HEAP32[$374>>2]|0;
          $376 = ($375|0)==(0|0);
          if ($376) {
           break;
          } else {
           $R$0$i18 = $375;$RP$0$i17 = $374;
          }
         }
         $377 = ($RP$0$i17>>>0)<($347>>>0);
         if ($377) {
          _abort();
          // unreachable;
         } else {
          HEAP32[$RP$0$i17>>2] = 0;
          $R$1$i20 = $R$0$i18;
          break;
         }
        } else {
         $356 = (($v$3$lcssa$i) + 8|0);
         $357 = HEAP32[$356>>2]|0;
         $358 = ($357>>>0)<($347>>>0);
         if ($358) {
          _abort();
          // unreachable;
         }
         $359 = (($357) + 12|0);
         $360 = HEAP32[$359>>2]|0;
         $361 = ($360|0)==($v$3$lcssa$i|0);
         if (!($361)) {
          _abort();
          // unreachable;
         }
         $362 = (($354) + 8|0);
         $363 = HEAP32[$362>>2]|0;
         $364 = ($363|0)==($v$3$lcssa$i|0);
         if ($364) {
          HEAP32[$359>>2] = $354;
          HEAP32[$362>>2] = $357;
          $R$1$i20 = $354;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $378 = ($352|0)==(0|0);
       do {
        if (!($378)) {
         $379 = (($v$3$lcssa$i) + 28|0);
         $380 = HEAP32[$379>>2]|0;
         $381 = ((14072 + ($380<<2)|0) + 304|0);
         $382 = HEAP32[$381>>2]|0;
         $383 = ($v$3$lcssa$i|0)==($382|0);
         if ($383) {
          HEAP32[$381>>2] = $R$1$i20;
          $cond$i21 = ($R$1$i20|0)==(0|0);
          if ($cond$i21) {
           $384 = 1 << $380;
           $385 = $384 ^ -1;
           $386 = HEAP32[((14072 + 4|0))>>2]|0;
           $387 = $386 & $385;
           HEAP32[((14072 + 4|0))>>2] = $387;
           break;
          }
         } else {
          $388 = HEAP32[((14072 + 16|0))>>2]|0;
          $389 = ($352>>>0)<($388>>>0);
          if ($389) {
           _abort();
           // unreachable;
          }
          $390 = (($352) + 16|0);
          $391 = HEAP32[$390>>2]|0;
          $392 = ($391|0)==($v$3$lcssa$i|0);
          if ($392) {
           HEAP32[$390>>2] = $R$1$i20;
          } else {
           $393 = (($352) + 20|0);
           HEAP32[$393>>2] = $R$1$i20;
          }
          $394 = ($R$1$i20|0)==(0|0);
          if ($394) {
           break;
          }
         }
         $395 = HEAP32[((14072 + 16|0))>>2]|0;
         $396 = ($R$1$i20>>>0)<($395>>>0);
         if ($396) {
          _abort();
          // unreachable;
         }
         $397 = (($R$1$i20) + 24|0);
         HEAP32[$397>>2] = $352;
         $398 = (($v$3$lcssa$i) + 16|0);
         $399 = HEAP32[$398>>2]|0;
         $400 = ($399|0)==(0|0);
         do {
          if (!($400)) {
           $401 = HEAP32[((14072 + 16|0))>>2]|0;
           $402 = ($399>>>0)<($401>>>0);
           if ($402) {
            _abort();
            // unreachable;
           } else {
            $403 = (($R$1$i20) + 16|0);
            HEAP32[$403>>2] = $399;
            $404 = (($399) + 24|0);
            HEAP32[$404>>2] = $R$1$i20;
            break;
           }
          }
         } while(0);
         $405 = (($v$3$lcssa$i) + 20|0);
         $406 = HEAP32[$405>>2]|0;
         $407 = ($406|0)==(0|0);
         if (!($407)) {
          $408 = HEAP32[((14072 + 16|0))>>2]|0;
          $409 = ($406>>>0)<($408>>>0);
          if ($409) {
           _abort();
           // unreachable;
          } else {
           $410 = (($R$1$i20) + 20|0);
           HEAP32[$410>>2] = $406;
           $411 = (($406) + 24|0);
           HEAP32[$411>>2] = $R$1$i20;
           break;
          }
         }
        }
       } while(0);
       $412 = ($rsize$3$lcssa$i>>>0)<(16);
       L204: do {
        if ($412) {
         $413 = (($rsize$3$lcssa$i) + ($247))|0;
         $414 = $413 | 3;
         $415 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$415>>2] = $414;
         $$sum18$i = (($413) + 4)|0;
         $416 = (($v$3$lcssa$i) + ($$sum18$i)|0);
         $417 = HEAP32[$416>>2]|0;
         $418 = $417 | 1;
         HEAP32[$416>>2] = $418;
        } else {
         $419 = $247 | 3;
         $420 = (($v$3$lcssa$i) + 4|0);
         HEAP32[$420>>2] = $419;
         $421 = $rsize$3$lcssa$i | 1;
         $$sum$i2334 = $247 | 4;
         $422 = (($v$3$lcssa$i) + ($$sum$i2334)|0);
         HEAP32[$422>>2] = $421;
         $$sum1$i24 = (($rsize$3$lcssa$i) + ($247))|0;
         $423 = (($v$3$lcssa$i) + ($$sum1$i24)|0);
         HEAP32[$423>>2] = $rsize$3$lcssa$i;
         $424 = $rsize$3$lcssa$i >>> 3;
         $425 = ($rsize$3$lcssa$i>>>0)<(256);
         if ($425) {
          $426 = $424 << 1;
          $427 = ((14072 + ($426<<2)|0) + 40|0);
          $428 = HEAP32[14072>>2]|0;
          $429 = 1 << $424;
          $430 = $428 & $429;
          $431 = ($430|0)==(0);
          do {
           if ($431) {
            $432 = $428 | $429;
            HEAP32[14072>>2] = $432;
            $$sum14$pre$i = (($426) + 2)|0;
            $$pre$i25 = ((14072 + ($$sum14$pre$i<<2)|0) + 40|0);
            $$pre$phi$i26Z2D = $$pre$i25;$F5$0$i = $427;
           } else {
            $$sum17$i = (($426) + 2)|0;
            $433 = ((14072 + ($$sum17$i<<2)|0) + 40|0);
            $434 = HEAP32[$433>>2]|0;
            $435 = HEAP32[((14072 + 16|0))>>2]|0;
            $436 = ($434>>>0)<($435>>>0);
            if (!($436)) {
             $$pre$phi$i26Z2D = $433;$F5$0$i = $434;
             break;
            }
            _abort();
            // unreachable;
           }
          } while(0);
          HEAP32[$$pre$phi$i26Z2D>>2] = $349;
          $437 = (($F5$0$i) + 12|0);
          HEAP32[$437>>2] = $349;
          $$sum15$i = (($247) + 8)|0;
          $438 = (($v$3$lcssa$i) + ($$sum15$i)|0);
          HEAP32[$438>>2] = $F5$0$i;
          $$sum16$i = (($247) + 12)|0;
          $439 = (($v$3$lcssa$i) + ($$sum16$i)|0);
          HEAP32[$439>>2] = $427;
          break;
         }
         $440 = $rsize$3$lcssa$i >>> 8;
         $441 = ($440|0)==(0);
         if ($441) {
          $I7$0$i = 0;
         } else {
          $442 = ($rsize$3$lcssa$i>>>0)>(16777215);
          if ($442) {
           $I7$0$i = 31;
          } else {
           $443 = (($440) + 1048320)|0;
           $444 = $443 >>> 16;
           $445 = $444 & 8;
           $446 = $440 << $445;
           $447 = (($446) + 520192)|0;
           $448 = $447 >>> 16;
           $449 = $448 & 4;
           $450 = $449 | $445;
           $451 = $446 << $449;
           $452 = (($451) + 245760)|0;
           $453 = $452 >>> 16;
           $454 = $453 & 2;
           $455 = $450 | $454;
           $456 = (14 - ($455))|0;
           $457 = $451 << $454;
           $458 = $457 >>> 15;
           $459 = (($456) + ($458))|0;
           $460 = $459 << 1;
           $461 = (($459) + 7)|0;
           $462 = $rsize$3$lcssa$i >>> $461;
           $463 = $462 & 1;
           $464 = $463 | $460;
           $I7$0$i = $464;
          }
         }
         $465 = ((14072 + ($I7$0$i<<2)|0) + 304|0);
         $$sum2$i = (($247) + 28)|0;
         $466 = (($v$3$lcssa$i) + ($$sum2$i)|0);
         HEAP32[$466>>2] = $I7$0$i;
         $$sum3$i27 = (($247) + 16)|0;
         $467 = (($v$3$lcssa$i) + ($$sum3$i27)|0);
         $$sum4$i28 = (($247) + 20)|0;
         $468 = (($v$3$lcssa$i) + ($$sum4$i28)|0);
         HEAP32[$468>>2] = 0;
         HEAP32[$467>>2] = 0;
         $469 = HEAP32[((14072 + 4|0))>>2]|0;
         $470 = 1 << $I7$0$i;
         $471 = $469 & $470;
         $472 = ($471|0)==(0);
         if ($472) {
          $473 = $469 | $470;
          HEAP32[((14072 + 4|0))>>2] = $473;
          HEAP32[$465>>2] = $349;
          $$sum5$i = (($247) + 24)|0;
          $474 = (($v$3$lcssa$i) + ($$sum5$i)|0);
          HEAP32[$474>>2] = $465;
          $$sum6$i = (($247) + 12)|0;
          $475 = (($v$3$lcssa$i) + ($$sum6$i)|0);
          HEAP32[$475>>2] = $349;
          $$sum7$i = (($247) + 8)|0;
          $476 = (($v$3$lcssa$i) + ($$sum7$i)|0);
          HEAP32[$476>>2] = $349;
          break;
         }
         $477 = HEAP32[$465>>2]|0;
         $478 = ($I7$0$i|0)==(31);
         if ($478) {
          $486 = 0;
         } else {
          $479 = $I7$0$i >>> 1;
          $480 = (25 - ($479))|0;
          $486 = $480;
         }
         $481 = (($477) + 4|0);
         $482 = HEAP32[$481>>2]|0;
         $483 = $482 & -8;
         $484 = ($483|0)==($rsize$3$lcssa$i|0);
         L225: do {
          if ($484) {
           $T$0$lcssa$i = $477;
          } else {
           $485 = $rsize$3$lcssa$i << $486;
           $K12$025$i = $485;$T$024$i = $477;
           while(1) {
            $493 = $K12$025$i >>> 31;
            $494 = ((($T$024$i) + ($493<<2)|0) + 16|0);
            $489 = HEAP32[$494>>2]|0;
            $495 = ($489|0)==(0|0);
            if ($495) {
             break;
            }
            $487 = $K12$025$i << 1;
            $488 = (($489) + 4|0);
            $490 = HEAP32[$488>>2]|0;
            $491 = $490 & -8;
            $492 = ($491|0)==($rsize$3$lcssa$i|0);
            if ($492) {
             $T$0$lcssa$i = $489;
             break L225;
            } else {
             $K12$025$i = $487;$T$024$i = $489;
            }
           }
           $496 = HEAP32[((14072 + 16|0))>>2]|0;
           $497 = ($494>>>0)<($496>>>0);
           if ($497) {
            _abort();
            // unreachable;
           } else {
            HEAP32[$494>>2] = $349;
            $$sum11$i = (($247) + 24)|0;
            $498 = (($v$3$lcssa$i) + ($$sum11$i)|0);
            HEAP32[$498>>2] = $T$024$i;
            $$sum12$i = (($247) + 12)|0;
            $499 = (($v$3$lcssa$i) + ($$sum12$i)|0);
            HEAP32[$499>>2] = $349;
            $$sum13$i = (($247) + 8)|0;
            $500 = (($v$3$lcssa$i) + ($$sum13$i)|0);
            HEAP32[$500>>2] = $349;
            break L204;
           }
          }
         } while(0);
         $501 = (($T$0$lcssa$i) + 8|0);
         $502 = HEAP32[$501>>2]|0;
         $503 = HEAP32[((14072 + 16|0))>>2]|0;
         $504 = ($T$0$lcssa$i>>>0)<($503>>>0);
         if ($504) {
          _abort();
          // unreachable;
         }
         $505 = ($502>>>0)<($503>>>0);
         if ($505) {
          _abort();
          // unreachable;
         } else {
          $506 = (($502) + 12|0);
          HEAP32[$506>>2] = $349;
          HEAP32[$501>>2] = $349;
          $$sum8$i = (($247) + 8)|0;
          $507 = (($v$3$lcssa$i) + ($$sum8$i)|0);
          HEAP32[$507>>2] = $502;
          $$sum9$i = (($247) + 12)|0;
          $508 = (($v$3$lcssa$i) + ($$sum9$i)|0);
          HEAP32[$508>>2] = $T$0$lcssa$i;
          $$sum10$i = (($247) + 24)|0;
          $509 = (($v$3$lcssa$i) + ($$sum10$i)|0);
          HEAP32[$509>>2] = 0;
          break;
         }
        }
       } while(0);
       $510 = (($v$3$lcssa$i) + 8|0);
       $mem$0 = $510;
       STACKTOP = sp;return ($mem$0|0);
      } else {
       $nb$0 = $247;
      }
     }
    }
   }
  }
 } while(0);
 $511 = HEAP32[((14072 + 8|0))>>2]|0;
 $512 = ($nb$0>>>0)>($511>>>0);
 if (!($512)) {
  $513 = (($511) - ($nb$0))|0;
  $514 = HEAP32[((14072 + 20|0))>>2]|0;
  $515 = ($513>>>0)>(15);
  if ($515) {
   $516 = (($514) + ($nb$0)|0);
   HEAP32[((14072 + 20|0))>>2] = $516;
   HEAP32[((14072 + 8|0))>>2] = $513;
   $517 = $513 | 1;
   $$sum2 = (($nb$0) + 4)|0;
   $518 = (($514) + ($$sum2)|0);
   HEAP32[$518>>2] = $517;
   $519 = (($514) + ($511)|0);
   HEAP32[$519>>2] = $513;
   $520 = $nb$0 | 3;
   $521 = (($514) + 4|0);
   HEAP32[$521>>2] = $520;
  } else {
   HEAP32[((14072 + 8|0))>>2] = 0;
   HEAP32[((14072 + 20|0))>>2] = 0;
   $522 = $511 | 3;
   $523 = (($514) + 4|0);
   HEAP32[$523>>2] = $522;
   $$sum1 = (($511) + 4)|0;
   $524 = (($514) + ($$sum1)|0);
   $525 = HEAP32[$524>>2]|0;
   $526 = $525 | 1;
   HEAP32[$524>>2] = $526;
  }
  $527 = (($514) + 8|0);
  $mem$0 = $527;
  STACKTOP = sp;return ($mem$0|0);
 }
 $528 = HEAP32[((14072 + 12|0))>>2]|0;
 $529 = ($nb$0>>>0)<($528>>>0);
 if ($529) {
  $530 = (($528) - ($nb$0))|0;
  HEAP32[((14072 + 12|0))>>2] = $530;
  $531 = HEAP32[((14072 + 24|0))>>2]|0;
  $532 = (($531) + ($nb$0)|0);
  HEAP32[((14072 + 24|0))>>2] = $532;
  $533 = $530 | 1;
  $$sum = (($nb$0) + 4)|0;
  $534 = (($531) + ($$sum)|0);
  HEAP32[$534>>2] = $533;
  $535 = $nb$0 | 3;
  $536 = (($531) + 4|0);
  HEAP32[$536>>2] = $535;
  $537 = (($531) + 8|0);
  $mem$0 = $537;
  STACKTOP = sp;return ($mem$0|0);
 }
 $538 = HEAP32[14544>>2]|0;
 $539 = ($538|0)==(0);
 do {
  if ($539) {
   $540 = (_sysconf(30)|0);
   $541 = (($540) + -1)|0;
   $542 = $541 & $540;
   $543 = ($542|0)==(0);
   if ($543) {
    HEAP32[((14544 + 8|0))>>2] = $540;
    HEAP32[((14544 + 4|0))>>2] = $540;
    HEAP32[((14544 + 12|0))>>2] = -1;
    HEAP32[((14544 + 16|0))>>2] = -1;
    HEAP32[((14544 + 20|0))>>2] = 0;
    HEAP32[((14072 + 444|0))>>2] = 0;
    $544 = (_time((0|0))|0);
    $545 = $544 & -16;
    $546 = $545 ^ 1431655768;
    HEAP32[14544>>2] = $546;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $547 = (($nb$0) + 48)|0;
 $548 = HEAP32[((14544 + 8|0))>>2]|0;
 $549 = (($nb$0) + 47)|0;
 $550 = (($548) + ($549))|0;
 $551 = (0 - ($548))|0;
 $552 = $550 & $551;
 $553 = ($552>>>0)>($nb$0>>>0);
 if (!($553)) {
  $mem$0 = 0;
  STACKTOP = sp;return ($mem$0|0);
 }
 $554 = HEAP32[((14072 + 440|0))>>2]|0;
 $555 = ($554|0)==(0);
 if (!($555)) {
  $556 = HEAP32[((14072 + 432|0))>>2]|0;
  $557 = (($556) + ($552))|0;
  $558 = ($557>>>0)<=($556>>>0);
  $559 = ($557>>>0)>($554>>>0);
  $or$cond1$i = $558 | $559;
  if ($or$cond1$i) {
   $mem$0 = 0;
   STACKTOP = sp;return ($mem$0|0);
  }
 }
 $560 = HEAP32[((14072 + 444|0))>>2]|0;
 $561 = $560 & 4;
 $562 = ($561|0)==(0);
 L269: do {
  if ($562) {
   $563 = HEAP32[((14072 + 24|0))>>2]|0;
   $564 = ($563|0)==(0|0);
   L271: do {
    if ($564) {
     label = 182;
    } else {
     $sp$0$i$i = ((14072 + 448|0));
     while(1) {
      $565 = HEAP32[$sp$0$i$i>>2]|0;
      $566 = ($565>>>0)>($563>>>0);
      if (!($566)) {
       $567 = (($sp$0$i$i) + 4|0);
       $568 = HEAP32[$567>>2]|0;
       $569 = (($565) + ($568)|0);
       $570 = ($569>>>0)>($563>>>0);
       if ($570) {
        break;
       }
      }
      $571 = (($sp$0$i$i) + 8|0);
      $572 = HEAP32[$571>>2]|0;
      $573 = ($572|0)==(0|0);
      if ($573) {
       label = 182;
       break L271;
      } else {
       $sp$0$i$i = $572;
      }
     }
     $574 = ($sp$0$i$i|0)==(0|0);
     if ($574) {
      label = 182;
     } else {
      $597 = HEAP32[((14072 + 12|0))>>2]|0;
      $598 = (($550) - ($597))|0;
      $599 = $598 & $551;
      $600 = ($599>>>0)<(2147483647);
      if ($600) {
       $601 = (_sbrk(($599|0))|0);
       $602 = HEAP32[$sp$0$i$i>>2]|0;
       $603 = HEAP32[$567>>2]|0;
       $604 = (($602) + ($603)|0);
       $605 = ($601|0)==($604|0);
       $$3$i = $605 ? $599 : 0;
       $$4$i = $605 ? $601 : (-1);
       $br$0$i = $601;$ssize$1$i = $599;$tbase$0$i = $$4$i;$tsize$0$i = $$3$i;
       label = 191;
      } else {
       $tsize$0323841$i = 0;
      }
     }
    }
   } while(0);
   do {
    if ((label|0) == 182) {
     $575 = (_sbrk(0)|0);
     $576 = ($575|0)==((-1)|0);
     if ($576) {
      $tsize$0323841$i = 0;
     } else {
      $577 = $575;
      $578 = HEAP32[((14544 + 4|0))>>2]|0;
      $579 = (($578) + -1)|0;
      $580 = $579 & $577;
      $581 = ($580|0)==(0);
      if ($581) {
       $ssize$0$i = $552;
      } else {
       $582 = (($579) + ($577))|0;
       $583 = (0 - ($578))|0;
       $584 = $582 & $583;
       $585 = (($552) - ($577))|0;
       $586 = (($585) + ($584))|0;
       $ssize$0$i = $586;
      }
      $587 = HEAP32[((14072 + 432|0))>>2]|0;
      $588 = (($587) + ($ssize$0$i))|0;
      $589 = ($ssize$0$i>>>0)>($nb$0>>>0);
      $590 = ($ssize$0$i>>>0)<(2147483647);
      $or$cond$i29 = $589 & $590;
      if ($or$cond$i29) {
       $591 = HEAP32[((14072 + 440|0))>>2]|0;
       $592 = ($591|0)==(0);
       if (!($592)) {
        $593 = ($588>>>0)<=($587>>>0);
        $594 = ($588>>>0)>($591>>>0);
        $or$cond2$i = $593 | $594;
        if ($or$cond2$i) {
         $tsize$0323841$i = 0;
         break;
        }
       }
       $595 = (_sbrk(($ssize$0$i|0))|0);
       $596 = ($595|0)==($575|0);
       $ssize$0$$i = $596 ? $ssize$0$i : 0;
       $$$i = $596 ? $575 : (-1);
       $br$0$i = $595;$ssize$1$i = $ssize$0$i;$tbase$0$i = $$$i;$tsize$0$i = $ssize$0$$i;
       label = 191;
      } else {
       $tsize$0323841$i = 0;
      }
     }
    }
   } while(0);
   L291: do {
    if ((label|0) == 191) {
     $606 = (0 - ($ssize$1$i))|0;
     $607 = ($tbase$0$i|0)==((-1)|0);
     if (!($607)) {
      $tbase$247$i = $tbase$0$i;$tsize$246$i = $tsize$0$i;
      label = 202;
      break L269;
     }
     $608 = ($br$0$i|0)!=((-1)|0);
     $609 = ($ssize$1$i>>>0)<(2147483647);
     $or$cond5$i = $608 & $609;
     $610 = ($ssize$1$i>>>0)<($547>>>0);
     $or$cond6$i = $or$cond5$i & $610;
     do {
      if ($or$cond6$i) {
       $611 = HEAP32[((14544 + 8|0))>>2]|0;
       $612 = (($549) - ($ssize$1$i))|0;
       $613 = (($612) + ($611))|0;
       $614 = (0 - ($611))|0;
       $615 = $613 & $614;
       $616 = ($615>>>0)<(2147483647);
       if ($616) {
        $617 = (_sbrk(($615|0))|0);
        $618 = ($617|0)==((-1)|0);
        if ($618) {
         (_sbrk(($606|0))|0);
         $tsize$0323841$i = $tsize$0$i;
         break L291;
        } else {
         $619 = (($615) + ($ssize$1$i))|0;
         $ssize$2$i = $619;
         break;
        }
       } else {
        $ssize$2$i = $ssize$1$i;
       }
      } else {
       $ssize$2$i = $ssize$1$i;
      }
     } while(0);
     $620 = ($br$0$i|0)==((-1)|0);
     if ($620) {
      $tsize$0323841$i = $tsize$0$i;
     } else {
      $tbase$247$i = $br$0$i;$tsize$246$i = $ssize$2$i;
      label = 202;
      break L269;
     }
    }
   } while(0);
   $621 = HEAP32[((14072 + 444|0))>>2]|0;
   $622 = $621 | 4;
   HEAP32[((14072 + 444|0))>>2] = $622;
   $tsize$1$i = $tsize$0323841$i;
   label = 199;
  } else {
   $tsize$1$i = 0;
   label = 199;
  }
 } while(0);
 if ((label|0) == 199) {
  $623 = ($552>>>0)<(2147483647);
  if ($623) {
   $624 = (_sbrk(($552|0))|0);
   $625 = (_sbrk(0)|0);
   $notlhs$i = ($624|0)!=((-1)|0);
   $notrhs$i = ($625|0)!=((-1)|0);
   $or$cond8$not$i = $notrhs$i & $notlhs$i;
   $626 = ($624>>>0)<($625>>>0);
   $or$cond9$i = $or$cond8$not$i & $626;
   if ($or$cond9$i) {
    $627 = $625;
    $628 = $624;
    $629 = (($627) - ($628))|0;
    $630 = (($nb$0) + 40)|0;
    $631 = ($629>>>0)>($630>>>0);
    $$tsize$1$i = $631 ? $629 : $tsize$1$i;
    if ($631) {
     $tbase$247$i = $624;$tsize$246$i = $$tsize$1$i;
     label = 202;
    }
   }
  }
 }
 if ((label|0) == 202) {
  $632 = HEAP32[((14072 + 432|0))>>2]|0;
  $633 = (($632) + ($tsize$246$i))|0;
  HEAP32[((14072 + 432|0))>>2] = $633;
  $634 = HEAP32[((14072 + 436|0))>>2]|0;
  $635 = ($633>>>0)>($634>>>0);
  if ($635) {
   HEAP32[((14072 + 436|0))>>2] = $633;
  }
  $636 = HEAP32[((14072 + 24|0))>>2]|0;
  $637 = ($636|0)==(0|0);
  L311: do {
   if ($637) {
    $638 = HEAP32[((14072 + 16|0))>>2]|0;
    $639 = ($638|0)==(0|0);
    $640 = ($tbase$247$i>>>0)<($638>>>0);
    $or$cond10$i = $639 | $640;
    if ($or$cond10$i) {
     HEAP32[((14072 + 16|0))>>2] = $tbase$247$i;
    }
    HEAP32[((14072 + 448|0))>>2] = $tbase$247$i;
    HEAP32[((14072 + 452|0))>>2] = $tsize$246$i;
    HEAP32[((14072 + 460|0))>>2] = 0;
    $641 = HEAP32[14544>>2]|0;
    HEAP32[((14072 + 36|0))>>2] = $641;
    HEAP32[((14072 + 32|0))>>2] = -1;
    $i$02$i$i = 0;
    while(1) {
     $642 = $i$02$i$i << 1;
     $643 = ((14072 + ($642<<2)|0) + 40|0);
     $$sum$i$i = (($642) + 3)|0;
     $644 = ((14072 + ($$sum$i$i<<2)|0) + 40|0);
     HEAP32[$644>>2] = $643;
     $$sum1$i$i = (($642) + 2)|0;
     $645 = ((14072 + ($$sum1$i$i<<2)|0) + 40|0);
     HEAP32[$645>>2] = $643;
     $646 = (($i$02$i$i) + 1)|0;
     $exitcond$i$i = ($646|0)==(32);
     if ($exitcond$i$i) {
      break;
     } else {
      $i$02$i$i = $646;
     }
    }
    $647 = (($tsize$246$i) + -40)|0;
    $648 = (($tbase$247$i) + 8|0);
    $649 = $648;
    $650 = $649 & 7;
    $651 = ($650|0)==(0);
    if ($651) {
     $655 = 0;
    } else {
     $652 = (0 - ($649))|0;
     $653 = $652 & 7;
     $655 = $653;
    }
    $654 = (($tbase$247$i) + ($655)|0);
    $656 = (($647) - ($655))|0;
    HEAP32[((14072 + 24|0))>>2] = $654;
    HEAP32[((14072 + 12|0))>>2] = $656;
    $657 = $656 | 1;
    $$sum$i14$i = (($655) + 4)|0;
    $658 = (($tbase$247$i) + ($$sum$i14$i)|0);
    HEAP32[$658>>2] = $657;
    $$sum2$i$i = (($tsize$246$i) + -36)|0;
    $659 = (($tbase$247$i) + ($$sum2$i$i)|0);
    HEAP32[$659>>2] = 40;
    $660 = HEAP32[((14544 + 16|0))>>2]|0;
    HEAP32[((14072 + 28|0))>>2] = $660;
   } else {
    $sp$075$i = ((14072 + 448|0));
    while(1) {
     $661 = HEAP32[$sp$075$i>>2]|0;
     $662 = (($sp$075$i) + 4|0);
     $663 = HEAP32[$662>>2]|0;
     $664 = (($661) + ($663)|0);
     $665 = ($tbase$247$i|0)==($664|0);
     if ($665) {
      label = 214;
      break;
     }
     $666 = (($sp$075$i) + 8|0);
     $667 = HEAP32[$666>>2]|0;
     $668 = ($667|0)==(0|0);
     if ($668) {
      break;
     } else {
      $sp$075$i = $667;
     }
    }
    if ((label|0) == 214) {
     $669 = (($sp$075$i) + 12|0);
     $670 = HEAP32[$669>>2]|0;
     $671 = $670 & 8;
     $672 = ($671|0)==(0);
     if ($672) {
      $673 = ($636>>>0)>=($661>>>0);
      $674 = ($636>>>0)<($tbase$247$i>>>0);
      $or$cond49$i = $673 & $674;
      if ($or$cond49$i) {
       $675 = (($663) + ($tsize$246$i))|0;
       HEAP32[$662>>2] = $675;
       $676 = HEAP32[((14072 + 12|0))>>2]|0;
       $677 = (($676) + ($tsize$246$i))|0;
       $678 = (($636) + 8|0);
       $679 = $678;
       $680 = $679 & 7;
       $681 = ($680|0)==(0);
       if ($681) {
        $685 = 0;
       } else {
        $682 = (0 - ($679))|0;
        $683 = $682 & 7;
        $685 = $683;
       }
       $684 = (($636) + ($685)|0);
       $686 = (($677) - ($685))|0;
       HEAP32[((14072 + 24|0))>>2] = $684;
       HEAP32[((14072 + 12|0))>>2] = $686;
       $687 = $686 | 1;
       $$sum$i18$i = (($685) + 4)|0;
       $688 = (($636) + ($$sum$i18$i)|0);
       HEAP32[$688>>2] = $687;
       $$sum2$i19$i = (($677) + 4)|0;
       $689 = (($636) + ($$sum2$i19$i)|0);
       HEAP32[$689>>2] = 40;
       $690 = HEAP32[((14544 + 16|0))>>2]|0;
       HEAP32[((14072 + 28|0))>>2] = $690;
       break;
      }
     }
    }
    $691 = HEAP32[((14072 + 16|0))>>2]|0;
    $692 = ($tbase$247$i>>>0)<($691>>>0);
    if ($692) {
     HEAP32[((14072 + 16|0))>>2] = $tbase$247$i;
    }
    $693 = (($tbase$247$i) + ($tsize$246$i)|0);
    $sp$168$i = ((14072 + 448|0));
    while(1) {
     $694 = HEAP32[$sp$168$i>>2]|0;
     $695 = ($694|0)==($693|0);
     if ($695) {
      label = 224;
      break;
     }
     $696 = (($sp$168$i) + 8|0);
     $697 = HEAP32[$696>>2]|0;
     $698 = ($697|0)==(0|0);
     if ($698) {
      break;
     } else {
      $sp$168$i = $697;
     }
    }
    if ((label|0) == 224) {
     $699 = (($sp$168$i) + 12|0);
     $700 = HEAP32[$699>>2]|0;
     $701 = $700 & 8;
     $702 = ($701|0)==(0);
     if ($702) {
      HEAP32[$sp$168$i>>2] = $tbase$247$i;
      $703 = (($sp$168$i) + 4|0);
      $704 = HEAP32[$703>>2]|0;
      $705 = (($704) + ($tsize$246$i))|0;
      HEAP32[$703>>2] = $705;
      $706 = (($tbase$247$i) + 8|0);
      $707 = $706;
      $708 = $707 & 7;
      $709 = ($708|0)==(0);
      if ($709) {
       $713 = 0;
      } else {
       $710 = (0 - ($707))|0;
       $711 = $710 & 7;
       $713 = $711;
      }
      $712 = (($tbase$247$i) + ($713)|0);
      $$sum107$i = (($tsize$246$i) + 8)|0;
      $714 = (($tbase$247$i) + ($$sum107$i)|0);
      $715 = $714;
      $716 = $715 & 7;
      $717 = ($716|0)==(0);
      if ($717) {
       $720 = 0;
      } else {
       $718 = (0 - ($715))|0;
       $719 = $718 & 7;
       $720 = $719;
      }
      $$sum108$i = (($720) + ($tsize$246$i))|0;
      $721 = (($tbase$247$i) + ($$sum108$i)|0);
      $722 = $721;
      $723 = $712;
      $724 = (($722) - ($723))|0;
      $$sum$i21$i = (($713) + ($nb$0))|0;
      $725 = (($tbase$247$i) + ($$sum$i21$i)|0);
      $726 = (($724) - ($nb$0))|0;
      $727 = $nb$0 | 3;
      $$sum1$i22$i = (($713) + 4)|0;
      $728 = (($tbase$247$i) + ($$sum1$i22$i)|0);
      HEAP32[$728>>2] = $727;
      $729 = HEAP32[((14072 + 24|0))>>2]|0;
      $730 = ($721|0)==($729|0);
      L338: do {
       if ($730) {
        $731 = HEAP32[((14072 + 12|0))>>2]|0;
        $732 = (($731) + ($726))|0;
        HEAP32[((14072 + 12|0))>>2] = $732;
        HEAP32[((14072 + 24|0))>>2] = $725;
        $733 = $732 | 1;
        $$sum42$i$i = (($$sum$i21$i) + 4)|0;
        $734 = (($tbase$247$i) + ($$sum42$i$i)|0);
        HEAP32[$734>>2] = $733;
       } else {
        $735 = HEAP32[((14072 + 20|0))>>2]|0;
        $736 = ($721|0)==($735|0);
        if ($736) {
         $737 = HEAP32[((14072 + 8|0))>>2]|0;
         $738 = (($737) + ($726))|0;
         HEAP32[((14072 + 8|0))>>2] = $738;
         HEAP32[((14072 + 20|0))>>2] = $725;
         $739 = $738 | 1;
         $$sum40$i$i = (($$sum$i21$i) + 4)|0;
         $740 = (($tbase$247$i) + ($$sum40$i$i)|0);
         HEAP32[$740>>2] = $739;
         $$sum41$i$i = (($738) + ($$sum$i21$i))|0;
         $741 = (($tbase$247$i) + ($$sum41$i$i)|0);
         HEAP32[$741>>2] = $738;
         break;
        }
        $$sum2$i23$i = (($tsize$246$i) + 4)|0;
        $$sum109$i = (($$sum2$i23$i) + ($720))|0;
        $742 = (($tbase$247$i) + ($$sum109$i)|0);
        $743 = HEAP32[$742>>2]|0;
        $744 = $743 & 3;
        $745 = ($744|0)==(1);
        if ($745) {
         $746 = $743 & -8;
         $747 = $743 >>> 3;
         $748 = ($743>>>0)<(256);
         L346: do {
          if ($748) {
           $$sum3738$i$i = $720 | 8;
           $$sum119$i = (($$sum3738$i$i) + ($tsize$246$i))|0;
           $749 = (($tbase$247$i) + ($$sum119$i)|0);
           $750 = HEAP32[$749>>2]|0;
           $$sum39$i$i = (($tsize$246$i) + 12)|0;
           $$sum120$i = (($$sum39$i$i) + ($720))|0;
           $751 = (($tbase$247$i) + ($$sum120$i)|0);
           $752 = HEAP32[$751>>2]|0;
           $753 = $747 << 1;
           $754 = ((14072 + ($753<<2)|0) + 40|0);
           $755 = ($750|0)==($754|0);
           do {
            if (!($755)) {
             $756 = HEAP32[((14072 + 16|0))>>2]|0;
             $757 = ($750>>>0)<($756>>>0);
             if ($757) {
              _abort();
              // unreachable;
             }
             $758 = (($750) + 12|0);
             $759 = HEAP32[$758>>2]|0;
             $760 = ($759|0)==($721|0);
             if ($760) {
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $761 = ($752|0)==($750|0);
           if ($761) {
            $762 = 1 << $747;
            $763 = $762 ^ -1;
            $764 = HEAP32[14072>>2]|0;
            $765 = $764 & $763;
            HEAP32[14072>>2] = $765;
            break;
           }
           $766 = ($752|0)==($754|0);
           do {
            if ($766) {
             $$pre57$i$i = (($752) + 8|0);
             $$pre$phi58$i$iZ2D = $$pre57$i$i;
            } else {
             $767 = HEAP32[((14072 + 16|0))>>2]|0;
             $768 = ($752>>>0)<($767>>>0);
             if ($768) {
              _abort();
              // unreachable;
             }
             $769 = (($752) + 8|0);
             $770 = HEAP32[$769>>2]|0;
             $771 = ($770|0)==($721|0);
             if ($771) {
              $$pre$phi58$i$iZ2D = $769;
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $772 = (($750) + 12|0);
           HEAP32[$772>>2] = $752;
           HEAP32[$$pre$phi58$i$iZ2D>>2] = $750;
          } else {
           $$sum34$i$i = $720 | 24;
           $$sum110$i = (($$sum34$i$i) + ($tsize$246$i))|0;
           $773 = (($tbase$247$i) + ($$sum110$i)|0);
           $774 = HEAP32[$773>>2]|0;
           $$sum5$i$i = (($tsize$246$i) + 12)|0;
           $$sum111$i = (($$sum5$i$i) + ($720))|0;
           $775 = (($tbase$247$i) + ($$sum111$i)|0);
           $776 = HEAP32[$775>>2]|0;
           $777 = ($776|0)==($721|0);
           do {
            if ($777) {
             $$sum67$i$i = $720 | 16;
             $$sum117$i = (($$sum2$i23$i) + ($$sum67$i$i))|0;
             $788 = (($tbase$247$i) + ($$sum117$i)|0);
             $789 = HEAP32[$788>>2]|0;
             $790 = ($789|0)==(0|0);
             if ($790) {
              $$sum118$i = (($$sum67$i$i) + ($tsize$246$i))|0;
              $791 = (($tbase$247$i) + ($$sum118$i)|0);
              $792 = HEAP32[$791>>2]|0;
              $793 = ($792|0)==(0|0);
              if ($793) {
               $R$1$i$i = 0;
               break;
              } else {
               $R$0$i$i = $792;$RP$0$i$i = $791;
              }
             } else {
              $R$0$i$i = $789;$RP$0$i$i = $788;
             }
             while(1) {
              $794 = (($R$0$i$i) + 20|0);
              $795 = HEAP32[$794>>2]|0;
              $796 = ($795|0)==(0|0);
              if (!($796)) {
               $R$0$i$i = $795;$RP$0$i$i = $794;
               continue;
              }
              $797 = (($R$0$i$i) + 16|0);
              $798 = HEAP32[$797>>2]|0;
              $799 = ($798|0)==(0|0);
              if ($799) {
               break;
              } else {
               $R$0$i$i = $798;$RP$0$i$i = $797;
              }
             }
             $800 = HEAP32[((14072 + 16|0))>>2]|0;
             $801 = ($RP$0$i$i>>>0)<($800>>>0);
             if ($801) {
              _abort();
              // unreachable;
             } else {
              HEAP32[$RP$0$i$i>>2] = 0;
              $R$1$i$i = $R$0$i$i;
              break;
             }
            } else {
             $$sum3536$i$i = $720 | 8;
             $$sum112$i = (($$sum3536$i$i) + ($tsize$246$i))|0;
             $778 = (($tbase$247$i) + ($$sum112$i)|0);
             $779 = HEAP32[$778>>2]|0;
             $780 = HEAP32[((14072 + 16|0))>>2]|0;
             $781 = ($779>>>0)<($780>>>0);
             if ($781) {
              _abort();
              // unreachable;
             }
             $782 = (($779) + 12|0);
             $783 = HEAP32[$782>>2]|0;
             $784 = ($783|0)==($721|0);
             if (!($784)) {
              _abort();
              // unreachable;
             }
             $785 = (($776) + 8|0);
             $786 = HEAP32[$785>>2]|0;
             $787 = ($786|0)==($721|0);
             if ($787) {
              HEAP32[$782>>2] = $776;
              HEAP32[$785>>2] = $779;
              $R$1$i$i = $776;
              break;
             } else {
              _abort();
              // unreachable;
             }
            }
           } while(0);
           $802 = ($774|0)==(0|0);
           if ($802) {
            break;
           }
           $$sum30$i$i = (($tsize$246$i) + 28)|0;
           $$sum113$i = (($$sum30$i$i) + ($720))|0;
           $803 = (($tbase$247$i) + ($$sum113$i)|0);
           $804 = HEAP32[$803>>2]|0;
           $805 = ((14072 + ($804<<2)|0) + 304|0);
           $806 = HEAP32[$805>>2]|0;
           $807 = ($721|0)==($806|0);
           do {
            if ($807) {
             HEAP32[$805>>2] = $R$1$i$i;
             $cond$i$i = ($R$1$i$i|0)==(0|0);
             if (!($cond$i$i)) {
              break;
             }
             $808 = 1 << $804;
             $809 = $808 ^ -1;
             $810 = HEAP32[((14072 + 4|0))>>2]|0;
             $811 = $810 & $809;
             HEAP32[((14072 + 4|0))>>2] = $811;
             break L346;
            } else {
             $812 = HEAP32[((14072 + 16|0))>>2]|0;
             $813 = ($774>>>0)<($812>>>0);
             if ($813) {
              _abort();
              // unreachable;
             }
             $814 = (($774) + 16|0);
             $815 = HEAP32[$814>>2]|0;
             $816 = ($815|0)==($721|0);
             if ($816) {
              HEAP32[$814>>2] = $R$1$i$i;
             } else {
              $817 = (($774) + 20|0);
              HEAP32[$817>>2] = $R$1$i$i;
             }
             $818 = ($R$1$i$i|0)==(0|0);
             if ($818) {
              break L346;
             }
            }
           } while(0);
           $819 = HEAP32[((14072 + 16|0))>>2]|0;
           $820 = ($R$1$i$i>>>0)<($819>>>0);
           if ($820) {
            _abort();
            // unreachable;
           }
           $821 = (($R$1$i$i) + 24|0);
           HEAP32[$821>>2] = $774;
           $$sum3132$i$i = $720 | 16;
           $$sum114$i = (($$sum3132$i$i) + ($tsize$246$i))|0;
           $822 = (($tbase$247$i) + ($$sum114$i)|0);
           $823 = HEAP32[$822>>2]|0;
           $824 = ($823|0)==(0|0);
           do {
            if (!($824)) {
             $825 = HEAP32[((14072 + 16|0))>>2]|0;
             $826 = ($823>>>0)<($825>>>0);
             if ($826) {
              _abort();
              // unreachable;
             } else {
              $827 = (($R$1$i$i) + 16|0);
              HEAP32[$827>>2] = $823;
              $828 = (($823) + 24|0);
              HEAP32[$828>>2] = $R$1$i$i;
              break;
             }
            }
           } while(0);
           $$sum115$i = (($$sum2$i23$i) + ($$sum3132$i$i))|0;
           $829 = (($tbase$247$i) + ($$sum115$i)|0);
           $830 = HEAP32[$829>>2]|0;
           $831 = ($830|0)==(0|0);
           if ($831) {
            break;
           }
           $832 = HEAP32[((14072 + 16|0))>>2]|0;
           $833 = ($830>>>0)<($832>>>0);
           if ($833) {
            _abort();
            // unreachable;
           } else {
            $834 = (($R$1$i$i) + 20|0);
            HEAP32[$834>>2] = $830;
            $835 = (($830) + 24|0);
            HEAP32[$835>>2] = $R$1$i$i;
            break;
           }
          }
         } while(0);
         $$sum9$i$i = $746 | $720;
         $$sum116$i = (($$sum9$i$i) + ($tsize$246$i))|0;
         $836 = (($tbase$247$i) + ($$sum116$i)|0);
         $837 = (($746) + ($726))|0;
         $oldfirst$0$i$i = $836;$qsize$0$i$i = $837;
        } else {
         $oldfirst$0$i$i = $721;$qsize$0$i$i = $726;
        }
        $838 = (($oldfirst$0$i$i) + 4|0);
        $839 = HEAP32[$838>>2]|0;
        $840 = $839 & -2;
        HEAP32[$838>>2] = $840;
        $841 = $qsize$0$i$i | 1;
        $$sum10$i$i = (($$sum$i21$i) + 4)|0;
        $842 = (($tbase$247$i) + ($$sum10$i$i)|0);
        HEAP32[$842>>2] = $841;
        $$sum11$i24$i = (($qsize$0$i$i) + ($$sum$i21$i))|0;
        $843 = (($tbase$247$i) + ($$sum11$i24$i)|0);
        HEAP32[$843>>2] = $qsize$0$i$i;
        $844 = $qsize$0$i$i >>> 3;
        $845 = ($qsize$0$i$i>>>0)<(256);
        if ($845) {
         $846 = $844 << 1;
         $847 = ((14072 + ($846<<2)|0) + 40|0);
         $848 = HEAP32[14072>>2]|0;
         $849 = 1 << $844;
         $850 = $848 & $849;
         $851 = ($850|0)==(0);
         do {
          if ($851) {
           $852 = $848 | $849;
           HEAP32[14072>>2] = $852;
           $$sum26$pre$i$i = (($846) + 2)|0;
           $$pre$i25$i = ((14072 + ($$sum26$pre$i$i<<2)|0) + 40|0);
           $$pre$phi$i26$iZ2D = $$pre$i25$i;$F4$0$i$i = $847;
          } else {
           $$sum29$i$i = (($846) + 2)|0;
           $853 = ((14072 + ($$sum29$i$i<<2)|0) + 40|0);
           $854 = HEAP32[$853>>2]|0;
           $855 = HEAP32[((14072 + 16|0))>>2]|0;
           $856 = ($854>>>0)<($855>>>0);
           if (!($856)) {
            $$pre$phi$i26$iZ2D = $853;$F4$0$i$i = $854;
            break;
           }
           _abort();
           // unreachable;
          }
         } while(0);
         HEAP32[$$pre$phi$i26$iZ2D>>2] = $725;
         $857 = (($F4$0$i$i) + 12|0);
         HEAP32[$857>>2] = $725;
         $$sum27$i$i = (($$sum$i21$i) + 8)|0;
         $858 = (($tbase$247$i) + ($$sum27$i$i)|0);
         HEAP32[$858>>2] = $F4$0$i$i;
         $$sum28$i$i = (($$sum$i21$i) + 12)|0;
         $859 = (($tbase$247$i) + ($$sum28$i$i)|0);
         HEAP32[$859>>2] = $847;
         break;
        }
        $860 = $qsize$0$i$i >>> 8;
        $861 = ($860|0)==(0);
        do {
         if ($861) {
          $I7$0$i$i = 0;
         } else {
          $862 = ($qsize$0$i$i>>>0)>(16777215);
          if ($862) {
           $I7$0$i$i = 31;
           break;
          }
          $863 = (($860) + 1048320)|0;
          $864 = $863 >>> 16;
          $865 = $864 & 8;
          $866 = $860 << $865;
          $867 = (($866) + 520192)|0;
          $868 = $867 >>> 16;
          $869 = $868 & 4;
          $870 = $869 | $865;
          $871 = $866 << $869;
          $872 = (($871) + 245760)|0;
          $873 = $872 >>> 16;
          $874 = $873 & 2;
          $875 = $870 | $874;
          $876 = (14 - ($875))|0;
          $877 = $871 << $874;
          $878 = $877 >>> 15;
          $879 = (($876) + ($878))|0;
          $880 = $879 << 1;
          $881 = (($879) + 7)|0;
          $882 = $qsize$0$i$i >>> $881;
          $883 = $882 & 1;
          $884 = $883 | $880;
          $I7$0$i$i = $884;
         }
        } while(0);
        $885 = ((14072 + ($I7$0$i$i<<2)|0) + 304|0);
        $$sum12$i$i = (($$sum$i21$i) + 28)|0;
        $886 = (($tbase$247$i) + ($$sum12$i$i)|0);
        HEAP32[$886>>2] = $I7$0$i$i;
        $$sum13$i$i = (($$sum$i21$i) + 16)|0;
        $887 = (($tbase$247$i) + ($$sum13$i$i)|0);
        $$sum14$i$i = (($$sum$i21$i) + 20)|0;
        $888 = (($tbase$247$i) + ($$sum14$i$i)|0);
        HEAP32[$888>>2] = 0;
        HEAP32[$887>>2] = 0;
        $889 = HEAP32[((14072 + 4|0))>>2]|0;
        $890 = 1 << $I7$0$i$i;
        $891 = $889 & $890;
        $892 = ($891|0)==(0);
        if ($892) {
         $893 = $889 | $890;
         HEAP32[((14072 + 4|0))>>2] = $893;
         HEAP32[$885>>2] = $725;
         $$sum15$i$i = (($$sum$i21$i) + 24)|0;
         $894 = (($tbase$247$i) + ($$sum15$i$i)|0);
         HEAP32[$894>>2] = $885;
         $$sum16$i$i = (($$sum$i21$i) + 12)|0;
         $895 = (($tbase$247$i) + ($$sum16$i$i)|0);
         HEAP32[$895>>2] = $725;
         $$sum17$i$i = (($$sum$i21$i) + 8)|0;
         $896 = (($tbase$247$i) + ($$sum17$i$i)|0);
         HEAP32[$896>>2] = $725;
         break;
        }
        $897 = HEAP32[$885>>2]|0;
        $898 = ($I7$0$i$i|0)==(31);
        if ($898) {
         $906 = 0;
        } else {
         $899 = $I7$0$i$i >>> 1;
         $900 = (25 - ($899))|0;
         $906 = $900;
        }
        $901 = (($897) + 4|0);
        $902 = HEAP32[$901>>2]|0;
        $903 = $902 & -8;
        $904 = ($903|0)==($qsize$0$i$i|0);
        L435: do {
         if ($904) {
          $T$0$lcssa$i28$i = $897;
         } else {
          $905 = $qsize$0$i$i << $906;
          $K8$052$i$i = $905;$T$051$i$i = $897;
          while(1) {
           $913 = $K8$052$i$i >>> 31;
           $914 = ((($T$051$i$i) + ($913<<2)|0) + 16|0);
           $909 = HEAP32[$914>>2]|0;
           $915 = ($909|0)==(0|0);
           if ($915) {
            break;
           }
           $907 = $K8$052$i$i << 1;
           $908 = (($909) + 4|0);
           $910 = HEAP32[$908>>2]|0;
           $911 = $910 & -8;
           $912 = ($911|0)==($qsize$0$i$i|0);
           if ($912) {
            $T$0$lcssa$i28$i = $909;
            break L435;
           } else {
            $K8$052$i$i = $907;$T$051$i$i = $909;
           }
          }
          $916 = HEAP32[((14072 + 16|0))>>2]|0;
          $917 = ($914>>>0)<($916>>>0);
          if ($917) {
           _abort();
           // unreachable;
          } else {
           HEAP32[$914>>2] = $725;
           $$sum23$i$i = (($$sum$i21$i) + 24)|0;
           $918 = (($tbase$247$i) + ($$sum23$i$i)|0);
           HEAP32[$918>>2] = $T$051$i$i;
           $$sum24$i$i = (($$sum$i21$i) + 12)|0;
           $919 = (($tbase$247$i) + ($$sum24$i$i)|0);
           HEAP32[$919>>2] = $725;
           $$sum25$i$i = (($$sum$i21$i) + 8)|0;
           $920 = (($tbase$247$i) + ($$sum25$i$i)|0);
           HEAP32[$920>>2] = $725;
           break L338;
          }
         }
        } while(0);
        $921 = (($T$0$lcssa$i28$i) + 8|0);
        $922 = HEAP32[$921>>2]|0;
        $923 = HEAP32[((14072 + 16|0))>>2]|0;
        $924 = ($T$0$lcssa$i28$i>>>0)<($923>>>0);
        if ($924) {
         _abort();
         // unreachable;
        }
        $925 = ($922>>>0)<($923>>>0);
        if ($925) {
         _abort();
         // unreachable;
        } else {
         $926 = (($922) + 12|0);
         HEAP32[$926>>2] = $725;
         HEAP32[$921>>2] = $725;
         $$sum20$i$i = (($$sum$i21$i) + 8)|0;
         $927 = (($tbase$247$i) + ($$sum20$i$i)|0);
         HEAP32[$927>>2] = $922;
         $$sum21$i$i = (($$sum$i21$i) + 12)|0;
         $928 = (($tbase$247$i) + ($$sum21$i$i)|0);
         HEAP32[$928>>2] = $T$0$lcssa$i28$i;
         $$sum22$i$i = (($$sum$i21$i) + 24)|0;
         $929 = (($tbase$247$i) + ($$sum22$i$i)|0);
         HEAP32[$929>>2] = 0;
         break;
        }
       }
      } while(0);
      $$sum1819$i$i = $713 | 8;
      $930 = (($tbase$247$i) + ($$sum1819$i$i)|0);
      $mem$0 = $930;
      STACKTOP = sp;return ($mem$0|0);
     }
    }
    $sp$0$i$i$i = ((14072 + 448|0));
    while(1) {
     $931 = HEAP32[$sp$0$i$i$i>>2]|0;
     $932 = ($931>>>0)>($636>>>0);
     if (!($932)) {
      $933 = (($sp$0$i$i$i) + 4|0);
      $934 = HEAP32[$933>>2]|0;
      $935 = (($931) + ($934)|0);
      $936 = ($935>>>0)>($636>>>0);
      if ($936) {
       break;
      }
     }
     $937 = (($sp$0$i$i$i) + 8|0);
     $938 = HEAP32[$937>>2]|0;
     $sp$0$i$i$i = $938;
    }
    $$sum$i15$i = (($934) + -47)|0;
    $$sum1$i16$i = (($934) + -39)|0;
    $939 = (($931) + ($$sum1$i16$i)|0);
    $940 = $939;
    $941 = $940 & 7;
    $942 = ($941|0)==(0);
    if ($942) {
     $945 = 0;
    } else {
     $943 = (0 - ($940))|0;
     $944 = $943 & 7;
     $945 = $944;
    }
    $$sum2$i17$i = (($$sum$i15$i) + ($945))|0;
    $946 = (($931) + ($$sum2$i17$i)|0);
    $947 = (($636) + 16|0);
    $948 = ($946>>>0)<($947>>>0);
    $949 = $948 ? $636 : $946;
    $950 = (($949) + 8|0);
    $951 = (($tsize$246$i) + -40)|0;
    $952 = (($tbase$247$i) + 8|0);
    $953 = $952;
    $954 = $953 & 7;
    $955 = ($954|0)==(0);
    if ($955) {
     $959 = 0;
    } else {
     $956 = (0 - ($953))|0;
     $957 = $956 & 7;
     $959 = $957;
    }
    $958 = (($tbase$247$i) + ($959)|0);
    $960 = (($951) - ($959))|0;
    HEAP32[((14072 + 24|0))>>2] = $958;
    HEAP32[((14072 + 12|0))>>2] = $960;
    $961 = $960 | 1;
    $$sum$i$i$i = (($959) + 4)|0;
    $962 = (($tbase$247$i) + ($$sum$i$i$i)|0);
    HEAP32[$962>>2] = $961;
    $$sum2$i$i$i = (($tsize$246$i) + -36)|0;
    $963 = (($tbase$247$i) + ($$sum2$i$i$i)|0);
    HEAP32[$963>>2] = 40;
    $964 = HEAP32[((14544 + 16|0))>>2]|0;
    HEAP32[((14072 + 28|0))>>2] = $964;
    $965 = (($949) + 4|0);
    HEAP32[$965>>2] = 27;
    ;HEAP32[$950+0>>2]=HEAP32[((14072 + 448|0))+0>>2]|0;HEAP32[$950+4>>2]=HEAP32[((14072 + 448|0))+4>>2]|0;HEAP32[$950+8>>2]=HEAP32[((14072 + 448|0))+8>>2]|0;HEAP32[$950+12>>2]=HEAP32[((14072 + 448|0))+12>>2]|0;
    HEAP32[((14072 + 448|0))>>2] = $tbase$247$i;
    HEAP32[((14072 + 452|0))>>2] = $tsize$246$i;
    HEAP32[((14072 + 460|0))>>2] = 0;
    HEAP32[((14072 + 456|0))>>2] = $950;
    $966 = (($949) + 28|0);
    HEAP32[$966>>2] = 7;
    $967 = (($949) + 32|0);
    $968 = ($967>>>0)<($935>>>0);
    if ($968) {
     $970 = $966;
     while(1) {
      $969 = (($970) + 4|0);
      HEAP32[$969>>2] = 7;
      $971 = (($970) + 8|0);
      $972 = ($971>>>0)<($935>>>0);
      if ($972) {
       $970 = $969;
      } else {
       break;
      }
     }
    }
    $973 = ($949|0)==($636|0);
    if (!($973)) {
     $974 = $949;
     $975 = $636;
     $976 = (($974) - ($975))|0;
     $977 = (($636) + ($976)|0);
     $$sum3$i$i = (($976) + 4)|0;
     $978 = (($636) + ($$sum3$i$i)|0);
     $979 = HEAP32[$978>>2]|0;
     $980 = $979 & -2;
     HEAP32[$978>>2] = $980;
     $981 = $976 | 1;
     $982 = (($636) + 4|0);
     HEAP32[$982>>2] = $981;
     HEAP32[$977>>2] = $976;
     $983 = $976 >>> 3;
     $984 = ($976>>>0)<(256);
     if ($984) {
      $985 = $983 << 1;
      $986 = ((14072 + ($985<<2)|0) + 40|0);
      $987 = HEAP32[14072>>2]|0;
      $988 = 1 << $983;
      $989 = $987 & $988;
      $990 = ($989|0)==(0);
      do {
       if ($990) {
        $991 = $987 | $988;
        HEAP32[14072>>2] = $991;
        $$sum10$pre$i$i = (($985) + 2)|0;
        $$pre$i$i = ((14072 + ($$sum10$pre$i$i<<2)|0) + 40|0);
        $$pre$phi$i$iZ2D = $$pre$i$i;$F$0$i$i = $986;
       } else {
        $$sum11$i$i = (($985) + 2)|0;
        $992 = ((14072 + ($$sum11$i$i<<2)|0) + 40|0);
        $993 = HEAP32[$992>>2]|0;
        $994 = HEAP32[((14072 + 16|0))>>2]|0;
        $995 = ($993>>>0)<($994>>>0);
        if (!($995)) {
         $$pre$phi$i$iZ2D = $992;$F$0$i$i = $993;
         break;
        }
        _abort();
        // unreachable;
       }
      } while(0);
      HEAP32[$$pre$phi$i$iZ2D>>2] = $636;
      $996 = (($F$0$i$i) + 12|0);
      HEAP32[$996>>2] = $636;
      $997 = (($636) + 8|0);
      HEAP32[$997>>2] = $F$0$i$i;
      $998 = (($636) + 12|0);
      HEAP32[$998>>2] = $986;
      break;
     }
     $999 = $976 >>> 8;
     $1000 = ($999|0)==(0);
     if ($1000) {
      $I1$0$i$i = 0;
     } else {
      $1001 = ($976>>>0)>(16777215);
      if ($1001) {
       $I1$0$i$i = 31;
      } else {
       $1002 = (($999) + 1048320)|0;
       $1003 = $1002 >>> 16;
       $1004 = $1003 & 8;
       $1005 = $999 << $1004;
       $1006 = (($1005) + 520192)|0;
       $1007 = $1006 >>> 16;
       $1008 = $1007 & 4;
       $1009 = $1008 | $1004;
       $1010 = $1005 << $1008;
       $1011 = (($1010) + 245760)|0;
       $1012 = $1011 >>> 16;
       $1013 = $1012 & 2;
       $1014 = $1009 | $1013;
       $1015 = (14 - ($1014))|0;
       $1016 = $1010 << $1013;
       $1017 = $1016 >>> 15;
       $1018 = (($1015) + ($1017))|0;
       $1019 = $1018 << 1;
       $1020 = (($1018) + 7)|0;
       $1021 = $976 >>> $1020;
       $1022 = $1021 & 1;
       $1023 = $1022 | $1019;
       $I1$0$i$i = $1023;
      }
     }
     $1024 = ((14072 + ($I1$0$i$i<<2)|0) + 304|0);
     $1025 = (($636) + 28|0);
     $I1$0$c$i$i = $I1$0$i$i;
     HEAP32[$1025>>2] = $I1$0$c$i$i;
     $1026 = (($636) + 20|0);
     HEAP32[$1026>>2] = 0;
     $1027 = (($636) + 16|0);
     HEAP32[$1027>>2] = 0;
     $1028 = HEAP32[((14072 + 4|0))>>2]|0;
     $1029 = 1 << $I1$0$i$i;
     $1030 = $1028 & $1029;
     $1031 = ($1030|0)==(0);
     if ($1031) {
      $1032 = $1028 | $1029;
      HEAP32[((14072 + 4|0))>>2] = $1032;
      HEAP32[$1024>>2] = $636;
      $1033 = (($636) + 24|0);
      HEAP32[$1033>>2] = $1024;
      $1034 = (($636) + 12|0);
      HEAP32[$1034>>2] = $636;
      $1035 = (($636) + 8|0);
      HEAP32[$1035>>2] = $636;
      break;
     }
     $1036 = HEAP32[$1024>>2]|0;
     $1037 = ($I1$0$i$i|0)==(31);
     if ($1037) {
      $1045 = 0;
     } else {
      $1038 = $I1$0$i$i >>> 1;
      $1039 = (25 - ($1038))|0;
      $1045 = $1039;
     }
     $1040 = (($1036) + 4|0);
     $1041 = HEAP32[$1040>>2]|0;
     $1042 = $1041 & -8;
     $1043 = ($1042|0)==($976|0);
     L489: do {
      if ($1043) {
       $T$0$lcssa$i$i = $1036;
      } else {
       $1044 = $976 << $1045;
       $K2$014$i$i = $1044;$T$013$i$i = $1036;
       while(1) {
        $1052 = $K2$014$i$i >>> 31;
        $1053 = ((($T$013$i$i) + ($1052<<2)|0) + 16|0);
        $1048 = HEAP32[$1053>>2]|0;
        $1054 = ($1048|0)==(0|0);
        if ($1054) {
         break;
        }
        $1046 = $K2$014$i$i << 1;
        $1047 = (($1048) + 4|0);
        $1049 = HEAP32[$1047>>2]|0;
        $1050 = $1049 & -8;
        $1051 = ($1050|0)==($976|0);
        if ($1051) {
         $T$0$lcssa$i$i = $1048;
         break L489;
        } else {
         $K2$014$i$i = $1046;$T$013$i$i = $1048;
        }
       }
       $1055 = HEAP32[((14072 + 16|0))>>2]|0;
       $1056 = ($1053>>>0)<($1055>>>0);
       if ($1056) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$1053>>2] = $636;
        $1057 = (($636) + 24|0);
        HEAP32[$1057>>2] = $T$013$i$i;
        $1058 = (($636) + 12|0);
        HEAP32[$1058>>2] = $636;
        $1059 = (($636) + 8|0);
        HEAP32[$1059>>2] = $636;
        break L311;
       }
      }
     } while(0);
     $1060 = (($T$0$lcssa$i$i) + 8|0);
     $1061 = HEAP32[$1060>>2]|0;
     $1062 = HEAP32[((14072 + 16|0))>>2]|0;
     $1063 = ($T$0$lcssa$i$i>>>0)<($1062>>>0);
     if ($1063) {
      _abort();
      // unreachable;
     }
     $1064 = ($1061>>>0)<($1062>>>0);
     if ($1064) {
      _abort();
      // unreachable;
     } else {
      $1065 = (($1061) + 12|0);
      HEAP32[$1065>>2] = $636;
      HEAP32[$1060>>2] = $636;
      $1066 = (($636) + 8|0);
      HEAP32[$1066>>2] = $1061;
      $1067 = (($636) + 12|0);
      HEAP32[$1067>>2] = $T$0$lcssa$i$i;
      $1068 = (($636) + 24|0);
      HEAP32[$1068>>2] = 0;
      break;
     }
    }
   }
  } while(0);
  $1069 = HEAP32[((14072 + 12|0))>>2]|0;
  $1070 = ($1069>>>0)>($nb$0>>>0);
  if ($1070) {
   $1071 = (($1069) - ($nb$0))|0;
   HEAP32[((14072 + 12|0))>>2] = $1071;
   $1072 = HEAP32[((14072 + 24|0))>>2]|0;
   $1073 = (($1072) + ($nb$0)|0);
   HEAP32[((14072 + 24|0))>>2] = $1073;
   $1074 = $1071 | 1;
   $$sum$i32 = (($nb$0) + 4)|0;
   $1075 = (($1072) + ($$sum$i32)|0);
   HEAP32[$1075>>2] = $1074;
   $1076 = $nb$0 | 3;
   $1077 = (($1072) + 4|0);
   HEAP32[$1077>>2] = $1076;
   $1078 = (($1072) + 8|0);
   $mem$0 = $1078;
   STACKTOP = sp;return ($mem$0|0);
  }
 }
 $1079 = (___errno_location()|0);
 HEAP32[$1079>>2] = 12;
 $mem$0 = 0;
 STACKTOP = sp;return ($mem$0|0);
}
function _free($mem) {
 $mem = $mem|0;
 var $$pre = 0, $$pre$phi68Z2D = 0, $$pre$phi70Z2D = 0, $$pre$phiZ2D = 0, $$pre67 = 0, $$pre69 = 0, $$sum = 0, $$sum16$pre = 0, $$sum17 = 0, $$sum18 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum2324 = 0, $$sum25 = 0, $$sum26 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0;
 var $$sum31 = 0, $$sum32 = 0, $$sum33 = 0, $$sum34 = 0, $$sum35 = 0, $$sum36 = 0, $$sum37 = 0, $$sum5 = 0, $$sum67 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0;
 var $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0;
 var $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0;
 var $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0;
 var $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0;
 var $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0;
 var $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0;
 var $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0;
 var $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0;
 var $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0;
 var $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0;
 var $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0;
 var $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0;
 var $322 = 0, $323 = 0, $324 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I18$0 = 0, $I18$0$c = 0, $K19$057 = 0;
 var $R$0 = 0, $R$1 = 0, $R7$0 = 0, $R7$1 = 0, $RP$0 = 0, $RP9$0 = 0, $T$0$lcssa = 0, $T$056 = 0, $cond = 0, $cond54 = 0, $p$0 = 0, $psize$0 = 0, $psize$1 = 0, $sp$0$i = 0, $sp$0$in$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($mem|0)==(0|0);
 if ($0) {
  STACKTOP = sp;return;
 }
 $1 = (($mem) + -8|0);
 $2 = HEAP32[((14072 + 16|0))>>2]|0;
 $3 = ($1>>>0)<($2>>>0);
 if ($3) {
  _abort();
  // unreachable;
 }
 $4 = (($mem) + -4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 3;
 $7 = ($6|0)==(1);
 if ($7) {
  _abort();
  // unreachable;
 }
 $8 = $5 & -8;
 $$sum = (($8) + -8)|0;
 $9 = (($mem) + ($$sum)|0);
 $10 = $5 & 1;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ($6|0)==(0);
   if ($13) {
    STACKTOP = sp;return;
   }
   $$sum2 = (-8 - ($12))|0;
   $14 = (($mem) + ($$sum2)|0);
   $15 = (($12) + ($8))|0;
   $16 = ($14>>>0)<($2>>>0);
   if ($16) {
    _abort();
    // unreachable;
   }
   $17 = HEAP32[((14072 + 20|0))>>2]|0;
   $18 = ($14|0)==($17|0);
   if ($18) {
    $$sum3 = (($8) + -4)|0;
    $104 = (($mem) + ($$sum3)|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = $105 & 3;
    $107 = ($106|0)==(3);
    if (!($107)) {
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    HEAP32[((14072 + 8|0))>>2] = $15;
    $108 = HEAP32[$104>>2]|0;
    $109 = $108 & -2;
    HEAP32[$104>>2] = $109;
    $110 = $15 | 1;
    $$sum26 = (($$sum2) + 4)|0;
    $111 = (($mem) + ($$sum26)|0);
    HEAP32[$111>>2] = $110;
    HEAP32[$9>>2] = $15;
    STACKTOP = sp;return;
   }
   $19 = $12 >>> 3;
   $20 = ($12>>>0)<(256);
   if ($20) {
    $$sum36 = (($$sum2) + 8)|0;
    $21 = (($mem) + ($$sum36)|0);
    $22 = HEAP32[$21>>2]|0;
    $$sum37 = (($$sum2) + 12)|0;
    $23 = (($mem) + ($$sum37)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $19 << 1;
    $26 = ((14072 + ($25<<2)|0) + 40|0);
    $27 = ($22|0)==($26|0);
    if (!($27)) {
     $28 = ($22>>>0)<($2>>>0);
     if ($28) {
      _abort();
      // unreachable;
     }
     $29 = (($22) + 12|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($30|0)==($14|0);
     if (!($31)) {
      _abort();
      // unreachable;
     }
    }
    $32 = ($24|0)==($22|0);
    if ($32) {
     $33 = 1 << $19;
     $34 = $33 ^ -1;
     $35 = HEAP32[14072>>2]|0;
     $36 = $35 & $34;
     HEAP32[14072>>2] = $36;
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    $37 = ($24|0)==($26|0);
    if ($37) {
     $$pre69 = (($24) + 8|0);
     $$pre$phi70Z2D = $$pre69;
    } else {
     $38 = ($24>>>0)<($2>>>0);
     if ($38) {
      _abort();
      // unreachable;
     }
     $39 = (($24) + 8|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($40|0)==($14|0);
     if ($41) {
      $$pre$phi70Z2D = $39;
     } else {
      _abort();
      // unreachable;
     }
    }
    $42 = (($22) + 12|0);
    HEAP32[$42>>2] = $24;
    HEAP32[$$pre$phi70Z2D>>2] = $22;
    $p$0 = $14;$psize$0 = $15;
    break;
   }
   $$sum28 = (($$sum2) + 24)|0;
   $43 = (($mem) + ($$sum28)|0);
   $44 = HEAP32[$43>>2]|0;
   $$sum29 = (($$sum2) + 12)|0;
   $45 = (($mem) + ($$sum29)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==($14|0);
   do {
    if ($47) {
     $$sum31 = (($$sum2) + 20)|0;
     $57 = (($mem) + ($$sum31)|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      $$sum30 = (($$sum2) + 16)|0;
      $60 = (($mem) + ($$sum30)|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)==(0|0);
      if ($62) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $61;$RP$0 = $60;
      }
     } else {
      $R$0 = $58;$RP$0 = $57;
     }
     while(1) {
      $63 = (($R$0) + 20|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = ($64|0)==(0|0);
      if (!($65)) {
       $R$0 = $64;$RP$0 = $63;
       continue;
      }
      $66 = (($R$0) + 16|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = ($67|0)==(0|0);
      if ($68) {
       break;
      } else {
       $R$0 = $67;$RP$0 = $66;
      }
     }
     $69 = ($RP$0>>>0)<($2>>>0);
     if ($69) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum35 = (($$sum2) + 8)|0;
     $48 = (($mem) + ($$sum35)|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49>>>0)<($2>>>0);
     if ($50) {
      _abort();
      // unreachable;
     }
     $51 = (($49) + 12|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ($52|0)==($14|0);
     if (!($53)) {
      _abort();
      // unreachable;
     }
     $54 = (($46) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)==($14|0);
     if ($56) {
      HEAP32[$51>>2] = $46;
      HEAP32[$54>>2] = $49;
      $R$1 = $46;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $70 = ($44|0)==(0|0);
   if ($70) {
    $p$0 = $14;$psize$0 = $15;
   } else {
    $$sum32 = (($$sum2) + 28)|0;
    $71 = (($mem) + ($$sum32)|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = ((14072 + ($72<<2)|0) + 304|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($14|0)==($74|0);
    if ($75) {
     HEAP32[$73>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $76 = 1 << $72;
      $77 = $76 ^ -1;
      $78 = HEAP32[((14072 + 4|0))>>2]|0;
      $79 = $78 & $77;
      HEAP32[((14072 + 4|0))>>2] = $79;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    } else {
     $80 = HEAP32[((14072 + 16|0))>>2]|0;
     $81 = ($44>>>0)<($80>>>0);
     if ($81) {
      _abort();
      // unreachable;
     }
     $82 = (($44) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)==($14|0);
     if ($84) {
      HEAP32[$82>>2] = $R$1;
     } else {
      $85 = (($44) + 20|0);
      HEAP32[$85>>2] = $R$1;
     }
     $86 = ($R$1|0)==(0|0);
     if ($86) {
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
    $87 = HEAP32[((14072 + 16|0))>>2]|0;
    $88 = ($R$1>>>0)<($87>>>0);
    if ($88) {
     _abort();
     // unreachable;
    }
    $89 = (($R$1) + 24|0);
    HEAP32[$89>>2] = $44;
    $$sum33 = (($$sum2) + 16)|0;
    $90 = (($mem) + ($$sum33)|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    do {
     if (!($92)) {
      $93 = HEAP32[((14072 + 16|0))>>2]|0;
      $94 = ($91>>>0)<($93>>>0);
      if ($94) {
       _abort();
       // unreachable;
      } else {
       $95 = (($R$1) + 16|0);
       HEAP32[$95>>2] = $91;
       $96 = (($91) + 24|0);
       HEAP32[$96>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum34 = (($$sum2) + 20)|0;
    $97 = (($mem) + ($$sum34)|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)==(0|0);
    if ($99) {
     $p$0 = $14;$psize$0 = $15;
    } else {
     $100 = HEAP32[((14072 + 16|0))>>2]|0;
     $101 = ($98>>>0)<($100>>>0);
     if ($101) {
      _abort();
      // unreachable;
     } else {
      $102 = (($R$1) + 20|0);
      HEAP32[$102>>2] = $98;
      $103 = (($98) + 24|0);
      HEAP32[$103>>2] = $R$1;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
   }
  } else {
   $p$0 = $1;$psize$0 = $8;
  }
 } while(0);
 $112 = ($p$0>>>0)<($9>>>0);
 if (!($112)) {
  _abort();
  // unreachable;
 }
 $$sum25 = (($8) + -4)|0;
 $113 = (($mem) + ($$sum25)|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = $114 & 1;
 $116 = ($115|0)==(0);
 if ($116) {
  _abort();
  // unreachable;
 }
 $117 = $114 & 2;
 $118 = ($117|0)==(0);
 if ($118) {
  $119 = HEAP32[((14072 + 24|0))>>2]|0;
  $120 = ($9|0)==($119|0);
  if ($120) {
   $121 = HEAP32[((14072 + 12|0))>>2]|0;
   $122 = (($121) + ($psize$0))|0;
   HEAP32[((14072 + 12|0))>>2] = $122;
   HEAP32[((14072 + 24|0))>>2] = $p$0;
   $123 = $122 | 1;
   $124 = (($p$0) + 4|0);
   HEAP32[$124>>2] = $123;
   $125 = HEAP32[((14072 + 20|0))>>2]|0;
   $126 = ($p$0|0)==($125|0);
   if (!($126)) {
    STACKTOP = sp;return;
   }
   HEAP32[((14072 + 20|0))>>2] = 0;
   HEAP32[((14072 + 8|0))>>2] = 0;
   STACKTOP = sp;return;
  }
  $127 = HEAP32[((14072 + 20|0))>>2]|0;
  $128 = ($9|0)==($127|0);
  if ($128) {
   $129 = HEAP32[((14072 + 8|0))>>2]|0;
   $130 = (($129) + ($psize$0))|0;
   HEAP32[((14072 + 8|0))>>2] = $130;
   HEAP32[((14072 + 20|0))>>2] = $p$0;
   $131 = $130 | 1;
   $132 = (($p$0) + 4|0);
   HEAP32[$132>>2] = $131;
   $133 = (($p$0) + ($130)|0);
   HEAP32[$133>>2] = $130;
   STACKTOP = sp;return;
  }
  $134 = $114 & -8;
  $135 = (($134) + ($psize$0))|0;
  $136 = $114 >>> 3;
  $137 = ($114>>>0)<(256);
  do {
   if ($137) {
    $138 = (($mem) + ($8)|0);
    $139 = HEAP32[$138>>2]|0;
    $$sum2324 = $8 | 4;
    $140 = (($mem) + ($$sum2324)|0);
    $141 = HEAP32[$140>>2]|0;
    $142 = $136 << 1;
    $143 = ((14072 + ($142<<2)|0) + 40|0);
    $144 = ($139|0)==($143|0);
    if (!($144)) {
     $145 = HEAP32[((14072 + 16|0))>>2]|0;
     $146 = ($139>>>0)<($145>>>0);
     if ($146) {
      _abort();
      // unreachable;
     }
     $147 = (($139) + 12|0);
     $148 = HEAP32[$147>>2]|0;
     $149 = ($148|0)==($9|0);
     if (!($149)) {
      _abort();
      // unreachable;
     }
    }
    $150 = ($141|0)==($139|0);
    if ($150) {
     $151 = 1 << $136;
     $152 = $151 ^ -1;
     $153 = HEAP32[14072>>2]|0;
     $154 = $153 & $152;
     HEAP32[14072>>2] = $154;
     break;
    }
    $155 = ($141|0)==($143|0);
    if ($155) {
     $$pre67 = (($141) + 8|0);
     $$pre$phi68Z2D = $$pre67;
    } else {
     $156 = HEAP32[((14072 + 16|0))>>2]|0;
     $157 = ($141>>>0)<($156>>>0);
     if ($157) {
      _abort();
      // unreachable;
     }
     $158 = (($141) + 8|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = ($159|0)==($9|0);
     if ($160) {
      $$pre$phi68Z2D = $158;
     } else {
      _abort();
      // unreachable;
     }
    }
    $161 = (($139) + 12|0);
    HEAP32[$161>>2] = $141;
    HEAP32[$$pre$phi68Z2D>>2] = $139;
   } else {
    $$sum5 = (($8) + 16)|0;
    $162 = (($mem) + ($$sum5)|0);
    $163 = HEAP32[$162>>2]|0;
    $$sum67 = $8 | 4;
    $164 = (($mem) + ($$sum67)|0);
    $165 = HEAP32[$164>>2]|0;
    $166 = ($165|0)==($9|0);
    do {
     if ($166) {
      $$sum9 = (($8) + 12)|0;
      $177 = (($mem) + ($$sum9)|0);
      $178 = HEAP32[$177>>2]|0;
      $179 = ($178|0)==(0|0);
      if ($179) {
       $$sum8 = (($8) + 8)|0;
       $180 = (($mem) + ($$sum8)|0);
       $181 = HEAP32[$180>>2]|0;
       $182 = ($181|0)==(0|0);
       if ($182) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $181;$RP9$0 = $180;
       }
      } else {
       $R7$0 = $178;$RP9$0 = $177;
      }
      while(1) {
       $183 = (($R7$0) + 20|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($184|0)==(0|0);
       if (!($185)) {
        $R7$0 = $184;$RP9$0 = $183;
        continue;
       }
       $186 = (($R7$0) + 16|0);
       $187 = HEAP32[$186>>2]|0;
       $188 = ($187|0)==(0|0);
       if ($188) {
        break;
       } else {
        $R7$0 = $187;$RP9$0 = $186;
       }
      }
      $189 = HEAP32[((14072 + 16|0))>>2]|0;
      $190 = ($RP9$0>>>0)<($189>>>0);
      if ($190) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0>>2] = 0;
       $R7$1 = $R7$0;
       break;
      }
     } else {
      $167 = (($mem) + ($8)|0);
      $168 = HEAP32[$167>>2]|0;
      $169 = HEAP32[((14072 + 16|0))>>2]|0;
      $170 = ($168>>>0)<($169>>>0);
      if ($170) {
       _abort();
       // unreachable;
      }
      $171 = (($168) + 12|0);
      $172 = HEAP32[$171>>2]|0;
      $173 = ($172|0)==($9|0);
      if (!($173)) {
       _abort();
       // unreachable;
      }
      $174 = (($165) + 8|0);
      $175 = HEAP32[$174>>2]|0;
      $176 = ($175|0)==($9|0);
      if ($176) {
       HEAP32[$171>>2] = $165;
       HEAP32[$174>>2] = $168;
       $R7$1 = $165;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $191 = ($163|0)==(0|0);
    if (!($191)) {
     $$sum18 = (($8) + 20)|0;
     $192 = (($mem) + ($$sum18)|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ((14072 + ($193<<2)|0) + 304|0);
     $195 = HEAP32[$194>>2]|0;
     $196 = ($9|0)==($195|0);
     if ($196) {
      HEAP32[$194>>2] = $R7$1;
      $cond54 = ($R7$1|0)==(0|0);
      if ($cond54) {
       $197 = 1 << $193;
       $198 = $197 ^ -1;
       $199 = HEAP32[((14072 + 4|0))>>2]|0;
       $200 = $199 & $198;
       HEAP32[((14072 + 4|0))>>2] = $200;
       break;
      }
     } else {
      $201 = HEAP32[((14072 + 16|0))>>2]|0;
      $202 = ($163>>>0)<($201>>>0);
      if ($202) {
       _abort();
       // unreachable;
      }
      $203 = (($163) + 16|0);
      $204 = HEAP32[$203>>2]|0;
      $205 = ($204|0)==($9|0);
      if ($205) {
       HEAP32[$203>>2] = $R7$1;
      } else {
       $206 = (($163) + 20|0);
       HEAP32[$206>>2] = $R7$1;
      }
      $207 = ($R7$1|0)==(0|0);
      if ($207) {
       break;
      }
     }
     $208 = HEAP32[((14072 + 16|0))>>2]|0;
     $209 = ($R7$1>>>0)<($208>>>0);
     if ($209) {
      _abort();
      // unreachable;
     }
     $210 = (($R7$1) + 24|0);
     HEAP32[$210>>2] = $163;
     $$sum19 = (($8) + 8)|0;
     $211 = (($mem) + ($$sum19)|0);
     $212 = HEAP32[$211>>2]|0;
     $213 = ($212|0)==(0|0);
     do {
      if (!($213)) {
       $214 = HEAP32[((14072 + 16|0))>>2]|0;
       $215 = ($212>>>0)<($214>>>0);
       if ($215) {
        _abort();
        // unreachable;
       } else {
        $216 = (($R7$1) + 16|0);
        HEAP32[$216>>2] = $212;
        $217 = (($212) + 24|0);
        HEAP32[$217>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum20 = (($8) + 12)|0;
     $218 = (($mem) + ($$sum20)|0);
     $219 = HEAP32[$218>>2]|0;
     $220 = ($219|0)==(0|0);
     if (!($220)) {
      $221 = HEAP32[((14072 + 16|0))>>2]|0;
      $222 = ($219>>>0)<($221>>>0);
      if ($222) {
       _abort();
       // unreachable;
      } else {
       $223 = (($R7$1) + 20|0);
       HEAP32[$223>>2] = $219;
       $224 = (($219) + 24|0);
       HEAP32[$224>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $225 = $135 | 1;
  $226 = (($p$0) + 4|0);
  HEAP32[$226>>2] = $225;
  $227 = (($p$0) + ($135)|0);
  HEAP32[$227>>2] = $135;
  $228 = HEAP32[((14072 + 20|0))>>2]|0;
  $229 = ($p$0|0)==($228|0);
  if ($229) {
   HEAP32[((14072 + 8|0))>>2] = $135;
   STACKTOP = sp;return;
  } else {
   $psize$1 = $135;
  }
 } else {
  $230 = $114 & -2;
  HEAP32[$113>>2] = $230;
  $231 = $psize$0 | 1;
  $232 = (($p$0) + 4|0);
  HEAP32[$232>>2] = $231;
  $233 = (($p$0) + ($psize$0)|0);
  HEAP32[$233>>2] = $psize$0;
  $psize$1 = $psize$0;
 }
 $234 = $psize$1 >>> 3;
 $235 = ($psize$1>>>0)<(256);
 if ($235) {
  $236 = $234 << 1;
  $237 = ((14072 + ($236<<2)|0) + 40|0);
  $238 = HEAP32[14072>>2]|0;
  $239 = 1 << $234;
  $240 = $238 & $239;
  $241 = ($240|0)==(0);
  if ($241) {
   $242 = $238 | $239;
   HEAP32[14072>>2] = $242;
   $$sum16$pre = (($236) + 2)|0;
   $$pre = ((14072 + ($$sum16$pre<<2)|0) + 40|0);
   $$pre$phiZ2D = $$pre;$F16$0 = $237;
  } else {
   $$sum17 = (($236) + 2)|0;
   $243 = ((14072 + ($$sum17<<2)|0) + 40|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = HEAP32[((14072 + 16|0))>>2]|0;
   $246 = ($244>>>0)<($245>>>0);
   if ($246) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $243;$F16$0 = $244;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $p$0;
  $247 = (($F16$0) + 12|0);
  HEAP32[$247>>2] = $p$0;
  $248 = (($p$0) + 8|0);
  HEAP32[$248>>2] = $F16$0;
  $249 = (($p$0) + 12|0);
  HEAP32[$249>>2] = $237;
  STACKTOP = sp;return;
 }
 $250 = $psize$1 >>> 8;
 $251 = ($250|0)==(0);
 if ($251) {
  $I18$0 = 0;
 } else {
  $252 = ($psize$1>>>0)>(16777215);
  if ($252) {
   $I18$0 = 31;
  } else {
   $253 = (($250) + 1048320)|0;
   $254 = $253 >>> 16;
   $255 = $254 & 8;
   $256 = $250 << $255;
   $257 = (($256) + 520192)|0;
   $258 = $257 >>> 16;
   $259 = $258 & 4;
   $260 = $259 | $255;
   $261 = $256 << $259;
   $262 = (($261) + 245760)|0;
   $263 = $262 >>> 16;
   $264 = $263 & 2;
   $265 = $260 | $264;
   $266 = (14 - ($265))|0;
   $267 = $261 << $264;
   $268 = $267 >>> 15;
   $269 = (($266) + ($268))|0;
   $270 = $269 << 1;
   $271 = (($269) + 7)|0;
   $272 = $psize$1 >>> $271;
   $273 = $272 & 1;
   $274 = $273 | $270;
   $I18$0 = $274;
  }
 }
 $275 = ((14072 + ($I18$0<<2)|0) + 304|0);
 $276 = (($p$0) + 28|0);
 $I18$0$c = $I18$0;
 HEAP32[$276>>2] = $I18$0$c;
 $277 = (($p$0) + 20|0);
 HEAP32[$277>>2] = 0;
 $278 = (($p$0) + 16|0);
 HEAP32[$278>>2] = 0;
 $279 = HEAP32[((14072 + 4|0))>>2]|0;
 $280 = 1 << $I18$0;
 $281 = $279 & $280;
 $282 = ($281|0)==(0);
 L199: do {
  if ($282) {
   $283 = $279 | $280;
   HEAP32[((14072 + 4|0))>>2] = $283;
   HEAP32[$275>>2] = $p$0;
   $284 = (($p$0) + 24|0);
   HEAP32[$284>>2] = $275;
   $285 = (($p$0) + 12|0);
   HEAP32[$285>>2] = $p$0;
   $286 = (($p$0) + 8|0);
   HEAP32[$286>>2] = $p$0;
  } else {
   $287 = HEAP32[$275>>2]|0;
   $288 = ($I18$0|0)==(31);
   if ($288) {
    $296 = 0;
   } else {
    $289 = $I18$0 >>> 1;
    $290 = (25 - ($289))|0;
    $296 = $290;
   }
   $291 = (($287) + 4|0);
   $292 = HEAP32[$291>>2]|0;
   $293 = $292 & -8;
   $294 = ($293|0)==($psize$1|0);
   L205: do {
    if ($294) {
     $T$0$lcssa = $287;
    } else {
     $295 = $psize$1 << $296;
     $K19$057 = $295;$T$056 = $287;
     while(1) {
      $303 = $K19$057 >>> 31;
      $304 = ((($T$056) + ($303<<2)|0) + 16|0);
      $299 = HEAP32[$304>>2]|0;
      $305 = ($299|0)==(0|0);
      if ($305) {
       break;
      }
      $297 = $K19$057 << 1;
      $298 = (($299) + 4|0);
      $300 = HEAP32[$298>>2]|0;
      $301 = $300 & -8;
      $302 = ($301|0)==($psize$1|0);
      if ($302) {
       $T$0$lcssa = $299;
       break L205;
      } else {
       $K19$057 = $297;$T$056 = $299;
      }
     }
     $306 = HEAP32[((14072 + 16|0))>>2]|0;
     $307 = ($304>>>0)<($306>>>0);
     if ($307) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$304>>2] = $p$0;
      $308 = (($p$0) + 24|0);
      HEAP32[$308>>2] = $T$056;
      $309 = (($p$0) + 12|0);
      HEAP32[$309>>2] = $p$0;
      $310 = (($p$0) + 8|0);
      HEAP32[$310>>2] = $p$0;
      break L199;
     }
    }
   } while(0);
   $311 = (($T$0$lcssa) + 8|0);
   $312 = HEAP32[$311>>2]|0;
   $313 = HEAP32[((14072 + 16|0))>>2]|0;
   $314 = ($T$0$lcssa>>>0)<($313>>>0);
   if ($314) {
    _abort();
    // unreachable;
   }
   $315 = ($312>>>0)<($313>>>0);
   if ($315) {
    _abort();
    // unreachable;
   } else {
    $316 = (($312) + 12|0);
    HEAP32[$316>>2] = $p$0;
    HEAP32[$311>>2] = $p$0;
    $317 = (($p$0) + 8|0);
    HEAP32[$317>>2] = $312;
    $318 = (($p$0) + 12|0);
    HEAP32[$318>>2] = $T$0$lcssa;
    $319 = (($p$0) + 24|0);
    HEAP32[$319>>2] = 0;
    break;
   }
  }
 } while(0);
 $320 = HEAP32[((14072 + 32|0))>>2]|0;
 $321 = (($320) + -1)|0;
 HEAP32[((14072 + 32|0))>>2] = $321;
 $322 = ($321|0)==(0);
 if ($322) {
  $sp$0$in$i = ((14072 + 456|0));
 } else {
  STACKTOP = sp;return;
 }
 while(1) {
  $sp$0$i = HEAP32[$sp$0$in$i>>2]|0;
  $323 = ($sp$0$i|0)==(0|0);
  $324 = (($sp$0$i) + 8|0);
  if ($323) {
   break;
  } else {
   $sp$0$in$i = $324;
  }
 }
 HEAP32[((14072 + 32|0))>>2] = -1;
 STACKTOP = sp;return;
}
function _calloc($n_elements,$elem_size) {
 $n_elements = $n_elements|0;
 $elem_size = $elem_size|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $req$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n_elements|0)==(0);
 if ($0) {
  $req$0 = 0;
 } else {
  $1 = Math_imul($elem_size, $n_elements)|0;
  $2 = $elem_size | $n_elements;
  $3 = ($2>>>0)>(65535);
  if ($3) {
   $4 = (($1>>>0) / ($n_elements>>>0))&-1;
   $5 = ($4|0)==($elem_size|0);
   $$ = $5 ? $1 : -1;
   $req$0 = $$;
  } else {
   $req$0 = $1;
  }
 }
 $6 = (_malloc($req$0)|0);
 $7 = ($6|0)==(0|0);
 if ($7) {
  STACKTOP = sp;return ($6|0);
 }
 $8 = (($6) + -4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 3;
 $11 = ($10|0)==(0);
 if ($11) {
  STACKTOP = sp;return ($6|0);
 }
 _memset(($6|0),0,($req$0|0))|0;
 STACKTOP = sp;return ($6|0);
}
function _realloc($oldmem,$bytes) {
 $oldmem = $oldmem|0;
 $bytes = $bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $mem$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($oldmem|0)==(0|0);
 do {
  if ($0) {
   $1 = (_malloc($bytes)|0);
   $mem$0 = $1;
  } else {
   $2 = ($bytes>>>0)>(4294967231);
   if ($2) {
    $3 = (___errno_location()|0);
    HEAP32[$3>>2] = 12;
    $mem$0 = 0;
    break;
   }
   $4 = ($bytes>>>0)<(11);
   if ($4) {
    $8 = 16;
   } else {
    $5 = (($bytes) + 11)|0;
    $6 = $5 & -8;
    $8 = $6;
   }
   $7 = (($oldmem) + -8|0);
   $9 = (_try_realloc_chunk($7,$8)|0);
   $10 = ($9|0)==(0|0);
   if (!($10)) {
    $11 = (($9) + 8|0);
    $mem$0 = $11;
    break;
   }
   $12 = (_malloc($bytes)|0);
   $13 = ($12|0)==(0|0);
   if ($13) {
    $mem$0 = 0;
   } else {
    $14 = (($oldmem) + -4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = $15 & -8;
    $17 = $15 & 3;
    $18 = ($17|0)==(0);
    $19 = $18 ? 8 : 4;
    $20 = (($16) - ($19))|0;
    $21 = ($20>>>0)<($bytes>>>0);
    $22 = $21 ? $20 : $bytes;
    _memcpy(($12|0),($oldmem|0),($22|0))|0;
    _free($oldmem);
    $mem$0 = $12;
   }
  }
 } while(0);
 STACKTOP = sp;return ($mem$0|0);
}
function _try_realloc_chunk($p,$nb) {
 $p = $p|0;
 $nb = $nb|0;
 var $$pre = 0, $$pre$phiZ2D = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum2728 = 0, $$sum3 = 0, $$sum4 = 0, $$sum5 = 0, $$sum78 = 0;
 var $$sum910 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $R$0 = 0, $R$1 = 0, $RP$0 = 0;
 var $cond = 0, $newp$0 = 0, $or$cond = 0, $storemerge = 0, $storemerge21 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($p) + 4|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 & -8;
 $3 = (($p) + ($2)|0);
 $4 = HEAP32[((14072 + 16|0))>>2]|0;
 $5 = ($p>>>0)<($4>>>0);
 if ($5) {
  _abort();
  // unreachable;
 }
 $6 = $1 & 3;
 $7 = ($6|0)!=(1);
 $8 = ($p>>>0)<($3>>>0);
 $or$cond = $7 & $8;
 if (!($or$cond)) {
  _abort();
  // unreachable;
 }
 $$sum2728 = $2 | 4;
 $9 = (($p) + ($$sum2728)|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & 1;
 $12 = ($11|0)==(0);
 if ($12) {
  _abort();
  // unreachable;
 }
 $13 = ($6|0)==(0);
 if ($13) {
  $14 = ($nb>>>0)<(256);
  if ($14) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $15 = (($nb) + 4)|0;
  $16 = ($2>>>0)<($15>>>0);
  if (!($16)) {
   $17 = (($2) - ($nb))|0;
   $18 = HEAP32[((14544 + 8|0))>>2]|0;
   $19 = $18 << 1;
   $20 = ($17>>>0)>($19>>>0);
   if (!($20)) {
    $newp$0 = $p;
    STACKTOP = sp;return ($newp$0|0);
   }
  }
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $21 = ($2>>>0)<($nb>>>0);
 if (!($21)) {
  $22 = (($2) - ($nb))|0;
  $23 = ($22>>>0)>(15);
  if (!($23)) {
   $newp$0 = $p;
   STACKTOP = sp;return ($newp$0|0);
  }
  $24 = (($p) + ($nb)|0);
  $25 = $1 & 1;
  $26 = $25 | $nb;
  $27 = $26 | 2;
  HEAP32[$0>>2] = $27;
  $$sum23 = (($nb) + 4)|0;
  $28 = (($p) + ($$sum23)|0);
  $29 = $22 | 3;
  HEAP32[$28>>2] = $29;
  $30 = HEAP32[$9>>2]|0;
  $31 = $30 | 1;
  HEAP32[$9>>2] = $31;
  _dispose_chunk($24,$22);
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $32 = HEAP32[((14072 + 24|0))>>2]|0;
 $33 = ($3|0)==($32|0);
 if ($33) {
  $34 = HEAP32[((14072 + 12|0))>>2]|0;
  $35 = (($34) + ($2))|0;
  $36 = ($35>>>0)>($nb>>>0);
  if (!($36)) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $37 = (($35) - ($nb))|0;
  $38 = (($p) + ($nb)|0);
  $39 = $1 & 1;
  $40 = $39 | $nb;
  $41 = $40 | 2;
  HEAP32[$0>>2] = $41;
  $$sum22 = (($nb) + 4)|0;
  $42 = (($p) + ($$sum22)|0);
  $43 = $37 | 1;
  HEAP32[$42>>2] = $43;
  HEAP32[((14072 + 24|0))>>2] = $38;
  HEAP32[((14072 + 12|0))>>2] = $37;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $44 = HEAP32[((14072 + 20|0))>>2]|0;
 $45 = ($3|0)==($44|0);
 if ($45) {
  $46 = HEAP32[((14072 + 8|0))>>2]|0;
  $47 = (($46) + ($2))|0;
  $48 = ($47>>>0)<($nb>>>0);
  if ($48) {
   $newp$0 = 0;
   STACKTOP = sp;return ($newp$0|0);
  }
  $49 = (($47) - ($nb))|0;
  $50 = ($49>>>0)>(15);
  if ($50) {
   $51 = (($p) + ($nb)|0);
   $52 = (($p) + ($47)|0);
   $53 = $1 & 1;
   $54 = $53 | $nb;
   $55 = $54 | 2;
   HEAP32[$0>>2] = $55;
   $$sum19 = (($nb) + 4)|0;
   $56 = (($p) + ($$sum19)|0);
   $57 = $49 | 1;
   HEAP32[$56>>2] = $57;
   HEAP32[$52>>2] = $49;
   $$sum20 = (($47) + 4)|0;
   $58 = (($p) + ($$sum20)|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = $59 & -2;
   HEAP32[$58>>2] = $60;
   $storemerge = $51;$storemerge21 = $49;
  } else {
   $61 = $1 & 1;
   $62 = $61 | $47;
   $63 = $62 | 2;
   HEAP32[$0>>2] = $63;
   $$sum17 = (($47) + 4)|0;
   $64 = (($p) + ($$sum17)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $65 | 1;
   HEAP32[$64>>2] = $66;
   $storemerge = 0;$storemerge21 = 0;
  }
  HEAP32[((14072 + 8|0))>>2] = $storemerge21;
  HEAP32[((14072 + 20|0))>>2] = $storemerge;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 $67 = $10 & 2;
 $68 = ($67|0)==(0);
 if (!($68)) {
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $69 = $10 & -8;
 $70 = (($69) + ($2))|0;
 $71 = ($70>>>0)<($nb>>>0);
 if ($71) {
  $newp$0 = 0;
  STACKTOP = sp;return ($newp$0|0);
 }
 $72 = (($70) - ($nb))|0;
 $73 = $10 >>> 3;
 $74 = ($10>>>0)<(256);
 do {
  if ($74) {
   $$sum15 = (($2) + 8)|0;
   $75 = (($p) + ($$sum15)|0);
   $76 = HEAP32[$75>>2]|0;
   $$sum16 = (($2) + 12)|0;
   $77 = (($p) + ($$sum16)|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = $73 << 1;
   $80 = ((14072 + ($79<<2)|0) + 40|0);
   $81 = ($76|0)==($80|0);
   if (!($81)) {
    $82 = ($76>>>0)<($4>>>0);
    if ($82) {
     _abort();
     // unreachable;
    }
    $83 = (($76) + 12|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = ($84|0)==($3|0);
    if (!($85)) {
     _abort();
     // unreachable;
    }
   }
   $86 = ($78|0)==($76|0);
   if ($86) {
    $87 = 1 << $73;
    $88 = $87 ^ -1;
    $89 = HEAP32[14072>>2]|0;
    $90 = $89 & $88;
    HEAP32[14072>>2] = $90;
    break;
   }
   $91 = ($78|0)==($80|0);
   if ($91) {
    $$pre = (($78) + 8|0);
    $$pre$phiZ2D = $$pre;
   } else {
    $92 = ($78>>>0)<($4>>>0);
    if ($92) {
     _abort();
     // unreachable;
    }
    $93 = (($78) + 8|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)==($3|0);
    if ($95) {
     $$pre$phiZ2D = $93;
    } else {
     _abort();
     // unreachable;
    }
   }
   $96 = (($76) + 12|0);
   HEAP32[$96>>2] = $78;
   HEAP32[$$pre$phiZ2D>>2] = $76;
  } else {
   $$sum = (($2) + 24)|0;
   $97 = (($p) + ($$sum)|0);
   $98 = HEAP32[$97>>2]|0;
   $$sum2 = (($2) + 12)|0;
   $99 = (($p) + ($$sum2)|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = ($100|0)==($3|0);
   do {
    if ($101) {
     $$sum4 = (($2) + 20)|0;
     $111 = (($p) + ($$sum4)|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = ($112|0)==(0|0);
     if ($113) {
      $$sum3 = (($2) + 16)|0;
      $114 = (($p) + ($$sum3)|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = ($115|0)==(0|0);
      if ($116) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $115;$RP$0 = $114;
      }
     } else {
      $R$0 = $112;$RP$0 = $111;
     }
     while(1) {
      $117 = (($R$0) + 20|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = ($118|0)==(0|0);
      if (!($119)) {
       $R$0 = $118;$RP$0 = $117;
       continue;
      }
      $120 = (($R$0) + 16|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = ($121|0)==(0|0);
      if ($122) {
       break;
      } else {
       $R$0 = $121;$RP$0 = $120;
      }
     }
     $123 = ($RP$0>>>0)<($4>>>0);
     if ($123) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum14 = (($2) + 8)|0;
     $102 = (($p) + ($$sum14)|0);
     $103 = HEAP32[$102>>2]|0;
     $104 = ($103>>>0)<($4>>>0);
     if ($104) {
      _abort();
      // unreachable;
     }
     $105 = (($103) + 12|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = ($106|0)==($3|0);
     if (!($107)) {
      _abort();
      // unreachable;
     }
     $108 = (($100) + 8|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = ($109|0)==($3|0);
     if ($110) {
      HEAP32[$105>>2] = $100;
      HEAP32[$108>>2] = $103;
      $R$1 = $100;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $124 = ($98|0)==(0|0);
   if (!($124)) {
    $$sum11 = (($2) + 28)|0;
    $125 = (($p) + ($$sum11)|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = ((14072 + ($126<<2)|0) + 304|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = ($3|0)==($128|0);
    if ($129) {
     HEAP32[$127>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $130 = 1 << $126;
      $131 = $130 ^ -1;
      $132 = HEAP32[((14072 + 4|0))>>2]|0;
      $133 = $132 & $131;
      HEAP32[((14072 + 4|0))>>2] = $133;
      break;
     }
    } else {
     $134 = HEAP32[((14072 + 16|0))>>2]|0;
     $135 = ($98>>>0)<($134>>>0);
     if ($135) {
      _abort();
      // unreachable;
     }
     $136 = (($98) + 16|0);
     $137 = HEAP32[$136>>2]|0;
     $138 = ($137|0)==($3|0);
     if ($138) {
      HEAP32[$136>>2] = $R$1;
     } else {
      $139 = (($98) + 20|0);
      HEAP32[$139>>2] = $R$1;
     }
     $140 = ($R$1|0)==(0|0);
     if ($140) {
      break;
     }
    }
    $141 = HEAP32[((14072 + 16|0))>>2]|0;
    $142 = ($R$1>>>0)<($141>>>0);
    if ($142) {
     _abort();
     // unreachable;
    }
    $143 = (($R$1) + 24|0);
    HEAP32[$143>>2] = $98;
    $$sum12 = (($2) + 16)|0;
    $144 = (($p) + ($$sum12)|0);
    $145 = HEAP32[$144>>2]|0;
    $146 = ($145|0)==(0|0);
    do {
     if (!($146)) {
      $147 = HEAP32[((14072 + 16|0))>>2]|0;
      $148 = ($145>>>0)<($147>>>0);
      if ($148) {
       _abort();
       // unreachable;
      } else {
       $149 = (($R$1) + 16|0);
       HEAP32[$149>>2] = $145;
       $150 = (($145) + 24|0);
       HEAP32[$150>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum13 = (($2) + 20)|0;
    $151 = (($p) + ($$sum13)|0);
    $152 = HEAP32[$151>>2]|0;
    $153 = ($152|0)==(0|0);
    if (!($153)) {
     $154 = HEAP32[((14072 + 16|0))>>2]|0;
     $155 = ($152>>>0)<($154>>>0);
     if ($155) {
      _abort();
      // unreachable;
     } else {
      $156 = (($R$1) + 20|0);
      HEAP32[$156>>2] = $152;
      $157 = (($152) + 24|0);
      HEAP32[$157>>2] = $R$1;
      break;
     }
    }
   }
  }
 } while(0);
 $158 = ($72>>>0)<(16);
 if ($158) {
  $159 = HEAP32[$0>>2]|0;
  $160 = $159 & 1;
  $161 = $70 | $160;
  $162 = $161 | 2;
  HEAP32[$0>>2] = $162;
  $$sum910 = $70 | 4;
  $163 = (($p) + ($$sum910)|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = $164 | 1;
  HEAP32[$163>>2] = $165;
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 } else {
  $166 = (($p) + ($nb)|0);
  $167 = HEAP32[$0>>2]|0;
  $168 = $167 & 1;
  $169 = $168 | $nb;
  $170 = $169 | 2;
  HEAP32[$0>>2] = $170;
  $$sum5 = (($nb) + 4)|0;
  $171 = (($p) + ($$sum5)|0);
  $172 = $72 | 3;
  HEAP32[$171>>2] = $172;
  $$sum78 = $70 | 4;
  $173 = (($p) + ($$sum78)|0);
  $174 = HEAP32[$173>>2]|0;
  $175 = $174 | 1;
  HEAP32[$173>>2] = $175;
  _dispose_chunk($166,$72);
  $newp$0 = $p;
  STACKTOP = sp;return ($newp$0|0);
 }
 return 0|0;
}
function _dispose_chunk($p,$psize) {
 $p = $p|0;
 $psize = $psize|0;
 var $$0 = 0, $$02 = 0, $$1 = 0, $$pre = 0, $$pre$phi63Z2D = 0, $$pre$phi65Z2D = 0, $$pre$phiZ2D = 0, $$pre62 = 0, $$pre64 = 0, $$sum = 0, $$sum1 = 0, $$sum12$pre = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum18 = 0, $$sum19 = 0, $$sum2 = 0;
 var $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0, $$sum25 = 0, $$sum26 = 0, $$sum27 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0, $$sum31 = 0, $$sum4 = 0, $$sum5 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0;
 var $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0;
 var $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0;
 var $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0;
 var $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0;
 var $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0;
 var $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0;
 var $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0;
 var $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0;
 var $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0;
 var $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0;
 var $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0;
 var $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I19$0 = 0, $I19$0$c = 0, $K20$049 = 0, $R$0 = 0, $R$1 = 0, $R7$0 = 0, $R7$1 = 0, $RP$0 = 0, $RP9$0 = 0, $T$0$lcssa = 0, $T$048 = 0, $cond = 0, $cond46 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $0 = (($p) + ($psize)|0);
 $1 = (($p) + 4|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 1;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   $5 = HEAP32[$p>>2]|0;
   $6 = $2 & 3;
   $7 = ($6|0)==(0);
   if ($7) {
    STACKTOP = sp;return;
   }
   $8 = (0 - ($5))|0;
   $9 = (($p) + ($8)|0);
   $10 = (($5) + ($psize))|0;
   $11 = HEAP32[((14072 + 16|0))>>2]|0;
   $12 = ($9>>>0)<($11>>>0);
   if ($12) {
    _abort();
    // unreachable;
   }
   $13 = HEAP32[((14072 + 20|0))>>2]|0;
   $14 = ($9|0)==($13|0);
   if ($14) {
    $$sum = (($psize) + 4)|0;
    $100 = (($p) + ($$sum)|0);
    $101 = HEAP32[$100>>2]|0;
    $102 = $101 & 3;
    $103 = ($102|0)==(3);
    if (!($103)) {
     $$0 = $9;$$02 = $10;
     break;
    }
    HEAP32[((14072 + 8|0))>>2] = $10;
    $104 = HEAP32[$100>>2]|0;
    $105 = $104 & -2;
    HEAP32[$100>>2] = $105;
    $106 = $10 | 1;
    $$sum20 = (4 - ($5))|0;
    $107 = (($p) + ($$sum20)|0);
    HEAP32[$107>>2] = $106;
    HEAP32[$0>>2] = $10;
    STACKTOP = sp;return;
   }
   $15 = $5 >>> 3;
   $16 = ($5>>>0)<(256);
   if ($16) {
    $$sum30 = (8 - ($5))|0;
    $17 = (($p) + ($$sum30)|0);
    $18 = HEAP32[$17>>2]|0;
    $$sum31 = (12 - ($5))|0;
    $19 = (($p) + ($$sum31)|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $15 << 1;
    $22 = ((14072 + ($21<<2)|0) + 40|0);
    $23 = ($18|0)==($22|0);
    if (!($23)) {
     $24 = ($18>>>0)<($11>>>0);
     if ($24) {
      _abort();
      // unreachable;
     }
     $25 = (($18) + 12|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==($9|0);
     if (!($27)) {
      _abort();
      // unreachable;
     }
    }
    $28 = ($20|0)==($18|0);
    if ($28) {
     $29 = 1 << $15;
     $30 = $29 ^ -1;
     $31 = HEAP32[14072>>2]|0;
     $32 = $31 & $30;
     HEAP32[14072>>2] = $32;
     $$0 = $9;$$02 = $10;
     break;
    }
    $33 = ($20|0)==($22|0);
    if ($33) {
     $$pre64 = (($20) + 8|0);
     $$pre$phi65Z2D = $$pre64;
    } else {
     $34 = ($20>>>0)<($11>>>0);
     if ($34) {
      _abort();
      // unreachable;
     }
     $35 = (($20) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = ($36|0)==($9|0);
     if ($37) {
      $$pre$phi65Z2D = $35;
     } else {
      _abort();
      // unreachable;
     }
    }
    $38 = (($18) + 12|0);
    HEAP32[$38>>2] = $20;
    HEAP32[$$pre$phi65Z2D>>2] = $18;
    $$0 = $9;$$02 = $10;
    break;
   }
   $$sum22 = (24 - ($5))|0;
   $39 = (($p) + ($$sum22)|0);
   $40 = HEAP32[$39>>2]|0;
   $$sum23 = (12 - ($5))|0;
   $41 = (($p) + ($$sum23)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==($9|0);
   do {
    if ($43) {
     $$sum24 = (16 - ($5))|0;
     $$sum25 = (($$sum24) + 4)|0;
     $53 = (($p) + ($$sum25)|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = ($54|0)==(0|0);
     if ($55) {
      $56 = (($p) + ($$sum24)|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = ($57|0)==(0|0);
      if ($58) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $57;$RP$0 = $56;
      }
     } else {
      $R$0 = $54;$RP$0 = $53;
     }
     while(1) {
      $59 = (($R$0) + 20|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = ($60|0)==(0|0);
      if (!($61)) {
       $R$0 = $60;$RP$0 = $59;
       continue;
      }
      $62 = (($R$0) + 16|0);
      $63 = HEAP32[$62>>2]|0;
      $64 = ($63|0)==(0|0);
      if ($64) {
       break;
      } else {
       $R$0 = $63;$RP$0 = $62;
      }
     }
     $65 = ($RP$0>>>0)<($11>>>0);
     if ($65) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0>>2] = 0;
      $R$1 = $R$0;
      break;
     }
    } else {
     $$sum29 = (8 - ($5))|0;
     $44 = (($p) + ($$sum29)|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = ($45>>>0)<($11>>>0);
     if ($46) {
      _abort();
      // unreachable;
     }
     $47 = (($45) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = ($48|0)==($9|0);
     if (!($49)) {
      _abort();
      // unreachable;
     }
     $50 = (($42) + 8|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = ($51|0)==($9|0);
     if ($52) {
      HEAP32[$47>>2] = $42;
      HEAP32[$50>>2] = $45;
      $R$1 = $42;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $66 = ($40|0)==(0|0);
   if ($66) {
    $$0 = $9;$$02 = $10;
   } else {
    $$sum26 = (28 - ($5))|0;
    $67 = (($p) + ($$sum26)|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = ((14072 + ($68<<2)|0) + 304|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = ($9|0)==($70|0);
    if ($71) {
     HEAP32[$69>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $72 = 1 << $68;
      $73 = $72 ^ -1;
      $74 = HEAP32[((14072 + 4|0))>>2]|0;
      $75 = $74 & $73;
      HEAP32[((14072 + 4|0))>>2] = $75;
      $$0 = $9;$$02 = $10;
      break;
     }
    } else {
     $76 = HEAP32[((14072 + 16|0))>>2]|0;
     $77 = ($40>>>0)<($76>>>0);
     if ($77) {
      _abort();
      // unreachable;
     }
     $78 = (($40) + 16|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = ($79|0)==($9|0);
     if ($80) {
      HEAP32[$78>>2] = $R$1;
     } else {
      $81 = (($40) + 20|0);
      HEAP32[$81>>2] = $R$1;
     }
     $82 = ($R$1|0)==(0|0);
     if ($82) {
      $$0 = $9;$$02 = $10;
      break;
     }
    }
    $83 = HEAP32[((14072 + 16|0))>>2]|0;
    $84 = ($R$1>>>0)<($83>>>0);
    if ($84) {
     _abort();
     // unreachable;
    }
    $85 = (($R$1) + 24|0);
    HEAP32[$85>>2] = $40;
    $$sum27 = (16 - ($5))|0;
    $86 = (($p) + ($$sum27)|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ($87|0)==(0|0);
    do {
     if (!($88)) {
      $89 = HEAP32[((14072 + 16|0))>>2]|0;
      $90 = ($87>>>0)<($89>>>0);
      if ($90) {
       _abort();
       // unreachable;
      } else {
       $91 = (($R$1) + 16|0);
       HEAP32[$91>>2] = $87;
       $92 = (($87) + 24|0);
       HEAP32[$92>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum28 = (($$sum27) + 4)|0;
    $93 = (($p) + ($$sum28)|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)==(0|0);
    if ($95) {
     $$0 = $9;$$02 = $10;
    } else {
     $96 = HEAP32[((14072 + 16|0))>>2]|0;
     $97 = ($94>>>0)<($96>>>0);
     if ($97) {
      _abort();
      // unreachable;
     } else {
      $98 = (($R$1) + 20|0);
      HEAP32[$98>>2] = $94;
      $99 = (($94) + 24|0);
      HEAP32[$99>>2] = $R$1;
      $$0 = $9;$$02 = $10;
      break;
     }
    }
   }
  } else {
   $$0 = $p;$$02 = $psize;
  }
 } while(0);
 $108 = HEAP32[((14072 + 16|0))>>2]|0;
 $109 = ($0>>>0)<($108>>>0);
 if ($109) {
  _abort();
  // unreachable;
 }
 $$sum1 = (($psize) + 4)|0;
 $110 = (($p) + ($$sum1)|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = $111 & 2;
 $113 = ($112|0)==(0);
 if ($113) {
  $114 = HEAP32[((14072 + 24|0))>>2]|0;
  $115 = ($0|0)==($114|0);
  if ($115) {
   $116 = HEAP32[((14072 + 12|0))>>2]|0;
   $117 = (($116) + ($$02))|0;
   HEAP32[((14072 + 12|0))>>2] = $117;
   HEAP32[((14072 + 24|0))>>2] = $$0;
   $118 = $117 | 1;
   $119 = (($$0) + 4|0);
   HEAP32[$119>>2] = $118;
   $120 = HEAP32[((14072 + 20|0))>>2]|0;
   $121 = ($$0|0)==($120|0);
   if (!($121)) {
    STACKTOP = sp;return;
   }
   HEAP32[((14072 + 20|0))>>2] = 0;
   HEAP32[((14072 + 8|0))>>2] = 0;
   STACKTOP = sp;return;
  }
  $122 = HEAP32[((14072 + 20|0))>>2]|0;
  $123 = ($0|0)==($122|0);
  if ($123) {
   $124 = HEAP32[((14072 + 8|0))>>2]|0;
   $125 = (($124) + ($$02))|0;
   HEAP32[((14072 + 8|0))>>2] = $125;
   HEAP32[((14072 + 20|0))>>2] = $$0;
   $126 = $125 | 1;
   $127 = (($$0) + 4|0);
   HEAP32[$127>>2] = $126;
   $128 = (($$0) + ($125)|0);
   HEAP32[$128>>2] = $125;
   STACKTOP = sp;return;
  }
  $129 = $111 & -8;
  $130 = (($129) + ($$02))|0;
  $131 = $111 >>> 3;
  $132 = ($111>>>0)<(256);
  do {
   if ($132) {
    $$sum18 = (($psize) + 8)|0;
    $133 = (($p) + ($$sum18)|0);
    $134 = HEAP32[$133>>2]|0;
    $$sum19 = (($psize) + 12)|0;
    $135 = (($p) + ($$sum19)|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = $131 << 1;
    $138 = ((14072 + ($137<<2)|0) + 40|0);
    $139 = ($134|0)==($138|0);
    if (!($139)) {
     $140 = ($134>>>0)<($108>>>0);
     if ($140) {
      _abort();
      // unreachable;
     }
     $141 = (($134) + 12|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = ($142|0)==($0|0);
     if (!($143)) {
      _abort();
      // unreachable;
     }
    }
    $144 = ($136|0)==($134|0);
    if ($144) {
     $145 = 1 << $131;
     $146 = $145 ^ -1;
     $147 = HEAP32[14072>>2]|0;
     $148 = $147 & $146;
     HEAP32[14072>>2] = $148;
     break;
    }
    $149 = ($136|0)==($138|0);
    if ($149) {
     $$pre62 = (($136) + 8|0);
     $$pre$phi63Z2D = $$pre62;
    } else {
     $150 = ($136>>>0)<($108>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($136) + 8|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = ($152|0)==($0|0);
     if ($153) {
      $$pre$phi63Z2D = $151;
     } else {
      _abort();
      // unreachable;
     }
    }
    $154 = (($134) + 12|0);
    HEAP32[$154>>2] = $136;
    HEAP32[$$pre$phi63Z2D>>2] = $134;
   } else {
    $$sum2 = (($psize) + 24)|0;
    $155 = (($p) + ($$sum2)|0);
    $156 = HEAP32[$155>>2]|0;
    $$sum3 = (($psize) + 12)|0;
    $157 = (($p) + ($$sum3)|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = ($158|0)==($0|0);
    do {
     if ($159) {
      $$sum5 = (($psize) + 20)|0;
      $169 = (($p) + ($$sum5)|0);
      $170 = HEAP32[$169>>2]|0;
      $171 = ($170|0)==(0|0);
      if ($171) {
       $$sum4 = (($psize) + 16)|0;
       $172 = (($p) + ($$sum4)|0);
       $173 = HEAP32[$172>>2]|0;
       $174 = ($173|0)==(0|0);
       if ($174) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $173;$RP9$0 = $172;
       }
      } else {
       $R7$0 = $170;$RP9$0 = $169;
      }
      while(1) {
       $175 = (($R7$0) + 20|0);
       $176 = HEAP32[$175>>2]|0;
       $177 = ($176|0)==(0|0);
       if (!($177)) {
        $R7$0 = $176;$RP9$0 = $175;
        continue;
       }
       $178 = (($R7$0) + 16|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = ($179|0)==(0|0);
       if ($180) {
        break;
       } else {
        $R7$0 = $179;$RP9$0 = $178;
       }
      }
      $181 = ($RP9$0>>>0)<($108>>>0);
      if ($181) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0>>2] = 0;
       $R7$1 = $R7$0;
       break;
      }
     } else {
      $$sum17 = (($psize) + 8)|0;
      $160 = (($p) + ($$sum17)|0);
      $161 = HEAP32[$160>>2]|0;
      $162 = ($161>>>0)<($108>>>0);
      if ($162) {
       _abort();
       // unreachable;
      }
      $163 = (($161) + 12|0);
      $164 = HEAP32[$163>>2]|0;
      $165 = ($164|0)==($0|0);
      if (!($165)) {
       _abort();
       // unreachable;
      }
      $166 = (($158) + 8|0);
      $167 = HEAP32[$166>>2]|0;
      $168 = ($167|0)==($0|0);
      if ($168) {
       HEAP32[$163>>2] = $158;
       HEAP32[$166>>2] = $161;
       $R7$1 = $158;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $182 = ($156|0)==(0|0);
    if (!($182)) {
     $$sum14 = (($psize) + 28)|0;
     $183 = (($p) + ($$sum14)|0);
     $184 = HEAP32[$183>>2]|0;
     $185 = ((14072 + ($184<<2)|0) + 304|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = ($0|0)==($186|0);
     if ($187) {
      HEAP32[$185>>2] = $R7$1;
      $cond46 = ($R7$1|0)==(0|0);
      if ($cond46) {
       $188 = 1 << $184;
       $189 = $188 ^ -1;
       $190 = HEAP32[((14072 + 4|0))>>2]|0;
       $191 = $190 & $189;
       HEAP32[((14072 + 4|0))>>2] = $191;
       break;
      }
     } else {
      $192 = HEAP32[((14072 + 16|0))>>2]|0;
      $193 = ($156>>>0)<($192>>>0);
      if ($193) {
       _abort();
       // unreachable;
      }
      $194 = (($156) + 16|0);
      $195 = HEAP32[$194>>2]|0;
      $196 = ($195|0)==($0|0);
      if ($196) {
       HEAP32[$194>>2] = $R7$1;
      } else {
       $197 = (($156) + 20|0);
       HEAP32[$197>>2] = $R7$1;
      }
      $198 = ($R7$1|0)==(0|0);
      if ($198) {
       break;
      }
     }
     $199 = HEAP32[((14072 + 16|0))>>2]|0;
     $200 = ($R7$1>>>0)<($199>>>0);
     if ($200) {
      _abort();
      // unreachable;
     }
     $201 = (($R7$1) + 24|0);
     HEAP32[$201>>2] = $156;
     $$sum15 = (($psize) + 16)|0;
     $202 = (($p) + ($$sum15)|0);
     $203 = HEAP32[$202>>2]|0;
     $204 = ($203|0)==(0|0);
     do {
      if (!($204)) {
       $205 = HEAP32[((14072 + 16|0))>>2]|0;
       $206 = ($203>>>0)<($205>>>0);
       if ($206) {
        _abort();
        // unreachable;
       } else {
        $207 = (($R7$1) + 16|0);
        HEAP32[$207>>2] = $203;
        $208 = (($203) + 24|0);
        HEAP32[$208>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum16 = (($psize) + 20)|0;
     $209 = (($p) + ($$sum16)|0);
     $210 = HEAP32[$209>>2]|0;
     $211 = ($210|0)==(0|0);
     if (!($211)) {
      $212 = HEAP32[((14072 + 16|0))>>2]|0;
      $213 = ($210>>>0)<($212>>>0);
      if ($213) {
       _abort();
       // unreachable;
      } else {
       $214 = (($R7$1) + 20|0);
       HEAP32[$214>>2] = $210;
       $215 = (($210) + 24|0);
       HEAP32[$215>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $216 = $130 | 1;
  $217 = (($$0) + 4|0);
  HEAP32[$217>>2] = $216;
  $218 = (($$0) + ($130)|0);
  HEAP32[$218>>2] = $130;
  $219 = HEAP32[((14072 + 20|0))>>2]|0;
  $220 = ($$0|0)==($219|0);
  if ($220) {
   HEAP32[((14072 + 8|0))>>2] = $130;
   STACKTOP = sp;return;
  } else {
   $$1 = $130;
  }
 } else {
  $221 = $111 & -2;
  HEAP32[$110>>2] = $221;
  $222 = $$02 | 1;
  $223 = (($$0) + 4|0);
  HEAP32[$223>>2] = $222;
  $224 = (($$0) + ($$02)|0);
  HEAP32[$224>>2] = $$02;
  $$1 = $$02;
 }
 $225 = $$1 >>> 3;
 $226 = ($$1>>>0)<(256);
 if ($226) {
  $227 = $225 << 1;
  $228 = ((14072 + ($227<<2)|0) + 40|0);
  $229 = HEAP32[14072>>2]|0;
  $230 = 1 << $225;
  $231 = $229 & $230;
  $232 = ($231|0)==(0);
  if ($232) {
   $233 = $229 | $230;
   HEAP32[14072>>2] = $233;
   $$sum12$pre = (($227) + 2)|0;
   $$pre = ((14072 + ($$sum12$pre<<2)|0) + 40|0);
   $$pre$phiZ2D = $$pre;$F16$0 = $228;
  } else {
   $$sum13 = (($227) + 2)|0;
   $234 = ((14072 + ($$sum13<<2)|0) + 40|0);
   $235 = HEAP32[$234>>2]|0;
   $236 = HEAP32[((14072 + 16|0))>>2]|0;
   $237 = ($235>>>0)<($236>>>0);
   if ($237) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $234;$F16$0 = $235;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $$0;
  $238 = (($F16$0) + 12|0);
  HEAP32[$238>>2] = $$0;
  $239 = (($$0) + 8|0);
  HEAP32[$239>>2] = $F16$0;
  $240 = (($$0) + 12|0);
  HEAP32[$240>>2] = $228;
  STACKTOP = sp;return;
 }
 $241 = $$1 >>> 8;
 $242 = ($241|0)==(0);
 if ($242) {
  $I19$0 = 0;
 } else {
  $243 = ($$1>>>0)>(16777215);
  if ($243) {
   $I19$0 = 31;
  } else {
   $244 = (($241) + 1048320)|0;
   $245 = $244 >>> 16;
   $246 = $245 & 8;
   $247 = $241 << $246;
   $248 = (($247) + 520192)|0;
   $249 = $248 >>> 16;
   $250 = $249 & 4;
   $251 = $250 | $246;
   $252 = $247 << $250;
   $253 = (($252) + 245760)|0;
   $254 = $253 >>> 16;
   $255 = $254 & 2;
   $256 = $251 | $255;
   $257 = (14 - ($256))|0;
   $258 = $252 << $255;
   $259 = $258 >>> 15;
   $260 = (($257) + ($259))|0;
   $261 = $260 << 1;
   $262 = (($260) + 7)|0;
   $263 = $$1 >>> $262;
   $264 = $263 & 1;
   $265 = $264 | $261;
   $I19$0 = $265;
  }
 }
 $266 = ((14072 + ($I19$0<<2)|0) + 304|0);
 $267 = (($$0) + 28|0);
 $I19$0$c = $I19$0;
 HEAP32[$267>>2] = $I19$0$c;
 $268 = (($$0) + 20|0);
 HEAP32[$268>>2] = 0;
 $269 = (($$0) + 16|0);
 HEAP32[$269>>2] = 0;
 $270 = HEAP32[((14072 + 4|0))>>2]|0;
 $271 = 1 << $I19$0;
 $272 = $270 & $271;
 $273 = ($272|0)==(0);
 if ($273) {
  $274 = $270 | $271;
  HEAP32[((14072 + 4|0))>>2] = $274;
  HEAP32[$266>>2] = $$0;
  $275 = (($$0) + 24|0);
  HEAP32[$275>>2] = $266;
  $276 = (($$0) + 12|0);
  HEAP32[$276>>2] = $$0;
  $277 = (($$0) + 8|0);
  HEAP32[$277>>2] = $$0;
  STACKTOP = sp;return;
 }
 $278 = HEAP32[$266>>2]|0;
 $279 = ($I19$0|0)==(31);
 if ($279) {
  $287 = 0;
 } else {
  $280 = $I19$0 >>> 1;
  $281 = (25 - ($280))|0;
  $287 = $281;
 }
 $282 = (($278) + 4|0);
 $283 = HEAP32[$282>>2]|0;
 $284 = $283 & -8;
 $285 = ($284|0)==($$1|0);
 L194: do {
  if ($285) {
   $T$0$lcssa = $278;
  } else {
   $286 = $$1 << $287;
   $K20$049 = $286;$T$048 = $278;
   while(1) {
    $294 = $K20$049 >>> 31;
    $295 = ((($T$048) + ($294<<2)|0) + 16|0);
    $290 = HEAP32[$295>>2]|0;
    $296 = ($290|0)==(0|0);
    if ($296) {
     break;
    }
    $288 = $K20$049 << 1;
    $289 = (($290) + 4|0);
    $291 = HEAP32[$289>>2]|0;
    $292 = $291 & -8;
    $293 = ($292|0)==($$1|0);
    if ($293) {
     $T$0$lcssa = $290;
     break L194;
    } else {
     $K20$049 = $288;$T$048 = $290;
    }
   }
   $297 = HEAP32[((14072 + 16|0))>>2]|0;
   $298 = ($295>>>0)<($297>>>0);
   if ($298) {
    _abort();
    // unreachable;
   }
   HEAP32[$295>>2] = $$0;
   $299 = (($$0) + 24|0);
   HEAP32[$299>>2] = $T$048;
   $300 = (($$0) + 12|0);
   HEAP32[$300>>2] = $$0;
   $301 = (($$0) + 8|0);
   HEAP32[$301>>2] = $$0;
   STACKTOP = sp;return;
  }
 } while(0);
 $302 = (($T$0$lcssa) + 8|0);
 $303 = HEAP32[$302>>2]|0;
 $304 = HEAP32[((14072 + 16|0))>>2]|0;
 $305 = ($T$0$lcssa>>>0)<($304>>>0);
 if ($305) {
  _abort();
  // unreachable;
 }
 $306 = ($303>>>0)<($304>>>0);
 if ($306) {
  _abort();
  // unreachable;
 }
 $307 = (($303) + 12|0);
 HEAP32[$307>>2] = $$0;
 HEAP32[$302>>2] = $$0;
 $308 = (($$0) + 8|0);
 HEAP32[$308>>2] = $303;
 $309 = (($$0) + 12|0);
 HEAP32[$309>>2] = $T$0$lcssa;
 $310 = (($$0) + 24|0);
 HEAP32[$310>>2] = 0;
 STACKTOP = sp;return;
}
function _isdigit($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($c) + -48)|0;
 $1 = ($0>>>0)<(10);
 $2 = $1&1;
 STACKTOP = sp;return ($2|0);
}
function _isspace($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($c|0)==(32);
 if ($0) {
  $4 = 1;
 } else {
  $1 = (($c) + -9)|0;
  $2 = ($1>>>0)<(5);
  $4 = $2;
 }
 $3 = $4&1;
 STACKTOP = sp;return ($3|0);
}
function ___intscan($f,$base,$pok,$0,$1) {
 $f = $f|0;
 $base = $base|0;
 $pok = $pok|0;
 $0 = $0|0;
 $1 = $1|0;
 var $$ = 0, $$1 = 0, $$121 = 0, $$122 = 0, $$18 = 0, $$19 = 0, $$base = 0, $$base16 = 0, $$lcssa = 0, $$sum = 0, $$sum10 = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum1352 = 0, $$sum14 = 0, $$sum15 = 0, $$sum9 = 0, $$sum934 = 0, $10 = 0;
 var $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0;
 var $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0;
 var $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0;
 var $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0;
 var $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0;
 var $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0;
 var $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0;
 var $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0;
 var $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0;
 var $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c$0 = 0, $c$1 = 0, $c$123 = 0, $c$2$be = 0, $c$2$lcssa = 0, $c$3$be = 0, $c$3$lcssa = 0, $c$359 = 0, $c$4$be = 0, $c$4$lcssa = 0, $c$5$be = 0, $c$6$be = 0, $c$6$lcssa = 0, $c$7$be = 0, $c$744 = 0;
 var $c$8 = 0, $c$9$be = 0, $neg$0 = 0, $or$cond = 0, $or$cond17 = 0, $or$cond20 = 0, $or$cond26 = 0, $or$cond2628 = 0, $phitmp93 = 0, $x$068 = 0, $x$135 = 0, $x$253 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $2 = ($base>>>0)>(36);
 if ($2) {
  $5 = (___errno_location()|0);
  HEAP32[$5>>2] = 22;
  $278 = 0;$279 = 0;
  tempRet0 = $278;
  STACKTOP = sp;return ($279|0);
 }
 $3 = (($f) + 4|0);
 $4 = (($f) + 100|0);
 while(1) {
  $6 = HEAP32[$3>>2]|0;
  $7 = HEAP32[$4>>2]|0;
  $8 = ($6>>>0)<($7>>>0);
  if ($8) {
   $9 = (($6) + 1|0);
   HEAP32[$3>>2] = $9;
   $10 = HEAP8[$6>>0]|0;
   $11 = $10&255;
   $13 = $11;
  } else {
   $12 = (___shgetc($f)|0);
   $13 = $12;
  }
  $14 = (_isspace($13)|0);
  $15 = ($14|0)==(0);
  if ($15) {
   break;
  }
 }
 $16 = ($13|0)==(45);
 do {
  if ((($13|0) == 43) | (($13|0) == 45)) {
   $17 = $16 << 31 >> 31;
   $18 = HEAP32[$3>>2]|0;
   $19 = HEAP32[$4>>2]|0;
   $20 = ($18>>>0)<($19>>>0);
   if ($20) {
    $21 = (($18) + 1|0);
    HEAP32[$3>>2] = $21;
    $22 = HEAP8[$18>>0]|0;
    $23 = $22&255;
    $c$0 = $23;$neg$0 = $17;
    break;
   } else {
    $24 = (___shgetc($f)|0);
    $c$0 = $24;$neg$0 = $17;
    break;
   }
  } else {
   $c$0 = $13;$neg$0 = 0;
  }
 } while(0);
 $25 = ($base|0)==(0);
 $26 = $base & -17;
 $27 = ($26|0)==(0);
 $28 = ($c$0|0)==(48);
 $or$cond = $27 & $28;
 do {
  if ($or$cond) {
   $29 = HEAP32[$3>>2]|0;
   $30 = HEAP32[$4>>2]|0;
   $31 = ($29>>>0)<($30>>>0);
   if ($31) {
    $32 = (($29) + 1|0);
    HEAP32[$3>>2] = $32;
    $33 = HEAP8[$29>>0]|0;
    $34 = $33&255;
    $37 = $34;
   } else {
    $35 = (___shgetc($f)|0);
    $37 = $35;
   }
   $36 = $37 | 32;
   $38 = ($36|0)==(120);
   if (!($38)) {
    $$base = $25 ? 8 : $base;
    $$1 = $$base;$c$1 = $37;
    label = 32;
    break;
   }
   $39 = HEAP32[$3>>2]|0;
   $40 = HEAP32[$4>>2]|0;
   $41 = ($39>>>0)<($40>>>0);
   if ($41) {
    $42 = (($39) + 1|0);
    HEAP32[$3>>2] = $42;
    $43 = HEAP8[$39>>0]|0;
    $44 = $43&255;
    $46 = $44;
   } else {
    $45 = (___shgetc($f)|0);
    $46 = $45;
   }
   $$sum15 = (($46) + 1)|0;
   $47 = (14568 + ($$sum15)|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = ($48&255)>(15);
   if ($49) {
    $50 = HEAP32[$4>>2]|0;
    $51 = ($50|0)==(0|0);
    if (!($51)) {
     $52 = HEAP32[$3>>2]|0;
     $53 = (($52) + -1|0);
     HEAP32[$3>>2] = $53;
    }
    $54 = ($pok|0)==(0);
    if ($54) {
     ___shlim($f,0);
     $278 = 0;$279 = 0;
     tempRet0 = $278;
     STACKTOP = sp;return ($279|0);
    }
    if ($51) {
     $278 = 0;$279 = 0;
     tempRet0 = $278;
     STACKTOP = sp;return ($279|0);
    }
    $55 = HEAP32[$3>>2]|0;
    $56 = (($55) + -1|0);
    HEAP32[$3>>2] = $56;
    $278 = 0;$279 = 0;
    tempRet0 = $278;
    STACKTOP = sp;return ($279|0);
   } else {
    $$122 = 16;$c$123 = $46;
    label = 47;
   }
  } else {
   $$base16 = $25 ? 10 : $base;
   $$sum = (($c$0) + 1)|0;
   $57 = (14568 + ($$sum)|0);
   $58 = HEAP8[$57>>0]|0;
   $59 = $58&255;
   $60 = ($59>>>0)<($$base16>>>0);
   if ($60) {
    $$1 = $$base16;$c$1 = $c$0;
    label = 32;
   } else {
    $61 = HEAP32[$4>>2]|0;
    $62 = ($61|0)==(0|0);
    if (!($62)) {
     $63 = HEAP32[$3>>2]|0;
     $64 = (($63) + -1|0);
     HEAP32[$3>>2] = $64;
    }
    ___shlim($f,0);
    $65 = (___errno_location()|0);
    HEAP32[$65>>2] = 22;
    $278 = 0;$279 = 0;
    tempRet0 = $278;
    STACKTOP = sp;return ($279|0);
   }
  }
 } while(0);
 if ((label|0) == 32) {
  $66 = ($$1|0)==(10);
  if ($66) {
   $67 = (($c$1) + -48)|0;
   $68 = ($67>>>0)<(10);
   if ($68) {
    $70 = $67;$x$068 = 0;
    while(1) {
     $69 = (($x$068) + ($70))|0;
     $71 = HEAP32[$3>>2]|0;
     $72 = HEAP32[$4>>2]|0;
     $73 = ($71>>>0)<($72>>>0);
     if ($73) {
      $74 = (($71) + 1|0);
      HEAP32[$3>>2] = $74;
      $75 = HEAP8[$71>>0]|0;
      $76 = $75&255;
      $c$2$be = $76;
     } else {
      $77 = (___shgetc($f)|0);
      $c$2$be = $77;
     }
     $78 = (($c$2$be) + -48)|0;
     $79 = ($78>>>0)<(10);
     $80 = ($69>>>0)<(429496729);
     $$ = $79 & $80;
     if (!($$)) {
      break;
     }
     $phitmp93 = ($69*10)|0;
     $70 = $78;$x$068 = $phitmp93;
    }
    $280 = $69;$281 = 0;$c$2$lcssa = $c$2$be;
   } else {
    $280 = 0;$281 = 0;$c$2$lcssa = $c$1;
   }
   $81 = (($c$2$lcssa) + -48)|0;
   $82 = ($81>>>0)<(10);
   if ($82) {
    $83 = $280;$84 = $281;$88 = $81;$c$359 = $c$2$lcssa;
    while(1) {
     $85 = (___muldi3(($83|0),($84|0),10,0)|0);
     $86 = tempRet0;
     $87 = ($88|0)<(0);
     $89 = $87 << 31 >> 31;
     $90 = $88 ^ -1;
     $91 = $89 ^ -1;
     $92 = ($86>>>0)>($91>>>0);
     $93 = ($86|0)==($91|0);
     $94 = ($85>>>0)>($90>>>0);
     $95 = $93 & $94;
     $96 = $92 | $95;
     if ($96) {
      $$lcssa = $88;$282 = $83;$283 = $84;$c$3$lcssa = $c$359;
      break;
     }
     $97 = (_i64Add(($85|0),($86|0),($88|0),($89|0))|0);
     $98 = tempRet0;
     $99 = HEAP32[$3>>2]|0;
     $100 = HEAP32[$4>>2]|0;
     $101 = ($99>>>0)<($100>>>0);
     if ($101) {
      $102 = (($99) + 1|0);
      HEAP32[$3>>2] = $102;
      $103 = HEAP8[$99>>0]|0;
      $104 = $103&255;
      $c$3$be = $104;
     } else {
      $105 = (___shgetc($f)|0);
      $c$3$be = $105;
     }
     $106 = (($c$3$be) + -48)|0;
     $107 = ($106>>>0)<(10);
     $108 = ($98>>>0)<(429496729);
     $109 = ($98|0)==(429496729);
     $110 = ($97>>>0)<(2576980378);
     $111 = $109 & $110;
     $112 = $108 | $111;
     $or$cond17 = $107 & $112;
     if ($or$cond17) {
      $83 = $97;$84 = $98;$88 = $106;$c$359 = $c$3$be;
     } else {
      $$lcssa = $106;$282 = $97;$283 = $98;$c$3$lcssa = $c$3$be;
      break;
     }
    }
    $113 = ($$lcssa>>>0)>(9);
    if ($113) {
     $252 = $283;$255 = $282;
    } else {
     $$121 = 10;$284 = $282;$285 = $283;$c$8 = $c$3$lcssa;
     label = 73;
    }
   } else {
    $252 = $281;$255 = $280;
   }
  } else {
   $$122 = $$1;$c$123 = $c$1;
   label = 47;
  }
 }
 L70: do {
  if ((label|0) == 47) {
   $114 = (($$122) + -1)|0;
   $115 = $114 & $$122;
   $116 = ($115|0)==(0);
   if ($116) {
    $121 = ($$122*23)|0;
    $122 = $121 >>> 5;
    $123 = $122 & 7;
    $124 = (14832 + ($123)|0);
    $125 = HEAP8[$124>>0]|0;
    $126 = $125 << 24 >> 24;
    $$sum934 = (($c$123) + 1)|0;
    $127 = (14568 + ($$sum934)|0);
    $128 = HEAP8[$127>>0]|0;
    $129 = $128&255;
    $130 = ($129>>>0)<($$122>>>0);
    if ($130) {
     $133 = $129;$x$135 = 0;
     while(1) {
      $131 = $x$135 << $126;
      $132 = $133 | $131;
      $134 = HEAP32[$3>>2]|0;
      $135 = HEAP32[$4>>2]|0;
      $136 = ($134>>>0)<($135>>>0);
      if ($136) {
       $137 = (($134) + 1|0);
       HEAP32[$3>>2] = $137;
       $138 = HEAP8[$134>>0]|0;
       $139 = $138&255;
       $c$4$be = $139;
      } else {
       $140 = (___shgetc($f)|0);
       $c$4$be = $140;
      }
      $$sum9 = (($c$4$be) + 1)|0;
      $141 = (14568 + ($$sum9)|0);
      $142 = HEAP8[$141>>0]|0;
      $143 = $142&255;
      $144 = ($143>>>0)<($$122>>>0);
      $145 = ($132>>>0)<(134217728);
      $$18 = $144 & $145;
      if ($$18) {
       $133 = $143;$x$135 = $132;
      } else {
       break;
      }
     }
     $149 = $142;$152 = 0;$155 = $132;$c$4$lcssa = $c$4$be;
    } else {
     $149 = $128;$152 = 0;$155 = 0;$c$4$lcssa = $c$123;
    }
    $146 = (_bitshift64Lshr(-1,-1,($126|0))|0);
    $147 = tempRet0;
    $148 = $149&255;
    $150 = ($148>>>0)>=($$122>>>0);
    $151 = ($152>>>0)>($147>>>0);
    $153 = ($152|0)==($147|0);
    $154 = ($155>>>0)>($146>>>0);
    $156 = $153 & $154;
    $157 = $151 | $156;
    $or$cond2628 = $150 | $157;
    if ($or$cond2628) {
     $$121 = $$122;$284 = $155;$285 = $152;$c$8 = $c$4$lcssa;
     label = 73;
     break;
    } else {
     $158 = $155;$159 = $152;$163 = $149;
    }
    while(1) {
     $160 = (_bitshift64Shl(($158|0),($159|0),($126|0))|0);
     $161 = tempRet0;
     $162 = $163&255;
     $164 = $162 | $160;
     $165 = HEAP32[$3>>2]|0;
     $166 = HEAP32[$4>>2]|0;
     $167 = ($165>>>0)<($166>>>0);
     if ($167) {
      $168 = (($165) + 1|0);
      HEAP32[$3>>2] = $168;
      $169 = HEAP8[$165>>0]|0;
      $170 = $169&255;
      $c$5$be = $170;
     } else {
      $171 = (___shgetc($f)|0);
      $c$5$be = $171;
     }
     $$sum10 = (($c$5$be) + 1)|0;
     $172 = (14568 + ($$sum10)|0);
     $173 = HEAP8[$172>>0]|0;
     $174 = $173&255;
     $175 = ($174>>>0)>=($$122>>>0);
     $176 = ($161>>>0)>($147>>>0);
     $177 = ($161|0)==($147|0);
     $178 = ($164>>>0)>($146>>>0);
     $179 = $177 & $178;
     $180 = $176 | $179;
     $or$cond26 = $175 | $180;
     if ($or$cond26) {
      $$121 = $$122;$284 = $164;$285 = $161;$c$8 = $c$5$be;
      label = 73;
      break L70;
     } else {
      $158 = $164;$159 = $161;$163 = $173;
     }
    }
   }
   $$sum1352 = (($c$123) + 1)|0;
   $117 = (14568 + ($$sum1352)|0);
   $118 = HEAP8[$117>>0]|0;
   $119 = $118&255;
   $120 = ($119>>>0)<($$122>>>0);
   if ($120) {
    $183 = $119;$x$253 = 0;
    while(1) {
     $181 = Math_imul($x$253, $$122)|0;
     $182 = (($183) + ($181))|0;
     $184 = HEAP32[$3>>2]|0;
     $185 = HEAP32[$4>>2]|0;
     $186 = ($184>>>0)<($185>>>0);
     if ($186) {
      $187 = (($184) + 1|0);
      HEAP32[$3>>2] = $187;
      $188 = HEAP8[$184>>0]|0;
      $189 = $188&255;
      $c$6$be = $189;
     } else {
      $190 = (___shgetc($f)|0);
      $c$6$be = $190;
     }
     $$sum13 = (($c$6$be) + 1)|0;
     $191 = (14568 + ($$sum13)|0);
     $192 = HEAP8[$191>>0]|0;
     $193 = $192&255;
     $194 = ($193>>>0)<($$122>>>0);
     $195 = ($182>>>0)<(119304647);
     $$19 = $194 & $195;
     if ($$19) {
      $183 = $193;$x$253 = $182;
     } else {
      break;
     }
    }
    $197 = $192;$286 = $182;$287 = 0;$c$6$lcssa = $c$6$be;
   } else {
    $197 = $118;$286 = 0;$287 = 0;$c$6$lcssa = $c$123;
   }
   $196 = $197&255;
   $198 = ($196>>>0)<($$122>>>0);
   if ($198) {
    $199 = (___udivdi3(-1,-1,($$122|0),0)|0);
    $200 = tempRet0;
    $202 = $287;$205 = $286;$211 = $197;$c$744 = $c$6$lcssa;
    while(1) {
     $201 = ($202>>>0)>($200>>>0);
     $203 = ($202|0)==($200|0);
     $204 = ($205>>>0)>($199>>>0);
     $206 = $203 & $204;
     $207 = $201 | $206;
     if ($207) {
      $$121 = $$122;$284 = $205;$285 = $202;$c$8 = $c$744;
      label = 73;
      break L70;
     }
     $208 = (___muldi3(($205|0),($202|0),($$122|0),0)|0);
     $209 = tempRet0;
     $210 = $211&255;
     $212 = $210 ^ -1;
     $213 = ($209>>>0)>(4294967295);
     $214 = ($209|0)==(-1);
     $215 = ($208>>>0)>($212>>>0);
     $216 = $214 & $215;
     $217 = $213 | $216;
     if ($217) {
      $$121 = $$122;$284 = $205;$285 = $202;$c$8 = $c$744;
      label = 73;
      break L70;
     }
     $218 = (_i64Add(($210|0),0,($208|0),($209|0))|0);
     $219 = tempRet0;
     $220 = HEAP32[$3>>2]|0;
     $221 = HEAP32[$4>>2]|0;
     $222 = ($220>>>0)<($221>>>0);
     if ($222) {
      $223 = (($220) + 1|0);
      HEAP32[$3>>2] = $223;
      $224 = HEAP8[$220>>0]|0;
      $225 = $224&255;
      $c$7$be = $225;
     } else {
      $226 = (___shgetc($f)|0);
      $c$7$be = $226;
     }
     $$sum14 = (($c$7$be) + 1)|0;
     $227 = (14568 + ($$sum14)|0);
     $228 = HEAP8[$227>>0]|0;
     $229 = $228&255;
     $230 = ($229>>>0)<($$122>>>0);
     if ($230) {
      $202 = $219;$205 = $218;$211 = $228;$c$744 = $c$7$be;
     } else {
      $$121 = $$122;$284 = $218;$285 = $219;$c$8 = $c$7$be;
      label = 73;
      break;
     }
    }
   } else {
    $$121 = $$122;$284 = $286;$285 = $287;$c$8 = $c$6$lcssa;
    label = 73;
   }
  }
 } while(0);
 if ((label|0) == 73) {
  $$sum11 = (($c$8) + 1)|0;
  $231 = (14568 + ($$sum11)|0);
  $232 = HEAP8[$231>>0]|0;
  $233 = $232&255;
  $234 = ($233>>>0)<($$121>>>0);
  if ($234) {
   while(1) {
    $235 = HEAP32[$3>>2]|0;
    $236 = HEAP32[$4>>2]|0;
    $237 = ($235>>>0)<($236>>>0);
    if ($237) {
     $238 = (($235) + 1|0);
     HEAP32[$3>>2] = $238;
     $239 = HEAP8[$235>>0]|0;
     $240 = $239&255;
     $c$9$be = $240;
    } else {
     $241 = (___shgetc($f)|0);
     $c$9$be = $241;
    }
    $$sum12 = (($c$9$be) + 1)|0;
    $242 = (14568 + ($$sum12)|0);
    $243 = HEAP8[$242>>0]|0;
    $244 = $243&255;
    $245 = ($244>>>0)<($$121>>>0);
    if (!($245)) {
     break;
    }
   }
   $246 = (___errno_location()|0);
   HEAP32[$246>>2] = 34;
   $252 = $1;$255 = $0;
  } else {
   $252 = $285;$255 = $284;
  }
 }
 $247 = HEAP32[$4>>2]|0;
 $248 = ($247|0)==(0|0);
 if (!($248)) {
  $249 = HEAP32[$3>>2]|0;
  $250 = (($249) + -1|0);
  HEAP32[$3>>2] = $250;
 }
 $251 = ($252>>>0)<($1>>>0);
 $253 = ($252|0)==($1|0);
 $254 = ($255>>>0)<($0>>>0);
 $256 = $253 & $254;
 $257 = $251 | $256;
 if (!($257)) {
  $258 = $0 & 1;
  $259 = ($258|0)==(0);
  $260 = (0)==(0);
  $261 = $259 & $260;
  $262 = ($neg$0|0)==(0);
  $or$cond20 = $261 & $262;
  if ($or$cond20) {
   $263 = (___errno_location()|0);
   HEAP32[$263>>2] = 34;
   $264 = (_i64Add(($0|0),($1|0),-1,-1)|0);
   $265 = tempRet0;
   $278 = $265;$279 = $264;
   tempRet0 = $278;
   STACKTOP = sp;return ($279|0);
  }
  $266 = ($252>>>0)>($1>>>0);
  $267 = ($252|0)==($1|0);
  $268 = ($255>>>0)>($0>>>0);
  $269 = $267 & $268;
  $270 = $266 | $269;
  if ($270) {
   $271 = (___errno_location()|0);
   HEAP32[$271>>2] = 34;
   $278 = $1;$279 = $0;
   tempRet0 = $278;
   STACKTOP = sp;return ($279|0);
  }
 }
 $272 = ($neg$0|0)<(0);
 $273 = $272 << 31 >> 31;
 $274 = $255 ^ $neg$0;
 $275 = $252 ^ $273;
 $276 = (_i64Subtract(($274|0),($275|0),($neg$0|0),($273|0))|0);
 $277 = tempRet0;
 $278 = $277;$279 = $276;
 tempRet0 = $278;
 STACKTOP = sp;return ($279|0);
}
function ___floatscan($f,$prec,$pok) {
 $f = $f|0;
 $prec = $prec|0;
 $pok = $pok|0;
 var $$$i = 0, $$$i22 = 0, $$0 = 0.0, $$02$i = 0, $$08$i = 0, $$09$i = 0, $$1$be$i = 0, $$1$ph$i = 0, $$10$i = 0, $$14$i = 0, $$2$i = 0, $$3$be$i = 0, $$3$lcssa$i = 0, $$38$i = 0, $$388$i = 0, $$397$i = 0, $$in = 0, $$k$0$i = 0, $$lcssa43$i = 0, $$lnz$0$i = 0;
 var $$neg20$i = 0, $$pn$i = 0.0, $$pre$i = 0.0, $$pre$i17 = 0, $$pre$phi$iZ2D = 0.0, $$promoted$i = 0, $$sink$off0$us$i = 0, $$sink$off0$us53$i = 0, $$x$4$i = 0, $$y$3$i = 0.0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0;
 var $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0;
 var $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0;
 var $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0;
 var $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0;
 var $18 = 0, $180 = 0, $181 = 0, $182 = 0.0, $183 = 0.0, $184 = 0.0, $185 = 0.0, $186 = 0, $187 = 0, $188 = 0.0, $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0;
 var $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0.0, $209 = 0.0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0;
 var $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0;
 var $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0;
 var $251 = 0, $252 = 0, $253 = 0, $254 = 0.0, $255 = 0.0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0.0, $264 = 0.0, $265 = 0.0, $266 = 0, $267 = 0, $268 = 0, $269 = 0;
 var $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0.0, $277 = 0.0, $278 = 0.0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0.0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0;
 var $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0.0;
 var $305 = 0.0, $306 = 0.0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0.0, $313 = 0.0, $314 = 0.0, $315 = 0.0, $316 = 0.0, $317 = 0.0, $318 = 0, $319 = 0, $32 = 0, $320 = 0.0, $321 = 0, $322 = 0;
 var $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0;
 var $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0;
 var $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0;
 var $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0;
 var $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0;
 var $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0.0, $418 = 0.0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0;
 var $431 = 0, $432 = 0, $433 = 0, $434 = 0.0, $435 = 0.0, $436 = 0.0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0.0, $447 = 0.0, $448 = 0.0, $449 = 0;
 var $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0.0, $459 = 0.0, $46 = 0, $460 = 0.0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0;
 var $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0.0, $473 = 0, $474 = 0.0, $475 = 0.0, $476 = 0, $477 = 0.0, $478 = 0, $479 = 0.0, $48 = 0, $480 = 0.0, $481 = 0, $482 = 0, $483 = 0, $484 = 0.0, $485 = 0.0;
 var $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0.0, $490 = 0, $491 = 0.0, $492 = 0.0, $493 = 0.0, $494 = 0, $495 = 0, $496 = 0, $497 = 0.0, $498 = 0.0, $499 = 0, $5 = 0, $50 = 0.0, $500 = 0, $501 = 0, $502 = 0;
 var $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0.0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0;
 var $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0;
 var $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0;
 var $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0;
 var $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0;
 var $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0;
 var $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0;
 var $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0.0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0;
 var $648 = 0.0, $649 = 0.0, $65 = 0, $650 = 0.0, $651 = 0, $652 = 0.0, $653 = 0.0, $654 = 0.0, $655 = 0.0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0;
 var $666 = 0.0, $667 = 0.0, $668 = 0.0, $669 = 0, $67 = 0, $670 = 0.0, $671 = 0.0, $672 = 0, $673 = 0, $674 = 0, $675 = 0.0, $676 = 0.0, $677 = 0.0, $678 = 0.0, $679 = 0, $68 = 0, $680 = 0, $681 = 0.0, $682 = 0, $683 = 0.0;
 var $684 = 0.0, $685 = 0.0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0.0, $691 = 0, $692 = 0, $693 = 0, $694 = 0.0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0;
 var $701 = 0.0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0.0, $707 = 0, $708 = 0, $709 = 0.0, $71 = 0, $710 = 0.0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0;
 var $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a$0$lcssa161$i = 0, $a$061$i = 0, $a$1$i = 0, $a$2$ph40$i = 0;
 var $a$3$i = 0, $a$3$i$ph = 0, $a$3$ph$i = 0, $a$427$i = 0, $a$5$i = 0, $bias$0$i = 0.0, $bias$07$i = 0.0, $bits$0$ph = 0, $brmerge = 0, $brmerge107 = 0, $c$0 = 0, $c$0$i = 0, $c$1$lcssa = 0, $c$1$ph$i = 0, $c$169 = 0, $c$2 = 0, $c$2$i = 0, $c$2$lcssa$i = 0, $c$364 = 0, $c$4 = 0;
 var $c$5 = 0, $c$6 = 0, $carry$063$i = 0, $carry1$0$us$i = 0, $carry1$0$us49$i = 0, $carry1$1$lcssa$lcssa$i = 0, $carry1$1$us$i = 0, $carry1$1$us54$i = 0, $carry3$030$i = 0, $cond$i = 0, $d$0$i = 0, $denormal$0$i = 0, $denormal$1$i = 0, $denormal$2$i = 0, $e2$0$ph$i = 0, $e2$0$us$i = 0, $e2$0$us44$i = 0, $e2$1$i = 0, $e2$1$i$ph = 0, $e2$1$ph$i = 0;
 var $e2$2$i = 0, $e2$3$i = 0, $emin$0$ph = 0, $exitcond$i = 0, $frac$0$i = 0.0, $frac$1$i = 0.0, $frac$2$i = 0.0, $gotdig$0$i = 0, $gotdig$0$i9 = 0, $gotdig$2$i = 0, $gotdig$2$i11 = 0, $gotdig$3$i = 0, $gotdig$3$lcssa$i = 0, $gotdig$381$i = 0, $gotdig$393$i = 0, $gotdig$4$i = 0, $gotrad$0$i = 0, $gotrad$0$i12 = 0, $gotrad$1$i = 0, $gotrad$1$lcssa$i = 0;
 var $gotrad$194$i = 0, $gotrad$2$i = 0, $gottail$0$i = 0, $gottail$1$i = 0, $gottail$2$i = 0, $i$0$lcssa = 0, $i$025$i = 0, $i$068 = 0, $i$1$i = 0, $i$166 = 0, $i$263 = 0, $i$3 = 0, $i$4 = 0, $j$0$lcssa$i = 0, $j$086$i = 0, $j$096$i = 0, $j$2$i = 0, $j$371$i = 0, $k$0$lcssa$i = 0, $k$084$i = 0;
 var $k$095$i = 0, $k$2$i = 0, $k$3$i = 0, $k$462$i = 0, $k$5$in$us$i = 0, $k$5$in$us48$i = 0, $k$5$us$i = 0, $k$5$us50$i = 0, $k$5$z$2$us$i = 0, $k$5$z$2$us56$i = 0, $k$628$i = 0, $lnz$0$lcssa$i = 0, $lnz$079$i = 0, $lnz$092$i = 0, $lnz$2$i = 0, $notlhs = 0, $notrhs = 0, $or$cond = 0, $or$cond$i = 0, $or$cond$i15 = 0;
 var $or$cond106$not = 0, $or$cond11$i = 0, $or$cond12$us$i = 0, $or$cond12$us55$i = 0, $or$cond13$i = 0, $or$cond15$i = 0, $or$cond16$i = 0, $or$cond17$i = 0, $or$cond3$i = 0, $or$cond3$i16 = 0, $or$cond4$i = 0, $or$cond5 = 0, $or$cond7$i = 0, $or$cond789$i = 0, $or$cond8 = 0, $rp$0$lcssa162$i = 0, $rp$060$i = 0, $rp$1$i18 = 0, $rp$2$ph38$i = 0, $rp$3$i$ph = 0;
 var $rp$3$ph33$i = 0, $rp$426$i = 0, $rp$5$i = 0, $scale$0$i = 0.0, $scale$1$i = 0.0, $scale$2$i = 0.0, $sign$0 = 0, $storemerge$i = 0, $sum$i = 0, $x$0$i = 0, $x$1$i = 0, $x$2$i = 0, $x$3$lcssa$i = 0, $x$313$i = 0, $x$4$lcssa$i = 0, $x$48$i = 0, $x$5$i = 0, $x$6$i = 0, $x$i = 0, $y$0$i = 0.0;
 var $y$1$i = 0.0, $y$1$i23 = 0.0, $y$2$i = 0.0, $y$2$i24 = 0.0, $y$3$i = 0.0, $y$3$lcssa$i = 0.0, $y$39$i = 0.0, $y$4$i = 0.0, $y$5$i = 0.0, $z$0$i = 0, $z$1$ph39$i = 0, $z$1$us$i = 0, $z$1$us45$i = 0, $z$2$us$i = 0, $z$2$us47$i = 0, $z$3$lcssa$lcssa$i = 0, $z$3$us$i = 0, $z$3$us57$i = 0, $z$4$i = 0, $z$5$ph$i = 0;
 var $z$7$1$i = 0, $z$7$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 512|0;
 $x$i = sp;
 if ((($prec|0) == 1)) {
  $bits$0$ph = 53;$emin$0$ph = -1074;
 } else if ((($prec|0) == 2)) {
  $bits$0$ph = 53;$emin$0$ph = -1074;
 } else if ((($prec|0) == 0)) {
  $bits$0$ph = 24;$emin$0$ph = -149;
 } else {
  $$0 = 0.0;
  STACKTOP = sp;return (+$$0);
 }
 $0 = (($f) + 4|0);
 $1 = (($f) + 100|0);
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = ($2>>>0)<($3>>>0);
  if ($4) {
   $5 = (($2) + 1|0);
   HEAP32[$0>>2] = $5;
   $6 = HEAP8[$2>>0]|0;
   $7 = $6&255;
   $9 = $7;
  } else {
   $8 = (___shgetc($f)|0);
   $9 = $8;
  }
  $10 = (_isspace($9)|0);
  $11 = ($10|0)==(0);
  if ($11) {
   break;
  }
 }
 $12 = ($9|0)==(45);
 do {
  if ((($9|0) == 43) | (($9|0) == 45)) {
   $13 = $12&1;
   $14 = $13 << 1;
   $15 = (1 - ($14))|0;
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = ($16>>>0)<($17>>>0);
   if ($18) {
    $19 = (($16) + 1|0);
    HEAP32[$0>>2] = $19;
    $20 = HEAP8[$16>>0]|0;
    $21 = $20&255;
    $c$0 = $21;$sign$0 = $15;
    break;
   } else {
    $22 = (___shgetc($f)|0);
    $c$0 = $22;$sign$0 = $15;
    break;
   }
  } else {
   $c$0 = $9;$sign$0 = 1;
  }
 } while(0);
 $c$169 = $c$0;$i$068 = 0;
 while(1) {
  $23 = $c$169 | 32;
  $24 = (14848 + ($i$068)|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25 << 24 >> 24;
  $27 = ($23|0)==($26|0);
  if (!($27)) {
   $c$1$lcssa = $c$169;$i$0$lcssa = $i$068;
   break;
  }
  $28 = ($i$068>>>0)<(7);
  do {
   if ($28) {
    $29 = HEAP32[$0>>2]|0;
    $30 = HEAP32[$1>>2]|0;
    $31 = ($29>>>0)<($30>>>0);
    if ($31) {
     $32 = (($29) + 1|0);
     HEAP32[$0>>2] = $32;
     $33 = HEAP8[$29>>0]|0;
     $34 = $33&255;
     $c$2 = $34;
     break;
    } else {
     $35 = (___shgetc($f)|0);
     $c$2 = $35;
     break;
    }
   } else {
    $c$2 = $c$169;
   }
  } while(0);
  $36 = (($i$068) + 1)|0;
  $37 = ($36>>>0)<(8);
  if ($37) {
   $c$169 = $c$2;$i$068 = $36;
  } else {
   $c$1$lcssa = $c$2;$i$0$lcssa = $36;
   break;
  }
 }
 do {
  if ((($i$0$lcssa|0) == 3)) {
   label = 23;
  } else if (!((($i$0$lcssa|0) == 8))) {
   $38 = ($i$0$lcssa>>>0)<(4);
   $39 = ($pok|0)==(0);
   $or$cond = $38 | $39;
   if (!($or$cond)) {
    $40 = ($i$0$lcssa|0)==(8);
    if ($40) {
     break;
    } else {
     label = 23;
     break;
    }
   }
   $52 = ($i$0$lcssa|0)==(0);
   L34: do {
    if ($52) {
     $c$364 = $c$1$lcssa;$i$263 = 0;
     while(1) {
      $53 = $c$364 | 32;
      $54 = (14864 + ($i$263)|0);
      $55 = HEAP8[$54>>0]|0;
      $56 = $55 << 24 >> 24;
      $57 = ($53|0)==($56|0);
      if (!($57)) {
       $c$5 = $c$364;$i$3 = $i$263;
       break L34;
      }
      $58 = ($i$263>>>0)<(2);
      do {
       if ($58) {
        $59 = HEAP32[$0>>2]|0;
        $60 = HEAP32[$1>>2]|0;
        $61 = ($59>>>0)<($60>>>0);
        if ($61) {
         $62 = (($59) + 1|0);
         HEAP32[$0>>2] = $62;
         $63 = HEAP8[$59>>0]|0;
         $64 = $63&255;
         $c$4 = $64;
         break;
        } else {
         $65 = (___shgetc($f)|0);
         $c$4 = $65;
         break;
        }
       } else {
        $c$4 = $c$364;
       }
      } while(0);
      $66 = (($i$263) + 1)|0;
      $67 = ($66>>>0)<(3);
      if ($67) {
       $c$364 = $c$4;$i$263 = $66;
      } else {
       $c$5 = $c$4;$i$3 = $66;
       break;
      }
     }
    } else {
     $c$5 = $c$1$lcssa;$i$3 = $i$0$lcssa;
    }
   } while(0);
   if ((($i$3|0) == 0)) {
    $113 = ($c$5|0)==(48);
    do {
     if ($113) {
      $114 = HEAP32[$0>>2]|0;
      $115 = HEAP32[$1>>2]|0;
      $116 = ($114>>>0)<($115>>>0);
      if ($116) {
       $117 = (($114) + 1|0);
       HEAP32[$0>>2] = $117;
       $118 = HEAP8[$114>>0]|0;
       $119 = $118&255;
       $122 = $119;
      } else {
       $120 = (___shgetc($f)|0);
       $122 = $120;
      }
      $121 = $122 | 32;
      $123 = ($121|0)==(120);
      if (!($123)) {
       $321 = HEAP32[$1>>2]|0;
       $322 = ($321|0)==(0|0);
       if ($322) {
        $c$6 = 48;
        break;
       }
       $323 = HEAP32[$0>>2]|0;
       $324 = (($323) + -1|0);
       HEAP32[$0>>2] = $324;
       $c$6 = 48;
       break;
      }
      $124 = HEAP32[$0>>2]|0;
      $125 = HEAP32[$1>>2]|0;
      $126 = ($124>>>0)<($125>>>0);
      if ($126) {
       $127 = (($124) + 1|0);
       HEAP32[$0>>2] = $127;
       $128 = HEAP8[$124>>0]|0;
       $129 = $128&255;
       $c$0$i = $129;$gotdig$0$i = 0;
      } else {
       $130 = (___shgetc($f)|0);
       $c$0$i = $130;$gotdig$0$i = 0;
      }
      while(1) {
       if ((($c$0$i|0) == 46)) {
        label = 70;
        break;
       } else if (!((($c$0$i|0) == 48))) {
        $169 = 0;$172 = 0;$212 = 0;$214 = 0;$c$2$i = $c$0$i;$gotdig$2$i = $gotdig$0$i;$gotrad$0$i = 0;$gottail$0$i = 0;$scale$0$i = 1.0;$x$0$i = 0;$y$0$i = 0.0;
        break;
       }
       $131 = HEAP32[$0>>2]|0;
       $132 = HEAP32[$1>>2]|0;
       $133 = ($131>>>0)<($132>>>0);
       if ($133) {
        $134 = (($131) + 1|0);
        HEAP32[$0>>2] = $134;
        $135 = HEAP8[$131>>0]|0;
        $136 = $135&255;
        $c$0$i = $136;$gotdig$0$i = 1;
        continue;
       } else {
        $137 = (___shgetc($f)|0);
        $c$0$i = $137;$gotdig$0$i = 1;
        continue;
       }
      }
      L66: do {
       if ((label|0) == 70) {
        $138 = HEAP32[$0>>2]|0;
        $139 = HEAP32[$1>>2]|0;
        $140 = ($138>>>0)<($139>>>0);
        if ($140) {
         $141 = (($138) + 1|0);
         HEAP32[$0>>2] = $141;
         $142 = HEAP8[$138>>0]|0;
         $143 = $142&255;
         $c$1$ph$i = $143;
        } else {
         $144 = (___shgetc($f)|0);
         $c$1$ph$i = $144;
        }
        $145 = ($c$1$ph$i|0)==(48);
        if ($145) {
         $155 = -1;$156 = -1;
         while(1) {
          $146 = HEAP32[$0>>2]|0;
          $147 = HEAP32[$1>>2]|0;
          $148 = ($146>>>0)<($147>>>0);
          if ($148) {
           $149 = (($146) + 1|0);
           HEAP32[$0>>2] = $149;
           $150 = HEAP8[$146>>0]|0;
           $151 = $150&255;
           $154 = $151;
          } else {
           $152 = (___shgetc($f)|0);
           $154 = $152;
          }
          $153 = ($154|0)==(48);
          if (!($153)) {
           $169 = 0;$172 = 0;$212 = $155;$214 = $156;$c$2$i = $154;$gotdig$2$i = 1;$gotrad$0$i = 1;$gottail$0$i = 0;$scale$0$i = 1.0;$x$0$i = 0;$y$0$i = 0.0;
           break L66;
          }
          $157 = (_i64Add(($155|0),($156|0),-1,-1)|0);
          $158 = tempRet0;
          $155 = $157;$156 = $158;
         }
        } else {
         $169 = 0;$172 = 0;$212 = 0;$214 = 0;$c$2$i = $c$1$ph$i;$gotdig$2$i = $gotdig$0$i;$gotrad$0$i = 1;$gottail$0$i = 0;$scale$0$i = 1.0;$x$0$i = 0;$y$0$i = 0.0;
        }
       }
      } while(0);
      L79: while(1) {
       $159 = (($c$2$i) + -48)|0;
       $160 = ($159>>>0)<(10);
       do {
        if ($160) {
         $d$0$i = $159;
         label = 84;
        } else {
         $161 = $c$2$i | 32;
         $162 = (($161) + -97)|0;
         $163 = ($162>>>0)<(6);
         $164 = ($c$2$i|0)==(46);
         $or$cond$i = $163 | $164;
         if (!($or$cond$i)) {
          $c$2$lcssa$i = $c$2$i;
          break L79;
         }
         if ($164) {
          $165 = ($gotrad$0$i|0)==(0);
          if ($165) {
           $712 = $172;$713 = $169;$714 = $172;$715 = $169;$gotdig$3$i = $gotdig$2$i;$gotrad$1$i = 1;$gottail$2$i = $gottail$0$i;$scale$2$i = $scale$0$i;$x$2$i = $x$0$i;$y$2$i = $y$0$i;
           break;
          } else {
           $c$2$lcssa$i = 46;
           break L79;
          }
         } else {
          $166 = ($c$2$i|0)>(57);
          $167 = (($161) + -87)|0;
          $$38$i = $166 ? $167 : $159;
          $d$0$i = $$38$i;
          label = 84;
          break;
         }
        }
       } while(0);
       if ((label|0) == 84) {
        label = 0;
        $168 = ($169|0)<(0);
        $170 = ($169|0)==(0);
        $171 = ($172>>>0)<(8);
        $173 = $170 & $171;
        $174 = $168 | $173;
        do {
         if ($174) {
          $175 = $x$0$i << 4;
          $176 = (($d$0$i) + ($175))|0;
          $gottail$1$i = $gottail$0$i;$scale$1$i = $scale$0$i;$x$1$i = $176;$y$1$i = $y$0$i;
         } else {
          $177 = ($169|0)<(0);
          $178 = ($169|0)==(0);
          $179 = ($172>>>0)<(14);
          $180 = $178 & $179;
          $181 = $177 | $180;
          if ($181) {
           $182 = (+($d$0$i|0));
           $183 = $scale$0$i * 0.0625;
           $184 = $183 * $182;
           $185 = $y$0$i + $184;
           $gottail$1$i = $gottail$0$i;$scale$1$i = $183;$x$1$i = $x$0$i;$y$1$i = $185;
           break;
          }
          $186 = ($d$0$i|0)!=(0);
          $187 = ($gottail$0$i|0)==(0);
          $or$cond3$i = $186 & $187;
          if ($or$cond3$i) {
           $188 = $scale$0$i * 0.5;
           $189 = $y$0$i + $188;
           $gottail$1$i = 1;$scale$1$i = $scale$0$i;$x$1$i = $x$0$i;$y$1$i = $189;
          } else {
           $gottail$1$i = $gottail$0$i;$scale$1$i = $scale$0$i;$x$1$i = $x$0$i;$y$1$i = $y$0$i;
          }
         }
        } while(0);
        $190 = (_i64Add(($172|0),($169|0),1,0)|0);
        $191 = tempRet0;
        $712 = $212;$713 = $214;$714 = $190;$715 = $191;$gotdig$3$i = 1;$gotrad$1$i = $gotrad$0$i;$gottail$2$i = $gottail$1$i;$scale$2$i = $scale$1$i;$x$2$i = $x$1$i;$y$2$i = $y$1$i;
       }
       $192 = HEAP32[$0>>2]|0;
       $193 = HEAP32[$1>>2]|0;
       $194 = ($192>>>0)<($193>>>0);
       if ($194) {
        $195 = (($192) + 1|0);
        HEAP32[$0>>2] = $195;
        $196 = HEAP8[$192>>0]|0;
        $197 = $196&255;
        $169 = $715;$172 = $714;$212 = $712;$214 = $713;$c$2$i = $197;$gotdig$2$i = $gotdig$3$i;$gotrad$0$i = $gotrad$1$i;$gottail$0$i = $gottail$2$i;$scale$0$i = $scale$2$i;$x$0$i = $x$2$i;$y$0$i = $y$2$i;
        continue;
       } else {
        $198 = (___shgetc($f)|0);
        $169 = $715;$172 = $714;$212 = $712;$214 = $713;$c$2$i = $198;$gotdig$2$i = $gotdig$3$i;$gotrad$0$i = $gotrad$1$i;$gottail$0$i = $gottail$2$i;$scale$0$i = $scale$2$i;$x$0$i = $x$2$i;$y$0$i = $y$2$i;
        continue;
       }
      }
      $199 = ($gotdig$2$i|0)==(0);
      if ($199) {
       $200 = HEAP32[$1>>2]|0;
       $201 = ($200|0)==(0|0);
       if (!($201)) {
        $202 = HEAP32[$0>>2]|0;
        $203 = (($202) + -1|0);
        HEAP32[$0>>2] = $203;
       }
       if ($39) {
        ___shlim($f,0);
       } else {
        if (!($201)) {
         $204 = HEAP32[$0>>2]|0;
         $205 = (($204) + -1|0);
         HEAP32[$0>>2] = $205;
         $206 = ($gotrad$0$i|0)==(0);
         if (!($206)) {
          $207 = (($204) + -2|0);
          HEAP32[$0>>2] = $207;
         }
        }
       }
       $208 = (+($sign$0|0));
       $209 = $208 * 0.0;
       $$0 = $209;
       STACKTOP = sp;return (+$$0);
      }
      $210 = ($gotrad$0$i|0)==(0);
      $211 = $210 ? $172 : $212;
      $213 = $210 ? $169 : $214;
      $215 = ($169|0)<(0);
      $216 = ($169|0)==(0);
      $217 = ($172>>>0)<(8);
      $218 = $216 & $217;
      $219 = $215 | $218;
      if ($219) {
       $221 = $172;$222 = $169;$x$313$i = $x$0$i;
       while(1) {
        $220 = $x$313$i << 4;
        $223 = (_i64Add(($221|0),($222|0),1,0)|0);
        $224 = tempRet0;
        $225 = ($224|0)<(0);
        $226 = ($224|0)==(0);
        $227 = ($223>>>0)<(8);
        $228 = $226 & $227;
        $229 = $225 | $228;
        if ($229) {
         $221 = $223;$222 = $224;$x$313$i = $220;
        } else {
         $x$3$lcssa$i = $220;
         break;
        }
       }
      } else {
       $x$3$lcssa$i = $x$0$i;
      }
      $230 = $c$2$lcssa$i | 32;
      $231 = ($230|0)==(112);
      do {
       if ($231) {
        $232 = (_scanexp($f,$pok)|0);
        $233 = tempRet0;
        $234 = ($232|0)==(0);
        $235 = ($233|0)==(-2147483648);
        $236 = $234 & $235;
        if ($236) {
         if ($39) {
          ___shlim($f,0);
          $$0 = 0.0;
          STACKTOP = sp;return (+$$0);
         } else {
          $237 = HEAP32[$1>>2]|0;
          $238 = ($237|0)==(0|0);
          if ($238) {
           $249 = 0;$250 = 0;
           break;
          }
          $239 = HEAP32[$0>>2]|0;
          $240 = (($239) + -1|0);
          HEAP32[$0>>2] = $240;
          $249 = 0;$250 = 0;
          break;
         }
        } else {
         $249 = $232;$250 = $233;
        }
       } else {
        $241 = HEAP32[$1>>2]|0;
        $242 = ($241|0)==(0|0);
        if ($242) {
         $249 = 0;$250 = 0;
        } else {
         $243 = HEAP32[$0>>2]|0;
         $244 = (($243) + -1|0);
         HEAP32[$0>>2] = $244;
         $249 = 0;$250 = 0;
        }
       }
      } while(0);
      $245 = (_bitshift64Shl(($211|0),($213|0),2)|0);
      $246 = tempRet0;
      $247 = (_i64Add(($245|0),($246|0),-32,-1)|0);
      $248 = tempRet0;
      $251 = (_i64Add(($247|0),($248|0),($249|0),($250|0))|0);
      $252 = tempRet0;
      $253 = ($x$3$lcssa$i|0)==(0);
      if ($253) {
       $254 = (+($sign$0|0));
       $255 = $254 * 0.0;
       $$0 = $255;
       STACKTOP = sp;return (+$$0);
      }
      $256 = (0 - ($emin$0$ph))|0;
      $257 = ($252|0)>(0);
      $258 = ($252|0)==(0);
      $259 = ($251>>>0)>($256>>>0);
      $260 = $258 & $259;
      $261 = $257 | $260;
      if ($261) {
       $262 = (___errno_location()|0);
       HEAP32[$262>>2] = 34;
       $263 = (+($sign$0|0));
       $264 = $263 * 1.79769313486231570815E+308;
       $265 = $264 * 1.79769313486231570815E+308;
       $$0 = $265;
       STACKTOP = sp;return (+$$0);
      }
      $266 = (($emin$0$ph) + -106)|0;
      $267 = ($266|0)<(0);
      $268 = $267 << 31 >> 31;
      $269 = ($252|0)<($268|0);
      $270 = ($252|0)==($268|0);
      $271 = ($251>>>0)<($266>>>0);
      $272 = $270 & $271;
      $273 = $269 | $272;
      if ($273) {
       $275 = (___errno_location()|0);
       HEAP32[$275>>2] = 34;
       $276 = (+($sign$0|0));
       $277 = $276 * 2.22507385850720138309E-308;
       $278 = $277 * 2.22507385850720138309E-308;
       $$0 = $278;
       STACKTOP = sp;return (+$$0);
      }
      $274 = ($x$3$lcssa$i|0)>(-1);
      if ($274) {
       $283 = $251;$284 = $252;$x$48$i = $x$3$lcssa$i;$y$39$i = $y$0$i;
       while(1) {
        $279 = !($y$39$i >= 0.5);
        $280 = $x$48$i << 1;
        if ($279) {
         $$pn$i = $y$39$i;$x$5$i = $280;
        } else {
         $281 = $280 | 1;
         $282 = $y$39$i + -1.0;
         $$pn$i = $282;$x$5$i = $281;
        }
        $y$4$i = $y$39$i + $$pn$i;
        $285 = (_i64Add(($283|0),($284|0),-1,-1)|0);
        $286 = tempRet0;
        $287 = ($x$5$i|0)>(-1);
        if ($287) {
         $283 = $285;$284 = $286;$x$48$i = $x$5$i;$y$39$i = $y$4$i;
        } else {
         $292 = $285;$293 = $286;$x$4$lcssa$i = $x$5$i;$y$3$lcssa$i = $y$4$i;
         break;
        }
       }
      } else {
       $292 = $251;$293 = $252;$x$4$lcssa$i = $x$3$lcssa$i;$y$3$lcssa$i = $y$0$i;
      }
      $288 = ($emin$0$ph|0)<(0);
      $289 = $288 << 31 >> 31;
      $290 = (_i64Subtract(32,0,($emin$0$ph|0),($289|0))|0);
      $291 = tempRet0;
      $294 = (_i64Add(($292|0),($293|0),($290|0),($291|0))|0);
      $295 = tempRet0;
      $296 = (0)>($295|0);
      $297 = (0)==($295|0);
      $298 = ($bits$0$ph>>>0)>($294>>>0);
      $299 = $297 & $298;
      $300 = $296 | $299;
      if ($300) {
       $301 = ($294|0)<(0);
       $$$i = $301 ? 0 : $294;
       $$02$i = $$$i;
      } else {
       $$02$i = $bits$0$ph;
      }
      $302 = ($$02$i|0)<(53);
      do {
       if ($302) {
        $303 = (84 - ($$02$i))|0;
        $304 = (+_scalbn(1.0,$303));
        $305 = (+($sign$0|0));
        $306 = (+_copysignl((+$304),(+$305)));
        $307 = ($$02$i|0)<(32);
        $308 = $y$3$lcssa$i != 0.0;
        $or$cond4$i = $307 & $308;
        if (!($or$cond4$i)) {
         $$pre$phi$iZ2D = $305;$bias$07$i = $306;$x$6$i = $x$4$lcssa$i;$y$5$i = $y$3$lcssa$i;
         break;
        }
        $309 = $x$4$lcssa$i & 1;
        $310 = ($309|0)==(0);
        $311 = $309 ^ 1;
        $$x$4$i = (($311) + ($x$4$lcssa$i))|0;
        $$y$3$i = $310 ? 0.0 : $y$3$lcssa$i;
        $$pre$phi$iZ2D = $305;$bias$07$i = $306;$x$6$i = $$x$4$i;$y$5$i = $$y$3$i;
       } else {
        $$pre$i = (+($sign$0|0));
        $$pre$phi$iZ2D = $$pre$i;$bias$07$i = 0.0;$x$6$i = $x$4$lcssa$i;$y$5$i = $y$3$lcssa$i;
       }
      } while(0);
      $312 = (+($x$6$i>>>0));
      $313 = $$pre$phi$iZ2D * $312;
      $314 = $bias$07$i + $313;
      $315 = $$pre$phi$iZ2D * $y$5$i;
      $316 = $315 + $314;
      $317 = $316 - $bias$07$i;
      $318 = $317 != 0.0;
      if (!($318)) {
       $319 = (___errno_location()|0);
       HEAP32[$319>>2] = 34;
      }
      $320 = (+_scalbnl($317,$292));
      $$0 = $320;
      STACKTOP = sp;return (+$$0);
     } else {
      $c$6 = $c$5;
     }
    } while(0);
    $sum$i = (($emin$0$ph) + ($bits$0$ph))|0;
    $325 = (0 - ($sum$i))|0;
    $$08$i = $c$6;$gotdig$0$i9 = 0;
    while(1) {
     if ((($$08$i|0) == 46)) {
      label = 139;
      break;
     } else if (!((($$08$i|0) == 48))) {
      $$2$i = $$08$i;$716 = 0;$717 = 0;$gotdig$2$i11 = $gotdig$0$i9;$gotrad$0$i12 = 0;
      break;
     }
     $326 = HEAP32[$0>>2]|0;
     $327 = HEAP32[$1>>2]|0;
     $328 = ($326>>>0)<($327>>>0);
     if ($328) {
      $329 = (($326) + 1|0);
      HEAP32[$0>>2] = $329;
      $330 = HEAP8[$326>>0]|0;
      $331 = $330&255;
      $$08$i = $331;$gotdig$0$i9 = 1;
      continue;
     } else {
      $332 = (___shgetc($f)|0);
      $$08$i = $332;$gotdig$0$i9 = 1;
      continue;
     }
    }
    L168: do {
     if ((label|0) == 139) {
      $333 = HEAP32[$0>>2]|0;
      $334 = HEAP32[$1>>2]|0;
      $335 = ($333>>>0)<($334>>>0);
      if ($335) {
       $336 = (($333) + 1|0);
       HEAP32[$0>>2] = $336;
       $337 = HEAP8[$333>>0]|0;
       $338 = $337&255;
       $$1$ph$i = $338;
      } else {
       $339 = (___shgetc($f)|0);
       $$1$ph$i = $339;
      }
      $340 = ($$1$ph$i|0)==(48);
      if ($340) {
       $349 = -1;$350 = -1;
       while(1) {
        $341 = HEAP32[$0>>2]|0;
        $342 = HEAP32[$1>>2]|0;
        $343 = ($341>>>0)<($342>>>0);
        if ($343) {
         $344 = (($341) + 1|0);
         HEAP32[$0>>2] = $344;
         $345 = HEAP8[$341>>0]|0;
         $346 = $345&255;
         $$1$be$i = $346;
        } else {
         $347 = (___shgetc($f)|0);
         $$1$be$i = $347;
        }
        $348 = ($$1$be$i|0)==(48);
        if (!($348)) {
         $$2$i = $$1$be$i;$716 = $349;$717 = $350;$gotdig$2$i11 = 1;$gotrad$0$i12 = 1;
         break L168;
        }
        $351 = (_i64Add(($349|0),($350|0),-1,-1)|0);
        $352 = tempRet0;
        $349 = $351;$350 = $352;
       }
      } else {
       $$2$i = $$1$ph$i;$716 = 0;$717 = 0;$gotdig$2$i11 = $gotdig$0$i9;$gotrad$0$i12 = 1;
      }
     }
    } while(0);
    HEAP32[$x$i>>2] = 0;
    $353 = (($$2$i) + -48)|0;
    $354 = ($353>>>0)<(10);
    $355 = ($$2$i|0)==(46);
    $or$cond789$i = $354 | $355;
    L182: do {
     if ($or$cond789$i) {
      $356 = (($x$i) + 496|0);
      $$397$i = $$2$i;$358 = 0;$359 = 0;$718 = $355;$719 = $353;$720 = $716;$721 = $717;$gotdig$393$i = $gotdig$2$i11;$gotrad$194$i = $gotrad$0$i12;$j$096$i = 0;$k$095$i = 0;$lnz$092$i = 0;
      while(1) {
       do {
        if ($718) {
         $cond$i = ($gotrad$194$i|0)==(0);
         if ($cond$i) {
          $722 = $358;$723 = $359;$724 = $358;$725 = $359;$gotdig$4$i = $gotdig$393$i;$gotrad$2$i = 1;$j$2$i = $j$096$i;$k$2$i = $k$095$i;$lnz$2$i = $lnz$092$i;
         } else {
          $$388$i = $$397$i;$405 = $720;$406 = $721;$421 = $358;$424 = $359;$gotdig$381$i = $gotdig$393$i;$j$086$i = $j$096$i;$k$084$i = $k$095$i;$lnz$079$i = $lnz$092$i;
          break L182;
         }
        } else {
         $357 = ($k$095$i|0)<(125);
         $360 = (_i64Add(($358|0),($359|0),1,0)|0);
         $361 = tempRet0;
         $362 = ($$397$i|0)!=(48);
         if (!($357)) {
          if (!($362)) {
           $722 = $720;$723 = $721;$724 = $360;$725 = $361;$gotdig$4$i = $gotdig$393$i;$gotrad$2$i = $gotrad$194$i;$j$2$i = $j$096$i;$k$2$i = $k$095$i;$lnz$2$i = $lnz$092$i;
           break;
          }
          $372 = HEAP32[$356>>2]|0;
          $373 = $372 | 1;
          HEAP32[$356>>2] = $373;
          $722 = $720;$723 = $721;$724 = $360;$725 = $361;$gotdig$4$i = $gotdig$393$i;$gotrad$2$i = $gotrad$194$i;$j$2$i = $j$096$i;$k$2$i = $k$095$i;$lnz$2$i = $lnz$092$i;
          break;
         }
         $$lnz$0$i = $362 ? $360 : $lnz$092$i;
         $363 = ($j$096$i|0)==(0);
         $364 = (($x$i) + ($k$095$i<<2)|0);
         if ($363) {
          $storemerge$i = $719;
         } else {
          $365 = HEAP32[$364>>2]|0;
          $366 = ($365*10)|0;
          $367 = (($$397$i) + -48)|0;
          $368 = (($367) + ($366))|0;
          $storemerge$i = $368;
         }
         HEAP32[$364>>2] = $storemerge$i;
         $369 = (($j$096$i) + 1)|0;
         $370 = ($369|0)==(9);
         $371 = $370&1;
         $$k$0$i = (($371) + ($k$095$i))|0;
         $$10$i = $370 ? 0 : $369;
         $722 = $720;$723 = $721;$724 = $360;$725 = $361;$gotdig$4$i = 1;$gotrad$2$i = $gotrad$194$i;$j$2$i = $$10$i;$k$2$i = $$k$0$i;$lnz$2$i = $$lnz$0$i;
        }
       } while(0);
       $374 = HEAP32[$0>>2]|0;
       $375 = HEAP32[$1>>2]|0;
       $376 = ($374>>>0)<($375>>>0);
       if ($376) {
        $377 = (($374) + 1|0);
        HEAP32[$0>>2] = $377;
        $378 = HEAP8[$374>>0]|0;
        $379 = $378&255;
        $$3$be$i = $379;
       } else {
        $380 = (___shgetc($f)|0);
        $$3$be$i = $380;
       }
       $381 = (($$3$be$i) + -48)|0;
       $382 = ($381>>>0)<(10);
       $383 = ($$3$be$i|0)==(46);
       $or$cond7$i = $382 | $383;
       if ($or$cond7$i) {
        $$397$i = $$3$be$i;$358 = $724;$359 = $725;$718 = $383;$719 = $381;$720 = $722;$721 = $723;$gotdig$393$i = $gotdig$4$i;$gotrad$194$i = $gotrad$2$i;$j$096$i = $j$2$i;$k$095$i = $k$2$i;$lnz$092$i = $lnz$2$i;
       } else {
        $$3$lcssa$i = $$3$be$i;$386 = $724;$387 = $722;$389 = $725;$390 = $723;$gotdig$3$lcssa$i = $gotdig$4$i;$gotrad$1$lcssa$i = $gotrad$2$i;$j$0$lcssa$i = $j$2$i;$k$0$lcssa$i = $k$2$i;$lnz$0$lcssa$i = $lnz$2$i;
        label = 162;
        break;
       }
      }
     } else {
      $$3$lcssa$i = $$2$i;$386 = 0;$387 = $716;$389 = 0;$390 = $717;$gotdig$3$lcssa$i = $gotdig$2$i11;$gotrad$1$lcssa$i = $gotrad$0$i12;$j$0$lcssa$i = 0;$k$0$lcssa$i = 0;$lnz$0$lcssa$i = 0;
      label = 162;
     }
    } while(0);
    if ((label|0) == 162) {
     $384 = ($gotrad$1$lcssa$i|0)==(0);
     $385 = $384 ? $386 : $387;
     $388 = $384 ? $389 : $390;
     $$388$i = $$3$lcssa$i;$405 = $385;$406 = $388;$421 = $386;$424 = $389;$gotdig$381$i = $gotdig$3$lcssa$i;$j$086$i = $j$0$lcssa$i;$k$084$i = $k$0$lcssa$i;$lnz$079$i = $lnz$0$lcssa$i;
    }
    $391 = ($gotdig$381$i|0)!=(0);
    if ($391) {
     $392 = $$388$i | 32;
     $393 = ($392|0)==(101);
     if ($393) {
      $394 = (_scanexp($f,$pok)|0);
      $395 = tempRet0;
      $396 = ($394|0)==(0);
      $397 = ($395|0)==(-2147483648);
      $398 = $396 & $397;
      do {
       if ($398) {
        if ($39) {
         ___shlim($f,0);
         $$0 = 0.0;
         STACKTOP = sp;return (+$$0);
        } else {
         $399 = HEAP32[$1>>2]|0;
         $400 = ($399|0)==(0|0);
         if ($400) {
          $403 = 0;$404 = 0;
          break;
         }
         $401 = HEAP32[$0>>2]|0;
         $402 = (($401) + -1|0);
         HEAP32[$0>>2] = $402;
         $403 = 0;$404 = 0;
         break;
        }
       } else {
        $403 = $394;$404 = $395;
       }
      } while(0);
      $407 = (_i64Add(($403|0),($404|0),($405|0),($406|0))|0);
      $408 = tempRet0;
      $420 = $407;$423 = $408;
     } else {
      label = 171;
     }
    } else {
     label = 171;
    }
    if ((label|0) == 171) {
     $409 = ($$388$i|0)>(-1);
     if ($409) {
      $410 = HEAP32[$1>>2]|0;
      $411 = ($410|0)==(0|0);
      if ($411) {
       $420 = $405;$423 = $406;
      } else {
       $412 = HEAP32[$0>>2]|0;
       $413 = (($412) + -1|0);
       HEAP32[$0>>2] = $413;
       $420 = $405;$423 = $406;
      }
     } else {
      $420 = $405;$423 = $406;
     }
    }
    if (!($391)) {
     $414 = (___errno_location()|0);
     HEAP32[$414>>2] = 22;
     ___shlim($f,0);
     $$0 = 0.0;
     STACKTOP = sp;return (+$$0);
    }
    $415 = HEAP32[$x$i>>2]|0;
    $416 = ($415|0)==(0);
    if ($416) {
     $417 = (+($sign$0|0));
     $418 = $417 * 0.0;
     $$0 = $418;
     STACKTOP = sp;return (+$$0);
    }
    $419 = ($420|0)==($421|0);
    $422 = ($423|0)==($424|0);
    $425 = $419 & $422;
    $426 = ($424|0)<(0);
    $427 = ($424|0)==(0);
    $428 = ($421>>>0)<(10);
    $429 = $427 & $428;
    $430 = $426 | $429;
    $or$cond$i15 = $425 & $430;
    do {
     if ($or$cond$i15) {
      $431 = ($bits$0$ph>>>0)>(30);
      if (!($431)) {
       $432 = $415 >>> $bits$0$ph;
       $433 = ($432|0)==(0);
       if (!($433)) {
        break;
       }
      }
      $434 = (+($sign$0|0));
      $435 = (+($415>>>0));
      $436 = $434 * $435;
      $$0 = $436;
      STACKTOP = sp;return (+$$0);
     }
    } while(0);
    $437 = (($emin$0$ph|0) / -2)&-1;
    $438 = ($437|0)<(0);
    $439 = $438 << 31 >> 31;
    $440 = ($423|0)>($439|0);
    $441 = ($423|0)==($439|0);
    $442 = ($420>>>0)>($437>>>0);
    $443 = $441 & $442;
    $444 = $440 | $443;
    if ($444) {
     $445 = (___errno_location()|0);
     HEAP32[$445>>2] = 34;
     $446 = (+($sign$0|0));
     $447 = $446 * 1.79769313486231570815E+308;
     $448 = $447 * 1.79769313486231570815E+308;
     $$0 = $448;
     STACKTOP = sp;return (+$$0);
    }
    $449 = (($emin$0$ph) + -106)|0;
    $450 = ($449|0)<(0);
    $451 = $450 << 31 >> 31;
    $452 = ($423|0)<($451|0);
    $453 = ($423|0)==($451|0);
    $454 = ($420>>>0)<($449>>>0);
    $455 = $453 & $454;
    $456 = $452 | $455;
    if ($456) {
     $457 = (___errno_location()|0);
     HEAP32[$457>>2] = 34;
     $458 = (+($sign$0|0));
     $459 = $458 * 2.22507385850720138309E-308;
     $460 = $459 * 2.22507385850720138309E-308;
     $$0 = $460;
     STACKTOP = sp;return (+$$0);
    }
    $461 = ($j$086$i|0)==(0);
    if ($461) {
     $k$3$i = $k$084$i;
    } else {
     $462 = ($j$086$i|0)<(9);
     if ($462) {
      $463 = (($x$i) + ($k$084$i<<2)|0);
      $$promoted$i = HEAP32[$463>>2]|0;
      $465 = $$promoted$i;$j$371$i = $j$086$i;
      while(1) {
       $464 = ($465*10)|0;
       $466 = (($j$371$i) + 1)|0;
       $exitcond$i = ($466|0)==(9);
       if ($exitcond$i) {
        break;
       } else {
        $465 = $464;$j$371$i = $466;
       }
      }
      HEAP32[$463>>2] = $464;
     }
     $467 = (($k$084$i) + 1)|0;
     $k$3$i = $467;
    }
    $468 = ($lnz$079$i|0)<(9);
    do {
     if ($468) {
      $469 = ($lnz$079$i|0)<=($420|0);
      $470 = ($420|0)<(18);
      $or$cond3$i16 = $469 & $470;
      if ($or$cond3$i16) {
       $471 = ($420|0)==(9);
       if ($471) {
        $472 = (+($sign$0|0));
        $473 = HEAP32[$x$i>>2]|0;
        $474 = (+($473>>>0));
        $475 = $472 * $474;
        $$0 = $475;
        STACKTOP = sp;return (+$$0);
       }
       $476 = ($420|0)<(9);
       if ($476) {
        $477 = (+($sign$0|0));
        $478 = HEAP32[$x$i>>2]|0;
        $479 = (+($478>>>0));
        $480 = $477 * $479;
        $481 = (8 - ($420))|0;
        $482 = (14880 + ($481<<2)|0);
        $483 = HEAP32[$482>>2]|0;
        $484 = (+($483|0));
        $485 = $480 / $484;
        $$0 = $485;
        STACKTOP = sp;return (+$$0);
       }
       $486 = Math_imul($420, -3)|0;
       $$neg20$i = (($bits$0$ph) + 27)|0;
       $487 = (($$neg20$i) + ($486))|0;
       $488 = ($487|0)>(30);
       $$pre$i17 = HEAP32[$x$i>>2]|0;
       if (!($488)) {
        $489 = $$pre$i17 >>> $487;
        $490 = ($489|0)==(0);
        if (!($490)) {
         break;
        }
       }
       $491 = (+($sign$0|0));
       $492 = (+($$pre$i17>>>0));
       $493 = $491 * $492;
       $494 = (($420) + -10)|0;
       $495 = (14880 + ($494<<2)|0);
       $496 = HEAP32[$495>>2]|0;
       $497 = (+($496|0));
       $498 = $493 * $497;
       $$0 = $498;
       STACKTOP = sp;return (+$$0);
      }
     }
    } while(0);
    $499 = (($420|0) % 9)&-1;
    $500 = ($499|0)==(0);
    if ($500) {
     $a$2$ph40$i = 0;$e2$0$ph$i = 0;$rp$2$ph38$i = $420;$z$1$ph39$i = $k$3$i;
    } else {
     $501 = ($420|0)>(-1);
     $502 = (($499) + 9)|0;
     $503 = $501 ? $499 : $502;
     $504 = (8 - ($503))|0;
     $505 = (14880 + ($504<<2)|0);
     $506 = HEAP32[$505>>2]|0;
     $507 = ($k$3$i|0)==(0);
     if ($507) {
      $a$0$lcssa161$i = 0;$rp$0$lcssa162$i = $420;$z$0$i = 0;
     } else {
      $508 = (1000000000 / ($506|0))&-1;
      $a$061$i = 0;$carry$063$i = 0;$k$462$i = 0;$rp$060$i = $420;
      while(1) {
       $509 = (($x$i) + ($k$462$i<<2)|0);
       $510 = HEAP32[$509>>2]|0;
       $511 = (($510>>>0) % ($506>>>0))&-1;
       $512 = (($510>>>0) / ($506>>>0))&-1;
       $513 = (($512) + ($carry$063$i))|0;
       HEAP32[$509>>2] = $513;
       $514 = Math_imul($511, $508)|0;
       $515 = ($k$462$i|0)==($a$061$i|0);
       $516 = ($513|0)==(0);
       $or$cond11$i = $515 & $516;
       $517 = (($k$462$i) + 1)|0;
       if ($or$cond11$i) {
        $518 = $517 & 127;
        $519 = (($rp$060$i) + -9)|0;
        $a$1$i = $518;$rp$1$i18 = $519;
       } else {
        $a$1$i = $a$061$i;$rp$1$i18 = $rp$060$i;
       }
       $520 = ($517|0)==($k$3$i|0);
       if ($520) {
        break;
       } else {
        $a$061$i = $a$1$i;$carry$063$i = $514;$k$462$i = $517;$rp$060$i = $rp$1$i18;
       }
      }
      $521 = ($514|0)==(0);
      if ($521) {
       $a$0$lcssa161$i = $a$1$i;$rp$0$lcssa162$i = $rp$1$i18;$z$0$i = $k$3$i;
      } else {
       $522 = (($k$3$i) + 1)|0;
       $523 = (($x$i) + ($k$3$i<<2)|0);
       HEAP32[$523>>2] = $514;
       $a$0$lcssa161$i = $a$1$i;$rp$0$lcssa162$i = $rp$1$i18;$z$0$i = $522;
      }
     }
     $524 = (9 - ($503))|0;
     $525 = (($524) + ($rp$0$lcssa162$i))|0;
     $a$2$ph40$i = $a$0$lcssa161$i;$e2$0$ph$i = 0;$rp$2$ph38$i = $525;$z$1$ph39$i = $z$0$i;
    }
    L280: while(1) {
     $526 = ($rp$2$ph38$i|0)<(18);
     $527 = (($x$i) + ($a$2$ph40$i<<2)|0);
     if ($526) {
      $e2$0$us$i = $e2$0$ph$i;$z$1$us$i = $z$1$ph39$i;
      while(1) {
       $529 = (($z$1$us$i) + 127)|0;
       $carry1$0$us$i = 0;$k$5$in$us$i = $529;$z$2$us$i = $z$1$us$i;
       while(1) {
        $k$5$us$i = $k$5$in$us$i & 127;
        $530 = (($x$i) + ($k$5$us$i<<2)|0);
        $531 = HEAP32[$530>>2]|0;
        $532 = (_bitshift64Shl(($531|0),0,29)|0);
        $533 = tempRet0;
        $534 = (_i64Add(($532|0),($533|0),($carry1$0$us$i|0),0)|0);
        $535 = tempRet0;
        $536 = ($535>>>0)>(0);
        $537 = ($535|0)==(0);
        $538 = ($534>>>0)>(1000000000);
        $539 = $537 & $538;
        $540 = $536 | $539;
        if ($540) {
         $541 = (___udivdi3(($534|0),($535|0),1000000000,0)|0);
         $542 = tempRet0;
         $543 = (___uremdi3(($534|0),($535|0),1000000000,0)|0);
         $544 = tempRet0;
         $$sink$off0$us$i = $543;$carry1$1$us$i = $541;
        } else {
         $$sink$off0$us$i = $534;$carry1$1$us$i = 0;
        }
        HEAP32[$530>>2] = $$sink$off0$us$i;
        $545 = (($z$2$us$i) + 127)|0;
        $546 = $545 & 127;
        $547 = ($k$5$us$i|0)!=($546|0);
        $548 = ($k$5$us$i|0)==($a$2$ph40$i|0);
        $or$cond12$us$i = $547 | $548;
        if ($or$cond12$us$i) {
         $z$3$us$i = $z$2$us$i;
        } else {
         $549 = ($$sink$off0$us$i|0)==(0);
         $k$5$z$2$us$i = $549 ? $k$5$us$i : $z$2$us$i;
         $z$3$us$i = $k$5$z$2$us$i;
        }
        $550 = (($k$5$us$i) + -1)|0;
        if ($548) {
         break;
        } else {
         $carry1$0$us$i = $carry1$1$us$i;$k$5$in$us$i = $550;$z$2$us$i = $z$3$us$i;
        }
       }
       $551 = (($e2$0$us$i) + -29)|0;
       $552 = ($carry1$1$us$i|0)==(0);
       if ($552) {
        $e2$0$us$i = $551;$z$1$us$i = $z$3$us$i;
       } else {
        $$lcssa43$i = $551;$carry1$1$lcssa$lcssa$i = $carry1$1$us$i;$z$3$lcssa$lcssa$i = $z$3$us$i;
        break;
       }
      }
     } else {
      $528 = ($rp$2$ph38$i|0)==(18);
      if ($528) {
       $e2$0$us44$i = $e2$0$ph$i;$z$1$us45$i = $z$1$ph39$i;
      } else {
       $a$3$ph$i = $a$2$ph40$i;$e2$1$ph$i = $e2$0$ph$i;$rp$3$ph33$i = $rp$2$ph38$i;$z$5$ph$i = $z$1$ph39$i;
       break;
      }
      while(1) {
       $553 = HEAP32[$527>>2]|0;
       $554 = ($553>>>0)<(9007199);
       if (!($554)) {
        $a$3$ph$i = $a$2$ph40$i;$e2$1$ph$i = $e2$0$us44$i;$rp$3$ph33$i = 18;$z$5$ph$i = $z$1$us45$i;
        break L280;
       }
       $555 = (($z$1$us45$i) + 127)|0;
       $carry1$0$us49$i = 0;$k$5$in$us48$i = $555;$z$2$us47$i = $z$1$us45$i;
       while(1) {
        $k$5$us50$i = $k$5$in$us48$i & 127;
        $556 = (($x$i) + ($k$5$us50$i<<2)|0);
        $557 = HEAP32[$556>>2]|0;
        $558 = (_bitshift64Shl(($557|0),0,29)|0);
        $559 = tempRet0;
        $560 = (_i64Add(($558|0),($559|0),($carry1$0$us49$i|0),0)|0);
        $561 = tempRet0;
        $562 = ($561>>>0)>(0);
        $563 = ($561|0)==(0);
        $564 = ($560>>>0)>(1000000000);
        $565 = $563 & $564;
        $566 = $562 | $565;
        if ($566) {
         $567 = (___udivdi3(($560|0),($561|0),1000000000,0)|0);
         $568 = tempRet0;
         $569 = (___uremdi3(($560|0),($561|0),1000000000,0)|0);
         $570 = tempRet0;
         $$sink$off0$us53$i = $569;$carry1$1$us54$i = $567;
        } else {
         $$sink$off0$us53$i = $560;$carry1$1$us54$i = 0;
        }
        HEAP32[$556>>2] = $$sink$off0$us53$i;
        $571 = (($z$2$us47$i) + 127)|0;
        $572 = $571 & 127;
        $573 = ($k$5$us50$i|0)!=($572|0);
        $574 = ($k$5$us50$i|0)==($a$2$ph40$i|0);
        $or$cond12$us55$i = $573 | $574;
        if ($or$cond12$us55$i) {
         $z$3$us57$i = $z$2$us47$i;
        } else {
         $575 = ($$sink$off0$us53$i|0)==(0);
         $k$5$z$2$us56$i = $575 ? $k$5$us50$i : $z$2$us47$i;
         $z$3$us57$i = $k$5$z$2$us56$i;
        }
        $576 = (($k$5$us50$i) + -1)|0;
        if ($574) {
         break;
        } else {
         $carry1$0$us49$i = $carry1$1$us54$i;$k$5$in$us48$i = $576;$z$2$us47$i = $z$3$us57$i;
        }
       }
       $577 = (($e2$0$us44$i) + -29)|0;
       $578 = ($carry1$1$us54$i|0)==(0);
       if ($578) {
        $e2$0$us44$i = $577;$z$1$us45$i = $z$3$us57$i;
       } else {
        $$lcssa43$i = $577;$carry1$1$lcssa$lcssa$i = $carry1$1$us54$i;$z$3$lcssa$lcssa$i = $z$3$us57$i;
        break;
       }
      }
     }
     $579 = (($rp$2$ph38$i) + 9)|0;
     $580 = (($a$2$ph40$i) + 127)|0;
     $581 = $580 & 127;
     $582 = ($581|0)==($z$3$lcssa$lcssa$i|0);
     if ($582) {
      $583 = (($z$3$lcssa$lcssa$i) + 127)|0;
      $584 = $583 & 127;
      $585 = (($x$i) + ($584<<2)|0);
      $586 = HEAP32[$585>>2]|0;
      $587 = (($z$3$lcssa$lcssa$i) + 126)|0;
      $588 = $587 & 127;
      $589 = (($x$i) + ($588<<2)|0);
      $590 = HEAP32[$589>>2]|0;
      $591 = $590 | $586;
      HEAP32[$589>>2] = $591;
      $z$4$i = $584;
     } else {
      $z$4$i = $z$3$lcssa$lcssa$i;
     }
     $592 = (($x$i) + ($581<<2)|0);
     HEAP32[$592>>2] = $carry1$1$lcssa$lcssa$i;
     $a$2$ph40$i = $581;$e2$0$ph$i = $$lcssa43$i;$rp$2$ph38$i = $579;$z$1$ph39$i = $z$4$i;
    }
    L311: while(1) {
     $629 = (($z$5$ph$i) + 1)|0;
     $627 = $629 & 127;
     $630 = (($z$5$ph$i) + 127)|0;
     $631 = $630 & 127;
     $632 = (($x$i) + ($631<<2)|0);
     $a$3$i$ph = $a$3$ph$i;$e2$1$i$ph = $e2$1$ph$i;$rp$3$i$ph = $rp$3$ph33$i;
     while(1) {
      $605 = ($rp$3$i$ph|0)==(18);
      $633 = ($rp$3$i$ph|0)>(27);
      $$14$i = $633 ? 9 : 1;
      $a$3$i = $a$3$i$ph;$e2$1$i = $e2$1$i$ph;
      while(1) {
       $i$025$i = 0;
       while(1) {
        $595 = (($i$025$i) + ($a$3$i))|0;
        $596 = $595 & 127;
        $597 = ($596|0)==($z$5$ph$i|0);
        if ($597) {
         $i$1$i = 2;
         break;
        }
        $598 = (($x$i) + ($596<<2)|0);
        $599 = HEAP32[$598>>2]|0;
        $600 = (14872 + ($i$025$i<<2)|0);
        $601 = HEAP32[$600>>2]|0;
        $602 = ($599>>>0)<($601>>>0);
        if ($602) {
         $i$1$i = 2;
         break;
        }
        $603 = ($599>>>0)>($601>>>0);
        $594 = (($i$025$i) + 1)|0;
        if ($603) {
         $i$1$i = $i$025$i;
         break;
        }
        $593 = ($594|0)<(2);
        if ($593) {
         $i$025$i = $594;
        } else {
         $i$1$i = $594;
         break;
        }
       }
       $604 = ($i$1$i|0)==(2);
       $or$cond13$i = $604 & $605;
       if ($or$cond13$i) {
        break L311;
       }
       $608 = (($$14$i) + ($e2$1$i))|0;
       $609 = ($a$3$i|0)==($z$5$ph$i|0);
       if ($609) {
        $a$3$i = $z$5$ph$i;$e2$1$i = $608;
       } else {
        break;
       }
      }
      $610 = 1 << $$14$i;
      $611 = (($610) + -1)|0;
      $612 = 1000000000 >>> $$14$i;
      $a$427$i = $a$3$i;$carry3$030$i = 0;$k$628$i = $a$3$i;$rp$426$i = $rp$3$i$ph;
      while(1) {
       $613 = (($x$i) + ($k$628$i<<2)|0);
       $614 = HEAP32[$613>>2]|0;
       $615 = $614 & $611;
       $616 = $614 >>> $$14$i;
       $617 = (($616) + ($carry3$030$i))|0;
       HEAP32[$613>>2] = $617;
       $618 = Math_imul($615, $612)|0;
       $619 = ($k$628$i|0)==($a$427$i|0);
       $620 = ($617|0)==(0);
       $or$cond15$i = $619 & $620;
       $621 = (($k$628$i) + 1)|0;
       $622 = $621 & 127;
       $623 = (($rp$426$i) + -9)|0;
       $rp$5$i = $or$cond15$i ? $623 : $rp$426$i;
       $a$5$i = $or$cond15$i ? $622 : $a$427$i;
       $624 = ($622|0)==($z$5$ph$i|0);
       if ($624) {
        break;
       } else {
        $a$427$i = $a$5$i;$carry3$030$i = $618;$k$628$i = $622;$rp$426$i = $rp$5$i;
       }
      }
      $625 = ($618|0)==(0);
      if ($625) {
       $a$3$i$ph = $a$5$i;$e2$1$i$ph = $608;$rp$3$i$ph = $rp$5$i;
       continue;
      }
      $626 = ($627|0)==($a$5$i|0);
      if (!($626)) {
       break;
      }
      $634 = HEAP32[$632>>2]|0;
      $635 = $634 | 1;
      HEAP32[$632>>2] = $635;
      $a$3$i$ph = $a$5$i;$e2$1$i$ph = $608;$rp$3$i$ph = $rp$5$i;
     }
     $628 = (($x$i) + ($z$5$ph$i<<2)|0);
     HEAP32[$628>>2] = $618;
     $a$3$ph$i = $a$5$i;$e2$1$ph$i = $608;$rp$3$ph33$i = $rp$5$i;$z$5$ph$i = $627;
    }
    $606 = $a$3$i & 127;
    $607 = ($606|0)==($z$5$ph$i|0);
    if ($607) {
     $636 = (($627) + -1)|0;
     $637 = (($x$i) + ($636<<2)|0);
     HEAP32[$637>>2] = 0;
     $z$7$i = $627;
    } else {
     $z$7$i = $z$5$ph$i;
    }
    $638 = (($x$i) + ($606<<2)|0);
    $639 = HEAP32[$638>>2]|0;
    $640 = (+($639>>>0));
    $641 = (($a$3$i) + 1)|0;
    $642 = $641 & 127;
    $643 = ($642|0)==($z$7$i|0);
    if ($643) {
     $702 = (($z$7$i) + 1)|0;
     $703 = $702 & 127;
     $704 = (($703) + -1)|0;
     $705 = (($x$i) + ($704<<2)|0);
     HEAP32[$705>>2] = 0;
     $z$7$1$i = $703;
    } else {
     $z$7$1$i = $z$7$i;
    }
    $706 = $640 * 1.0E+9;
    $707 = (($x$i) + ($642<<2)|0);
    $708 = HEAP32[$707>>2]|0;
    $709 = (+($708>>>0));
    $710 = $706 + $709;
    $667 = (+($sign$0|0));
    $649 = $667 * $710;
    $687 = (($e2$1$i) + 53)|0;
    $645 = (($687) - ($emin$0$ph))|0;
    $711 = ($645|0)<($bits$0$ph|0);
    if ($711) {
     $644 = ($645|0)<(0);
     $$$i22 = $644 ? 0 : $645;
     $$09$i = $$$i22;$denormal$0$i = 1;
    } else {
     $$09$i = $bits$0$ph;$denormal$0$i = 0;
    }
    $646 = ($$09$i|0)<(53);
    if ($646) {
     $647 = (105 - ($$09$i))|0;
     $648 = (+_scalbn(1.0,$647));
     $650 = (+_copysignl((+$648),(+$649)));
     $651 = (53 - ($$09$i))|0;
     $652 = (+_scalbn(1.0,$651));
     $653 = (+_fmodl((+$649),(+$652)));
     $654 = $649 - $653;
     $655 = $650 + $654;
     $bias$0$i = $650;$frac$0$i = $653;$y$1$i23 = $655;
    } else {
     $bias$0$i = 0.0;$frac$0$i = 0.0;$y$1$i23 = $649;
    }
    $656 = (($a$3$i) + 2)|0;
    $657 = $656 & 127;
    $658 = ($657|0)==($z$7$1$i|0);
    do {
     if ($658) {
      $frac$2$i = $frac$0$i;
     } else {
      $659 = (($x$i) + ($657<<2)|0);
      $660 = HEAP32[$659>>2]|0;
      $661 = ($660>>>0)<(500000000);
      do {
       if ($661) {
        $662 = ($660|0)==(0);
        if ($662) {
         $663 = (($a$3$i) + 3)|0;
         $664 = $663 & 127;
         $665 = ($664|0)==($z$7$1$i|0);
         if ($665) {
          $frac$1$i = $frac$0$i;
          break;
         }
        }
        $666 = $667 * 0.25;
        $668 = $666 + $frac$0$i;
        $frac$1$i = $668;
       } else {
        $669 = ($660>>>0)>(500000000);
        if ($669) {
         $670 = $667 * 0.75;
         $671 = $670 + $frac$0$i;
         $frac$1$i = $671;
         break;
        }
        $672 = (($a$3$i) + 3)|0;
        $673 = $672 & 127;
        $674 = ($673|0)==($z$7$1$i|0);
        if ($674) {
         $675 = $667 * 0.5;
         $676 = $675 + $frac$0$i;
         $frac$1$i = $676;
         break;
        } else {
         $677 = $667 * 0.75;
         $678 = $677 + $frac$0$i;
         $frac$1$i = $678;
         break;
        }
       }
      } while(0);
      $679 = (53 - ($$09$i))|0;
      $680 = ($679|0)>(1);
      if (!($680)) {
       $frac$2$i = $frac$1$i;
       break;
      }
      $681 = (+_fmodl((+$frac$1$i),1.0));
      $682 = $681 != 0.0;
      if ($682) {
       $frac$2$i = $frac$1$i;
       break;
      }
      $683 = $frac$1$i + 1.0;
      $frac$2$i = $683;
     }
    } while(0);
    $684 = $y$1$i23 + $frac$2$i;
    $685 = $684 - $bias$0$i;
    $686 = $687 & 2147483647;
    $688 = (-2 - ($sum$i))|0;
    $689 = ($686|0)>($688|0);
    do {
     if ($689) {
      $690 = (+Math_abs((+$685)));
      $691 = !($690 >= 9007199254740992.0);
      if ($691) {
       $denormal$2$i = $denormal$0$i;$e2$2$i = $e2$1$i;$y$2$i24 = $685;
      } else {
       $692 = ($denormal$0$i|0)!=(0);
       $693 = ($$09$i|0)==($645|0);
       $or$cond16$i = $692 & $693;
       $denormal$1$i = $or$cond16$i ? 0 : $denormal$0$i;
       $694 = $685 * 0.5;
       $695 = (($e2$1$i) + 1)|0;
       $denormal$2$i = $denormal$1$i;$e2$2$i = $695;$y$2$i24 = $694;
      }
      $696 = (($e2$2$i) + 50)|0;
      $697 = ($696|0)>($325|0);
      if (!($697)) {
       $698 = ($denormal$2$i|0)!=(0);
       $699 = $frac$2$i != 0.0;
       $or$cond17$i = $698 & $699;
       if (!($or$cond17$i)) {
        $e2$3$i = $e2$2$i;$y$3$i = $y$2$i24;
        break;
       }
      }
      $700 = (___errno_location()|0);
      HEAP32[$700>>2] = 34;
      $e2$3$i = $e2$2$i;$y$3$i = $y$2$i24;
     } else {
      $e2$3$i = $e2$1$i;$y$3$i = $685;
     }
    } while(0);
    $701 = (+_scalbnl($y$3$i,$e2$3$i));
    $$0 = $701;
    STACKTOP = sp;return (+$$0);
   } else if ((($i$3|0) == 3)) {
    $68 = HEAP32[$0>>2]|0;
    $69 = HEAP32[$1>>2]|0;
    $70 = ($68>>>0)<($69>>>0);
    if ($70) {
     $71 = (($68) + 1|0);
     HEAP32[$0>>2] = $71;
     $72 = HEAP8[$68>>0]|0;
     $73 = $72&255;
     $76 = $73;
    } else {
     $74 = (___shgetc($f)|0);
     $76 = $74;
    }
    $75 = ($76|0)==(40);
    if ($75) {
     $i$4 = 1;
    } else {
     $77 = HEAP32[$1>>2]|0;
     $78 = ($77|0)==(0|0);
     if ($78) {
      $$0 = nan;
      STACKTOP = sp;return (+$$0);
     }
     $79 = HEAP32[$0>>2]|0;
     $80 = (($79) + -1|0);
     HEAP32[$0>>2] = $80;
     $$0 = nan;
     STACKTOP = sp;return (+$$0);
    }
    while(1) {
     $81 = HEAP32[$0>>2]|0;
     $82 = HEAP32[$1>>2]|0;
     $83 = ($81>>>0)<($82>>>0);
     if ($83) {
      $84 = (($81) + 1|0);
      HEAP32[$0>>2] = $84;
      $85 = HEAP8[$81>>0]|0;
      $86 = $85&255;
      $89 = $86;
     } else {
      $87 = (___shgetc($f)|0);
      $89 = $87;
     }
     $88 = (($89) + -48)|0;
     $90 = ($88>>>0)<(10);
     $91 = (($89) + -65)|0;
     $92 = ($91>>>0)<(26);
     $or$cond8 = $90 | $92;
     if (!($or$cond8)) {
      $93 = (($89) + -97)|0;
      $94 = ($93>>>0)<(26);
      $95 = ($89|0)==(95);
      $or$cond5 = $94 | $95;
      if (!($or$cond5)) {
       break;
      }
     }
     $107 = (($i$4) + 1)|0;
     $i$4 = $107;
    }
    $96 = ($89|0)==(41);
    if ($96) {
     $$0 = nan;
     STACKTOP = sp;return (+$$0);
    }
    $97 = HEAP32[$1>>2]|0;
    $98 = ($97|0)==(0|0);
    if (!($98)) {
     $99 = HEAP32[$0>>2]|0;
     $100 = (($99) + -1|0);
     HEAP32[$0>>2] = $100;
    }
    if ($39) {
     $102 = (___errno_location()|0);
     HEAP32[$102>>2] = 22;
     ___shlim($f,0);
     $$0 = 0.0;
     STACKTOP = sp;return (+$$0);
    }
    $101 = ($i$4|0)==(0);
    $brmerge107 = $101 | $98;
    if ($brmerge107) {
     $$0 = nan;
     STACKTOP = sp;return (+$$0);
    } else {
     $$in = $i$4;
    }
    while(1) {
     $103 = (($$in) + -1)|0;
     $104 = HEAP32[$0>>2]|0;
     $105 = (($104) + -1|0);
     HEAP32[$0>>2] = $105;
     $106 = ($103|0)==(0);
     if ($106) {
      $$0 = nan;
      break;
     } else {
      $$in = $103;
     }
    }
    STACKTOP = sp;return (+$$0);
   } else {
    $108 = HEAP32[$1>>2]|0;
    $109 = ($108|0)==(0|0);
    if (!($109)) {
     $110 = HEAP32[$0>>2]|0;
     $111 = (($110) + -1|0);
     HEAP32[$0>>2] = $111;
    }
    $112 = (___errno_location()|0);
    HEAP32[$112>>2] = 22;
    ___shlim($f,0);
    $$0 = 0.0;
    STACKTOP = sp;return (+$$0);
   }
  }
 } while(0);
 if ((label|0) == 23) {
  $41 = HEAP32[$1>>2]|0;
  $42 = ($41|0)==(0|0);
  if (!($42)) {
   $43 = HEAP32[$0>>2]|0;
   $44 = (($43) + -1|0);
   HEAP32[$0>>2] = $44;
  }
  $notlhs = ($pok|0)==(0);
  $notrhs = ($i$0$lcssa>>>0)<(4);
  $or$cond106$not = $notrhs | $notlhs;
  $brmerge = $or$cond106$not | $42;
  if (!($brmerge)) {
   $i$166 = $i$0$lcssa;
   while(1) {
    $45 = HEAP32[$0>>2]|0;
    $46 = (($45) + -1|0);
    HEAP32[$0>>2] = $46;
    $47 = (($i$166) + -1)|0;
    $48 = ($47>>>0)>(3);
    if ($48) {
     $i$166 = $47;
    } else {
     break;
    }
   }
  }
 }
 $49 = (+($sign$0|0));
 $50 = $49 * inf;
 $51 = $50;
 $$0 = $51;
 STACKTOP = sp;return (+$$0);
}
function _scanexp($f,$pok) {
 $f = $f|0;
 $pok = $pok|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $c$0 = 0, $c$1$be = 0;
 var $c$18 = 0, $c$2$be = 0, $c$2$lcssa = 0, $c$23 = 0, $c$3$be = 0, $neg$0 = 0, $or$cond = 0, $or$cond2 = 0, $phitmp16 = 0, $x$09 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 4|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = (($f) + 100|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($1>>>0)<($3>>>0);
 if ($4) {
  $5 = (($1) + 1|0);
  HEAP32[$0>>2] = $5;
  $6 = HEAP8[$1>>0]|0;
  $7 = $6&255;
  $10 = $7;
 } else {
  $8 = (___shgetc($f)|0);
  $10 = $8;
 }
 $9 = ($10|0)==(45);
 if ((($10|0) == 43) | (($10|0) == 45)) {
  $11 = $9&1;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($12>>>0)<($13>>>0);
  if ($14) {
   $15 = (($12) + 1|0);
   HEAP32[$0>>2] = $15;
   $16 = HEAP8[$12>>0]|0;
   $17 = $16&255;
   $20 = $17;
  } else {
   $18 = (___shgetc($f)|0);
   $20 = $18;
  }
  $19 = (($20) + -48)|0;
  $21 = ($19>>>0)<(10);
  $22 = ($pok|0)==(0);
  $or$cond = $21 | $22;
  if ($or$cond) {
   $c$0 = $20;$neg$0 = $11;
  } else {
   $23 = HEAP32[$2>>2]|0;
   $24 = ($23|0)==(0|0);
   if ($24) {
    $c$0 = $20;$neg$0 = $11;
   } else {
    $25 = HEAP32[$0>>2]|0;
    $26 = (($25) + -1|0);
    HEAP32[$0>>2] = $26;
    $c$0 = $20;$neg$0 = $11;
   }
  }
 } else {
  $c$0 = $10;$neg$0 = 0;
 }
 $27 = (($c$0) + -48)|0;
 $28 = ($27>>>0)>(9);
 if ($28) {
  $29 = HEAP32[$2>>2]|0;
  $30 = ($29|0)==(0|0);
  if ($30) {
   $95 = -2147483648;$96 = 0;
   tempRet0 = $95;
   STACKTOP = sp;return ($96|0);
  }
  $31 = HEAP32[$0>>2]|0;
  $32 = (($31) + -1|0);
  HEAP32[$0>>2] = $32;
  $95 = -2147483648;$96 = 0;
  tempRet0 = $95;
  STACKTOP = sp;return ($96|0);
 } else {
  $c$18 = $c$0;$x$09 = 0;
 }
 while(1) {
  $33 = (($c$18) + -48)|0;
  $34 = (($33) + ($x$09))|0;
  $35 = HEAP32[$0>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = ($35>>>0)<($36>>>0);
  if ($37) {
   $38 = (($35) + 1|0);
   HEAP32[$0>>2] = $38;
   $39 = HEAP8[$35>>0]|0;
   $40 = $39&255;
   $c$1$be = $40;
  } else {
   $41 = (___shgetc($f)|0);
   $c$1$be = $41;
  }
  $42 = (($c$1$be) + -48)|0;
  $43 = ($42>>>0)<(10);
  $44 = ($34|0)<(214748364);
  $$ = $43 & $44;
  if (!($$)) {
   break;
  }
  $phitmp16 = ($34*10)|0;
  $c$18 = $c$1$be;$x$09 = $phitmp16;
 }
 $45 = ($34|0)<(0);
 $46 = $45 << 31 >> 31;
 $47 = (($c$1$be) + -48)|0;
 $48 = ($47>>>0)<(10);
 if ($48) {
  $51 = $34;$52 = $46;$c$23 = $c$1$be;
  while(1) {
   $53 = (___muldi3(($51|0),($52|0),10,0)|0);
   $54 = tempRet0;
   $55 = ($c$23|0)<(0);
   $56 = $55 << 31 >> 31;
   $57 = (_i64Add(($c$23|0),($56|0),-48,-1)|0);
   $58 = tempRet0;
   $59 = (_i64Add(($57|0),($58|0),($53|0),($54|0))|0);
   $60 = tempRet0;
   $61 = HEAP32[$0>>2]|0;
   $62 = HEAP32[$2>>2]|0;
   $63 = ($61>>>0)<($62>>>0);
   if ($63) {
    $64 = (($61) + 1|0);
    HEAP32[$0>>2] = $64;
    $65 = HEAP8[$61>>0]|0;
    $66 = $65&255;
    $c$2$be = $66;
   } else {
    $67 = (___shgetc($f)|0);
    $c$2$be = $67;
   }
   $68 = (($c$2$be) + -48)|0;
   $69 = ($68>>>0)<(10);
   $70 = ($60|0)<(21474836);
   $71 = ($60|0)==(21474836);
   $72 = ($59>>>0)<(2061584302);
   $73 = $71 & $72;
   $74 = $70 | $73;
   $or$cond2 = $69 & $74;
   if ($or$cond2) {
    $51 = $59;$52 = $60;$c$23 = $c$2$be;
   } else {
    $89 = $59;$90 = $60;$c$2$lcssa = $c$2$be;
    break;
   }
  }
 } else {
  $89 = $34;$90 = $46;$c$2$lcssa = $c$1$be;
 }
 $49 = (($c$2$lcssa) + -48)|0;
 $50 = ($49>>>0)<(10);
 if ($50) {
  while(1) {
   $75 = HEAP32[$0>>2]|0;
   $76 = HEAP32[$2>>2]|0;
   $77 = ($75>>>0)<($76>>>0);
   if ($77) {
    $78 = (($75) + 1|0);
    HEAP32[$0>>2] = $78;
    $79 = HEAP8[$75>>0]|0;
    $80 = $79&255;
    $c$3$be = $80;
   } else {
    $81 = (___shgetc($f)|0);
    $c$3$be = $81;
   }
   $82 = (($c$3$be) + -48)|0;
   $83 = ($82>>>0)<(10);
   if (!($83)) {
    break;
   }
  }
 }
 $84 = HEAP32[$2>>2]|0;
 $85 = ($84|0)==(0|0);
 if (!($85)) {
  $86 = HEAP32[$0>>2]|0;
  $87 = (($86) + -1|0);
  HEAP32[$0>>2] = $87;
 }
 $88 = ($neg$0|0)!=(0);
 $91 = (_i64Subtract(0,0,($89|0),($90|0))|0);
 $92 = tempRet0;
 $93 = $88 ? $91 : $89;
 $94 = $88 ? $92 : $90;
 $95 = $94;$96 = $93;
 tempRet0 = $95;
 STACKTOP = sp;return ($96|0);
}
function ___shlim($f,$lim) {
 $f = $f|0;
 $lim = $lim|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 104|0);
 HEAP32[$0>>2] = $lim;
 $1 = (($f) + 8|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = (($f) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $2;
 $6 = $4;
 $7 = (($5) - ($6))|0;
 $8 = (($f) + 108|0);
 HEAP32[$8>>2] = $7;
 $9 = ($lim|0)!=(0);
 $10 = ($7|0)>($lim|0);
 $or$cond = $9 & $10;
 if ($or$cond) {
  $11 = (($4) + ($lim)|0);
  $12 = (($f) + 100|0);
  HEAP32[$12>>2] = $11;
  STACKTOP = sp;return;
 } else {
  $13 = (($f) + 100|0);
  HEAP32[$13>>2] = $2;
  STACKTOP = sp;return;
 }
}
function ___shgetc($f) {
 $f = $f|0;
 var $$0 = 0, $$phi$trans$insert = 0, $$phi$trans$insert2 = 0, $$pre = 0, $$pre3 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 104|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  label = 3;
 } else {
  $3 = (($f) + 108|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($4|0)<($1|0);
  if ($5) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $6 = (___uflow($f)|0);
  $7 = ($6|0)<(0);
  if (!($7)) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ($9|0)==(0);
   $$phi$trans$insert = (($f) + 8|0);
   $$pre = HEAP32[$$phi$trans$insert>>2]|0;
   if ($10) {
    label = 8;
   } else {
    $11 = (($f) + 4|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = $$pre;
    $14 = $12;
    $15 = (($13) - ($14))|0;
    $16 = (($f) + 108|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (($9) - ($17))|0;
    $19 = (($18) + -1)|0;
    $20 = ($15|0)>($19|0);
    if ($20) {
     $21 = (($12) + ($19)|0);
     $22 = (($f) + 100|0);
     HEAP32[$22>>2] = $21;
    } else {
     label = 8;
    }
   }
   if ((label|0) == 8) {
    $23 = (($f) + 100|0);
    HEAP32[$23>>2] = $$pre;
   }
   $24 = ($$pre|0)==(0|0);
   $$phi$trans$insert2 = (($f) + 4|0);
   $$pre3 = HEAP32[$$phi$trans$insert2>>2]|0;
   if (!($24)) {
    $25 = $$pre;
    $26 = $$pre3;
    $27 = (($f) + 108|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = (($25) + 1)|0;
    $30 = (($29) - ($26))|0;
    $31 = (($30) + ($28))|0;
    HEAP32[$27>>2] = $31;
   }
   $32 = (($$pre3) + -1|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = ($34|0)==($6|0);
   if ($35) {
    $$0 = $6;
    STACKTOP = sp;return ($$0|0);
   }
   $36 = $6&255;
   HEAP8[$32>>0] = $36;
   $$0 = $6;
   STACKTOP = sp;return ($$0|0);
  }
 }
 $8 = (($f) + 100|0);
 HEAP32[$8>>2] = 0;
 $$0 = -1;
 STACKTOP = sp;return ($$0|0);
}
function _frexp($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $$0 = 0.0, $$01 = 0.0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0.0, $8 = 0, $9 = 0, $storemerge = 0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = (_bitshift64Lshr(($0|0),($1|0),52)|0);
 $3 = tempRet0;
 $4 = $2 & 2047;
 if ((($4|0) == 2047)) {
  $$0 = $x;
  STACKTOP = sp;return (+$$0);
 } else if ((($4|0) == 0)) {
  $5 = $x != 0.0;
  if ($5) {
   $6 = $x * 18446744073709551616.0;
   $7 = (+_frexp($6,$e));
   $8 = HEAP32[$e>>2]|0;
   $9 = (($8) + -64)|0;
   $$01 = $7;$storemerge = $9;
  } else {
   $$01 = $x;$storemerge = 0;
  }
  HEAP32[$e>>2] = $storemerge;
  $$0 = $$01;
  STACKTOP = sp;return (+$$0);
 } else {
  $10 = (($4) + -1022)|0;
  HEAP32[$e>>2] = $10;
  $11 = $1 & -2146435073;
  $12 = $11 | 1071644672;
  HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $12;$13 = +HEAPF64[tempDoublePtr>>3];
  $$0 = $13;
  STACKTOP = sp;return (+$$0);
 }
 return +0;
}
function _frexpl($x,$e) {
 $x = +$x;
 $e = $e|0;
 var $0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (+_frexp($x,$e));
 STACKTOP = sp;return (+$0);
}
function _scalbn($x,$n) {
 $x = +$x;
 $n = $n|0;
 var $$ = 0, $$0 = 0, $$1 = 0, $0 = 0, $1 = 0.0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $2 = 0, $3 = 0, $4 = 0.0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0.0, $9 = 0, $y$0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)>(1023);
 if ($0) {
  $1 = $x * 8.98846567431157953864E+307;
  $2 = (($n) + -1023)|0;
  $3 = ($2|0)>(1023);
  if ($3) {
   $4 = $1 * 8.98846567431157953864E+307;
   $5 = (($n) + -2046)|0;
   $6 = ($5|0)>(1023);
   $$ = $6 ? 1023 : $5;
   $$0 = $$;$y$0 = $4;
  } else {
   $$0 = $2;$y$0 = $1;
  }
 } else {
  $7 = ($n|0)<(-1022);
  if ($7) {
   $8 = $x * 2.22507385850720138309E-308;
   $9 = (($n) + 1022)|0;
   $10 = ($9|0)<(-1022);
   if ($10) {
    $11 = $8 * 2.22507385850720138309E-308;
    $12 = (($n) + 2044)|0;
    $13 = ($12|0)<(-1022);
    $$1 = $13 ? -1022 : $12;
    $$0 = $$1;$y$0 = $11;
   } else {
    $$0 = $9;$y$0 = $8;
   }
  } else {
   $$0 = $n;$y$0 = $x;
  }
 }
 $14 = (($$0) + 1023)|0;
 $15 = (_bitshift64Shl(($14|0),0,52)|0);
 $16 = tempRet0;
 HEAP32[tempDoublePtr>>2] = $15;HEAP32[tempDoublePtr+4>>2] = $16;$17 = +HEAPF64[tempDoublePtr>>3];
 $18 = $y$0 * $17;
 STACKTOP = sp;return (+$18);
}
function _scalbnl($x,$n) {
 $x = +$x;
 $n = $n|0;
 var $0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (+_scalbn($x,$n));
 STACKTOP = sp;return (+$0);
}
function _wctomb($s,$wc) {
 $s = $s|0;
 $wc = $wc|0;
 var $$0 = 0, $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 if ($0) {
  $$0 = 0;
 } else {
  $1 = (_wcrtomb($s,$wc,0)|0);
  $$0 = $1;
 }
 STACKTOP = sp;return ($$0|0);
}
function _wcrtomb($s,$wc,$st) {
 $s = $s|0;
 $wc = $wc|0;
 $st = $st|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($s|0)==(0|0);
 if ($0) {
  $$0 = 1;
  STACKTOP = sp;return ($$0|0);
 }
 $1 = ($wc>>>0)<(128);
 if ($1) {
  $2 = $wc&255;
  HEAP8[$s>>0] = $2;
  $$0 = 1;
  STACKTOP = sp;return ($$0|0);
 }
 $3 = ($wc>>>0)<(2048);
 if ($3) {
  $4 = $wc >>> 6;
  $5 = $4 | 192;
  $6 = $5&255;
  $7 = (($s) + 1|0);
  HEAP8[$s>>0] = $6;
  $8 = $wc & 63;
  $9 = $8 | 128;
  $10 = $9&255;
  HEAP8[$7>>0] = $10;
  $$0 = 2;
  STACKTOP = sp;return ($$0|0);
 }
 $11 = ($wc>>>0)<(55296);
 $12 = (($wc) + -57344)|0;
 $13 = ($12>>>0)<(8192);
 $or$cond = $11 | $13;
 if ($or$cond) {
  $14 = $wc >>> 12;
  $15 = $14 | 224;
  $16 = $15&255;
  $17 = (($s) + 1|0);
  HEAP8[$s>>0] = $16;
  $18 = $wc >>> 6;
  $19 = $18 & 63;
  $20 = $19 | 128;
  $21 = $20&255;
  $22 = (($s) + 2|0);
  HEAP8[$17>>0] = $21;
  $23 = $wc & 63;
  $24 = $23 | 128;
  $25 = $24&255;
  HEAP8[$22>>0] = $25;
  $$0 = 3;
  STACKTOP = sp;return ($$0|0);
 }
 $26 = (($wc) + -65536)|0;
 $27 = ($26>>>0)<(1048576);
 if ($27) {
  $28 = $wc >>> 18;
  $29 = $28 | 240;
  $30 = $29&255;
  $31 = (($s) + 1|0);
  HEAP8[$s>>0] = $30;
  $32 = $wc >>> 12;
  $33 = $32 & 63;
  $34 = $33 | 128;
  $35 = $34&255;
  $36 = (($s) + 2|0);
  HEAP8[$31>>0] = $35;
  $37 = $wc >>> 6;
  $38 = $37 & 63;
  $39 = $38 | 128;
  $40 = $39&255;
  $41 = (($s) + 3|0);
  HEAP8[$36>>0] = $40;
  $42 = $wc & 63;
  $43 = $42 | 128;
  $44 = $43&255;
  HEAP8[$41>>0] = $44;
  $$0 = 4;
  STACKTOP = sp;return ($$0|0);
 } else {
  $45 = (___errno_location()|0);
  HEAP32[$45>>2] = 84;
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function ___toread($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 74|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = (($2) + 255)|0;
 $4 = $3 | $2;
 $5 = $4&255;
 HEAP8[$0>>0] = $5;
 $6 = (($f) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($f) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($7>>>0)>($9>>>0);
 if ($10) {
  $11 = (($f) + 36|0);
  $12 = HEAP32[$11>>2]|0;
  (FUNCTION_TABLE_iiii[$12 & 3]($f,0,0)|0);
 }
 $13 = (($f) + 16|0);
 HEAP32[$13>>2] = 0;
 $14 = (($f) + 28|0);
 HEAP32[$14>>2] = 0;
 HEAP32[$6>>2] = 0;
 $15 = HEAP32[$f>>2]|0;
 $16 = $15 & 20;
 $17 = ($16|0)==(0);
 if ($17) {
  $21 = HEAP32[$8>>2]|0;
  $22 = (($f) + 8|0);
  HEAP32[$22>>2] = $21;
  $23 = (($f) + 4|0);
  HEAP32[$23>>2] = $21;
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 }
 $18 = $15 & 4;
 $19 = ($18|0)==(0);
 if ($19) {
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 $20 = $15 | 32;
 HEAP32[$f>>2] = $20;
 $$0 = -1;
 STACKTOP = sp;return ($$0|0);
}
function ___towrite($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 74|0);
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = (($2) + 255)|0;
 $4 = $3 | $2;
 $5 = $4&255;
 HEAP8[$0>>0] = $5;
 $6 = HEAP32[$f>>2]|0;
 $7 = $6 & 8;
 $8 = ($7|0)==(0);
 if ($8) {
  $10 = (($f) + 8|0);
  HEAP32[$10>>2] = 0;
  $11 = (($f) + 4|0);
  HEAP32[$11>>2] = 0;
  $12 = (($f) + 44|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($f) + 28|0);
  HEAP32[$14>>2] = $13;
  $15 = (($f) + 20|0);
  HEAP32[$15>>2] = $13;
  $16 = (($f) + 48|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($13) + ($17)|0);
  $19 = (($f) + 16|0);
  HEAP32[$19>>2] = $18;
  $$0 = 0;
  STACKTOP = sp;return ($$0|0);
 } else {
  $9 = $6 | 32;
  HEAP32[$f>>2] = $9;
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function ___uflow($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $c = sp;
 $0 = (($f) + 8|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  $3 = (___toread($f)|0);
  $4 = ($3|0)==(0);
  if ($4) {
   label = 3;
  } else {
   $$0 = -1;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $5 = (($f) + 32|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = (FUNCTION_TABLE_iiii[$6 & 3]($f,$c,1)|0);
  $8 = ($7|0)==(1);
  if ($8) {
   $9 = HEAP8[$c>>0]|0;
   $10 = $9&255;
   $$0 = $10;
  } else {
   $$0 = -1;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function ___fwritex($s,$l,$f) {
 $s = $s|0;
 $l = $l|0;
 $f = $f|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $$pre = 0, $$pre6 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i$0 = 0, $i$1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 do {
  if ($2) {
   $3 = (___towrite($f)|0);
   $4 = ($3|0)==(0);
   if ($4) {
    $$pre6 = HEAP32[$0>>2]|0;
    $8 = $$pre6;
    break;
   } else {
    $$0 = 0;
    STACKTOP = sp;return ($$0|0);
   }
  } else {
   $8 = $1;
  }
 } while(0);
 $5 = (($f) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $8;
 $9 = $6;
 $10 = (($7) - ($9))|0;
 $11 = ($10>>>0)<($l>>>0);
 if ($11) {
  $12 = (($f) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (FUNCTION_TABLE_iiii[$13 & 3]($f,$s,$l)|0);
  $$0 = $14;
  STACKTOP = sp;return ($$0|0);
 }
 $15 = (($f) + 75|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)>(-1);
 L11: do {
  if ($17) {
   $i$0 = $l;
   while(1) {
    $18 = ($i$0|0)==(0);
    if ($18) {
     $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
     break L11;
    }
    $19 = (($i$0) + -1)|0;
    $20 = (($s) + ($19)|0);
    $21 = HEAP8[$20>>0]|0;
    $22 = ($21<<24>>24)==(10);
    if ($22) {
     break;
    } else {
     $i$0 = $19;
    }
   }
   $23 = (($f) + 36|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (FUNCTION_TABLE_iiii[$24 & 3]($f,$s,$i$0)|0);
   $26 = ($25>>>0)<($i$0>>>0);
   if ($26) {
    $$0 = $i$0;
    STACKTOP = sp;return ($$0|0);
   } else {
    $27 = (($s) + ($i$0)|0);
    $28 = (($l) - ($i$0))|0;
    $$pre = HEAP32[$5>>2]|0;
    $$01 = $28;$$02 = $27;$29 = $$pre;$i$1 = $i$0;
    break;
   }
  } else {
   $$01 = $l;$$02 = $s;$29 = $6;$i$1 = 0;
  }
 } while(0);
 _memcpy(($29|0),($$02|0),($$01|0))|0;
 $30 = HEAP32[$5>>2]|0;
 $31 = (($30) + ($$01)|0);
 HEAP32[$5>>2] = $31;
 $32 = (($i$1) + ($$01))|0;
 $$0 = $32;
 STACKTOP = sp;return ($$0|0);
}
function _sprintf($s,$fmt,$varargs) {
 $s = $s|0;
 $fmt = $fmt|0;
 $varargs = $varargs|0;
 var $0 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $ap = sp;
 HEAP32[$ap>>2] = $varargs;
 $0 = (_vsprintf($s,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _MUSL_vfprintf($f,$fmt,$ap) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$ = 0, $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ap2 = 0, $internal_buf = 0, $nl_arg = 0, $nl_type = 0, $ret$1 = 0, $vacopy_currentptr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $ap2 = sp + 120|0;
 $nl_type = sp + 80|0;
 $nl_arg = sp;
 $internal_buf = sp + 136|0;
 dest=$nl_type+0|0; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $vacopy_currentptr = HEAP32[$ap>>2]|0;
 HEAP32[$ap2>>2] = $vacopy_currentptr;
 $0 = (_printf_core(0,$fmt,$ap2,$nl_arg,$nl_type)|0);
 $1 = ($0|0)<(0);
 if ($1) {
  $$0 = -1;
  STACKTOP = sp;return ($$0|0);
 }
 $2 = (($f) + 48|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $6 = (($f) + 44|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$6>>2] = $internal_buf;
  $8 = (($f) + 28|0);
  HEAP32[$8>>2] = $internal_buf;
  $9 = (($f) + 20|0);
  HEAP32[$9>>2] = $internal_buf;
  HEAP32[$2>>2] = 80;
  $10 = (($internal_buf) + 80|0);
  $11 = (($f) + 16|0);
  HEAP32[$11>>2] = $10;
  $12 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
  $13 = ($7|0)==(0|0);
  if ($13) {
   $ret$1 = $12;
  } else {
   $14 = (($f) + 36|0);
   $15 = HEAP32[$14>>2]|0;
   (FUNCTION_TABLE_iiii[$15 & 3]($f,0,0)|0);
   $16 = HEAP32[$9>>2]|0;
   $17 = ($16|0)==(0|0);
   $$ = $17 ? -1 : $12;
   HEAP32[$6>>2] = $7;
   HEAP32[$2>>2] = 0;
   HEAP32[$11>>2] = 0;
   HEAP32[$8>>2] = 0;
   HEAP32[$9>>2] = 0;
   $ret$1 = $$;
  }
 } else {
  $5 = (_printf_core($f,$fmt,$ap2,$nl_arg,$nl_type)|0);
  $ret$1 = $5;
 }
 $$0 = $ret$1;
 STACKTOP = sp;return ($$0|0);
}
function _printf_core($f,$fmt,$ap,$nl_arg,$nl_type) {
 $f = $f|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 $nl_arg = $nl_arg|0;
 $nl_type = $nl_type|0;
 var $$ = 0, $$$5$i = 0, $$$i = 0, $$$p$i = 0, $$0 = 0, $$0$lcssa$i = 0, $$0$lcssa$i$i = 0, $$0$lcssa$i104$i = 0, $$0$lcssa$i128$i = 0, $$0$lcssa$i143$i = 0, $$0$lcssa$i38 = 0, $$0$lcssa$i39$i = 0, $$0$lcssa$i44 = 0, $$0$lcssa$i46 = 0, $$0$lcssa$i46$i = 0, $$0$lcssa$i49$i = 0, $$0$lcssa$i53 = 0, $$0$lcssa$i56$i = 0, $$0$lcssa$i60 = 0, $$0$lcssa$i63$i = 0;
 var $$0$lcssa$i67 = 0, $$0$lcssa$i69$i = 0, $$0$lcssa$i76$i = 0, $$0$lcssa$i78 = 0, $$0$lcssa$i85$i = 0, $$0$lcssa$i97$i = 0, $$01$i = 0, $$01$i$i = 0, $$01$i102$i = 0, $$01$i126$i = 0, $$01$i141$i = 0, $$01$i36 = 0, $$01$i37$i = 0, $$01$i44$i = 0, $$01$i51 = 0, $$01$i54$i = 0, $$01$i58 = 0, $$01$i61$i = 0, $$01$i65 = 0, $$01$i67$i = 0;
 var $$01$i74$i = 0, $$01$i76 = 0, $$01$i95$i = 0, $$01$lcssa$off0$i = 0, $$01$lcssa$off0$i$i = 0, $$01$lcssa$off0$i86$i = 0, $$012$i = 0, $$012$i73 = 0, $$03$i41 = 0, $$05$i = 0, $$05$i$i = 0, $$05$i80$i = 0, $$06$i = 0.0, $$1$i = 0.0, $$1$lcssa$i$i = 0, $$1$lcssa$i113$i = 0, $$113$i = 0, $$12$i = 0, $$12$i$i = 0, $$12$i111$i = 0;
 var $$12$i120$i = 0, $$12$i135$i = 0, $$12$i88$i = 0, $$14 = 0, $$15 = 0, $$16 = 0, $$18 = 0, $$2$i = 0.0, $$2$us$i = 0.0, $$2$us$us$i = 0.0, $$2$us160$i = 0.0, $$20$i = 0, $$20$us$i = 0, $$21$i = 0, $$22$i = 0.0, $$24$i = 0, $$26$i = 0, $$29$$24$i = 0, $$29$$26$i = 0, $$29$i = 0;
 var $$3$i = 0.0, $$310$i = 0, $$32$i = 0, $$4$i = 0.0, $$411$lcssa$i = 0, $$411176$i = 0, $$5194$i = 0, $$a$3$i = 0, $$a$3$us$i = 0, $$a$3$us319$i = 0, $$a$3$us320$i = 0, $$a$3321$i = 0, $$a$3322$i = 0, $$fl$4 = 0, $$lcssa94 = 0, $$mask$i = 0, $$mask$i31 = 0, $$mask1$i = 0, $$mask1$i30 = 0, $$neg152$i = 0;
 var $$neg153$i = 0, $$not$i = 0, $$p$5 = 0, $$p$i = 0, $$pn$i = 0, $$pr$i = 0, $$pr147$i = 0, $$pre = 0, $$pre$i = 0, $$pre291 = 0, $$pre292 = 0, $$pre312$i = 0, $$sink = 0, $$sink300$off32 = 0, $$sum$i = 0, $$sum14$i = 0, $$sum15$i = 0, $$z$3$i = 0, $$z$4$us$i = 0, $0 = 0;
 var $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0;
 var $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0;
 var $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0.0, $104 = 0, $1040 = 0.0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1048$phi = 0, $1049 = 0, $1049$phi = 0, $105 = 0;
 var $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0.0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0.0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0.0, $369 = 0, $37 = 0, $370 = 0.0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0.0, $407 = 0.0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0.0, $422 = 0, $423 = 0, $424 = 0, $425 = 0.0, $426 = 0.0, $427 = 0.0, $428 = 0.0, $429 = 0.0, $43 = 0, $430 = 0.0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0.0, $48 = 0, $480 = 0.0, $481 = 0.0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0.0, $495 = 0.0, $496 = 0.0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0.0, $51 = 0, $510 = 0.0;
 var $511 = 0.0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0.0, $525 = 0.0, $526 = 0.0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0;
 var $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0;
 var $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0.0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0;
 var $584 = 0.0, $585 = 0.0, $586 = 0.0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0;
 var $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0;
 var $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0;
 var $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0;
 var $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0;
 var $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0;
 var $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0;
 var $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0.0;
 var $728 = 0.0, $729 = 0, $73 = 0, $730 = 0.0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0;
 var $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0;
 var $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0;
 var $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0;
 var $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0;
 var $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0;
 var $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0;
 var $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0;
 var $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0;
 var $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0;
 var $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0;
 var $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0;
 var $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0;
 var $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0;
 var $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0;
 var $999 = 0, $a$0 = 0, $a$1 = 0, $a$1$lcssa$i = 0, $a$1257$i = 0, $a$2 = 0, $a$2$i = 0, $a$3$lcssa$i = 0, $a$3245$i = 0, $a$3245$us$i = 0, $a$6$i = 0, $a$7$i = 0, $arglist_current = 0, $arglist_current11 = 0, $arglist_current14 = 0, $arglist_current17 = 0, $arglist_current2 = 0, $arglist_current20 = 0, $arglist_current23 = 0, $arglist_current26 = 0;
 var $arglist_current29 = 0, $arglist_current32 = 0, $arglist_current35 = 0, $arglist_current38 = 0, $arglist_current41 = 0, $arglist_current44 = 0, $arglist_current47 = 0, $arglist_current5 = 0, $arglist_current50 = 0, $arglist_current53 = 0, $arglist_current56 = 0, $arglist_current59 = 0, $arglist_current62 = 0, $arglist_current8 = 0, $arglist_next = 0, $arglist_next12 = 0, $arglist_next15 = 0, $arglist_next18 = 0, $arglist_next21 = 0, $arglist_next24 = 0;
 var $arglist_next27 = 0, $arglist_next3 = 0, $arglist_next30 = 0, $arglist_next33 = 0, $arglist_next36 = 0, $arglist_next39 = 0, $arglist_next42 = 0, $arglist_next45 = 0, $arglist_next48 = 0, $arglist_next51 = 0, $arglist_next54 = 0, $arglist_next57 = 0, $arglist_next6 = 0, $arglist_next60 = 0, $arglist_next63 = 0, $arglist_next9 = 0, $argpos$0 = 0, $big$i = 0, $brmerge$i = 0, $buf = 0;
 var $buf$i = 0, $carry$0250$i = 0, $carry3$0239$i = 0, $carry3$0239$us$i = 0, $cnt$0 = 0, $cnt$1 = 0, $d$0251$i = 0, $d$1238$i = 0, $d$1238$us$i = 0, $d$2$a$3$i = 0, $d$2$lcssa$i = 0, $d$2220$i = 0, $d$3$i = 0, $d$4184$i = 0, $d$5175$i = 0, $d$6193$i = 0, $e$0233$i = 0, $e$1$i = 0, $e$2215$i = 0, $e$3$i = 0;
 var $e$4$i = 0, $e2$i = 0, $ebuf0$i = 0, $estr$0$i = 0, $estr$1$lcssa$i = 0, $estr$1$ph$i = 0, $estr$1201$i = 0, $estr$2$i = 0, $exitcond$i = 0, $fl$0102 = 0, $fl$0106 = 0, $fl$1 = 0, $fl$1$ = 0, $fl$3 = 0, $fl$4 = 0, $fl$6 = 0, $i$0$lcssa = 0, $i$0167 = 0, $i$0169 = 0, $i$0234$i = 0;
 var $i$03$i = 0, $i$03$i23 = 0, $i$1$lcssa$i = 0, $i$1175 = 0, $i$1226$i = 0, $i$2216$i = 0, $i$291 = 0, $i$3210$i = 0, $i$390 = 0, $isdigit = 0, $isdigit$i = 0, $isdigit$i25 = 0, $isdigit10 = 0, $isdigit12 = 0, $isdigit2$i = 0, $isdigit2$i22 = 0, $isdigittmp = 0, $isdigittmp$i = 0, $isdigittmp$i24 = 0, $isdigittmp1$i = 0;
 var $isdigittmp1$i21 = 0, $isdigittmp11 = 0, $isdigittmp9 = 0, $j$0$i = 0, $j$0225$i = 0, $j$0227$i = 0, $j$1211$i = 0, $j$2$i = 0, $l$0 = 0, $l$0$i = 0, $l$1$i = 0, $l$1$lcssa = 0, $l$1168 = 0, $l10n$0 = 0, $l10n$0$phi = 0, $l10n$1 = 0, $l10n$2 = 0, $l10n$3 = 0, $mb = 0, $or$cond = 0;
 var $or$cond$i = 0, $or$cond$i$i = 0, $or$cond$i101$i = 0, $or$cond$i36$i = 0, $or$cond$i43$i = 0, $or$cond$i50 = 0, $or$cond$i53$i = 0, $or$cond$i57 = 0, $or$cond$i60$i = 0, $or$cond$i64 = 0, $or$cond$i72 = 0, $or$cond$i73$i = 0, $or$cond$i75 = 0, $or$cond$i94$i = 0, $or$cond19$i = 0, $or$cond19315$i = 0, $or$cond23$i = 0, $or$cond29$i = 0, $or$cond29174$i = 0, $or$cond3$i = 0;
 var $or$cond30$i = 0, $p$0 = 0, $p$1 = 0, $p$2 = 0, $p$2$ = 0, $p$4296 = 0, $p$5 = 0, $pad$i = 0, $pl$0 = 0, $pl$0$i = 0, $pl$1 = 0, $pl$1$i = 0, $pl$2 = 0, $prefix$0 = 0, $prefix$0$$i = 0, $prefix$0$i = 0, $prefix$1 = 0, $prefix$2 = 0, $r$0$a$7$i = 0, $re$0$i = 0;
 var $re$1166$i = 0, $round$0165$i = 0.0, $round6$1$i = 0.0, $s$0$i = 0, $s$0$us$i = 0, $s$0$us$us$i = 0, $s$0$us159$i = 0, $s$1$i = 0, $s$1$lcssa$i = 0, $s$1$us$i = 0, $s$1$us$us$i = 0, $s$1$us161$i = 0, $s1$0$i = 0, $s7$0181$i = 0, $s7$1$i = 0, $s8$0$lcssa$i = 0, $s8$0170$i = 0, $s9$0$i = 0, $s9$1189$i = 0, $s9$2$i = 0;
 var $sext = 0, $sext86 = 0, $small$0$i = 0.0, $small$1$i = 0.0, $st$0 = 0, $storemerge = 0, $storemerge13 = 0, $storemerge8100 = 0, $storemerge8105 = 0, $t$0 = 0, $t$1 = 0, $w$$i = 0, $w$0 = 0, $w$1 = 0, $w$18$i = 0, $w$2 = 0, $w$31$i = 0, $wc = 0, $ws$0170 = 0, $ws$1176 = 0;
 var $y$03$i = 0, $y$03$i$i = 0, $y$03$i110$i = 0, $y$03$i119$i = 0, $y$03$i134$i = 0, $y$03$i87$i = 0, $z$0$i = 0, $z$0$lcssa = 0, $z$095 = 0, $z$1$lcssa$i = 0, $z$1256$i = 0, $z$2 = 0, $z$2$i = 0, $z$2316$i = 0, $z$2317$i = 0, $z$3$lcssa$i = 0, $z$3244$i = 0, $z$3244$us$i = 0, $z$4$i = 0, $z$4$us$i = 0;
 var $z$5$i = 0, $z$6$i = 0, $z$7$$i = 0, $z$7$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 848|0;
 $big$i = sp + 16|0;
 $e2$i = sp + 8|0;
 $buf$i = sp + 812|0;
 $0 = $buf$i;
 $ebuf0$i = sp + 800|0;
 $pad$i = sp + 544|0;
 $buf = sp + 504|0;
 $wc = sp;
 $mb = sp + 496|0;
 $1 = ($f|0)!=(0|0);
 $2 = (($buf) + 40|0);
 $3 = $2;
 $4 = (($buf) + 39|0);
 $5 = (($wc) + 4|0);
 $6 = (($ebuf0$i) + 12|0);
 $7 = (($ebuf0$i) + 11|0);
 $8 = $6;
 $9 = (($8) - ($0))|0;
 $10 = (-2 - ($0))|0;
 $11 = (($8) + 2)|0;
 $12 = (($big$i) + 264|0);
 $13 = (($buf$i) + 9|0);
 $14 = $13;
 $15 = (($buf$i) + 8|0);
 $1048 = 0;$1049 = 0;$22 = $fmt;$cnt$0 = 0;$l$0 = 0;$l10n$0 = 0;
 L1: while(1) {
  $16 = ($cnt$0|0)>(-1);
  do {
   if ($16) {
    $17 = (2147483647 - ($cnt$0))|0;
    $18 = ($l$0|0)>($17|0);
    if ($18) {
     $19 = (___errno_location()|0);
     HEAP32[$19>>2] = 75;
     $cnt$1 = -1;
     break;
    } else {
     $20 = (($l$0) + ($cnt$0))|0;
     $cnt$1 = $20;
     break;
    }
   } else {
    $cnt$1 = $cnt$0;
   }
  } while(0);
  $21 = HEAP8[$22>>0]|0;
  $23 = ($21<<24>>24)==(0);
  if ($23) {
   label = 339;
   break;
  } else {
   $1050 = $21;$25 = $22;
  }
  while(1) {
   if ((($1050<<24>>24) == 37)) {
    $27 = $25;$z$095 = $25;
    label = 9;
    break;
   } else if ((($1050<<24>>24) == 0)) {
    $$lcssa94 = $25;$z$0$lcssa = $25;
    break;
   }
   $24 = (($25) + 1|0);
   $$pre = HEAP8[$24>>0]|0;
   $1050 = $$pre;$25 = $24;
  }
  L12: do {
   if ((label|0) == 9) {
    while(1) {
     label = 0;
     $26 = (($27) + 1|0);
     $28 = HEAP8[$26>>0]|0;
     $29 = ($28<<24>>24)==(37);
     if (!($29)) {
      $$lcssa94 = $27;$z$0$lcssa = $z$095;
      break L12;
     }
     $30 = (($z$095) + 1|0);
     $31 = (($27) + 2|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = ($32<<24>>24)==(37);
     if ($33) {
      $27 = $31;$z$095 = $30;
      label = 9;
     } else {
      $$lcssa94 = $31;$z$0$lcssa = $30;
      break;
     }
    }
   }
  } while(0);
  $34 = $z$0$lcssa;
  $35 = $22;
  $36 = (($34) - ($35))|0;
  if ($1) {
   (___fwritex($22,$36,$f)|0);
  }
  $37 = ($z$0$lcssa|0)==($22|0);
  if (!($37)) {
   $l10n$0$phi = $l10n$0;$1049$phi = $1049;$1048$phi = $1048;$22 = $$lcssa94;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$0$phi;$1049 = $1049$phi;$1048 = $1048$phi;
   continue;
  }
  $38 = (($$lcssa94) + 1|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39 << 24 >> 24;
  $isdigittmp = (($40) + -48)|0;
  $isdigit = ($isdigittmp>>>0)<(10);
  if ($isdigit) {
   $41 = (($$lcssa94) + 2|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = ($42<<24>>24)==(36);
   if ($43) {
    $44 = (($$lcssa94) + 3|0);
    $$pre291 = HEAP8[$44>>0]|0;
    $46 = $$pre291;$argpos$0 = $isdigittmp;$l10n$1 = 1;$storemerge = $44;
   } else {
    $46 = $39;$argpos$0 = -1;$l10n$1 = $l10n$0;$storemerge = $38;
   }
  } else {
   $46 = $39;$argpos$0 = -1;$l10n$1 = $l10n$0;$storemerge = $38;
  }
  $45 = $46 << 24 >> 24;
  $47 = (($45) + -32)|0;
  $48 = ($47>>>0)<(32);
  L25: do {
   if ($48) {
    $50 = $45;$55 = $46;$fl$0106 = 0;$storemerge8105 = $storemerge;
    while(1) {
     $49 = (($50) + -32)|0;
     $51 = 1 << $49;
     $52 = $51 & 75913;
     $53 = ($52|0)==(0);
     if ($53) {
      $65 = $55;$fl$0102 = $fl$0106;$storemerge8100 = $storemerge8105;
      break L25;
     }
     $54 = $55 << 24 >> 24;
     $56 = (($54) + -32)|0;
     $57 = 1 << $56;
     $58 = $57 | $fl$0106;
     $59 = (($storemerge8105) + 1|0);
     $60 = HEAP8[$59>>0]|0;
     $61 = $60 << 24 >> 24;
     $62 = (($61) + -32)|0;
     $63 = ($62>>>0)<(32);
     if ($63) {
      $50 = $61;$55 = $60;$fl$0106 = $58;$storemerge8105 = $59;
     } else {
      $65 = $60;$fl$0102 = $58;$storemerge8100 = $59;
      break;
     }
    }
   } else {
    $65 = $46;$fl$0102 = 0;$storemerge8100 = $storemerge;
   }
  } while(0);
  $64 = ($65<<24>>24)==(42);
  do {
   if ($64) {
    $66 = (($storemerge8100) + 1|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67 << 24 >> 24;
    $isdigittmp11 = (($68) + -48)|0;
    $isdigit12 = ($isdigittmp11>>>0)<(10);
    if ($isdigit12) {
     $69 = (($storemerge8100) + 2|0);
     $70 = HEAP8[$69>>0]|0;
     $71 = ($70<<24>>24)==(36);
     if ($71) {
      $72 = (($nl_type) + ($isdigittmp11<<2)|0);
      HEAP32[$72>>2] = 10;
      $73 = HEAP8[$66>>0]|0;
      $74 = $73 << 24 >> 24;
      $75 = (($74) + -48)|0;
      $76 = (($nl_arg) + ($75<<3)|0);
      $77 = $76;
      $78 = $77;
      $79 = HEAP32[$78>>2]|0;
      $80 = (($77) + 4)|0;
      $81 = $80;
      $82 = HEAP32[$81>>2]|0;
      $83 = (($storemerge8100) + 3|0);
      $l10n$2 = 1;$storemerge13 = $83;$w$0 = $79;
     } else {
      label = 24;
     }
    } else {
     label = 24;
    }
    if ((label|0) == 24) {
     label = 0;
     $84 = ($l10n$1|0)==(0);
     if (!($84)) {
      $$0 = -1;
      label = 357;
      break L1;
     }
     if (!($1)) {
      $100 = $66;$fl$1 = $fl$0102;$l10n$3 = 0;$w$1 = 0;
      break;
     }
     $arglist_current = HEAP32[$ap>>2]|0;
     $85 = HEAP32[$arglist_current>>2]|0;
     $arglist_next = (($arglist_current) + 4|0);
     HEAP32[$ap>>2] = $arglist_next;
     $l10n$2 = 0;$storemerge13 = $66;$w$0 = $85;
    }
    $86 = ($w$0|0)<(0);
    if ($86) {
     $87 = $fl$0102 | 8192;
     $88 = (0 - ($w$0))|0;
     $100 = $storemerge13;$fl$1 = $87;$l10n$3 = $l10n$2;$w$1 = $88;
    } else {
     $100 = $storemerge13;$fl$1 = $fl$0102;$l10n$3 = $l10n$2;$w$1 = $w$0;
    }
   } else {
    $89 = $65 << 24 >> 24;
    $isdigittmp1$i = (($89) + -48)|0;
    $isdigit2$i = ($isdigittmp1$i>>>0)<(10);
    if ($isdigit2$i) {
     $92 = $89;$95 = $storemerge8100;$i$03$i = 0;
     while(1) {
      $90 = ($i$03$i*10)|0;
      $91 = (($92) + -48)|0;
      $93 = (($91) + ($90))|0;
      $94 = (($95) + 1|0);
      $96 = HEAP8[$94>>0]|0;
      $97 = $96 << 24 >> 24;
      $isdigittmp$i = (($97) + -48)|0;
      $isdigit$i = ($isdigittmp$i>>>0)<(10);
      if ($isdigit$i) {
       $92 = $97;$95 = $94;$i$03$i = $93;
      } else {
       break;
      }
     }
     $98 = ($93|0)<(0);
     if ($98) {
      $$0 = -1;
      label = 357;
      break L1;
     } else {
      $100 = $94;$fl$1 = $fl$0102;$l10n$3 = $l10n$1;$w$1 = $93;
     }
    } else {
     $100 = $storemerge8100;$fl$1 = $fl$0102;$l10n$3 = $l10n$1;$w$1 = 0;
    }
   }
  } while(0);
  $99 = HEAP8[$100>>0]|0;
  $101 = ($99<<24>>24)==(46);
  L46: do {
   if ($101) {
    $102 = (($100) + 1|0);
    $103 = HEAP8[$102>>0]|0;
    $104 = ($103<<24>>24)==(42);
    if (!($104)) {
     $125 = $103 << 24 >> 24;
     $isdigittmp1$i21 = (($125) + -48)|0;
     $isdigit2$i22 = ($isdigittmp1$i21>>>0)<(10);
     if ($isdigit2$i22) {
      $128 = $125;$131 = $102;$i$03$i23 = 0;
     } else {
      $1051 = $102;$p$0 = 0;
      break;
     }
     while(1) {
      $126 = ($i$03$i23*10)|0;
      $127 = (($128) + -48)|0;
      $129 = (($127) + ($126))|0;
      $130 = (($131) + 1|0);
      $132 = HEAP8[$130>>0]|0;
      $133 = $132 << 24 >> 24;
      $isdigittmp$i24 = (($133) + -48)|0;
      $isdigit$i25 = ($isdigittmp$i24>>>0)<(10);
      if ($isdigit$i25) {
       $128 = $133;$131 = $130;$i$03$i23 = $129;
      } else {
       $1051 = $130;$p$0 = $129;
       break L46;
      }
     }
    }
    $105 = (($100) + 2|0);
    $106 = HEAP8[$105>>0]|0;
    $107 = $106 << 24 >> 24;
    $isdigittmp9 = (($107) + -48)|0;
    $isdigit10 = ($isdigittmp9>>>0)<(10);
    if ($isdigit10) {
     $108 = (($100) + 3|0);
     $109 = HEAP8[$108>>0]|0;
     $110 = ($109<<24>>24)==(36);
     if ($110) {
      $111 = (($nl_type) + ($isdigittmp9<<2)|0);
      HEAP32[$111>>2] = 10;
      $112 = HEAP8[$105>>0]|0;
      $113 = $112 << 24 >> 24;
      $114 = (($113) + -48)|0;
      $115 = (($nl_arg) + ($114<<3)|0);
      $116 = $115;
      $117 = $116;
      $118 = HEAP32[$117>>2]|0;
      $119 = (($116) + 4)|0;
      $120 = $119;
      $121 = HEAP32[$120>>2]|0;
      $122 = (($100) + 4|0);
      $1051 = $122;$p$0 = $118;
      break;
     }
    }
    $123 = ($l10n$3|0)==(0);
    if (!($123)) {
     $$0 = -1;
     label = 357;
     break L1;
    }
    if ($1) {
     $arglist_current2 = HEAP32[$ap>>2]|0;
     $124 = HEAP32[$arglist_current2>>2]|0;
     $arglist_next3 = (($arglist_current2) + 4|0);
     HEAP32[$ap>>2] = $arglist_next3;
     $1051 = $105;$p$0 = $124;
    } else {
     $1051 = $105;$p$0 = 0;
    }
   } else {
    $1051 = $100;$p$0 = -1;
   }
  } while(0);
  $135 = $1051;$st$0 = 0;
  while(1) {
   $134 = HEAP8[$135>>0]|0;
   $136 = $134 << 24 >> 24;
   $137 = (($136) + -65)|0;
   $138 = ($137>>>0)>(57);
   if ($138) {
    $$0 = -1;
    label = 357;
    break L1;
   }
   $139 = (($135) + 1|0);
   $140 = ((14912 + (($st$0*58)|0)|0) + ($137)|0);
   $141 = HEAP8[$140>>0]|0;
   $142 = $141&255;
   $143 = (($142) + -1)|0;
   $144 = ($143>>>0)<(8);
   if ($144) {
    $135 = $139;$st$0 = $142;
   } else {
    break;
   }
  }
  $145 = ($141<<24>>24)==(0);
  if ($145) {
   $$0 = -1;
   label = 357;
   break;
  }
  $146 = ($141<<24>>24)==(19);
  $147 = ($argpos$0|0)>(-1);
  L65: do {
   if ($146) {
    if ($147) {
     $$0 = -1;
     label = 357;
     break L1;
    }
    if ($1) {
     $198 = $134;$206 = $1049;$228 = $1048;
    } else {
     $1049$phi = $1049;$1048$phi = $1048;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;$1049 = $1049$phi;$1048 = $1048$phi;
     continue L1;
    }
   } else {
    L69: do {
     if ($147) {
      $148 = (($nl_type) + ($argpos$0<<2)|0);
      HEAP32[$148>>2] = $142;
      $149 = (($nl_arg) + ($argpos$0<<3)|0);
      $150 = $149;
      $151 = $150;
      $152 = HEAP32[$151>>2]|0;
      $153 = (($150) + 4)|0;
      $154 = $153;
      $155 = HEAP32[$154>>2]|0;
      $$sink = $152;$$sink300$off32 = $155;
      label = 64;
     } else {
      if (!($1)) {
       $$0 = 0;
       label = 357;
       break L1;
      }
      $156 = ($141&255)>(20);
      if ($156) {
       $198 = $134;$206 = $1049;$228 = $1048;
       break L65;
      }
      do {
       switch ($142|0) {
       case 11:  {
        $arglist_current11 = HEAP32[$ap>>2]|0;
        $162 = HEAP32[$arglist_current11>>2]|0;
        $arglist_next12 = (($arglist_current11) + 4|0);
        HEAP32[$ap>>2] = $arglist_next12;
        $163 = $162;
        $1052 = $163;$1053 = 0;
        break L69;
        break;
       }
       case 18:  {
        $arglist_current32 = HEAP32[$ap>>2]|0;
        HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current32>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current32+4>>2];$193 = +HEAPF64[tempDoublePtr>>3];
        $arglist_next33 = (($arglist_current32) + 8|0);
        HEAP32[$ap>>2] = $arglist_next33;
        HEAPF64[tempDoublePtr>>3] = $193;$194 = HEAP32[tempDoublePtr>>2]|0;
        $195 = HEAP32[tempDoublePtr+4>>2]|0;
        $$sink = $194;$$sink300$off32 = $195;
        label = 64;
        break L69;
        break;
       }
       case 10:  {
        $arglist_current8 = HEAP32[$ap>>2]|0;
        $158 = HEAP32[$arglist_current8>>2]|0;
        $arglist_next9 = (($arglist_current8) + 4|0);
        HEAP32[$ap>>2] = $arglist_next9;
        $159 = ($158|0)<(0);
        $160 = $159 << 31 >> 31;
        $161 = $158;
        $1052 = $161;$1053 = $160;
        break L69;
        break;
       }
       case 12:  {
        $arglist_current14 = HEAP32[$ap>>2]|0;
        $164 = $arglist_current14;
        $165 = $164;
        $166 = HEAP32[$165>>2]|0;
        $167 = (($164) + 4)|0;
        $168 = $167;
        $169 = HEAP32[$168>>2]|0;
        $arglist_next15 = (($arglist_current14) + 8|0);
        HEAP32[$ap>>2] = $arglist_next15;
        $170 = $166;
        $1052 = $170;$1053 = $169;
        break L69;
        break;
       }
       case 9:  {
        $arglist_current5 = HEAP32[$ap>>2]|0;
        $157 = HEAP32[$arglist_current5>>2]|0;
        $arglist_next6 = (($arglist_current5) + 4|0);
        HEAP32[$ap>>2] = $arglist_next6;
        $1052 = $157;$1053 = $1048;
        break L69;
        break;
       }
       case 15:  {
        $arglist_current23 = HEAP32[$ap>>2]|0;
        $180 = HEAP32[$arglist_current23>>2]|0;
        $arglist_next24 = (($arglist_current23) + 4|0);
        HEAP32[$ap>>2] = $arglist_next24;
        $181 = $180&255;
        $182 = $181 << 24 >> 24;
        $183 = ($182|0)<(0);
        $184 = $183 << 31 >> 31;
        $sext = $180 << 24;
        $185 = $sext >> 24;
        $186 = $185;
        $1052 = $186;$1053 = $184;
        break L69;
        break;
       }
       case 13:  {
        $arglist_current17 = HEAP32[$ap>>2]|0;
        $171 = HEAP32[$arglist_current17>>2]|0;
        $arglist_next18 = (($arglist_current17) + 4|0);
        HEAP32[$ap>>2] = $arglist_next18;
        $172 = $171&65535;
        $173 = $172 << 16 >> 16;
        $174 = ($173|0)<(0);
        $175 = $174 << 31 >> 31;
        $sext86 = $171 << 16;
        $176 = $sext86 >> 16;
        $177 = $176;
        $1052 = $177;$1053 = $175;
        break L69;
        break;
       }
       case 16:  {
        $arglist_current26 = HEAP32[$ap>>2]|0;
        $187 = HEAP32[$arglist_current26>>2]|0;
        $arglist_next27 = (($arglist_current26) + 4|0);
        HEAP32[$ap>>2] = $arglist_next27;
        $$mask$i31 = $187 & 255;
        $188 = $$mask$i31;
        $1052 = $188;$1053 = 0;
        break L69;
        break;
       }
       case 17:  {
        $arglist_current29 = HEAP32[$ap>>2]|0;
        HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current29>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current29+4>>2];$189 = +HEAPF64[tempDoublePtr>>3];
        $arglist_next30 = (($arglist_current29) + 8|0);
        HEAP32[$ap>>2] = $arglist_next30;
        HEAPF64[tempDoublePtr>>3] = $189;$190 = HEAP32[tempDoublePtr>>2]|0;
        $191 = HEAP32[tempDoublePtr+4>>2]|0;
        $192 = $190;
        $1052 = $192;$1053 = $191;
        break L69;
        break;
       }
       case 14:  {
        $arglist_current20 = HEAP32[$ap>>2]|0;
        $178 = HEAP32[$arglist_current20>>2]|0;
        $arglist_next21 = (($arglist_current20) + 4|0);
        HEAP32[$ap>>2] = $arglist_next21;
        $$mask1$i30 = $178 & 65535;
        $179 = $$mask1$i30;
        $1052 = $179;$1053 = 0;
        break L69;
        break;
       }
       default: {
        $1052 = $1049;$1053 = $1048;
        break L69;
       }
       }
      } while(0);
     }
    } while(0);
    if ((label|0) == 64) {
     label = 0;
     $196 = $$sink;
     if ($1) {
      $1052 = $196;$1053 = $$sink300$off32;
     } else {
      $1048 = $$sink300$off32;$1049 = $196;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
      continue L1;
     }
    }
    $$pre292 = HEAP8[$135>>0]|0;
    $198 = $$pre292;$206 = $1052;$228 = $1053;
   }
  } while(0);
  $197 = $198 << 24 >> 24;
  $199 = ($st$0|0)==(0);
  if ($199) {
   $t$0 = $197;
  } else {
   $200 = $197 & 15;
   $201 = ($200|0)==(3);
   $202 = $197 & -33;
   $$ = $201 ? $202 : $197;
   $t$0 = $$;
  }
  $203 = $fl$1 & 8192;
  $204 = ($203|0)==(0);
  $205 = $fl$1 & -65537;
  $fl$1$ = $204 ? $fl$1 : $205;
  L92: do {
   switch ($t$0|0) {
   case 105: case 100:  {
    $266 = $206;
    $267 = ($228|0)<(0);
    if ($267) {
     $268 = (_i64Subtract(0,0,($266|0),($228|0))|0);
     $269 = tempRet0;
     $270 = $268;
     $276 = $270;$278 = $269;$pl$0 = 1;$prefix$0 = 15376;
     label = 90;
     break L92;
    }
    $271 = $fl$1$ & 2048;
    $272 = ($271|0)==(0);
    if ($272) {
     $273 = $fl$1$ & 1;
     $274 = ($273|0)==(0);
     $$16 = $274 ? 15376 : ((15376 + 2|0));
     $276 = $206;$278 = $228;$pl$0 = $273;$prefix$0 = $$16;
     label = 90;
    } else {
     $276 = $206;$278 = $228;$pl$0 = 1;$prefix$0 = ((15376 + 1|0));
     label = 90;
    }
    break;
   }
   case 88: case 120:  {
    $fl$3 = $fl$1$;$p$1 = $p$0;$t$1 = $t$0;
    label = 78;
    break;
   }
   case 117:  {
    $276 = $206;$278 = $228;$pl$0 = 0;$prefix$0 = 15376;
    label = 90;
    break;
   }
   case 83:  {
    $331 = ($p$0|0)==(0);
    if ($331) {
     $1058 = $206;$1059 = $206;$i$0167 = 0;
     label = 111;
    } else {
     $1056 = $206;$1057 = $206;$p$4296 = $p$0;
     label = 105;
    }
    break;
   }
   case 112:  {
    $221 = ($p$0>>>0)>(8);
    $222 = $221 ? $p$0 : 8;
    $223 = $fl$1$ | 8;
    $fl$3 = $223;$p$1 = $222;$t$1 = 120;
    label = 78;
    break;
   }
   case 67:  {
    $330 = $206;
    HEAP32[$wc>>2] = $330;
    HEAP32[$5>>2] = 0;
    $1056 = $wc;$1057 = $wc;$p$4296 = -1;
    label = 105;
    break;
   }
   case 99:  {
    $318 = $206;
    $319 = $318&255;
    HEAP8[$4>>0] = $319;
    $1054 = $228;$1055 = $206;$a$2 = $4;$fl$6 = $205;$p$5 = 1;$pl$2 = 0;$prefix$2 = 15376;$z$2 = $2;
    break;
   }
   case 65: case 71: case 70: case 69: case 97: case 103: case 102: case 101:  {
    $367 = $206;
    HEAP32[tempDoublePtr>>2] = $367;HEAP32[tempDoublePtr+4>>2] = $228;$368 = +HEAPF64[tempDoublePtr>>3];
    HEAP32[$e2$i>>2] = 0;
    $369 = ($228|0)<(0);
    if ($369) {
     $370 = -$368;
     $$06$i = $370;$pl$0$i = 1;$prefix$0$i = 15400;
    } else {
     $371 = $fl$1$ & 2048;
     $372 = ($371|0)==(0);
     if ($372) {
      $373 = $fl$1$ & 1;
      $374 = ($373|0)==(0);
      $$$i = $374 ? ((15400 + 1|0)) : ((15400 + 6|0));
      $$06$i = $368;$pl$0$i = $373;$prefix$0$i = $$$i;
     } else {
      $$06$i = $368;$pl$0$i = 1;$prefix$0$i = ((15400 + 3|0));
     }
    }
    HEAPF64[tempDoublePtr>>3] = $$06$i;$375 = HEAP32[tempDoublePtr>>2]|0;
    $376 = HEAP32[tempDoublePtr+4>>2]|0;
    $377 = $376 & 2146435072;
    $378 = ($377>>>0)<(2146435072);
    $379 = ($377|0)==(2146435072);
    $380 = (0)<(0);
    $381 = $379 & $380;
    $382 = $378 | $381;
    if (!($382)) {
     $383 = $t$0 & 32;
     $384 = ($383|0)!=(0);
     $385 = $384 ? 15424 : 15432;
     $386 = ($$06$i != $$06$i) | (0.0 != 0.0);
     if ($386) {
      $387 = $384 ? 15440 : 15448;
      $pl$1$i = 0;$s1$0$i = $387;
     } else {
      $pl$1$i = $pl$0$i;$s1$0$i = $385;
     }
     $388 = (($pl$1$i) + 3)|0;
     $389 = $fl$1$ & 8192;
     $390 = ($389|0)==(0);
     $391 = ($388|0)<($w$1|0);
     $or$cond$i36$i = $390 & $391;
     if ($or$cond$i36$i) {
      $392 = (($w$1) - ($388))|0;
      $393 = ($392>>>0)>(256);
      $394 = $393 ? 256 : $392;
      _memset(($pad$i|0),32,($394|0))|0;
      $395 = ($392>>>0)>(255);
      if ($395) {
       $$01$i37$i = $392;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $396 = (($$01$i37$i) + -256)|0;
        $397 = ($396>>>0)>(255);
        if ($397) {
         $$01$i37$i = $396;
        } else {
         $$0$lcssa$i39$i = $396;
         break;
        }
       }
      } else {
       $$0$lcssa$i39$i = $392;
      }
      (___fwritex($pad$i,$$0$lcssa$i39$i,$f)|0);
     }
     (___fwritex($prefix$0$i,$pl$1$i,$f)|0);
     (___fwritex($s1$0$i,3,$f)|0);
     $398 = $fl$1$ & 73728;
     $399 = ($398|0)==(8192);
     $or$cond$i43$i = $399 & $391;
     if ($or$cond$i43$i) {
      $400 = (($w$1) - ($388))|0;
      $401 = ($400>>>0)>(256);
      $402 = $401 ? 256 : $400;
      _memset(($pad$i|0),32,($402|0))|0;
      $403 = ($400>>>0)>(255);
      if ($403) {
       $$01$i44$i = $400;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $404 = (($$01$i44$i) + -256)|0;
        $405 = ($404>>>0)>(255);
        if ($405) {
         $$01$i44$i = $404;
        } else {
         $$0$lcssa$i46$i = $404;
         break;
        }
       }
      } else {
       $$0$lcssa$i46$i = $400;
      }
      (___fwritex($pad$i,$$0$lcssa$i46$i,$f)|0);
     }
     $w$$i = $391 ? $w$1 : $388;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$$i;$l10n$0 = $l10n$3;
     continue L1;
    }
    $406 = (+_frexpl($$06$i,$e2$i));
    $407 = $406 * 2.0;
    $408 = $407 != 0.0;
    if ($408) {
     $409 = HEAP32[$e2$i>>2]|0;
     $410 = (($409) + -1)|0;
     HEAP32[$e2$i>>2] = $410;
    }
    $411 = $t$0 | 32;
    $412 = ($411|0)==(97);
    if ($412) {
     $413 = $t$0 & 32;
     $414 = ($413|0)==(0);
     $415 = (($prefix$0$i) + 9|0);
     $prefix$0$$i = $414 ? $prefix$0$i : $415;
     $416 = $pl$0$i | 2;
     $417 = ($p$0>>>0)>(11);
     $418 = (12 - ($p$0))|0;
     $re$0$i = $417 ? 0 : $418;
     $419 = ($re$0$i|0)==(0);
     do {
      if ($419) {
       $$1$i = $407;
      } else {
       $re$1166$i = $re$0$i;$round$0165$i = 8.0;
       while(1) {
        $420 = (($re$1166$i) + -1)|0;
        $421 = $round$0165$i * 16.0;
        $422 = ($420|0)==(0);
        if ($422) {
         break;
        } else {
         $re$1166$i = $420;$round$0165$i = $421;
        }
       }
       $423 = HEAP8[$prefix$0$$i>>0]|0;
       $424 = ($423<<24>>24)==(45);
       if ($424) {
        $425 = -$407;
        $426 = $425 - $421;
        $427 = $421 + $426;
        $428 = -$427;
        $$1$i = $428;
        break;
       } else {
        $429 = $407 + $421;
        $430 = $429 - $421;
        $$1$i = $430;
        break;
       }
      }
     } while(0);
     $431 = HEAP32[$e2$i>>2]|0;
     $432 = ($431|0)<(0);
     $433 = (0 - ($431))|0;
     $434 = $432 ? $433 : $431;
     $435 = ($434|0)<(0);
     if ($435) {
      $436 = ($434|0)<(0);
      $437 = $436 << 31 >> 31;
      $$05$i$i = $6;$438 = $434;$439 = $437;
      while(1) {
       $440 = (___uremdi3(($438|0),($439|0),10,0)|0);
       $441 = tempRet0;
       $442 = $440 | 48;
       $443 = $442&255;
       $444 = (($$05$i$i) + -1|0);
       HEAP8[$444>>0] = $443;
       $445 = (___udivdi3(($438|0),($439|0),10,0)|0);
       $446 = tempRet0;
       $447 = ($439>>>0)>(9);
       $448 = ($439|0)==(9);
       $449 = ($438>>>0)>(4294967295);
       $450 = $448 & $449;
       $451 = $447 | $450;
       if ($451) {
        $$05$i$i = $444;$438 = $445;$439 = $446;
       } else {
        break;
       }
      }
      $$0$lcssa$i49$i = $444;$$01$lcssa$off0$i$i = $445;
     } else {
      $$0$lcssa$i49$i = $6;$$01$lcssa$off0$i$i = $434;
     }
     $452 = ($$01$lcssa$off0$i$i|0)==(0);
     if ($452) {
      $$1$lcssa$i$i = $$0$lcssa$i49$i;
     } else {
      $$12$i$i = $$0$lcssa$i49$i;$y$03$i$i = $$01$lcssa$off0$i$i;
      while(1) {
       $453 = (($y$03$i$i>>>0) % 10)&-1;
       $454 = $453 | 48;
       $455 = $454&255;
       $456 = (($$12$i$i) + -1|0);
       HEAP8[$456>>0] = $455;
       $457 = (($y$03$i$i>>>0) / 10)&-1;
       $458 = ($y$03$i$i>>>0)<(10);
       if ($458) {
        $$1$lcssa$i$i = $456;
        break;
       } else {
        $$12$i$i = $456;$y$03$i$i = $457;
       }
      }
     }
     $459 = ($$1$lcssa$i$i|0)==($6|0);
     if ($459) {
      HEAP8[$7>>0] = 48;
      $estr$0$i = $7;
     } else {
      $estr$0$i = $$1$lcssa$i$i;
     }
     $460 = HEAP32[$e2$i>>2]|0;
     $461 = $460 >> 31;
     $462 = $461 & 2;
     $463 = (($462) + 43)|0;
     $464 = $463&255;
     $465 = (($estr$0$i) + -1|0);
     HEAP8[$465>>0] = $464;
     $466 = (($t$0) + 15)|0;
     $467 = $466&255;
     $468 = (($estr$0$i) + -2|0);
     HEAP8[$468>>0] = $467;
     $469 = ($p$0|0)>(0);
     $470 = $fl$1$ & 8;
     $471 = ($470|0)==(0);
     if ($469) {
      if ($471) {
       $$2$us$us$i = $$1$i;$s$0$us$us$i = $buf$i;
       while(1) {
        $472 = (~~(($$2$us$us$i)));
        $473 = (15456 + ($472)|0);
        $474 = HEAP8[$473>>0]|0;
        $475 = $474&255;
        $476 = $475 | $413;
        $477 = $476&255;
        $478 = (($s$0$us$us$i) + 1|0);
        HEAP8[$s$0$us$us$i>>0] = $477;
        $479 = (+($472|0));
        $480 = $$2$us$us$i - $479;
        $481 = $480 * 16.0;
        $482 = $478;
        $483 = (($482) - ($0))|0;
        $484 = ($483|0)==(1);
        if ($484) {
         $485 = (($s$0$us$us$i) + 2|0);
         HEAP8[$478>>0] = 46;
         $s$1$us$us$i = $485;
        } else {
         $s$1$us$us$i = $478;
        }
        $486 = $481 != 0.0;
        if ($486) {
         $$2$us$us$i = $481;$s$0$us$us$i = $s$1$us$us$i;
        } else {
         $s$1$lcssa$i = $s$1$us$us$i;
         break;
        }
       }
      } else {
       $$2$us$i = $$1$i;$s$0$us$i = $buf$i;
       while(1) {
        $487 = (~~(($$2$us$i)));
        $488 = (15456 + ($487)|0);
        $489 = HEAP8[$488>>0]|0;
        $490 = $489&255;
        $491 = $490 | $413;
        $492 = $491&255;
        $493 = (($s$0$us$i) + 1|0);
        HEAP8[$s$0$us$i>>0] = $492;
        $494 = (+($487|0));
        $495 = $$2$us$i - $494;
        $496 = $495 * 16.0;
        $497 = $493;
        $498 = (($497) - ($0))|0;
        $499 = ($498|0)==(1);
        if ($499) {
         $500 = (($s$0$us$i) + 2|0);
         HEAP8[$493>>0] = 46;
         $s$1$us$i = $500;
        } else {
         $s$1$us$i = $493;
        }
        $501 = $496 != 0.0;
        if ($501) {
         $$2$us$i = $496;$s$0$us$i = $s$1$us$i;
        } else {
         $s$1$lcssa$i = $s$1$us$i;
         break;
        }
       }
      }
     } else {
      if ($471) {
       $$2$us160$i = $$1$i;$s$0$us159$i = $buf$i;
       while(1) {
        $502 = (~~(($$2$us160$i)));
        $503 = (15456 + ($502)|0);
        $504 = HEAP8[$503>>0]|0;
        $505 = $504&255;
        $506 = $505 | $413;
        $507 = $506&255;
        $508 = (($s$0$us159$i) + 1|0);
        HEAP8[$s$0$us159$i>>0] = $507;
        $509 = (+($502|0));
        $510 = $$2$us160$i - $509;
        $511 = $510 * 16.0;
        $512 = $508;
        $513 = (($512) - ($0))|0;
        $514 = ($513|0)==(1);
        $515 = $511 != 0.0;
        $or$cond$i72 = $514 & $515;
        if ($or$cond$i72) {
         $516 = (($s$0$us159$i) + 2|0);
         HEAP8[$508>>0] = 46;
         $s$1$us161$i = $516;
        } else {
         $s$1$us161$i = $508;
        }
        if ($515) {
         $$2$us160$i = $511;$s$0$us159$i = $s$1$us161$i;
        } else {
         $s$1$lcssa$i = $s$1$us161$i;
         break;
        }
       }
      } else {
       $$2$i = $$1$i;$s$0$i = $buf$i;
       while(1) {
        $517 = (~~(($$2$i)));
        $518 = (15456 + ($517)|0);
        $519 = HEAP8[$518>>0]|0;
        $520 = $519&255;
        $521 = $520 | $413;
        $522 = $521&255;
        $523 = (($s$0$i) + 1|0);
        HEAP8[$s$0$i>>0] = $522;
        $524 = (+($517|0));
        $525 = $$2$i - $524;
        $526 = $525 * 16.0;
        $527 = $523;
        $528 = (($527) - ($0))|0;
        $529 = ($528|0)==(1);
        if ($529) {
         $530 = (($s$0$i) + 2|0);
         HEAP8[$523>>0] = 46;
         $s$1$i = $530;
        } else {
         $s$1$i = $523;
        }
        $531 = $526 != 0.0;
        if ($531) {
         $$2$i = $526;$s$0$i = $s$1$i;
        } else {
         $s$1$lcssa$i = $s$1$i;
         break;
        }
       }
      }
     }
     $532 = ($p$0|0)==(0);
     $$pre312$i = $s$1$lcssa$i;
     do {
      if ($532) {
       label = 174;
      } else {
       $533 = (($10) + ($$pre312$i))|0;
       $534 = ($533|0)<($p$0|0);
       if (!($534)) {
        label = 174;
        break;
       }
       $535 = $468;
       $536 = (($11) + ($p$0))|0;
       $537 = (($536) - ($535))|0;
       $l$0$i = $537;
      }
     } while(0);
     if ((label|0) == 174) {
      label = 0;
      $538 = $468;
      $539 = (($9) - ($538))|0;
      $540 = (($539) + ($$pre312$i))|0;
      $l$0$i = $540;
     }
     $541 = (($l$0$i) + ($416))|0;
     $542 = $fl$1$ & 73728;
     $543 = ($542|0)==(0);
     $544 = ($541|0)<($w$1|0);
     $or$cond$i53$i = $543 & $544;
     if ($or$cond$i53$i) {
      $545 = (($w$1) - ($541))|0;
      $546 = ($545>>>0)>(256);
      $547 = $546 ? 256 : $545;
      _memset(($pad$i|0),32,($547|0))|0;
      $548 = ($545>>>0)>(255);
      if ($548) {
       $$01$i54$i = $545;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $549 = (($$01$i54$i) + -256)|0;
        $550 = ($549>>>0)>(255);
        if ($550) {
         $$01$i54$i = $549;
        } else {
         $$0$lcssa$i56$i = $549;
         break;
        }
       }
      } else {
       $$0$lcssa$i56$i = $545;
      }
      (___fwritex($pad$i,$$0$lcssa$i56$i,$f)|0);
     }
     (___fwritex($prefix$0$$i,$416,$f)|0);
     $551 = ($542|0)==(65536);
     $or$cond$i60$i = $551 & $544;
     if ($or$cond$i60$i) {
      $552 = (($w$1) - ($541))|0;
      $553 = ($552>>>0)>(256);
      $554 = $553 ? 256 : $552;
      _memset(($pad$i|0),48,($554|0))|0;
      $555 = ($552>>>0)>(255);
      if ($555) {
       $$01$i61$i = $552;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $556 = (($$01$i61$i) + -256)|0;
        $557 = ($556>>>0)>(255);
        if ($557) {
         $$01$i61$i = $556;
        } else {
         $$0$lcssa$i63$i = $556;
         break;
        }
       }
      } else {
       $$0$lcssa$i63$i = $552;
      }
      (___fwritex($pad$i,$$0$lcssa$i63$i,$f)|0);
     }
     $558 = (($$pre312$i) - ($0))|0;
     (___fwritex($buf$i,$558,$f)|0);
     $559 = $468;
     $560 = (($8) - ($559))|0;
     $561 = (($l$0$i) - ($560))|0;
     $562 = (($561) - ($558))|0;
     $563 = ($562|0)>(0);
     if ($563) {
      $564 = ($562>>>0)>(256);
      $565 = $564 ? 256 : $562;
      _memset(($pad$i|0),48,($565|0))|0;
      $566 = ($562>>>0)>(255);
      if ($566) {
       $$01$i67$i = $562;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $567 = (($$01$i67$i) + -256)|0;
        $568 = ($567>>>0)>(255);
        if ($568) {
         $$01$i67$i = $567;
        } else {
         $$0$lcssa$i69$i = $567;
         break;
        }
       }
      } else {
       $$0$lcssa$i69$i = $562;
      }
      (___fwritex($pad$i,$$0$lcssa$i69$i,$f)|0);
     }
     (___fwritex($468,$560,$f)|0);
     $569 = ($542|0)==(8192);
     $or$cond$i73$i = $569 & $544;
     if ($or$cond$i73$i) {
      $570 = (($w$1) - ($541))|0;
      $571 = ($570>>>0)>(256);
      $572 = $571 ? 256 : $570;
      _memset(($pad$i|0),32,($572|0))|0;
      $573 = ($570>>>0)>(255);
      if ($573) {
       $$01$i74$i = $570;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $574 = (($$01$i74$i) + -256)|0;
        $575 = ($574>>>0)>(255);
        if ($575) {
         $$01$i74$i = $574;
        } else {
         $$0$lcssa$i76$i = $574;
         break;
        }
       }
      } else {
       $$0$lcssa$i76$i = $570;
      }
      (___fwritex($pad$i,$$0$lcssa$i76$i,$f)|0);
     }
     $w$18$i = $544 ? $w$1 : $541;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$18$i;$l10n$0 = $l10n$3;
     continue L1;
    }
    $576 = ($p$0|0)<(0);
    $$p$i = $576 ? 6 : $p$0;
    if ($408) {
     $577 = $407 * 268435456.0;
     $578 = HEAP32[$e2$i>>2]|0;
     $579 = (($578) + -28)|0;
     HEAP32[$e2$i>>2] = $579;
     $$3$i = $577;$581 = $579;
    } else {
     $$pre$i = HEAP32[$e2$i>>2]|0;
     $$3$i = $407;$581 = $$pre$i;
    }
    $580 = ($581|0)<(0);
    $$32$i = $580 ? $big$i : $12;
    $$4$i = $$3$i;$z$0$i = $$32$i;
    while(1) {
     $582 = (~~(($$4$i))>>>0);
     HEAP32[$z$0$i>>2] = $582;
     $583 = (($z$0$i) + 4|0);
     $584 = (+($582>>>0));
     $585 = $$4$i - $584;
     $586 = $585 * 1.0E+9;
     $587 = $586 != 0.0;
     if ($587) {
      $$4$i = $586;$z$0$i = $583;
     } else {
      break;
     }
    }
    $$pr$i = HEAP32[$e2$i>>2]|0;
    $588 = ($$pr$i|0)>(0);
    if ($588) {
     $590 = $$pr$i;$a$1257$i = $$32$i;$z$1256$i = $583;
     while(1) {
      $589 = ($590|0)>(29);
      $591 = $589 ? 29 : $590;
      $592 = (($z$1256$i) + -4|0);
      $593 = ($592>>>0)<($a$1257$i>>>0);
      do {
       if ($593) {
        $594 = HEAP32[$592>>2]|0;
        $595 = ($594|0)==(0);
        $596 = ($z$1256$i>>>0)>($a$1257$i>>>0);
        $or$cond19315$i = $595 & $596;
        $z$2316$i = $or$cond19315$i ? $592 : $z$1256$i;
        $a$2$i = $a$1257$i;$z$2317$i = $z$2316$i;
       } else {
        $carry$0250$i = 0;$d$0251$i = $592;
        while(1) {
         $597 = HEAP32[$d$0251$i>>2]|0;
         $598 = (_bitshift64Shl(($597|0),0,($591|0))|0);
         $599 = tempRet0;
         $600 = (_i64Add(($598|0),($599|0),($carry$0250$i|0),0)|0);
         $601 = tempRet0;
         $602 = (___uremdi3(($600|0),($601|0),1000000000,0)|0);
         $603 = tempRet0;
         HEAP32[$d$0251$i>>2] = $602;
         $604 = (___udivdi3(($600|0),($601|0),1000000000,0)|0);
         $605 = tempRet0;
         $606 = (($d$0251$i) + -4|0);
         $607 = ($606>>>0)<($a$1257$i>>>0);
         if ($607) {
          break;
         } else {
          $carry$0250$i = $604;$d$0251$i = $606;
         }
        }
        $608 = HEAP32[$592>>2]|0;
        $609 = ($608|0)==(0);
        $610 = ($z$1256$i>>>0)>($a$1257$i>>>0);
        $or$cond19$i = $609 & $610;
        $z$2$i = $or$cond19$i ? $592 : $z$1256$i;
        $611 = ($604|0)==(0);
        if ($611) {
         $a$2$i = $a$1257$i;$z$2317$i = $z$2$i;
         break;
        }
        $612 = (($a$1257$i) + -4|0);
        HEAP32[$612>>2] = $604;
        $a$2$i = $612;$z$2317$i = $z$2$i;
       }
      } while(0);
      $613 = HEAP32[$e2$i>>2]|0;
      $614 = (($613) - ($591))|0;
      HEAP32[$e2$i>>2] = $614;
      $615 = ($614|0)>(0);
      if ($615) {
       $590 = $614;$a$1257$i = $a$2$i;$z$1256$i = $z$2317$i;
      } else {
       $$pr147$i = $614;$a$1$lcssa$i = $a$2$i;$z$1$lcssa$i = $z$2317$i;
       break;
      }
     }
    } else {
     $$pr147$i = $$pr$i;$a$1$lcssa$i = $$32$i;$z$1$lcssa$i = $583;
    }
    $616 = ($$pr147$i|0)<(0);
    L234: do {
     if ($616) {
      $617 = ($411|0)==(102);
      $618 = (($$p$i|0) / 9)&-1;
      $619 = (($618) + 2)|0;
      if ($617) {
       $620 = $$32$i;
       $621 = (($$32$i) + ($619<<2)|0);
       $623 = $$pr147$i;$a$3245$us$i = $a$1$lcssa$i;$z$3244$us$i = $z$1$lcssa$i;
       while(1) {
        $622 = (0 - ($623))|0;
        $624 = ($622|0)>(9);
        $$20$us$i = $624 ? 9 : $622;
        $625 = ($a$3245$us$i>>>0)<($z$3244$us$i>>>0);
        do {
         if ($625) {
          $650 = 1 << $$20$us$i;
          $644 = (($650) + -1)|0;
          $647 = 1000000000 >>> $$20$us$i;
          $carry3$0239$us$i = 0;$d$1238$us$i = $a$3245$us$i;
          while(1) {
           $642 = HEAP32[$d$1238$us$i>>2]|0;
           $643 = $642 & $644;
           $645 = $642 >>> $$20$us$i;
           $646 = (($645) + ($carry3$0239$us$i))|0;
           HEAP32[$d$1238$us$i>>2] = $646;
           $633 = Math_imul($643, $647)|0;
           $648 = (($d$1238$us$i) + 4|0);
           $649 = ($648>>>0)<($z$3244$us$i>>>0);
           if ($649) {
            $carry3$0239$us$i = $633;$d$1238$us$i = $648;
           } else {
            break;
           }
          }
          $629 = HEAP32[$a$3245$us$i>>2]|0;
          $630 = ($629|0)==(0);
          $631 = (($a$3245$us$i) + 4|0);
          $$a$3$us$i = $630 ? $631 : $a$3245$us$i;
          $632 = ($633|0)==(0);
          if ($632) {
           $$a$3$us320$i = $$a$3$us$i;$z$4$us$i = $z$3244$us$i;
           break;
          }
          $634 = (($z$3244$us$i) + 4|0);
          HEAP32[$z$3244$us$i>>2] = $633;
          $$a$3$us320$i = $$a$3$us$i;$z$4$us$i = $634;
         } else {
          $626 = HEAP32[$a$3245$us$i>>2]|0;
          $627 = ($626|0)==(0);
          $628 = (($a$3245$us$i) + 4|0);
          $$a$3$us319$i = $627 ? $628 : $a$3245$us$i;
          $$a$3$us320$i = $$a$3$us319$i;$z$4$us$i = $z$3244$us$i;
         }
        } while(0);
        $635 = $z$4$us$i;
        $636 = (($635) - ($620))|0;
        $637 = $636 >> 2;
        $638 = ($637|0)>($619|0);
        $$z$4$us$i = $638 ? $621 : $z$4$us$i;
        $639 = HEAP32[$e2$i>>2]|0;
        $640 = (($639) + ($$20$us$i))|0;
        HEAP32[$e2$i>>2] = $640;
        $641 = ($640|0)<(0);
        if ($641) {
         $623 = $640;$a$3245$us$i = $$a$3$us320$i;$z$3244$us$i = $$z$4$us$i;
        } else {
         $a$3$lcssa$i = $$a$3$us320$i;$z$3$lcssa$i = $$z$4$us$i;
         break L234;
        }
       }
      } else {
       $652 = $$pr147$i;$a$3245$i = $a$1$lcssa$i;$z$3244$i = $z$1$lcssa$i;
      }
      while(1) {
       $651 = (0 - ($652))|0;
       $653 = ($651|0)>(9);
       $$20$i = $653 ? 9 : $651;
       $654 = ($a$3245$i>>>0)<($z$3244$i>>>0);
       do {
        if ($654) {
         $658 = 1 << $$20$i;
         $659 = (($658) + -1)|0;
         $660 = 1000000000 >>> $$20$i;
         $carry3$0239$i = 0;$d$1238$i = $a$3245$i;
         while(1) {
          $661 = HEAP32[$d$1238$i>>2]|0;
          $662 = $661 & $659;
          $663 = $661 >>> $$20$i;
          $664 = (($663) + ($carry3$0239$i))|0;
          HEAP32[$d$1238$i>>2] = $664;
          $665 = Math_imul($662, $660)|0;
          $666 = (($d$1238$i) + 4|0);
          $667 = ($666>>>0)<($z$3244$i>>>0);
          if ($667) {
           $carry3$0239$i = $665;$d$1238$i = $666;
          } else {
           break;
          }
         }
         $668 = HEAP32[$a$3245$i>>2]|0;
         $669 = ($668|0)==(0);
         $670 = (($a$3245$i) + 4|0);
         $$a$3$i = $669 ? $670 : $a$3245$i;
         $671 = ($665|0)==(0);
         if ($671) {
          $$a$3322$i = $$a$3$i;$z$4$i = $z$3244$i;
          break;
         }
         $672 = (($z$3244$i) + 4|0);
         HEAP32[$z$3244$i>>2] = $665;
         $$a$3322$i = $$a$3$i;$z$4$i = $672;
        } else {
         $655 = HEAP32[$a$3245$i>>2]|0;
         $656 = ($655|0)==(0);
         $657 = (($a$3245$i) + 4|0);
         $$a$3321$i = $656 ? $657 : $a$3245$i;
         $$a$3322$i = $$a$3321$i;$z$4$i = $z$3244$i;
        }
       } while(0);
       $673 = $z$4$i;
       $674 = $$a$3322$i;
       $675 = (($673) - ($674))|0;
       $676 = $675 >> 2;
       $677 = ($676|0)>($619|0);
       if ($677) {
        $678 = (($$a$3322$i) + ($619<<2)|0);
        $z$5$i = $678;
       } else {
        $z$5$i = $z$4$i;
       }
       $679 = HEAP32[$e2$i>>2]|0;
       $680 = (($679) + ($$20$i))|0;
       HEAP32[$e2$i>>2] = $680;
       $681 = ($680|0)<(0);
       if ($681) {
        $652 = $680;$a$3245$i = $$a$3322$i;$z$3244$i = $z$5$i;
       } else {
        $a$3$lcssa$i = $$a$3322$i;$z$3$lcssa$i = $z$5$i;
        break;
       }
      }
     } else {
      $a$3$lcssa$i = $a$1$lcssa$i;$z$3$lcssa$i = $z$1$lcssa$i;
     }
    } while(0);
    $682 = ($a$3$lcssa$i>>>0)<($z$3$lcssa$i>>>0);
    $683 = $$32$i;
    do {
     if ($682) {
      $684 = $a$3$lcssa$i;
      $685 = (($683) - ($684))|0;
      $686 = $685 >> 2;
      $687 = ($686*9)|0;
      $688 = HEAP32[$a$3$lcssa$i>>2]|0;
      $689 = ($688>>>0)<(10);
      if ($689) {
       $e$1$i = $687;
       break;
      } else {
       $e$0233$i = $687;$i$0234$i = 10;
      }
      while(1) {
       $690 = ($i$0234$i*10)|0;
       $691 = (($e$0233$i) + 1)|0;
       $692 = ($688>>>0)<($690>>>0);
       if ($692) {
        $e$1$i = $691;
        break;
       } else {
        $e$0233$i = $691;$i$0234$i = $690;
       }
      }
     } else {
      $e$1$i = 0;
     }
    } while(0);
    $693 = ($411|0)!=(102);
    $694 = $693 ? $e$1$i : 0;
    $695 = (($$p$i) - ($694))|0;
    $696 = ($411|0)==(103);
    $697 = ($$p$i|0)!=(0);
    $$21$i = $696 & $697;
    $$neg152$i = $$21$i << 31 >> 31;
    $698 = (($695) + ($$neg152$i))|0;
    $699 = $z$3$lcssa$i;
    $700 = (($699) - ($683))|0;
    $701 = $700 >> 2;
    $702 = ($701*9)|0;
    $703 = (($702) + -9)|0;
    $704 = ($698|0)<($703|0);
    if ($704) {
     $705 = (($698) + 9216)|0;
     $706 = (($705|0) / 9)&-1;
     $$sum$i = (($706) + -1023)|0;
     $707 = (($$32$i) + ($$sum$i<<2)|0);
     $708 = (($705|0) % 9)&-1;
     $j$0225$i = (($708) + 1)|0;
     $709 = ($j$0225$i|0)<(9);
     if ($709) {
      $i$1226$i = 10;$j$0227$i = $j$0225$i;
      while(1) {
       $710 = ($i$1226$i*10)|0;
       $j$0$i = (($j$0227$i) + 1)|0;
       $exitcond$i = ($j$0$i|0)==(9);
       if ($exitcond$i) {
        $i$1$lcssa$i = $710;
        break;
       } else {
        $i$1226$i = $710;$j$0227$i = $j$0$i;
       }
      }
     } else {
      $i$1$lcssa$i = 10;
     }
     $711 = HEAP32[$707>>2]|0;
     $712 = (($711>>>0) % ($i$1$lcssa$i>>>0))&-1;
     $713 = ($712|0)==(0);
     if ($713) {
      $$sum14$i = (($706) + -1022)|0;
      $714 = (($$32$i) + ($$sum14$i<<2)|0);
      $715 = ($714|0)==($z$3$lcssa$i|0);
      if ($715) {
       $a$6$i = $a$3$lcssa$i;$d$3$i = $707;$e$3$i = $e$1$i;
      } else {
       label = 232;
      }
     } else {
      label = 232;
     }
     do {
      if ((label|0) == 232) {
       label = 0;
       $716 = (($711>>>0) / ($i$1$lcssa$i>>>0))&-1;
       $717 = $716 & 1;
       $718 = ($717|0)==(0);
       $$22$i = $718 ? 9007199254740992.0 : 9007199254740994.0;
       $719 = (($i$1$lcssa$i|0) / 2)&-1;
       $720 = ($712>>>0)<($719>>>0);
       do {
        if ($720) {
         $small$0$i = 0.5;
        } else {
         $721 = ($712|0)==($719|0);
         if ($721) {
          $$sum15$i = (($706) + -1022)|0;
          $722 = (($$32$i) + ($$sum15$i<<2)|0);
          $723 = ($722|0)==($z$3$lcssa$i|0);
          if ($723) {
           $small$0$i = 1.0;
           break;
          }
         }
         $small$0$i = 1.5;
        }
       } while(0);
       $724 = ($pl$0$i|0)==(0);
       do {
        if ($724) {
         $round6$1$i = $$22$i;$small$1$i = $small$0$i;
        } else {
         $725 = HEAP8[$prefix$0$i>>0]|0;
         $726 = ($725<<24>>24)==(45);
         if (!($726)) {
          $round6$1$i = $$22$i;$small$1$i = $small$0$i;
          break;
         }
         $727 = $$22$i * -1.0;
         $728 = $small$0$i * -1.0;
         $round6$1$i = $727;$small$1$i = $728;
        }
       } while(0);
       $729 = (($711) - ($712))|0;
       HEAP32[$707>>2] = $729;
       $730 = $round6$1$i + $small$1$i;
       $731 = $730 != $round6$1$i;
       if (!($731)) {
        $a$6$i = $a$3$lcssa$i;$d$3$i = $707;$e$3$i = $e$1$i;
        break;
       }
       $732 = (($729) + ($i$1$lcssa$i))|0;
       HEAP32[$707>>2] = $732;
       $733 = ($732>>>0)>(999999999);
       if ($733) {
        $d$2220$i = $707;
        while(1) {
         $734 = (($d$2220$i) + -4|0);
         HEAP32[$d$2220$i>>2] = 0;
         $735 = HEAP32[$734>>2]|0;
         $736 = (($735) + 1)|0;
         HEAP32[$734>>2] = $736;
         $737 = ($736>>>0)>(999999999);
         if ($737) {
          $d$2220$i = $734;
         } else {
          $d$2$lcssa$i = $734;
          break;
         }
        }
       } else {
        $d$2$lcssa$i = $707;
       }
       $738 = ($d$2$lcssa$i>>>0)<($a$3$lcssa$i>>>0);
       $d$2$a$3$i = $738 ? $d$2$lcssa$i : $a$3$lcssa$i;
       $739 = $d$2$a$3$i;
       $740 = (($683) - ($739))|0;
       $741 = $740 >> 2;
       $742 = ($741*9)|0;
       $743 = HEAP32[$d$2$a$3$i>>2]|0;
       $744 = ($743>>>0)<(10);
       if ($744) {
        $a$6$i = $d$2$a$3$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $742;
        break;
       } else {
        $e$2215$i = $742;$i$2216$i = 10;
       }
       while(1) {
        $745 = ($i$2216$i*10)|0;
        $746 = (($e$2215$i) + 1)|0;
        $747 = ($743>>>0)<($745>>>0);
        if ($747) {
         $a$6$i = $d$2$a$3$i;$d$3$i = $d$2$lcssa$i;$e$3$i = $746;
         break;
        } else {
         $e$2215$i = $746;$i$2216$i = $745;
        }
       }
      }
     } while(0);
     $748 = (($d$3$i) + 4|0);
     $749 = ($z$3$lcssa$i>>>0)>($748>>>0);
     $$z$3$i = $749 ? $748 : $z$3$lcssa$i;
     $z$6$i = $$z$3$i;
     while(1) {
      $750 = (($z$6$i) + -4|0);
      $751 = HEAP32[$750>>2]|0;
      $752 = ($751|0)==(0);
      $753 = ($z$6$i>>>0)>($a$6$i>>>0);
      $or$cond23$i = $752 & $753;
      if ($or$cond23$i) {
       $z$6$i = $750;
      } else {
       $a$7$i = $a$6$i;$e$4$i = $e$3$i;$z$7$i = $z$6$i;
       break;
      }
     }
    } else {
     $a$7$i = $a$3$lcssa$i;$e$4$i = $e$1$i;$z$7$i = $z$3$lcssa$i;
    }
    $754 = (0 - ($e$4$i))|0;
    do {
     if ($696) {
      $755 = ($$p$i|0)==(0);
      $756 = $755&1;
      $$$p$i = (($756) + ($$p$i))|0;
      $757 = ($$$p$i|0)>($e$4$i|0);
      $758 = ($e$4$i|0)>(-5);
      $or$cond3$i = $757 & $758;
      if ($or$cond3$i) {
       $759 = (($t$0) + -1)|0;
       $$neg153$i = (($$$p$i) + -1)|0;
       $760 = (($$neg153$i) - ($e$4$i))|0;
       $$012$i73 = $759;$$29$i = $760;
      } else {
       $761 = (($t$0) + -2)|0;
       $762 = (($$$p$i) + -1)|0;
       $$012$i73 = $761;$$29$i = $762;
      }
      $763 = $fl$1$ & 8;
      $764 = ($763|0)==(0);
      if (!($764)) {
       $$113$i = $$012$i73;$$310$i = $$29$i;
       break;
      }
      $765 = ($z$7$i>>>0)>($a$7$i>>>0);
      do {
       if ($765) {
        $766 = (($z$7$i) + -4|0);
        $767 = HEAP32[$766>>2]|0;
        $768 = ($767|0)==(0);
        if ($768) {
         $j$2$i = 9;
         break;
        }
        $769 = (($767>>>0) % 10)&-1;
        $770 = ($769|0)==(0);
        if ($770) {
         $i$3210$i = 10;$j$1211$i = 0;
        } else {
         $j$2$i = 0;
         break;
        }
        while(1) {
         $771 = ($i$3210$i*10)|0;
         $772 = (($j$1211$i) + 1)|0;
         $773 = (($767>>>0) % ($771>>>0))&-1;
         $774 = ($773|0)==(0);
         if ($774) {
          $i$3210$i = $771;$j$1211$i = $772;
         } else {
          $j$2$i = $772;
          break;
         }
        }
       } else {
        $j$2$i = 9;
       }
      } while(0);
      $775 = $$012$i73 | 32;
      $776 = ($775|0)==(102);
      $777 = $z$7$i;
      $778 = (($777) - ($683))|0;
      $779 = $778 >> 2;
      $780 = ($779*9)|0;
      $781 = (($780) + -9)|0;
      if ($776) {
       $782 = (($781) - ($j$2$i))|0;
       $783 = ($782|0)<(0);
       $$24$i = $783 ? 0 : $782;
       $784 = ($$29$i|0)<($$24$i|0);
       $$29$$24$i = $784 ? $$29$i : $$24$i;
       $$113$i = $$012$i73;$$310$i = $$29$$24$i;
       break;
      } else {
       $785 = (($781) + ($e$4$i))|0;
       $786 = (($785) - ($j$2$i))|0;
       $787 = ($786|0)<(0);
       $$26$i = $787 ? 0 : $786;
       $788 = ($$29$i|0)<($$26$i|0);
       $$29$$26$i = $788 ? $$29$i : $$26$i;
       $$113$i = $$012$i73;$$310$i = $$29$$26$i;
       break;
      }
     } else {
      $$113$i = $t$0;$$310$i = $$p$i;
     }
    } while(0);
    $789 = ($$310$i|0)!=(0);
    if ($789) {
     $793 = 1;
    } else {
     $790 = $fl$1$ & 8;
     $791 = ($790|0)!=(0);
     $793 = $791;
    }
    $792 = $793&1;
    $794 = $$113$i | 32;
    $795 = ($794|0)==(102);
    if ($795) {
     $796 = ($e$4$i|0)>(0);
     $797 = $796 ? $e$4$i : 0;
     $$pn$i = $797;$estr$2$i = 0;
    } else {
     $798 = ($e$4$i|0)<(0);
     $799 = $798 ? $754 : $e$4$i;
     $800 = ($799|0)<(0);
     if ($800) {
      $801 = ($799|0)<(0);
      $802 = $801 << 31 >> 31;
      $$05$i80$i = $6;$803 = $799;$804 = $802;
      while(1) {
       $805 = (___uremdi3(($803|0),($804|0),10,0)|0);
       $806 = tempRet0;
       $807 = $805 | 48;
       $808 = $807&255;
       $809 = (($$05$i80$i) + -1|0);
       HEAP8[$809>>0] = $808;
       $810 = (___udivdi3(($803|0),($804|0),10,0)|0);
       $811 = tempRet0;
       $812 = ($804>>>0)>(9);
       $813 = ($804|0)==(9);
       $814 = ($803>>>0)>(4294967295);
       $815 = $813 & $814;
       $816 = $812 | $815;
       if ($816) {
        $$05$i80$i = $809;$803 = $810;$804 = $811;
       } else {
        break;
       }
      }
      $$0$lcssa$i85$i = $809;$$01$lcssa$off0$i86$i = $810;
     } else {
      $$0$lcssa$i85$i = $6;$$01$lcssa$off0$i86$i = $799;
     }
     $817 = ($$01$lcssa$off0$i86$i|0)==(0);
     if ($817) {
      $estr$1$ph$i = $$0$lcssa$i85$i;
     } else {
      $$12$i88$i = $$0$lcssa$i85$i;$y$03$i87$i = $$01$lcssa$off0$i86$i;
      while(1) {
       $818 = (($y$03$i87$i>>>0) % 10)&-1;
       $819 = $818 | 48;
       $820 = $819&255;
       $821 = (($$12$i88$i) + -1|0);
       HEAP8[$821>>0] = $820;
       $822 = (($y$03$i87$i>>>0) / 10)&-1;
       $823 = ($y$03$i87$i>>>0)<(10);
       if ($823) {
        $estr$1$ph$i = $821;
        break;
       } else {
        $$12$i88$i = $821;$y$03$i87$i = $822;
       }
      }
     }
     $824 = $estr$1$ph$i;
     $825 = (($8) - ($824))|0;
     $826 = ($825|0)<(2);
     if ($826) {
      $estr$1201$i = $estr$1$ph$i;
      while(1) {
       $827 = (($estr$1201$i) + -1|0);
       HEAP8[$827>>0] = 48;
       $828 = $827;
       $829 = (($8) - ($828))|0;
       $830 = ($829|0)<(2);
       if ($830) {
        $estr$1201$i = $827;
       } else {
        $estr$1$lcssa$i = $827;
        break;
       }
      }
     } else {
      $estr$1$lcssa$i = $estr$1$ph$i;
     }
     $831 = $e$4$i >> 31;
     $832 = $831 & 2;
     $833 = (($832) + 43)|0;
     $834 = $833&255;
     $835 = (($estr$1$lcssa$i) + -1|0);
     HEAP8[$835>>0] = $834;
     $836 = $$113$i&255;
     $837 = (($estr$1$lcssa$i) + -2|0);
     HEAP8[$837>>0] = $836;
     $838 = $837;
     $839 = (($8) - ($838))|0;
     $$pn$i = $839;$estr$2$i = $837;
    }
    $840 = (($pl$0$i) + 1)|0;
    $841 = (($840) + ($$310$i))|0;
    $l$1$i = (($841) + ($792))|0;
    $842 = (($l$1$i) + ($$pn$i))|0;
    $843 = $fl$1$ & 73728;
    $844 = ($843|0)==(0);
    $845 = ($842|0)<($w$1|0);
    $or$cond$i94$i = $844 & $845;
    if ($or$cond$i94$i) {
     $846 = (($w$1) - ($842))|0;
     $847 = ($846>>>0)>(256);
     $848 = $847 ? 256 : $846;
     _memset(($pad$i|0),32,($848|0))|0;
     $849 = ($846>>>0)>(255);
     if ($849) {
      $$01$i95$i = $846;
      while(1) {
       (___fwritex($pad$i,256,$f)|0);
       $850 = (($$01$i95$i) + -256)|0;
       $851 = ($850>>>0)>(255);
       if ($851) {
        $$01$i95$i = $850;
       } else {
        $$0$lcssa$i97$i = $850;
        break;
       }
      }
     } else {
      $$0$lcssa$i97$i = $846;
     }
     (___fwritex($pad$i,$$0$lcssa$i97$i,$f)|0);
    }
    (___fwritex($prefix$0$i,$pl$0$i,$f)|0);
    $852 = ($843|0)==(65536);
    $or$cond$i101$i = $852 & $845;
    if ($or$cond$i101$i) {
     $853 = (($w$1) - ($842))|0;
     $854 = ($853>>>0)>(256);
     $855 = $854 ? 256 : $853;
     _memset(($pad$i|0),48,($855|0))|0;
     $856 = ($853>>>0)>(255);
     if ($856) {
      $$01$i102$i = $853;
      while(1) {
       (___fwritex($pad$i,256,$f)|0);
       $857 = (($$01$i102$i) + -256)|0;
       $858 = ($857>>>0)>(255);
       if ($858) {
        $$01$i102$i = $857;
       } else {
        $$0$lcssa$i104$i = $857;
        break;
       }
      }
     } else {
      $$0$lcssa$i104$i = $853;
     }
     (___fwritex($pad$i,$$0$lcssa$i104$i,$f)|0);
    }
    do {
     if ($795) {
      $859 = ($a$7$i>>>0)>($$32$i>>>0);
      $r$0$a$7$i = $859 ? $$32$i : $a$7$i;
      $d$4184$i = $r$0$a$7$i;
      while(1) {
       $860 = HEAP32[$d$4184$i>>2]|0;
       $861 = ($860|0)==(0);
       if ($861) {
        $$1$lcssa$i113$i = $13;
       } else {
        $$12$i111$i = $13;$y$03$i110$i = $860;
        while(1) {
         $862 = (($y$03$i110$i>>>0) % 10)&-1;
         $863 = $862 | 48;
         $864 = $863&255;
         $865 = (($$12$i111$i) + -1|0);
         HEAP8[$865>>0] = $864;
         $866 = (($y$03$i110$i>>>0) / 10)&-1;
         $867 = ($y$03$i110$i>>>0)<(10);
         if ($867) {
          $$1$lcssa$i113$i = $865;
          break;
         } else {
          $$12$i111$i = $865;$y$03$i110$i = $866;
         }
        }
       }
       $868 = ($d$4184$i|0)==($r$0$a$7$i|0);
       do {
        if ($868) {
         $872 = ($$1$lcssa$i113$i|0)==($13|0);
         if (!($872)) {
          $s7$1$i = $$1$lcssa$i113$i;
          break;
         }
         HEAP8[$15>>0] = 48;
         $s7$1$i = $15;
        } else {
         $869 = ($$1$lcssa$i113$i>>>0)>($buf$i>>>0);
         if ($869) {
          $s7$0181$i = $$1$lcssa$i113$i;
         } else {
          $s7$1$i = $$1$lcssa$i113$i;
          break;
         }
         while(1) {
          $870 = (($s7$0181$i) + -1|0);
          HEAP8[$870>>0] = 48;
          $871 = ($870>>>0)>($buf$i>>>0);
          if ($871) {
           $s7$0181$i = $870;
          } else {
           $s7$1$i = $870;
           break;
          }
         }
        }
       } while(0);
       $873 = $s7$1$i;
       $874 = (($14) - ($873))|0;
       (___fwritex($s7$1$i,$874,$f)|0);
       $875 = (($d$4184$i) + 4|0);
       $876 = ($875>>>0)>($$32$i>>>0);
       if ($876) {
        break;
       } else {
        $d$4184$i = $875;
       }
      }
      if (!($789)) {
       $877 = $fl$1$ & 8;
       $878 = ($877|0)==(0);
       if ($878) {
        break;
       }
      }
      (___fwritex(15472,1,$f)|0);
      $879 = ($875>>>0)<($z$7$i>>>0);
      $880 = ($$310$i|0)>(0);
      $or$cond29174$i = $879 & $880;
      if ($or$cond29174$i) {
       $$411176$i = $$310$i;$d$5175$i = $875;
       while(1) {
        $881 = HEAP32[$d$5175$i>>2]|0;
        $882 = ($881|0)==(0);
        if ($882) {
         $s8$0170$i = $13;
         label = 295;
        } else {
         $$12$i120$i = $13;$y$03$i119$i = $881;
         while(1) {
          $883 = (($y$03$i119$i>>>0) % 10)&-1;
          $884 = $883 | 48;
          $885 = $884&255;
          $886 = (($$12$i120$i) + -1|0);
          HEAP8[$886>>0] = $885;
          $887 = (($y$03$i119$i>>>0) / 10)&-1;
          $888 = ($y$03$i119$i>>>0)<(10);
          if ($888) {
           break;
          } else {
           $$12$i120$i = $886;$y$03$i119$i = $887;
          }
         }
         $889 = ($886>>>0)>($buf$i>>>0);
         if ($889) {
          $s8$0170$i = $886;
          label = 295;
         } else {
          $s8$0$lcssa$i = $886;
         }
        }
        if ((label|0) == 295) {
         while(1) {
          label = 0;
          $890 = (($s8$0170$i) + -1|0);
          HEAP8[$890>>0] = 48;
          $891 = ($890>>>0)>($buf$i>>>0);
          if ($891) {
           $s8$0170$i = $890;
           label = 295;
          } else {
           $s8$0$lcssa$i = $890;
           break;
          }
         }
        }
        $892 = ($$411176$i|0)>(9);
        $893 = $892 ? 9 : $$411176$i;
        (___fwritex($s8$0$lcssa$i,$893,$f)|0);
        $894 = (($d$5175$i) + 4|0);
        $895 = (($$411176$i) + -9)|0;
        $896 = ($894>>>0)<($z$7$i>>>0);
        $897 = ($895|0)>(0);
        $or$cond29$i = $896 & $897;
        if ($or$cond29$i) {
         $$411176$i = $895;$d$5175$i = $894;
        } else {
         $$411$lcssa$i = $895;
         break;
        }
       }
      } else {
       $$411$lcssa$i = $$310$i;
      }
      $898 = ($$411$lcssa$i|0)>(0);
      if (!($898)) {
       break;
      }
      $899 = ($$411$lcssa$i>>>0)>(256);
      $900 = $899 ? 256 : $$411$lcssa$i;
      _memset(($pad$i|0),48,($900|0))|0;
      $901 = ($$411$lcssa$i>>>0)>(255);
      if ($901) {
       $$01$i126$i = $$411$lcssa$i;
       while(1) {
        (___fwritex($pad$i,256,$f)|0);
        $902 = (($$01$i126$i) + -256)|0;
        $903 = ($902>>>0)>(255);
        if ($903) {
         $$01$i126$i = $902;
        } else {
         $$0$lcssa$i128$i = $902;
         break;
        }
       }
      } else {
       $$0$lcssa$i128$i = $$411$lcssa$i;
      }
      (___fwritex($pad$i,$$0$lcssa$i128$i,$f)|0);
     } else {
      $904 = ($z$7$i>>>0)>($a$7$i>>>0);
      $905 = (($a$7$i) + 4|0);
      $z$7$$i = $904 ? $z$7$i : $905;
      $906 = ($$310$i|0)>(-1);
      do {
       if ($906) {
        $907 = $fl$1$ & 8;
        $$not$i = ($907|0)!=(0);
        $$5194$i = $$310$i;$d$6193$i = $a$7$i;
        while(1) {
         $908 = HEAP32[$d$6193$i>>2]|0;
         $909 = ($908|0)==(0);
         if ($909) {
          label = 306;
         } else {
          $$12$i135$i = $13;$y$03$i134$i = $908;
          while(1) {
           $910 = (($y$03$i134$i>>>0) % 10)&-1;
           $911 = $910 | 48;
           $912 = $911&255;
           $913 = (($$12$i135$i) + -1|0);
           HEAP8[$913>>0] = $912;
           $914 = (($y$03$i134$i>>>0) / 10)&-1;
           $915 = ($y$03$i134$i>>>0)<(10);
           if ($915) {
            break;
           } else {
            $$12$i135$i = $913;$y$03$i134$i = $914;
           }
          }
          $916 = ($913|0)==($13|0);
          if ($916) {
           label = 306;
          } else {
           $s9$0$i = $913;
          }
         }
         if ((label|0) == 306) {
          label = 0;
          HEAP8[$15>>0] = 48;
          $s9$0$i = $15;
         }
         $917 = ($d$6193$i|0)==($a$7$i|0);
         do {
          if ($917) {
           $921 = (($s9$0$i) + 1|0);
           (___fwritex($s9$0$i,1,$f)|0);
           $922 = ($$5194$i|0)>(0);
           $brmerge$i = $922 | $$not$i;
           if (!($brmerge$i)) {
            $s9$2$i = $921;
            break;
           }
           (___fwritex(15472,1,$f)|0);
           $s9$2$i = $921;
          } else {
           $918 = ($s9$0$i>>>0)>($buf$i>>>0);
           if ($918) {
            $s9$1189$i = $s9$0$i;
           } else {
            $s9$2$i = $s9$0$i;
            break;
           }
           while(1) {
            $919 = (($s9$1189$i) + -1|0);
            HEAP8[$919>>0] = 48;
            $920 = ($919>>>0)>($buf$i>>>0);
            if ($920) {
             $s9$1189$i = $919;
            } else {
             $s9$2$i = $919;
             break;
            }
           }
          }
         } while(0);
         $923 = $s9$2$i;
         $924 = (($14) - ($923))|0;
         $925 = ($924|0)<($$5194$i|0);
         $$$5$i = $925 ? $924 : $$5194$i;
         (___fwritex($s9$2$i,$$$5$i,$f)|0);
         $926 = (($$5194$i) - ($924))|0;
         $927 = (($d$6193$i) + 4|0);
         $928 = ($927>>>0)<($z$7$$i>>>0);
         $929 = ($926|0)>(-1);
         $or$cond30$i = $928 & $929;
         if ($or$cond30$i) {
          $$5194$i = $926;$d$6193$i = $927;
         } else {
          break;
         }
        }
        $930 = ($926|0)>(0);
        if (!($930)) {
         break;
        }
        $931 = ($926>>>0)>(256);
        $932 = $931 ? 256 : $926;
        _memset(($pad$i|0),48,($932|0))|0;
        $933 = ($926>>>0)>(255);
        if ($933) {
         $$01$i141$i = $926;
         while(1) {
          (___fwritex($pad$i,256,$f)|0);
          $934 = (($$01$i141$i) + -256)|0;
          $935 = ($934>>>0)>(255);
          if ($935) {
           $$01$i141$i = $934;
          } else {
           $$0$lcssa$i143$i = $934;
           break;
          }
         }
        } else {
         $$0$lcssa$i143$i = $926;
        }
        (___fwritex($pad$i,$$0$lcssa$i143$i,$f)|0);
       }
      } while(0);
      $936 = $estr$2$i;
      $937 = (($8) - ($936))|0;
      (___fwritex($estr$2$i,$937,$f)|0);
     }
    } while(0);
    $938 = ($843|0)==(8192);
    $or$cond$i$i = $938 & $845;
    if ($or$cond$i$i) {
     $939 = (($w$1) - ($842))|0;
     $940 = ($939>>>0)>(256);
     $941 = $940 ? 256 : $939;
     _memset(($pad$i|0),32,($941|0))|0;
     $942 = ($939>>>0)>(255);
     if ($942) {
      $$01$i$i = $939;
      while(1) {
       (___fwritex($pad$i,256,$f)|0);
       $943 = (($$01$i$i) + -256)|0;
       $944 = ($943>>>0)>(255);
       if ($944) {
        $$01$i$i = $943;
       } else {
        $$0$lcssa$i$i = $943;
        break;
       }
      }
     } else {
      $$0$lcssa$i$i = $939;
     }
     (___fwritex($pad$i,$$0$lcssa$i$i,$f)|0);
    }
    $w$31$i = $845 ? $w$1 : $842;
    $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$31$i;$l10n$0 = $l10n$3;
    continue L1;
    break;
   }
   case 110:  {
    switch ($st$0|0) {
    case 7:  {
     $215 = ($cnt$1|0)<(0);
     $216 = $215 << 31 >> 31;
     $217 = $206;
     $218 = $217;
     HEAP32[$218>>2] = $cnt$1;
     $219 = (($217) + 4)|0;
     $220 = $219;
     HEAP32[$220>>2] = $216;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 4:  {
     $214 = $cnt$1&255;
     HEAP8[$206>>0] = $214;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 6:  {
     HEAP32[$206>>2] = $cnt$1;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 0:  {
     HEAP32[$206>>2] = $cnt$1;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 1:  {
     HEAP32[$206>>2] = $cnt$1;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 2:  {
     $207 = ($cnt$1|0)<(0);
     $208 = $207 << 31 >> 31;
     $209 = $206;
     $210 = $209;
     HEAP32[$210>>2] = $cnt$1;
     $211 = (($209) + 4)|0;
     $212 = $211;
     HEAP32[$212>>2] = $208;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    case 3:  {
     $213 = $cnt$1&65535;
     HEAP16[$206>>1] = $213;
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
     break;
    }
    default: {
     $1048 = $228;$1049 = $206;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $36;$l10n$0 = $l10n$3;
     continue L1;
    }
    }
    break;
   }
   case 111:  {
    $248 = $206;
    $249 = ($248|0)==(0);
    $250 = ($228|0)==(0);
    $251 = $249 & $250;
    if ($251) {
     $$0$lcssa$i44 = $2;
    } else {
     $$03$i41 = $2;$253 = $248;$257 = $228;
     while(1) {
      $252 = $253 & 7;
      $254 = $252 | 48;
      $255 = $254&255;
      $256 = (($$03$i41) + -1|0);
      HEAP8[$256>>0] = $255;
      $258 = (_bitshift64Lshr(($253|0),($257|0),3)|0);
      $259 = tempRet0;
      $260 = ($258|0)==(0);
      $261 = ($259|0)==(0);
      $262 = $260 & $261;
      if ($262) {
       $$0$lcssa$i44 = $256;
       break;
      } else {
       $$03$i41 = $256;$253 = $258;$257 = $259;
      }
     }
    }
    $263 = $fl$1$ & 8;
    $264 = ($263|0)==(0);
    if ($264) {
     $307 = $206;$310 = $228;$a$0 = $$0$lcssa$i44;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = 0;$prefix$1 = 15376;
     label = 95;
    } else {
     $$14 = $251 ? 15376 : ((15376 + 5|0));
     $265 = $251&1;
     $$15 = $265 ^ 1;
     $307 = $206;$310 = $228;$a$0 = $$0$lcssa$i44;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $$15;$prefix$1 = $$14;
     label = 95;
    }
    break;
   }
   case 109:  {
    $320 = (___errno_location()|0);
    $321 = HEAP32[$320>>2]|0;
    $322 = (_strerror(($321|0))|0);
    $a$1 = $322;
    label = 100;
    break;
   }
   case 115:  {
    $323 = ($206|0)==(0|0);
    $$18 = $323 ? 15392 : $206;
    $a$1 = $$18;
    label = 100;
    break;
   }
   default: {
    $1054 = $228;$1055 = $206;$a$2 = $22;$fl$6 = $fl$1$;$p$5 = $p$0;$pl$2 = 0;$prefix$2 = 15376;$z$2 = $2;
   }
   }
  } while(0);
  L437: do {
   if ((label|0) == 78) {
    label = 0;
    $224 = $206;
    $225 = $t$1 & 32;
    $226 = ($224|0)==(0);
    $227 = ($228|0)==(0);
    $229 = $226 & $227;
    if ($229) {
     $307 = $206;$310 = $228;$a$0 = $2;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 15376;
     label = 95;
    } else {
     $$012$i = $2;$231 = $224;$238 = $228;
     while(1) {
      $230 = $231 & 15;
      $232 = (15456 + ($230)|0);
      $233 = HEAP8[$232>>0]|0;
      $234 = $233&255;
      $235 = $234 | $225;
      $236 = $235&255;
      $237 = (($$012$i) + -1|0);
      HEAP8[$237>>0] = $236;
      $239 = (_bitshift64Lshr(($231|0),($238|0),4)|0);
      $240 = tempRet0;
      $241 = ($239|0)==(0);
      $242 = ($240|0)==(0);
      $243 = $241 & $242;
      if ($243) {
       break;
      } else {
       $$012$i = $237;$231 = $239;$238 = $240;
      }
     }
     $244 = $fl$3 & 8;
     $245 = ($244|0)==(0);
     if ($245) {
      $307 = $206;$310 = $228;$a$0 = $237;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 0;$prefix$1 = 15376;
      label = 95;
     } else {
      $246 = $t$1 >> 4;
      $247 = (15376 + ($246)|0);
      $307 = $206;$310 = $228;$a$0 = $237;$fl$4 = $fl$3;$p$2 = $p$1;$pl$1 = 2;$prefix$1 = $247;
      label = 95;
     }
    }
   }
   else if ((label|0) == 90) {
    label = 0;
    $275 = $276;
    $277 = ($278>>>0)>(0);
    $279 = ($278|0)==(0);
    $280 = ($275>>>0)>(4294967295);
    $281 = $279 & $280;
    $282 = $277 | $281;
    if ($282) {
     $$05$i = $2;$283 = $275;$284 = $278;
     while(1) {
      $285 = (___uremdi3(($283|0),($284|0),10,0)|0);
      $286 = tempRet0;
      $287 = $285 | 48;
      $288 = $287&255;
      $289 = (($$05$i) + -1|0);
      HEAP8[$289>>0] = $288;
      $290 = (___udivdi3(($283|0),($284|0),10,0)|0);
      $291 = tempRet0;
      $292 = ($284>>>0)>(9);
      $293 = ($284|0)==(9);
      $294 = ($283>>>0)>(4294967295);
      $295 = $293 & $294;
      $296 = $292 | $295;
      if ($296) {
       $$05$i = $289;$283 = $290;$284 = $291;
      } else {
       break;
      }
     }
     $$0$lcssa$i46 = $289;$$01$lcssa$off0$i = $290;
    } else {
     $$0$lcssa$i46 = $2;$$01$lcssa$off0$i = $275;
    }
    $297 = ($$01$lcssa$off0$i|0)==(0);
    if ($297) {
     $307 = $276;$310 = $278;$a$0 = $$0$lcssa$i46;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $pl$0;$prefix$1 = $prefix$0;
     label = 95;
    } else {
     $$12$i = $$0$lcssa$i46;$y$03$i = $$01$lcssa$off0$i;
     while(1) {
      $298 = (($y$03$i>>>0) % 10)&-1;
      $299 = $298 | 48;
      $300 = $299&255;
      $301 = (($$12$i) + -1|0);
      HEAP8[$301>>0] = $300;
      $302 = (($y$03$i>>>0) / 10)&-1;
      $303 = ($y$03$i>>>0)<(10);
      if ($303) {
       $307 = $276;$310 = $278;$a$0 = $301;$fl$4 = $fl$1$;$p$2 = $p$0;$pl$1 = $pl$0;$prefix$1 = $prefix$0;
       label = 95;
       break;
      } else {
       $$12$i = $301;$y$03$i = $302;
      }
     }
    }
   }
   else if ((label|0) == 100) {
    label = 0;
    $324 = (_memchr($a$1,0,$p$0)|0);
    $325 = ($324|0)==(0|0);
    if ($325) {
     $326 = (($a$1) + ($p$0)|0);
     $1054 = $228;$1055 = $206;$a$2 = $a$1;$fl$6 = $205;$p$5 = $p$0;$pl$2 = 0;$prefix$2 = 15376;$z$2 = $326;
     break;
    } else {
     $327 = $324;
     $328 = $a$1;
     $329 = (($327) - ($328))|0;
     $1054 = $228;$1055 = $206;$a$2 = $a$1;$fl$6 = $205;$p$5 = $329;$pl$2 = 0;$prefix$2 = 15376;$z$2 = $324;
     break;
    }
   }
   else if ((label|0) == 105) {
    label = 0;
    $i$0169 = 0;$l$1168 = 0;$ws$0170 = $1056;
    while(1) {
     $335 = HEAP32[$ws$0170>>2]|0;
     $336 = ($335|0)==(0);
     if ($336) {
      $i$0$lcssa = $i$0169;$l$1$lcssa = $l$1168;
      break;
     }
     $337 = (_wctomb($mb,$335)|0);
     $338 = ($337|0)>(-1);
     if (!($338)) {
      $$0 = -1;
      label = 357;
      break L1;
     }
     $339 = (($p$4296) - ($i$0169))|0;
     $340 = ($337>>>0)>($339>>>0);
     $334 = (($337) + ($i$0169))|0;
     if ($340) {
      $1058 = $1056;$1059 = $1057;$i$0167 = $i$0169;
      label = 111;
      break L437;
     }
     $332 = (($ws$0170) + 4|0);
     $333 = ($334>>>0)<($p$4296>>>0);
     if ($333) {
      $i$0169 = $334;$l$1168 = $337;$ws$0170 = $332;
     } else {
      $i$0$lcssa = $334;$l$1$lcssa = $337;
      break;
     }
    }
    $341 = ($l$1$lcssa|0)<(0);
    if ($341) {
     $$0 = -1;
     label = 357;
     break L1;
    } else {
     $1058 = $1056;$1059 = $1057;$i$0167 = $i$0$lcssa;
     label = 111;
    }
   }
  } while(0);
  if ((label|0) == 95) {
   label = 0;
   $304 = ($p$2|0)>(-1);
   $305 = $fl$4 & -65537;
   $$fl$4 = $304 ? $305 : $fl$4;
   $306 = $307;
   $308 = ($306|0)==(0);
   $309 = ($310|0)==(0);
   $311 = $308 & $309;
   $312 = ($p$2|0)==(0);
   $or$cond = $311 & $312;
   if ($or$cond) {
    $1054 = $310;$1055 = $307;$a$2 = $2;$fl$6 = $$fl$4;$p$5 = 0;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $2;
   } else {
    $313 = $a$0;
    $314 = (($3) - ($313))|0;
    $315 = $311&1;
    $316 = (($315) + ($314))|0;
    $317 = ($p$2|0)>($316|0);
    $p$2$ = $317 ? $p$2 : $316;
    $1054 = $310;$1055 = $307;$a$2 = $a$0;$fl$6 = $$fl$4;$p$5 = $p$2$;$pl$2 = $pl$1;$prefix$2 = $prefix$1;$z$2 = $2;
   }
  }
  else if ((label|0) == 111) {
   label = 0;
   $342 = $fl$1$ & 73728;
   $343 = ($342|0)==(0);
   $344 = ($i$0167|0)<($w$1|0);
   $or$cond$i57 = $343 & $344;
   if ($or$cond$i57) {
    $345 = (($w$1) - ($i$0167))|0;
    $346 = ($345>>>0)>(256);
    $347 = $346 ? 256 : $345;
    _memset(($pad$i|0),32,($347|0))|0;
    $348 = ($345>>>0)>(255);
    if ($348) {
     $$01$i58 = $345;
     while(1) {
      (___fwritex($pad$i,256,$f)|0);
      $349 = (($$01$i58) + -256)|0;
      $350 = ($349>>>0)>(255);
      if ($350) {
       $$01$i58 = $349;
      } else {
       $$0$lcssa$i60 = $349;
       break;
      }
     }
    } else {
     $$0$lcssa$i60 = $345;
    }
    (___fwritex($pad$i,$$0$lcssa$i60,$f)|0);
   }
   $351 = ($i$0167|0)==(0);
   L473: do {
    if (!($351)) {
     $i$1175 = 0;$ws$1176 = $1058;
     while(1) {
      $352 = HEAP32[$ws$1176>>2]|0;
      $353 = ($352|0)==(0);
      if ($353) {
       break L473;
      }
      $354 = (_wctomb($mb,$352)|0);
      $355 = (($354) + ($i$1175))|0;
      $356 = ($355|0)>($i$0167|0);
      if ($356) {
       break L473;
      }
      $357 = (($ws$1176) + 4|0);
      (___fwritex($mb,$354,$f)|0);
      $358 = ($355>>>0)<($i$0167>>>0);
      if ($358) {
       $i$1175 = $355;$ws$1176 = $357;
      } else {
       break;
      }
     }
    }
   } while(0);
   $359 = ($342|0)==(8192);
   $or$cond$i64 = $359 & $344;
   if ($or$cond$i64) {
    $360 = (($w$1) - ($i$0167))|0;
    $361 = ($360>>>0)>(256);
    $362 = $361 ? 256 : $360;
    _memset(($pad$i|0),32,($362|0))|0;
    $363 = ($360>>>0)>(255);
    if ($363) {
     $$01$i65 = $360;
     while(1) {
      (___fwritex($pad$i,256,$f)|0);
      $364 = (($$01$i65) + -256)|0;
      $365 = ($364>>>0)>(255);
      if ($365) {
       $$01$i65 = $364;
      } else {
       $$0$lcssa$i67 = $364;
       break;
      }
     }
    } else {
     $$0$lcssa$i67 = $360;
    }
    (___fwritex($pad$i,$$0$lcssa$i67,$f)|0);
   }
   $366 = $344 ? $w$1 : $i$0167;
   $1048 = $228;$1049 = $1059;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $366;$l10n$0 = $l10n$3;
   continue;
  }
  $945 = $z$2;
  $946 = $a$2;
  $947 = (($945) - ($946))|0;
  $948 = ($p$5|0)<($947|0);
  $$p$5 = $948 ? $947 : $p$5;
  $949 = (($pl$2) + ($$p$5))|0;
  $950 = ($w$1|0)<($949|0);
  $w$2 = $950 ? $949 : $w$1;
  $951 = $fl$6 & 73728;
  $952 = ($951|0)==(0);
  $953 = ($949|0)<($w$2|0);
  $or$cond$i75 = $952 & $953;
  if ($or$cond$i75) {
   $954 = (($w$2) - ($949))|0;
   $955 = ($954>>>0)>(256);
   $956 = $955 ? 256 : $954;
   _memset(($pad$i|0),32,($956|0))|0;
   $957 = ($954>>>0)>(255);
   if ($957) {
    $$01$i76 = $954;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $958 = (($$01$i76) + -256)|0;
     $959 = ($958>>>0)>(255);
     if ($959) {
      $$01$i76 = $958;
     } else {
      $$0$lcssa$i78 = $958;
      break;
     }
    }
   } else {
    $$0$lcssa$i78 = $954;
   }
   (___fwritex($pad$i,$$0$lcssa$i78,$f)|0);
  }
  (___fwritex($prefix$2,$pl$2,$f)|0);
  $960 = ($951|0)==(65536);
  $or$cond$i50 = $960 & $953;
  if ($or$cond$i50) {
   $961 = (($w$2) - ($949))|0;
   $962 = ($961>>>0)>(256);
   $963 = $962 ? 256 : $961;
   _memset(($pad$i|0),48,($963|0))|0;
   $964 = ($961>>>0)>(255);
   if ($964) {
    $$01$i51 = $961;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $965 = (($$01$i51) + -256)|0;
     $966 = ($965>>>0)>(255);
     if ($966) {
      $$01$i51 = $965;
     } else {
      $$0$lcssa$i53 = $965;
      break;
     }
    }
   } else {
    $$0$lcssa$i53 = $961;
   }
   (___fwritex($pad$i,$$0$lcssa$i53,$f)|0);
  }
  $967 = ($947|0)<($$p$5|0);
  if ($967) {
   $968 = (($$p$5) - ($947))|0;
   $969 = ($968>>>0)>(256);
   $970 = $969 ? 256 : $968;
   _memset(($pad$i|0),48,($970|0))|0;
   $971 = ($968>>>0)>(255);
   if ($971) {
    $$01$i36 = $968;
    while(1) {
     (___fwritex($pad$i,256,$f)|0);
     $972 = (($$01$i36) + -256)|0;
     $973 = ($972>>>0)>(255);
     if ($973) {
      $$01$i36 = $972;
     } else {
      $$0$lcssa$i38 = $972;
      break;
     }
    }
   } else {
    $$0$lcssa$i38 = $968;
   }
   (___fwritex($pad$i,$$0$lcssa$i38,$f)|0);
  }
  (___fwritex($a$2,$947,$f)|0);
  $974 = ($951|0)==(8192);
  $or$cond$i = $974 & $953;
  if (!($or$cond$i)) {
   $1048 = $1054;$1049 = $1055;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$2;$l10n$0 = $l10n$3;
   continue;
  }
  $975 = (($w$2) - ($949))|0;
  $976 = ($975>>>0)>(256);
  $977 = $976 ? 256 : $975;
  _memset(($pad$i|0),32,($977|0))|0;
  $978 = ($975>>>0)>(255);
  if ($978) {
   $$01$i = $975;
   while(1) {
    (___fwritex($pad$i,256,$f)|0);
    $979 = (($$01$i) + -256)|0;
    $980 = ($979>>>0)>(255);
    if ($980) {
     $$01$i = $979;
    } else {
     $$0$lcssa$i = $979;
     break;
    }
   }
  } else {
   $$0$lcssa$i = $975;
  }
  (___fwritex($pad$i,$$0$lcssa$i,$f)|0);
  $1048 = $1054;$1049 = $1055;$22 = $139;$cnt$0 = $cnt$1;$l$0 = $w$2;$l10n$0 = $l10n$3;
 }
 if ((label|0) == 339) {
  $981 = ($f|0)==(0|0);
  if (!($981)) {
   $$0 = $cnt$1;
   STACKTOP = sp;return ($$0|0);
  }
  $982 = ($l10n$0|0)==(0);
  if ($982) {
   $$0 = 0;
   STACKTOP = sp;return ($$0|0);
  } else {
   $i$291 = 1;
  }
  while(1) {
   $983 = (($nl_type) + ($i$291<<2)|0);
   $984 = HEAP32[$983>>2]|0;
   $985 = ($984|0)==(0);
   if ($985) {
    $i$390 = $i$291;
    break;
   }
   $986 = (($nl_arg) + ($i$291<<3)|0);
   $987 = ($984>>>0)>(20);
   L523: do {
    if (!($987)) {
     do {
      switch ($984|0) {
      case 9:  {
       $arglist_current35 = HEAP32[$ap>>2]|0;
       $988 = HEAP32[$arglist_current35>>2]|0;
       $arglist_next36 = (($arglist_current35) + 4|0);
       HEAP32[$ap>>2] = $arglist_next36;
       HEAP32[$986>>2] = $988;
       break L523;
       break;
      }
      case 10:  {
       $arglist_current38 = HEAP32[$ap>>2]|0;
       $989 = HEAP32[$arglist_current38>>2]|0;
       $arglist_next39 = (($arglist_current38) + 4|0);
       HEAP32[$ap>>2] = $arglist_next39;
       $990 = ($989|0)<(0);
       $991 = $990 << 31 >> 31;
       $992 = $986;
       $993 = $992;
       HEAP32[$993>>2] = $989;
       $994 = (($992) + 4)|0;
       $995 = $994;
       HEAP32[$995>>2] = $991;
       break L523;
       break;
      }
      case 11:  {
       $arglist_current41 = HEAP32[$ap>>2]|0;
       $996 = HEAP32[$arglist_current41>>2]|0;
       $arglist_next42 = (($arglist_current41) + 4|0);
       HEAP32[$ap>>2] = $arglist_next42;
       $997 = $986;
       $998 = $997;
       HEAP32[$998>>2] = $996;
       $999 = (($997) + 4)|0;
       $1000 = $999;
       HEAP32[$1000>>2] = 0;
       break L523;
       break;
      }
      case 12:  {
       $arglist_current44 = HEAP32[$ap>>2]|0;
       $1001 = $arglist_current44;
       $1002 = $1001;
       $1003 = HEAP32[$1002>>2]|0;
       $1004 = (($1001) + 4)|0;
       $1005 = $1004;
       $1006 = HEAP32[$1005>>2]|0;
       $arglist_next45 = (($arglist_current44) + 8|0);
       HEAP32[$ap>>2] = $arglist_next45;
       $1007 = $986;
       $1008 = $1007;
       HEAP32[$1008>>2] = $1003;
       $1009 = (($1007) + 4)|0;
       $1010 = $1009;
       HEAP32[$1010>>2] = $1006;
       break L523;
       break;
      }
      case 13:  {
       $arglist_current47 = HEAP32[$ap>>2]|0;
       $1011 = HEAP32[$arglist_current47>>2]|0;
       $arglist_next48 = (($arglist_current47) + 4|0);
       HEAP32[$ap>>2] = $arglist_next48;
       $1012 = $1011&65535;
       $1013 = $1012 << 16 >> 16;
       $1014 = ($1013|0)<(0);
       $1015 = $1014 << 31 >> 31;
       $1016 = $986;
       $1017 = $1016;
       HEAP32[$1017>>2] = $1013;
       $1018 = (($1016) + 4)|0;
       $1019 = $1018;
       HEAP32[$1019>>2] = $1015;
       break L523;
       break;
      }
      case 14:  {
       $arglist_current50 = HEAP32[$ap>>2]|0;
       $1020 = HEAP32[$arglist_current50>>2]|0;
       $arglist_next51 = (($arglist_current50) + 4|0);
       HEAP32[$ap>>2] = $arglist_next51;
       $$mask1$i = $1020 & 65535;
       $1021 = $986;
       $1022 = $1021;
       HEAP32[$1022>>2] = $$mask1$i;
       $1023 = (($1021) + 4)|0;
       $1024 = $1023;
       HEAP32[$1024>>2] = 0;
       break L523;
       break;
      }
      case 15:  {
       $arglist_current53 = HEAP32[$ap>>2]|0;
       $1025 = HEAP32[$arglist_current53>>2]|0;
       $arglist_next54 = (($arglist_current53) + 4|0);
       HEAP32[$ap>>2] = $arglist_next54;
       $1026 = $1025&255;
       $1027 = $1026 << 24 >> 24;
       $1028 = ($1027|0)<(0);
       $1029 = $1028 << 31 >> 31;
       $1030 = $986;
       $1031 = $1030;
       HEAP32[$1031>>2] = $1027;
       $1032 = (($1030) + 4)|0;
       $1033 = $1032;
       HEAP32[$1033>>2] = $1029;
       break L523;
       break;
      }
      case 16:  {
       $arglist_current56 = HEAP32[$ap>>2]|0;
       $1034 = HEAP32[$arglist_current56>>2]|0;
       $arglist_next57 = (($arglist_current56) + 4|0);
       HEAP32[$ap>>2] = $arglist_next57;
       $$mask$i = $1034 & 255;
       $1035 = $986;
       $1036 = $1035;
       HEAP32[$1036>>2] = $$mask$i;
       $1037 = (($1035) + 4)|0;
       $1038 = $1037;
       HEAP32[$1038>>2] = 0;
       break L523;
       break;
      }
      case 17:  {
       $arglist_current59 = HEAP32[$ap>>2]|0;
       HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current59>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current59+4>>2];$1039 = +HEAPF64[tempDoublePtr>>3];
       $arglist_next60 = (($arglist_current59) + 8|0);
       HEAP32[$ap>>2] = $arglist_next60;
       HEAPF64[$986>>3] = $1039;
       break L523;
       break;
      }
      case 18:  {
       $arglist_current62 = HEAP32[$ap>>2]|0;
       HEAP32[tempDoublePtr>>2]=HEAP32[$arglist_current62>>2];HEAP32[tempDoublePtr+4>>2]=HEAP32[$arglist_current62+4>>2];$1040 = +HEAPF64[tempDoublePtr>>3];
       $arglist_next63 = (($arglist_current62) + 8|0);
       HEAP32[$ap>>2] = $arglist_next63;
       HEAPF64[$986>>3] = $1040;
       break L523;
       break;
      }
      default: {
       break L523;
      }
      }
     } while(0);
    }
   } while(0);
   $1041 = (($i$291) + 1)|0;
   $1042 = ($1041|0)<(10);
   if ($1042) {
    $i$291 = $1041;
   } else {
    $$0 = 1;
    label = 357;
    break;
   }
  }
  if ((label|0) == 357) {
   STACKTOP = sp;return ($$0|0);
  }
  while(1) {
   $1045 = (($nl_type) + ($i$390<<2)|0);
   $1046 = HEAP32[$1045>>2]|0;
   $1047 = ($1046|0)==(0);
   $1044 = (($i$390) + 1)|0;
   if (!($1047)) {
    $$0 = -1;
    label = 357;
    break;
   }
   $1043 = ($1044|0)<(10);
   if ($1043) {
    $i$390 = $1044;
   } else {
    $$0 = 1;
    label = 357;
    break;
   }
  }
  if ((label|0) == 357) {
   STACKTOP = sp;return ($$0|0);
  }
 }
 else if ((label|0) == 357) {
  STACKTOP = sp;return ($$0|0);
 }
 return 0|0;
}
function _vsnprintf($s,$n,$fmt,$ap) {
 $s = $s|0;
 $n = $n|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $$$02 = 0, $$0 = 0, $$01 = 0, $$02 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $f = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $b = sp + 112|0;
 $f = sp;
 dest=$f+0|0; src=15480+0|0; stop=dest+112|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $0 = (($n) + -1)|0;
 $1 = ($0>>>0)>(2147483646);
 if ($1) {
  $2 = ($n|0)==(0);
  if ($2) {
   $$01 = $b;$$02 = 1;
  } else {
   $3 = (___errno_location()|0);
   HEAP32[$3>>2] = 75;
   $$0 = -1;
   STACKTOP = sp;return ($$0|0);
  }
 } else {
  $$01 = $s;$$02 = $n;
 }
 $4 = $$01;
 $5 = (-2 - ($4))|0;
 $6 = ($$02>>>0)>($5>>>0);
 $$$02 = $6 ? $5 : $$02;
 $7 = (($f) + 48|0);
 HEAP32[$7>>2] = $$$02;
 $8 = (($f) + 20|0);
 HEAP32[$8>>2] = $$01;
 $9 = (($f) + 44|0);
 HEAP32[$9>>2] = $$01;
 $10 = (($$01) + ($$$02)|0);
 $11 = (($f) + 16|0);
 HEAP32[$11>>2] = $10;
 $12 = (($f) + 28|0);
 HEAP32[$12>>2] = $10;
 $13 = (_MUSL_vfprintf($f,$fmt,$ap)|0);
 $14 = ($$$02|0)==(0);
 if ($14) {
  $$0 = $13;
  STACKTOP = sp;return ($$0|0);
 }
 $15 = HEAP32[$8>>2]|0;
 $16 = HEAP32[$11>>2]|0;
 $17 = ($15|0)==($16|0);
 $18 = $17 << 31 >> 31;
 $19 = (($15) + ($18)|0);
 HEAP8[$19>>0] = 0;
 $$0 = $13;
 STACKTOP = sp;return ($$0|0);
}
function _sn_write($f,$s,$l) {
 $f = $f|0;
 $s = $s|0;
 $l = $l|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($f) + 16|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = (($f) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $1;
 $5 = $3;
 $6 = (($4) - ($5))|0;
 $7 = ($6>>>0)>($l>>>0);
 $l$ = $7 ? $l : $6;
 _memcpy(($3|0),($s|0),($l$|0))|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + ($l$)|0);
 HEAP32[$2>>2] = $9;
 STACKTOP = sp;return ($l|0);
}
function _vsprintf($s,$fmt,$ap) {
 $s = $s|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_vsnprintf($s,2147483647,$fmt,$ap)|0);
 STACKTOP = sp;return ($0|0);
}
function _atoi($s) {
 $s = $s|0;
 var $$0 = 0, $$1$ph = 0, $$12 = 0, $$neg1 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n$0$lcssa = 0, $n$03 = 0, $neg$0 = 0, $neg$1$ph = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $$0 = $s;
 while(1) {
  $0 = HEAP8[$$0>>0]|0;
  $1 = $0 << 24 >> 24;
  $2 = (_isspace($1)|0);
  $3 = ($2|0)==(0);
  $4 = (($$0) + 1|0);
  if ($3) {
   break;
  } else {
   $$0 = $4;
  }
 }
 $5 = HEAP8[$$0>>0]|0;
 $6 = $5 << 24 >> 24;
 if ((($6|0) == 45)) {
  $neg$0 = 1;
  label = 5;
 } else if ((($6|0) == 43)) {
  $neg$0 = 0;
  label = 5;
 } else {
  $$1$ph = $$0;$8 = $5;$neg$1$ph = 0;
 }
 if ((label|0) == 5) {
  $$pre = HEAP8[$4>>0]|0;
  $$1$ph = $4;$8 = $$pre;$neg$1$ph = $neg$0;
 }
 $7 = $8 << 24 >> 24;
 $9 = (_isdigit($7)|0);
 $10 = ($9|0)==(0);
 if ($10) {
  $n$0$lcssa = 0;
  $20 = ($neg$1$ph|0)!=(0);
  $21 = (0 - ($n$0$lcssa))|0;
  $22 = $20 ? $n$0$lcssa : $21;
  STACKTOP = sp;return ($22|0);
 } else {
  $$12 = $$1$ph;$n$03 = 0;
 }
 while(1) {
  $11 = ($n$03*10)|0;
  $12 = (($$12) + 1|0);
  $13 = HEAP8[$$12>>0]|0;
  $14 = $13 << 24 >> 24;
  $$neg1 = (($11) + 48)|0;
  $15 = (($$neg1) - ($14))|0;
  $16 = HEAP8[$12>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = (_isdigit($17)|0);
  $19 = ($18|0)==(0);
  if ($19) {
   $n$0$lcssa = $15;
   break;
  } else {
   $$12 = $12;$n$03 = $15;
  }
 }
 $20 = ($neg$1$ph|0)!=(0);
 $21 = (0 - ($n$0$lcssa))|0;
 $22 = $20 ? $n$0$lcssa : $21;
 STACKTOP = sp;return ($22|0);
}
function _memchr($src,$c,$n) {
 $src = $src|0;
 $c = $c|0;
 $n = $n|0;
 var $$0$lcssa = 0, $$0$lcssa34 = 0, $$013 = 0, $$1$lcssa = 0, $$17 = 0, $$24 = 0, $$3 = 0, $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond12 = 0, $s$0$lcssa = 0, $s$0$lcssa33 = 0, $s$014 = 0, $s$15 = 0, $s$2 = 0, $w$0$lcssa = 0, $w$08 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $c & 255;
 $1 = $src;
 $2 = $1 & 3;
 $3 = ($2|0)==(0);
 $4 = ($n|0)==(0);
 $or$cond12 = $3 | $4;
 L1: do {
  if ($or$cond12) {
   $$0$lcssa = $n;$$lcssa = $4;$s$0$lcssa = $src;
   label = 5;
  } else {
   $5 = $c&255;
   $$013 = $n;$s$014 = $src;
   while(1) {
    $6 = HEAP8[$s$014>>0]|0;
    $7 = ($6<<24>>24)==($5<<24>>24);
    if ($7) {
     $$0$lcssa34 = $$013;$s$0$lcssa33 = $s$014;
     label = 6;
     break L1;
    }
    $8 = (($s$014) + 1|0);
    $9 = (($$013) + -1)|0;
    $10 = $8;
    $11 = $10 & 3;
    $12 = ($11|0)==(0);
    $13 = ($9|0)==(0);
    $or$cond = $12 | $13;
    if ($or$cond) {
     $$0$lcssa = $9;$$lcssa = $13;$s$0$lcssa = $8;
     label = 5;
     break;
    } else {
     $$013 = $9;$s$014 = $8;
    }
   }
  }
 } while(0);
 if ((label|0) == 5) {
  if ($$lcssa) {
   $$3 = 0;$s$2 = $s$0$lcssa;
  } else {
   $$0$lcssa34 = $$0$lcssa;$s$0$lcssa33 = $s$0$lcssa;
   label = 6;
  }
 }
 L8: do {
  if ((label|0) == 6) {
   $14 = HEAP8[$s$0$lcssa33>>0]|0;
   $15 = $c&255;
   $16 = ($14<<24>>24)==($15<<24>>24);
   if ($16) {
    $$3 = $$0$lcssa34;$s$2 = $s$0$lcssa33;
   } else {
    $17 = Math_imul($0, 16843009)|0;
    $18 = ($$0$lcssa34>>>0)>(3);
    L11: do {
     if ($18) {
      $$17 = $$0$lcssa34;$w$08 = $s$0$lcssa33;
      while(1) {
       $19 = HEAP32[$w$08>>2]|0;
       $20 = $19 ^ $17;
       $21 = (($20) + -16843009)|0;
       $22 = $20 & -2139062144;
       $23 = $22 ^ -2139062144;
       $24 = $23 & $21;
       $25 = ($24|0)==(0);
       if (!($25)) {
        $$1$lcssa = $$17;$w$0$lcssa = $w$08;
        break L11;
       }
       $26 = (($w$08) + 4|0);
       $27 = (($$17) + -4)|0;
       $28 = ($27>>>0)>(3);
       if ($28) {
        $$17 = $27;$w$08 = $26;
       } else {
        $$1$lcssa = $27;$w$0$lcssa = $26;
        break;
       }
      }
     } else {
      $$1$lcssa = $$0$lcssa34;$w$0$lcssa = $s$0$lcssa33;
     }
    } while(0);
    $29 = ($$1$lcssa|0)==(0);
    if ($29) {
     $$3 = 0;$s$2 = $w$0$lcssa;
    } else {
     $$24 = $$1$lcssa;$s$15 = $w$0$lcssa;
     while(1) {
      $30 = HEAP8[$s$15>>0]|0;
      $31 = ($30<<24>>24)==($15<<24>>24);
      if ($31) {
       $$3 = $$24;$s$2 = $s$15;
       break L8;
      }
      $32 = (($s$15) + 1|0);
      $33 = (($$24) + -1)|0;
      $34 = ($33|0)==(0);
      if ($34) {
       $$3 = 0;$s$2 = $32;
       break;
      } else {
       $$24 = $33;$s$15 = $32;
      }
     }
    }
   }
  }
 } while(0);
 $35 = ($$3|0)!=(0);
 $36 = $35 ? $s$2 : 0;
 STACKTOP = sp;return ($36|0);
}
function _strcmp($l,$r) {
 $l = $l|0;
 $r = $r|0;
 var $$027 = 0, $$08 = 0, $$lcssa = 0, $$lcssa4 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0;
 var $or$cond3 = 0, $or$cond36 = 0, $or$cond5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP8[$l>>0]|0;
 $1 = HEAP8[$r>>0]|0;
 $2 = ($0<<24>>24)!=($1<<24>>24);
 $3 = ($0<<24>>24)==(0);
 $or$cond5 = $2 | $3;
 $4 = ($1<<24>>24)==(0);
 $or$cond36 = $or$cond5 | $4;
 if ($or$cond36) {
  $$lcssa = $0;$$lcssa4 = $1;
  $12 = $$lcssa&255;
  $13 = $$lcssa4&255;
  $14 = (($12) - ($13))|0;
  STACKTOP = sp;return ($14|0);
 } else {
  $$027 = $l;$$08 = $r;
 }
 while(1) {
  $5 = (($$027) + 1|0);
  $6 = (($$08) + 1|0);
  $7 = HEAP8[$5>>0]|0;
  $8 = HEAP8[$6>>0]|0;
  $9 = ($7<<24>>24)!=($8<<24>>24);
  $10 = ($7<<24>>24)==(0);
  $or$cond = $9 | $10;
  $11 = ($8<<24>>24)==(0);
  $or$cond3 = $or$cond | $11;
  if ($or$cond3) {
   $$lcssa = $7;$$lcssa4 = $8;
   break;
  } else {
   $$027 = $5;$$08 = $6;
  }
 }
 $12 = $$lcssa&255;
 $13 = $$lcssa4&255;
 $14 = (($12) - ($13))|0;
 STACKTOP = sp;return ($14|0);
}
function _strncmp($_l,$_r,$n) {
 $_l = $_l|0;
 $_r = $_r|0;
 $n = $n|0;
 var $$03 = 0, $$08 = 0, $$08$in = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$06 = 0, $notlhs = 0;
 var $notrhs = 0, $or$cond$not = 0, $or$cond4 = 0, $r$0$lcssa = 0, $r$07 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 if ($0) {
  $$03 = 0;
  STACKTOP = sp;return ($$03|0);
 }
 $1 = HEAP8[$_l>>0]|0;
 $2 = ($1<<24>>24)==(0);
 L4: do {
  if ($2) {
   $11 = 0;$r$0$lcssa = $_r;
  } else {
   $$08$in = $n;$5 = $1;$l$06 = $_l;$r$07 = $_r;
   while(1) {
    $$08 = (($$08$in) + -1)|0;
    $3 = HEAP8[$r$07>>0]|0;
    $notlhs = ($3<<24>>24)!=(0);
    $notrhs = ($$08|0)!=(0);
    $or$cond$not = $notrhs & $notlhs;
    $4 = ($5<<24>>24)==($3<<24>>24);
    $or$cond4 = $or$cond$not & $4;
    if (!($or$cond4)) {
     $11 = $5;$r$0$lcssa = $r$07;
     break L4;
    }
    $6 = (($l$06) + 1|0);
    $7 = (($r$07) + 1|0);
    $8 = HEAP8[$6>>0]|0;
    $9 = ($8<<24>>24)==(0);
    if ($9) {
     $11 = 0;$r$0$lcssa = $7;
     break;
    } else {
     $$08$in = $$08;$5 = $8;$l$06 = $6;$r$07 = $7;
    }
   }
  }
 } while(0);
 $10 = $11&255;
 $12 = HEAP8[$r$0$lcssa>>0]|0;
 $13 = $12&255;
 $14 = (($10) - ($13))|0;
 $$03 = $14;
 STACKTOP = sp;return ($$03|0);
}
function runPostSets() {
 
}
function _i64Subtract(a, b, c, d) {
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a - c)>>>0;
    h = (b - d)>>>0;
    h = (b - d - (((c>>>0) > (a>>>0))|0))>>>0; // Borrow one from high word to low word on underflow.
    return ((tempRet0 = h,l|0)|0);
}
function _strlen(ptr) {
    ptr = ptr|0;
    var curr = 0;
    curr = ptr;
    while (((HEAP8[((curr)>>0)])|0)) {
      curr = (curr + 1)|0;
    }
    return (curr - ptr)|0;
}
function _strcat(pdest, psrc) {
    pdest = pdest|0; psrc = psrc|0;
    var i = 0;
    var pdestEnd = 0;
    pdestEnd = (pdest + (_strlen(pdest)|0))|0;
    do {
      HEAP8[((pdestEnd+i)>>0)]=HEAP8[((psrc+i)>>0)];
      i = (i+1)|0;
    } while (((HEAP8[(((psrc)+(i-1))>>0)])|0));
    return pdest|0;
}
function _bitshift64Shl(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = (high << bits) | ((low&(ander << (32 - bits))) >>> (32 - bits));
      return low << bits;
    }
    tempRet0 = low << (bits - 32);
    return 0;
}
function _i64Add(a, b, c, d) {
    /*
      x = a + b*2^32
      y = c + d*2^32
      result = l + h*2^32
    */
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a + c)>>>0;
    h = (b + d + (((l>>>0) < (a>>>0))|0))>>>0; // Add carry from low word to high word on overflow.
    return ((tempRet0 = h,l|0)|0);
}
function _bitshift64Lshr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >>> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = 0;
    return (high >>> (bits - 32))|0;
}
function _memset(ptr, value, num) {
    ptr = ptr|0; value = value|0; num = num|0;
    var stop = 0, value4 = 0, stop4 = 0, unaligned = 0;
    stop = (ptr + num)|0;
    if ((num|0) >= 20) {
      // This is unaligned, but quite large, so work hard to get to aligned settings
      value = value & 0xff;
      unaligned = ptr & 3;
      value4 = value | (value << 8) | (value << 16) | (value << 24);
      stop4 = stop & ~3;
      if (unaligned) {
        unaligned = (ptr + 4 - unaligned)|0;
        while ((ptr|0) < (unaligned|0)) { // no need to check for stop, since we have large num
          HEAP8[((ptr)>>0)]=value;
          ptr = (ptr+1)|0;
        }
      }
      while ((ptr|0) < (stop4|0)) {
        HEAP32[((ptr)>>2)]=value4;
        ptr = (ptr+4)|0;
      }
    }
    while ((ptr|0) < (stop|0)) {
      HEAP8[((ptr)>>0)]=value;
      ptr = (ptr+1)|0;
    }
    return (ptr-num)|0;
}
function _memcpy(dest, src, num) {

    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if ((num|0) >= 4096) return _emscripten_memcpy_big(dest|0, src|0, num|0)|0;
    ret = dest|0;
    if ((dest&3) == (src&3)) {
      while (dest & 3) {
        if ((num|0) == 0) return ret|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
        dest = (dest+1)|0;
        src = (src+1)|0;
        num = (num-1)|0;
      }
      while ((num|0) >= 4) {
        HEAP32[((dest)>>2)]=((HEAP32[((src)>>2)])|0);
        dest = (dest+4)|0;
        src = (src+4)|0;
        num = (num-4)|0;
      }
    }
    while ((num|0) > 0) {
      HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      dest = (dest+1)|0;
      src = (src+1)|0;
      num = (num-1)|0;
    }
    return ret|0;
}
function _bitshift64Ashr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = (high|0) < 0 ? -1 : 0;
    return (high >> (bits - 32))|0;
  }
function _llvm_ctlz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((ctlz_i8)+(x >>> 24))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((ctlz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((ctlz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((ctlz_i8)+(x&0xff))>>0)])|0) + 24)|0;
  }

function _llvm_cttz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((cttz_i8)+(x & 0xff))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((cttz_i8)+(x >>> 24))>>0)])|0) + 24)|0;
  }

// ======== compiled code from system/lib/compiler-rt , see readme therein
function ___muldsi3($a, $b) {
  $a = $a | 0;
  $b = $b | 0;
  var $1 = 0, $2 = 0, $3 = 0, $6 = 0, $8 = 0, $11 = 0, $12 = 0;
  $1 = $a & 65535;
  $2 = $b & 65535;
  $3 = Math_imul($2, $1) | 0;
  $6 = $a >>> 16;
  $8 = ($3 >>> 16) + (Math_imul($2, $6) | 0) | 0;
  $11 = $b >>> 16;
  $12 = Math_imul($11, $1) | 0;
  return (tempRet0 = (($8 >>> 16) + (Math_imul($11, $6) | 0) | 0) + ((($8 & 65535) + $12 | 0) >>> 16) | 0, 0 | ($8 + $12 << 16 | $3 & 65535)) | 0;
}
function ___divdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $7$0 = 0, $7$1 = 0, $8$0 = 0, $10$0 = 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  $7$0 = $2$0 ^ $1$0;
  $7$1 = $2$1 ^ $1$1;
  $8$0 = ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, 0) | 0;
  $10$0 = _i64Subtract($8$0 ^ $7$0, tempRet0 ^ $7$1, $7$0, $7$1) | 0;
  return (tempRet0 = tempRet0, $10$0) | 0;
}
function ___remdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $10$0 = 0, $10$1 = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, $rem) | 0;
  $10$0 = _i64Subtract(HEAP32[$rem >> 2] ^ $1$0, HEAP32[$rem + 4 >> 2] ^ $1$1, $1$0, $1$1) | 0;
  $10$1 = tempRet0;
  STACKTOP = __stackBase__;
  return (tempRet0 = $10$1, $10$0) | 0;
}
function ___muldi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $x_sroa_0_0_extract_trunc = 0, $y_sroa_0_0_extract_trunc = 0, $1$0 = 0, $1$1 = 0, $2 = 0;
  $x_sroa_0_0_extract_trunc = $a$0;
  $y_sroa_0_0_extract_trunc = $b$0;
  $1$0 = ___muldsi3($x_sroa_0_0_extract_trunc, $y_sroa_0_0_extract_trunc) | 0;
  $1$1 = tempRet0;
  $2 = Math_imul($a$1, $y_sroa_0_0_extract_trunc) | 0;
  return (tempRet0 = ((Math_imul($b$1, $x_sroa_0_0_extract_trunc) | 0) + $2 | 0) + $1$1 | $1$1 & 0, 0 | $1$0 & -1) | 0;
}
function ___udivdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0;
  $1$0 = ___udivmoddi4($a$0, $a$1, $b$0, $b$1, 0) | 0;
  return (tempRet0 = tempRet0, $1$0) | 0;
}
function ___uremdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 8 | 0;
  $rem = __stackBase__ | 0;
  ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) | 0;
  STACKTOP = __stackBase__;
  return (tempRet0 = HEAP32[$rem + 4 >> 2] | 0, HEAP32[$rem >> 2] | 0) | 0;
}
function ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  $rem = $rem | 0;
  var $n_sroa_0_0_extract_trunc = 0, $n_sroa_1_4_extract_shift$0 = 0, $n_sroa_1_4_extract_trunc = 0, $d_sroa_0_0_extract_trunc = 0, $d_sroa_1_4_extract_shift$0 = 0, $d_sroa_1_4_extract_trunc = 0, $4 = 0, $17 = 0, $37 = 0, $49 = 0, $51 = 0, $57 = 0, $58 = 0, $66 = 0, $78 = 0, $86 = 0, $88 = 0, $89 = 0, $91 = 0, $92 = 0, $95 = 0, $105 = 0, $117 = 0, $119 = 0, $125 = 0, $126 = 0, $130 = 0, $q_sroa_1_1_ph = 0, $q_sroa_0_1_ph = 0, $r_sroa_1_1_ph = 0, $r_sroa_0_1_ph = 0, $sr_1_ph = 0, $d_sroa_0_0_insert_insert99$0 = 0, $d_sroa_0_0_insert_insert99$1 = 0, $137$0 = 0, $137$1 = 0, $carry_0203 = 0, $sr_1202 = 0, $r_sroa_0_1201 = 0, $r_sroa_1_1200 = 0, $q_sroa_0_1199 = 0, $q_sroa_1_1198 = 0, $147 = 0, $149 = 0, $r_sroa_0_0_insert_insert42$0 = 0, $r_sroa_0_0_insert_insert42$1 = 0, $150$1 = 0, $151$0 = 0, $152 = 0, $154$0 = 0, $r_sroa_0_0_extract_trunc = 0, $r_sroa_1_4_extract_trunc = 0, $155 = 0, $carry_0_lcssa$0 = 0, $carry_0_lcssa$1 = 0, $r_sroa_0_1_lcssa = 0, $r_sroa_1_1_lcssa = 0, $q_sroa_0_1_lcssa = 0, $q_sroa_1_1_lcssa = 0, $q_sroa_0_0_insert_ext75$0 = 0, $q_sroa_0_0_insert_ext75$1 = 0, $q_sroa_0_0_insert_insert77$1 = 0, $_0$0 = 0, $_0$1 = 0;
  $n_sroa_0_0_extract_trunc = $a$0;
  $n_sroa_1_4_extract_shift$0 = $a$1;
  $n_sroa_1_4_extract_trunc = $n_sroa_1_4_extract_shift$0;
  $d_sroa_0_0_extract_trunc = $b$0;
  $d_sroa_1_4_extract_shift$0 = $b$1;
  $d_sroa_1_4_extract_trunc = $d_sroa_1_4_extract_shift$0;
  if (($n_sroa_1_4_extract_trunc | 0) == 0) {
    $4 = ($rem | 0) != 0;
    if (($d_sroa_1_4_extract_trunc | 0) == 0) {
      if ($4) {
        HEAP32[$rem >> 2] = ($n_sroa_0_0_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
        HEAP32[$rem + 4 >> 2] = 0;
      }
      $_0$1 = 0;
      $_0$0 = ($n_sroa_0_0_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$4) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    }
  }
  $17 = ($d_sroa_1_4_extract_trunc | 0) == 0;
  do {
    if (($d_sroa_0_0_extract_trunc | 0) == 0) {
      if ($17) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
          HEAP32[$rem + 4 >> 2] = 0;
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      if (($n_sroa_0_0_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0;
          HEAP32[$rem + 4 >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_1_4_extract_trunc >>> 0);
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_1_4_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $37 = $d_sroa_1_4_extract_trunc - 1 | 0;
      if (($37 & $d_sroa_1_4_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0 | $a$0 & -1;
          HEAP32[$rem + 4 >> 2] = $37 & $n_sroa_1_4_extract_trunc | $a$1 & 0;
        }
        $_0$1 = 0;
        $_0$0 = $n_sroa_1_4_extract_trunc >>> ((_llvm_cttz_i32($d_sroa_1_4_extract_trunc | 0) | 0) >>> 0);
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $49 = _llvm_ctlz_i32($d_sroa_1_4_extract_trunc | 0) | 0;
      $51 = $49 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
      if ($51 >>> 0 <= 30) {
        $57 = $51 + 1 | 0;
        $58 = 31 - $51 | 0;
        $sr_1_ph = $57;
        $r_sroa_0_1_ph = $n_sroa_1_4_extract_trunc << $58 | $n_sroa_0_0_extract_trunc >>> ($57 >>> 0);
        $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($57 >>> 0);
        $q_sroa_0_1_ph = 0;
        $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $58;
        break;
      }
      if (($rem | 0) == 0) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = 0 | $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$17) {
        $117 = _llvm_ctlz_i32($d_sroa_1_4_extract_trunc | 0) | 0;
        $119 = $117 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        if ($119 >>> 0 <= 31) {
          $125 = $119 + 1 | 0;
          $126 = 31 - $119 | 0;
          $130 = $119 - 31 >> 31;
          $sr_1_ph = $125;
          $r_sroa_0_1_ph = $n_sroa_0_0_extract_trunc >>> ($125 >>> 0) & $130 | $n_sroa_1_4_extract_trunc << $126;
          $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($125 >>> 0) & $130;
          $q_sroa_0_1_ph = 0;
          $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $126;
          break;
        }
        if (($rem | 0) == 0) {
          $_0$1 = 0;
          $_0$0 = 0;
          return (tempRet0 = $_0$1, $_0$0) | 0;
        }
        HEAP32[$rem >> 2] = 0 | $a$0 & -1;
        HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $66 = $d_sroa_0_0_extract_trunc - 1 | 0;
      if (($66 & $d_sroa_0_0_extract_trunc | 0) != 0) {
        $86 = (_llvm_ctlz_i32($d_sroa_0_0_extract_trunc | 0) | 0) + 33 | 0;
        $88 = $86 - (_llvm_ctlz_i32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        $89 = 64 - $88 | 0;
        $91 = 32 - $88 | 0;
        $92 = $91 >> 31;
        $95 = $88 - 32 | 0;
        $105 = $95 >> 31;
        $sr_1_ph = $88;
        $r_sroa_0_1_ph = $91 - 1 >> 31 & $n_sroa_1_4_extract_trunc >>> ($95 >>> 0) | ($n_sroa_1_4_extract_trunc << $91 | $n_sroa_0_0_extract_trunc >>> ($88 >>> 0)) & $105;
        $r_sroa_1_1_ph = $105 & $n_sroa_1_4_extract_trunc >>> ($88 >>> 0);
        $q_sroa_0_1_ph = $n_sroa_0_0_extract_trunc << $89 & $92;
        $q_sroa_1_1_ph = ($n_sroa_1_4_extract_trunc << $89 | $n_sroa_0_0_extract_trunc >>> ($95 >>> 0)) & $92 | $n_sroa_0_0_extract_trunc << $91 & $88 - 33 >> 31;
        break;
      }
      if (($rem | 0) != 0) {
        HEAP32[$rem >> 2] = $66 & $n_sroa_0_0_extract_trunc;
        HEAP32[$rem + 4 >> 2] = 0;
      }
      if (($d_sroa_0_0_extract_trunc | 0) == 1) {
        $_0$1 = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$0 = 0 | $a$0 & -1;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      } else {
        $78 = _llvm_cttz_i32($d_sroa_0_0_extract_trunc | 0) | 0;
        $_0$1 = 0 | $n_sroa_1_4_extract_trunc >>> ($78 >>> 0);
        $_0$0 = $n_sroa_1_4_extract_trunc << 32 - $78 | $n_sroa_0_0_extract_trunc >>> ($78 >>> 0) | 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
    }
  } while (0);
  if (($sr_1_ph | 0) == 0) {
    $q_sroa_1_1_lcssa = $q_sroa_1_1_ph;
    $q_sroa_0_1_lcssa = $q_sroa_0_1_ph;
    $r_sroa_1_1_lcssa = $r_sroa_1_1_ph;
    $r_sroa_0_1_lcssa = $r_sroa_0_1_ph;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = 0;
  } else {
    $d_sroa_0_0_insert_insert99$0 = 0 | $b$0 & -1;
    $d_sroa_0_0_insert_insert99$1 = $d_sroa_1_4_extract_shift$0 | $b$1 & 0;
    $137$0 = _i64Add($d_sroa_0_0_insert_insert99$0, $d_sroa_0_0_insert_insert99$1, -1, -1) | 0;
    $137$1 = tempRet0;
    $q_sroa_1_1198 = $q_sroa_1_1_ph;
    $q_sroa_0_1199 = $q_sroa_0_1_ph;
    $r_sroa_1_1200 = $r_sroa_1_1_ph;
    $r_sroa_0_1201 = $r_sroa_0_1_ph;
    $sr_1202 = $sr_1_ph;
    $carry_0203 = 0;
    while (1) {
      $147 = $q_sroa_0_1199 >>> 31 | $q_sroa_1_1198 << 1;
      $149 = $carry_0203 | $q_sroa_0_1199 << 1;
      $r_sroa_0_0_insert_insert42$0 = 0 | ($r_sroa_0_1201 << 1 | $q_sroa_1_1198 >>> 31);
      $r_sroa_0_0_insert_insert42$1 = $r_sroa_0_1201 >>> 31 | $r_sroa_1_1200 << 1 | 0;
      _i64Subtract($137$0, $137$1, $r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1) | 0;
      $150$1 = tempRet0;
      $151$0 = $150$1 >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1;
      $152 = $151$0 & 1;
      $154$0 = _i64Subtract($r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1, $151$0 & $d_sroa_0_0_insert_insert99$0, ((($150$1 | 0) < 0 ? -1 : 0) >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1) & $d_sroa_0_0_insert_insert99$1) | 0;
      $r_sroa_0_0_extract_trunc = $154$0;
      $r_sroa_1_4_extract_trunc = tempRet0;
      $155 = $sr_1202 - 1 | 0;
      if (($155 | 0) == 0) {
        break;
      } else {
        $q_sroa_1_1198 = $147;
        $q_sroa_0_1199 = $149;
        $r_sroa_1_1200 = $r_sroa_1_4_extract_trunc;
        $r_sroa_0_1201 = $r_sroa_0_0_extract_trunc;
        $sr_1202 = $155;
        $carry_0203 = $152;
      }
    }
    $q_sroa_1_1_lcssa = $147;
    $q_sroa_0_1_lcssa = $149;
    $r_sroa_1_1_lcssa = $r_sroa_1_4_extract_trunc;
    $r_sroa_0_1_lcssa = $r_sroa_0_0_extract_trunc;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = $152;
  }
  $q_sroa_0_0_insert_ext75$0 = $q_sroa_0_1_lcssa;
  $q_sroa_0_0_insert_ext75$1 = 0;
  $q_sroa_0_0_insert_insert77$1 = $q_sroa_1_1_lcssa | $q_sroa_0_0_insert_ext75$1;
  if (($rem | 0) != 0) {
    HEAP32[$rem >> 2] = 0 | $r_sroa_0_1_lcssa;
    HEAP32[$rem + 4 >> 2] = $r_sroa_1_1_lcssa | 0;
  }
  $_0$1 = (0 | $q_sroa_0_0_insert_ext75$0) >>> 31 | $q_sroa_0_0_insert_insert77$1 << 1 | ($q_sroa_0_0_insert_ext75$1 << 1 | $q_sroa_0_0_insert_ext75$0 >>> 31) & 0 | $carry_0_lcssa$1;
  $_0$0 = ($q_sroa_0_0_insert_ext75$0 << 1 | 0 >>> 31) & -2 | $carry_0_lcssa$0;
  return (tempRet0 = $_0$1, $_0$0) | 0;
}
// =======================================================================



// EMSCRIPTEN_END_FUNCS

    
    function dynCall_iiii(index,a1,a2,a3) {
      index = index|0;
      a1=a1|0; a2=a2|0; a3=a3|0;
      return FUNCTION_TABLE_iiii[index&3](a1|0,a2|0,a3|0)|0;
    }
  
function b0(p0,p1,p2) { p0 = p0|0;p1 = p1|0;p2 = p2|0; nullFunc_iiii(0);return 0; }
  // EMSCRIPTEN_END_FUNCS
  var FUNCTION_TABLE_iiii = [b0,_sn_write,_do_read,b0];

    return { _i64Subtract: _i64Subtract, _strcat: _strcat, _free: _free, _main: _main, _realloc: _realloc, _i64Add: _i64Add, _strlen: _strlen, _memset: _memset, _malloc: _malloc, _memcpy: _memcpy, _bitshift64Lshr: _bitshift64Lshr, _calloc: _calloc, _bitshift64Shl: _bitshift64Shl, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_iiii: dynCall_iiii };
  })
  // EMSCRIPTEN_END_ASM
  ({ "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array }, { "abort": abort, "assert": assert, "asmPrintInt": asmPrintInt, "asmPrintFloat": asmPrintFloat, "min": Math_min, "nullFunc_iiii": nullFunc_iiii, "invoke_iiii": invoke_iiii, "_fabs": _fabs, "_llvm_pow_f64": _llvm_pow_f64, "_send": _send, "_fread": _fread, "_fmod": _fmod, "_lseek": _lseek, "__reallyNegative": __reallyNegative, "_fsync": _fsync, "_fflush": _fflush, "_pwrite": _pwrite, "_strerror_r": _strerror_r, "_open": _open, "_sbrk": _sbrk, "_emscripten_memcpy_big": _emscripten_memcpy_big, "_fileno": _fileno, "_sysconf": _sysconf, "_close": _close, "_fseek": _fseek, "_pread": _pread, "_printf": _printf, "_fclose": _fclose, "_log": _log, "_write": _write, "_ftell": _ftell, "___errno_location": ___errno_location, "_recv": _recv, "_copysign": _copysign, "_mkport": _mkport, "__exit": __exit, "_read": _read, "_copysignl": _copysignl, "_abort": _abort, "_fwrite": _fwrite, "_time": _time, "_fprintf": _fprintf, "_strerror": _strerror, "_ceil": _ceil, "__formatString": __formatString, "_sqrt": _sqrt, "_fopen": _fopen, "_exit": _exit, "___setErrNo": ___setErrNo, "_fmodl": _fmodl, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8, "ctlz_i8": ctlz_i8, "NaN": NaN, "Infinity": Infinity, "_stderr": _stderr, "_stdout": _stdout }, buffer);
  var _i64Subtract = Module["_i64Subtract"] = asm["_i64Subtract"];
var _strcat = Module["_strcat"] = asm["_strcat"];
var _free = Module["_free"] = asm["_free"];
var _main = Module["_main"] = asm["_main"];
var _realloc = Module["_realloc"] = asm["_realloc"];
var _i64Add = Module["_i64Add"] = asm["_i64Add"];
var _strlen = Module["_strlen"] = asm["_strlen"];
var _memset = Module["_memset"] = asm["_memset"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _bitshift64Lshr = Module["_bitshift64Lshr"] = asm["_bitshift64Lshr"];
var _calloc = Module["_calloc"] = asm["_calloc"];
var _bitshift64Shl = Module["_bitshift64Shl"] = asm["_bitshift64Shl"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
  
  Runtime.stackAlloc = asm['stackAlloc'];
  Runtime.stackSave = asm['stackSave'];
  Runtime.stackRestore = asm['stackRestore'];
  Runtime.setTempRet0 = asm['setTempRet0'];
  Runtime.getTempRet0 = asm['getTempRet0'];
  

// TODO: strip out parts of this we do not need

//======= begin closure i64 code =======

// Copyright 2009 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Defines a Long class for representing a 64-bit two's-complement
 * integer value, which faithfully simulates the behavior of a Java "long". This
 * implementation is derived from LongLib in GWT.
 *
 */

var i64Math = (function() { // Emscripten wrapper
  var goog = { math: {} };


  /**
   * Constructs a 64-bit two's-complement integer, given its low and high 32-bit
   * values as *signed* integers.  See the from* functions below for more
   * convenient ways of constructing Longs.
   *
   * The internal representation of a long is the two given signed, 32-bit values.
   * We use 32-bit pieces because these are the size of integers on which
   * Javascript performs bit-operations.  For operations like addition and
   * multiplication, we split each number into 16-bit pieces, which can easily be
   * multiplied within Javascript's floating-point representation without overflow
   * or change in sign.
   *
   * In the algorithms below, we frequently reduce the negative case to the
   * positive case by negating the input(s) and then post-processing the result.
   * Note that we must ALWAYS check specially whether those values are MIN_VALUE
   * (-2^63) because -MIN_VALUE == MIN_VALUE (since 2^63 cannot be represented as
   * a positive number, it overflows back into a negative).  Not handling this
   * case would often result in infinite recursion.
   *
   * @param {number} low  The low (signed) 32 bits of the long.
   * @param {number} high  The high (signed) 32 bits of the long.
   * @constructor
   */
  goog.math.Long = function(low, high) {
    /**
     * @type {number}
     * @private
     */
    this.low_ = low | 0;  // force into 32 signed bits.

    /**
     * @type {number}
     * @private
     */
    this.high_ = high | 0;  // force into 32 signed bits.
  };


  // NOTE: Common constant values ZERO, ONE, NEG_ONE, etc. are defined below the
  // from* methods on which they depend.


  /**
   * A cache of the Long representations of small integer values.
   * @type {!Object}
   * @private
   */
  goog.math.Long.IntCache_ = {};


  /**
   * Returns a Long representing the given (32-bit) integer value.
   * @param {number} value The 32-bit integer in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromInt = function(value) {
    if (-128 <= value && value < 128) {
      var cachedObj = goog.math.Long.IntCache_[value];
      if (cachedObj) {
        return cachedObj;
      }
    }

    var obj = new goog.math.Long(value | 0, value < 0 ? -1 : 0);
    if (-128 <= value && value < 128) {
      goog.math.Long.IntCache_[value] = obj;
    }
    return obj;
  };


  /**
   * Returns a Long representing the given value, provided that it is a finite
   * number.  Otherwise, zero is returned.
   * @param {number} value The number in question.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromNumber = function(value) {
    if (isNaN(value) || !isFinite(value)) {
      return goog.math.Long.ZERO;
    } else if (value <= -goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MIN_VALUE;
    } else if (value + 1 >= goog.math.Long.TWO_PWR_63_DBL_) {
      return goog.math.Long.MAX_VALUE;
    } else if (value < 0) {
      return goog.math.Long.fromNumber(-value).negate();
    } else {
      return new goog.math.Long(
          (value % goog.math.Long.TWO_PWR_32_DBL_) | 0,
          (value / goog.math.Long.TWO_PWR_32_DBL_) | 0);
    }
  };


  /**
   * Returns a Long representing the 64-bit integer that comes by concatenating
   * the given high and low bits.  Each is assumed to use 32 bits.
   * @param {number} lowBits The low 32-bits.
   * @param {number} highBits The high 32-bits.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromBits = function(lowBits, highBits) {
    return new goog.math.Long(lowBits, highBits);
  };


  /**
   * Returns a Long representation of the given string, written using the given
   * radix.
   * @param {string} str The textual representation of the Long.
   * @param {number=} opt_radix The radix in which the text is written.
   * @return {!goog.math.Long} The corresponding Long value.
   */
  goog.math.Long.fromString = function(str, opt_radix) {
    if (str.length == 0) {
      throw Error('number format error: empty string');
    }

    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (str.charAt(0) == '-') {
      return goog.math.Long.fromString(str.substring(1), radix).negate();
    } else if (str.indexOf('-') >= 0) {
      throw Error('number format error: interior "-" character: ' + str);
    }

    // Do several (8) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 8));

    var result = goog.math.Long.ZERO;
    for (var i = 0; i < str.length; i += 8) {
      var size = Math.min(8, str.length - i);
      var value = parseInt(str.substring(i, i + size), radix);
      if (size < 8) {
        var power = goog.math.Long.fromNumber(Math.pow(radix, size));
        result = result.multiply(power).add(goog.math.Long.fromNumber(value));
      } else {
        result = result.multiply(radixToPower);
        result = result.add(goog.math.Long.fromNumber(value));
      }
    }
    return result;
  };


  // NOTE: the compiler should inline these constant values below and then remove
  // these variables, so there should be no runtime penalty for these.


  /**
   * Number used repeated below in calculations.  This must appear before the
   * first call to any from* function below.
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_16_DBL_ = 1 << 16;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_24_DBL_ = 1 << 24;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_32_DBL_ =
      goog.math.Long.TWO_PWR_16_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_31_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ / 2;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_48_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_16_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_64_DBL_ =
      goog.math.Long.TWO_PWR_32_DBL_ * goog.math.Long.TWO_PWR_32_DBL_;


  /**
   * @type {number}
   * @private
   */
  goog.math.Long.TWO_PWR_63_DBL_ =
      goog.math.Long.TWO_PWR_64_DBL_ / 2;


  /** @type {!goog.math.Long} */
  goog.math.Long.ZERO = goog.math.Long.fromInt(0);


  /** @type {!goog.math.Long} */
  goog.math.Long.ONE = goog.math.Long.fromInt(1);


  /** @type {!goog.math.Long} */
  goog.math.Long.NEG_ONE = goog.math.Long.fromInt(-1);


  /** @type {!goog.math.Long} */
  goog.math.Long.MAX_VALUE =
      goog.math.Long.fromBits(0xFFFFFFFF | 0, 0x7FFFFFFF | 0);


  /** @type {!goog.math.Long} */
  goog.math.Long.MIN_VALUE = goog.math.Long.fromBits(0, 0x80000000 | 0);


  /**
   * @type {!goog.math.Long}
   * @private
   */
  goog.math.Long.TWO_PWR_24_ = goog.math.Long.fromInt(1 << 24);


  /** @return {number} The value, assuming it is a 32-bit integer. */
  goog.math.Long.prototype.toInt = function() {
    return this.low_;
  };


  /** @return {number} The closest floating-point representation to this value. */
  goog.math.Long.prototype.toNumber = function() {
    return this.high_ * goog.math.Long.TWO_PWR_32_DBL_ +
           this.getLowBitsUnsigned();
  };


  /**
   * @param {number=} opt_radix The radix in which the text should be written.
   * @return {string} The textual representation of this value.
   */
  goog.math.Long.prototype.toString = function(opt_radix) {
    var radix = opt_radix || 10;
    if (radix < 2 || 36 < radix) {
      throw Error('radix out of range: ' + radix);
    }

    if (this.isZero()) {
      return '0';
    }

    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        // We need to change the Long value before it can be negated, so we remove
        // the bottom-most digit in this base and then recurse to do the rest.
        var radixLong = goog.math.Long.fromNumber(radix);
        var div = this.div(radixLong);
        var rem = div.multiply(radixLong).subtract(this);
        return div.toString(radix) + rem.toInt().toString(radix);
      } else {
        return '-' + this.negate().toString(radix);
      }
    }

    // Do several (6) digits each time through the loop, so as to
    // minimize the calls to the very expensive emulated div.
    var radixToPower = goog.math.Long.fromNumber(Math.pow(radix, 6));

    var rem = this;
    var result = '';
    while (true) {
      var remDiv = rem.div(radixToPower);
      var intval = rem.subtract(remDiv.multiply(radixToPower)).toInt();
      var digits = intval.toString(radix);

      rem = remDiv;
      if (rem.isZero()) {
        return digits + result;
      } else {
        while (digits.length < 6) {
          digits = '0' + digits;
        }
        result = '' + digits + result;
      }
    }
  };


  /** @return {number} The high 32-bits as a signed value. */
  goog.math.Long.prototype.getHighBits = function() {
    return this.high_;
  };


  /** @return {number} The low 32-bits as a signed value. */
  goog.math.Long.prototype.getLowBits = function() {
    return this.low_;
  };


  /** @return {number} The low 32-bits as an unsigned value. */
  goog.math.Long.prototype.getLowBitsUnsigned = function() {
    return (this.low_ >= 0) ?
        this.low_ : goog.math.Long.TWO_PWR_32_DBL_ + this.low_;
  };


  /**
   * @return {number} Returns the number of bits needed to represent the absolute
   *     value of this Long.
   */
  goog.math.Long.prototype.getNumBitsAbs = function() {
    if (this.isNegative()) {
      if (this.equals(goog.math.Long.MIN_VALUE)) {
        return 64;
      } else {
        return this.negate().getNumBitsAbs();
      }
    } else {
      var val = this.high_ != 0 ? this.high_ : this.low_;
      for (var bit = 31; bit > 0; bit--) {
        if ((val & (1 << bit)) != 0) {
          break;
        }
      }
      return this.high_ != 0 ? bit + 33 : bit + 1;
    }
  };


  /** @return {boolean} Whether this value is zero. */
  goog.math.Long.prototype.isZero = function() {
    return this.high_ == 0 && this.low_ == 0;
  };


  /** @return {boolean} Whether this value is negative. */
  goog.math.Long.prototype.isNegative = function() {
    return this.high_ < 0;
  };


  /** @return {boolean} Whether this value is odd. */
  goog.math.Long.prototype.isOdd = function() {
    return (this.low_ & 1) == 1;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long equals the other.
   */
  goog.math.Long.prototype.equals = function(other) {
    return (this.high_ == other.high_) && (this.low_ == other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long does not equal the other.
   */
  goog.math.Long.prototype.notEquals = function(other) {
    return (this.high_ != other.high_) || (this.low_ != other.low_);
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than the other.
   */
  goog.math.Long.prototype.lessThan = function(other) {
    return this.compare(other) < 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is less than or equal to the other.
   */
  goog.math.Long.prototype.lessThanOrEqual = function(other) {
    return this.compare(other) <= 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than the other.
   */
  goog.math.Long.prototype.greaterThan = function(other) {
    return this.compare(other) > 0;
  };


  /**
   * @param {goog.math.Long} other Long to compare against.
   * @return {boolean} Whether this Long is greater than or equal to the other.
   */
  goog.math.Long.prototype.greaterThanOrEqual = function(other) {
    return this.compare(other) >= 0;
  };


  /**
   * Compares this Long with the given one.
   * @param {goog.math.Long} other Long to compare against.
   * @return {number} 0 if they are the same, 1 if the this is greater, and -1
   *     if the given one is greater.
   */
  goog.math.Long.prototype.compare = function(other) {
    if (this.equals(other)) {
      return 0;
    }

    var thisNeg = this.isNegative();
    var otherNeg = other.isNegative();
    if (thisNeg && !otherNeg) {
      return -1;
    }
    if (!thisNeg && otherNeg) {
      return 1;
    }

    // at this point, the signs are the same, so subtraction will not overflow
    if (this.subtract(other).isNegative()) {
      return -1;
    } else {
      return 1;
    }
  };


  /** @return {!goog.math.Long} The negation of this value. */
  goog.math.Long.prototype.negate = function() {
    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.MIN_VALUE;
    } else {
      return this.not().add(goog.math.Long.ONE);
    }
  };


  /**
   * Returns the sum of this and the given Long.
   * @param {goog.math.Long} other Long to add to this one.
   * @return {!goog.math.Long} The sum of this and the given Long.
   */
  goog.math.Long.prototype.add = function(other) {
    // Divide each number into 4 chunks of 16 bits, and then sum the chunks.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 + b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 + b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 + b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 + b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns the difference of this and the given Long.
   * @param {goog.math.Long} other Long to subtract from this.
   * @return {!goog.math.Long} The difference of this and the given Long.
   */
  goog.math.Long.prototype.subtract = function(other) {
    return this.add(other.negate());
  };


  /**
   * Returns the product of this and the given long.
   * @param {goog.math.Long} other Long to multiply with this.
   * @return {!goog.math.Long} The product of this and the other.
   */
  goog.math.Long.prototype.multiply = function(other) {
    if (this.isZero()) {
      return goog.math.Long.ZERO;
    } else if (other.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      return other.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return this.isOdd() ? goog.math.Long.MIN_VALUE : goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().multiply(other.negate());
      } else {
        return this.negate().multiply(other).negate();
      }
    } else if (other.isNegative()) {
      return this.multiply(other.negate()).negate();
    }

    // If both longs are small, use float multiplication
    if (this.lessThan(goog.math.Long.TWO_PWR_24_) &&
        other.lessThan(goog.math.Long.TWO_PWR_24_)) {
      return goog.math.Long.fromNumber(this.toNumber() * other.toNumber());
    }

    // Divide each long into 4 chunks of 16 bits, and then add up 4x4 products.
    // We can skip products that would overflow.

    var a48 = this.high_ >>> 16;
    var a32 = this.high_ & 0xFFFF;
    var a16 = this.low_ >>> 16;
    var a00 = this.low_ & 0xFFFF;

    var b48 = other.high_ >>> 16;
    var b32 = other.high_ & 0xFFFF;
    var b16 = other.low_ >>> 16;
    var b00 = other.low_ & 0xFFFF;

    var c48 = 0, c32 = 0, c16 = 0, c00 = 0;
    c00 += a00 * b00;
    c16 += c00 >>> 16;
    c00 &= 0xFFFF;
    c16 += a16 * b00;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c16 += a00 * b16;
    c32 += c16 >>> 16;
    c16 &= 0xFFFF;
    c32 += a32 * b00;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a16 * b16;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c32 += a00 * b32;
    c48 += c32 >>> 16;
    c32 &= 0xFFFF;
    c48 += a48 * b00 + a32 * b16 + a16 * b32 + a00 * b48;
    c48 &= 0xFFFF;
    return goog.math.Long.fromBits((c16 << 16) | c00, (c48 << 16) | c32);
  };


  /**
   * Returns this Long divided by the given one.
   * @param {goog.math.Long} other Long by which to divide.
   * @return {!goog.math.Long} This Long divided by the given one.
   */
  goog.math.Long.prototype.div = function(other) {
    if (other.isZero()) {
      throw Error('division by zero');
    } else if (this.isZero()) {
      return goog.math.Long.ZERO;
    }

    if (this.equals(goog.math.Long.MIN_VALUE)) {
      if (other.equals(goog.math.Long.ONE) ||
          other.equals(goog.math.Long.NEG_ONE)) {
        return goog.math.Long.MIN_VALUE;  // recall that -MIN_VALUE == MIN_VALUE
      } else if (other.equals(goog.math.Long.MIN_VALUE)) {
        return goog.math.Long.ONE;
      } else {
        // At this point, we have |other| >= 2, so |this/other| < |MIN_VALUE|.
        var halfThis = this.shiftRight(1);
        var approx = halfThis.div(other).shiftLeft(1);
        if (approx.equals(goog.math.Long.ZERO)) {
          return other.isNegative() ? goog.math.Long.ONE : goog.math.Long.NEG_ONE;
        } else {
          var rem = this.subtract(other.multiply(approx));
          var result = approx.add(rem.div(other));
          return result;
        }
      }
    } else if (other.equals(goog.math.Long.MIN_VALUE)) {
      return goog.math.Long.ZERO;
    }

    if (this.isNegative()) {
      if (other.isNegative()) {
        return this.negate().div(other.negate());
      } else {
        return this.negate().div(other).negate();
      }
    } else if (other.isNegative()) {
      return this.div(other.negate()).negate();
    }

    // Repeat the following until the remainder is less than other:  find a
    // floating-point that approximates remainder / other *from below*, add this
    // into the result, and subtract it from the remainder.  It is critical that
    // the approximate value is less than or equal to the real value so that the
    // remainder never becomes negative.
    var res = goog.math.Long.ZERO;
    var rem = this;
    while (rem.greaterThanOrEqual(other)) {
      // Approximate the result of division. This may be a little greater or
      // smaller than the actual value.
      var approx = Math.max(1, Math.floor(rem.toNumber() / other.toNumber()));

      // We will tweak the approximate result by changing it in the 48-th digit or
      // the smallest non-fractional digit, whichever is larger.
      var log2 = Math.ceil(Math.log(approx) / Math.LN2);
      var delta = (log2 <= 48) ? 1 : Math.pow(2, log2 - 48);

      // Decrease the approximation until it is smaller than the remainder.  Note
      // that if it is too large, the product overflows and is negative.
      var approxRes = goog.math.Long.fromNumber(approx);
      var approxRem = approxRes.multiply(other);
      while (approxRem.isNegative() || approxRem.greaterThan(rem)) {
        approx -= delta;
        approxRes = goog.math.Long.fromNumber(approx);
        approxRem = approxRes.multiply(other);
      }

      // We know the answer can't be zero... and actually, zero would cause
      // infinite recursion since we would make no progress.
      if (approxRes.isZero()) {
        approxRes = goog.math.Long.ONE;
      }

      res = res.add(approxRes);
      rem = rem.subtract(approxRem);
    }
    return res;
  };


  /**
   * Returns this Long modulo the given one.
   * @param {goog.math.Long} other Long by which to mod.
   * @return {!goog.math.Long} This Long modulo the given one.
   */
  goog.math.Long.prototype.modulo = function(other) {
    return this.subtract(this.div(other).multiply(other));
  };


  /** @return {!goog.math.Long} The bitwise-NOT of this value. */
  goog.math.Long.prototype.not = function() {
    return goog.math.Long.fromBits(~this.low_, ~this.high_);
  };


  /**
   * Returns the bitwise-AND of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to AND.
   * @return {!goog.math.Long} The bitwise-AND of this and the other.
   */
  goog.math.Long.prototype.and = function(other) {
    return goog.math.Long.fromBits(this.low_ & other.low_,
                                   this.high_ & other.high_);
  };


  /**
   * Returns the bitwise-OR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to OR.
   * @return {!goog.math.Long} The bitwise-OR of this and the other.
   */
  goog.math.Long.prototype.or = function(other) {
    return goog.math.Long.fromBits(this.low_ | other.low_,
                                   this.high_ | other.high_);
  };


  /**
   * Returns the bitwise-XOR of this Long and the given one.
   * @param {goog.math.Long} other The Long with which to XOR.
   * @return {!goog.math.Long} The bitwise-XOR of this and the other.
   */
  goog.math.Long.prototype.xor = function(other) {
    return goog.math.Long.fromBits(this.low_ ^ other.low_,
                                   this.high_ ^ other.high_);
  };


  /**
   * Returns this Long with bits shifted to the left by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the left by the given amount.
   */
  goog.math.Long.prototype.shiftLeft = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var low = this.low_;
      if (numBits < 32) {
        var high = this.high_;
        return goog.math.Long.fromBits(
            low << numBits,
            (high << numBits) | (low >>> (32 - numBits)));
      } else {
        return goog.math.Long.fromBits(0, low << (numBits - 32));
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount.
   */
  goog.math.Long.prototype.shiftRight = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >> numBits);
      } else {
        return goog.math.Long.fromBits(
            high >> (numBits - 32),
            high >= 0 ? 0 : -1);
      }
    }
  };


  /**
   * Returns this Long with bits shifted to the right by the given amount, with
   * the new top bits matching the current sign bit.
   * @param {number} numBits The number of bits by which to shift.
   * @return {!goog.math.Long} This shifted to the right by the given amount, with
   *     zeros placed into the new leading bits.
   */
  goog.math.Long.prototype.shiftRightUnsigned = function(numBits) {
    numBits &= 63;
    if (numBits == 0) {
      return this;
    } else {
      var high = this.high_;
      if (numBits < 32) {
        var low = this.low_;
        return goog.math.Long.fromBits(
            (low >>> numBits) | (high << (32 - numBits)),
            high >>> numBits);
      } else if (numBits == 32) {
        return goog.math.Long.fromBits(high, 0);
      } else {
        return goog.math.Long.fromBits(high >>> (numBits - 32), 0);
      }
    }
  };

  //======= begin jsbn =======

  var navigator = { appName: 'Modern Browser' }; // polyfill a little

  // Copyright (c) 2005  Tom Wu
  // All Rights Reserved.
  // http://www-cs-students.stanford.edu/~tjw/jsbn/

  /*
   * Copyright (c) 2003-2005  Tom Wu
   * All Rights Reserved.
   *
   * Permission is hereby granted, free of charge, to any person obtaining
   * a copy of this software and associated documentation files (the
   * "Software"), to deal in the Software without restriction, including
   * without limitation the rights to use, copy, modify, merge, publish,
   * distribute, sublicense, and/or sell copies of the Software, and to
   * permit persons to whom the Software is furnished to do so, subject to
   * the following conditions:
   *
   * The above copyright notice and this permission notice shall be
   * included in all copies or substantial portions of the Software.
   *
   * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
   * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
   * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
   *
   * IN NO EVENT SHALL TOM WU BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
   * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
   * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF
   * THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT
   * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
   *
   * In addition, the following condition applies:
   *
   * All redistributions must retain an intact copy of this copyright notice
   * and disclaimer.
   */

  // Basic JavaScript BN library - subset useful for RSA encryption.

  // Bits per digit
  var dbits;

  // JavaScript engine analysis
  var canary = 0xdeadbeefcafe;
  var j_lm = ((canary&0xffffff)==0xefcafe);

  // (public) Constructor
  function BigInteger(a,b,c) {
    if(a != null)
      if("number" == typeof a) this.fromNumber(a,b,c);
      else if(b == null && "string" != typeof a) this.fromString(a,256);
      else this.fromString(a,b);
  }

  // return new, unset BigInteger
  function nbi() { return new BigInteger(null); }

  // am: Compute w_j += (x*this_i), propagate carries,
  // c is initial carry, returns final carry.
  // c < 3*dvalue, x < 2*dvalue, this_i < dvalue
  // We need to select the fastest one that works in this environment.

  // am1: use a single mult and divide to get the high bits,
  // max digit bits should be 26 because
  // max internal value = 2*dvalue^2-2*dvalue (< 2^53)
  function am1(i,x,w,j,c,n) {
    while(--n >= 0) {
      var v = x*this[i++]+w[j]+c;
      c = Math.floor(v/0x4000000);
      w[j++] = v&0x3ffffff;
    }
    return c;
  }
  // am2 avoids a big mult-and-extract completely.
  // Max digit bits should be <= 30 because we do bitwise ops
  // on values up to 2*hdvalue^2-hdvalue-1 (< 2^31)
  function am2(i,x,w,j,c,n) {
    var xl = x&0x7fff, xh = x>>15;
    while(--n >= 0) {
      var l = this[i]&0x7fff;
      var h = this[i++]>>15;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x7fff)<<15)+w[j]+(c&0x3fffffff);
      c = (l>>>30)+(m>>>15)+xh*h+(c>>>30);
      w[j++] = l&0x3fffffff;
    }
    return c;
  }
  // Alternately, set max digit bits to 28 since some
  // browsers slow down when dealing with 32-bit numbers.
  function am3(i,x,w,j,c,n) {
    var xl = x&0x3fff, xh = x>>14;
    while(--n >= 0) {
      var l = this[i]&0x3fff;
      var h = this[i++]>>14;
      var m = xh*l+h*xl;
      l = xl*l+((m&0x3fff)<<14)+w[j]+c;
      c = (l>>28)+(m>>14)+xh*h;
      w[j++] = l&0xfffffff;
    }
    return c;
  }
  if(j_lm && (navigator.appName == "Microsoft Internet Explorer")) {
    BigInteger.prototype.am = am2;
    dbits = 30;
  }
  else if(j_lm && (navigator.appName != "Netscape")) {
    BigInteger.prototype.am = am1;
    dbits = 26;
  }
  else { // Mozilla/Netscape seems to prefer am3
    BigInteger.prototype.am = am3;
    dbits = 28;
  }

  BigInteger.prototype.DB = dbits;
  BigInteger.prototype.DM = ((1<<dbits)-1);
  BigInteger.prototype.DV = (1<<dbits);

  var BI_FP = 52;
  BigInteger.prototype.FV = Math.pow(2,BI_FP);
  BigInteger.prototype.F1 = BI_FP-dbits;
  BigInteger.prototype.F2 = 2*dbits-BI_FP;

  // Digit conversions
  var BI_RM = "0123456789abcdefghijklmnopqrstuvwxyz";
  var BI_RC = new Array();
  var rr,vv;
  rr = "0".charCodeAt(0);
  for(vv = 0; vv <= 9; ++vv) BI_RC[rr++] = vv;
  rr = "a".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;
  rr = "A".charCodeAt(0);
  for(vv = 10; vv < 36; ++vv) BI_RC[rr++] = vv;

  function int2char(n) { return BI_RM.charAt(n); }
  function intAt(s,i) {
    var c = BI_RC[s.charCodeAt(i)];
    return (c==null)?-1:c;
  }

  // (protected) copy this to r
  function bnpCopyTo(r) {
    for(var i = this.t-1; i >= 0; --i) r[i] = this[i];
    r.t = this.t;
    r.s = this.s;
  }

  // (protected) set from integer value x, -DV <= x < DV
  function bnpFromInt(x) {
    this.t = 1;
    this.s = (x<0)?-1:0;
    if(x > 0) this[0] = x;
    else if(x < -1) this[0] = x+DV;
    else this.t = 0;
  }

  // return bigint initialized to value
  function nbv(i) { var r = nbi(); r.fromInt(i); return r; }

  // (protected) set from string and radix
  function bnpFromString(s,b) {
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 256) k = 8; // byte array
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else { this.fromRadix(s,b); return; }
    this.t = 0;
    this.s = 0;
    var i = s.length, mi = false, sh = 0;
    while(--i >= 0) {
      var x = (k==8)?s[i]&0xff:intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-") mi = true;
        continue;
      }
      mi = false;
      if(sh == 0)
        this[this.t++] = x;
      else if(sh+k > this.DB) {
        this[this.t-1] |= (x&((1<<(this.DB-sh))-1))<<sh;
        this[this.t++] = (x>>(this.DB-sh));
      }
      else
        this[this.t-1] |= x<<sh;
      sh += k;
      if(sh >= this.DB) sh -= this.DB;
    }
    if(k == 8 && (s[0]&0x80) != 0) {
      this.s = -1;
      if(sh > 0) this[this.t-1] |= ((1<<(this.DB-sh))-1)<<sh;
    }
    this.clamp();
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) clamp off excess high words
  function bnpClamp() {
    var c = this.s&this.DM;
    while(this.t > 0 && this[this.t-1] == c) --this.t;
  }

  // (public) return string representation in given radix
  function bnToString(b) {
    if(this.s < 0) return "-"+this.negate().toString(b);
    var k;
    if(b == 16) k = 4;
    else if(b == 8) k = 3;
    else if(b == 2) k = 1;
    else if(b == 32) k = 5;
    else if(b == 4) k = 2;
    else return this.toRadix(b);
    var km = (1<<k)-1, d, m = false, r = "", i = this.t;
    var p = this.DB-(i*this.DB)%k;
    if(i-- > 0) {
      if(p < this.DB && (d = this[i]>>p) > 0) { m = true; r = int2char(d); }
      while(i >= 0) {
        if(p < k) {
          d = (this[i]&((1<<p)-1))<<(k-p);
          d |= this[--i]>>(p+=this.DB-k);
        }
        else {
          d = (this[i]>>(p-=k))&km;
          if(p <= 0) { p += this.DB; --i; }
        }
        if(d > 0) m = true;
        if(m) r += int2char(d);
      }
    }
    return m?r:"0";
  }

  // (public) -this
  function bnNegate() { var r = nbi(); BigInteger.ZERO.subTo(this,r); return r; }

  // (public) |this|
  function bnAbs() { return (this.s<0)?this.negate():this; }

  // (public) return + if this > a, - if this < a, 0 if equal
  function bnCompareTo(a) {
    var r = this.s-a.s;
    if(r != 0) return r;
    var i = this.t;
    r = i-a.t;
    if(r != 0) return (this.s<0)?-r:r;
    while(--i >= 0) if((r=this[i]-a[i]) != 0) return r;
    return 0;
  }

  // returns bit length of the integer x
  function nbits(x) {
    var r = 1, t;
    if((t=x>>>16) != 0) { x = t; r += 16; }
    if((t=x>>8) != 0) { x = t; r += 8; }
    if((t=x>>4) != 0) { x = t; r += 4; }
    if((t=x>>2) != 0) { x = t; r += 2; }
    if((t=x>>1) != 0) { x = t; r += 1; }
    return r;
  }

  // (public) return the number of bits in "this"
  function bnBitLength() {
    if(this.t <= 0) return 0;
    return this.DB*(this.t-1)+nbits(this[this.t-1]^(this.s&this.DM));
  }

  // (protected) r = this << n*DB
  function bnpDLShiftTo(n,r) {
    var i;
    for(i = this.t-1; i >= 0; --i) r[i+n] = this[i];
    for(i = n-1; i >= 0; --i) r[i] = 0;
    r.t = this.t+n;
    r.s = this.s;
  }

  // (protected) r = this >> n*DB
  function bnpDRShiftTo(n,r) {
    for(var i = n; i < this.t; ++i) r[i-n] = this[i];
    r.t = Math.max(this.t-n,0);
    r.s = this.s;
  }

  // (protected) r = this << n
  function bnpLShiftTo(n,r) {
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<cbs)-1;
    var ds = Math.floor(n/this.DB), c = (this.s<<bs)&this.DM, i;
    for(i = this.t-1; i >= 0; --i) {
      r[i+ds+1] = (this[i]>>cbs)|c;
      c = (this[i]&bm)<<bs;
    }
    for(i = ds-1; i >= 0; --i) r[i] = 0;
    r[ds] = c;
    r.t = this.t+ds+1;
    r.s = this.s;
    r.clamp();
  }

  // (protected) r = this >> n
  function bnpRShiftTo(n,r) {
    r.s = this.s;
    var ds = Math.floor(n/this.DB);
    if(ds >= this.t) { r.t = 0; return; }
    var bs = n%this.DB;
    var cbs = this.DB-bs;
    var bm = (1<<bs)-1;
    r[0] = this[ds]>>bs;
    for(var i = ds+1; i < this.t; ++i) {
      r[i-ds-1] |= (this[i]&bm)<<cbs;
      r[i-ds] = this[i]>>bs;
    }
    if(bs > 0) r[this.t-ds-1] |= (this.s&bm)<<cbs;
    r.t = this.t-ds;
    r.clamp();
  }

  // (protected) r = this - a
  function bnpSubTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]-a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c -= a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c -= a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c -= a.s;
    }
    r.s = (c<0)?-1:0;
    if(c < -1) r[i++] = this.DV+c;
    else if(c > 0) r[i++] = c;
    r.t = i;
    r.clamp();
  }

  // (protected) r = this * a, r != this,a (HAC 14.12)
  // "this" should be the larger one if appropriate.
  function bnpMultiplyTo(a,r) {
    var x = this.abs(), y = a.abs();
    var i = x.t;
    r.t = i+y.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < y.t; ++i) r[i+x.t] = x.am(0,y[i],r,i,0,x.t);
    r.s = 0;
    r.clamp();
    if(this.s != a.s) BigInteger.ZERO.subTo(r,r);
  }

  // (protected) r = this^2, r != this (HAC 14.16)
  function bnpSquareTo(r) {
    var x = this.abs();
    var i = r.t = 2*x.t;
    while(--i >= 0) r[i] = 0;
    for(i = 0; i < x.t-1; ++i) {
      var c = x.am(i,x[i],r,2*i,0,1);
      if((r[i+x.t]+=x.am(i+1,2*x[i],r,2*i+1,c,x.t-i-1)) >= x.DV) {
        r[i+x.t] -= x.DV;
        r[i+x.t+1] = 1;
      }
    }
    if(r.t > 0) r[r.t-1] += x.am(i,x[i],r,2*i,0,1);
    r.s = 0;
    r.clamp();
  }

  // (protected) divide this by m, quotient and remainder to q, r (HAC 14.20)
  // r != q, this != m.  q or r may be null.
  function bnpDivRemTo(m,q,r) {
    var pm = m.abs();
    if(pm.t <= 0) return;
    var pt = this.abs();
    if(pt.t < pm.t) {
      if(q != null) q.fromInt(0);
      if(r != null) this.copyTo(r);
      return;
    }
    if(r == null) r = nbi();
    var y = nbi(), ts = this.s, ms = m.s;
    var nsh = this.DB-nbits(pm[pm.t-1]);	// normalize modulus
    if(nsh > 0) { pm.lShiftTo(nsh,y); pt.lShiftTo(nsh,r); }
    else { pm.copyTo(y); pt.copyTo(r); }
    var ys = y.t;
    var y0 = y[ys-1];
    if(y0 == 0) return;
    var yt = y0*(1<<this.F1)+((ys>1)?y[ys-2]>>this.F2:0);
    var d1 = this.FV/yt, d2 = (1<<this.F1)/yt, e = 1<<this.F2;
    var i = r.t, j = i-ys, t = (q==null)?nbi():q;
    y.dlShiftTo(j,t);
    if(r.compareTo(t) >= 0) {
      r[r.t++] = 1;
      r.subTo(t,r);
    }
    BigInteger.ONE.dlShiftTo(ys,t);
    t.subTo(y,y);	// "negative" y so we can replace sub with am later
    while(y.t < ys) y[y.t++] = 0;
    while(--j >= 0) {
      // Estimate quotient digit
      var qd = (r[--i]==y0)?this.DM:Math.floor(r[i]*d1+(r[i-1]+e)*d2);
      if((r[i]+=y.am(0,qd,r,j,0,ys)) < qd) {	// Try it out
        y.dlShiftTo(j,t);
        r.subTo(t,r);
        while(r[i] < --qd) r.subTo(t,r);
      }
    }
    if(q != null) {
      r.drShiftTo(ys,q);
      if(ts != ms) BigInteger.ZERO.subTo(q,q);
    }
    r.t = ys;
    r.clamp();
    if(nsh > 0) r.rShiftTo(nsh,r);	// Denormalize remainder
    if(ts < 0) BigInteger.ZERO.subTo(r,r);
  }

  // (public) this mod a
  function bnMod(a) {
    var r = nbi();
    this.abs().divRemTo(a,null,r);
    if(this.s < 0 && r.compareTo(BigInteger.ZERO) > 0) a.subTo(r,r);
    return r;
  }

  // Modular reduction using "classic" algorithm
  function Classic(m) { this.m = m; }
  function cConvert(x) {
    if(x.s < 0 || x.compareTo(this.m) >= 0) return x.mod(this.m);
    else return x;
  }
  function cRevert(x) { return x; }
  function cReduce(x) { x.divRemTo(this.m,null,x); }
  function cMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }
  function cSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  Classic.prototype.convert = cConvert;
  Classic.prototype.revert = cRevert;
  Classic.prototype.reduce = cReduce;
  Classic.prototype.mulTo = cMulTo;
  Classic.prototype.sqrTo = cSqrTo;

  // (protected) return "-1/this % 2^DB"; useful for Mont. reduction
  // justification:
  //         xy == 1 (mod m)
  //         xy =  1+km
  //   xy(2-xy) = (1+km)(1-km)
  // x[y(2-xy)] = 1-k^2m^2
  // x[y(2-xy)] == 1 (mod m^2)
  // if y is 1/x mod m, then y(2-xy) is 1/x mod m^2
  // should reduce x and y(2-xy) by m^2 at each step to keep size bounded.
  // JS multiply "overflows" differently from C/C++, so care is needed here.
  function bnpInvDigit() {
    if(this.t < 1) return 0;
    var x = this[0];
    if((x&1) == 0) return 0;
    var y = x&3;		// y == 1/x mod 2^2
    y = (y*(2-(x&0xf)*y))&0xf;	// y == 1/x mod 2^4
    y = (y*(2-(x&0xff)*y))&0xff;	// y == 1/x mod 2^8
    y = (y*(2-(((x&0xffff)*y)&0xffff)))&0xffff;	// y == 1/x mod 2^16
    // last step - calculate inverse mod DV directly;
    // assumes 16 < DB <= 32 and assumes ability to handle 48-bit ints
    y = (y*(2-x*y%this.DV))%this.DV;		// y == 1/x mod 2^dbits
    // we really want the negative inverse, and -DV < y < DV
    return (y>0)?this.DV-y:-y;
  }

  // Montgomery reduction
  function Montgomery(m) {
    this.m = m;
    this.mp = m.invDigit();
    this.mpl = this.mp&0x7fff;
    this.mph = this.mp>>15;
    this.um = (1<<(m.DB-15))-1;
    this.mt2 = 2*m.t;
  }

  // xR mod m
  function montConvert(x) {
    var r = nbi();
    x.abs().dlShiftTo(this.m.t,r);
    r.divRemTo(this.m,null,r);
    if(x.s < 0 && r.compareTo(BigInteger.ZERO) > 0) this.m.subTo(r,r);
    return r;
  }

  // x/R mod m
  function montRevert(x) {
    var r = nbi();
    x.copyTo(r);
    this.reduce(r);
    return r;
  }

  // x = x/R mod m (HAC 14.32)
  function montReduce(x) {
    while(x.t <= this.mt2)	// pad x so am has enough room later
      x[x.t++] = 0;
    for(var i = 0; i < this.m.t; ++i) {
      // faster way of calculating u0 = x[i]*mp mod DV
      var j = x[i]&0x7fff;
      var u0 = (j*this.mpl+(((j*this.mph+(x[i]>>15)*this.mpl)&this.um)<<15))&x.DM;
      // use am to combine the multiply-shift-add into one call
      j = i+this.m.t;
      x[j] += this.m.am(0,u0,x,i,0,this.m.t);
      // propagate carry
      while(x[j] >= x.DV) { x[j] -= x.DV; x[++j]++; }
    }
    x.clamp();
    x.drShiftTo(this.m.t,x);
    if(x.compareTo(this.m) >= 0) x.subTo(this.m,x);
  }

  // r = "x^2/R mod m"; x != r
  function montSqrTo(x,r) { x.squareTo(r); this.reduce(r); }

  // r = "xy/R mod m"; x,y != r
  function montMulTo(x,y,r) { x.multiplyTo(y,r); this.reduce(r); }

  Montgomery.prototype.convert = montConvert;
  Montgomery.prototype.revert = montRevert;
  Montgomery.prototype.reduce = montReduce;
  Montgomery.prototype.mulTo = montMulTo;
  Montgomery.prototype.sqrTo = montSqrTo;

  // (protected) true iff this is even
  function bnpIsEven() { return ((this.t>0)?(this[0]&1):this.s) == 0; }

  // (protected) this^e, e < 2^32, doing sqr and mul with "r" (HAC 14.79)
  function bnpExp(e,z) {
    if(e > 0xffffffff || e < 1) return BigInteger.ONE;
    var r = nbi(), r2 = nbi(), g = z.convert(this), i = nbits(e)-1;
    g.copyTo(r);
    while(--i >= 0) {
      z.sqrTo(r,r2);
      if((e&(1<<i)) > 0) z.mulTo(r2,g,r);
      else { var t = r; r = r2; r2 = t; }
    }
    return z.revert(r);
  }

  // (public) this^e % m, 0 <= e < 2^32
  function bnModPowInt(e,m) {
    var z;
    if(e < 256 || m.isEven()) z = new Classic(m); else z = new Montgomery(m);
    return this.exp(e,z);
  }

  // protected
  BigInteger.prototype.copyTo = bnpCopyTo;
  BigInteger.prototype.fromInt = bnpFromInt;
  BigInteger.prototype.fromString = bnpFromString;
  BigInteger.prototype.clamp = bnpClamp;
  BigInteger.prototype.dlShiftTo = bnpDLShiftTo;
  BigInteger.prototype.drShiftTo = bnpDRShiftTo;
  BigInteger.prototype.lShiftTo = bnpLShiftTo;
  BigInteger.prototype.rShiftTo = bnpRShiftTo;
  BigInteger.prototype.subTo = bnpSubTo;
  BigInteger.prototype.multiplyTo = bnpMultiplyTo;
  BigInteger.prototype.squareTo = bnpSquareTo;
  BigInteger.prototype.divRemTo = bnpDivRemTo;
  BigInteger.prototype.invDigit = bnpInvDigit;
  BigInteger.prototype.isEven = bnpIsEven;
  BigInteger.prototype.exp = bnpExp;

  // public
  BigInteger.prototype.toString = bnToString;
  BigInteger.prototype.negate = bnNegate;
  BigInteger.prototype.abs = bnAbs;
  BigInteger.prototype.compareTo = bnCompareTo;
  BigInteger.prototype.bitLength = bnBitLength;
  BigInteger.prototype.mod = bnMod;
  BigInteger.prototype.modPowInt = bnModPowInt;

  // "constants"
  BigInteger.ZERO = nbv(0);
  BigInteger.ONE = nbv(1);

  // jsbn2 stuff

  // (protected) convert from radix string
  function bnpFromRadix(s,b) {
    this.fromInt(0);
    if(b == null) b = 10;
    var cs = this.chunkSize(b);
    var d = Math.pow(b,cs), mi = false, j = 0, w = 0;
    for(var i = 0; i < s.length; ++i) {
      var x = intAt(s,i);
      if(x < 0) {
        if(s.charAt(i) == "-" && this.signum() == 0) mi = true;
        continue;
      }
      w = b*w+x;
      if(++j >= cs) {
        this.dMultiply(d);
        this.dAddOffset(w,0);
        j = 0;
        w = 0;
      }
    }
    if(j > 0) {
      this.dMultiply(Math.pow(b,j));
      this.dAddOffset(w,0);
    }
    if(mi) BigInteger.ZERO.subTo(this,this);
  }

  // (protected) return x s.t. r^x < DV
  function bnpChunkSize(r) { return Math.floor(Math.LN2*this.DB/Math.log(r)); }

  // (public) 0 if this == 0, 1 if this > 0
  function bnSigNum() {
    if(this.s < 0) return -1;
    else if(this.t <= 0 || (this.t == 1 && this[0] <= 0)) return 0;
    else return 1;
  }

  // (protected) this *= n, this >= 0, 1 < n < DV
  function bnpDMultiply(n) {
    this[this.t] = this.am(0,n-1,this,0,0,this.t);
    ++this.t;
    this.clamp();
  }

  // (protected) this += n << w words, this >= 0
  function bnpDAddOffset(n,w) {
    if(n == 0) return;
    while(this.t <= w) this[this.t++] = 0;
    this[w] += n;
    while(this[w] >= this.DV) {
      this[w] -= this.DV;
      if(++w >= this.t) this[this.t++] = 0;
      ++this[w];
    }
  }

  // (protected) convert to radix string
  function bnpToRadix(b) {
    if(b == null) b = 10;
    if(this.signum() == 0 || b < 2 || b > 36) return "0";
    var cs = this.chunkSize(b);
    var a = Math.pow(b,cs);
    var d = nbv(a), y = nbi(), z = nbi(), r = "";
    this.divRemTo(d,y,z);
    while(y.signum() > 0) {
      r = (a+z.intValue()).toString(b).substr(1) + r;
      y.divRemTo(d,y,z);
    }
    return z.intValue().toString(b) + r;
  }

  // (public) return value as integer
  function bnIntValue() {
    if(this.s < 0) {
      if(this.t == 1) return this[0]-this.DV;
      else if(this.t == 0) return -1;
    }
    else if(this.t == 1) return this[0];
    else if(this.t == 0) return 0;
    // assumes 16 < DB < 32
    return ((this[1]&((1<<(32-this.DB))-1))<<this.DB)|this[0];
  }

  // (protected) r = this + a
  function bnpAddTo(a,r) {
    var i = 0, c = 0, m = Math.min(a.t,this.t);
    while(i < m) {
      c += this[i]+a[i];
      r[i++] = c&this.DM;
      c >>= this.DB;
    }
    if(a.t < this.t) {
      c += a.s;
      while(i < this.t) {
        c += this[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += this.s;
    }
    else {
      c += this.s;
      while(i < a.t) {
        c += a[i];
        r[i++] = c&this.DM;
        c >>= this.DB;
      }
      c += a.s;
    }
    r.s = (c<0)?-1:0;
    if(c > 0) r[i++] = c;
    else if(c < -1) r[i++] = this.DV+c;
    r.t = i;
    r.clamp();
  }

  BigInteger.prototype.fromRadix = bnpFromRadix;
  BigInteger.prototype.chunkSize = bnpChunkSize;
  BigInteger.prototype.signum = bnSigNum;
  BigInteger.prototype.dMultiply = bnpDMultiply;
  BigInteger.prototype.dAddOffset = bnpDAddOffset;
  BigInteger.prototype.toRadix = bnpToRadix;
  BigInteger.prototype.intValue = bnIntValue;
  BigInteger.prototype.addTo = bnpAddTo;

  //======= end jsbn =======

  // Emscripten wrapper
  var Wrapper = {
    abs: function(l, h) {
      var x = new goog.math.Long(l, h);
      var ret;
      if (x.isNegative()) {
        ret = x.negate();
      } else {
        ret = x;
      }
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
    },
    ensureTemps: function() {
      if (Wrapper.ensuredTemps) return;
      Wrapper.ensuredTemps = true;
      Wrapper.two32 = new BigInteger();
      Wrapper.two32.fromString('4294967296', 10);
      Wrapper.two64 = new BigInteger();
      Wrapper.two64.fromString('18446744073709551616', 10);
      Wrapper.temp1 = new BigInteger();
      Wrapper.temp2 = new BigInteger();
    },
    lh2bignum: function(l, h) {
      var a = new BigInteger();
      a.fromString(h.toString(), 10);
      var b = new BigInteger();
      a.multiplyTo(Wrapper.two32, b);
      var c = new BigInteger();
      c.fromString(l.toString(), 10);
      var d = new BigInteger();
      c.addTo(b, d);
      return d;
    },
    stringify: function(l, h, unsigned) {
      var ret = new goog.math.Long(l, h).toString();
      if (unsigned && ret[0] == '-') {
        // unsign slowly using jsbn bignums
        Wrapper.ensureTemps();
        var bignum = new BigInteger();
        bignum.fromString(ret, 10);
        ret = new BigInteger();
        Wrapper.two64.addTo(bignum, ret);
        ret = ret.toString(10);
      }
      return ret;
    },
    fromString: function(str, base, min, max, unsigned) {
      Wrapper.ensureTemps();
      var bignum = new BigInteger();
      bignum.fromString(str, base);
      var bigmin = new BigInteger();
      bigmin.fromString(min, 10);
      var bigmax = new BigInteger();
      bigmax.fromString(max, 10);
      if (unsigned && bignum.compareTo(BigInteger.ZERO) < 0) {
        var temp = new BigInteger();
        bignum.addTo(Wrapper.two64, temp);
        bignum = temp;
      }
      var error = false;
      if (bignum.compareTo(bigmin) < 0) {
        bignum = bigmin;
        error = true;
      } else if (bignum.compareTo(bigmax) > 0) {
        bignum = bigmax;
        error = true;
      }
      var ret = goog.math.Long.fromString(bignum.toString()); // min-max checks should have clamped this to a range goog.math.Long can handle well
      HEAP32[tempDoublePtr>>2] = ret.low_;
      HEAP32[tempDoublePtr+4>>2] = ret.high_;
      if (error) throw 'range error';
    }
  };
  return Wrapper;
})();

//======= end closure i64 code =======



// === Auto-generated postamble setup entry stuff ===

if (memoryInitializer) {
  if (ENVIRONMENT_IS_NODE || ENVIRONMENT_IS_SHELL) {
    var data = Module['readBinary'](memoryInitializer);
    HEAPU8.set(data, STATIC_BASE);
  } else {
    addRunDependency('memory initializer');
    Browser.asyncLoad(memoryInitializer, function(data) {
      for (var i = 0; i < data.length; i++) {
        assert(HEAPU8[STATIC_BASE + i] === 0, "area for memory initializer should not have been touched before it's loaded");
      }
      HEAPU8.set(data, STATIC_BASE);
      removeRunDependency('memory initializer');
    }, function(data) {
      throw 'could not load memory initializer ' + memoryInitializer;
    });
  }
}

function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun'] && shouldRunNow) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString(Module['thisProgram'] || '/bin/this.program'), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);

  initialStackTop = STACKTOP;

  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    if (!Module['noExitRuntime']) {
      exit(ret);
    }
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    Module.printErr('run() called, but dependencies remain, so not running');
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    ensureInitRuntime();

    preMain();

    if (ENVIRONMENT_IS_WEB && preloadStartTime !== null) {
      Module.printErr('pre-main prep time: ' + (Date.now() - preloadStartTime) + ' ms');
    }

    if (Module['_main'] && shouldRunNow) {
      Module['callMain'](args);
    }

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      if (!ABORT) doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status) {
  ABORT = true;
  EXITSTATUS = status;
  STACKTOP = initialStackTop;

  // exit the runtime
  exitRuntime();

  // TODO We should handle this differently based on environment.
  // In the browser, the best we can do is throw an exception
  // to halt execution, but in node we could process.exit and
  // I'd imagine SM shell would have something equivalent.
  // This would let us set a proper exit status (which
  // would be great for checking test exit statuses).
  // https://github.com/kripken/emscripten/issues/1371

  // throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

function abort(text) {
  if (text) {
    Module.print(text);
    Module.printErr(text);
  }

  ABORT = true;
  EXITSTATUS = 1;

  var extra = '';

  throw 'abort() at ' + stackTrace() + extra;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}


run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



//# sourceMappingURL=cwsq.js.map